dnl Network-Integrated Multimedia Middleware (NMM)
dnl http://www.motama.com/nmm.html
dnl 
dnl Copyright (C) 2006-2007
dnl Motama GmbH, Saarbruecken, Germany
dnl http://www.motama.com
dnl
dnl See file LICENCE for terms and conditions of usage




dnl NMM_CKECK_COMPILER_FLAG
dnl checks the C++ compiler for support for a specific flag
AC_DEFUN([NMM_CHECK_COMPILER_FLAG],
[
AC_MSG_CHECKING(whether $CXX supports -$1)
nmm_cache=`echo $1 | sed 'y%.=/+-%___p_%'`
AC_CACHE_VAL(nmm_cv_prog_cxx_$nmm_cache,
[
echo 'void f(){}' >conftest.cc
if test -z "`$CXX -$1 -c conftest.cc 2>&1`"; then
  eval "nmm_cv_prog_cxx_$nmm_cache=yes"
else
  eval "nmm_cv_prog_cxx_$nmm_cache=no"
fi
rm -f conftest*
])
if eval "test \"`echo '$nmm_cv_prog_cxx_'$nmm_cache`\" = yes"; then
 AC_MSG_RESULT(yes)
 :
 $2
else
 AC_MSG_RESULT(no)
 :
 $3
fi
])


dnl just a wrapper to clean up configure.in
AC_DEFUN([NMM_PROG_LIBTOOL],
[
if test -z "$LIBTOOL"; then
  dnl libtool is only for C, so I must force him
  dnl to find the correct flags for C++
  nmm_save_cc=$CC
  nmm_save_cflags="$CFLAGS"
  CC=$CXX
  CFLAGS="$CXXFLAGS"
  AC_LANG_SAVE
  AC_LANG_C
  AC_LIBTOOL_DLOPEN
  AC_PROG_LIBTOOL
  AC_LANG_RESTORE
  CC=$nmm_save_cc
  CFLAGS="$nmm_save_cflags"
  LIBTOOL_SHELL='/bin/sh ./libtool'
  LDFLAGS="$LDFLAGS"
else
  LIBTOOL_SHELL=$LIBTOOL
fi
])


dnl NMM_CHECK_EXTRA_LIBS
dnl check in fact for extra libs AND extra incs and adds them
dnl to the appropriate varables for compiling and linking
AC_DEFUN([NMM_CHECK_EXTRA_LIBS],
[
AC_MSG_CHECKING(for extra includes)
AC_ARG_WITH(extra-includes, [  --with-extra-includes=DIR   adds non standard include paths],
  nmm_use_extra_includes="$withval",
  nmm_use_extra_includes=NONE
)
nmm_extra_includes=
if test -n "nmm_use_extra_includes" && \
   test "$nmm_use_extra_includes" != "NONE"; then

   ac_save_ifs=$IFS
   IFS=':'
   for dir in $nmm_use_extra_includes; do
     nmm_extra_includes="$nmm_extra_includes $dir"
     USER_INCLUDES="$USER_INCLUDES -I$dir"
   done
   IFS=$ac_save_ifs
   nmm_use_extra_includes="added"
else
   nmm_use_extra_includes="no"
fi
AC_SUBST(USER_INCLUDES)

AC_MSG_RESULT($nmm_use_extra_includes)

nmm_extra_libs=
AC_MSG_CHECKING(for extra libs)
AC_ARG_WITH(extra-libs, [  --with-extra-libs=DIR       adds non standard library paths],
  nmm_use_extra_libs=$withval,
  nmm_use_extra_libs=NONE
)
if test -n "$nmm_use_extra_libs" && \
   test "$nmm_use_extra_libs" != "NONE"; then

   ac_save_ifs=$IFS
   IFS=':'
   for dir in $nmm_use_extra_libs; do
     nmm_extra_libs="$nmm_extra_libs $dir"
     NMM_EXTRA_RPATH="$NMM_EXTRA_RPATH -rpath $dir"
     USER_LDFLAGS="$USER_LDFLAGS -L$dir"
   done
   IFS=$ac_save_ifs
   nmm_use_extra_libs="added"
else
   nmm_use_extra_libs="no"
fi
AC_SUBST(USER_LDFLAGS)

AC_MSG_RESULT($nmm_use_extra_libs)

])
dnl =====================================================================

dnl NMM_ENABLE_DEBUG
dnl to get a configure switch to enable debugging
AC_DEFUN([NMM_ENABLE_DEBUG],[
  AC_ARG_ENABLE(debug,[  --enable-debug              creates debugging code [default=no]],[
   if test x$enableval = "xno";
     then
       nmm_use_debug_code="no"
     else
       nmm_use_debug_code="yes"
   fi
  ],nmm_use_debug_code="no")
  dnl DEBUG options for C
  if test "x$nmm_use_debug_code" = "xyes"; then
    if test "x$ac_cv_prog_cc_g" = "xyes"; then
      CFLAGS="-g -O0"
    else
      CFLAGS="-g"
    fi
  elif test "x$GCC" = "xyes"; then
    CFLAGS="-O2"
  else
    CFLAGS=""
  fi
  dnl end debug options for C
  dnl DEBUG options for C++
  if test "x$nmm_use_debug_code" = "xyes"; then
    if test "x$ac_cv_prog_cxx_g" = "xyes"; then
      CXXFLAGS="-g -O0  -Wall -W -Wpointer-arith -Wwrite-strings"
      ARM_CXXFLAGS="-g -O0  -Wall-W -Wpointer-arith -Wwrite-strings"
    else
      ARM_CXXFLAGS="-g"
    fi
  elif test "x$GXX" = "xyes"; then
    CXXFLAGS="-O2"
    if test "x$HAVE_CROSSCOMPILER" = "x1"; then
        ARM_CXXFLAGS=""
    else
        ARM_CXXFLAGS="-O2"
    fi
  else
    CXXFLAGS=""
    ARM_CXXFLAGS=""
  fi
  dnl end DEBUG options for C++
  dnl DEBUG options for yacc
  if test "x$nmm_use_debug_code" = "xyes"; then
#    YFLAGS="--debug --verbose"
    YFLAGS=""
  else
    YFLAGS=""
  fi
  dnl end DEBUG options for yacc
  dnl NO DEBUG options
  if test "x$LDFLAGS" = "x"; then
    if test "x$nmm_use_debug_code" = "xno"; then
      if test "x$GCC" = "xyes"; then
        LDFLAGS=""
      fi
    fi
  fi
  dnl end NO DEBUG options
])


dnl NMM_ENABLE_STRICT
dnl to get a configure switch to enable strict compilation
AC_DEFUN([NMM_ENABLE_STRICT],[
  AC_ARG_ENABLE(strict,[  --enable-strict             compiles with strict compiler options (may not work!)],[
    if test "x$enableval" = "xno"; then
         nmm_use_strict_options="no"
       else
         nmm_use_strict_options="yes"
    fi
   ], [nmm_use_strict_options="no"])
  dnl strict options for C
  if test "x$GCC" = "xyes"; then
    if test "x$nmm_use_strict_options" = "xyes"; then
      CFLAGS="$CFLAGS -W -Wall -ansi -Wpointer-arith -Wmissing-prototypes -Wwrite-strings"
    fi
  fi
  dnl end strict options for C
  dnl STRICT options for C++
  if test "x$GXX" = "xyes"; then
    if test "x$nmm_use_strict_options" = "xyes"; then
      CXXFLAGS="$CXXFLAGS -Wall -W -Wpointer-arith -Wwrite-strings -Wcast-qual -Wbad-function-cast -Wcast-align"
      ARM_CXXFLAGS="$ARM_CXXFLAGS -Wall -W -Wpointer-arith -Wwrite-strings -Wcast-qual -Wbad-function-cast -Wcast-align"
    fi
  fi
  dnl end STRICT options for C++
])


dnl NMM_ENABLE_PROFILING
dnl to get a configure switch to enable profiling
AC_DEFUN([NMM_ENABLE_PROFILING],[
  dnl 17-10-01 mlohse
  dnl begin profiling
  AC_ARG_ENABLE(profiling,[  --enable-profiling          creates profiling code [default=no]],
  [
   if test $enableval = "no";
     then
       nmm_use_profiling_code="no"
     else
       nmm_use_profiling_code="yes"
   fi
  ], [nmm_use_profiling_code="no"
    ])
  dnl end profiling

  dnl 17-10-01 mlohse
  dnl begin profiling for C
  if test "$nmm_use_profiling_code" = "yes"; then
    CFLAGS="$CFLAGS -pg -fprofile-arcs -DPROFILING"
  fi
  dnl end profiling for C

  dnl 17-10-01 mlohse
  dnl begin profiling for C++
  if test "$nmm_use_profiling_code" = "yes"; then
    CXXFLAGS="$CXXFLAGS -pg -fprofile-arcs -DPROFILING"
    ARM_CXXFLAGS="$ARM_CXXFLAGS -pg -fprofile-arcs -DPROFILING"
  fi
  dnl end profiling for C++
])


dnl NMM_ENABLE_FINAL
dnl to disable all NMM-internal messages to get maximal
dnl performace
AC_DEFUN([NMM_ENABLE_FINAL],[
  dnl 13-11-01 mlohse
  dnl begin messages
  AC_ARG_ENABLE(final,[  --enable-final              creates final code without messages [default=no]],
  [
   if test $enableval = "no";
     then
       nmm_disable_messages="no"
     else
       nmm_disable_messages="yes"
   fi
  ], [nmm_disable_messages="no"
    ])
  dnl end messages

  dnl 13-11-01 mlohse
  dnl begin messages for C
  if test "$nmm_disable_messages" = "yes"; then
    CFLAGS="$CFLAGS -D_NMM_FINAL_VERSION_"
  fi
  dnl end messages for C

  dnl 13-11-01 mlohse
  dnl begin messages for C++
  if test "$nmm_disable_messages" = "yes"; then
    CXXFLAGS="$CXXFLAGS -D_NMM_FINAL_VERSION_"
    ARM_CXXFLAGS="$ARM_CXXFLAGS -D_NMM_FINAL_VERSION_"
  fi
  dnl end messages for C++
])


dnl NMM_USE_NO_CHECK_NEW
dnl adds no-check-new flag to c++ compiler options
AC_DEFUN([NMM_USE_NO_CHECK_NEW],[
NMM_CHECK_COMPILER_FLAG(fno-check-new, [USE_NO_CHECK_NEW="-fno-check-new"  ],USE_NO_CHECK_NEW= )
CXXFLAGS="$CXXFLAGS $USE_NO_CHECK_NEW"
ARM_CXXFLAGS="$ARM_CXXFLAGS $USE_NO_CHECK_NEW"
])


dnl NMM_USE_EXCEPTIONS
dnl adds exception flag to c++ compiler options
AC_DEFUN([NMM_USE_EXCEPTIONS],[
NMM_CHECK_COMPILER_FLAG(fexceptions,   [USE_EXCEPTIONS="-fexceptions"      ],USE_EXCEPTIONS=   )
CXXFLAGS="$CXXFLAGS $USE_EXCEPTIONS"
ARM_CXXFLAGS="$ARM_CXXFLAGS $USE_EXCEPTIONS"
])


dnl NMM_USE_RTTI
dnl adds rtti flag to c++ compiler options
AC_DEFUN([NMM_USE_RTTI],[
NMM_CHECK_COMPILER_FLAG(frtti,         [USE_RTTI="-frtti"                  ],USE_RTTI=         )
ARM_CXXFLAGS="$ARM_CXXFLAGS $USE_RTTI"
])


dnl NMM_ARCH_X86
dnl checks wether arch is x86 and defines ARCH_?86 accordingly
dnl also checks wether arch is arm and defines ARCH_ARM 
dnl and if X86_64 architecture, defines ARCH_X86_64 then
dnl finally all defines are also set as automake conditionals
AC_DEFUN([NMM_ARCH_X86],
[
AC_MSG_CHECKING(architecture)
arch_x86_64=
arch_x86=
arch_arm=
case "$target_cpu" in
  i?86)
    arch_x86=1
    AC_DEFINE_UNQUOTED(NMM_UPCASE_WORD([ARCH_X86]), 1, [Define if you have a x86 architecture])
    AC_MSG_RESULT(x86)
    case "$target_cpu" in
      i386)
        AC_DEFINE_UNQUOTED(NMM_UPCASE_WORD([ARCH_386]), 1, [Define if you have a 386 architecture])
      ;;
      i486)
        AC_DEFINE_UNQUOTED(NMM_UPCASE_WORD([ARCH_486]), 1, [Define if you have a 486 architecture])
      ;;
      i586)
        AC_DEFINE_UNQUOTED(NMM_UPCASE_WORD([ARCH_586]), 1, [Define if you have a 586 architecture])
      ;;
      i686)
        AC_DEFINE_UNQUOTED(NMM_UPCASE_WORD([ARCH_686]), 1, [Define if you have a 686 architecture])
      ;;
    esac
  ;;
  arm)
    arch_arm=1
    AC_MSG_RESULT(arm)
    AC_DEFINE_UNQUOTED(NMM_UPCASE_WORD([ARCH_ARM]), 1, [Define if you have a arm architecture])
  ;;
  x86_64)
  	arch_x86_64=1
	AC_MSG_RESULT(x86_64)
    	AC_DEFINE_UNQUOTED(NMM_UPCASE_WORD([ARCH_X86_64]), 1, [Define if you have a x86_64 architecture])
  	;;
  *)
    AC_MSG_RESULT(unknown)
    AC_MSG_WARN([*** No x86 architecture found!  ***])
  ;;
esac
AM_CONDITIONAL(ARCH_X86, test "x$arch_x86" != "x")

AM_CONDITIONAL(ARCH_386, test "x$target_cpu" = "xi386")
AM_CONDITIONAL(ARCH_486, test "x$target_cpu" = "xi486")
AM_CONDITIONAL(ARCH_586, test "x$target_cpu" = "xi586")
AM_CONDITIONAL(ARCH_686, test "x$target_cpu" = "xi686")

AM_CONDITIONAL(ARCH_ARM, test "x$arch_arm" != "x")
AM_CONDITIONAL(ARCH_X86_64, test "x$arch_x86_64" != "x")

unset arch_x86_64
unset arch_x86
unset arch_arm
])


dnl NMM_ARG_WITH
dnl is a somewhat enhanced wrapper vor AC_ARG_WITH
AC_DEFUN([NMM_ARG_WITH],
[
AC_ARG_WITH([$1],
            AC_HELP_STRING([--with-$1=DIR],
                           [use $1 (libs from DIR/lib, includes from DIR/include, default is to check standard paths)]),
[
if test "x$withval" = "x"; then
  ac_cv_use_$1=CHECK
else
  ac_cv_use_$1=$withval
fi
], ac_cv_use_$1=CHECK)
])


dnl NMM_UPCASE_WORD
dnl uppercase a word to create a valid define
dnl errror 25072003: After replacing the old implementation (see
dnl                  below), which also did not work properly in some
dnl                  cases, maybe the new implementation can be
dnl                  directly use!
AC_DEFUN([NMM_UPCASE_WORD],translit($1, `a-z', `A-Z'))
dnl [`echo $1 | tr '.abcdefghijklmnopqrstuvwxyz' '_ABCDEFGHIJKLMNOPQRSTUVWXYZ'`])


dnl NMM_TRY_LINK(incpaths,
dnl		 headers,
dnl		 libpaths, 
dnl		 libs, 
dnl		 testmain,
dnl		 ACTION-IF-NOT-FOUND, 
dnl		 ACTION-IF-FOUND)
dnl
dnl usage:
dnl incpaths: additional paths to search for the headers (adional to
dnl           the default search paths!)
dnl headers:  headers, as they are included in source files, multiple
dnl           in [] (maybe prepended by system headers, for undefined
dnl           symbols in the specified headers
dnl libpaths: additional paths to search for the libs (aditional to the
dnl	      default search paths!)
dnl libs:     libs, that the test program should be linked against,
dnl           multiple in [].
dnl           libs are specified by their library names, NOT their
dnl           filenames, e.g. for liblame.so the library name is "lame"
dnl testmain: source code to a apear in an main function of a test
dnl           program, always in []
AC_DEFUN([NMM_TRY_LINK],
[
nmmtrylink_incpaths="$1"
nmmtrylink_headers="$2"
nmmtrylink_libpaths="$3"
nmmtrylink_libraries="$4"
nmmtrylink_foundit=NOTFOUND

nmm_save_cppflags="$CPPFLAGS"
nmm_save_cflags="$CFLAGS"
nmm_save_cxxflags="$CXXFLAGS"
nmm_save_ldflags="$LDFLAGS"
nmm_save_libs="$LIBS"

nmmtrylink_inc_headers=
for i in $nmmtrylink_headers; do
  nmmtrylink_inc_headers="$nmmtrylink_inc_headers
#include <$i>"
done

LIBS="$nmm_save_libs"
for l in $nmmtrylink_libraries; do
  LIBS="$LIBS -l$l" 
done

dnl extern_c is defined by the NMM_CHECK_EXTPACKAGE_EXTERNC and
dnl indicates, that the specified headers should be clasped in a
dnl 'extern "C"' declaration
if test "x$extern_c" = "x1"; then
        nmmtrylink_inc_headers="extern \"C\" {
$nmmtrylink_inc_headers
}"
fi

dnl iterate over all include-dirs with all lib-dirs
for i in $nmmtrylink_incpaths stdincpath; do
  if test $i = "stdincpath";  then
    CPPFLAGS="$nmm_save_cppflags"
    ADDEDINCLUDES=""
    CFLAGS="$nmm_save_cflags"
    CXXFLAGS="$nmm_save_cxxflags"
  else
    CPPFLAGS="$nmm_save_cppflags"
    ADDEDINCLUDES="-I$i"
    CFLAGS="$nmm_save_cflags $ADDEDINCLUDES"
    CXXFLAGS="$nmm_save_cxxflags $ADDEDINCLUDES"
  fi

  for l in $nmmtrylink_libpaths stdlibpath; do 
    if test $l = "stdlibpath"; then
      ADDEDLDFLAGS=""
      LDFLAGS="$nmm_save_ldflags"
    else
      ADDEDLDFLAGS="-L$l"
      LDFLAGS="$nmm_save_ldflags $ADDEDLDFLAGS"
    fi

    dnl try compiling & linking with current lib- and include-dirs
    AC_TRY_LINK([$nmmtrylink_inc_headers], [$5], [
      nmmtrylink_foundit=FOUND
      found_cflags="$ADDEDINCLUDES"
      found_ldflags="$ADDEDLDFLAGS"
      found_libs="$LIBS"    
      nmmtrylink_incdir=$i
      nmmtrylink_libdir=$l
      ])
    
    test "x$nmmtrylink_foundit" = "xFOUND" && break
  done

  test "x$nmmtrylink_foundit" = "xFOUND" && break
done

CPPFLAGS="$nmm_save_cppflags"
CFLAGS="$nmm_save_cflags"
CXXFLAGS="$nmm_save_cxxflags"
LDFLAGS="$nmm_save_ldflags"
LIBS="$nmm_save_libs"

if test "x$nmmtrylink_foundit" = "xFOUND"; then
  dnl if we found a working lib-/include-dir pair, we execude arg #7
  [$7]
else
  dnl if not, we execude arg #6
  [$6]
fi

unset nmmtrylink_incpaths
unset nmmtrylink_headers
unset nmmtrylink_libpaths
unset nmmtrylink_libraries
unset nmmtrylink_foundit
unset nmm_save_cppflags
unset nmm_save_cflags
unset nmm_save_cxxflags
unset nmm_save_ldflags
unset nmm_save_libs
unset nmmtrylink_inc_headers
unset found_cflags
unset found_ldflags
unset found_libs
unset nmmtrylink_incdir
unset nmmtrylink_libdir
])


dnl NMM_CHECK_EXTPACKAGE_EXTERNC(name, libincs, libs, testmain, [NODENAME])
dnl
dnl usage:
dnl see NMM_CHECK_EXTPACKAGE.
dnl The behavior is the same as NMM_CHECK_EXTPACKAGE except, that the
dnl header files are included with 'extern "C" { ... }'.
AC_DEFUN([NMM_CHECK_EXTPACKAGE_EXTERNC],
[
extern_c=1
NMM_CHECK_EXTPACKAGE($1,$2,$3,$4,$5,$6)
unset extern_c
])


dnl NMM_CHECK_EXTPACKAGE(name, libincs, libs, testmain, [NODENAME], longdesc)
dnl
dnl usage:
dnl name:     name of the package as configure should print it in "Checking..."
dnl incs:     headers, as they are included in source files, multiple in []
dnl libs:     library names, as needed for linking, multiple in []
dnl testmain: source code to a apear in an main function of a test program, always in []
dnl NODENAME: name of the node, that should not be build if package not installed
dnl           could be used for conditionals in Makefile.am(s)
dnl           if NODENAME is "1": package ist needed for build always
dnl longdesc: long description given out, when package not found (URL is recommended)
AC_DEFUN([NMM_CHECK_EXTPACKAGE],
[
extpack_must=0
if test "x$5" = x1; then
  extpack_must=1
fi

extpack_specified=
extpack_inc_path=
extpack_includes=
extpack_lib_path=
extpack_ldadd=
with_extpack=

AC_MSG_CHECKING([for $1])

NMM_ARG_WITH($1)

AC_LANG_PUSH(C++)

if test "x$ac_cv_use_$1" != "xCHECK" ; then
  extpack_specified=1
  extpack_inc_path=$ac_cv_use_$1/include
  extpack_lib_path=$ac_cv_use_$1/lib
fi

NMM_TRY_LINK([$extpack_inc_path $nmm_extra_includes], [$2],
	     [$extpack_lib_path $nmm_extra_libs], [$3],
	     [$4],
               dnl this code is executed if the specified headers/libs
               dnl are not found
	       if test $extpack_must = 1; then
	         AC_MSG_RESULT(no)
	         AC_MSG_ERROR([***  NMM needs $1 for base functionality!  ***])
	       else
	         AC_MSG_RESULT(no)
                 if test "x$6" != "x1"; then
	           AC_MSG_WARN([*** $1 not found. NMM will suffer functionality!  ***])
                   if test "x$6" != "x"; then
                     AC_MSG_RESULT(Hint: $6)
                   fi
                 fi
	         $1_incs=
	         $1_libs=
	       fi
	       HAVE_$5=0
	     ,
               dnl this code is executed if the specified headers/libs
               dnl are found
	       AC_MSG_RESULT(yes: incs=$nmmtrylink_incdir libs=$nmmtrylink_libdir)
	       $1_incs="$found_cflags"
	       $1_libs="$found_ldflags $found_libs"
	       if test "x$extpack_must" != "x1"; then
	         AC_DEFINE_UNQUOTED([HAVE_$5],, Define if you have $5)
	       fi
	       HAVE_$5=1
	     )

AC_SUBST($1_incs)
AC_SUBST($1_libs)

AC_LANG_POP()

unset extpack_must
unset extpack_specified
unset extpack_inc_path
unset extpack_includes
unset extpack_lib_path
unset extpack_ldadd
unset with_extpack
])


dnl NMM_PROG_NASM
dnl checks for existence of nasm assembler and configures Makefiles for
dnl usage. Add workarounds for libtool (nasm doesn't understand PIC).
dnl if not present, configures Makefiles to use (slower) C sources.
AC_DEFUN([NMM_PROG_NASM],[
  AC_ARG_ENABLE(nasm,
                [  --disable-nasm          disables (possible) usage of nasm [default=no]],,
                enable_nasm=yes)  

dnl we disable nasm support if we are cross-compiling
if test "x$HAVE_CROSSCOMPILER" = "x1"; then
   AC_MSG_CHECKING([for nasm])
   AC_MSG_RESULT([no])
   AC_MSG_WARN([***  Disabling nasm support while cross-compiling!  ***])
   NMM_USE_NASM=0
else
   NMM_USE_NASM=0
   AC_CHECK_PROG(NASM, nasm, nasm)
   if test "x$NASM" = "xnasm"; then
      NMM_USE_NASM=1
      STRIP_FPIC='$(SHELL) $(top_srcdir)/strip_fPIC.sh'
      AC_SUBST(STRIP_FPIC)
   else
      AC_MSG_WARN([*** nasm was not found! No assembler optimized code will be integrated!  ***])
      AC_MSG_RESULT([Hint: nasm can be found under http://nasm.sourceforge.net/ .])
   fi
fi

if test "x$enable_nasm" != "xyes"; then
   AC_MSG_WARN([*** disabled (possible) usage of nasm!  ***])
   NMM_USE_NASM=0
fi

AC_SUBST(NMM_USE_NASM)
AM_CONDITIONAL(NMM_USE_NASM, test "x$NMM_USE_NASM" = "x1")
])


dnl NMM_CROSS_CC
dnl checks, wether CC is found as a cross compiler and sets
dnl CC_FOR_BUILD a gcc to compile the NMM idl compiler for the build
dnl system.
AC_DEFUN([NMM_CROSS_CC],[
if test "x$HAVE_CROSSCOMPILER" = "x1"; then
   AC_CHECK_PROG(CC_FOR_BUILD, gcc, gcc)
   if test "x$CC_FOR_BUILD" != "xgcc"; then
      AC_MSG_ERROR([***  No build-host gcc found. NMM needs systems gcc  ***
***  to create compilation tools!                    ***])
   fi
else
   CC_FOR_BUILD=$CC
   AC_SUBST(CC_FOR_BUILD)
fi
])


dnl NMM_CROSS_CXX
dnl checks, wether CXX is found as a cross compiler and sets
dnl CXX_FOR_BUILD a gcc to compile the NMM idl compiler for the build
dnl system.
AC_DEFUN([NMM_CROSS_CXX],[
if test "x$HAVE_CROSSCOMPILER" = "x1"; then
   AC_CHECK_PROG(CXX_FOR_BUILD, g++, g++)
   if test "x$CXX_FOR_BUILD" != "xg++"; then
      AC_MSG_ERROR([***  No build-host g++ found. NMM needs systems gcc  ***
***  to create compilation tools!                    ***])
   fi
else
   CXX_FOR_BUILD=$CXX
   AC_SUBST(CXX_FOR_BUILD)
fi
])


dnl NMM_PROG_YACC
dnl checks for a working yacc.
AC_DEFUN([NMM_PROG_YACC],[
AC_PROG_YACC
if echo $YACC | grep bison >/dev/null; then
   foundayacc=1
else
   dnl if $YACC is not bison, test if it is program at all
   save_IFS="$IFS"
   IFS=:
   for p in $PATH; do
       test -x "$p/$YACC" || continue
       foundayacc=1
   done
   IFS="$save_IFS"
   if test "x$foundayacc" = "x1"; then
      AC_MSG_WARN([
*** Your yacc program is not GNU bison. If you encounter problems, ***
*** compiling NMM, which indicate problems with your yacc program, ***
*** try installing bison from ftp://ftp.gnu.org/gnu/bison/ .       ***])
    fi
fi
if test "x$foundayacc" = "x"; then
   AC_MSG_WARN([
*** No suitable yacc program found. You will run into trouble, if  ***
*** you try to regenerate C++ sources generated by yacc.           ***])
   if test "x${am_missing_run}" = "x"; then
     am_missing_run="${missing_dir}/missing "
   fi
   YACC="${am_missing_run}bison"
fi
])


dnl NMM_PROG_LEX
dnl checks for a working lex.
AC_DEFUN([NMM_PROG_LEX],[
AM_PROG_LEX
if test "x$LEX" != "xflex"; then
   save_IFS="$IFS"
   IFS=:
   for p in $PATH; do
       test -x "$p/$YACC" || continue
       foundalexer=1
   done
   IFS="$save_IFS"
   if "x$foundalexer" = "x"; then
      AC_MSG_WARN([
*** No suitable lex program found. You will run into trouble, if ***
*** you try to regenerate C sources generated by lex.            ***])
   fi
fi
])


dnl NMM_PROGS_DOCBOOK
dnl checks for the needed docbook utils.
dnl defines HAVE_DOCBOOKUTILS=1 if found.
AC_DEFUN([NMM_PROGS_DOCBOOK],[
found_docbook_utils=0
AC_CHECK_PROGS(DB2HTML,db2html docbook2html)
if test "x$DB2HTML" != "x"; then
   found_docbook_utils=1
   AC_CHECK_PROGS(DB2PDF,db2pdf docbook2pdf)
   if test "x$DB2PDF" = "x"; then
      found_docbook_utils=0
      AC_MSG_WARN([
*** No db2pdf tool found. Maybe your docbook-utils are not ***
*** installed properly?                                    ***])
   fi
else
  found_docbook_utils=0
fi
if test "x$found_docbook_utils" != "x1"; then
  AC_MSG_WARN([
*** No docbook-utils found. Please install if you want the ***
*** according documentation to be build.                   ***])
  AC_MSG_RESULT([Hint: docbook-utils can be found under http://sources.redhat.com/docbook-tools/ .])
else
  HAVE_DOCBOOKUTILS=1
  AC_DEFINE_UNQUOTED([HAVE_DOCBOOKUTILS],, Define if you have docbook utils)
fi
])

dnl NMM_PROGS_HTMLDOC
dnl checks for htmldocs
dnl defines HAVE_HTMLDOC=1 if found
AC_DEFUN([NMM_PROG_HTMLDOC],[
found_htmldoc=0
AC_CHECK_PROG(HTMLDOC, htmldoc, htmldoc)
if test "x$HTMLDOC" != "x"; then
  found_htmldoc=1
else
  found_htmldoc=0
fi
if test "x$found_htmldoc" != "x1"; then
  AC_MSG_WARN([
*** htmldoc not found. Please install if you want ***
*** pluginlist.pdf to be generated.               ***])
  AC_MSG_RESULT([Hint: htmldoc can be found under http://www.easysw.com/htmldoc/ .])
else
  HAVE_HTMLDOC=1
  AC_DEFINE_UNQUOTED([HAVE_HTMLDOC],, Define if you have htmldoc)
fi
])


dnl NMM_PATH_XTRA
dnl wrapper for AC_[]PATH_[]XTRA with cross compiling tests
AC_DEFUN([NMM_PATH_XTRA],[
AC_PATH_XTRA
if test "x$HAVE_CROSSCOMPILER" = "x1"; then
    no_x=yes
    AC_MSG_WARN([*** X11 support disabled while cross compiling  ***])
else
    if test "x$no_x" != "xyes"; then
        AC_DEFINE_UNQUOTED([HAVE_XDISPLAYNODE],, Define if you have XDISPLAYNODE)
        HAVE_XDISPLAYNODE=1
	
	x11_cxxflags="$CXXFLAGS"
	CXXFLAGS="$CXXFLAGS $X_CFLAGS"
	NMM_TRY_LINK(,[X11/Xutil.h Xm/MwmUtil.h],,,, have_mwmutil_h_internal=0, have_mwmutil_h_internal=1)
	if test "x$have_mwmutil_h_internal" = "x1"; then
		AC_DEFINE_UNQUOTED([HAVE_MWMUTIL_H],, Define if you have Xm/MwmUtil.h)
	fi
	unset have_mwmutil_h_internal
	CXXFLAGS="$x11_cxxflags"
	unset x11_cxxflags
    else
	HAVE_XDISPLAYNODE=0
        AC_MSG_WARN([*** X11 not found. NMM will suffer funcionality!  ***])
    fi
fi
])



dnl NMM_PATH_VORBIS
dnl wrapper for AM_[]PATH_[]VORBIS with cross compiling tests.
AC_DEFUN([NMM_PATH_VORBIS],[
dnl we use AM_[]PATH_[]VORBIS here (defined below!) as
dnl XIPH_[]PATH_[]VORBIS from vorbis-1.0 is buggy!
dnl
dnl the AC_[]TRY_[]LINK below makes sure, that we have found vorbis
dnl libs >= 1.0rc3
AM_PATH_VORBIS([
        nmm_save_cflags="$CFLAGS"
        nmm_save_cppflags="$CPPFLAGS"
        nmm_save_cxxflags="$CXXFLAGS"
        nmm_save_ldflags="$LDFLAGS"
        CFLAGS="$VORBIS_CFLAGS $CFLAGS $USER_INCLUDES"
        CPPFLAGS="$VORBIS_CFLAGS $CPPFLAGS $USER_INCLUDES"
        CXXFLAGS="$VORBIS_CFLAGS $CXXFLAGS $USER_INCLUDES"
        LDFLAGS="$VORBIS_LIBS $VORBISENC_LIBS $VORBISFILE_LIBS $LDFLAGS $USER_LDFLAGS"
        AC_TRY_LINK([
#include <stdio.h>
#include <vorbis/codec.h>
],              [
vorbis_bitrate_addblock(NULL);
vorbis_bitrate_flushpacket(NULL, NULL);
return 0;
],              [HAVE_OGGVORBISNODES=1 
                AC_DEFINE_UNQUOTED([HAVE_OGGVORBISNODES],, Define if you have OGGVORBISNODES)
                AC_DEFINE_UNQUOTED([HAVE_OGGVORBISNODES],, Define if you have OGGVORBISNODES)
                ],[AC_MSG_WARN([
*** The version of the Vorbis libraries found is to old. NMM needs   ***
*** a version greater or equal to 1.0rc3 of the Vorbis libs to work. ***
*** Disabling Vorbis support for NMM!                                ***])
                HAVE_OGGVORBISNODES=0])
       CFLAGS="$nmm_save_cflags"
       CPPFLAGS="$nmm_save_cppflags"
       CXXFLAGS="$nmm_save_cxxflags"
       LDFLAGS="$nmm_save_ldflags"
       ],
       [AC_MSG_WARN([*** OggVorbis not found. NMM will suffer funcionality!  ***])])
])

dnl NMM_SUMMARY
dnl summarizes the results of the tests and checks at the end of the
dnl configure run
AC_DEFUN([NMM_SUMMARY],[
cat <<EOF



Summary:
========

Results to be summaries here...


run "make" to compile now.
Hint: You can use "make -j 10 parallel" to speed up the build,
      esspecially on SMP machines.

EOF
])

dnl we must provide our own macro for libxml2 as long as their xml-config
dnl script adds the "-I/usr/include/libxml2/libxml" CFLAG which leeds to a
dnl include problem as tree.h is taken from there instead of the one in
dnl /usr/include, provided by STL

# Configure paths for LIBXML2
# Toshio Kuratomi 2001-04-21
# Adapted from:
# Configure paths for GLIB
# Owen Taylor     97-11-3

dnl NMM_PATH_XML2([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]]])
dnl Test for XML, and define XML_CFLAGS and XML_LIBS
dnl
AC_DEFUN([NMM_PATH_XML2],[ 
AC_ARG_WITH(xml-prefix,
            [  --with-xml-prefix=PFX       Prefix where libxml is installed (optional)],
            xml_config_prefix="$withval", xml_config_prefix="")
AC_ARG_WITH(xml-exec-prefix,
            [  --with-xml-exec-prefix=PFX  Exec prefix where libxml is installed (optional)],
            xml_config_exec_prefix="$withval", xml_config_exec_prefix="")
AC_ARG_ENABLE(xmltest,
              [  --disable-xmltest           Do not try to compile and run a test LIBXML program],,
              enable_xmltest=yes)

  if test x$xml_config_exec_prefix != x ; then
     xml_config_args="$xml_config_args --exec-prefix=$xml_config_exec_prefix"
     if test x${XML2_CONFIG+set} != xset ; then
        XML2_CONFIG=$xml_config_exec_prefix/bin/xml2-config
     fi
  fi
  if test x$xml_config_prefix != x ; then
     xml_config_args="$xml_config_args --prefix=$xml_config_prefix"
     if test x${XML2_CONFIG+set} != xset ; then
        XML2_CONFIG=$xml_config_prefix/bin/xml2-config
     fi
  fi

  AC_PATH_PROG(XML2_CONFIG, xml2-config, no)
  min_xml_version=ifelse([$1], ,2.0.0,[$1])
  AC_MSG_CHECKING(for libxml - version >= $min_xml_version)
  no_xml=""
  if test "$XML2_CONFIG" = "no" ; then
    no_xml=yes
  else
# patch to avoid conflicting include tree.h from this package and STL
#    XML_CFLAGS=`$XML2_CONFIG $xml_config_args --cflags`
    tmp_cflags=`$XML2_CONFIG $xml_config_args --cflags`
    for p in $tmp_cflags; do
      case $p in
        */libxml2/libxml*)
          continue;
          ;;
        *)
          XML_CFLAGS="$XML_CFLAGS $p"
          ;;
      esac
    done
    XML_LIBS=`$XML2_CONFIG $xml_config_args --libs`
    xml_config_major_version=`$XML2_CONFIG $xml_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    xml_config_minor_version=`$XML2_CONFIG $xml_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    xml_config_micro_version=`$XML2_CONFIG $xml_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`
    if test "x$enable_xmltest" = "xyes" ; then
      ac_save_CFLAGS="$CFLAGS"
      ac_save_CXXFLAGS="$CXXFLAGS"
      ac_save_LIBS="$LIBS"
      CFLAGS="$CFLAGS $XML_CFLAGS"
      CXXFLAGS="$CXXFLAGS $XML_CFLAGS"
      LIBS="$XML_LIBS $LIBS"
dnl
dnl Now check if the installed libxml is sufficiently new.
dnl (Also sanity checks the results of xml2-config to some extent)
dnl
      rm -f conf.xmltest
      AC_TRY_RUN([
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
/* #include <xmlversion.h> */
/* patch to avoid conflicting include tree.h from this package and STL */
#include <libxml/xmlversion.h>

int 
main()
{
  int xml_major_version, xml_minor_version, xml_micro_version;
  int major, minor, micro;
  char *tmp_version;

  system("touch conf.xmltest");

  /* Capture xml2-config output via autoconf/configure variables */
  /* HP/UX 9 (%@#!) writes to sscanf strings */
  tmp_version = (char *)strdup("$min_xml_version");
  if (sscanf(tmp_version, "%d.%d.%d", &major, &minor, &micro) != 3) {
     printf("%s, bad version string from xml2-config\n", "$min_xml_version");
     exit(1);
   }
   free(tmp_version);

   /* Capture the version information from the header files */
   tmp_version = (char *)strdup(LIBXML_DOTTED_VERSION);
   if (sscanf(tmp_version, "%d.%d.%d", &xml_major_version, &xml_minor_version, &xml_micro_version) != 3) {
     printf("%s, bad version string from libxml includes\n", "LIBXML_DOTTED_VERSION");
     exit(1);
   }
   free(tmp_version);

 /* Compare xml2-config output to the libxml headers */
  if ((xml_major_version != $xml_config_major_version) ||
      (xml_minor_version != $xml_config_minor_version) ||
      (xml_micro_version != $xml_config_micro_version))
    {
      printf("*** libxml header files (version %d.%d.%d) do not match\n",
         xml_major_version, xml_minor_version, xml_micro_version);
      printf("*** xml2-config (version %d.%d.%d)\n",
         $xml_config_major_version, $xml_config_minor_version, $xml_config_micro_version);
      return 1;
    } 
/* Compare the headers to the library to make sure we match */
  /* Less than ideal -- doesn't provide us with return value feedback, 
   * only exits if there's a serious mismatch between header and library.
   */
    LIBXML_TEST_VERSION;

    /* Test that the library is greater than our minimum version */
    if ((xml_major_version > major) ||
        ((xml_major_version == major) && (xml_minor_version > minor)) ||
        ((xml_major_version == major) && (xml_minor_version == minor) &&
        (xml_micro_version >= micro)))
      {
        return 0;
       }
     else
      {
        printf("\n*** An old version of libxml (%d.%d.%d) was found.\n",
               xml_major_version, xml_minor_version, xml_micro_version);
        printf("*** You need a version of libxml newer than %d.%d.%d. The latest version of\n",
           major, minor, micro);
        printf("*** libxml is always available from ftp://ftp.xmlsoft.org.\n");
        printf("***\n");
        printf("*** If you have already installed a sufficiently new version, this error\n");
        printf("*** probably means that the wrong copy of the xml2-config shell script is\n");
        printf("*** being found. The easiest way to fix this is to remove the old version\n");
        printf("*** of LIBXML, but you can also set the XML2_CONFIG environment to point to the\n");
        printf("*** correct copy of xml2-config. (In this case, you will have to\n");
        printf("*** modify your LD_LIBRARY_PATH enviroment variable, or edit /etc/ld.so.conf\n");
        printf("*** so that the correct libraries are found at run-time))\n");
    }
  return 1;
}
],, no_xml=yes,[echo $ac_n "cross compiling; assumed OK... $ac_c"])
       CFLAGS="$ac_save_CFLAGS"
       CXXFLAGS="$ac_save_CXXFLAGS"
       LIBS="$ac_save_LIBS"
     fi
  fi

  if test "x$no_xml" = x ; then
     AC_MSG_RESULT(yes (version $xml_config_major_version.$xml_config_minor_version.$xml_config_micro_version))
     ifelse([$2], , :, [$2])     
  else
     AC_MSG_RESULT(no)
     if test "$XML2_CONFIG" = "no" ; then
       echo "*** The xml2-config script installed by LIBXML could not be found"
       echo "*** If libxml was installed in PREFIX, make sure PREFIX/bin is in"
       echo "*** your path, or set the XML2_CONFIG environment variable to the"
       echo "*** full path to xml2-config."
     else
       if test -f conf.xmltest ; then
        :
       else
          echo "*** Could not run libxml test program, checking why..."
          CFLAGS="$CFLAGS $XML_CFLAGS"
          CXXFLAGS="$CXXFLAGS $XML_CFLAGS"
          LIBS="$LIBS $XML_LIBS"
          AC_TRY_LINK([
/* #include <xmlversion.h> */
/* patch to avoid conflicting include tree.h from this package and STL */
#include <libxml/xmlversion.h>
#include <stdio.h>
],      [ LIBXML_TEST_VERSION; return 0;],
        [ echo "*** The test program compiled, but did not run. This usually means"
          echo "*** that the run-time linker is not finding LIBXML or finding the wrong"
          echo "*** version of LIBXML. If it is not finding LIBXML, you'll need to set your"
          echo "*** LD_LIBRARY_PATH environment variable, or edit /etc/ld.so.conf to point"
          echo "*** to the installed location  Also, make sure you have run ldconfig if that"
          echo "*** is required on your system"
          echo "***"
          echo "*** If you have an old version installed, it is best to remove it, although"
          echo "*** you may also be able to get things to work by modifying LD_LIBRARY_PATH" ],
        [ echo "*** The test program failed to compile or link. See the file config.log for the"
          echo "*** exact error that occured. This usually means LIBXML was incorrectly installed"
          echo "*** or that you have moved LIBXML since it was installed. In the latter case, you"
          echo "*** may want to edit the xml2-config script: $XML2_CONFIG" ])
          CFLAGS="$ac_save_CFLAGS"
          CXXFLAGS="$ac_save_CXXFLAGS"
          LIBS="$ac_save_LIBS"
       fi
     fi

     XML_CFLAGS=""
     XML_LIBS=""
     ifelse([$3], , :, [$3])
  fi
  AC_SUBST(XML_CFLAGS)
  AC_SUBST(XML_LIBS)
  rm -f conf.xmltest
])



dnl we must provide our own macro for libvorbis as they haed the great
dnl idea to rename the macro from version 1.0rc3 to 1.0 (final). The
dnl previosly called AM_PATH_VORBIS macro works fine and for that
dnl reason is included below. The new on (XIPH_PATH_VORBIS) tells me
dnl (at least on our Debian Woody/Sarge systems, that it can't link
dnl against its own lib!

dnl the stuff is copied because I don't understand how to do it right
dnl it's part of the vorbis / vorbis_nmm hack begun above

# Configure paths for libvorbis
# Jack Moffitt <jack@icecast.org> 10-21-2000
# Shamelessly stolen from Owen Taylor and Manish Singh

dnl AM_PATH_VORBIS([ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]])
dnl Test for libvorbis, and define VORBIS_CFLAGS and VORBIS_LIBS
dnl
AC_DEFUN([AM_PATH_VORBIS_NMM],
[dnl 
dnl Get the cflags and libraries
dnl
AC_ARG_WITH(vorbis,[  --with-vorbis=PFX           Prefix where libvorbis is installed (optional)], vorbis_prefix="$withval", vorbis_prefix="")
AC_ARG_WITH(vorbis-libraries,[  --with-vorbis-libraries=DIR Directory where libvorbis library is installed (optional)], vorbis_libraries="$withval", vorbis_libraries="")
AC_ARG_WITH(vorbis-includes,[  --with-vorbis-includes=DIR  Directory where libvorbis header files are installed (optional)], vorbis_includes="$withval", vorbis_includes="")
AC_ARG_ENABLE(vorbistest, [  --disable-vorbistest        Do not try to compile and run a test Vorbis program],, enable_vorbistest=yes)

  if test "x$vorbis_libraries" != "x" ; then
    VORBIS_LIBS="-L$vorbis_libraries"
  elif test "x$vorbis_prefix" != "x" ; then
    VORBIS_LIBS="-L$vorbis_prefix/lib"
  elif test "x$prefix" != "xNONE"; then
    VORBIS_LIBS="-L$prefix/lib"
  fi

  VORBIS_LIBS="$VORBIS_LIBS $USER_LDFLAGS -lvorbis_nmm -lm"
  VORBISFILE_LIBS="-lvorbisfile_nmm"
  VORBISENC_LIBS="$USER_LDFLAGS -lvorbisenc_nmm"

  if test "x$vorbis_includes" != "x" ; then
    VORBIS_CFLAGS="-I$vorbis_includes"
  elif test "x$vorbis_prefix" != "x" ; then
    VORBIS_CFLAGS="-I$vorbis_prefix/include"
  elif test "x$prefix" != "xNONE"; then
    VORBIS_CFLAGS="-I$prefix/include"
  fi


  AC_MSG_CHECKING(for Vorbis_nmm)
  no_vorbis=""


  if test "x$enable_vorbistest" = "xyes" ; then
    ac_save_CFLAGS="$CFLAGS"
    ac_save_LIBS="$LIBS"
    CFLAGS="$CFLAGS $VORBIS_CFLAGS $OGG_CFLAGS $USER_INCLUDES"
    LIBS="$LIBS $VORBIS_LIBS $OGG_LIBS $USER_INCLUDES $USER_LDFLAGS"
dnl
dnl Now check if the installed Vorbis is sufficiently new.
dnl
      rm -f conf.vorbistest
      AC_TRY_RUN([
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <vorbis/codec.h>
// first test
int main ()
{
  system("touch conf.vorbistest");
  return 0;
}

],, no_vorbis=yes,[echo $ac_n "cross compiling; assumed OK... $ac_c"])
       CFLAGS="$ac_save_CFLAGS"
       LIBS="$ac_save_LIBS"
  fi

  if test "x$no_vorbis" = "x" ; then
     AC_MSG_RESULT(yes)
     ifelse([$1], , :, [$1])
  else
     AC_MSG_RESULT(no)
     if test -f conf.vorbistest ; then
       :
     else
       CFLAGS="$CFLAGS $VORBIS_CFLAGS"
       LIBS="$LIBS $VORBIS_LIBS $OGG_LIBS"
       AC_TRY_LINK([
#include <stdio.h>
#include <vorbis/codec.h>
],     [ return 0; ],
       [ echo "*** The test program compiled, but did not run. This usually means"
       echo "*** that the run-time linker is not finding Vorbis or finding the wrong"
       echo "*** version of Vorbis. If it is not finding Vorbis, you'll need to set your"
       echo "*** LD_LIBRARY_PATH environment variable, or edit /etc/ld.so.conf to point"
       echo "*** to the installed location  Also, make sure you have run ldconfig if that"
       echo "*** is required on your system"
       echo "***"
       echo "*** If you have an old version installed, it is best to remove it, although"
       echo "*** you may also be able to get things to work by modifying LD_LIBRARY_PATH"],
       [])
       CFLAGS="$ac_save_CFLAGS"
       LIBS="$ac_save_LIBS"
     fi
     VORBIS_CFLAGS=""
     VORBIS_LIBS=""
     VORBISFILE_LIBS=""
     VORBISENC_LIBS=""
     ifelse([$2], , :, [$2])
  fi
  AC_SUBST(VORBIS_CFLAGS)
  AC_SUBST(VORBIS_LIBS)
  AC_SUBST(VORBISFILE_LIBS)
  AC_SUBST(VORBISENC_LIBS)
  rm -f conf.vorbistest
])


dnl AM_PATH_VORBIS([ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]])
dnl Test for libvorbis, and define VORBIS_CFLAGS and VORBIS_LIBS
dnl
AC_DEFUN([AM_PATH_VORBIS],
[dnl 
dnl Get the cflags and libraries
dnl
AC_ARG_WITH(vorbis,[  --with-vorbis=PFX          Prefix where libvorbis is installed (optional)], vorbis_prefix="$withval", vorbis_prefix="")
AC_ARG_WITH(vorbis-libraries,[  --with-vorbis-libraries=DIR Directory where libvorbis library is installed (optional)], vorbis_libraries="$withval", vorbis_libraries="")
AC_ARG_WITH(vorbis-includes,[  --with-vorbis-includes=DIR  Directory where libvorbis header files are installed (optional)], vorbis_includes="$withval", vorbis_includes="")
AC_ARG_ENABLE(vorbistest, [  --disable-vorbistest        Do not try to compile and run a test Vorbis program],, enable_vorbistest=yes)

  if test "x$vorbis_libraries" != "x" ; then
    VORBIS_LIBS="-L$vorbis_libraries"
  elif test "x$vorbis_prefix" != "x" ; then
    VORBIS_LIBS="-L$vorbis_prefix/lib"
  elif test "x$prefix" != "xNONE"; then
    VORBIS_LIBS="-L$prefix/lib"
  fi

  VORBIS_LIBS="$VORBIS_LIBS -lvorbis -lm"
  VORBISFILE_LIBS="-lvorbisfile"
  VORBISENC_LIBS="-lvorbisenc"

  if test "x$vorbis_includes" != "x" ; then
    VORBIS_CFLAGS="-I$vorbis_includes"
  elif test "x$vorbis_prefix" != "x" ; then
    VORBIS_CFLAGS="-I$vorbis_prefix/include"
  elif test "x$prefix" != "xNONE"; then
    VORBIS_CFLAGS="-I$prefix/include"
  fi


  AC_MSG_CHECKING(for Vorbis)
  no_vorbis=""


  if test "x$enable_vorbistest" = "xyes" ; then
    ac_save_CFLAGS="$CFLAGS"
    ac_save_LIBS="$LIBS"
    CFLAGS="$CFLAGS $VORBIS_CFLAGS $OGG_CFLAGS"
    LIBS="$LIBS $VORBIS_LIBS $OGG_LIBS"
dnl
dnl Now check if the installed Vorbis is sufficiently new.
dnl
      rm -f conf.vorbistest
      AC_TRY_RUN([
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <vorbis/codec.h>

int main ()
{
  system("touch conf.vorbistest");
  return 0;
}

],, no_vorbis=yes,[echo $ac_n "cross compiling; assumed OK... $ac_c"])
       CFLAGS="$ac_save_CFLAGS"
       LIBS="$ac_save_LIBS"
  fi

  if test "x$no_vorbis" = "x" ; then
     AC_MSG_RESULT(yes)
     ifelse([$1], , :, [$1])     
  else
     AC_MSG_RESULT(no)
     if test -f conf.vorbistest ; then
       :
     else
       echo "*** Could not run Vorbis test program, checking why..."
       CFLAGS="$CFLAGS $VORBIS_CFLAGS"
       LIBS="$LIBS $VORBIS_LIBS $OGG_LIBS"
       AC_TRY_LINK([
#include <stdio.h>
#include <vorbis/codec.h>
],     [ return 0; ],
       [ echo "*** The test program compiled, but did not run. This usually means"
       echo "*** that the run-time linker is not finding Vorbis or finding the wrong"
       echo "*** version of Vorbis. If it is not finding Vorbis, you'll need to set your"
       echo "*** LD_LIBRARY_PATH environment variable, or edit /etc/ld.so.conf to point"
       echo "*** to the installed location  Also, make sure you have run ldconfig if that"
       echo "*** is required on your system"
       echo "***"
       echo "*** If you have an old version installed, it is best to remove it, although"
       echo "*** you may also be able to get things to work by modifying LD_LIBRARY_PATH"],
       [ echo "*** The test program failed to compile or link. See the file config.log for the"
       echo "*** exact error that occured. This usually means Vorbis was incorrectly installed"
       echo "*** or that you have moved Vorbis since it was installed." ])
       CFLAGS="$ac_save_CFLAGS"
       LIBS="$ac_save_LIBS"
     fi
     VORBIS_CFLAGS=""
     VORBIS_LIBS=""
     VORBISFILE_LIBS=""
     VORBISENC_LIBS=""
     ifelse([$2], , :, [$2])
  fi
  AC_SUBST(VORBIS_CFLAGS)
  AC_SUBST(VORBIS_LIBS)
  AC_SUBST(VORBISFILE_LIBS)
  AC_SUBST(VORBISENC_LIBS)
  rm -f conf.vorbistest
])
