/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#include "ClicConsoleInput.hpp"

#include "nmm/utils/thread/MutexGuard.hpp"

#include "nmm/utils/net/NetReactor.hpp"

#include <iostream>
#include <cassert>

namespace NMM {
  
  ClicConsoleInput::ClicConsoleInput()
    : __interactive(true),
      __interrupted(false),
      __cond_mutex(),
      __condition(__cond_mutex)
  {}
  
  ClicConsoleInput::~ClicConsoleInput() {
    /* unregister the file descriptor for standard input */
    SNetReactor::getInstance().unregisterSocket(0);
  }
  
  char ClicConsoleInput::waitUntilKeyPressed() {
    /* register the file descriptor and wait for an event */
    MutexGuard mg(__cond_mutex);
    if(__interactive) {
      SNetReactor::getInstance().registerSocket(0, this);
    }
    __condition.wait();
    if(__interrupted) {
      /* if we were interrupted, no key has been pressed */
      return 0;
    }
    
    /* read the character from the console and return it */
    assert(__interactive);
    char tmp;
    std::cin >> tmp;
    return tmp;
  }
  
  void ClicConsoleInput::interrupt() {
    /* Notify the main thread that it was interrupted */
    MutexGuard mg(__cond_mutex);
    __interrupted = true;
    __condition.notify();
  }
  
  NetHandler::NetResult ClicConsoleInput::receiveEvent(SocketHandle, short, short ) {
    /* Notify the main thread to read data from stdin */
    MutexGuard mg(__cond_mutex);
    __interrupted = false;
    __condition.notify();
    
    /* unregister the file descriptor */
    return NetHandler::REMOVE;
  }
  
}
