/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "Timer.hpp"
#include "TimerListener.hpp"

namespace NMM {
  Timer::Timer (TimerListener& listener) : __is_running(false), __listener(listener), __thread_condition(__condition_mutex), __sec(0), __nsec(0)
  {

  }

  Timer::~Timer() {
    stop();
  }

  void Timer::start(int sec, int nsec) {
    MutexGuard mg(__condition_mutex);
    if ( !__is_running ) {
      __sec         = sec;
      __nsec        = nsec;
      __is_running  = true;
      activateThread();
    }
  }

  void Timer::stop() {
    if ( __is_running ) {
      __condition_mutex.lock();
      __is_running = false;
      __thread_condition.notify();
      __condition_mutex.unlock();
      deactivateThread();
    }
  }

  void Timer::run() {
    while (__is_running) {
      __condition_mutex.lock();  // lock the mutex
      __thread_condition.relativeTimedWait(__sec, __nsec);

      if (__is_running) {
        __condition_mutex.unlock();
        __listener.timeout(this);
      } else {
        __condition_mutex.unlock();
      }
    }
  }

} // NMM

