/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#ifndef _NMM_TIMERLISTENER_HPP_
#define _NMM_TIMERLISTENER_HPP_

#include "Timer.hpp"

namespace NMM {
  /** Listener for Timer  
   * 
   * Class to be implemented by other classes, which use a timer to be
   * notified of elapsed time period.
   */
  class TimerListener
  {
  public:
    /** Destructor
     */
    virtual ~TimerListener() {}

    /** Listener callback.
     * 
     * This method is called from the timer.  It must be implemented
     * by classes inheriting from this class.
     *
     * @param timer timer that has expired
     */
    virtual void timeout(Timer* timer) = 0;        
  };

}

#endif
