/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

// rtp streaming server for video

#include <iostream>
#include <getopt.h>
#include "nmm/utils/directory_utils.hpp"
#include "nmm/comm/netstrategy/RTPAddress.hpp"
#include "nmm/comm/netstrategy/MulticastIPAddress.hpp"
// include NMM 
#include "nmm/multimedia/connect.hpp"
#include "nmm/comm/ProxyObject.hpp"
#include "nmm/services/registry/ProxyApplication.hpp"
#include "nmm/services/registry/GraphDescription.hpp"

// include used interfaces
#include "nmm/plugins/interfaces/IFileHandler.hpp"
#include "nmm/plugins/net/INet.hpp"

// import namespace
using namespace NMM;


void usage(const string& name) {
  cerr << "Usage :" << endl
       << "  " << name << " [options] -i <input-file> -a <audio-port> -v <video-port>" << endl
       << endl
       << "Options:" << endl
       << "  -i <input file> : Sets an input file " << endl
       << "  -a <audio-port> : Sets the multicast port to stream audio " << endl
       << "  -v <video-port> : Sets the multicast port to stream video " << endl
       << "  -V : enable verbose mode" << endl
       << "  -h : Show this help" << endl
       << "  -n : specifies the number of streamed videos" << endl
       << endl;
}


int audio_rtp_port  = 0;
int audio_rtcp_port = 0;
  
int video_rtp_port  = 0;
int video_rtcp_port = 0;

int transcoded_video_rtp_port = 0;
int transcoded_video_rtcp_port = 0;


string input_file;

string transcoding_host = "localhost";


void start_graph(bool ac3_mpeg, GraphDescription& graph, NMMApplication* app) { 

}


int main(int argc, char ** argv)
{
  if(argc < 4) {
    usage( argv[0] );
    return 1;
  }
  
  bool ac3_mpeg = false;
    
  const char *opt_string = "n:i:a:v:dVh";
  struct option options [] = {
    {"input-file",    1,NULL,'i'},
    {"audio-port",    1,NULL,'a'},
    {"video-port"    ,1,NULL,'v'},
    {"Verbose"       ,0,NULL,'V'},
    {"dvd",           0,NULL,'d'},
    {"help",          0,NULL,'h'},
    {NULL,            0,NULL,0}
  };
  
  int ch,long_option_index;
  int counter = 1;
  int verbose = 0;
  while((ch=getopt_long(argc,argv,opt_string,options,&long_option_index))!=EOF){
      // which option did we get?
    switch (ch) {
      case 'i':
        input_file = getAbsolutePath(optarg);
        break;
      case 'a':
        audio_rtp_port  = atoi(optarg);
	audio_rtcp_port = audio_rtp_port+1;
        break;
      case 'v':
        video_rtp_port  = atoi(optarg);
	video_rtcp_port = video_rtp_port+1;
        transcoded_video_rtp_port  =video_rtp_port+2;
	transcoded_video_rtcp_port = video_rtp_port+3;
        break;
      case 'V':
        ++verbose; 
        break;
      case 'd':
        ac3_mpeg = true;
        break;
      case 'n':
        counter = atoi(optarg);
	break;
      case 'h':
        usage(argv[0]);
        exit(0);         
      default:
        usage(argv[0]);
        throw Exception("Invalid option");
    }
  }
    
  cerr << argv[0] << " running ... " << endl << endl;
  
  // disable all debug output (error, warning, message, debug)
  // see documentation Debug, Error, Warning and Message Output in NMM
  NamedObject::getGlobalInstance().setErrorStream(0, NamedObject::ALL_LEVELS);
  NamedObject::getGlobalInstance().setWarningStream(0, NamedObject::ALL_LEVELS);
  NamedObject::getGlobalInstance().setMessageStream(0, NamedObject::ALL_LEVELS);
  NamedObject::getGlobalInstance().setDebugStream(0, NamedObject::ALL_LEVELS);
  if (verbose > 0) {
    NamedObject::getGlobalInstance().setErrorStream(&cout, NamedObject::ALL_LEVELS);
  }
  if (verbose > 1) {
    NamedObject::getGlobalInstance().setWarningStream(&cout, NamedObject::ALL_LEVELS);
  } 
  if (verbose > 2) {
    NamedObject::getGlobalInstance().setMessageStream(&cout, NamedObject::ALL_LEVELS);
  }
  if (verbose > 3) {
    NamedObject::getGlobalInstance().setDebugStream(&cout, NamedObject::ALL_LEVELS);
  }

  // the NMM application for the example
  NMMApplication* app = ProxyApplication::getApplication(argc, argv);

  // the description of the graph we use	
  GraphDescription* graph = new GraphDescription();
    
  // NodeDescriptions for streaming flowgraph without transcoding.
  NodeDescription readfile_descr;
  readfile_descr.setNodeName("GenericReadNode");
  
  NodeDescription mpeg_demux("MPEGDemuxNode");
  NodeDescription mpeg_vsh("MPEGVSHDetectionNode");
  
  NodeDescription ac3_dec("AC3DecodeNode");
  NodeDescription mpeg_enc("MPEGAudioEncodeNode");

  NodeDescription audio_netsink_descr("NetSinkNode");
  NodeDescription video_netsink_descr("NetSinkNode");

  
  // NodeDescriptions for streaming flowgraph with transcoding video 
  NodeDescription mpeg_dec("MPEGVideoDecodeNode");
  mpeg_dec.setLocation(transcoding_host);
 
  NodeDescription deinterlace_descr("YUVDeInterlaceNode");
  deinterlace_descr.setLocation(transcoding_host);

  NodeDescription scaler ("VideoScalerNode");
  scaler.setLocation(transcoding_host);

  NodeDescription ffmpeg_enc("FFMpegEncodeNode");
  ffmpeg_enc.setLocation(transcoding_host);

  NodeDescription video_transcode_netsink("NetSinkNode");
  video_transcode_netsink.setLocation(transcoding_host);


  // create a flow graph: readfile -> net_sink 
  graph->addEdge(readfile_descr, mpeg_demux);

  
  if ( ac3_mpeg ) {  // build graph with ac3 audio

    graph->addEdge(mpeg_demux,  "ac3_audio0", ac3_dec);
    Format enc_format(Format::audio_mpeg_type);
    enc_format.addIntParamValue(Format::bitrate_param, 320000);
    graph->addEdge(ac3_dec, mpeg_enc);     
    
    graph->addEdge(mpeg_enc   , "default",    mpeg_vsh, "audio_input_stream", &enc_format );
    graph->addEdge(mpeg_vsh,  "audio_output_stream", audio_netsink_descr);
    
    graph->addEdge(mpeg_demux ,"mpeg_video0", mpeg_vsh, "video_input_stream"); 
    graph->addEdge(mpeg_vsh,   "video_output_stream", video_netsink_descr);    

  } else {  // build graph with mpeg audio

    graph->addEdge(mpeg_demux,  "mpeg_audio0", mpeg_vsh, "audio_input_stream");  
    graph->addEdge(mpeg_demux,  "mpeg_video0", mpeg_vsh, "video_input_stream");  
    graph->addEdge(mpeg_vsh,  "audio_output_stream", audio_netsink_descr);
    graph->addEdge(mpeg_vsh,  "video_output_stream", video_netsink_descr);    

    // define the formats for transcoding nodes
    Format video_format(Format::video_mpeg_type);
    video_format.addIntParamValue(Format::version_param, 2);
    video_format.addIntParamValue(Format::x_resolution_param, 720);
    video_format.addIntParamValue(Format::y_resolution_param, 540);
    video_format.addIntParamValue(Format::ratio_x_param, 4);
    video_format.addIntParamValue(Format::ratio_y_param, 3);
    video_format.addFloatParamValue(Format::framerate_param, 25.0);

    Format scaler_format(Format::video_raw_type);
    scaler_format.addIntParamValue(Format::x_resolution_param, 320);
    scaler_format.addIntParamValue(Format::y_resolution_param, 240);
    scaler_format.addIntParamValue(Format::ratio_x_param, 320);
    scaler_format.addIntParamValue(Format::ratio_y_param, 240);
    scaler_format.addFloatParamValue(Format::framerate_param, 25.0);

    
     Format video_encoder_format(Format::video_h263plus_type);
     video_encoder_format.addIntParamValue(Format::bitrate_param, 1000000);

    // Setup the transcoding flow graph 
    graph->addEdge(mpeg_vsh, "video_output_stream", mpeg_dec, &video_format);
    graph->addEdge(mpeg_dec, scaler);
    graph->addEdge(scaler, ffmpeg_enc, &scaler_format);
    graph->addEdge(ffmpeg_enc, video_transcode_netsink, &video_encoder_format);
  }   

  
  // request complete graph
  cerr << "request GraphDescription from registry" << endl;
  ClientRegistry& registry = app->getRegistry();
  registry.requestGraph(*graph);

  // set the filename by requesting appropriate interface
  cerr << "set filename to " << input_file << endl;
  INode* readfile = graph->getINode(readfile_descr);
  IFileHandler_var filehandler (readfile->getParentObject()->getCheckedInterface<IFileHandler>());
  filehandler->setFilename(input_file);

  
  // define and set the Multicast IP addres for audio
  INode* netsink = graph->getINode(audio_netsink_descr);
  INet_var i_net (netsink->getParentObject()->getCheckedInterface<INet>());
  MulticastIPAddress ip_addr("239.0.0.2");
  RTPAddress rtp_addr (ip_addr, audio_rtp_port, audio_rtcp_port);
  i_net->setTransportStrategy(&rtp_addr);

  cerr << "Using audio rtp port : " << audio_rtp_port << endl;
  cerr << "Using audio rtcp port : " << audio_rtcp_port << endl;
  

  // define and set the Multicast IP addres for video
  INode* netsink2 = graph->getINode(video_netsink_descr);
  INet_var i_net2 (netsink2->getParentObject()->getCheckedInterface<INet>());
  MulticastIPAddress ip_addr2("239.0.0.2");
  RTPAddress rtp_addr2 (ip_addr2, video_rtp_port, video_rtcp_port);
  i_net2->setTransportStrategy(&rtp_addr2);

  cerr << "Using video rtp port : " << video_rtp_port << endl;
  cerr << "Using video rtcp port : " << video_rtcp_port << endl;


  // define and set the Multicast IP addres for transcoded video
  INode* netsink3 = graph->getINode(video_transcode_netsink);
  INet_var i_net3 (netsink3->getParentObject()->getCheckedInterface<INet>());
  MulticastIPAddress ip_addr3("239.0.0.2");
  RTPAddress rtp_addr3 (ip_addr3, transcoded_video_rtp_port, transcoded_video_rtcp_port);
  i_net3->setTransportStrategy(&rtp_addr3);

  cerr << "Using transcoded video rtp port : " << transcoded_video_rtp_port << endl;
  cerr << "Using transcoded video rtcp port : " << transcoded_video_rtcp_port << endl;
  
  // realize and start graph
  cerr << "realizeGraph()" << endl;
  graph->realizeGraph();
  cerr << "startGraph()" << endl;
  graph->startGraph();   

      
  // wait ...
  cerr << "Playing audio ... enter 'q <enter>' to exit." << endl;
  string s;
  cin >> s;

    // stop and release graph
    cerr << "stopGraph()" << endl;
    graph->stopGraph();

    cerr << "releaseGraph()" << endl;
    app->getRegistry().releaseGraph(*graph);


  delete app;
  return 0;
}
