/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "BindingException.hpp"

#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  BindingException::BindingException()
    : SerializedException("BindingException"),
      __input_jack(0),
      __output_jack(0)
  {
    setTypeName("BindingException");
  }
  
  BindingException::BindingException(const std::string& s)
    : SerializedException("BindingException : " + s),
      __input_jack(0),
      __output_jack(0)
  {
    setTypeName("BindingException");
  }
  
  BindingException::BindingException(const InputJack* i_jack, const OutputJack* o_jack, const std::string& s)
    : SerializedException("BindingException : " + s),
      __input_jack(i_jack),
      __output_jack(o_jack)
  {
    setTypeName("BindingException");
  }
  
  BindingException* BindingException::copy() const {
    return new BindingException(*this);
  }
  
  TConstructor<BindingException> __nmm__binding_exception("BindingException", "NMM");
  
}

#include "nmm/nmm_suffix.hpp"
