/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 * 
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "CompositeAddress.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  CompositeAddress::CompositeAddress(const NMMReference& ref, const std::string& strategy_type, const TypeInfo& t_info) throw()
    : TSAddress(ref, strategy_type, t_info, LOCAL | SYSTEM | NETWORK)
  {
    __child_addresses.useTValue(false);
    registerAttributes();
  }
  
  CompositeAddress::CompositeAddress(const CompositeAddress& address) throw()
    : TSAddress(address)
  {
    __child_addresses.useTValue(false);
    
    for(unsigned int i=0; i<address.__child_addresses.size(); ++i) {
      if(address.__child_addresses[i]) {
        __child_addresses.push_back(address.__child_addresses[i]->copy());
      } else {
        __child_addresses.push_back(0);
      }
    }
    registerAttributes();
  }
  
 CompositeAddress::~CompositeAddress() {
    for (unsigned int i = 0; i < __child_addresses.size(); ++i ) {
      delete __child_addresses[i];
    }
  }
  
  unsigned int CompositeAddress::addAddress(const TSAddress& address) throw() {
    /* update binding type */
    setBindingType(getBindingType() & address.getBindingType());
    
    /* add address to address list */
    __child_addresses.push_back(address.copy());
    return __child_addresses.size() - 1;
  }
  
  unsigned int CompositeAddress::getNumberOfAddresses() const throw() {
    return __child_addresses.size();
  }
  
  const TSAddress* CompositeAddress::getAddress(unsigned int ts_index) const throw() {
    if(__child_addresses[ts_index]) {
      return dynamic_cast<const TSAddress*>(__child_addresses[ts_index]);
    } else {
      return 0;
    }
  }
  
  void CompositeAddress::registerAttributes() throw() {
    insertArgument(&__child_addresses);
  }
  
};

#include "nmm/nmm_suffix.hpp"
