/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ConnectorAddress.hpp"
#include "IConnectionManager.hpp"

#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  ConnectorAddress::ConnectorAddress()
    : TSAddress(0, string(), TypeInfo("ConnectorAddress", "NMM"), TSAddress::LOCAL | TSAddress::SYSTEM | TSAddress::NETWORK),
      __icm(NULL),
      __ts_reference(0)
  {
    insertArgument(&__icm);
    insertArgument(&__ts_reference);
  }
  
  ConnectorAddress::ConnectorAddress(const string& strategy_type, const NMMReference& ref /*= 0*/)
    : TSAddress(ref, strategy_type, TypeInfo("ConnectorAddress", "NMM"), TSAddress::LOCAL | TSAddress::SYSTEM | TSAddress::NETWORK),
      __icm(NULL),
      __ts_reference(0)
  {
    insertArgument(&__icm);
    insertArgument(&__ts_reference);
  }

  ConnectorAddress::ConnectorAddress(const TypeInfo& info, unsigned int binding_type, const string& strategy_type, const NMMReference& ref /*= 0*/)
    : TSAddress(ref, strategy_type, info, binding_type),
      __icm(NULL),
      __ts_reference(0)
  {
    insertArgument(&__icm);
    insertArgument(&__ts_reference);
  }
  
  ConnectorAddress::ConnectorAddress(const ConnectorAddress& other)
    : TSAddress(other),
      __icm(NULL),
      __ts_reference(other.__ts_reference)
  { 
    if (other.__icm.getValue() != NULL) {
      __icm.setValue(dynamic_cast<IConnectionManager*>(other.__icm.getValue()->copy()));
    }
    
    insertArgument(&__icm);
    insertArgument(&__ts_reference);
  }
  
  ConnectorAddress* ConnectorAddress::copy() const {
    return new ConnectorAddress(*this);
  }
  
  ConnectorAddress::~ConnectorAddress() {
    delete __icm.getRValue();
    __icm.setValue(NULL);
  }
  
  ConnectorAddress& ConnectorAddress::operator=(const ConnectorAddress& other) {
    TSAddress::operator=(other);
    
    if(&other != this) {
      delete __icm.getRValue();
      if (other.__icm.getValue() != NULL) {
        __icm.setValue(dynamic_cast<IConnectionManager*>(other.__icm.getValue()->copy()));
      }
      
      __ts_reference = other.__ts_reference;
    }
    return *this;
  }
  
  void ConnectorAddress::setConnectionManager(const IConnectionManager& icm) {
    __icm.setValue(dynamic_cast<IConnectionManager*>(icm.copy()));
  }
  
  
  template<>
  class TConstructor<ConnectorAddress>
    : public ValueConstructor
  {
  public:
    TConstructor(const TypeInfo& t_info)
      : ValueConstructor(t_info)
    {}
    
    virtual ConnectorAddress*          createValue()    const {return new ConnectorAddress();}
    virtual TValue<ConnectorAddress>*  createTValue()   const {return new TValue<ConnectorAddress>();}
    
    virtual TValue<TSAddress*>*  createTPointerValue()  const {return new TValue<TSAddress*>();}
    
    virtual TConstructor<ConnectorAddress>* copy()      const {return new TConstructor(*this); }
  private:
    string __type_name;
    string __namespace;
  };
  
  TConstructor<ConnectorAddress> __connector_address(TypeInfo("ConnectorAddress", "NMM"));
  
}

#include "nmm/nmm_suffix.hpp"
