/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "FormatStrategyMap.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  FormatStrategyMap::FormatStrategyMap()
  {}
  
  FormatStrategyMap::~FormatStrategyMap()
  {}
  
  void FormatStrategyMap::addSupportedFormat(const TSDescription& strategy_type, const Format& format) {
    __format_map.push_back(pair<Format, TSDescription>(format, strategy_type));
  }
  
  std::list<TSDescription> FormatStrategyMap::getSupportedStrategies(const Format& format) const {
    std::list<TSDescription> result;
    
    for (FormatMapConstIterator i(__format_map.begin());
         i != __format_map.end();
         ++i) {      
      Format_var intersection(format.createIntersectionFormat(&i->first));
      if (intersection.get()) {
        result.push_back(i->second); 
      }
    }
    
    result.unique();
    return result;
  }

  std::list<Format*> FormatStrategyMap::getSupportedFormats(const TSDescription& strategy_type) {
    std::list<Format*> result;
    
    for (FormatMapIterator i(__format_map.begin());
         i != __format_map.end();
         ++i) {
      if (i->second == strategy_type) {
	result.push_back(&i->first); 
      }
    }
    
    result.unique();
    return result;
  }
  
  bool FormatStrategyMap::isSupported(const Format& format, const TSDescription& strategy_type) {
    for (FormatMapConstIterator i(__format_map.begin());
         i != __format_map.end();
         ++i) {
      if (i->second == strategy_type) {
        Format_var intersection(format.createIntersectionFormat(&i->first));
        if (intersection.get()) {
          return true;
        }
      }
    }
    
    return false;
  }
  
  void FormatStrategyMap::removeStrategyType(const TSDescription& strategy_type) {
    for (FormatMapIterator i(__format_map.begin());
         i != __format_map.end();
         ) {
      if (i->second == strategy_type) {
        i = __format_map.erase(i);
	  } else {
        ++i;
	  }
    }
  }
  
  void FormatStrategyMap::display(ostream& out) {
    /* Begin> Find all formats */
    Format top("*/*");
    for (FormatMapConstIterator i(__format_map.begin());
         i != __format_map.end();
         ++i) {
      out << "==== Format: " << endl
          << i->first << endl
          << "==== Supports " << i->second << endl;
    }
    /* End< */
  }
}

#include "nmm/nmm_suffix.hpp"
