/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ICompositeStrategy_HPP
#define NMM_ICompositeStrategy_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "ITransportStrategy.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Interface for composite transport strategies.
   * This is the interface of the %CompositeStrategy.
   */

  class DLLEXPORT ICompositeStrategy : virtual public ITransportStrategy { 
  public:
    ICompositeStrategy() : NMM::Interface(ICompositeStrategy::ICompositeStrategy_type, "NMM") { };

    ICompositeStrategy(NMM::NMMObject& parent) : NMM::Interface (parent, ICompositeStrategy::ICompositeStrategy_type, "NMM") { };

    virtual ~ICompositeStrategy() {};

    static const char* ICompositeStrategy_type;

    virtual  string getInterfaceType() const { return ICompositeStrategy_type; }

    NMM::Interface* copy() const { return new ICompositeStrategy(*this); }
    static void used(); 
    static const char* getChildStrategyCount_event;
    static const char* getChildStrategy_event;
    /** Returns number of child strategies.
     *
     * @return The number of child strategies in the composite
     */

    virtual nmm_uint32 getChildStrategyCount() const
      throw(const NMM::SerializedException&);
    /** Returns interface for child strategy.
     *
     * @param index Index of child strategy
     * @return Interface to the child strategy with the given index
     */

    virtual ITransportStrategy* getChildStrategy(const nmm_uint32& index) const
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ICompositeStrategy> ICompositeStrategy_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
