/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IInputJack.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/IOutputJack.hpp"

#include "nmm/comm/IInstreamCompositeStrategy.hpp"
#include "nmm/comm/InstreamChannel.hpp"

namespace NMM
{
  void IInputJack::used() {} 
  
  
  InstreamChannel* IInputJack::connectTo(const IOutputJack& jack) 
      throw(const NMM::SerializedException&, const BindingException&)
  {
    NMM::TOutValue<InstreamChannel* > return_value;
    NMM::TInValue<IOutputJack > jack_value(jack, false);
    NMM::Event event("IInputJack::connectTo", &jack_value);
    event.setReturnValue(&return_value);
    if(const_cast<IInputJack*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(BindingException* ex = dynamic_cast<BindingException*>(event.getException())) {
          throw BindingException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IOutputJack* IInputJack::getDataPartner()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<IOutputJack* > return_value;
    NMM::Event event("IInputJack::getDataPartner");
    event.setReturnValue(&return_value);
    if(const_cast<IInputJack*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IInstreamCompositeStrategy* IInputJack::getTransportStrategy() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<IInstreamCompositeStrategy* > return_value;
    NMM::Event event("IInputJack::getTransportStrategy");
    event.setReturnValue(&return_value);
    if(const_cast<IInputJack*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IInputJack::IInputJack_type = "IInputJack";
  NMM::TConstructor<IInputJack > __IInputJack_prototype("IInputJack","NMM");


  const char* IInputJack::connectTo_event = "IInputJack::connectTo";
  const char* IInputJack::getDataPartner_event = "IInputJack::getDataPartner";
  const char* IInputJack::getTransportStrategy_event = "IInputJack::getTransportStrategy";


}


#include "nmm/nmm_suffix.hpp"

