/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IInstreamCompositeStrategy.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IInstreamCompositeStrategy::used() {} 
  
  
  ITransportStrategy* IInstreamCompositeStrategy::getOneWayBufferReceiverStrategy() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ITransportStrategy* > return_value;
    NMM::Event event("IInstreamCompositeStrategy::getOneWayBufferReceiverStrategy");
    event.setReturnValue(&return_value);
    if(const_cast<IInstreamCompositeStrategy*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ITransportStrategy* IInstreamCompositeStrategy::getOneWayCEventReceiverStrategy() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ITransportStrategy* > return_value;
    NMM::Event event("IInstreamCompositeStrategy::getOneWayCEventReceiverStrategy");
    event.setReturnValue(&return_value);
    if(const_cast<IInstreamCompositeStrategy*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ITransportStrategy* IInstreamCompositeStrategy::getOneWayBufferSenderStrategy() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ITransportStrategy* > return_value;
    NMM::Event event("IInstreamCompositeStrategy::getOneWayBufferSenderStrategy");
    event.setReturnValue(&return_value);
    if(const_cast<IInstreamCompositeStrategy*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ITransportStrategy* IInstreamCompositeStrategy::getOneWayCEventSenderStrategy() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ITransportStrategy* > return_value;
    NMM::Event event("IInstreamCompositeStrategy::getOneWayCEventSenderStrategy");
    event.setReturnValue(&return_value);
    if(const_cast<IInstreamCompositeStrategy*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IInstreamCompositeStrategy::notifyEventReceived(const nmm_int32& stream_counter) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > stream_counter_value(stream_counter, false);
    NMM::Event event("IInstreamCompositeStrategy::notifyEventReceived", &stream_counter_value);
    if(const_cast<IInstreamCompositeStrategy*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IInstreamCompositeStrategy::setInstreamMuxStrategy(const IInstreamCompositeStrategy::MuxDemuxStrategy& strategy)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<IInstreamCompositeStrategy::MuxDemuxStrategy > strategy_value(strategy, false);
    NMM::Event event("IInstreamCompositeStrategy::setInstreamMuxStrategy", &strategy_value);
    if(const_cast<IInstreamCompositeStrategy*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IInstreamCompositeStrategy::setInstreamDemuxStrategy(const IInstreamCompositeStrategy::MuxDemuxStrategy& strategy)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<IInstreamCompositeStrategy::MuxDemuxStrategy > strategy_value(strategy, false);
    NMM::Event event("IInstreamCompositeStrategy::setInstreamDemuxStrategy", &strategy_value);
    if(const_cast<IInstreamCompositeStrategy*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IInstreamCompositeStrategy::setBufferLossInterval(const nmm_uint32& loss_interval)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > loss_interval_value(loss_interval, false);
    NMM::Event event("IInstreamCompositeStrategy::setBufferLossInterval", &loss_interval_value);
    if(const_cast<IInstreamCompositeStrategy*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IInstreamCompositeStrategy::IInstreamCompositeStrategy_type = "IInstreamCompositeStrategy";
  NMM::TConstructor<IInstreamCompositeStrategy > __IInstreamCompositeStrategy_prototype("IInstreamCompositeStrategy","NMM");
  TValueConstructor< IInstreamCompositeStrategy::TransportRole > __IInstreamCompositeStrategy_TransportRole_prototype(TypeInfo("IInstreamCompositeStrategy::TransportRole", "NMM"));

  NMM::OValueStream& operator<<(NMM::OValueStream& ov, const IInstreamCompositeStrategy::TransportRole& t)
  {
    ov.beginType(TypeInfo("IInstreamCompositeStrategy::TransportRole", "NMM"));
    nmm_int32 temp = static_cast<nmm_int32>(t);
    ov << temp;
    ov.endType();
    return ov;
  }

  NMM::IValueStream& operator>>(NMM::IValueStream& iv, IInstreamCompositeStrategy::TransportRole& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("IInstreamCompositeStrategy::TransportRole", "NMM")) {
      throw NMM::InvalidTypeException(TypeInfo("IInstreamCompositeStrategy::TransportRole", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    nmm_int32 temp;
    iv >> temp;
    t = static_cast<IInstreamCompositeStrategy::TransportRole>(temp);
    iv.goUp();
    return iv;
  }

  TValueConstructor< IInstreamCompositeStrategy::MuxDemuxStrategy > __IInstreamCompositeStrategy_MuxDemuxStrategy_prototype(TypeInfo("IInstreamCompositeStrategy::MuxDemuxStrategy", "NMM"));

  NMM::OValueStream& operator<<(NMM::OValueStream& ov, const IInstreamCompositeStrategy::MuxDemuxStrategy& t)
  {
    ov.beginType(TypeInfo("IInstreamCompositeStrategy::MuxDemuxStrategy", "NMM"));
    nmm_int32 temp = static_cast<nmm_int32>(t);
    ov << temp;
    ov.endType();
    return ov;
  }

  NMM::IValueStream& operator>>(NMM::IValueStream& iv, IInstreamCompositeStrategy::MuxDemuxStrategy& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("IInstreamCompositeStrategy::MuxDemuxStrategy", "NMM")) {
      throw NMM::InvalidTypeException(TypeInfo("IInstreamCompositeStrategy::MuxDemuxStrategy", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    nmm_int32 temp;
    iv >> temp;
    t = static_cast<IInstreamCompositeStrategy::MuxDemuxStrategy>(temp);
    iv.goUp();
    return iv;
  }



  const char* IInstreamCompositeStrategy::getOneWayBufferReceiverStrategy_event = "IInstreamCompositeStrategy::getOneWayBufferReceiverStrategy";
  const char* IInstreamCompositeStrategy::getOneWayCEventReceiverStrategy_event = "IInstreamCompositeStrategy::getOneWayCEventReceiverStrategy";
  const char* IInstreamCompositeStrategy::getOneWayBufferSenderStrategy_event = "IInstreamCompositeStrategy::getOneWayBufferSenderStrategy";
  const char* IInstreamCompositeStrategy::getOneWayCEventSenderStrategy_event = "IInstreamCompositeStrategy::getOneWayCEventSenderStrategy";
  const char* IInstreamCompositeStrategy::notifyEventReceived_event = "IInstreamCompositeStrategy::notifyEventReceived";
  NMM::Event* IInstreamCompositeStrategy::create_notifyEventReceived(const nmm_int32& stream_counter)
  {
    NMM::TInValue<nmm_int32 >* stream_counter_value = new NMM::TInValue<nmm_int32 >();
    stream_counter_value->setValue(stream_counter);

    return new NMM::Event("IInstreamCompositeStrategy::notifyEventReceived", stream_counter_value);
  }

  const char* IInstreamCompositeStrategy::setInstreamMuxStrategy_event = "IInstreamCompositeStrategy::setInstreamMuxStrategy";
  const char* IInstreamCompositeStrategy::setInstreamDemuxStrategy_event = "IInstreamCompositeStrategy::setInstreamDemuxStrategy";
  const char* IInstreamCompositeStrategy::setBufferLossInterval_event = "IInstreamCompositeStrategy::setBufferLossInterval";


}


#include "nmm/nmm_suffix.hpp"

