/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IJack.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/TSAddress.hpp"

namespace NMM
{
  void IJack::used() {} 
  
  
  string IJack::getJackTag() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("IJack::getJackTag");
    event.setReturnValue(&return_value);
    if(const_cast<IJack*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool IJack::isConnected() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("IJack::isConnected");
    event.setReturnValue(&return_value);
    if(const_cast<IJack*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IJack::setTransportStrategy(const TSAddress* address) 
      throw(const NMM::SerializedException&, const BindingException&)
  {
    NMM::TInValue<TSAddress* > address_value (const_cast<TSAddress* >(address), false);
    NMM::Event event("IJack::setTransportStrategy", &address_value);
    if(const_cast<IJack*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(BindingException* ex = dynamic_cast<BindingException*>(event.getException())) {
          throw BindingException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IJack::disconnect()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IJack::disconnect");
    if(const_cast<IJack*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IJack::IJack_type = "IJack";
  NMM::TConstructor<IJack > __IJack_prototype("IJack","NMM");


  const char* IJack::getJackTag_event = "IJack::getJackTag";
  const char* IJack::isConnected_event = "IJack::isConnected";
  const char* IJack::setTransportStrategy_event = "IJack::setTransportStrategy";
  const char* IJack::disconnect_event = "IJack::disconnect";


}


#include "nmm/nmm_suffix.hpp"

