/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "INamed.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void INamed::used() {} 
  
  
  void INamed::setName(const string& name)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > name_value(name, false);
    NMM::Event event("INamed::setName", &name_value);
    if(const_cast<INamed*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  string INamed::getName() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("INamed::getName");
    event.setReturnValue(&return_value);
    if(const_cast<INamed*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void INamed::setErrorStream(const string& filename, const nmm_int32& error_level)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > filename_value(filename, false);
    NMM::TInValue<nmm_int32 > error_level_value(error_level, false);
    NMM::Event event("INamed::setErrorStream", &filename_value, &error_level_value);
    if(const_cast<INamed*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void INamed::setDebugStream(const string& filename, const nmm_int32& debug_level)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > filename_value(filename, false);
    NMM::TInValue<nmm_int32 > debug_level_value(debug_level, false);
    NMM::Event event("INamed::setDebugStream", &filename_value, &debug_level_value);
    if(const_cast<INamed*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void INamed::setWarningStream(const string& filename, const nmm_int32& warning_level)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > filename_value(filename, false);
    NMM::TInValue<nmm_int32 > warning_level_value(warning_level, false);
    NMM::Event event("INamed::setWarningStream", &filename_value, &warning_level_value);
    if(const_cast<INamed*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void INamed::setMessageStream(const string& filename, const nmm_int32& message_level)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > filename_value(filename, false);
    NMM::TInValue<nmm_int32 > message_level_value(message_level, false);
    NMM::Event event("INamed::setMessageStream", &filename_value, &message_level_value);
    if(const_cast<INamed*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* INamed::INamed_type = "INamed";
  NMM::TConstructor<INamed > __INamed_prototype("INamed","NMM");


  const char* INamed::setName_event = "INamed::setName";
  const char* INamed::getName_event = "INamed::getName";
  const char* INamed::setErrorStream_event = "INamed::setErrorStream";
  const char* INamed::setDebugStream_event = "INamed::setDebugStream";
  const char* INamed::setWarningStream_event = "INamed::setWarningStream";
  const char* INamed::setMessageStream_event = "INamed::setMessageStream";


}


#include "nmm/nmm_suffix.hpp"

