/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IObjectAcceptor.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/serialize/StdValue.hpp"

namespace NMM
{
  void IObjectAcceptor::used() {} 
  
  
  void IObjectAcceptor::connectToObject(const NMMReference& reference) 
      throw(const NMM::SerializedException&, const BindingException&)
  {
    NMM::TInValue<NMMReference > reference_value(reference, false);
    NMM::Event event("IObjectAcceptor::connectToObject", &reference_value);
    if(const_cast<IObjectAcceptor*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(BindingException* ex = dynamic_cast<BindingException*>(event.getException())) {
          throw BindingException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IObjectAcceptor::connectToService(const string& service) 
      throw(const NMM::SerializedException&, const BindingException&)
  {
    NMM::TInValue<string > service_value(service, false);
    NMM::Event event("IObjectAcceptor::connectToService", &service_value);
    if(const_cast<IObjectAcceptor*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(BindingException* ex = dynamic_cast<BindingException*>(event.getException())) {
          throw BindingException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IObjectAcceptor::connectToJack(const NMMReference& reference) 
      throw(const NMM::SerializedException&, const BindingException&)
  {
    NMM::TInValue<NMMReference > reference_value(reference, false);
    NMM::Event event("IObjectAcceptor::connectToJack", &reference_value);
    if(const_cast<IObjectAcceptor*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(BindingException* ex = dynamic_cast<BindingException*>(event.getException())) {
          throw BindingException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IObjectAcceptor::connectToOutOfBandCompositeStrategy(NMMReference& reference, const nmm_uint32& roles) 
      throw(const NMM::SerializedException&, const BindingException&)
  {
    NMM::TInOutValue<NMMReference > reference_value;
    reference_value.setValue(reference);
    NMM::TInValue<nmm_uint32 > roles_value(roles, false);
    NMM::Event event("IObjectAcceptor::connectToOutOfBandCompositeStrategy", &reference_value, &roles_value);
    if(const_cast<IObjectAcceptor*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(BindingException* ex = dynamic_cast<BindingException*>(event.getException())) {
          throw BindingException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    reference = reference_value.getRValue();
  }
  
  void IObjectAcceptor::connectToInstreamCompositeStrategy(NMMReference& reference, const nmm_uint32& roles) 
      throw(const NMM::SerializedException&, const BindingException&)
  {
    NMM::TInOutValue<NMMReference > reference_value;
    reference_value.setValue(reference);
    NMM::TInValue<nmm_uint32 > roles_value(roles, false);
    NMM::Event event("IObjectAcceptor::connectToInstreamCompositeStrategy", &reference_value, &roles_value);
    if(const_cast<IObjectAcceptor*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(BindingException* ex = dynamic_cast<BindingException*>(event.getException())) {
          throw BindingException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    reference = reference_value.getRValue();
  }
  
  list<string >  IObjectAcceptor::getServiceList() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<list<string >  > return_value;
    NMM::Event event("IObjectAcceptor::getServiceList");
    event.setReturnValue(&return_value);
    if(const_cast<IObjectAcceptor*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IObjectAcceptor::IObjectAcceptor_type = "IObjectAcceptor";
  NMM::TConstructor<IObjectAcceptor > __IObjectAcceptor_prototype("IObjectAcceptor","NMM");


  const char* IObjectAcceptor::connectToObject_event = "IObjectAcceptor::connectToObject";
  const char* IObjectAcceptor::connectToService_event = "IObjectAcceptor::connectToService";
  const char* IObjectAcceptor::connectToJack_event = "IObjectAcceptor::connectToJack";
  const char* IObjectAcceptor::connectToOutOfBandCompositeStrategy_event = "IObjectAcceptor::connectToOutOfBandCompositeStrategy";
  const char* IObjectAcceptor::connectToInstreamCompositeStrategy_event = "IObjectAcceptor::connectToInstreamCompositeStrategy";
  const char* IObjectAcceptor::getServiceList_event = "IObjectAcceptor::getServiceList";


}


#include "nmm/nmm_suffix.hpp"

