/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IOutOfBandCompositeStrategy.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IOutOfBandCompositeStrategy::used() {} 
  
  
  ITransportStrategy* IOutOfBandCompositeStrategy::getOneWayEventRequestReceiverStrategy() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ITransportStrategy* > return_value;
    NMM::Event event("IOutOfBandCompositeStrategy::getOneWayEventRequestReceiverStrategy");
    event.setReturnValue(&return_value);
    if(const_cast<IOutOfBandCompositeStrategy*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ITransportStrategy* IOutOfBandCompositeStrategy::getOneWayEventReplyReceiverStrategy() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ITransportStrategy* > return_value;
    NMM::Event event("IOutOfBandCompositeStrategy::getOneWayEventReplyReceiverStrategy");
    event.setReturnValue(&return_value);
    if(const_cast<IOutOfBandCompositeStrategy*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ITransportStrategy* IOutOfBandCompositeStrategy::getOneWayEventRequestSenderStrategy() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ITransportStrategy* > return_value;
    NMM::Event event("IOutOfBandCompositeStrategy::getOneWayEventRequestSenderStrategy");
    event.setReturnValue(&return_value);
    if(const_cast<IOutOfBandCompositeStrategy*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ITransportStrategy* IOutOfBandCompositeStrategy::getOneWayEventReplySenderStrategy() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ITransportStrategy* > return_value;
    NMM::Event event("IOutOfBandCompositeStrategy::getOneWayEventReplySenderStrategy");
    event.setReturnValue(&return_value);
    if(const_cast<IOutOfBandCompositeStrategy*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ITransportStrategy* IOutOfBandCompositeStrategy::getRequestReplyEventReceiverStrategy() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ITransportStrategy* > return_value;
    NMM::Event event("IOutOfBandCompositeStrategy::getRequestReplyEventReceiverStrategy");
    event.setReturnValue(&return_value);
    if(const_cast<IOutOfBandCompositeStrategy*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ITransportStrategy* IOutOfBandCompositeStrategy::getRequestReplyEventSenderStrategy() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ITransportStrategy* > return_value;
    NMM::Event event("IOutOfBandCompositeStrategy::getRequestReplyEventSenderStrategy");
    event.setReturnValue(&return_value);
    if(const_cast<IOutOfBandCompositeStrategy*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IOutOfBandCompositeStrategy::IOutOfBandCompositeStrategy_type = "IOutOfBandCompositeStrategy";
  NMM::TConstructor<IOutOfBandCompositeStrategy > __IOutOfBandCompositeStrategy_prototype("IOutOfBandCompositeStrategy","NMM");
  TValueConstructor< IOutOfBandCompositeStrategy::TransportRole > __IOutOfBandCompositeStrategy_TransportRole_prototype(TypeInfo("IOutOfBandCompositeStrategy::TransportRole", "NMM"));

  NMM::OValueStream& operator<<(NMM::OValueStream& ov, const IOutOfBandCompositeStrategy::TransportRole& t)
  {
    ov.beginType(TypeInfo("IOutOfBandCompositeStrategy::TransportRole", "NMM"));
    nmm_int32 temp = static_cast<nmm_int32>(t);
    ov << temp;
    ov.endType();
    return ov;
  }

  NMM::IValueStream& operator>>(NMM::IValueStream& iv, IOutOfBandCompositeStrategy::TransportRole& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("IOutOfBandCompositeStrategy::TransportRole", "NMM")) {
      throw NMM::InvalidTypeException(TypeInfo("IOutOfBandCompositeStrategy::TransportRole", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    nmm_int32 temp;
    iv >> temp;
    t = static_cast<IOutOfBandCompositeStrategy::TransportRole>(temp);
    iv.goUp();
    return iv;
  }



  const char* IOutOfBandCompositeStrategy::getOneWayEventRequestReceiverStrategy_event = "IOutOfBandCompositeStrategy::getOneWayEventRequestReceiverStrategy";
  const char* IOutOfBandCompositeStrategy::getOneWayEventReplyReceiverStrategy_event = "IOutOfBandCompositeStrategy::getOneWayEventReplyReceiverStrategy";
  const char* IOutOfBandCompositeStrategy::getOneWayEventRequestSenderStrategy_event = "IOutOfBandCompositeStrategy::getOneWayEventRequestSenderStrategy";
  const char* IOutOfBandCompositeStrategy::getOneWayEventReplySenderStrategy_event = "IOutOfBandCompositeStrategy::getOneWayEventReplySenderStrategy";
  const char* IOutOfBandCompositeStrategy::getRequestReplyEventReceiverStrategy_event = "IOutOfBandCompositeStrategy::getRequestReplyEventReceiverStrategy";
  const char* IOutOfBandCompositeStrategy::getRequestReplyEventSenderStrategy_event = "IOutOfBandCompositeStrategy::getRequestReplyEventSenderStrategy";


}


#include "nmm/nmm_suffix.hpp"

