/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IOutOfBandCompositeStrategyImpl_HPP
#define NMM_IOutOfBandCompositeStrategyImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IOutOfBandCompositeStrategy.hpp"

#include "ICompositeStrategyImpl.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Out-of-band composite transport strategy.
   * This is an interface for the %OutOfBandCompositeStrategy. It provides access to child transport strategies and
   * the request/reply unit. It also defines internal configuration events.
   */

  class DLLEXPORT IOutOfBandCompositeStrategyImpl  : virtual public ICompositeStrategyImpl { 
  public:
    IOutOfBandCompositeStrategyImpl() { addInterface(new IOutOfBandCompositeStrategy(*this)); registerEvents(); };

    virtual ~IOutOfBandCompositeStrategyImpl() {};

  protected:
    /** Returns child strategy for receiving one-way %Event requests.
     * If this method returns 0, then the binding is realized by a request-reply
     * leaf strategy, which is returned by getRequestReplyEventReceiverStrategy().
     *
     * @return Interface to child strategy for receiving one-way %Event requests.
     */

    virtual ITransportStrategy* getOneWayEventRequestReceiverStrategy() const = 0;
    /** Returns child strategy for receiving one-way %Event replies.
     * If this method returns 0, then the binding is realized by a request-reply
     * leaf strategy, which is returned by getRequestReplyEventSenderStrategy().
     *
     * @return Interface to child strategy for receiving one-way %Event replies.
     */

    virtual ITransportStrategy* getOneWayEventReplyReceiverStrategy() const = 0;
    /** Returns child strategy for sending one-way %Event requests.
     * If this method returns 0, then the binding is realized by a request-reply
     * leaf strategy, which is returned by getRequestReplyEventReceiverStrategy().
     *
     * @return Interface to child strategy for sending one-way %Event requests.
     */

    virtual ITransportStrategy* getOneWayEventRequestSenderStrategy() const = 0;
    /** Returns child strategy for sending one-way %Event replies.
     * If this method returns 0, then the binding is realized by a request-reply
     * leaf strategy, which is returned by getRequestReplyEventSenderStrategy().
     *
     * @return Interface to child strategy for sending one-way %Event replies.
     */

    virtual ITransportStrategy* getOneWayEventReplySenderStrategy() const = 0;
    /** Returns child strategy for receiving request-reply Events.
     * If this method returns 0, then the binding is realized by two one-way
     * leaf strategies, which are returned by getOneWayEventRequestReceiverStrategy()
     * and getOneWayEventReplySenderStrategy().
     *
     * @return Interface to child strategy for receiving request-reply Events
     */

    virtual ITransportStrategy* getRequestReplyEventReceiverStrategy() const = 0;
    /** Returns child strategy for sending request-reply Events.
     * If this method returns 0, then the binding is realized by two one-way
     * leaf strategies, which are returned by getOneWayEventRequestSenderStrategy()
     * and getOneWayEventReplyReceiverStrategy().
     *
     * @return Interface to child strategy for sending request-reply Events
     */

    virtual ITransportStrategy* getRequestReplyEventSenderStrategy() const = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
