/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IOutputJack.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/IInputJack.hpp"

#include "nmm/comm/IInstreamCompositeStrategy.hpp"
#include "nmm/comm/InstreamChannel.hpp"

namespace NMM
{
  void IOutputJack::used() {} 
  
  
  InstreamChannel* IOutputJack::connectTo(const IInputJack& jack) 
      throw(const NMM::SerializedException&, const BindingException&)
  {
    NMM::TOutValue<InstreamChannel* > return_value;
    NMM::TInValue<IInputJack > jack_value(jack, false);
    NMM::Event event("IOutputJack::connectTo", &jack_value);
    event.setReturnValue(&return_value);
    if(const_cast<IOutputJack*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(BindingException* ex = dynamic_cast<BindingException*>(event.getException())) {
          throw BindingException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IInputJack* IOutputJack::getDataPartner()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<IInputJack* > return_value;
    NMM::Event event("IOutputJack::getDataPartner");
    event.setReturnValue(&return_value);
    if(const_cast<IOutputJack*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IInstreamCompositeStrategy* IOutputJack::getTransportStrategy() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<IInstreamCompositeStrategy* > return_value;
    NMM::Event event("IOutputJack::getTransportStrategy");
    event.setReturnValue(&return_value);
    if(const_cast<IOutputJack*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IOutputJack::IOutputJack_type = "IOutputJack";
  NMM::TConstructor<IOutputJack > __IOutputJack_prototype("IOutputJack","NMM");


  const char* IOutputJack::connectTo_event = "IOutputJack::connectTo";
  const char* IOutputJack::getDataPartner_event = "IOutputJack::getDataPartner";
  const char* IOutputJack::getTransportStrategy_event = "IOutputJack::getTransportStrategy";


}


#include "nmm/nmm_suffix.hpp"

