/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IQueued.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IQueued::used() {} 
  
  
  unsigned int IQueued::getSize(const string& id /* = "default" */) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<unsigned int > return_value;
    NMM::TInValue<string > id_value(id, false);
    NMM::Event event("IQueued::getSize", &id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IQueued*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 IQueued::getMaxSize(const string& id /* = "default" */) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::TInValue<string > id_value(id, false);
    NMM::Event event("IQueued::getMaxSize", &id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IQueued*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IQueued::setMaxSize(const nmm_uint32& size, const string& id /* = "default" */)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_uint32 > size_value(size, false);
    NMM::TInValue<string > id_value(id, false);
    NMM::Event event("IQueued::setMaxSize", &size_value, &id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IQueued*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IQueued::Mode IQueued::getMode(const string& id /* = "default" */) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<IQueued::Mode > return_value;
    NMM::TInValue<string > id_value(id, false);
    NMM::Event event("IQueued::getMode", &id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IQueued*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IQueued::setMode(const IQueued::Mode& mode, const string& id /* = "default" */)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<IQueued::Mode > mode_value(mode, false);
    NMM::TInValue<string > id_value(id, false);
    NMM::Event event("IQueued::setMode", &mode_value, &id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IQueued*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IQueued::IQueued_type = "IQueued";
  NMM::TConstructor<IQueued > __IQueued_prototype("IQueued","NMM");
  TValueConstructor< IQueued::Mode > __IQueued_Mode_prototype(TypeInfo("IQueued::Mode", "NMM"));

  NMM::OValueStream& operator<<(NMM::OValueStream& ov, const IQueued::Mode& t)
  {
    ov.beginType(TypeInfo("IQueued::Mode", "NMM"));
    nmm_int32 temp = static_cast<nmm_int32>(t);
    ov << temp;
    ov.endType();
    return ov;
  }

  NMM::IValueStream& operator>>(NMM::IValueStream& iv, IQueued::Mode& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("IQueued::Mode", "NMM")) {
      throw NMM::InvalidTypeException(TypeInfo("IQueued::Mode", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    nmm_int32 temp;
    iv >> temp;
    t = static_cast<IQueued::Mode>(temp);
    iv.goUp();
    return iv;
  }



  const char* IQueued::getSize_event = "IQueued::getSize";
  const char* IQueued::getMaxSize_event = "IQueued::getMaxSize";
  const char* IQueued::setMaxSize_event = "IQueued::setMaxSize";
  const char* IQueued::getMode_event = "IQueued::getMode";
  const char* IQueued::setMode_event = "IQueued::setMode";


}


#include "nmm/nmm_suffix.hpp"

