/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ISetValueStream.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ISetValueStream::used() {} 
  
  
  void ISetValueStream::setOValueStreamStack(const OValueStream* value_stream)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<OValueStream* > value_stream_value (const_cast<OValueStream* >(value_stream), false);
    NMM::Event event("ISetValueStream::setOValueStreamStack", &value_stream_value);
    if(const_cast<ISetValueStream*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ISetValueStream::setIValueStreamStack(const IValueStream* value_stream)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<IValueStream* > value_stream_value (const_cast<IValueStream* >(value_stream), false);
    NMM::Event event("ISetValueStream::setIValueStreamStack", &value_stream_value);
    if(const_cast<ISetValueStream*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ISetValueStream::setOValueStream(const string& value_stream_type) 
      throw(const NMM::SerializedException&, const BindingException&)
  {
    NMM::TInValue<string > value_stream_type_value(value_stream_type, false);
    NMM::Event event("ISetValueStream::setOValueStream", &value_stream_type_value);
    if(const_cast<ISetValueStream*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(BindingException* ex = dynamic_cast<BindingException*>(event.getException())) {
          throw BindingException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ISetValueStream::setIValueStream(const string& value_stream_type) 
      throw(const NMM::SerializedException&, const BindingException&)
  {
    NMM::TInValue<string > value_stream_type_value(value_stream_type, false);
    NMM::Event event("ISetValueStream::setIValueStream", &value_stream_type_value);
    if(const_cast<ISetValueStream*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(BindingException* ex = dynamic_cast<BindingException*>(event.getException())) {
          throw BindingException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* ISetValueStream::ISetValueStream_type = "ISetValueStream";
  NMM::TConstructor<ISetValueStream > __ISetValueStream_prototype("ISetValueStream","NMM");


  const char* ISetValueStream::setOValueStreamStack_event = "ISetValueStream::setOValueStreamStack";
  const char* ISetValueStream::setIValueStreamStack_event = "ISetValueStream::setIValueStreamStack";
  const char* ISetValueStream::setOValueStream_event = "ISetValueStream::setOValueStream";
  const char* ISetValueStream::setIValueStream_event = "ISetValueStream::setIValueStream";


}


#include "nmm/nmm_suffix.hpp"

