/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ITransportStrategy.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ITransportStrategy::used() {} 
  
  
  ITransportStrategy* ITransportStrategy::getPartnerStrategy() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ITransportStrategy* > return_value;
    NMM::Event event("ITransportStrategy::getPartnerStrategy");
    event.setReturnValue(&return_value);
    if(const_cast<ITransportStrategy*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ITransportStrategy* ITransportStrategy::getThisStrategy() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ITransportStrategy* > return_value;
    NMM::Event event("ITransportStrategy::getThisStrategy");
    event.setReturnValue(&return_value);
    if(const_cast<ITransportStrategy*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ITransportStrategy::ITransportStrategy_type = "ITransportStrategy";
  NMM::TConstructor<ITransportStrategy > __ITransportStrategy_prototype("ITransportStrategy","NMM");


  const char* ITransportStrategy::getPartnerStrategy_event = "ITransportStrategy::getPartnerStrategy";
  const char* ITransportStrategy::getThisStrategy_event = "ITransportStrategy::getThisStrategy";


}


#include "nmm/nmm_suffix.hpp"

