/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ITransportStrategy_HPP
#define NMM_ITransportStrategy_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{
  /** Interface to request pointer to interface of partner startegy. 
   */

  class DLLEXPORT ITransportStrategy : virtual public NMM::Interface { 
  public:
    ITransportStrategy() : NMM::Interface(ITransportStrategy::ITransportStrategy_type, "NMM") { };

    ITransportStrategy(NMM::NMMObject& parent) : NMM::Interface (parent, ITransportStrategy::ITransportStrategy_type, "NMM") { };

    virtual ~ITransportStrategy() {};

    static const char* ITransportStrategy_type;

    virtual  string getInterfaceType() const { return ITransportStrategy_type; }

    NMM::Interface* copy() const { return new ITransportStrategy(*this); }
    static void used(); 
    static const char* getPartnerStrategy_event;
    static const char* getThisStrategy_event;
    /** Returns an interface to the partner strategy of this %TransportStrategy.
     *
     *  This method returns an interface to the partner strategy of this %TransportStrategy.
     *
     *  @brief Get partner strategy
     *  @return An interface to the partner %TransportStrategy
     */

    virtual ITransportStrategy* getPartnerStrategy() const
      throw(const NMM::SerializedException&);
    /** Returns an interface to the transport strategy itself.
     *
     * This method returns an interface to this %TransportStrategy.
     * It is sent by getPartnerStrategy to get the interface of the partner strategy.
     * 
     * @return An interface to the this %TransportStrategy
     */

    virtual ITransportStrategy* getThisStrategy() const
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ITransportStrategy> ITransportStrategy_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
