/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ITransportStrategyImpl_HPP
#define NMM_ITransportStrategyImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "ITransportStrategy.hpp"


#include "visibility.hpp"

namespace NMM
{
  /** Interface to request pointer to interface of partner startegy. 
   */

  class DLLEXPORT ITransportStrategyImpl  : public virtual NMM::NMMObject { 
  public:
    ITransportStrategyImpl() { addInterface(new ITransportStrategy(*this)); registerEvents(); };

    virtual ~ITransportStrategyImpl() {};

  protected:
    /** Returns an interface to the partner strategy of this %TransportStrategy.
     *
     *  This method returns an interface to the partner strategy of this %TransportStrategy.
     *
     *  @brief Get partner strategy
     *  @return An interface to the partner %TransportStrategy
     */

    virtual ITransportStrategy* getPartnerStrategy() const = 0;
    /** Returns an interface to the transport strategy itself.
     *
     * This method returns an interface to this %TransportStrategy.
     * It is sent by getPartnerStrategy to get the interface of the partner strategy.
     * 
     * @return An interface to the this %TransportStrategy
     */

    virtual ITransportStrategy* getThisStrategy() const = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
