/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 * 
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "InstreamCompositeAddress.hpp"

#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  InstreamCompositeAddress::InstreamCompositeAddress(const NMMReference& ref) throw()
    : CompositeAddress(ref, "InstreamCompositeStrategy", TypeInfo("InstreamCompositeAddress", "NMM")),
      __one_way_buffer_receiver_address(-1),
      __one_way_cevent_receiver_address(-1),
      __one_way_buffer_sender_address(-1),
      __one_way_cevent_sender_address(-1)
  {
    registerAttributes();
  }
  
  InstreamCompositeAddress::InstreamCompositeAddress(const InstreamCompositeAddress& address) throw()
    : CompositeAddress(address),
      __one_way_buffer_receiver_address(address.__one_way_buffer_receiver_address.getRValue()),
      __one_way_cevent_receiver_address(address.__one_way_cevent_receiver_address.getRValue()),
      __one_way_buffer_sender_address(address.__one_way_buffer_sender_address.getRValue()),
      __one_way_cevent_sender_address(address.__one_way_cevent_sender_address.getRValue())
  {
    registerAttributes();
  }
  
  InstreamCompositeAddress* InstreamCompositeAddress::copy() const throw() {
    return new InstreamCompositeAddress(*this);
  }
  
  bool InstreamCompositeAddress::isComplete() const throw() {
    return
      getOneWayBufferReceiverAddress() >= 0 &&
      getOneWayCEventReceiverAddress() >= 0 &&
      getOneWayBufferSenderAddress() >= 0 &&
      getOneWayCEventSenderAddress() >= 0;
  }
  
  void InstreamCompositeAddress::registerAttributes() throw() {
    insertArgument(&__one_way_buffer_receiver_address);
    insertArgument(&__one_way_cevent_receiver_address);
    insertArgument(&__one_way_buffer_sender_address);
    insertArgument(&__one_way_cevent_sender_address);
  }
  
  
  template<>
  class TConstructor<InstreamCompositeAddress> : public ValueConstructor {
  public:
    TConstructor(const TypeInfo& t_info)
      : ValueConstructor(t_info)
    {}
    
    virtual InstreamCompositeAddress*          createValue()          const {return new InstreamCompositeAddress();}
    virtual TValue<InstreamCompositeAddress>*  createTValue()         const {return new TValue<InstreamCompositeAddress>();}

    virtual TValue<TSAddress*>*  createTPointerValue()  const {return new TValue<TSAddress*>();}

    virtual TConstructor<InstreamCompositeAddress>* copy()  const {return new TConstructor(*this); }
  private:
    string  __type_name;
    string  __namespace;
  };
  
  TConstructor<InstreamCompositeAddress>  __InstreamCompositeAddress_ctor(TypeInfo("InstreamCompositeAddress", "NMM"));
  
};

#include "nmm/nmm_suffix.hpp"
