/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_INTERFACE_NOT_FOUND_EXCEPTION_HPP
#define NMM_INTERFACE_NOT_FOUND_EXCEPTION_HPP NMM_INTERFACE_NOT_FOUND_EXCEPTION_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/serialize/SerializedException.hpp"

#include "visibility.hpp"

namespace NMM {
  
  /** Exception thrown whenever an Interface could not be found.
   */
  class DLLEXPORT InterfaceNotFoundException : public SerializedException {
  public:

    /** Default Constructor.
     */
    InterfaceNotFoundException()
      : SerializedException("InterfaceNotFoundException") { setTypeName("InterfaceNotFoundException"); }
    
    /** Constructor with exception comment.
     *
     * @param message exception comment
     */
    InterfaceNotFoundException(const std::string& message)
      : SerializedException("InterfaceNotFoundException : " + message) { setTypeName("InterfaceNotFoundException"); }
    
    /** Constructor with exception comment and pointer to NamedObject which threw the exception.
     *
     * @param n NamedObject which threw the exception
     * @param s exception comment
     */
    InterfaceNotFoundException(const NamedObject* n, const std::string& s)
      : SerializedException(n, ((s == "") ? "InterfaceNotFoundException" : "InterfaceNotFoundException : " + s)) { setTypeName("InterfaceNotFoundException");};
    
    /** Copies this InterfaceNotFoundException.
     *
     * @returns pointer to copy of this InterfaceNotFoundException.
     */
    InterfaceNotFoundException* copy() const { return new InterfaceNotFoundException(*this); }
  };
  
}

#include "nmm/nmm_suffix.hpp"

#endif
