/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 * 
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_LOCAL_ACCEPTOR_HPP
#define NMM_LOCAL_ACCEPTOR_HPP NMM_LOCAL_ACCEPTOR_HPP

#include "nmm/nmm_prefix.hpp"

#include "ObjectAcceptor.hpp"
#include "nmm/utils/Singleton.hpp"

namespace NMM {
  
  /** Local Acceptor.
   *
   * This is a trivial singleton %ObjectAcceptor for the  %LocalStrategy.
   */
  class LocalAcceptor : public ObjectAcceptor {
	DECLARE_SINGLETON(LocalAcceptor, CreateUsingNew, DefaultLifetime, MultiThreaded);
    friend class CreateUsingNew<LocalAcceptor>;
  private:
    /** Default Constructor.
     */
    LocalAcceptor();

    /** Copyconstructor.
     *
     * @param local_acceptor LocalAcceptor to assign values from
     */
    LocalAcceptor(const LocalAcceptor& local_acceptor);

    /** Assignment Operator.
     *
     * @param local_acceptor LocalAcceptor to assign values from
     * @return reference to argument
     */
    const LocalAcceptor& operator=(const LocalAcceptor& local_acceptor);
  };
  
  typedef LocalAcceptor::Singleton SLocalAcceptor;
  
}

#include "nmm/nmm_suffix.hpp"

#endif
