/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 * 
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "LocalAddress.hpp"
#include "ProcessUserInfo.hpp"
#include "NMMDefaultPort.hpp"

#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  LocalAddress::LocalAddress(const NMMReference& reference)
    : TSAddress(reference, "LocalStrategy", TypeInfo("LocalAddress", "NMM"), TSAddress::LOCAL)
  {
    setHostname( SProcessUserInfo::getInstance().getHostname() );
    setPort(NMM::default_port);
  }
  
  
  template<>
  class TConstructor<LocalAddress> : public ValueConstructor {
  public:
    TConstructor(const TypeInfo& t_info)
      : ValueConstructor(t_info)
    {}
    
    virtual LocalAddress*          createValue()     const {return new LocalAddress();}
    virtual TValue<LocalAddress>*  createTValue()    const {return new TValue<LocalAddress>();}
    
    virtual TValue<TSAddress*>* createTPointerValue()  const {return new TValue<TSAddress*>();}
    
    virtual TConstructor<LocalAddress>* copy()  const {return new TConstructor(*this); }
  };
  
  TConstructor<LocalAddress> __local_address(TypeInfo("LocalAddress", "NMM"));
}

#include "nmm/nmm_suffix.hpp"
