/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#ifndef NMM_OBJECTID_HPP
#define NMM_OBJECTID_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/nmm_types.hpp"
#include "nmm/nmm_inttypes.hpp"

#include <list>

#include "visibility.hpp"
namespace NMM {
  class IValueStream;
  class OValueStream;
  
  typedef nmm_int64 ObjectID; ///< Typedef for the ObjectID

  /** Serialization operator for a list of ObjectIDs.
   *
   * @param ov OValueStream to write to
   * @param string_list list of ObjectIDs to write
   * @return reference to used OValueStream
   */
  DLLEXPORT OValueStream& operator<<(OValueStream& ov, const std::list<ObjectID>& string_list);

  /** Deserialization operator for a list of ObjectIDs.
   *
   * @param iv IValueStream to read from
   * @param string_list list of ObjectIDs to read into
   * @return reference to used IValueStream
   */
  DLLEXPORT IValueStream& operator>>(IValueStream& iv, std::list<ObjectID>& string_list);
}
#include "nmm/nmm_suffix.hpp"

#endif

