/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 * 
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "OutOfBandCompositeAddress.hpp"

#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  OutOfBandCompositeAddress::OutOfBandCompositeAddress(const NMMReference& ref) throw()
    : CompositeAddress(ref, "OutOfBandCompositeStrategy", TypeInfo("OutOfBandCompositeAddress", "NMM")),
      __one_way_event_request_receiver_address(-1),
      __one_way_event_reply_receiver_address(-1),
      __one_way_event_request_sender_address(-1),
      __one_way_event_reply_sender_address(-1),
      __request_reply_event_receiver_address(-1),
      __request_reply_event_sender_address(-1)
  {
    registerAttributes();
  }
  
  OutOfBandCompositeAddress::OutOfBandCompositeAddress(const OutOfBandCompositeAddress& address) throw()
    : CompositeAddress(address),
      __one_way_event_request_receiver_address(address.__one_way_event_request_receiver_address.getRValue()),
      __one_way_event_reply_receiver_address(address.__one_way_event_reply_receiver_address.getRValue()),
      __one_way_event_request_sender_address(address.__one_way_event_request_sender_address.getRValue()),
      __one_way_event_reply_sender_address(address.__one_way_event_reply_sender_address.getRValue()),
      __request_reply_event_receiver_address(address.__request_reply_event_receiver_address),
      __request_reply_event_sender_address(address.__request_reply_event_sender_address)
  {
    registerAttributes();
  }
  
  OutOfBandCompositeAddress* OutOfBandCompositeAddress::copy() const throw() {
    return new OutOfBandCompositeAddress(*this);
  }
  
  bool OutOfBandCompositeAddress::isComplete() const throw() {
    return
      (getRequestReplyEventReceiverAddress() >= 0 ||
       (getOneWayEventRequestReceiverAddress() >= 0 &&
        getOneWayEventReplySenderAddress()))
      &&
      (getRequestReplyEventSenderAddress() >= 0 ||
       (getOneWayEventRequestSenderAddress() >= 0 &&
        getOneWayEventReplyReceiverAddress()))
      ;
  }
  
  void OutOfBandCompositeAddress::registerAttributes() throw() {
    insertArgument(&__one_way_event_request_receiver_address);
    insertArgument(&__one_way_event_reply_receiver_address);
    insertArgument(&__one_way_event_request_sender_address);
    insertArgument(&__one_way_event_reply_sender_address);
    insertArgument(&__request_reply_event_receiver_address);
    insertArgument(&__request_reply_event_sender_address);
  }
  
  
  template<>
  class TConstructor<OutOfBandCompositeAddress> : public ValueConstructor {
  public:
    TConstructor(const TypeInfo& t_info)
      : ValueConstructor(t_info)
    {}
    
    virtual OutOfBandCompositeAddress*          createValue()          const {return new OutOfBandCompositeAddress();}
    virtual TValue<OutOfBandCompositeAddress>*  createTValue()         const {return new TValue<OutOfBandCompositeAddress>();}

    virtual TValue<TSAddress*>*  createTPointerValue()  const {return new TValue<TSAddress*>();}

    virtual TConstructor<OutOfBandCompositeAddress>* copy()  const {return new TConstructor(*this); }
  private:
    string  __type_name;
    string  __namespace;
  };
  
  static TConstructor<OutOfBandCompositeAddress>  __OutOfBandCompositeAddress_ctor(TypeInfo("OutOfBandCompositeAddress", "NMM"));
  
};

#include "nmm/nmm_suffix.hpp"
