/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ProcessUserInfo.hpp"

#include "nmm/Exception.hpp"

#include "nmm/utils/directory_utils.hpp"
#include "nmm/utils/NamedObject.hpp"

#include <vector>
#include <string>
using std::vector;
using std::string;

#include <cerrno>

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  ProcessUserInfo::ProcessUserInfo()
    : __username("undefined logname"),
      __hostname(),
      __domainname(),
      __pid(::getpid())
  {
    /* BEGIN> determine hostname */
    {
      char* logname = ::getenv("LOGNAME");
      if(logname!=NULL){
          __username = logname;
      }	
      vector<char> tmp(255, '0');
      memset(&tmp[0], 0x0, 255);
      
      if (gethostname(&tmp[0], 255) == -1) {
        switch(errno) {
        case EINVAL        : throw Exception("length of hostname is  negative  or,  for  sethostname,  length is larger than the maximum allowed size, or length is smaller than the actual size.");
          
        case EPERM         : throw Exception("For sethostname, the caller was not the superuser.");
          
        case EFAULT        : throw Exception("name is an invalid address.");
          
        case ENAMETOOLONG  : throw Exception("length is larger than the maximum allowed size, or length is smaller than the actual size.");
          
        default            : throw Exception("Unkown error");
        }
      }
      __hostname = string(&tmp[0]);
    }
    /* END< */

    /* BEGIN> determine domain name */
    {
      vector<char> tmp(255, '0');
      memset(&tmp[0], 0x0, 255);
      
      if(getdomainname(&tmp[0], 255) != -1) {
        __domainname = string(&tmp[0]);
      }
    }
    /* END< */
    
    /* BEGIN> determine home directory */
    __home_directory = getUserHomeDirectory();
    GLOBAL_DEBUG_STREAM("Home directory is " << __home_directory << endl);
    /* END< */
  }
  
  ProcessUserInfo::~ProcessUserInfo() {}
}

DEFINE_SINGLETON(NMM::ProcessUserInfo);

#include "nmm/nmm_suffix.hpp"
