/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ServiceProxy.hpp"

#include "IObjectAcceptor.hpp"
#include "BindingException.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  ServiceProxy::ServiceProxy() {
    proxy_object = 0;
    object_connector = 0;
  }

  ServiceProxy::~ServiceProxy() {
    if(object_connector) {
      delete object_connector;
    }
  }
  
  void ServiceProxy::connectTo(const TSAddress& address, const string& service)
    throw (const BindingException&)
  {
    proxy_object = new ProxyObject(true);
    proxy_object->connectToObject(address);
    
    if(object_connector != 0) {
      delete object_connector;
      object_connector = 0;
    }

    object_connector = proxy_object->getInterface<IObjectAcceptor>();
    if(object_connector == 0) {
      throw BindingException();
    }
    object_connector->connectToService(service);
  }
  
}

#include "nmm/nmm_suffix.hpp"
