/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#ifndef NMM_SERVICEPROXY_HPP
#define NMM_SERVICEPROXY_HPP NMM_SERVICEPROXY_HPP

#include "nmm/nmm_prefix.hpp"

#include "ProxyObject.hpp"

#include "visibility.hpp"

namespace NMM {
  
  class IObjectAcceptor;

  /** Helper class for connecting to a distributed object.
   */
  
  class DLLEXPORT ServiceProxy {
  protected:
    ProxyObject* proxy_object; ///< for connection to distributed object
    IObjectAcceptor* object_connector; ///< for connection to service of distributed object
  public:
    /** Default constructor.
     */
    ServiceProxy();

    /** Destructor.
     */
    ~ServiceProxy();

    /** Requests interface of type T.
     *
     * @return Interface of type T
     */
    template <typename T> T* getInterface() const { 
      return proxy_object->getInterface<T>();
    }

    /** Requests interface of type T or throws InterfaceNotFoundException if Interface could not be found.
     *
     * @return Interface of type T
     */
    template <typename T> T* getCheckedInterface() const
      throw (const InterfaceNotFoundException&) {
      return proxy_object->getCheckedInterface<T>();
    }
  
    /** Connects to given TSAddress and requests service.
     *
     * @param address TSAddres of distributed object
     * @param service service to request
     */
    void connectTo(const TSAddress& address, const string& service)
      throw (const BindingException&);
  };
  
}

#include "nmm/nmm_suffix.hpp"

#endif
