/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 * 
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "SimpleInstreamDemultiplexer.hpp"
#include "InstreamCompositeStrategy.hpp"

#include "nmm/comm/messaging/Buffer.hpp"
#include "nmm/comm/messaging/CEvent.hpp"
#include "nmm/comm/messaging/ITSOneWayBufferReceiver.hpp"
#include "nmm/comm/messaging/ITSOneWayCEventReceiver.hpp"

#include "nmm/utils/thread/MutexGuard.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  Result SimpleInstreamDemultiplexer::receiveCSMessage(Message* message) {
    /* BEGIN> Dispatch message to correct interface */
    switch(message->getType()) {
    case Message::BUFFER:
      {
        Buffer* buffer = dynamic_cast<Buffer*>(message);
        Result result = getParent()->getOneWayBufferReceiver()->receiveOneWayBuffer(buffer);
        return result;
      }
    case Message::EVENT:
    case Message::FLUSH_EVENT:
      {
        CEvent* cevent = dynamic_cast<CEvent*>(message);
        Result result = getParent()->getOneWayCEventReceiver()->receiveOneWayCEvent(cevent);
        return result;
      }
    default:
      /* DOES NOT HAPPEN */
      return FAILURE;
    }
    /* END< */
  }
  
  void SimpleInstreamDemultiplexer::receiveSignal(Signal signal, ISignalSender* /*sender*/) {
    if(signal == FREE_QUEUE_SLOT || signal == FREE_UPSTR_QUEUE_SLOT) {
      /* if both child strategies are unblocked, unblock composite strategy */
      if(getParent()->getSignalReceiver()) {
        getParent()->getSignalReceiver()->receiveSignal(signal);
      } else {
        GLOBAL_WARNING_STREAM1("InstreamDemultiplexer> No signal receiver -> discarding signal" << endl);
      }
    }
  }

  void SimpleInstreamDemultiplexer::flushITSReceiver() {
    if ( getParent() ) {
      if ( getParent()->getOneWayBufferReceiver() ) {
        getParent()->getOneWayBufferReceiver()->flushITSReceiver();
      }
    }
  }
}

#include "nmm/nmm_suffix.hpp"
