/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 * 
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_SIMPLE_INSTREAM_DEMULTIPLEXER_HPP
#define NMM_SIMPLE_INSTREAM_DEMULTIPLEXER_HPP NMM_SIMPLE_INSTREAM_DEMULTIPLEXER_HPP

#include "nmm/nmm_prefix.hpp"

#include "InstreamDemultiplexer.hpp"

namespace NMM {
  
  /** Simple instream demultiplexer. 
   *
   * This class implements a
   * demultiplexing strategy for simple instream bindings with
   * exactly one sub-binding.
   */
  class SimpleInstreamDemultiplexer
    : public InstreamDemultiplexer
  {
  public:
    /** @name Receiver Interface
     */
    //@{
    /** This method passes a message to a top-level %TransportStrategy and returns the queue status.
     * @param message The message
     * @return The queue status of the %TransportStrategy
     */
    virtual Result receiveCSMessage(Message* message);
    
    /** Receives a signal from a child strategy.
     *
     * @param signal Signal to receive
     * @param sender sender of signal
     */
    virtual void receiveSignal(Signal signal, ISignalSender* sender);
    //@}

    /** This mehtod flushes the used TransportStrategies.
      */
    virtual void flushITSReceiver();      
  };
  
}

#include "nmm/nmm_suffix.hpp"

#endif
