/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 * 
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#ifndef NMM_SIMPLE_INSTREAM_MULTIPLEXER_HPP
#define NMM_SIMPLE_INSTREAM_MULTIPLEXER_HPP NMM_SIMPLE_INSTREAM_MULTIPLEXER_HPP

#include "nmm/nmm_prefix.hpp"

#include "InstreamMultiplexer.hpp"

#include "nmm/utils/thread/ThreadMutex.hpp"

namespace NMM {

  class LatencyQueue;

  /** Simple instream message multiplexer. This instream multiplexer
   * is used for simple bindings with exactly one sub-binding.
   */
  class SimpleInstreamMultiplexer
    : public InstreamMultiplexer
  {
  public:
    /** Default constructor.
     */
    SimpleInstreamMultiplexer() throw();

    /** Virtual destructor.
     */
    virtual ~SimpleInstreamMultiplexer() throw();

    /** @name Receiver Interface
     */
    //@{
    /** Receives %CEvent one-way.
     *
     * This method passes a %CEvent to the %TransportStrategy without waiting for a reply.
     *
     * @param cevent The buffer to be passed to the %TransportStrategy.
     */
    virtual Result receiveOneWayCEvent(CEvent* cevent);

    /** Receives %Buffer one-way.
     *
     * This method passes a %Buffer to the %TransportStrategy without waiting for a reply.
     *
     * @param buffer The buffer to be passed to the %TransportStrategy.
     */
    virtual Result receiveOneWayBuffer(Buffer* buffer);

    /** Receives a signal.
     *
     * @param signal Signal to receive
     */
    virtual void receiveSignal(Signal signal);
    //@}

  private:
    LatencyQueue* __latency_queue; ///< queue for received messages
  };
  
}

#include "nmm/nmm_suffix.hpp"

#endif
