/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "StdNamedObject.hpp"
#include <fstream>

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  StdNamedObject::StdNamedObject(const string& name)
    : NamedObject(name.c_str()),
      __error_stream(4), __warning_stream(4), __debug_stream(4), __message_stream(4)
  {
    //Initialize the stream vector
    for (unsigned int i = 0; i< __error_stream.size(); ++i)
      __error_stream[i] = __debug_stream[i] = __warning_stream[i] = __message_stream[i] = 0;
  }

  StdNamedObject::~StdNamedObject() {
    for (unsigned int i = 0; i < __error_stream.size(); ++i) {
      delete __error_stream[i];
      delete __debug_stream[i];
      delete __warning_stream[i];
      delete __message_stream[i];
    }
  }

  void StdNamedObject::setErrorStream(const string& filename, const nmm_int32& level) {
    if (__error_stream[level])
      delete __error_stream[level];

    __error_stream[level] = new fstream(filename.c_str(), ios::out);
    MessageLevel m_level = static_cast<const MessageLevel>(level);
    NamedObject::setErrorStream(__error_stream[level], m_level);
  }

  void StdNamedObject::setDebugStream(const string& filename, const nmm_int32& level) {
    if (__debug_stream[level])
      delete __debug_stream[level];

    __debug_stream[level] = new fstream(filename.c_str(), ios::out);
    MessageLevel m_level = static_cast<const MessageLevel>(level);
    NamedObject::setDebugStream(__debug_stream[level], m_level);
  }

  void StdNamedObject::setWarningStream(const string& filename, const nmm_int32& level) {
    if (__warning_stream[level])
      delete __warning_stream[level];

    __warning_stream[level] = new fstream(filename.c_str(), ios::out);
    MessageLevel m_level = static_cast<const MessageLevel>(level);
    NamedObject::setWarningStream(__warning_stream[level], m_level);
  }

  void StdNamedObject::setMessageStream(const string& filename, const nmm_int32& level) {
    if (__message_stream[level])
      delete __message_stream[level];

    __message_stream[level] = new fstream(filename.c_str(), ios::out);
    MessageLevel m_level = static_cast<const MessageLevel>(level);
    NamedObject::setMessageStream(__message_stream[level], m_level);
  }

}


#include "nmm/nmm_suffix.hpp"
