/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 * 
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "TSAddress.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  TSAddress::TSAddress(const NMMReference& reference /*= 0*/, const string& stratgey_type /*= ""*/,
                       const TypeInfo& t_info /*= TypeInfo("TSAddress", "NMM")*/, int binding_type /*= 0*/)
    : SerializedValue(t_info),
      __reference(reference),
      __ts_desc(stratgey_type, 0, 0),
      __binding_type(binding_type),
      __host_name(),
      __port(0)
  {
    registerAttributes();
  }
  
  TSAddress::TSAddress(const TSAddress& address)
    : SerializedValue(address),
      __reference(address.__reference),
      __ts_desc(address.__ts_desc),
      __binding_type(address.__binding_type),
      __host_name(address.__host_name),
      __port(address.__port)
  {
    registerAttributes();
  }
  
  TSAddress* TSAddress::copy() const {
    return new TSAddress(*this);
  }
  
  void TSAddress::registerAttributes() {
    insertArgument(&__reference);
    insertArgument(&__ts_desc);
    insertArgument(&__binding_type);
    insertArgument(&__host_name);
    insertArgument(&__port);
  }
  
}

#include "nmm/nmm_suffix.hpp"
