/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_TS_ADDRESS_OWNER_HPP
#define NMM_TS_ADDRESS_OWNER_HPP NMM_TS_ADDRESS_OWNER_HPP

#include "nmm/nmm_prefix.hpp"

#include "visibility.hpp"

namespace NMM {
  
  /** An object which has a local %TSAddress.
   *
   * This interface is implemented by a transport strategy which uses
   * indirect connection setup. It provides a method for getting the
   * local address of the strategy.
   */
  class DLLEXPORT TSAddressOwner {
  public:
    /** Virtual destructor.
     */
    virtual ~TSAddressOwner() {}
    
    /** Returns local address.
     *
     * @return local address
     */
    virtual const TSAddress& getLocalAddress() const throw() = 0;
  };
  
}

#include "nmm/nmm_suffix.hpp"

#endif
