/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 * 
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "TSDescription.hpp"

#include "nmm/comm/serialize/StdValue.hpp"
#include "nmm/comm/serialize/TValueConstructor.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include <iostream>

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  TSDescription::TSDescription()
    : SerializedValue("TSDescription", "NMM"),
      __strategy_type(),
      __interfaces(0),
      __capabilities(0)
  {
    registerAttributes();
  }
  
  TSDescription::TSDescription(const string& strategy_type, unsigned int interfaces, unsigned int capabilities)
    : SerializedValue("TSDescription", "NMM"),
      __strategy_type(strategy_type),
      __interfaces(interfaces),
      __capabilities(capabilities)
  {
    registerAttributes();
  }
  
  TSDescription::TSDescription(const TSDescription& ts_desc)
    : SerializedValue(ts_desc),
      __strategy_type(ts_desc.__strategy_type),
      __interfaces(ts_desc.__interfaces),
      __capabilities(ts_desc.__capabilities)
  {
    registerAttributes();
  }
  
  TSDescription* TSDescription::copy() const {
    return new TSDescription(*this);
  }
  
  unsigned int TSDescription::getPartnerInterfaces(unsigned int interfaces) throw() {
    unsigned int inverse_interfaces = 0;    
    if(interfaces & TSDescription::ITS_ONE_WAY_EVENT_SENDER) {
      inverse_interfaces |= TSDescription::ITS_ONE_WAY_EVENT_RECEIVER;
    }
    if(interfaces & TSDescription::ITS_ONE_WAY_EVENT_RECEIVER) {
      inverse_interfaces |= TSDescription::ITS_ONE_WAY_EVENT_SENDER;
    }
    if(interfaces & TSDescription::ITS_ONE_WAY_BUFFER_SENDER) {
      inverse_interfaces |= TSDescription::ITS_ONE_WAY_BUFFER_RECEIVER;
    }
    if(interfaces & TSDescription::ITS_ONE_WAY_BUFFER_RECEIVER) {
      inverse_interfaces |= TSDescription::ITS_ONE_WAY_BUFFER_SENDER;
    }
    if(interfaces & TSDescription::ITS_ONE_WAY_CEVENT_SENDER) {
      inverse_interfaces |= TSDescription::ITS_ONE_WAY_CEVENT_RECEIVER;
    }
    if(interfaces & TSDescription::ITS_ONE_WAY_CEVENT_RECEIVER) {
      inverse_interfaces |= TSDescription::ITS_ONE_WAY_CEVENT_SENDER;
    }
    if(interfaces & TSDescription::ITS_REQUEST_REPLY_EVENT_SENDER) {
      inverse_interfaces |= TSDescription::ITS_REQUEST_REPLY_EVENT_RECEIVER;
    }
    if(interfaces & TSDescription::ITS_REQUEST_REPLY_EVENT_RECEIVER) {
      inverse_interfaces |= TSDescription::ITS_REQUEST_REPLY_EVENT_SENDER;
    }
    return inverse_interfaces;
  }
  
  TConstructor<TSDescription> __nmm__default_ts_description("TSDescription", "NMM");
  
  
  OValueStream& operator<<(OValueStream& ov, const list<TSDescription>& ts_desc_list) {
    return writeList<TSDescription>(ov, ts_desc_list, "TSDescription");
  }
  
  IValueStream& operator>>(IValueStream& ov, list<TSDescription>& ts_desc_list) {
    return readList<TSDescription>(ov, ts_desc_list, "TSDescription");
  }
  
  void TSDescription::registerAttributes() {
    insertArgument(&__strategy_type);
    insertArgument(&__interfaces);
    insertArgument(&__capabilities);
  }
  
  std::ostream& operator<<(std::ostream& os, const TSDescription& tsd) {
    /* display strategy type */
    os << "(\"" << tsd.getStrategyType() << "\", ";
    
    /* display interfaces */
    if(tsd.getInterfaces() == 0) {
      os << 0;
    } else {
      unsigned int interfaces = tsd.getInterfaces();
      bool first = true;
      while(interfaces != 0) {
        if(!first) {
          os << "|";
        }
        if(interfaces & TSDescription::ITS_REQUEST_REPLY_EVENT_SENDER) {
          os << "ITS_REQUEST_REPLY_EVENT_SENDER";
          interfaces &= ~TSDescription::ITS_REQUEST_REPLY_EVENT_SENDER;
          continue;
        }
        if(interfaces & TSDescription::ITS_REQUEST_REPLY_EVENT_RECEIVER) {
          os << "ITS_REQUEST_REPLY_EVENT_RECEIVER";
          interfaces &= ~TSDescription::ITS_REQUEST_REPLY_EVENT_RECEIVER;
          continue;
        }
        if(interfaces & TSDescription::ITS_ONE_WAY_EVENT_SENDER) {
          os << "ITS_ONE_WAY_EVENT_SENDER";
          interfaces &= ~TSDescription::ITS_ONE_WAY_EVENT_SENDER;
          continue;
        }
        if(interfaces & TSDescription::ITS_ONE_WAY_EVENT_RECEIVER) {
          os << "ITS_ONE_WAY_EVENT_RECEIVER";
          interfaces &= ~TSDescription::ITS_ONE_WAY_EVENT_RECEIVER;
          continue;
        }
        if(interfaces & TSDescription::ITS_ONE_WAY_CEVENT_SENDER) {
          os << "ITS_ONE_WAY_CEVENT_SENDER";
          interfaces &= ~TSDescription::ITS_ONE_WAY_CEVENT_SENDER;
          continue;
        }
        if(interfaces & TSDescription::ITS_ONE_WAY_CEVENT_RECEIVER) {
          os << "ITS_ONE_WAY_CEVENT_RECEIVER";
          interfaces &= ~TSDescription::ITS_ONE_WAY_CEVENT_RECEIVER;
          continue;
        }
        if(interfaces & TSDescription::ITS_ONE_WAY_BUFFER_SENDER) {
          os << "ITS_ONE_WAY_BUFFER_SENDER";
          interfaces &= ~TSDescription::ITS_ONE_WAY_BUFFER_SENDER;
          continue;
        }
        if(interfaces & TSDescription::ITS_ONE_WAY_BUFFER_RECEIVER) {
          os << "ITS_ONE_WAY_BUFFER_RECEIVER";
          interfaces &= ~TSDescription::ITS_ONE_WAY_BUFFER_RECEIVER;
          continue;
        }
        break;
      }
    }
    os << ", ";
    
    /* display capabilities */
    if(tsd.getCapabilities() == 0) {
      os << "0";
    } else {
      unsigned int capabilities = tsd.getCapabilities();
      bool first = true;
      while(capabilities != 0) {
        if(!first) {
          os << " ";
        }
        first = false;
        if(capabilities & TSDescription::TS_RELIABLE) {
          os << "TS_RELIABLE";
          capabilities &= ~TSDescription::TS_RELIABLE;
          continue;
        }
        if(capabilities & TSDescription::TS_RELIABLE_CONNECT) {
          os << "TS_RELIABLE_CONNECT";
          capabilities &= ~TSDescription::TS_RELIABLE_CONNECT;
          continue;
        }
        if(capabilities & TSDescription::TS_PRESERVE_MESSAGE_HEADERS) {
          os << "TS_PRESERVE_MESSAGE_HEADERS";
          capabilities &= ~TSDescription::TS_PRESERVE_MESSAGE_HEADERS;
          continue;
        }
      }
    }
    os << ")";
    
    return os;
  }
  
  TValueConstructor< list<TSDescription> >     list_TSDescription_value("list_TSDescription", "std");
  
}

#include "nmm/nmm_suffix.hpp"
