/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_TS_FACTORY_OBJECT_HPP
#define NMM_TS_FACTORY_OBJECT_HPP NMM_TS_FACTORY_OBJECT_HPP

#include "nmm/nmm_prefix.hpp"

#include "visibility.hpp"

namespace NMM {
  
  class ConnectorAcceptor;
  class TransportStrategy;
  
  /** Transport strategy factory object.
   *
   * This abstract class is used to create TransportStrategies.
   *
   * @see TSFactory TransportStrategy ConnectorAcceptor
   */
  class DLLEXPORT TSFactoryObject {
  public:
    /** @name Constructor & Destructor
     */
    //@{
    /** Default constructor.
     */
    TSFactoryObject();

    /** Constructor with %ConnectorAcceptor.
     *
     * @param connector_acceptor of TSFactoryObject
     */
    TSFactoryObject(ConnectorAcceptor* connector_acceptor);
    
    /** Copy constructor.
     *
     * @param other TSFactorObject to copy
     */
    TSFactoryObject(const TSFactoryObject& other);
    
    /** Destructor.
     */
    virtual ~TSFactoryObject();
    
    /** Virtual copy constructor.
     *
     * @return pointer to copy of this TSFactoryObject
     */
    virtual TSFactoryObject* copy() const = 0;
    //@}
    
    /** @name Factory method
     */
    //@{
    /** This method creates a new TransportStrategy.
     *
     * @return pointer to TransportStrategy.
     */
    virtual TransportStrategy* createInstance() const = 0;
    //@}
    
    /** @name %ConnectorAcceptor
     */
    //@{
    /** This method returns a pointer to a ConnectorAcceptor.
     *
     * @return pointer to ConnectorAcceptor
     */
    ConnectorAcceptor* getConnectorAcceptor() const;
    //@}
    
  private:
    ConnectorAcceptor* __connector_acceptor;             ///< Connector acceptor
  };
  
  inline ConnectorAcceptor* TSFactoryObject::getConnectorAcceptor() const {
    return __connector_acceptor;
  }
  
}

#include "nmm/nmm_suffix.hpp"

#endif
