/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_TS_FORMAT_LIST_HPP
#define NMM_TS_FORMAT_LIST_HPP NMM_TS_FORMAT_LIST_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/format/Format.hpp"

#include <list>

namespace NMM {
  
  /** Format list for transport strategies.
   *
   * Encapsulates a list of formats that should be registered with a
   * transport strategy.
   */
  class TSFormatList {
  public:
    
    /** Default constructor.
     */
    TSFormatList() {}
    /** Returns list of formats.
     *
     * @return list of formats
     */
    inline const std::list<Format>& getFormatList() const { return __formats; }
    
  protected:
    /** Adds Format to list.
     *
     * @param format Format to add
     */
    inline void addFormat(const Format& format) { __formats.push_back(format); }
    
  private:
    std::list<Format> __formats; ///< list of formats
  };
  
}

#include "nmm/nmm_suffix.hpp"

#endif
