/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_T_TS_FACTORY_OBJECT_HPP
#define NMM_T_TS_FACTORY_OBJECT_HPP NMM_T_TS_FACTORY_OBJECT_HPP

#include "nmm/nmm_prefix.hpp"

#include "TSFactoryObject.hpp"

namespace NMM {
  
  /** Template transport strategy factory object.
   *
   * This parametrized implementation of TSFactorObject uses the new
   * operator to allocate a TransportStrategy.
   *
   * @see TSFactory TransportStrategy ConnectorAcceptor
   */
  template<class T>
  class TTSFactoryObject
    : public TSFactoryObject
  {
  public:
    /** @name Constructor & Destructor
     */
    //@{
    /** Default constructor.
     */
    TTSFactoryObject() {}
    
    /** Constructor with a %ConnectorAcceptor.
     *
     * @param connector_acceptor ConnectorAcceptor for the TSFactoryObject
     */
    TTSFactoryObject(ConnectorAcceptor* connector_acceptor) : TSFactoryObject(connector_acceptor) {}
    
    /** Virtual copy constructor.
     *
     * @return pointer to copy of this TTSFactoryObject of type T
     */
    virtual TTSFactoryObject<T>* copy() const { return new TTSFactoryObject<T>(*this); }
    //@}
    
    /** @name Factory methods
     */
    //@{
    /** This method creates a new TransportStrategy using the new operator.
     *
     * @return pointer to new TransportStrategy
     */
    virtual TransportStrategy* createInstance() const {
      return new T();
    }
    //@}
  };
  
}

#include "nmm/nmm_suffix.hpp"

#endif
