/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 * 
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_UNRELIABLE_INSTREAM_MULTIPLEXER_HPP
#define NMM_UNRELIABLE_INSTREAM_MULTIPLEXER_HPP NMM_UNRELIABLE_INSTREAM_MULTIPLEXER_HPP

#include "nmm/nmm_prefix.hpp"

#include "InstreamMultiplexer.hpp"

namespace NMM {
  
  /** Unreliable instream message multiplexer. 
   *
   * This instream multiplexer is used for parallel bindings with at least one unreliable binding.
   */
  class UnreliableInstreamMultiplexer
    : public InstreamMultiplexer
  {
  public:
    /** @name Receiver Interface
     */
    //@{
    /** Receives %CEvent one-way.
     *
     * This method passes a %CEvent to the %TransportStrategy without waiting for a reply.
     * @param cevent The buffer to be passed to the %TransportStrategy.
     * @return SUCCESS if CEvent could be delivered successful, BLOCKED if it has been blocked, FAILURE otherwise
     */
    virtual Result receiveOneWayCEvent(CEvent* cevent);
    
    /** Receives %Buffer one-way.
     *
     * This method passes a %Buffer to the %TransportStrategy without waiting for a reply.
     *
     * @param buffer The buffer to be passed to the %TransportStrategy.
     * @return SUCCESS if CEvent could be delivered successful, BLOCKED if it has been blocked, FAILURE otherwise
     */
    virtual Result receiveOneWayBuffer(Buffer* buffer);
    
    /** Receives a signal.
     *
     * @param signal Signal to be received
     */
    virtual void receiveSignal(Signal signal);
    //@}
  };
  
}

#include "nmm/nmm_suffix.hpp"

#endif
