/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "UserInfo.hpp"
#include "ProcessUserInfo.hpp"

#include "nmm/utils/SecurityCheck.hpp"

#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  ostream& operator<<(ostream& o, const UserInfo& ui){
    o << "Host    : " << ui.getHostname() << endl;
    o << "Domain  : " << ui.getDomainname() << endl;
    o << "User    : " << ui.getUsername() << endl;
    o << "Pid     : " << ui.getPid() << endl;
    return o;
  }
  
  
  UserInfo::UserInfo()
    : SerializedValue("UserInfo", "NMM")
  {
    registerAttributes();

    /* Begin> initialize the attributes */
    __username    = SProcessUserInfo::getInstance().getUsername();
    __passwd      = SSecurityCheck::getInstance().getPasswd();
    __domainname  = SProcessUserInfo::getInstance().getDomainname();
    __hostname    = SProcessUserInfo::getInstance().getHostname();
    __pid         = SProcessUserInfo::getInstance().getPid();
    /* End< */
  }

  UserInfo::UserInfo(const UserInfo& ui)
    : SerializedValue(ui)
  {
    registerAttributes();
    assign(ui);
  }

  void UserInfo::assign(const UserInfo& ui) {
    /* Begin> assign the attributes */
    __username    = ui.getUsername();
    __passwd      = ui.getPasswd();
    __domainname  = ui.getDomainname();
    __hostname    = ui.getHostname();
    __pid         = ui.getPid();
    /* End < */
  }

  const UserInfo& UserInfo::operator=(const UserInfo& ui)  {
    SerializedValue::operator=(ui);
    assign(ui);
    return *this;
  }

  void UserInfo::registerAttributes() {
    /* Begin> register the attributes by the baseclass */
    insertArgument(&__hostname);
    insertArgument(&__domainname);
    insertArgument(&__username);
    insertArgument(&__passwd);
    insertArgument(&__pid);
    /* End< */
  }

  bool UserInfo::operator==(const UserInfo& u_info) const {
    /* Begin> compare all attributes */

    /* compare the hostname */
    if (this->getHostname() != u_info.getHostname())
      return false;

    /* compare the domainname */
    if (this->getDomainname() != u_info.getDomainname())
      return false;

    /* compare the username */
    if (getUsername() != u_info.getUsername())
      return false;

    /* compare process id */
    if (getPid() != u_info.getPid()) {
      return false;
    }
    /* End< */

    return true;
  }
  
  TConstructor<UserInfo>    __nmm__user_info("UserInfo", "NMM");
  
}

#include "nmm/nmm_suffix.hpp"
