/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#include "R_FLOAT.hpp"
#include <iostream>

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  R_FLOAT::R_FLOAT(float lParam, float hParam, float l_weight, float h_weight) 
  {
    m_l_value  = lParam;
    m_h_value  = hParam;
    m_l_weight = l_weight;
    m_h_weight = h_weight;
  }

  R_FLOAT::~R_FLOAT()
  {
  }

  bool R_FLOAT::isValueIncluded(float value)
  {
    if ( (m_l_value <= value) && (m_h_value >= value) )
      return true;
    else
      return false;
  }

  bool R_FLOAT::isRangeIncluded(float lValue, float hValue)
  {
    if ( (m_l_value <= lValue) && (m_h_value >= hValue) )
      return true;
    else
      return false;
  }

  bool R_FLOAT::isRangeMatching(float lValue, float hValue)
  {
    if ( (!this->isValueIncluded(lValue)) && (!this->isValueIncluded(hValue)) )
      return false;
    else
      return true;
  }

  bool R_FLOAT::isRangeIdentical(float lValue, float hValue) const
  {
    if ( (m_l_value ==lValue) && (m_h_value == hValue))
      return true;
    else
      return false;
  }

  float R_FLOAT::getHighValue() const 
  {
    return m_h_value;
  }
  
  float R_FLOAT::getLowValue() const 
  {
    return m_l_value;
  }
  
  float R_FLOAT::getLowWeight() const 
  {
    return m_l_weight;
  }

  float R_FLOAT::getHighWeight() const 
  {
    return m_h_weight;
  }

  void R_FLOAT::setHighValue(float h_param) 
  {
    m_h_value = h_param;
  }
  
  void R_FLOAT::setLowValue(float l_param) 
  {
    m_l_value = l_param;
  }

  void R_FLOAT::setHighWeight(float h_weight) 
  {
    m_h_weight = h_weight;
  }
  
  void R_FLOAT::setLowWeight(float l_weight) 
  {
    m_l_weight = l_weight;
  }

  void R_FLOAT::showValue() 
  {
    GLOBAL_DEBUG_STREAM2( "[ " << m_l_value << " , " << m_h_value << " ]" << endl);
    GLOBAL_DEBUG_STREAM2( "[ " << m_l_weight << " , " << m_h_weight << " ]" << endl);
    GLOBAL_DEBUG_STREAM2( "standardized weight: " << this->getStandardizedWeight() << endl);
  }

  float R_FLOAT::getStandardizedWeight()
  {
    float A_t = ((m_h_value - m_l_value) * (m_h_weight - m_l_weight) * 0.5);
    float A_r = ((m_h_value - m_l_value) * m_l_weight);
    if ((m_h_value - m_l_value) == 0)
      return m_l_weight;
    float result =  ((A_t + A_r)/(m_h_value - m_l_value));
    return result;
  }

  ostream & operator << (ostream & os, const R_FLOAT & reference)
  {
    ((R_FLOAT)reference).showValue();
    return os;
  }

  bool operator == (const R_FLOAT & lhs, const R_FLOAT & rhs)
  {
    return lhs.isRangeIdentical(rhs.getLowValue(), lhs.getHighValue());
  }


}

#include "nmm/nmm_suffix.hpp"
