/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#include "R_FParameter.hpp"
#include "R_FLOAT.hpp"
#include "nmm/comm/serialize/IValueStream.hpp"
#include "nmm/comm/serialize/OValueStream.hpp"


#include "nmm/nmm_prefix.hpp"

namespace NMM 
{
  R_FParameter::R_FParameter(const string & pName) : Parameter(pName) 
  {
    m_parameter_type = Parameter::R_FLOAT;
    m_counter        = m_value_list.begin();
    m_default_range  = 0;
  }

  R_FParameter::R_FParameter(const string & pName, void *pValue) : Parameter(pName) 
  {
    m_parameter_type = Parameter::R_FLOAT;
    m_value_list.push_back((NMM::R_FLOAT *)pValue);
    m_counter       = m_value_list.begin();
    m_default_value = static_cast<NMM::R_FLOAT *>(pValue)->getLowValue();
    m_default_range = m_value_list.front();
  }

  R_FParameter::~R_FParameter()
  {
    for (list<NMM::R_FLOAT *>::const_iterator itr = m_value_list.begin(); itr != m_value_list.end(); itr ++)
      {
	delete *itr;
      }
  }

  void R_FParameter::addValue(void *pValue) 
  {
    if ( m_value_list.empty()) m_default_value = static_cast<NMM::R_FLOAT *>(pValue)->getLowValue();
    // add the new value at the correct position
    R_FLOAT_Iterator temp_itr;
    bool add = true;
    for (R_FLOAT_Iterator itr = m_value_list.begin(); itr != m_value_list.end(); itr++)
      {
	if ( (*itr)->getLowValue() >= ((NMM::R_FLOAT *)pValue)->getLowValue())
	  {
	    if (add)
	      {
		temp_itr = itr;
		add = false;
	      }
	  }
      }
    if (!add)
      m_value_list.insert(temp_itr, (NMM::R_FLOAT *)pValue);
    else
      m_value_list.push_back((NMM::R_FLOAT *)pValue);
    m_counter       = m_value_list.begin();
    
    m_default_range = m_value_list.front();
  }

  void R_FParameter::addValue(NMM::R_FLOAT * value)
  {
    if ( m_value_list.empty()) m_default_value = value->getLowValue();
    // add the new value at the correct position
    R_FLOAT_Iterator temp_itr;
    bool add = true;
    for (R_FLOAT_Iterator itr = m_value_list.begin(); itr != m_value_list.end(); itr++)
      {
	if ( (*itr)->getLowValue() >= value->getLowValue())
	  {
	    if (add)
	      {
		temp_itr = itr;
		add = false;
	      }
	  }
      }
    if (!add)
      m_value_list.insert(temp_itr, value);
    else
      m_value_list.push_back(value);
    m_counter       = m_value_list.begin();

    m_default_range = m_value_list.front();
  }

  NMM::R_FLOAT * R_FParameter::getDefaultRange() const
  {
    return m_default_range;
  }

  bool R_FParameter::isRangeIntersection(float m_lValue, float m_hValue, float lValue, float hValue) const
  {
    if (// [1 [2 2] 1] 
	( (m_lValue <= lValue)
	 && (m_hValue >= hValue))
	// [2 [1 1] 2]
	|| ((m_lValue >= lValue)
	    && (m_hValue <= hValue))
	// [2 [1 2] 1]
	|| ((m_lValue >= lValue)
	    && (m_lValue <= hValue))
	// [2 [1 2] 1]
	|| ((m_hValue >= lValue)
	    && (m_lValue <= hValue)))
      {
	return true;
      }
    return false;
  }

  bool R_FParameter::isValueIncluded(void *pValue) const
  {
    r_float_list::const_iterator i;
    for (i = m_value_list.begin(); i != m_value_list.end(); i++) 
      {
	if (((*i)->getLowValue()<=*((float *)pValue)) && ((*i)->getHighValue()>=*((float *)pValue))) {
	  return true;
	}
      }
    return false;
  }

  bool R_FParameter::isValueIncluded(float value) const
  {
    r_float_list::const_iterator itr;
    for (itr = m_value_list.begin(); itr != m_value_list.end() ; itr ++)
      {
	if ( (*itr)->isValueIncluded(value))
	  return true;
      }
    return false;
  }

  bool R_FParameter::isValueIncluded(void *lParam, void *hParam) const
  {
    r_float_list::const_iterator itr;
    for (itr = m_value_list.begin(); itr != m_value_list.end(); itr ++)
      {
	if (// [1 [2 2] 1] 
	    (((*itr)->getLowValue() <= *((float *)lParam))
	      && ((*itr)->getHighValue() >= *((float *)hParam)))
	    // [2 [1 1] 2]
	     || (((*itr)->getLowValue() >= *((float *)lParam))
		 && ((*itr)->getHighValue() <= *((float *)hParam)))
	    // [2 [1 2] 1]
	     || (((*itr)->getLowValue() >= *((float *)lParam))
		 && ((*itr)->getLowValue() <= *((float *)hParam)))
	    // [2 [1 2] 1]
	     || (((*itr)->getHighValue() >= *((float *)lParam))
		 && ((*itr)->getHighValue() <= *((float *)hParam)))
	     
	     || ( this->isValueIncluded(lParam) && this->isValueIncluded(hParam)))
	  {
	    return true;
	  }
      }
    return false;
  }

  bool R_FParameter::isValueIncluded(float lValue, float hValue) const
  {
    r_float_list::const_iterator itr;
    for (itr = m_value_list.begin(); itr != m_value_list.end(); itr ++)
      {
	if ( (*itr)->isRangeIncluded(lValue,hValue) )
	  return true;
      }
    return false;
  }

  bool R_FParameter::isRangeMatching(float lValue, float hValue) const
  {
    r_float_list::const_iterator itr;
    for (itr = m_value_list.begin(); itr != m_value_list.end(); itr ++)
      {
	if ( (*itr)->isRangeMatching(lValue,hValue))
	  return true;
      }
    return false;
  }

  bool R_FParameter::isRangeIdentical(float lValue, float hValue) const
  {
    for (r_float_list::const_iterator itr = m_value_list.begin(); itr != m_value_list.end(); itr ++)
      {
	if ( (*itr)->isRangeIdentical(lValue,hValue))
	  return true;
      }
    return false;
  }

  void R_FParameter::showParameter() const
  {
    GLOBAL_DEBUG_STREAM2( "Parameter-Name  : " << m_parameter_name << endl);
    GLOBAL_DEBUG_STREAM2( "Parameter-Type  : R_FLOAT" << endl);
    GLOBAL_DEBUG_STREAM2( "Parameter-Weight: " << m_parameter_weight << endl);
    for (r_float_list::const_iterator i = m_value_list.begin(); i != m_value_list.end(); i++) 
      {
	GLOBAL_DEBUG_STREAM2("Value: " << "[ " << (*i)->getLowValue() << " , " << (*i)->getHighValue() << " ]" 
			 << " - Weight: [ " << (*i)->getLowWeight() << " , " << (*i)->getHighWeight() << " ]" 
			 << " Standardized weight :" << (*i)->getStandardizedWeight() << endl);
      }
  }

  ostream& R_FParameter::showParameter(ostream& os) const
  {
    os << m_parameter_name << "[weight:" << m_parameter_weight << "]=r_float(";

    for (r_float_list::const_iterator i = m_value_list.begin(); i != m_value_list.end(); i++) 
      {
        if (i != m_value_list.begin()) {
          os << ", ";
        }
	os << "(" << (*i)->getLowValue() << "-" << (*i)->getHighValue() 
           << ")[low_weight:" << (*i)->getLowWeight() 
           << ", std_weight:" << (*i)->getStandardizedWeight() 
           << ", high_weight:" << (*i)->getHighWeight() << "]";
      }
    os << ")";
    return os;
  }

  ostream& R_FParameter::showParameterVerbose(ostream& os) const
  {
    os << "Parameter-Name  : " << m_parameter_name << endl;
    os << "Parameter-Type  : R_FLOAT" << endl;
    os << "Parameter-Weight: " << m_parameter_weight << endl;
    for (r_float_list::const_iterator i = m_value_list.begin(); i != m_value_list.end(); i++) 
      {
 	os <<"Value: " << "[ " << (*i)->getLowValue() << " , " << (*i)->getHighValue() << " ]" 
           << " - Weight: [ " << (*i)->getLowWeight() << " , " << (*i)->getHighWeight() << " ]" 
           << " Standardized weight :" << (*i)->getStandardizedWeight() << endl;
      }
    return os;
  }
  
  void * R_FParameter::nextValue() const
  {
    NMM::R_FLOAT *temp = NULL;
    if (m_counter != m_value_list.end()) 
      {
	temp = *m_counter;
	m_counter++;
	return temp;
      }
    else 
      {
	m_counter=m_value_list.begin();
	return NULL;
      }
  }

  void * R_FParameter::getFirstValue() const
  {
    m_counter = (++m_value_list.begin());
    return *(m_value_list.begin());
  }

  Parameter * R_FParameter::duplicate() const
  {
    R_FParameter * tempParameter = new R_FParameter(m_parameter_name);
    for (r_float_list::const_iterator itr = m_value_list.begin(); itr != m_value_list.end() ; itr ++)
      {
	NMM::R_FLOAT *temp = new NMM::R_FLOAT();
	temp->setLowValue((*itr)->getLowValue());
	temp->setHighValue((*itr)->getHighValue());
	temp->setLowWeight((*itr)->getLowWeight());
	temp->setHighWeight((*itr)->getHighWeight());
	tempParameter->addValue(temp);
      }
    return (Parameter *)tempParameter;
  }
  
  Parameter * R_FParameter::getIntersection(const Parameter *reference) const
  {
    // okay, this is a quick and dirty hack because someone else forgot
    // to handle this case and I am *not* going to implement it properly!
    // the idea is to use a new R_FParameter instead of the wildcard parameter
    if(reference->getParameterType() == Parameter::WILDCARD) {
      R_FParameter* tmp_reference = new R_FParameter(getParameterName());
      float l = this->getLowestValue(); 
      float h = this->getHighestValue(); 
      NMM::R_FLOAT* f_range = new NMM::R_FLOAT(this->getLowestValue(), 
					       this->getHighestValue(), 
					       this->getWeight((void *) &l),
					       this->getWeight((void *) &h));
      tmp_reference->addValue(f_range);
      Parameter* return_reference = this->getIntersection(tmp_reference);
      delete tmp_reference;
      return return_reference;
    }
    // quick and dirty hack END

    if (reference->getParameterType() == Parameter::FLOAT)
      return reference->getIntersection(this);

    R_FParameter *   result_parameter   = new R_FParameter(m_parameter_name);
    bool             running            = true;
    float            tempfloat;
    bool             increase_itr       = false;
    bool             increase_reference = false;
//     R_FLOAT_Iterator itr                = m_value_list.begin();
    list<NMM::R_FLOAT *>::const_iterator itr = m_value_list.begin();
    
    NMM::R_FLOAT * first_element     = *itr;
    NMM::R_FLOAT * first_element_ref = (NMM::R_FLOAT *)reference->getFirstValue();
    
    if ( (first_element == NULL ) || (first_element_ref == NULL))
      running = false;

    while (running)
      {
	float temp = first_element_ref->getLowValue();
	float temp2= first_element_ref->getHighValue();
	increase_itr = false;
	increase_reference = false;

	if (this->isRangeIntersection(first_element->getLowValue(), first_element->getHighValue(), temp , temp2 ))
	  // there is somewhere a machting between the intervals
	  {
	    NMM::R_FLOAT *new_element = new NMM::R_FLOAT();
	    // [2 [1 1] 2]
	    if ( (first_element_ref->getLowValue() <= first_element->getLowValue()) 
		 && (first_element_ref->getHighValue() >= first_element->getHighValue()) )
	      {
		new_element->setLowValue(first_element->getLowValue());
		new_element->setHighValue(first_element->getHighValue());
		tempfloat = first_element->getLowValue();
		new_element->setLowWeight(((reference)->getWeight(&tempfloat) + first_element->getLowWeight())/2);
		tempfloat = first_element->getHighValue();
		new_element->setHighWeight(((reference)->getWeight(&tempfloat) + first_element->getHighWeight())/2);
		// next interval
		itr ++;
		increase_itr = true;
	      }
	    // [1 [2 2] 1]
	    if ( (first_element->getLowValue() <= first_element_ref->getLowValue()) 
		 && (first_element->getHighValue() >= first_element_ref->getHighValue()) )
	      {
		new_element->setLowValue(first_element_ref->getLowValue());
		new_element->setHighValue(first_element_ref->getHighValue());
		tempfloat = first_element_ref->getLowValue();
		new_element->setLowWeight((this->getWeight(&tempfloat) + first_element_ref->getLowWeight())/2);
		tempfloat = first_element_ref->getHighValue();
		new_element->setHighWeight((this->getWeight(&tempfloat) + first_element_ref->getHighWeight())/2);
		// next interval
		increase_reference = true;
	      }
	    // [1 [2 1] 2]
	    if ( (first_element->getLowValue() <= first_element_ref->getLowValue())
		 && (first_element->getHighValue() <= first_element_ref->getHighValue()) )
	      {
		new_element->setLowValue(first_element_ref->getLowValue());
		new_element->setHighValue(first_element->getHighValue());
		tempfloat = first_element_ref->getLowValue();
		new_element->setLowWeight((this->getWeight(&tempfloat) + first_element_ref->getLowWeight())/2);
		tempfloat = first_element->getHighValue();
		new_element->setHighWeight(((reference)->getWeight(&tempfloat) + first_element->getHighWeight())/2);
		// next interval
		itr ++;
		increase_itr = true;
	      }
	    // [2 [1 2] 1]
	    if ( (first_element->getLowValue() >= first_element_ref->getLowValue())
		 && (first_element->getHighValue() >= first_element_ref->getHighValue()) )
	      {
		new_element->setLowValue(first_element->getLowValue());
		new_element->setHighValue(first_element_ref->getHighValue());
		tempfloat = first_element->getLowValue();
		new_element->setLowWeight(((reference)->getWeight(&tempfloat) + first_element->getLowWeight())/2);
		tempfloat = first_element_ref->getHighValue();
		new_element->setHighWeight((this->getWeight(&tempfloat) + first_element_ref->getHighWeight())/2);
		// next interval
		increase_reference = true;
	      }
	    if (increase_reference)
	      first_element_ref = (NMM::R_FLOAT *)reference->nextValue();
	    if (increase_itr)
	      first_element = *itr;
	    result_parameter->addValue(new_element);
	  }
	else
	  // there is no matching between the intervals
	  {
	    // [2 2] [1 1]
	    if (first_element->getLowValue() >= first_element_ref->getHighValue())
	      first_element_ref = (NMM::R_FLOAT *)reference->nextValue();
	    // [1 1] [2 2]
	    else 
	      {
		itr ++;
		first_element = *itr;
	      }
	  }
	if ( (itr == m_value_list.end() ) || (first_element_ref == NULL) )
	  running = false;
      }
    return result_parameter;
  }

  float R_FParameter::getWeight(void * weight) const
  {
    float value = *(float *)weight;
    for (r_float_list::const_iterator itr = m_value_list.begin(); itr != m_value_list.end(); itr ++)
      {
	if ( ((*itr)->getLowValue() <= *(float *)weight)
	     && ((*itr)->getHighValue() >= *(float *)weight))
	{
	  if ( (*itr)->getLowWeight() <= (*itr)->getHighValue() )
	    // for monotonously increasing interval
	    return ( (*itr)->getLowWeight()+( (value-(*itr)->getLowValue()) * this->computeFactor(*itr)) );
	  else
	    // for monotonously deacreasing interval
	    return ( (*itr)->getLowWeight() - ( (value-(*itr)->getLowValue()) * this->computeFactor(*itr)) );
	}
      }
    return 0;
  }

  float R_FParameter::computeFactor(NMM::R_FLOAT * param) const
  {
    if ( ((param->getHighWeight()) - param->getLowWeight()) == 0 ) return 0;
    return  ( (param->getHighWeight() - param->getLowWeight()) / ( param->getHighValue() - param->getLowValue() ) );
  }

  uint R_FParameter::getNumberOfValues() const
  {
    return m_value_list.size();
  }
  
  bool R_FParameter::isParameterIdentical(const Parameter *reference) const
  {
    if ( (reference->getParameterType() != Parameter::R_FLOAT))
      return false;
    if (m_value_list.size() != reference->getNumberOfValues() )
      return false;

    for (r_float_list::const_iterator itr = m_value_list.begin(); itr != m_value_list.end(); itr++)
      {
	if (!dynamic_cast<R_FParameter *>(const_cast<Parameter *>(reference))->isRangeIdentical((*itr)->getLowValue(), 
												(*itr)->getHighValue()))
	  return false;
      }
    return true;
  }

  bool R_FParameter::isParameterMatching(const Parameter * reference) const
  {
    if (reference->getParameterType() == FLOAT)
      return reference->isParameterMatching(this);

    for (r_float_list::const_iterator itr = m_value_list.begin(); itr != m_value_list.end(); itr ++)
      {
	if ( reference->isRangeMatching( (*itr)->getLowValue(), (*itr)->getHighValue() ))
	  return true;
      }
    return false;
  }

  float R_FParameter::getStandardizedWeight() const
  {
    float local_weight = 0;
    for (r_float_list::const_iterator itr = m_value_list.begin(); itr != m_value_list.end(); itr ++)
      {
	local_weight +=(*itr)->getStandardizedWeight();
      }
    return ((local_weight/m_value_list.size()) * m_parameter_weight);
  }

  float R_FParameter::getDefaultValue() const
  {
    return m_default_value;
  }

  void R_FParameter::setDefaultValue(float value)
  {
    m_default_value = value;
  }

  float R_FParameter::getLowestValue() const
  {
    float tempfloat = (*m_value_list.begin())->getLowValue();
    for (r_float_list::const_iterator itr = m_value_list.begin(); itr != m_value_list.end(); itr ++)
      {
	if ( (*itr)->getLowValue() < tempfloat)
	  tempfloat = (*itr)->getLowValue();
      }
    return tempfloat;
  }

  float R_FParameter::getHighestValue() const
  {
    float tempfloat = (*m_value_list.begin())->getHighValue();
    for (r_float_list::const_iterator itr = m_value_list.begin(); itr != m_value_list.end(); itr ++)
      {
	if ( (*itr)->getHighValue() > tempfloat)
	  tempfloat = (*itr)->getHighValue();
      }
    return tempfloat;
  }

  const r_float_list & R_FParameter::getRFloatValueList() const
  {
    return m_value_list;
  }

  void R_FParameter::fromStream(IValueStream& iv) {
    if (iv.getCurrentType() != "R_FParameter" || iv.getCurrentNamespace() != "NMM") {
      GLOBAL_ERROR_STREAM ("Invalid type or namespace:" << endl);
      GLOBAL_ERROR_STREAM ("Current type : " << iv.getCurrentType() << "                Expected type: R_FParameter" << endl);
      GLOBAL_ERROR_STREAM ("Current namespace : " << iv.getCurrentNamespace() << "      Expected namespace: NMM " << endl);

      iv.nextElement();
      return;
    }
    if (!iv.goDown()) {
      GLOBAL_ERROR_STREAM ("R_FParameter is empty" << endl);
      iv.nextElement();
      return;
    }

    string par_name;
    nmm_float weight = 0.0;
    iv >> par_name;
    iv >> weight;

    setParameterName(par_name);
    setParameterWeight(weight);

    nmm_float v1, v2;
    nmm_float w1, w2;
    while(!iv.isEmpty()) {
      iv >> v1;
      iv >> w1;
      iv >> v2;
      iv >> w2;
      NMM::R_FLOAT * rf = new NMM::R_FLOAT(v1, w1, v2, w2);
      addValue( rf );
    }
    iv.goUp();
  }

  void R_FParameter::toStream(OValueStream& ov) const  {
    ov.beginType("R_FParameter", "NMM");

    string p_name = getParameterName();
    nmm_float weight = getParameterWeight();

    ov << p_name;
    ov << weight;

    for (r_float_list::const_iterator it = m_value_list.begin(); it != m_value_list.end(); it++)
    {
      nmm_float w1, w2;
      nmm_float v1, v2;
      v1 = (*it)->getLowValue();
      v2 = (*it)->getHighValue();
      w1 = (*it)->getLowWeight();
      w2 = (*it)->getHighWeight();

      ov << v1;
      ov << w1;
      ov << v2;
      ov << w2;
    }

    ov.endType();
  }

}


#include "nmm/nmm_suffix.hpp"
