/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */



#include <iostream>
#include "R_INT.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  R_INT::R_INT(int32_t lParam, int32_t hParam, float lWeightParam, float hWeightParam) 
  {
    m_l_value  = lParam;
    m_h_value  = hParam;
    m_l_weight = lWeightParam;
    m_h_weight = hWeightParam;
  }

  R_INT::~R_INT()
  {
  }
  
  bool R_INT::isValueIncluded(int32_t value)
  {
    return ( (m_l_value <= value) && (m_h_value >= value));
  }

  bool R_INT::isRangeIncluded(int32_t lValue, int32_t hValue)
  {
    return ( (this->isValueIncluded(lValue)) && (this->isValueIncluded(hValue)) );
  }

  bool R_INT::isRangeMatching(int32_t lValue, int32_t hValue)
  {
    return ( (this->isValueIncluded(lValue)) || (this->isValueIncluded(hValue)) );
  }

  bool R_INT::isRangeIdentical(int32_t lValue, int32_t hValue) const
  {
    if ( (m_l_value == lValue) && (m_h_value == hValue))
      return true;
    else
      return false;
  }

  int32_t R_INT::getLowValue() const 
  {
    return m_l_value;
  }

  int32_t R_INT::getHighValue() const 
  {
    return m_h_value;
  }
  
  float R_INT::getLowWeight() const 
  {
    return m_l_weight;
  }
  
  float R_INT::getHighWeight() const 
  {
    return m_h_weight;
  }

  void R_INT::setLowValue(int32_t lParam) 
  {
    m_l_value = lParam;
  }
  
  void R_INT::setHighValue(int32_t hParam) 
  {
    m_h_value = hParam;
  }

  void R_INT::setLowWeight(float lParam) 
  {
    m_l_weight = lParam;
  }

  void R_INT::setHighWeight(float hParam) 
  {
    m_h_weight = hParam;
  }
  
  void R_INT::showValue() 
  {
    GLOBAL_DEBUG_STREAM2( "[ " << m_l_value << " , " << m_h_value << " ]" << endl);
    GLOBAL_DEBUG_STREAM2( "[ " << m_l_weight << " , " << m_h_weight << " ]" << endl);
    GLOBAL_DEBUG_STREAM2( "standardized weight: " << this->getStandardizedWeight() << endl);
  }

  float R_INT::getStandardizedWeight()
  {
    float A_t = ((m_h_value - m_l_value) * (m_h_weight - m_l_weight) * 0.5);
    float A_r = ((m_h_value - m_l_value) * m_l_weight);
    if ((m_h_value - m_l_value) == 0)
      return m_l_weight;
    float result =  ((A_t + A_r)/(m_h_value - m_l_value));
    return result;
  }

  ostream & operator << (ostream & os, const R_INT & reference)
  {
    ((R_INT)reference).showValue();
    return os;
  }

  bool operator == (const R_INT & lhs, const R_INT & rhs)
  {
    return lhs.isRangeIdentical(rhs.getLowValue(), rhs.getHighValue());
  }
}

#include "nmm/nmm_suffix.hpp"
