/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#include "SParameter.hpp"
#include "nmm/comm/serialize/IValueStream.hpp"
#include "nmm/comm/serialize/OValueStream.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  SParameter::SParameter(const string & pName) : Parameter(pName) 
  {
    m_parameter_type = Parameter::STRING;
    m_value_counter  = m_value_list.begin();
    m_pair_counter   = m_value_list.begin();
  }

  SParameter::SParameter(const string & pName, string pValue, float pWeight) : Parameter(pName)
  {
    m_value_list.insert(make_pair(pValue,pWeight));
    m_parameter_type = NMM::Parameter::STRING;
    m_value_counter  = m_value_list.begin();
    m_pair_counter   = m_value_list.begin();
    m_default_value  = pValue;
  }

  void SParameter::addValue(void *pValue)
  {
    this->addValue(pValue,DEFAULT_WEIGHT);
  }

  void SParameter::addValue(void *pValue, float pWeight)
  {
    if ( m_value_list.empty()) m_default_value = *(static_cast<string *>(pValue));
    m_value_list.insert(make_pair( *((string *)pValue), pWeight));
    m_value_counter  = m_value_list.begin();
    m_pair_counter   = m_value_list.begin();
  }

  void SParameter::addValue(const string & value, float weight)
  {
    if (m_value_list.empty()) m_default_value = value;
    m_value_list.insert(make_pair(value,weight));
    m_value_counter = m_value_list.begin();
    m_pair_counter  = m_value_list.begin();
  }

  void SParameter::showParameter() const
  {
    GLOBAL_DEBUG_STREAM2( "Parameter-Name : " << m_parameter_name << endl);
    GLOBAL_DEBUG_STREAM2( "Parameter-Type : STRING" << endl);
    GLOBAL_DEBUG_STREAM2( "Parameter-Weight: " << m_parameter_weight << endl);
    for (map<string,float>::const_iterator i = m_value_list.begin(); i != m_value_list.end(); i++) {
      GLOBAL_DEBUG_STREAM2( "Value :" << (*i).first << " - Weight: " << (*i).second << endl);
    }
  }

  ostream& SParameter::showParameter(ostream& os) const
  {
    os << m_parameter_name << "[weight:" << m_parameter_weight << "]=string(";

    for (map<string,float>::const_iterator i = m_value_list.begin(); i != m_value_list.end(); i++) {
        if (i != m_value_list.begin()) {
          os << ", ";
        }
	os << (*i).first << "[weight:" << (*i).second << "]";
    }
    return os;
  }

  ostream& SParameter::showParameterVerbose(ostream& os) const
  {
    os << "Parameter-Name : " << m_parameter_name << endl;
    os << "Parameter-Type : STRING" << endl;
    os << "Parameter-Weight: " << m_parameter_weight << endl;
    for (map<string,float>::const_iterator i = m_value_list.begin(); i != m_value_list.end(); i++) {
      os << "Value :" << (*i).first << " - Weight: " << (*i).second << endl;
    }
    return os;
  }

  bool SParameter::isValueIncluded(void *pValue) const
  {
    string* temp         = (string *)pValue;
    map<string,float>::const_iterator i;
    for (i = m_value_list.begin(); i != m_value_list.end(); i++) {
      if ( (*i).first == (*temp) ) return true;
    }
    return false;
  }

  bool SParameter::isValueIncluded(const string & value) const
  {
    map<string, float>::const_iterator temp = m_value_list.find(value);
    if (temp == m_value_list.end())
      return false;
    else
      return true;
  }

  void * SParameter::nextValue() const
  {
//     STRING_Iterator tempitr;
    map <string, float>::const_iterator tempitr;
    if (m_value_counter != m_value_list.end() )
      {
	tempitr = m_value_counter;
	m_value_counter ++;
	return static_cast<void*>(const_cast<string*>(&((*tempitr).first)));
      }
    else
      {
	m_value_counter = m_value_list.begin();
	return NULL;
      }
  }
  
  map<string, float>::const_iterator SParameter::nextPair() const
  {
//     STRING_Iterator tempitr;
    map <string, float>::const_iterator tempitr;
    if (m_pair_counter != m_value_list.end() )
      {
	tempitr = m_pair_counter;
	m_pair_counter ++;
	return tempitr;
      }
    else 
      {
	m_pair_counter = m_value_list.begin();
	return m_value_list.end();
      }

  }
  
  Parameter * SParameter::duplicate() const
  {
    Parameter *tempParameter = new SParameter(m_parameter_name);
    string tempstring;
    for (map<string, float>::const_iterator itr = m_value_list.begin() ; itr != m_value_list.end(); itr ++)
      {
	tempstring = (*itr).first;
	tempParameter->addValue( & tempstring, (*itr).second );
      }
    return tempParameter;
  }

  float SParameter::getWeight(void *pValue) const
  {
    map<string,float>::const_iterator itr = m_value_list.find(*(string *)pValue);
    return (*itr).second;
  }

  uint SParameter::getNumberOfValues() const
  {
    return m_value_list.size();
  }

  void * SParameter::getFirstValue() const
  {
    m_value_counter    = (++m_value_list.begin());
    return (string*)&((*m_value_list.begin()).first);
  }

  Parameter *SParameter::getIntersection(const Parameter *reference) const
  {
    SParameter *new_parameter = new SParameter(m_parameter_name);
    string temp_string;
    for (map<string, float>::const_iterator itr = m_value_list.begin(); itr != m_value_list.end(); itr ++)
      {
	temp_string = (*itr).first;
	if (reference->isValueIncluded(&temp_string))
	  {
	    new_parameter->addValue(&temp_string, ((*itr).second + reference->getWeight(&temp_string))/2);
	  }
      }
    return new_parameter;
  }

  bool SParameter::isParameterIdentical(const Parameter * reference) const
  {
    if (reference->getParameterType() != Parameter::STRING)
      return false;
    if (m_value_list.size() != reference->getNumberOfValues())
      return false;
    string tempstring;
    for (map<string, float>::const_iterator itr = m_value_list.begin(); itr != m_value_list.end(); itr ++)
      {
	tempstring = (*itr).first;
	if (!reference->isValueIncluded(&tempstring))
	  return false;
      }
    return true;
  }

  bool SParameter::isParameterMatching(const Parameter * reference) const
  {
    for (map<string, float>::const_iterator itr = m_value_list.begin(); itr != m_value_list.end(); itr ++)
      {
	if ( reference->isValueIncluded((*itr).first))
	  return true;
      }
    return false;
  }

  float SParameter::getStandardizedWeight() const
  {
    float local_weight = 0;
    for (map<string,float>::const_iterator itr = m_value_list.begin(); itr != m_value_list.end(); itr ++)
      {
	local_weight += (*itr).second;
      }
    return ((local_weight/m_value_list.size()) * m_parameter_weight);
  }

  void SParameter::setDefaultValue(const string & value)
  {
    m_default_value = value;
  }

  const string & SParameter::getDefaultValue() const
  {
    return m_default_value;
  }

  const string_list SParameter::getStringValueList() const
  {
    string_list result;
    for (map<string,float>::const_iterator itr = m_value_list.begin(); itr != m_value_list.end(); itr ++)
      {
	result.push_back((*itr).first);
      }
    return result;
  }


  void SParameter::fromStream(IValueStream& iv) {
    if (iv.getCurrentType() !="SParameter" || iv.getCurrentNamespace() != "NMM") {
      GLOBAL_ERROR_STREAM ("Invalid type or namespace:" << endl);
      GLOBAL_ERROR_STREAM ("Current type : " << iv.getCurrentType() << "                Expected type: Format" << endl);
      GLOBAL_ERROR_STREAM ("Current namespace : " << iv.getCurrentNamespace() << "      Expected namespace: NMM " << endl);

      iv.nextElement();
      return;
    }

    if (!iv.goDown() ){
      GLOBAL_ERROR_STREAM ("SParameter is empty" << endl);

      iv.nextElement();
      return;
    }

    string tmp_str;
    nmm_float weight = 0.0;

    iv >> tmp_str;
    iv >> weight;

    setParameterName(tmp_str);
    setParameterWeight(weight);

    float f;
    string s;

    while (!iv.isEmpty())
    {
      iv >> s;
      iv >> f;
      addValue(s, f);
    }
    iv.goUp();
  }

  void SParameter::toStream(OValueStream& ov )  const {
    ov.beginType("SParameter", "NMM");

    string p_name = getParameterName();
    nmm_float weight = getParameterWeight();

    ov << p_name;
    ov << weight;

    for (map<string, float>::const_iterator it = m_value_list.begin(); it != m_value_list.end(); it++)
    {
      ov << (*it).first;
      ov << (*it).second;
    }

    ov.endType();
  }

}

#include "nmm/nmm_suffix.hpp"
