/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 * 
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_FORMAT_NOT_SUPPORTED_EXCEPTION_HPP
#define NMM_FORMAT_NOT_SUPPORTED_EXCEPTION_HPP NMM_FORMAT_NOT_SUPPORTED_EXCEPTION_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/serialize/SerializedException.hpp"

#include "visibility.hpp"

namespace NMM {
  
  /** The format is not supported.
   *
   * This exception is thrown by the configure method of a leaf
   * transport strategy if the given format is not supported.
   *
   * @see ITSOneWatBufferReceiver TransportStrategy
   */
  class DLLEXPORT FormatNotSupportedException : public SerializedException {
  public:
    /** Default constructor.
     */
    FormatNotSupportedException() 
      : SerializedException("FormatNotSupportedExcpetion")
    {
      setTypeName("FormatNotSupportedException");
    };
    
    /** Constructor, which allows to specify an extra comment.
     *
     * @param s extra comment
     */
    FormatNotSupportedException(const std::string& s)
      : SerializedException("FormatNotSupportedException : " + s)
    {
      setTypeName("FormatNotSupportedException");
    };
    
    /** Constructor, which allows to specify the NamedObject which
     * threw the exception and an extra comment.
     *
     * @param n NamedObject which threw the exception 
     * @param s extra comment
     */
    FormatNotSupportedException(const NamedObject* n, const std::string& s = std::string())
      : SerializedException(n, ((s == "") ? "FormatNotSupportedException" : "FormatNotSupportedException : " + s))
    {
      setTypeName("FormatNotSupportedException");
    };
    
    /** Copy constructor for this Exception.
     *
     * @return Pointer to copy of this exception
     */
    FormatNotSupportedException* copy() const {
      return new FormatNotSupportedException(*this);
    }
  };
  
}

#include "nmm/nmm_suffix.hpp"

#endif
