/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 * 
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_I_CS_EVENT_RECEIVER_HPP
#define NMM_I_CS_EVENT_RECEIVER_HPP NMM_I_CS_EVENT_RECEIVER_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/Result.hpp"

#include "visibility.hpp"

namespace NMM {  
  
  class Event;
  
  /** A interface for top-level %TransportStrategy which receives Events.
   */
  class DLLEXPORT ICSEventReceiver {
  public:
    /** Virtual destructor.
     */
    virtual ~ICSEventReceiver();
    
    /** Receive an event and demultiplex it.
     *
     * This method passes an event to a top-level %TransportStrategy
     * and returns the event result.
     *
     * @param event The event
     * @return The result of the event
     */
    virtual Result receiveCSEvent(Event& event) = 0;
  };
  
}

#include "nmm/nmm_suffix.hpp"

#endif
