/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_I_CS_SIGNAL_RECEIVER_HPP
#define NMM_I_CS_SIGNAL_RECEIVER_HPP NMM_I_CS_SIGNAL_RECEIVER_HPP

#include "nmm/nmm_prefix.hpp"

#include "Signal.hpp"

#include "visibility.hpp"

namespace NMM {
  
  class ISignalSender;
  
  /** Composite signal receiver interface.
   *
   * This signal receiver interface is used by the %InstreamCompositeStrategy.
   * It accepts an additional argument which identifies the origin of
   * the signal.
   *
   * @see Signal ISignalReceiver ISignalSender
   */
  class DLLEXPORT ICSSignalReceiver {
  public:
    /** Virtual destructor.
     */
    virtual ~ICSSignalReceiver();
    
    /** Receive a signal from a specific origin.
     *
     * @param signal Signal ID
     * @param sender Signal origin
     */
    virtual void receiveSignal(Signal signal, ISignalSender* sender) = 0;
  };
  
}

#include "nmm/nmm_suffix.hpp"

#endif
