/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_I_EVENT_RECEIVER_HPP
#define NMM_I_EVENT_RECEIVER_HPP NMM_I_EVENT_RECEIVER_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/Result.hpp"

#include "visibility.hpp"

namespace NMM {  
  
  class Event;
  class IEventSender;
  
  /** An interface for object which receives Events.
   *
   * This interface is implemented by all high-level objects which can receive NMM Events.
   *
   * @see IEventSender Event
   */
  class DLLEXPORT IEventReceiver {
  public:
    /** Virtual destructor.
     */
    virtual ~IEventReceiver();
    
    /** Receive NMM %Event.
     *
     * This method is called to send an %Event to this object.
     *
     * @param event The event to send to this object
     * @param sender The object which sent the event.
     * @return @c SUCCESS if the event was successfully handled, or @c FAILURE otherwise
     */
    virtual Result receiveEvent(Event& event, const IEventSender* sender = 0) = 0;
  };
  
}

#include "nmm/nmm_suffix.hpp"

#endif
