/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_I_EVENT_SENDER_HPP
#define NMM_I_EVENT_SENDER_HPP NMM_I_EVENT_SENDER_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/Result.hpp"

#include "visibility.hpp"

namespace NMM {
  
  class IEventReceiver;
  
  /** An object which sends Events.
   *
   * This interface is implemented by all objects which send NMM Events to high-level objects.
   *
   * @see IEventReceiver
   */
  class DLLEXPORT IEventSender {
  public:

    /** Constructor.
     */
    IEventSender() throw();
    
    /** Virtual destructor.
     */
    virtual ~IEventSender();
    
    /** This method sets the event receiver.
     *
     * @param receiver The event reveiver of this event sender.
     */
    void setEventReceiver(IEventReceiver* receiver);
    
    /** This method returns the event receiver.
     *
     * @return The event reveiver of this event sender.
     */
    IEventReceiver* getEventReceiver() const;
    
  private:
    IEventReceiver* __receiver; ///< The event reveiver of this event sender.
  };
  
  
  /** Set event receiver.
     *
     * @param receiver pointer to IEventReceiver
     */
    inline void IEventSender::setEventReceiver(IEventReceiver* receiver) {
    __receiver = receiver;
  }
  
   /** Get event receiver.
     *
     * @return pointer to IEventReceiver
     */
  inline IEventReceiver* IEventSender::getEventReceiver() const {
    return __receiver;
  }
  
}

#include "nmm/nmm_suffix.hpp"

#endif
