/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IExternalBufferManager.hpp"
#include "BufferManager.hpp"
#include "B_Allocator.hpp"
#include "Buffer.hpp"

#include "nmm/utils/NamedObject.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  IExternalBufferManager::IExternalBufferManager() {
    buffer_manager = 0;
  }

  IExternalBufferManager::~IExternalBufferManager() {
    if (buffer_manager) buffer_manager->release();
  }

  Buffer* IExternalBufferManager::getNewBuffer(const size_t size) {
    //DEBUG_STREAM2("getNewBuffer with size " << size << endl);
    if (buffer_manager == 0) {
      NamedObject *obj_ptr = dynamic_cast<NamedObject *>(this);
      if(obj_ptr) {
	string allocator_name = obj_ptr->getName() + " B_Allocator";
#ifdef ARCH_ARM
	buffer_manager = new B_Allocator(size, 1, 4, allocator_name.c_str()); 
#else
	buffer_manager = new B_Allocator(size, 4, 10, allocator_name.c_str()); 
#endif
      }
      else {
#ifdef ARCH_ARM
	buffer_manager = new B_Allocator(size, 1, 4); 
#else
	buffer_manager = new B_Allocator(size, 4, 10); 
#endif
      }
    }

    // Request buffer
    Buffer* new_buffer = buffer_manager->requestBuffer(size);

    // reset the delay start time
    new_buffer->unsetDelayStartTime();

    return new_buffer;
  }

  Buffer* IExternalBufferManager::getNewBuffer(const size_t size, 
                                               const Buffer *buffer) {
    Buffer *new_buffer = getNewBuffer(size);
    if (new_buffer != NULL) {                    // copy the header
      // only copy timestamp if it was set before (otherwise, it should be all zero anyway)
      if(buffer->getTimestamp().is_set) {
	new_buffer->setTimestamp(buffer->getTimestamp());
      }

      // copy the delay start time
      if(buffer->isDelayStartTimeSet())
      {
        new_buffer->setDelayStartTime(buffer->getDelayStartTime());
      }
    }
    return new_buffer;
  }

  void IExternalBufferManager::setBufferManager(BufferManager *manager) {
    if (buffer_manager)
      buffer_manager->release();
    buffer_manager = manager;
  }

  const BufferManager* IExternalBufferManager::getBufferManager() const {
    return buffer_manager;
  }

  BufferManager* IExternalBufferManager::requestBufferManager()
  {
    if (buffer_manager == 0) {
      NamedObject *obj_ptr = dynamic_cast<NamedObject *>(this);
      if(obj_ptr) {
	string allocator_name = obj_ptr->getName() + " B_Allocator";
#ifdef ARCH_ARM
	buffer_manager = new B_Allocator(0, 0, 4, allocator_name.c_str()); 
#else
	buffer_manager = new B_Allocator(0, 0, 10, allocator_name.c_str()); 
#endif
      }
      else {
#ifdef ARCH_ARM
	buffer_manager = new B_Allocator(0, 0, 4); 
#else
	buffer_manager = new B_Allocator(0, 0, 10); 
#endif
      }
    }
    
    return buffer_manager->request();
  }

}

#include "nmm/nmm_suffix.hpp"
