/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_I_MESSAGE_RECEIVER_HPP
#define NMM_I_MESSAGE_RECEIVER_HPP NMM_I_MESSAGE_RECEIVER_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/Result.hpp"

#include "visibility.hpp"

namespace NMM {  
  
  class Message;  
  class IMessageSender;

  /** An interface for objects which receive Messages.
   *
   * This interface is implemented by all high-level objects which can receive NMM Messages.
   *
   * @see IMessageSender Message
   */
  class DLLEXPORT IMessageReceiver {
  public:
    /** Virtual destructor.
     */
    virtual ~IMessageReceiver();
    
    /** Receive NMM %Message.
     *
     * This method is called to send a %Message to this object.
     * The method returns the queue status after receiving the message. If the message could not be
     * received (for example because the object's internal queue is full), then the result is @c FAILURE.
     * If the object can't receive any more messages, then @c BLOCKED is returned. If the message was
     * successfully passed to the object, then the result is @c SUCCESS.
     *
     * @param message The message to send to this object.
     * @param sender The object which sent the message.
     * @return The queue status after receiving the message.
     */
    virtual Result receiveMessage(Message* message, const IMessageSender* sender) = 0;
  };
  
}

#include "nmm/nmm_suffix.hpp"

#endif
