/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_I_MESSAGE_SENDER_HPP
#define NMM_I_MESSAGE_SENDER_HPP NMM_I_MESSAGE_SENDER_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/Result.hpp"

#include "visibility.hpp"

namespace NMM {
  
  class IMessageReceiver;
  
  /** An object which sends Messages.
   *
   * This interface is implemented by all objects which send NMM Messages to high-level objects.
   *
   * @see IMessageReceiver
   */
  class DLLEXPORT IMessageSender {
  public:

    /** Constructor.
     */
    IMessageSender() throw();
    
    /** Virtual destructor.
     */
    virtual ~IMessageSender();
    
    /** This method sets the message receiver.
     *
     * @param receiver The message reveiver of this message sender.
     */
    void setMessageReceiver(IMessageReceiver* receiver);
    
    /** This method returns the message receiver.
     *
     * @return The message reveiver of this message sender.
     */
    IMessageReceiver* getMessageReceiver() const;
    
  private:
    IMessageReceiver* __receiver; ///< The message reveiver of this message sender.
  };
  
  
   /** Set message receiver.
     *
     * @param receiver pointer to IMessageReceiver to set
     */
    inline void IMessageSender::setMessageReceiver(IMessageReceiver* receiver) {
    __receiver = receiver;
  }
  
   /** Get message receiver.
     *
     * @return pointer to IMessageReceiver 
     */
    inline IMessageReceiver* IMessageSender::getMessageReceiver() const {
    return __receiver;
  }
  
}

#include "nmm/nmm_suffix.hpp"

#endif
