/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 * 
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_I_SIGNAL_SENDER_HPP
#define NMM_I_SIGNAL_SENDER_HPP NMM_I_SIGNAL_SENDER_HPP

#include "nmm/nmm_prefix.hpp"

#include "visibility.hpp"

namespace NMM {
  
  class ISignalReceiver;
  
  /** An object which sends signals to a signal receiver.
   *
   * @see ISignalReceiver ICSSignalReceiver Signal
   */
  class DLLEXPORT ISignalSender {
  public:
    /** Virtual destructor.
     */
    virtual ~ISignalSender();
    
    /** Set the signal receiver.
     *
     * This method sets the object which receives signals that control the flow of messages.
     *
     * @param signal_receiver Pointer to the signal receiver
     */
    void setSignalReceiver(ISignalReceiver* signal_receiver);
    
    /** Get the signal receiver.
     *
     * This method returns the object which receives signals that control the flow of messages.
     *
     * @return Pointer to the signal receiver
     */
    ISignalReceiver* getSignalReceiver() const;
    
  private:
    ISignalReceiver* __signal_receiver; ///< Pointer to object which receives signals
  };
  
  
  /** Set the signal receiver.
     *
     * This method sets the object which receives signals that control the flow of messages.
     *
     * @param Pointer to the signal receiver
     */
  inline void ISignalSender::setSignalReceiver(ISignalReceiver* signal_receiver) {
    __signal_receiver = signal_receiver;
  }
  
   /** Get the signal receiver.
     *
     * This method gets the object which receives signals that control the flow of messages.
     *
     * @return Pointer to the signal receiver
     */
  inline ISignalReceiver* ISignalSender::getSignalReceiver() const {
    return __signal_receiver;
  }
  
}

#include "nmm/nmm_suffix.hpp"

#endif
