/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#ifndef NMM_I_STATE_HPP
#define NMM_I_STATE_HPP NMM_I_STATE_HPP

#include "nmm/nmm_prefix.hpp"

#include "UnsuccessfulRequiredCallException.hpp"

#include "visibility.hpp"

namespace NMM {
  
  /** This class is a superclass for objetcs with state.
   */
  class DLLEXPORT IState {
  public:

    /** Virtual destructor.
     */
    virtual ~IState() {}

    /** Get state.
     *
     * @return state
     */
    virtual int getState() const = 0;

    /** Change to state.
     *
     * @param state state to change to
     */
    virtual void changeState(int state)
      throw (const UnsuccessfulRequiredCallException&) = 0;
  };
  
}

#include "nmm/nmm_suffix.hpp"

#endif
