/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_I_TS_ONE_WAY_BUFFER_RECEIVER_HPP
#define NMM_I_TS_ONE_WAY_BUFFER_RECEIVER_HPP NMM_I_TS_ONE_WAY_BUFFER_RECEIVER_HPP

#include "nmm/nmm_prefix.hpp"

#include "ISignalSender.hpp"
#include "FormatNotSupportedException.hpp"
#include "nmm/Result.hpp"

namespace NMM {
  
  class Format;
  class Buffer;
  
  /** One-way buffer receiver transport strategy interface.
   *
   * Interface for a %TransportStrategy which receives Buffers one-way
   * from a one-way %Buffer sender.
   */
  class ITSOneWayBufferReceiver
    : virtual public ISignalSender
  {
  public:
    /** Receive %Buffer one-way.
     *
     * This method passes a %Buffer to the %TransportStrategy without
     * waiting for a reply.
     *
     * @param buffer The buffer to be passed to the %TransportStrategy.
     */
    virtual Result receiveOneWayBuffer(Buffer* buffer) = 0;
    
    /** Configure the %TransportStrategy for the given instream format.
     *
     * @param format The instream format
     * @exception FormatNotSupportedException if configuration fails or the format is not supported
     */
    virtual void configure(const Format& format)
      throw(const FormatNotSupportedException&) = 0;
      
    /** Flushes the buffer receiver.
     *
     * This method flushes the buffer receiver. 
     */
    virtual void flushITSReceiver() = 0;      
  };
  
}

#include "nmm/nmm_suffix.hpp"

#endif
