/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 * 
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_I_TS_ONE_WAY_CEVENT_RECEIVER_HPP
#define NMM_I_TS_ONE_WAY_CEVENT_RECEIVER_HPP NMM_I_TS_ONE_WAY_CEVENT_RECEIVER_HPP

#include "nmm/nmm_prefix.hpp"

#include "ISignalSender.hpp"
#include "nmm/Result.hpp"

namespace NMM {
  
  class CEvent;
  
  /** One-way composite event receiver interface.
   *
   * Interface for a %TransportStrategy which receives CEvents one-way
   * from a one-way %CEvent sender.
   */
  class ITSOneWayCEventReceiver
    : virtual public ISignalSender
  {
  public:
    /** Receive %CEvent one-way.
     *
     * This method passes a %CEvent to the %TransportStrategy without
     * waiting for a reply.
     *
     * @param cevent The buffer to be passed to the %TransportStrategy.
     */
    virtual Result receiveOneWayCEvent(CEvent* cevent) = 0;

    /** Flushes the event receiver. 
     * 
     * This method flushes the event receiver. 
     */
    virtual void flushITSReceiver() = 0;      
  };
  
}

#include "nmm/nmm_suffix.hpp"

#endif
