/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 * 
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_I_TS_ONE_WAY_EVENT_RECEIVER_HPP
#define NMM_I_TS_ONE_WAY_EVENT_RECEIVER_HPP NMM_I_TS_ONE_WAY_EVENT_RECEIVER_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/Result.hpp"

namespace NMM {
  
  class Event;
  
  /** One-way event receiver transport strategy interface.
   *
   * Interface for a leaf %TransportStrategy which receives events
   * one-way from a one-way %Event sender.
   */
  class ITSOneWayEventReceiver {
  public:
    /** Virtual destructor.
     */
    virtual ~ITSOneWayEventReceiver() {}
    
    /** Receive %Event one-way.
     *
     * This method passes an %Event to the %TransportStrategy without
     * waiting for a reply.
     *
     * @param event The event to be passed to the %TransportStrategy
     */
    virtual void receiveOneWayEvent(Event& event) = 0;
  };
  
}

#include "nmm/nmm_suffix.hpp"

#endif
