/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#ifndef NMM_ILLEGAL_STATE_EXCEPTION_HPP
#define NMM_ILLEGAL_STATE_EXCEPTION_HPP NMM_ILLEGAL_STATE_EXCEPTION_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/serialize/SerializedException.hpp"

#include "visibility.hpp"

namespace NMM {
  
  /** Operation illegal in current state.
   *
   * This exception will be thrown, if a operation is performed which
   * is illegal in current state.
   */
  class DLLEXPORT IllegalStateException : public SerializedException
  {
  public:
    /**
     * Default constructor.
     */
    IllegalStateException () 
      : SerializedException("IllegalStateExcpetion") { setTypeName("IllegalStateException"); };

    /**
     * Constructor, which allows to specify an extra comment.
     * 
     * @param s extra comment
     */
    IllegalStateException (const string& s) 
      : SerializedException("IllegalStateException : " + s) {  setTypeName("IllegalStateException"); };

    /**
     * Constructor, which allows to specify the NamedObject which
     * threw the exception and an extra comment.
     *
     * @param n named object 
     * @param s extra comment
     */
    IllegalStateException (const NamedObject* n, const string& s = "") 
      : SerializedException(n, ((s == "") ? "IllegalStateException" : "IllegalStateException : " + s)) { setTypeName("IllegalStateException"); };

    /**
     * Copy constructor.
     *
     * @param exception exception to copy
     */
    IllegalStateException (const IllegalStateException& exception)
      : SerializedException(exception) {}

    /**
     * Copy this Exception.
     *
     * @return Pointer to copy of this exception.
     */
    IllegalStateException* copy() const { return new IllegalStateException(*this); }
  };
  
}

#include "nmm/nmm_suffix.hpp"

#endif
