/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#include "Message.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  Message::Message(Type t)
    : SerializedValue("Message", "NMM"),
      label(0),
      mutex(ThreadMutex::RECURSIVE)
  {
    // time is initialized by its standard constructor
    setDirection(DOWNSTREAM);
    type.setValue(t);
    setMandatoryBit(false);
    ref_count = 0;

    registerValues();
  }

  Message::Message(const Message& m)
    : SerializedValue(m),
      label(m.label),
      mutex(ThreadMutex::RECURSIVE)
  {
    setDirection(m.getDirection());
    type.setValue(m.getType());
    time.setValue(m.getTimestamp());
    delay_time.setValue(m.getDelayStartTime());
    setMandatoryBit(m.getMandatoryBit());
    ref_count = 0;

    registerValues();
  }
  
  Message::~Message() {}
  
  void Message::release() const 
  {
    mutex.lock();
    ref_count--;
    if (ref_count <= 0) {
      mutex.unlock();
      erase();
    }
    else {
      mutex.unlock();
    }
  }

  void Message::increaseCount(unsigned int ref) const 
  {
    mutex.lock();
    ref_count += ref;
    mutex.unlock();
  }
  
  void Message::setType(Type new_type) {
    type.setValue(new_type);
  }

  void Message::registerValues() {
    insertArgument(&direction);
    insertArgument(&type);
    insertArgument(&time);
    insertArgument(&mandatory);
    insertArgument(&label);
    insertArgument(&delay_time);
  }

  Message& Message::operator=(const Message& message) {
    direction  = message.getDirection();
    time       = message.getTimestamp();
    mandatory  = message.getMandatoryBit();
    delay_time = message.getDelayStartTime();
    return *this;
  }

  OValueStream& operator<<(OValueStream& ov, const Message::Type& t) {
    nmm_int32 temp = static_cast<nmm_int32>(t);
    ov << temp;
    return ov;
  }

  IValueStream& operator>>(IValueStream& iv, Message::Type& t) {
    nmm_int32 temp;
    iv >> temp;
    t = static_cast<Message::Type>(temp);
    return iv;
  }

  OValueStream& operator<<(OValueStream& ov, const Message::Direction& t) {
    nmm_int32 temp = static_cast<nmm_int32>(t);
    ov << temp;
    return ov;
  }

  IValueStream& operator>>(IValueStream& iv, Message::Direction& t) {
    nmm_int32 temp;
    iv >> temp;
    t = static_cast<Message::Direction>(temp);
    return iv;
  }
  
}

#include "nmm/nmm_suffix.hpp"
