/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "Timestamp.hpp"

#include "nmm/comm/serialize/OValueStream.hpp"
#include "nmm/comm/serialize/IValueStream.hpp"
#include "nmm/comm/serialize/TValueConstructor.hpp"

#include "nmm/comm/serialize/SerializeUtilsTime.hpp"

#include "nmm/utils/NamedObject.hpp"

#include <ostream>

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  std::ostream& operator<<(std::ostream& stream, const Timestamp& time) {
    return stream << "Counter: "     << time.stream_counter
		  << ", created at " << time.sync_time
                  << ", valid "      << time.is_valid; 
  };
  
  OValueStream& operator<<(OValueStream& ov, const Timestamp& ts) {
    ov.beginType("Timestamp", "NMM");
    ov << ts.sync_time;
    ov << ts.stream_counter;
    ov << ts.is_valid;
    ov << ts.is_set;
    ov.endType();

    return ov;
  }

  IValueStream& operator>>(IValueStream& iv, Timestamp& ts) {
    if (iv.getCurrentType() != "Timestamp" && iv.getCurrentNamespace() != "NMM") {
      GLOBAL_ERROR_STREAM("Invalid type assignment");
      iv.nextElement();
      return iv;
    }

    if (!iv.goDown()) {
      GLOBAL_ERROR_STREAM("Timstamp has no data");
      iv.nextElement();
      return iv;
    }

    iv >> ts.sync_time;
    iv >> ts.stream_counter;
    iv >> ts.is_valid;
    iv >> ts.is_set;

    iv.goUp();

    return iv;
  }

  TValueConstructor<Timestamp> timestamp_prototype("Timestamp", "NMM");
}


#include "nmm/nmm_suffix.hpp"
