/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "TransferBuffer2.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  TransferBuffer2::TransferBuffer2() : size(0), data(0)
  {
  }

  TransferBuffer2::TransferBuffer2(const TransferBuffer2& buf) 
      : size(buf.getSize())
      , data(const_cast<TransferBuffer2&>(buf).getData())
  {
  }

  TransferBuffer2* TransferBuffer2::copy() const 
  { 
    return new TransferBuffer2(*this); 
  }

  const TransferBuffer2& TransferBuffer2::operator=(const TransferBuffer2& buf)
  {
    setData(buf.getData(), buf.getSize());
    return *this;
  }

  TransferBuffer2::~TransferBuffer2() 
  {
  }

  void TransferBuffer2::setData(const char* d, size_t s) 
  {
    size = s;
    data = const_cast<char*>(d);
  }
  
  void TransferBuffer2::setData(char* d, size_t s) 
  {
    size = s;
    data = d;
  }

  char* TransferBuffer2::getData() 
  {
    return data;
  }

  const char* TransferBuffer2::getData() const
  {
    return data;
  }

  size_t TransferBuffer2::getSize() const
  {
    return size;
  }

  OValueStream& operator<<(OValueStream& o_stream, const TransferBuffer2& my_struct) {
    o_stream.beginType(TypeInfo("TransferBuffer2", "NMM"));

    nmm_uint32 size = static_cast<nmm_uint32>(my_struct.getSize());
    o_stream << size;
    o_stream.writeBuffer(my_struct.getData(), static_cast<nmm_uint32>(my_struct.getSize()));
    
    o_stream.endType();                             //End the new type
    
    return o_stream;                                //return a reference to o_stream
  }

  IValueStream& operator>>(IValueStream& i_stream, TransferBuffer2& my_struct) {
    /*Begin> Type checking */
    if (i_stream.getCurrentTypeInfo() != TypeInfo("TransferBuffer2", "NMM")) {
      cerr << "Invalid Type assignment" << endl;
      // Throw an exception
      throw InvalidTypeException(i_stream.getCurrentTypeInfo(),
				 TypeInfo("TransferBuffer2", "NMM"));
    }
    /*End< */

    /*Begin> Try to access the attributes */
    if (!i_stream.goDown()) {
      cerr << "TransferBuffer2 has no attributes" << endl;
      i_stream.nextElement();   //Don't forget to skip this element.
      return i_stream;
    }
    /*End< */

    /*Begin> read the attributes in the same order as they are written*/
    nmm_uint32 size;
    i_stream >> size;

    char* p = new char[size];
    i_stream.readBuffer(p, size);
    my_struct.setData(p, size);

    /*End<*/

    i_stream.goUp();  //Don't forget to go up that calls nextElement() too.

    return i_stream;  //return a reference to the current ValueStream
  }




  TValueConstructor<TransferBuffer2>    __nmm__transfer_buffer2("TransferBuffer2", "NMM");

}

#include "nmm/nmm_suffix.hpp"
