/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_UNEXPECTED_EXCEPTION_HPP
#define NMM_UNEXPECTED_EXCEPTION_HPP NMM_UNEXPECTED_EXCEPTION_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/serialize/SerializedException.hpp"

#include "visibility.hpp"

namespace NMM {

  /** Exception is thrown whenever Exception could not be specified in detail.
   */
  
  class DLLEXPORT UnexpectedException : public SerializedException {
  public:
    /**
     * Default constructor.
     */
    UnexpectedException ();
    
    /** Constructor, which allows to specify an extra comment.
     *
     * @param s extra comment
     */
    UnexpectedException(const std::string& s);
    
    /** Constructor, which allows to specify the NamedObject.
     *
     * Constructor, which allows to specify the NamedObject which
     * threw the exception and an extra comment
     *
     * @param n named object
     * @param s extra comment
     */
    UnexpectedException (const NamedObject* n, const std::string& s = std::string());
    
    /** CopyConstructor.
     *
     * @param exception exception to copy
     */
    UnexpectedException (const UnexpectedException& exception);
    
    /** Copies this exception.
     *
     * @return pointer to copy of this exception
     */
    UnexpectedException* copy() const;
  };
  
}

#include "nmm/nmm_suffix.hpp"

#endif
