/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#ifndef NMM_UNKNOWN_EVENT_EXCEPTION_HPP
#define NMM_UNKNOWN_EVENT_EXCEPTION_HPP NMM_UNKNOWN_EVENT_EXCEPTION_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/serialize/SerializedException.hpp"

#include "visibility.hpp"

namespace NMM {

  /** Exception throwm when there is no event registered with this event key.
   */
  class DLLEXPORT UnknownEventException : public SerializedException
  {
  public:

    /** Default constructor.
     */
    UnknownEventException()
      : SerializedException("UnknownEventException") { setTypeName("UnknownEventException"); }

    /** Constructor with exception message.
     *
     * @param message exception message
     */
    UnknownEventException(const string& message)
      : SerializedException("UnknownEventException : " + message) { setTypeName("UnknownEventException"); }

    /** Constructor with exception message and name of NamedObject which caused exception.
     *
     * @param n NamedObject which caused exception
     * @param s exception message
     */
    UnknownEventException(const NamedObject* n, const string& s)
      : SerializedException(n, ((s == "") ? "UnknownEventException" : "UnknownEventException : " + s)) { setTypeName("UnknownEventException"); };

    /** Copy UnknownEventException.
     *
     * @return pointer to copy of this UnknownEventException        
     */
    UnknownEventException* copy() const { return new UnknownEventException(*this); }
  };
  
}

#include "nmm/nmm_suffix.hpp"

#endif
