/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#ifndef NMM_UNSUCCESSFUL_REQUIRED_CALL_EXCEPTION_HPP
#define NMM_UNSUCCESSFUL_REQUIRED_CALL_EXCEPTION_HPP NMM_UNSUCCESSFUL_REQUIRED_CALL_EXCEPTION_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/serialize/SerializedException.hpp"

#include "visibility.hpp"

namespace NMM {

  /** At least one required call was not successful.
   *
   * This Exception should be thrown if a required method was not
   * called or completed without success when changing the state of an
   * object
   */
  class DLLEXPORT UnsuccessfulRequiredCallException : public SerializedException
  {
  public:
    /** Default constructor.
     */
    UnsuccessfulRequiredCallException () 
      : SerializedException("UnsuccessfulRequiredCallExcpetion") { setTypeName("UnsuccessfulRequiredCallException"); };

    /** Constructor, which allows to specify an extra comment.
     *
     * @param s extra comment
     */
    UnsuccessfulRequiredCallException (const string& s) 
      : SerializedException("UnsuccessfulRequiredCallException : " + s) {setTypeName("UnsuccessfulRequiredCallException");};

    /** Constructor, which allows to specify the NamedObject.
     * 
     * Constructor, which allows to specify the NamedObject which
     * threw the exception and an extra comment
     *
     * @param n pointer named object
     * @param s extra comment
     */
    UnsuccessfulRequiredCallException (const NamedObject* n, const string& s = "") 
      : SerializedException(n, ((s == "") ? "UnsuccessfulRequiredCallException" : "UnsuccessfulRequiredCallException : " + s)) {setTypeName("UnsuccessfulRequiredCallException");};

    /** Copies this exception.
     *
     * @return pointer to copy of this exception
     */
    UnsuccessfulRequiredCallException* copy() const { return new UnsuccessfulRequiredCallException(*this); }
  };
  
}

#include "nmm/nmm_suffix.hpp"

#endif
