/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IByteCounter.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IByteCounter::used() {} 
  
  
  nmm_int32 IByteCounter::getSendingCounter() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("IByteCounter::getSendingCounter");
    event.setReturnValue(&return_value);
    if(const_cast<IByteCounter*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 IByteCounter::getReceivingCounter() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("IByteCounter::getReceivingCounter");
    event.setReturnValue(&return_value);
    if(const_cast<IByteCounter*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IByteCounter::resetSendingCounter()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IByteCounter::resetSendingCounter");
    if(const_cast<IByteCounter*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IByteCounter::resetReceivingCounter()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IByteCounter::resetReceivingCounter");
    if(const_cast<IByteCounter*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IByteCounter::IByteCounter_type = "IByteCounter";
  NMM::TConstructor<IByteCounter > __IByteCounter_prototype("IByteCounter","NMM");


  const char* IByteCounter::getSendingCounter_event = "IByteCounter::getSendingCounter";
  const char* IByteCounter::getReceivingCounter_event = "IByteCounter::getReceivingCounter";
  const char* IByteCounter::resetSendingCounter_event = "IByteCounter::resetSendingCounter";
  const char* IByteCounter::resetReceivingCounter_event = "IByteCounter::resetReceivingCounter";


}


#include "nmm/nmm_suffix.hpp"

