/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IByteCounter_HPP
#define NMM_IByteCounter_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{
  /** Sent and Transmitted Byte Counter.
   * This is an interface for getting statistics about the
   * number of bytes sent or received by a transport strategy.
   *
   * @see TransportStrategy
   */

  class DLLEXPORT IByteCounter : virtual public NMM::Interface { 
  public:
    IByteCounter() : NMM::Interface(IByteCounter::IByteCounter_type, "NMM") { };

    IByteCounter(NMM::NMMObject& parent) : NMM::Interface (parent, IByteCounter::IByteCounter_type, "NMM") { };

    virtual ~IByteCounter() {};

    static const char* IByteCounter_type;

    virtual  string getInterfaceType() const { return IByteCounter_type; }

    NMM::Interface* copy() const { return new IByteCounter(*this); }
    static void used(); 
    static const char* getSendingCounter_event;
    static const char* getReceivingCounter_event;
    static const char* resetSendingCounter_event;
    static const char* resetReceivingCounter_event;
    /** Get total number of bytes sent over the network.
     *
     * @return Total number of bytes sent over the network
     */

    virtual nmm_int32 getSendingCounter() const
      throw(const NMM::SerializedException&);
    /** Get total number of bytes received from the network.
     *
     * @return Total number of bytes received from the network
     */

    virtual nmm_int32 getReceivingCounter() const
      throw(const NMM::SerializedException&);
    /** Reset the counter for sent bytes.
     * This method resets the internal counter which is used by getSendingCounter()
     * to count the number of bytes sent over the network.
     */

    virtual void resetSendingCounter()
      throw(const NMM::SerializedException&);
    /** Reset the counter for received bytes.
     * This method resets the internal counter which is used by getReceivingCounter()
     * to count the number of bytes received from the network.
     */

    virtual void resetReceivingCounter()
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IByteCounter> IByteCounter_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
