/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IByteCounterImpl_HPP
#define NMM_IByteCounterImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IByteCounter.hpp"


#include "visibility.hpp"

namespace NMM
{
  /** Sent and Transmitted Byte Counter.
   * This is an interface for getting statistics about the
   * number of bytes sent or received by a transport strategy.
   *
   * @see TransportStrategy
   */

  class DLLEXPORT IByteCounterImpl  : public virtual NMM::NMMObject { 
  public:
    IByteCounterImpl() { addInterface(new IByteCounter(*this)); registerEvents(); };

    virtual ~IByteCounterImpl() {};

  protected:
    /** Get total number of bytes sent over the network.
     *
     * @return Total number of bytes sent over the network
     */

    virtual nmm_int32 getSendingCounter() const = 0;
    /** Get total number of bytes received from the network.
     *
     * @return Total number of bytes received from the network
     */

    virtual nmm_int32 getReceivingCounter() const = 0;
    /** Reset the counter for sent bytes.
     * This method resets the internal counter which is used by getSendingCounter()
     * to count the number of bytes sent over the network.
     */

    virtual void resetSendingCounter() = 0;
    /** Reset the counter for received bytes.
     * This method resets the internal counter which is used by getReceivingCounter()
     * to count the number of bytes received from the network.
     */

    virtual void resetReceivingCounter() = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
