/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IIPControl.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IIPControl::used() {} 
  
  
  void IIPControl::setRecvBufferSize(const nmm_int32& buffer_size)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > buffer_size_value(buffer_size, false);
    NMM::Event event("IIPControl::setRecvBufferSize", &buffer_size_value);
    if(const_cast<IIPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_int32 IIPControl::getRecvBufferSize() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("IIPControl::getRecvBufferSize");
    event.setReturnValue(&return_value);
    if(const_cast<IIPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IIPControl::setSendBufferSize(const nmm_int32& buffer_size) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > buffer_size_value(buffer_size, false);
    NMM::Event event("IIPControl::setSendBufferSize", &buffer_size_value);
    if(const_cast<IIPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_int32 IIPControl::getSendBufferSize() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("IIPControl::getSendBufferSize");
    event.setReturnValue(&return_value);
    if(const_cast<IIPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IIPControl::setPriority(const nmm_int32& priority)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > priority_value(priority, false);
    NMM::Event event("IIPControl::setPriority", &priority_value);
    if(const_cast<IIPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_int32 IIPControl::getPriority() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("IIPControl::getPriority");
    event.setReturnValue(&return_value);
    if(const_cast<IIPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IIPControl::setInstreamBufferPriority(const Sock::QOS_TAG& qos_tag)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<Sock::QOS_TAG > qos_tag_value(qos_tag, false);
    NMM::Event event("IIPControl::setInstreamBufferPriority", &qos_tag_value);
    if(const_cast<IIPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IIPControl::setInstreamEventPriority(const Sock::QOS_TAG& qos_tag)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<Sock::QOS_TAG > qos_tag_value(qos_tag, false);
    NMM::Event event("IIPControl::setInstreamEventPriority", &qos_tag_value);
    if(const_cast<IIPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IIPControl::setOutOfBandEventPriority(const Sock::QOS_TAG& qos_tag)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<Sock::QOS_TAG > qos_tag_value(qos_tag, false);
    NMM::Event event("IIPControl::setOutOfBandEventPriority", &qos_tag_value);
    if(const_cast<IIPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  Sock::QOS_TAG IIPControl::getInstreamBufferPriority()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Sock::QOS_TAG > return_value;
    NMM::Event event("IIPControl::getInstreamBufferPriority");
    event.setReturnValue(&return_value);
    if(const_cast<IIPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Sock::QOS_TAG IIPControl::getInstreamEventPriority()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Sock::QOS_TAG > return_value;
    NMM::Event event("IIPControl::getInstreamEventPriority");
    event.setReturnValue(&return_value);
    if(const_cast<IIPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Sock::QOS_TAG IIPControl::getOutOfBandEventPriority()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Sock::QOS_TAG > return_value;
    NMM::Event event("IIPControl::getOutOfBandEventPriority");
    event.setReturnValue(&return_value);
    if(const_cast<IIPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IIPControl::IIPControl_type = "IIPControl";
  NMM::TConstructor<IIPControl > __IIPControl_prototype("IIPControl","NMM");


  const char* IIPControl::setRecvBufferSize_event = "IIPControl::setRecvBufferSize";
  const char* IIPControl::getRecvBufferSize_event = "IIPControl::getRecvBufferSize";
  const char* IIPControl::setSendBufferSize_event = "IIPControl::setSendBufferSize";
  NMM::Event* IIPControl::create_setSendBufferSize(const nmm_int32& buffer_size)
  {
    NMM::TInValue<nmm_int32 >* buffer_size_value = new NMM::TInValue<nmm_int32 >();
    buffer_size_value->setValue(buffer_size);

    return new NMM::Event("IIPControl::setSendBufferSize", buffer_size_value);
  }

  const char* IIPControl::getSendBufferSize_event = "IIPControl::getSendBufferSize";
  const char* IIPControl::setPriority_event = "IIPControl::setPriority";
  const char* IIPControl::getPriority_event = "IIPControl::getPriority";
  const char* IIPControl::setInstreamBufferPriority_event = "IIPControl::setInstreamBufferPriority";
  const char* IIPControl::setInstreamEventPriority_event = "IIPControl::setInstreamEventPriority";
  const char* IIPControl::setOutOfBandEventPriority_event = "IIPControl::setOutOfBandEventPriority";
  const char* IIPControl::getInstreamBufferPriority_event = "IIPControl::getInstreamBufferPriority";
  const char* IIPControl::getInstreamEventPriority_event = "IIPControl::getInstreamEventPriority";
  const char* IIPControl::getOutOfBandEventPriority_event = "IIPControl::getOutOfBandEventPriority";


}


#include "nmm/nmm_suffix.hpp"

