/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "INetStream.hpp"
#include <iostream>

#ifndef BUILD_PPC
#include <algorithm> //required for std::swap

#define ByteSwap(x) ByteSwap2((unsigned char *) &x,sizeof(x))

void ByteSwap2(unsigned char * b, int n)
{
   register int i = 0;
   register int j = n-1;
   while (i<j)
   {
      std::swap(b[i], b[j]);
      i++, j--;
   }
}

#else 
  #define ByteSwap(x) (x)
#endif


#include "nmm/nmm_prefix.hpp"

namespace NMM {  
  
  INetStream::INetStream()
    : IStream(TypeInfo("INetStream", "NMM")),
      __sock(0),
      __read_counter(0)
  {}
  
  INetStream::INetStream(const INetStream& other)
    : IStream(other),
      __sock(other.__sock),
      __read_counter(0)
  {}
  
  INetStream::INetStream(const TypeInfo& ti)
    : IStream(ti),
      __sock(0),
      __read_counter(0)
  {}
  
  INetStream::~INetStream() {}
  
  INetStream& INetStream::operator>>(nmm_uint16& value) {
    readValue(&value, SIZE_OF_NMM_UINT16);
    ByteSwap(value);
    return *this;
  }
  INetStream& INetStream::operator>>(nmm_uint32& value) {
    readValue(&value, SIZE_OF_NMM_UINT32);
    ByteSwap(value);
    return *this;
  }
  INetStream& INetStream::operator>>(nmm_uint64& value) {
    readValue(&value, SIZE_OF_NMM_UINT64);
    ByteSwap(value);
    return *this;
  }
  INetStream& INetStream::operator>>(nmm_int16& value ) {
    readValue(&value, SIZE_OF_NMM_INT16);
    ByteSwap(value);
    return *this;
  }
  INetStream& INetStream::operator>>(nmm_int32& value) {
    readValue(&value, SIZE_OF_NMM_UINT32);
    ByteSwap(value);
    return *this;
  }
  INetStream& INetStream::operator>>(nmm_int64& value) {
    readValue(&value, SIZE_OF_NMM_INT64);
    ByteSwap(value);
    return *this;
  }
  INetStream& INetStream::operator>>(nmm_bool&  value) {
    readValue(&value, SIZE_OF_NMM_BOOL);
    return *this;
  }
  INetStream& INetStream::operator>>(nmm_double& value) {
    readValue(&value, SIZE_OF_NMM_DOUBLE);
    ByteSwap(value);

    return *this;
  }
  INetStream& INetStream::operator>>(nmm_float& value) {
    readValue(&value, SIZE_OF_NMM_FLOAT);
    ByteSwap(value);
    return *this;
  }
  INetStream& INetStream::operator>>(nmm_char& value) {
    readValue(&value, SIZE_OF_NMM_CHAR);
    return *this;
  }
  INetStream& INetStream::operator>>(nmm_uchar& value) {
    readValue(&value, SIZE_OF_NMM_CHAR);
    return *this;
  }
  INetStream& INetStream::operator>>(string& value) {
    nmm_int32 size = 0;
    (*this) >> size;
    if (size == 0) {   //string is empty
      value.resize(0);
    } else {
      /* Added additional check before calling method resize, to avoid
       * needless memory allocation*/
      if (static_cast<nmm_int32>(value.size()) != size) {
        value.resize(size);
      }
      readValue(&value[0], size);
    }
    return *this;
  }

  void INetStream::readBuffer(char* value, nmm_uint32 size ) {
    readValue(value, size);
  }

  void INetStream::readBuffer(unsigned char* value, nmm_uint32 size ) {
    readValue(value, size);
  }

  bool INetStream::receivePacket() {
    return true;
  }
  
}
#include "nmm/nmm_suffix.hpp"
