/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IPAddress.hpp"

#include "nmm/utils/net/nmm_netutils.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  void IPAddress::setIPAddress(const uint32_t& ip) throw() {
    __ip_address = ip;
    __net_addr.s_addr = htonl(ip);
  }
  
  void IPAddress::setHostname(const std::string& host)
    throw(const BindingException&)
  {
    if ((__net_addr.s_addr = inet_addr(host.c_str())) == INADDR_NONE) {
      struct hostent* he;
      if (!(he = gethostbyname(host.c_str()))) {
        throw BindingException("Failed to resolve host " + host);
      }
      __net_addr.s_addr = *((uint32_t*) he->h_addr);
    }
    
    __ip_address = ntohl(__net_addr.s_addr);
  }
  
  void IPAddress::updateAttributes() {
    setIPAddress(__ip_address.getRValue());
  }
  
  IPAddress::IPAddress(const TypeInfo& typeinfo) throw()
    : SerializedValue(typeinfo)
  {
    setIPAddress(INADDR_NONE);
    insertArgument(&__ip_address);
  }
  
  IPAddress::IPAddress(const TypeInfo& typeinfo,
                       const std::string& hostname)
    throw (const BindingException&)
    : SerializedValue(typeinfo)
  {
    setHostname(hostname);
    insertArgument(&__ip_address);
  }
  
  IPAddress::IPAddress(const IPAddress& addr) throw()
    : SerializedValue(addr)
  {
    setIPAddress(addr.getIPAddress());
    insertArgument(&__ip_address);
  }
  
  string IPAddress::toString() const throw() {
    return string(inet_ntoa(getINetAddr()));
  }
  
}

#include "nmm/nmm_suffix.hpp"
