/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IRTPControl.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IRTPControl::used() {} 
  
  
  void IRTPControl::setCNAME(const string& cname)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > cname_value(cname, false);
    NMM::Event event("IRTPControl::setCNAME", &cname_value);
    if(const_cast<IRTPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  string IRTPControl::getCNAME() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("IRTPControl::getCNAME");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IRTPControl::setTTL(const nmm_uint32& ttl)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > ttl_value(ttl, false);
    NMM::Event event("IRTPControl::setTTL", &ttl_value);
    if(const_cast<IRTPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_uint32 IRTPControl::getTTL() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("IRTPControl::getTTL");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IRTPControl::setBufferSize(const nmm_uint32& buffer_size)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > buffer_size_value(buffer_size, false);
    NMM::Event event("IRTPControl::setBufferSize", &buffer_size_value);
    if(const_cast<IRTPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_uint32 IRTPControl::getBufferSize() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("IRTPControl::getBufferSize");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IRTPControl::setMaxFrameSize(const nmm_uint32& max_frame_size)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > max_frame_size_value(max_frame_size, false);
    NMM::Event event("IRTPControl::setMaxFrameSize", &max_frame_size_value);
    if(const_cast<IRTPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_uint32 IRTPControl::getMaxFrameSize() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("IRTPControl::getMaxFrameSize");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IRTPControl::setSessionBandwidth(const nmm_uint32& bandwidth)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > bandwidth_value(bandwidth, false);
    NMM::Event event("IRTPControl::setSessionBandwidth", &bandwidth_value);
    if(const_cast<IRTPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_uint32 IRTPControl::getSessionBandwidth() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("IRTPControl::getSessionBandwidth");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IRTPControl::sendRTCPReport()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IRTPControl::sendRTCPReport");
    if(const_cast<IRTPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IRTPControl::IRTPControl_type = "IRTPControl";
  NMM::TConstructor<IRTPControl > __IRTPControl_prototype("IRTPControl","NMM");


  const char* IRTPControl::setCNAME_event = "IRTPControl::setCNAME";
  const char* IRTPControl::getCNAME_event = "IRTPControl::getCNAME";
  const char* IRTPControl::setTTL_event = "IRTPControl::setTTL";
  const char* IRTPControl::getTTL_event = "IRTPControl::getTTL";
  const char* IRTPControl::setBufferSize_event = "IRTPControl::setBufferSize";
  const char* IRTPControl::getBufferSize_event = "IRTPControl::getBufferSize";
  const char* IRTPControl::setMaxFrameSize_event = "IRTPControl::setMaxFrameSize";
  const char* IRTPControl::getMaxFrameSize_event = "IRTPControl::getMaxFrameSize";
  const char* IRTPControl::setSessionBandwidth_event = "IRTPControl::setSessionBandwidth";
  const char* IRTPControl::getSessionBandwidth_event = "IRTPControl::getSessionBandwidth";
  const char* IRTPControl::sendRTCPReport_event = "IRTPControl::sendRTCPReport";


}


#include "nmm/nmm_suffix.hpp"

