/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IRTPReceptionStats.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IRTPReceptionStats::used() {} 
  
  
  nmm_uint32 IRTPReceptionStats::getNumberOfReceivedPackets() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("IRTPReceptionStats::getNumberOfReceivedPackets");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPReceptionStats*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 IRTPReceptionStats::getNumberOfExpectedPackets() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("IRTPReceptionStats::getNumberOfExpectedPackets");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPReceptionStats*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint64 IRTPReceptionStats::getNumberOfBytesReceived() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint64 > return_value;
    NMM::Event event("IRTPReceptionStats::getNumberOfBytesReceived");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPReceptionStats*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Interval IRTPReceptionStats::getJitter() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Interval > return_value;
    NMM::Event event("IRTPReceptionStats::getJitter");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPReceptionStats*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IRTPReceptionStats::IRTPReceptionStats_type = "IRTPReceptionStats";
  NMM::TConstructor<IRTPReceptionStats > __IRTPReceptionStats_prototype("IRTPReceptionStats","NMM");


  const char* IRTPReceptionStats::getNumberOfReceivedPackets_event = "IRTPReceptionStats::getNumberOfReceivedPackets";
  const char* IRTPReceptionStats::getNumberOfExpectedPackets_event = "IRTPReceptionStats::getNumberOfExpectedPackets";
  const char* IRTPReceptionStats::getNumberOfBytesReceived_event = "IRTPReceptionStats::getNumberOfBytesReceived";
  const char* IRTPReceptionStats::getJitter_event = "IRTPReceptionStats::getJitter";


}


#include "nmm/nmm_suffix.hpp"

