/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IRTPReceptionStatsImpl_HPP
#define NMM_IRTPReceptionStatsImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IRTPReceptionStats.hpp"


#include "visibility.hpp"

namespace NMM
{
  /** Interface for querying RTP reception statistics.
   *
   * This interface can be used for querying the reception statistics
   * of the RTP sink which are obtained by RTCP sender reports.
   *
   * @todo Find out what all methods do, remove undocumented methods
   *
   * @see IRTPControl IRTPTransmissionStats RTPReceiverStrategy
   */

  class DLLEXPORT IRTPReceptionStatsImpl  : public virtual NMM::NMMObject { 
  public:
    IRTPReceptionStatsImpl() { addInterface(new IRTPReceptionStats(*this)); registerEvents(); };

    virtual ~IRTPReceptionStatsImpl() {};

  protected:
    /** Get the total number of RTP packets received.
     */

    virtual nmm_uint32 getNumberOfReceivedPackets() const = 0;
    /** Get the total number of RTP packets expected.
     */

    virtual nmm_uint32 getNumberOfExpectedPackets() const = 0;
    /** Get the number of received bytes.
     */

    virtual nmm_uint64 getNumberOfBytesReceived() const = 0;
    /** Get the inter-arrival jitter.
     *
     * This method gets the packet inter-arrival jitter based on the
     * receiver reports. It is defined by RFC3550 as follows:
     *
     * "An estimate of the statistical variance of the RTP data packet
     * interarrival time, measured in timestamp units and expressed as
     * an unsigned integer. The interarrival jitter @a J is defined to
     * be the mean deviation (smoothed absolute value) of the
     * difference @a D in packet spacing at the receiver compared to
     * the sender for a pair of packets."
     *
     * @return The inter-arrival jitter in seconds
     */

    virtual Interval getJitter() const = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
