/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IRTPTransmissionStats.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IRTPTransmissionStats::used() {} 
  
  
  nmm_uint32 IRTPTransmissionStats::getLastReceivedPacketNumber() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("IRTPTransmissionStats::getLastReceivedPacketNumber");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPTransmissionStats*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 IRTPTransmissionStats::getFirstReportetPacketNumber() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("IRTPTransmissionStats::getFirstReportetPacketNumber");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPTransmissionStats*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint64 IRTPTransmissionStats::getTotalNumberOfPackets() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint64 > return_value;
    NMM::Event event("IRTPTransmissionStats::getTotalNumberOfPackets");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPTransmissionStats*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint64 IRTPTransmissionStats::getTotalNumberOfBytes() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint64 > return_value;
    NMM::Event event("IRTPTransmissionStats::getTotalNumberOfBytes");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPTransmissionStats*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 IRTPTransmissionStats::getNumberOfLostPackets() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("IRTPTransmissionStats::getNumberOfLostPackets");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPTransmissionStats*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Interval IRTPTransmissionStats::getJitter() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Interval > return_value;
    NMM::Event event("IRTPTransmissionStats::getJitter");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPTransmissionStats*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_float IRTPTransmissionStats::getPacketLossRatio() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_float > return_value;
    NMM::Event event("IRTPTransmissionStats::getPacketLossRatio");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPTransmissionStats*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Interval IRTPTransmissionStats::getRoundTripDelay() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Interval > return_value;
    NMM::Event event("IRTPTransmissionStats::getRoundTripDelay");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPTransmissionStats*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 IRTPTransmissionStats::getPacketsLostBetweenRR() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("IRTPTransmissionStats::getPacketsLostBetweenRR");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPTransmissionStats*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IRTPTransmissionStats::IRTPTransmissionStats_type = "IRTPTransmissionStats";
  NMM::TConstructor<IRTPTransmissionStats > __IRTPTransmissionStats_prototype("IRTPTransmissionStats","NMM");


  const char* IRTPTransmissionStats::getLastReceivedPacketNumber_event = "IRTPTransmissionStats::getLastReceivedPacketNumber";
  const char* IRTPTransmissionStats::getFirstReportetPacketNumber_event = "IRTPTransmissionStats::getFirstReportetPacketNumber";
  const char* IRTPTransmissionStats::getTotalNumberOfPackets_event = "IRTPTransmissionStats::getTotalNumberOfPackets";
  const char* IRTPTransmissionStats::getTotalNumberOfBytes_event = "IRTPTransmissionStats::getTotalNumberOfBytes";
  const char* IRTPTransmissionStats::getNumberOfLostPackets_event = "IRTPTransmissionStats::getNumberOfLostPackets";
  const char* IRTPTransmissionStats::getJitter_event = "IRTPTransmissionStats::getJitter";
  const char* IRTPTransmissionStats::getPacketLossRatio_event = "IRTPTransmissionStats::getPacketLossRatio";
  const char* IRTPTransmissionStats::getRoundTripDelay_event = "IRTPTransmissionStats::getRoundTripDelay";
  const char* IRTPTransmissionStats::getPacketsLostBetweenRR_event = "IRTPTransmissionStats::getPacketsLostBetweenRR";


}


#include "nmm/nmm_suffix.hpp"

