/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IRTPTransmissionStatsImpl_HPP
#define NMM_IRTPTransmissionStatsImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IRTPTransmissionStats.hpp"


#include "visibility.hpp"

namespace NMM
{
  /** Interface for querying RTP transmission statistics.
   *
   * This interface can be used for querying the transmission
   * statistics of the RTP sink which are obtained by RTCP receiver
   * reports.
   *
   * @todo Find out what all methods do, remove undocumented methods
   *
   * @see IRTPControl IRTPReceptionStats RTPSenderStrategy
   */

  class DLLEXPORT IRTPTransmissionStatsImpl  : public virtual NMM::NMMObject { 
  public:
    IRTPTransmissionStatsImpl() { addInterface(new IRTPTransmissionStats(*this)); registerEvents(); };

    virtual ~IRTPTransmissionStatsImpl() {};

  protected:
    /** Get the sequence number of the last received packet.
     *
     * @todo Find out what this method REALLY returns 
     */

    virtual nmm_uint32 getLastReceivedPacketNumber() const = 0;
    /** Get the sequence number of the first received packet.
     *
     * @todo Find out what this method REALLY returns
     */

    virtual nmm_uint32 getFirstReportetPacketNumber() const = 0;
    /** Get the total number of packets transmitted.
     */

    virtual nmm_uint64 getTotalNumberOfPackets() const = 0;
    /** Get the total number of bytes transmitted.
     */

    virtual nmm_uint64 getTotalNumberOfBytes() const = 0;
    /** Get the number of lost packets.
     *
     * @todo Find out what this method REALLY returns
     */

    virtual nmm_uint32 getNumberOfLostPackets() const = 0;
    /** Get the inter-arrival jitter.
     *
     * This method gets the packet inter-arrival jitter based on the
     * receiver reports. It is defined by RFC3550 as follows:
     *
     * "An estimate of the statistical variance of the RTP data packet
     * interarrival time, measured in timestamp units and expressed as
     * an unsigned integer. The interarrival jitter @a J is defined to
     * be the mean deviation (smoothed absolute value) of the
     * difference @a D in packet spacing at the receiver compared to
     * the sender for a pair of packets."
     *
     * @return The inter-arrival jitter in timestamp units
     */

    virtual Interval getJitter() const = 0;

    virtual nmm_float getPacketLossRatio() const = 0;

    virtual Interval getRoundTripDelay() const = 0;

    virtual nmm_int32 getPacketsLostBetweenRR() const = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
