/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ITCPControl.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ITCPControl::used() {} 
  
  
  void ITCPControl::enableNoDelay(const nmm_bool& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > value_value(value, false);
    NMM::Event event("ITCPControl::enableNoDelay", &value_value);
    if(const_cast<ITCPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_bool ITCPControl::isDelayEnabled() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ITCPControl::isDelayEnabled");
    event.setReturnValue(&return_value);
    if(const_cast<ITCPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITCPControl::setMaxSegment(const nmm_int32& mss)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > mss_value(mss, false);
    NMM::Event event("ITCPControl::setMaxSegment", &mss_value);
    if(const_cast<ITCPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_int32 ITCPControl::getMaxSegment() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("ITCPControl::getMaxSegment");
    event.setReturnValue(&return_value);
    if(const_cast<ITCPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITCPControl::enablePartialFrames(const nmm_bool& enable)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > enable_value(enable, false);
    NMM::Event event("ITCPControl::enablePartialFrames", &enable_value);
    if(const_cast<ITCPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_bool ITCPControl::isPartialFramesEnabled()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ITCPControl::isPartialFramesEnabled");
    event.setReturnValue(&return_value);
    if(const_cast<ITCPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 ITCPControl::getUnreadQueueCount() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("ITCPControl::getUnreadQueueCount");
    event.setReturnValue(&return_value);
    if(const_cast<ITCPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool ITCPControl::allUrgentDataSend() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ITCPControl::allUrgentDataSend");
    event.setReturnValue(&return_value);
    if(const_cast<ITCPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 ITCPControl::getUnsendQueueCount() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("ITCPControl::getUnsendQueueCount");
    event.setReturnValue(&return_value);
    if(const_cast<ITCPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITCPControl::enableKeepAlive(const nmm_bool& enable)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > enable_value(enable, false);
    NMM::Event event("ITCPControl::enableKeepAlive", &enable_value);
    if(const_cast<ITCPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  bool ITCPControl::getKeepAlive() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<bool > return_value;
    NMM::Event event("ITCPControl::getKeepAlive");
    event.setReturnValue(&return_value);
    if(const_cast<ITCPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ITCPControl::ITCPControl_type = "ITCPControl";
  NMM::TConstructor<ITCPControl > __ITCPControl_prototype("ITCPControl","NMM");


  const char* ITCPControl::enableNoDelay_event = "ITCPControl::enableNoDelay";
  const char* ITCPControl::isDelayEnabled_event = "ITCPControl::isDelayEnabled";
  const char* ITCPControl::setMaxSegment_event = "ITCPControl::setMaxSegment";
  const char* ITCPControl::getMaxSegment_event = "ITCPControl::getMaxSegment";
  const char* ITCPControl::enablePartialFrames_event = "ITCPControl::enablePartialFrames";
  const char* ITCPControl::isPartialFramesEnabled_event = "ITCPControl::isPartialFramesEnabled";
  const char* ITCPControl::getUnreadQueueCount_event = "ITCPControl::getUnreadQueueCount";
  const char* ITCPControl::allUrgentDataSend_event = "ITCPControl::allUrgentDataSend";
  const char* ITCPControl::getUnsendQueueCount_event = "ITCPControl::getUnsendQueueCount";
  const char* ITCPControl::enableKeepAlive_event = "ITCPControl::enableKeepAlive";
  const char* ITCPControl::getKeepAlive_event = "ITCPControl::getKeepAlive";


}


#include "nmm/nmm_suffix.hpp"

