/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IUDPControl.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IUDPControl::used() {} 
  
  
  void IUDPControl::resetStatistics()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IUDPControl::resetStatistics");
    if(const_cast<IUDPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_uint32 IUDPControl::getTotalSentBytes() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("IUDPControl::getTotalSentBytes");
    event.setReturnValue(&return_value);
    if(const_cast<IUDPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 IUDPControl::getTotalReceivedBytes() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("IUDPControl::getTotalReceivedBytes");
    event.setReturnValue(&return_value);
    if(const_cast<IUDPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 IUDPControl::getTotalSentPackets() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("IUDPControl::getTotalSentPackets");
    event.setReturnValue(&return_value);
    if(const_cast<IUDPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 IUDPControl::getTotalReceivedPackets() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("IUDPControl::getTotalReceivedPackets");
    event.setReturnValue(&return_value);
    if(const_cast<IUDPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 IUDPControl::getTotalDiscardedPackets() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("IUDPControl::getTotalDiscardedPackets");
    event.setReturnValue(&return_value);
    if(const_cast<IUDPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 IUDPControl::getTotalLostPackets() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("IUDPControl::getTotalLostPackets");
    event.setReturnValue(&return_value);
    if(const_cast<IUDPControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IUDPControl::IUDPControl_type = "IUDPControl";
  NMM::TConstructor<IUDPControl > __IUDPControl_prototype("IUDPControl","NMM");


  const char* IUDPControl::resetStatistics_event = "IUDPControl::resetStatistics";
  const char* IUDPControl::getTotalSentBytes_event = "IUDPControl::getTotalSentBytes";
  const char* IUDPControl::getTotalReceivedBytes_event = "IUDPControl::getTotalReceivedBytes";
  const char* IUDPControl::getTotalSentPackets_event = "IUDPControl::getTotalSentPackets";
  const char* IUDPControl::getTotalReceivedPackets_event = "IUDPControl::getTotalReceivedPackets";
  const char* IUDPControl::getTotalDiscardedPackets_event = "IUDPControl::getTotalDiscardedPackets";
  const char* IUDPControl::getTotalLostPackets_event = "IUDPControl::getTotalLostPackets";


}


#include "nmm/nmm_suffix.hpp"

