/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "LiveThread.hpp"
#include "BufferedSink.hpp"

#include "nmm/utils/NamedObject.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  LiveThread::LiveThread(UsageEnvironment& env, MediaSource& source, MediaSink& sink)
    : __env(env),
      __source(source),
      __sink(sink),
      __live_stop(0)
  {}
  
  void LiveThread::start() {
    GLOBAL_DEBUG_STREAM("Starting LIVE thread" << endl);
    __live_stop = 0;
    activateThread();
  }
  
  void LiveThread::stop() {    
    GLOBAL_DEBUG_STREAM("Joining LIVE thread" << endl);
    
    __live_stop = 1;
    __sink.stopPlaying();
    
    deactivateThread();
  }
  
  void LiveThread::run() {
    // XXX: This is ugly! LIVE is an object oriented library
    // XXX: so you should pass an object instead of a function...
    __sink.startPlaying(__source, &afterPlaying, this);
    __env.taskScheduler().doEventLoop(&__live_stop);
  }
  
  void LiveThread::afterPlaying(void* /*obj*/) {
    // XXX: fw: Notify the RTPStrategy ?
  }
  
}

#include "nmm/nmm_suffix.hpp"
