/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_LIVE_THREAD_HPP
#define NMM_LIVE_THREAD_HPP NMM_LIVE_THREAD_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/utils/thread/ThreadedObject.hpp"

#include <live/UsageEnvironment.hh>

class MediaSource;
class MediaSink;

namespace NMM {
  
  /** LIVE event handling.
   *
   * This class encapsulates the event handling thread of the LIVE
   * library.
   */
  class LiveThread :
    public ThreadedObject 
  {
  public:
    /** Constructor.
     *
     * @param env LIVE Usage environment
     * @param source The media source
     * @param sink The media sink
     */
    LiveThread(UsageEnvironment& env,
               MediaSource& source, MediaSink& sink);
    
    /** This method starts the event handling thread.
     */
    void start();
    
    /** This method stops the event handling thread.
     */
    void stop();
    
  protected:
    /** Main thread method.
     *
     * This method runs the LIVE main loop
     */
    virtual void run();
    
  private:
    /** Stop handler for LIVE.
     *
     * This method is passed as a handler to LIVE which is called when
     * the thread is stopped.
     *
     * @param obj Pointer to this object
     */
    static void afterPlaying(void* obj);
    
  private:
    /** The LIVE usage environment. */
    UsageEnvironment& __env;
    
    /* The LIVE media source. */
    MediaSource& __source;
    
    /* The LIVE media sink. */
    MediaSink& __sink;
    
    /* Stop flag for LIVE*/
    char __live_stop;
  };
  
}

#include "nmm/nmm_suffix.hpp"

#endif
