/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ONetStream.hpp"

#ifndef BUILD_PPC
  #include <algorithm> //required for std::swap

  #define ByteSwap(x) ByteSwap3((unsigned char *) &x,sizeof(x))

  void ByteSwap3(unsigned char * b, int n)
  {
     register int i = 0;
     register int j = n-1;
     while (i<j)
     {
        std::swap(b[i], b[j]);
        i++, j--;
     }
  }

#else 
  #define ByteSwap(x) (x)
#endif


#include "nmm/nmm_prefix.hpp"

namespace NMM {

  ONetStream::ONetStream()
    : OStream(TypeInfo("ONetStream", "NMM")),
      __sock(0),
      __stream(0),
      __write_counter(0)
  { /*__stream = &cerr;*/ }

  ONetStream::ONetStream(const ONetStream& other)
    : OStream(other),
      __sock(other.__sock),
      __stream(other.__stream),
      __write_counter(0)
  { /*__stream = &cerr;*/ }
  
  ONetStream::ONetStream(const TypeInfo& ti)
    : OStream(ti),
      __sock(0),
      __stream(0),
      __write_counter(0)
  { /*__stream = &cerr;*/ }
  
  ONetStream::~ONetStream() {}
  
   ONetStream& ONetStream::operator<<(const nmm_uint16& value) {
    nmm_uint16 tmp_value = value;
    ByteSwap(tmp_value); 
    writeValue( &tmp_value, SIZE_OF_NMM_UINT16);
    return *this;
  }
  
   ONetStream& ONetStream::operator<<(const nmm_uint32& value) {
    nmm_uint32 tmp_value = value;
    ByteSwap(tmp_value); 
    writeValue(&tmp_value, SIZE_OF_NMM_UINT32);
    return *this;
  }
  
   ONetStream& ONetStream::operator<<(const nmm_uint64& value) {
    nmm_uint64 tmp_value = value;
    ByteSwap(tmp_value); 
    writeValue(&tmp_value, SIZE_OF_NMM_UINT64);
    return *this;
  }
  
   ONetStream& ONetStream::operator<<(const nmm_int16&value) {
    nmm_int16 tmp_value = value;
    ByteSwap(tmp_value); 
    writeValue(&tmp_value, SIZE_OF_NMM_INT16);
    return *this;
  }
  
   ONetStream& ONetStream::operator<<(const nmm_int32& value) {
    nmm_int32 tmp_value = value;
    ByteSwap(tmp_value); 
    writeValue(&tmp_value, SIZE_OF_NMM_UINT32);
    return *this;
  }

   ONetStream& ONetStream::operator<<(const nmm_int64& value) {
    nmm_int64 tmp_value = value;
    ByteSwap(tmp_value); 
    writeValue(&tmp_value, SIZE_OF_NMM_INT64);
    return *this;
  }
  
   ONetStream& ONetStream::operator<<(const nmm_bool& value) {
    char tmp;
    if ( value ) {
      tmp = 0x1;
    } else {
      tmp = 0x0;
    }

    writeValue(&tmp,1);
    return *this;
  }

   ONetStream& ONetStream::operator<<(const nmm_double& value) {
    nmm_double tmp_value = value;
    ByteSwap(tmp_value);
    writeValue(&tmp_value, SIZE_OF_NMM_DOUBLE);
    return *this;
  }
   ONetStream& ONetStream::operator<<(const nmm_float& value) {
    nmm_float tmp_value = value;
    ByteSwap(tmp_value);
    writeValue(&tmp_value, SIZE_OF_NMM_FLOAT);
    return *this;
  };
   ONetStream& ONetStream::operator<<(const nmm_char& value) {
    writeValue(&value, 1);
    return *this;
  }

   ONetStream& ONetStream::operator<<(const nmm_uchar& value) {
    writeValue(&value, 1);
    return *this;
  }

   ONetStream& ONetStream::operator<<(const string& value) {
    nmm_uint32 size = static_cast<nmm_uint32>(value.size());
    (*this) << size;
    
    if (value.size() == 0) {
      return *this;
    }
    writeValue(value.c_str(), value.size());
    return *this;
  }
  
  void ONetStream::writeBuffer(const char* value, nmm_uint32 size) {
    writeValue(value, size);
  }
  
  void ONetStream::writeBuffer(const unsigned char* value, nmm_uint32 size) {
    writeValue(value, size);
  }
  
}

#include "nmm/nmm_suffix.hpp"
