/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "RTPAddress.hpp"
#include "IPAddress.hpp"

#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  RTPAddress::RTPAddress(const NMMReference& reference /*= 0*/) throw()
    : ConnectorAddress(getStaticTypeInfo(), TSAddress::LOCAL | TSAddress::SYSTEM | TSAddress::NETWORK, "RTPStrategy", reference),
      __rtp_port(0),
      __rtcp_port(0),
      __ip_address(NULL)
  {
    insertArgument(&__rtp_port);
    insertArgument(&__rtcp_port);
    insertArgument(&__ip_address);
  }
  
  RTPAddress::RTPAddress(const IPAddress& addr,
                         unsigned int rtp_port, unsigned int rtcp_port,
                         const NMMReference& reference /*= 0*/) throw()
    : ConnectorAddress(getStaticTypeInfo(), TSAddress::LOCAL | TSAddress::SYSTEM | TSAddress::NETWORK, "RTPStrategy", reference),
      __rtp_port(rtp_port),
      __rtcp_port(rtcp_port),
      __ip_address(NULL)
  {
    setIPAddress(addr);
    
    insertArgument(&__rtp_port);
    insertArgument(&__rtcp_port);
    insertArgument(&__ip_address);
  }
  
  RTPAddress::RTPAddress(const RTPAddress& addr) throw()
    : ConnectorAddress(addr),
      __rtp_port(addr.__rtp_port),
      __rtcp_port(addr.__rtcp_port),
      __ip_address(NULL)
  {
    setIPAddress(addr.getIPAddress());
    
    insertArgument(&__rtp_port);
    insertArgument(&__rtcp_port);
    insertArgument(&__ip_address);
  }
  
  void RTPAddress::setIPAddress(const IPAddress& ip_address) throw() {
    if (__ip_address.getRValue()) {
      delete __ip_address.getRValue();
    }
    
    __ip_address = ip_address.copy();
  }
  
  
  template<>
  class TConstructor<RTPAddress> : public ValueConstructor {
  public:
    TConstructor(const TypeInfo& t_info)
      : ValueConstructor(t_info)
    { }
    virtual RTPAddress*          createValue()          const {return new RTPAddress();}
    virtual TValue<RTPAddress>*  createTValue()         const {return new TValue<RTPAddress>();}
    
    virtual TValue<TSAddress*>*  createTPointerValue()  const {return new TValue<TSAddress*>();}
    
    virtual TConstructor<RTPAddress>* copy()  const {return new TConstructor(*this); }
  private:
    string  __type_name;
    string  __namespace;
  };
  
  TConstructor<RTPAddress> __rtp_address(RTPAddress::getStaticTypeInfo());
  TypeInfo RTPAddress::getStaticTypeInfo() throw() {
    return TypeInfo("RTPAddress", "NMM");
  }
  
}

#include "nmm/nmm_suffix.hpp"
