/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
 

#include "RTPPayloadtypeFormatMatching.hpp"

namespace NMM {
  
  const char* RTPPayloadtypeFormatMatching::getMimeSubtypeFromPayloadType(int payload_type){
    switch (payload_type){
    case 0:
      return "pcmu";
      break;
    case 8:
      return "pcma";
      break;
    case 10:
    case 11:
      return "l16";
      break;
    case 14:
      return "mpa";
      break;
    case 26:
      return "jpeg";
      break;
    case 32:
      return "mpv";
      break;
    case 34:
      return "h263";
      break;
    case 97:
      return "amr_nb";
      break;      
    case 98:
      return "amr_wb";
      break;            
    default:
      return "unknown";
    }
  }

  Format* RTPPayloadtypeFormatMatching::getFormatFromPayloadType(int payload_type) {
    Format* format = 0;
    
    switch (payload_type) {
      
    case 0:
      format = new Format(Format::audio_raw_type);
      format->addIntParamValue(Format::channels_param, 1);
      format->addIntParamValue(Format::bitpersample_param, 8);
      format->addIntParamValue(Format::sample_rate_param, 8000);
      format->addStringParamValue(Format::encoding_param, "mu_law");
      format->addStringParamValue(Format::sign_param, "signed");
      break;
      
      /* Not supported yet
       * Perhaps you have to add some more params to the format
       case 3:
       format = new Format(Format::audio_gsm_type);
       format->addIntParamValue(Format::channels_param, 1);
       format->addIntParamValue(Format::sample_rate_param, 8000);
       break;
      */

    case 8:
      format = new Format(Format::audio_raw_type);
      format->addIntParamValue(Format::channels_param, 1);
      format->addIntParamValue(Format::bitpersample_param, 8);
      format->addIntParamValue(Format::sample_rate_param, 8000);
      format->addStringParamValue(Format::encoding_param, "a_law");
      format->addStringParamValue(Format::sign_param, "signed");
      break;    
    
    case 10:
      format = new Format(Format::audio_raw_type);
      format->addIntParamValue(Format::channels_param, 2);
      format->addIntParamValue(Format::bitpersample_param, 16);
      format->addIntParamValue(Format::sample_rate_param, 44100);
      format->addStringParamValue(Format::encoding_param, "linear");
      format->addStringParamValue(Format::sign_param, "signed");
      break;
      
    case 11:
      format = new Format(Format::audio_raw_type);
      format->addIntParamValue(Format::channels_param, 1);
      format->addIntParamValue(Format::bitpersample_param, 16);
      format->addIntParamValue(Format::sample_rate_param, 44100);
      format->addStringParamValue(Format::encoding_param, "linear");
      format->addStringParamValue(Format::sign_param, "signed");
      break;
      
    case 14:
      format = new Format(Format::audio_mpeg_type);
      format->addIntParamValue(Format::layer_param, 2);
      format->addIntParamValue(Format::layer_param, 3);
      format->addIntParamValue(Format::audio_codec_param, 0x55);
      //format->addWildcard(Format::sample_rate_param);
      //format->addWildcard(Format::channels_param);
      //format->addWildcard(Format::bitpersample_param);
      //format->addWildcard(Format::bitrate_param);
      break;
      
    case 26:
      format = new Format(Format::video_jpeg_type);
      format->addWildcard(Format::x_resolution_param);
      format->addWildcard(Format::y_resolution_param);
      format->addWildcard(Format::ratio_x_param);
      format->addWildcard(Format::ratio_y_param);
      format->addWildcard(Format::framerate_param);
      format->addWildcard(Format::bitrate_param);
      break;
      
    case 32:
      format = new Format(Format::video_mpeg_type);
      format->addIntParamValue(Format::version_param, 1);
      format->addIntParamValue(Format::version_param, 2);
      //format->addWildcard(Format::x_resolution_param);
      //format->addWildcard(Format::y_resolution_param);
      //format->addWildcard(Format::ratio_x_param);
      //format->addWildcard(Format::ratio_y_param);
      //format->addWildcard(Format::framerate_param);
      break;
      
    case 34:
      format = new Format(Format::video_h263plus_type);
      format->addWildcard(Format::x_resolution_param);
      format->addWildcard(Format::y_resolution_param);
      format->addWildcard(Format::ratio_x_param);
      format->addWildcard(Format::ratio_y_param);
      format->addWildcard(Format::framerate_param);
      format->addWildcard(Format::bitrate_param);
      break;

    case 96:
      format = new Format(Format::video_mpeg4_type);
      format->addWildcard(Format::x_resolution_param);
      format->addWildcard(Format::y_resolution_param);
      format->addWildcard(Format::ratio_x_param);
      format->addWildcard(Format::ratio_y_param);
      format->addWildcard(Format::framerate_param);
      format->addWildcard(Format::bitrate_param);
      format->addStringParamValue(Format::compressor_param, "DIVX");
      format->addStringParamValue(Format::compressor_param, "DX50");
      
    case 97:
      format =  new Format(Format::audio_amr_type);
      format->addIntParamValue(Format::channels_param, 1);
      format->addIntParamValue(Format::sample_rate_param, 8000);
      format->addIntParamValue(Format::bitpersample_param, 16);
      format->addIntParamValue(Format::bitrate_param, 4750);
      format->addIntParamValue(Format::bitrate_param, 5150);
      format->addIntParamValue(Format::bitrate_param, 5900);
      format->addIntParamValue(Format::bitrate_param, 6700);
      format->addIntParamValue(Format::bitrate_param, 7400);
      format->addIntParamValue(Format::bitrate_param, 7950);
      format->addIntParamValue(Format::bitrate_param, 10200);
      format->addIntParamValue(Format::bitrate_param, 12200);
      format->addStringParamValue(Format::encoding_param, "amr_nb");
      
    case 98:
      format =  new Format(Format::audio_amr_type);
      format->addIntParamValue(Format::channels_param, 1);
      format->addIntParamValue(Format::sample_rate_param, 16000);
      format->addIntParamValue(Format::bitpersample_param, 16);
      //format->addWildcard(Format::bitrate_param);
      format->addIntParamValue(Format::bitrate_param, 6600);
      format->addIntParamValue(Format::bitrate_param, 8850);
      format->addIntParamValue(Format::bitrate_param, 12650);
      format->addIntParamValue(Format::bitrate_param, 14250);
      format->addIntParamValue(Format::bitrate_param, 15850);
      format->addIntParamValue(Format::bitrate_param, 18250);
      format->addIntParamValue(Format::bitrate_param, 19850);
      format->addIntParamValue(Format::bitrate_param, 23050);
      format->addIntParamValue(Format::bitrate_param, 23850);
      format->addStringParamValue(Format::encoding_param, "amr_wb");      

      /*format = new Format(Format::video_mpeg4_type);
        format->addWildcard(Format::x_resolution_param);
        format->addWildcard(Format::y_resolution_param);
        format->addWildcard(Format::ratio_x_param);
        format->addWildcard(Format::ratio_y_param);
        format->addWildcard(Format::framerate_param);
        format->addWildcard(Format::bitrate_param);
        format->addStringParamValue(Format::compressor_param, "DIVX");*/
      break;

    default:
      format = new Format(Format::wild_wild_type);
      
    }
    
    return format;
  }

  int RTPPayloadtypeFormatMatching::getPayloadTypeFromFormat(Format* format) {
    string format_type = format->getFormatType();
    try{
      if(format_type==Format::audio_raw_type){
        string encoding = format->getStringValue(Format::encoding_param);
        int channels = format->getIntValue(Format::channels_param);
        int bitpersample = format->getIntValue(Format::bitpersample_param);
        if(channels==1 && bitpersample==8 && format->isParamValueIncluded(Format::sample_rate_param, 8000)){
          if(encoding=="mu_law"){
            // audio/mu_law
            return 0;
          }else if(encoding=="a_law"){
            // audio/a_law
            return 8;
          }else{
            return -1;
          } 
        }else if(bitpersample==16 && encoding=="linear" && format->isParamValueIncluded(Format::sample_rate_param, 44100)){
          if(channels==1){
            // audio/l16
            return 11;
          }else if(channels==2){
            // audio/l16
            return 10;
          }else{
            return -1;
          }
        }

      }else if(format_type==Format::audio_mpeg_type){
        int layer=0;
        if(format->isParameterSpecified(Format::layer_param)) 
          layer=format->getIntValue(Format::layer_param);
        int audio_codec = 0;
        if(format->isParameterSpecified(Format::audio_codec_param)) 
          audio_codec = format->getIntValue(Format::audio_codec_param);
        if((layer==2 || layer==3) && audio_codec==0x55){
          // audio/mpeg
          return 14;
        }else{
          return -1;
        }
        
      }else if(format_type==Format::audio_amr_type){
        string encoding = format->getStringValue(Format::encoding_param);
        int channels = format->getIntValue(Format::channels_param);
        int bitpersample = format->getIntValue(Format::bitpersample_param);
        int sample_rate = format->getIntValue(Format::sample_rate_param);
        if ((channels == 1) && (bitpersample == 16) && (sample_rate == 8000) && (encoding == "amr_nb"))   
          return 97;
        else if ((channels == 1) && (bitpersample == 16) && (sample_rate == 16000) && (encoding == "amr_wb"))
          return 98;
        else
          return -1;
        
      }else if(format_type==Format::video_jpeg_type){
        // video/jpeg
        return 26;
    
      }else if(format_type==Format::video_mpeg_type){
        int version = format->getIntValue(Format::version_param);
        if(version==1 || version==2){
          // video/mpeg
          return 32;
        }else{
          return -1;
        }
    
      }else if(format_type==Format::video_h263plus_type){
        // audio/h263
        return 34;
    
      }else if(format_type==Format::Format::video_mpeg4_type){
        string compressor = format->getStringValue(Format::compressor_param);
        if(compressor=="DIVX" || compressor=="DX50"){
          return 96;
        }else{
          return -1;
        }
      }
    }catch(ParameterException e){
      GLOBAL_ERROR_STREAM(e << endl);
    }
    
    return -1;
  }

} // namespace NMM
