/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 * 
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "SerializeINetAddr.hpp"

#include "nmm/comm/serialize/TValueConstructor.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"

#include "nmm/utils/net/INetAddr.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  IValueStream& operator>>(IValueStream& s, INetAddr& addr) {
    if((s.getCurrentType() != "INetAddr") || (s.getCurrentNamespace() != "NMM")) {
      GLOBAL_ERROR_STREAM("Invalid Type! Can not assign " << s.getCurrentType()<< " to an NMM::INetAddr. " << endl);
      throw InvalidTypeException(TypeInfo("NMM::INetAddr", "NMM"), s.getCurrentTypeInfo());
    }
    
    if(!s.goDown()) {
      GLOBAL_ERROR_STREAM("Could not access the interval values " << endl);
      s.nextElement();
      return s;
    }
    
    std::string protocol;
    nmm_uint32 port;
    nmm_uint32 address;
    s >> protocol;
    s >> port;
    s >> address;
    addr.setTransportProtocol(protocol);
    addr.setPort(port);
    addr.setAddress(address);
    
    s.goUp();
    
    /* update attributes after deserialization */
    addr.updateAttributes();
    
    return s;
  }
  
  OValueStream& operator<<(OValueStream& s, const INetAddr& addr) {
    s.beginType("INetAddr", "NMM");
    
    std::string protocol = addr.getTransportProtocol();
    nmm_uint32 port = addr.getPort();
    nmm_uint32 address = addr.getIpAddress();
    s << protocol;
    s << port;
    s << address;
    
    s.endType();
    return s;
  }
  
  TValueConstructor<INetAddr>   __inet_addr(TypeInfo("INetAddr", "NMM"));
  
}

#include "nmm/nmm_suffix.hpp"
