/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "TCPAcceptor.hpp"
#include "TCPAddress.hpp"
#include "TCPStrategy.hpp"

#include "nmm/comm/ProcessUserInfo.hpp"
#include "nmm/comm/TSFactory.hpp"

#include "nmm/utils/net/NetReactor.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  TCPAcceptor::TCPAcceptor(unsigned int port) {
    addr.setPort(port);
    open(addr);
    SNetReactor::getInstance().registerSocket( getSocket(), this);

    /*Begin> Register the default address for tcp connections */
    TCPAddress address;
    address.getINetAddr().setPort(port);
    address.getINetAddr().setAddress(SProcessUserInfo::getInstance().getHostname().c_str());
    
    STSFactory::getInstance().setDefaultAddress("TCPStrategy", &address);
    /*End< */
  }

  TCPAcceptor::TCPAcceptor(unsigned int begin_port, unsigned int end_port ) {
    addr.setPort(begin_port);
    tryOpen(addr, begin_port, end_port);
    SNetReactor::getInstance().registerSocket( getSocket(), this);

    /*Begin> Register the default address for tcp connections */
    TCPAddress address;
    address.getINetAddr().setPort(addr.getPort());
    address.getINetAddr().setAddress(SProcessUserInfo::getInstance().getHostname().c_str());

    STSFactory::getInstance().setDefaultAddress("TCPStrategy", &address);
    /*End< */
  }
  
  TCPAcceptor::~TCPAcceptor() {
    SNetReactor::getInstance().unregisterSocket( getSocket() );
    close();
  }

  NetHandler::NetResult TCPAcceptor::receiveData(SocketHandle) {
    GLOBAL_MESSAGE_STREAM("TCPConnection established" << endl);
  
    /* create a new TCPStrategy */
    TCPStrategy* strategy = dynamic_cast<TCPStrategy*>(STSFactory::getInstance().createTransportStrategy("TCPStrategy", 0));
    
    /* This object receives the first event */
    strategy->setEventReceiver(this);

    /* accept the connection */
    accept(strategy->getStreamSocket());
  
    /* set the TCP_NODELAY flag */
    strategy->enableNoDelay(true);
  
    /* register the Socket by the NetReactor */
    SNetReactor::getInstance().internalRegisterSocket( strategy->getStreamSocket().getSocket(), strategy);
    return VOID;
  }

}

#include "nmm/nmm_suffix.hpp"
