/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 * 
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "TCPAddress.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  TCPAddress::TCPAddress (const NMMReference& reference)
    : TSAddress(reference, "TCPStrategy", TypeInfo("TCPAddress", "NMM"), TSAddress::LOCAL|TSAddress::SYSTEM|TSAddress::NETWORK)
  {
    insertArgument(&__net_addr);
  }

  TCPAddress::TCPAddress (const string& name, int port, const NMMReference& reference)
    : TSAddress(reference, "TCPStrategy", TypeInfo("TCPAddress", "NMM"), TSAddress::LOCAL|TSAddress::SYSTEM|TSAddress::NETWORK)
  {
    insertArgument(&__net_addr);
    setPort(port);
    setHostname(name);
  }

  TCPAddress::TCPAddress (const INetAddr& net_addr, const NMMReference& reference)
    : TSAddress(reference, "TCPStrategy", TypeInfo("TCPAddress", "NMM"), TSAddress::LOCAL|TSAddress::SYSTEM|TSAddress::NETWORK),
      __net_addr(net_addr)
  {
    insertArgument(&__net_addr);
  }

  TCPAddress::TCPAddress (const TCPAddress& addr)
    : TSAddress(addr)
  {
    insertArgument(&__net_addr);
    *this = addr;
  }

  void TCPAddress::setPort(int port) {
    __net_addr.getRValue().setPort(port);
    TSAddress::setPort(port);
  }

  void TCPAddress::setHostname(const string& host) {
    __net_addr.getRValue().setAddress(host.c_str());
    TSAddress::setHostname(host);
  }  
  
  template<>
  class TConstructor<TCPAddress> : public ValueConstructor {
  public:
    TConstructor(const TypeInfo& t_info)
      : ValueConstructor(t_info)
    { }
    virtual TCPAddress*          createValue()          const {return new TCPAddress();}
    virtual TValue<TCPAddress>*  createTValue()         const {return new TValue<TCPAddress>();}

    virtual TValue<TSAddress*>*  createTPointerValue()  const {return new TValue<TSAddress*>();}

    virtual TConstructor<TCPAddress>* copy()  const {return new TConstructor(*this); }
  private:
    string  __type_name;
    string  __namespace;
  };
  
  TConstructor<TCPAddress>      __tcp_address(TypeInfo("TCPAddress", "NMM"));
  
}

#include "nmm/nmm_suffix.hpp"
