/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 * 
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "UDPAddress.hpp"

#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  UDPAddress::UDPAddress (const NMMReference& reference)
    : ConnectorAddress(TypeInfo("UDPAddress", "NMM"), TSAddress::LOCAL|TSAddress::SYSTEM|TSAddress::NETWORK,
                       "UDPStrategy", reference)
  {
    getINetAddr().setTransportProtocol("udp");
    
    insertArgument(&__net_addr);
  }
  
  UDPAddress::UDPAddress (const string& name, int port, const NMMReference& reference)
    : ConnectorAddress(TypeInfo("UDPAddress", "NMM"), TSAddress::LOCAL|TSAddress::SYSTEM|TSAddress::NETWORK,
                       "UDPStrategy", reference)
  {
    getINetAddr().setTransportProtocol("udp");
    setPort(port);
    setHostname(name);
    
    insertArgument(&__net_addr);
  }
  
  UDPAddress::UDPAddress (const UDPAddress& addr)
    : ConnectorAddress(addr)
  {
    *this = addr;
    insertArgument(&__net_addr);
  }
  
  void UDPAddress::setPort(int port)  {
    __net_addr.getRValue().setPort(port);
    TSAddress::setPort(port);
  }
  
  void UDPAddress::setHostname(const string& host) {
    __net_addr.getRValue().setAddress(host.c_str());
    TSAddress::setHostname(host);
  }
  
  
  template<>
  class TConstructor<UDPAddress> : public ValueConstructor {
  public:
    TConstructor(const TypeInfo& t_info)
      : ValueConstructor(t_info)
    {}
    virtual UDPAddress*          createValue()          const {return new UDPAddress();}
    virtual TValue<UDPAddress>*  createTValue()         const {return new TValue<UDPAddress>();}
    
    virtual TValue<TSAddress*>*  createTPointerValue()  const {return new TValue<TSAddress*>();}
    
    virtual TConstructor<UDPAddress>* copy()  const {return new TConstructor(*this); }
  private:
    string  __type_name;
    string  __namespace;
  };
  
  TConstructor<UDPAddress> __udp_address(TypeInfo("UDPAddress", "NMM"));
  
}

#include "nmm/nmm_suffix.hpp"
