/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "UnbufferedINetStream.hpp"

#include <cassert>

#include "nmm/utils/net/Sock.hpp"

#include "nmm/comm/serialize/TConstructor.hpp"

#include <cerrno>

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  UnbufferedINetStream::UnbufferedINetStream()
    : INetStream(TypeInfo("UnbufferedINetStream", "NMM"))
  {}
  
  UnbufferedINetStream::UnbufferedINetStream(const TypeInfo& ti)
    : INetStream(ti)
  {}
  
  UnbufferedINetStream::~UnbufferedINetStream()
  {}
  
  UnbufferedINetStream* UnbufferedINetStream::copy() const {
    return new UnbufferedINetStream(*this);
  }
  
  void UnbufferedINetStream::readValue(void* value, size_t size) {
    /* read bytes from the socket */
    assert(getSock());
    ssize_t retval = getSock()->recv(static_cast<char*>(value), size);
    if(retval == 0) {
      throw ConnectionClosed(EPIPE);
    }
    
    /* count bytes */
    countReadBytes(retval);
    
    /* if not all values are read, call this method again to read the rest */
    if(static_cast<size_t>(retval) != size) {
      char* new_offset = static_cast<char*>(value) + retval;
      readValue(new_offset, size - retval);
    }
  }
  
  template<>
  class TConstructor<UnbufferedINetStream> : public ValueConstructor {
  public:
    TConstructor(const TypeInfo& t_info)
      : ValueConstructor(t_info)
    {}
    virtual UnbufferedINetStream*          createValue()          const {return new UnbufferedINetStream();}
    virtual TValue<UnbufferedINetStream>*  createTValue()         const {return new TValue<UnbufferedINetStream>();}
    
    virtual TValue<INetStream*>*  createTPointerValue()  const {return new TValue<INetStream*>();}
    
    virtual TConstructor<UnbufferedINetStream>* copy()  const {return new TConstructor(*this); }
  private:
    string  __type_name;
    string  __namespace;
  };
  
  TConstructor<UnbufferedINetStream> __nmm__unbuffered_i_net_stream(TypeInfo("UnbufferedINetStream", "NMM"));
  
}

#include "nmm/nmm_suffix.hpp"
