/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "UnbufferedONetStream.hpp"

#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/utils/net/Sock.hpp"

#include <cassert>

#include <cerrno>

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  UnbufferedONetStream::UnbufferedONetStream()
    : ONetStream(TypeInfo("UnbufferedONetStream", "NMM"))
  {}
  
  UnbufferedONetStream::UnbufferedONetStream(const TypeInfo& ti)
    : ONetStream(ti)
  {}
  
  UnbufferedONetStream::~UnbufferedONetStream()
  {}
  
  UnbufferedONetStream* UnbufferedONetStream::copy() const {
    return new UnbufferedONetStream(*this);
  }
  
  void UnbufferedONetStream::writeValue(const void* value, size_t size) {
    /* write bytes to the socket */
    assert(getSock());
    ssize_t retval = getSock()->send(static_cast<const char*>(value), size);
    if(retval == 0) {
      throw ConnectionClosed(EPIPE);
    }
    
    /* count bytes */
    countWriteBytes(retval);
    
    /* if not all values are written, write the rest */
    if(static_cast<size_t>(retval) != size) {
      const char* new_offset = static_cast<const char*>(value) + retval;
      writeValue(new_offset, size - retval);
    }
  }
  
  template<>
  class TConstructor<UnbufferedONetStream> : public ValueConstructor {
  public:
    TConstructor(const TypeInfo& t_info)
      : ValueConstructor(t_info)
    {}
    virtual UnbufferedONetStream*          createValue()          const {return new UnbufferedONetStream();}
    virtual TValue<UnbufferedONetStream>*  createTValue()         const {return new TValue<UnbufferedONetStream>();}
    
    virtual TValue<ONetStream*>*  createTPointerValue()  const {return new TValue<ONetStream*>();}
    
    virtual TConstructor<UnbufferedONetStream>* copy()  const {return new TConstructor(*this); }
  private:
    string  __type_name;
    string  __namespace;
  };
  
  TConstructor<UnbufferedONetStream> __nmm__unbuffered_o_net_stream(TypeInfo("UnbufferedONetStream", "NMM"));
  
}

#include "nmm/nmm_suffix.hpp"
