/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_UNBUFFERED_O_NET_STREAM_HPP
#define NMM_UNBUFFERED_O_NET_STREAM_HPP NMM_UNBUFFERED_O_NET_STREAM_HPP

#include "nmm/nmm_prefix.hpp"

#include "ONetStream.hpp"

#include "visibility.hpp"

namespace NMM {

  /** Unbuffered network output stream.
   * This class is a network output stream which directly writes to a socket.
   */
  class DLLEXPORT UnbufferedONetStream
    : public ONetStream
  {
  public:
    /** @name Constructor & Destructor
     */
    //@{
    /** Constructor.
     */
    UnbufferedONetStream();

    /** Virtual destructor.
     */
    virtual ~UnbufferedONetStream();

    /** Virtual copy constructor.
     */
    virtual UnbufferedONetStream* copy() const;
    //@}

  protected:
    /** Subclass constructor.
     * This constructor must be used by subclasses when initializing the base class.
     * Every subclass must also define this subclass constructor as a protected constructor.
     *
     * @param ti Type information for this object
     */
    UnbufferedONetStream(const TypeInfo& ti);

    /** Socket write method.
     * This method performs the socket write operation. It is invoked by the implementation
     * of the serialization operators.
     *
     * This implementation directly writes data to the socket.
     *
     * @param data Binary data to be serialized
     * @param size Size of binary data to serialize (in bytes)
     */
    virtual void writeValue(const void* data, size_t size);
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
