/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_UNICAST_IP_ADDRESS_HPP
#define NMM_UNICAST_IP_ADDRESS_HPP NMM_UNICAST_IP_ADDRESS_HPP

#include "nmm/nmm_prefix.hpp"

#include "IPAddress.hpp"

#include "visibility.hpp"

namespace NMM {

  /** A unicast IP address.
   * An IP address which is a part of a transport strategy address.
   * This class only represents unicast IP addresses.
   *
   * @see MulticastIPAddress
   */
  class DLLEXPORT UnicastIPAddress : public IPAddress {
  public:
    /** @name Constructor & Destructor
     */
    //@{
    /** Constructor.
     */
    UnicastIPAddress() throw();

    /** Constructor with host name.
     * This constructor accepts a host name or a string representation of an IP address.
     *
     * @param hostname Host name or IP address
     * @exception BindingException if the host was not found
     */
    UnicastIPAddress(const string& hostname)
      throw (const BindingException&);

    /** Copy constructor.
     *
     * @param addr Object to copy from
     */
    UnicastIPAddress(const UnicastIPAddress& addr) throw();

    /** Virtual copy constructor.
     *
     * @return A copy of this object which has the same type
     */
    inline virtual UnicastIPAddress* copy() const throw() {
      return new UnicastIPAddress(*this);
    }
    //@}

  public:
    /** This method returns the type information for this class.
     *
     * @return Type information for this class
     */
    static TypeInfo getStaticTypeInfo() throw();
  };
  
}

#include "nmm/nmm_suffix.hpp"

#endif
