/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "GenericMagicNumber.hpp"

#include "StdTypeInfo.hpp"
#include "TValueStreamConstructor.hpp"
#include "MagicNumberIValueStream.hpp"
#include "MagicNumberOValueStream.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  // Magic numbers for C++ POD types, std::string and char pointers.
  // These numbers are magic, so don't mess with them!

  const TypeInfo* GenericMagicNumber::magic_number_map[] =
    { 0, 0,
      &nmm_uint16_type_info, &nmm_int16_type_info,
      &nmm_uint32_type_info, &nmm_int32_type_info,
      &nmm_uint64_type_info, &nmm_int64_type_info,
      &nmm_bool_type_info,
      &nmm_double_type_info, &nmm_float_type_info,
      &nmm_char_type_info, &nmm_uchar_type_info,
      &char_ptr_type_info, &unsigned_char_ptr_type_info,
      &string_type_info
    };

  const int GenericMagicNumber::magic_number[] = {
    0, 1,
    2, 3,
    4, 5,
    6, 7,
    8,
    9, 10,
    11, 12,
    13, 14,
    15, 16
  };
  
  TValueStreamConstructor<MagicNumberIValueStream, MagicNumberOValueStream>
  magic_number_value_stream(MagicNumberIValueStream::__i_value_stream_type,
                            MagicNumberOValueStream::__o_value_stream_type);
  
}

#include "nmm/nmm_suffix.hpp"
