/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IStream.hpp"
#include "TValue.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  IStream::IStream(const TypeInfo& ti)
    : SerializedValue(ti),
      __own_partner_stream(false),
      __partner_stream(0)
  {
    registerAttributes();
  }
  
  IStream::IStream(const IStream& stream)
    : SerializedValue(stream),
      __own_partner_stream(false),
      __partner_stream(0)
  {
    registerAttributes();
    
    assign(stream);
  }
  
  IStream& IStream::operator=(const IStream& stream) {
    if(this != &stream) {
      assign(stream);
    }
    return *this;
  }
  
  void IStream::readLine(string& mybuf, char endchar) {
    char temp;
    readBuffer(&temp,1);
    while (temp!=endchar) {
      mybuf+=temp;
      readBuffer(&temp,1);
    }
  }
  
  void IStream::assign(const IStream& stream) {
    if(stream.__own_partner_stream) {
      setIPartnerStreamDeepCopy(stream.getIPartnerStream());
    } else {
      setIPartnerStream(stream.__partner_stream->getRValue());
    }
  }
  
  IStream::~IStream() {
    setIPartnerStream(0);
    
    delete __partner_stream;
  }
  
  void IStream::registerAttributes() {
    __partner_stream = new TValue<IStream*>(0);
    insertArgument(__partner_stream);
  }
  
  IStream* IStream::getIPartnerStream() {
    return __partner_stream->getRValue();
  }
  
  const IStream* IStream::getIPartnerStream() const {
    return __partner_stream->getRValue();
  }
  
  void IStream::setIPartnerStream(IStream* partner_stream) {
    if(getIPartnerStream() != partner_stream) {
      if(__own_partner_stream) {
        /* delete old partner stream */
        delete getIPartnerStream();
        __own_partner_stream = false;
      }
      
      /* shallow copy partner stream */
      __partner_stream->setValue(partner_stream);
    }
  }
  
  void IStream::setIPartnerStreamDeepCopy(const IStream* partner_stream) {
    if(!__own_partner_stream || getIPartnerStream() != partner_stream) {
      if(__own_partner_stream) {
        /* delete old partner stream */
        delete getIPartnerStream();
        __own_partner_stream = false;
      }
      
      if(partner_stream) {
        /* deep copy partner stream */
        __partner_stream->setValue(partner_stream->copy());
        __own_partner_stream = true;
      } else {
        __partner_stream->setValue(0);
      }
    }
  }
  
}

#include "nmm/nmm_suffix.hpp"
