/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "MagicNumberManager.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  MagicNumberManager::~MagicNumberManager() {
    for (HASH_MAP_STL::hash_map<int, TypeInfo*>::iterator it = __magic_number_map.begin(); it != __magic_number_map.end(); ++it)  {
      delete it->second;
    }
  }
  
  const TypeInfo* MagicNumberManager::getTypeInfo(int magic_number) {
    /* find the iterator of the magic number */
    HASH_MAP_STL::hash_map<int, TypeInfo*>::iterator it = __magic_number_map.find(magic_number);
    /* If the magic number is unknown return NULL */
    if (it == __magic_number_map.end())
      return 0;

    /* return the TypeInfo */
    return it->second;
  }
  
  int MagicNumberManager::getMagicNumber(const std::string& type_name, const std::string& ns) {
    /* find the iterator of the given TypeInfo */
    HASH_MAP_STL::hash_map<std::string, HASH_MAP_STL::hash_map<std::string, int, string_hash >, string_hash >::iterator it = __type_info_map.find(ns);

    /* If the namespace is unknown return an invalid magic number */
    if ( it == __type_info_map.end() )
      return -1;

    HASH_MAP_STL::hash_map<std::string, int, string_hash >::iterator it_2 = it->second.find(type_name);
    /* If the typename is unknown return an invalid magic number */
    if (it_2 == it->second.end())
      return -1;

    /* Return the magic number */
    return it_2->second;
  }
  
  void MagicNumberManager::addMagicNumber(const TypeInfo& type, int magic_number) {
    HASH_MAP_STL::hash_map<std::string, HASH_MAP_STL::hash_map<std::string, int, string_hash >, string_hash >::iterator it = __type_info_map.find(type.t_namespace);
    if (it == __type_info_map.end())
      it = __type_info_map.insert( std::pair<std::string, HASH_MAP_STL::hash_map<std::string, int, string_hash > >
                                   (type.t_namespace, HASH_MAP_STL::hash_map<std::string, int, string_hash>()) ).first;
    
    it->second.insert(std::pair<std::string, int>(type.t_name, magic_number));

    __magic_number_map.insert(std::pair< int, TypeInfo* >(magic_number, new TypeInfo(type)));
  }
  
  /* register some magic numbers */
  
  MagicNumberRegister __nmm_event_magic_number(TypeInfo("Event", "NMM"), 20);
  MagicNumberRegister __nmm_cevent_magic_number(TypeInfo("CEvent", "NMM"), 21);
  MagicNumberRegister __nmm_buffer_magic_number(TypeInfo("Buffer", "NMM"), 22);
  MagicNumberRegister __nmm_nmmguid_magic_number(TypeInfo("NMMGuid", "NMM"), 23);
  MagicNumberRegister __nmm_nodedescription_magic_number(TypeInfo("NodeDescription", "NMM"), 24);
  MagicNumberRegister __nmm_userinfo_magic_number(TypeInfo("UserInfo", "NMM"), 25);
  MagicNumberRegister __nmm__registry_id_magic_number(TypeInfo("RegistryID", "NMM"), 26);
  MagicNumberRegister __nmm_list_Response_value_magic_number(TypeInfo("list_Response", "std"), 27);
  MagicNumberRegister __nmm_list_RegistryID_value_magic_number(TypeInfo("list_RegistryID", "std"), 28);
  MagicNumberRegister __nmm_list_ObjectID_value_magic_number(TypeInfo("list_ObjectID", "std"), 29);
  MagicNumberRegister __nmm_pair_response_value_magic_number(TypeInfo("pair_RegistryID_list_ObjectID", "std"), 30);
  MagicNumberRegister __nmm_sequence_value_magic_number(TypeInfo("SequenceValue", "NMM"), 31);

  MagicNumberRegister __nmm_NULL_value_magic_number(TypeInfo("NULL", "std"), 32);

  MagicNumberRegister __nmm_node_info_number(TypeInfo("NodeInfo", "NMM"), 33);
  MagicNumberRegister __nmm_mgm_number(TypeInfo("MemoryMgm", "NMM"), 34);

  MagicNumberRegister __nmm_timestamp_magic_number(TypeInfo("Timestamp", "NMM"), 40);
  MagicNumberRegister __nmm_time_magic_number(TypeInfo("Time", "NMM"), 41);

  MagicNumberRegister __nmm_ts_description_number(TypeInfo("TSDescription", "NMM"), 50);



  MagicNumberRegister __nmm_list_string_value_magic_number(TypeInfo("list_string", "std"), 200);
  MagicNumberRegister __nmm_list_float_value_magic_number (TypeInfo("list_float", "std"), 201);
  MagicNumberRegister __nmm_list_double_value_magic_number(TypeInfo("list_double", "std"), 202);
  MagicNumberRegister __nmm_list_u_int_value_magic_number(TypeInfo("list_unsigned_int", "std"), 202);
  MagicNumberRegister __nmm_list_su_int_value_magic_number(TypeInfo("list_short_unsigned_int", "std"), 203);
  MagicNumberRegister __nmm_list_int_value_magic_number(TypeInfo("list_int", "std"), 204);
  MagicNumberRegister __nmm_list_s_int_value_magic_number(TypeInfo("list_short_int", "std"), 205);
  MagicNumberRegister __nmm_list_long_value_magic_number(TypeInfo("list_long", "std"), 206);
  MagicNumberRegister __nmm_list_u_long_value_magic_number (TypeInfo("list_unsigned_long", "std"), 207);
  MagicNumberRegister __nmm_list_NodeInfo_magic_number(TypeInfo("list_NodeInfo", "std"), 208);


  MagicNumberRegister __nmm_vector_nmm_int32_magic_number(TypeInfo("vector_int32_t", "std"), 300);

  MagicNumberRegister __nmm_pair_int_int_value_magic_number (TypeInfo("pair_int_int", "std"), 400);
  MagicNumberRegister __nmm_pair_unigned_long_long_unsigned_long_long_magic_number (TypeInfo("pair_unsigned_long_long_unsigned_long_long", "std"), 401);

  MagicNumberRegister __nmm_map_string_string_magic_number (TypeInfo("map_string_string", "std"), 500);

  
}

DEFINE_SINGLETON(NMM::MagicNumberManager);

#include "nmm/nmm_suffix.hpp"
