/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "OStream.hpp"
#include "TValue.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  OStream::OStream(const TypeInfo& ti)
    : SerializedValue(ti),
      __own_partner_stream(false),
      __partner_stream(0)
  {
    registerAttributes();
  }
  
  OStream::OStream(const OStream& stream)
    : SerializedValue(stream),
      __own_partner_stream(false),
      __partner_stream(0)
  {
    registerAttributes();
    
    assign(stream);
  }
  
  OStream& OStream::operator=(const OStream& stream) {
    if(this != &stream) {
      assign(stream);
    }
    return *this;
  }
  
  void OStream::assign(const OStream& stream) {
    if(stream.__own_partner_stream) {
      setOPartnerStreamDeepCopy(stream.getOPartnerStream());
    } else {
      setOPartnerStream(stream.__partner_stream->getRValue());
    }
  }
  
  OStream::~OStream() {
    setOPartnerStream(0);
    
    delete __partner_stream;
  }
  
  void OStream::writeLine(const string& mybuf) {
    writeBuffer(mybuf.c_str(), static_cast<nmm_uint32>(mybuf.size()));
  }
  
  void OStream::registerAttributes() {
    __partner_stream = new TValue<OStream*>(0);
    insertArgument(__partner_stream);
  }
  
  OStream* OStream::getOPartnerStream() {
    return __partner_stream->getRValue();
  }
  
  const OStream* OStream::getOPartnerStream() const {
    return __partner_stream->getRValue();    
  }
  
  void OStream::setOPartnerStream(OStream* partner_stream) {
    if(getOPartnerStream() != partner_stream) {
      if(__own_partner_stream) {
        /* delete old partner stream */
        delete getOPartnerStream();
        __own_partner_stream = false;
      }
      
      /* shallow copy partner stream */
      __partner_stream->setValue(partner_stream);
    }
  }
  
  void OStream::setOPartnerStreamDeepCopy(const OStream* partner_stream) {
    if(!__own_partner_stream || getOPartnerStream() != partner_stream) {
      if(__own_partner_stream) {
        /* delete old partner stream */
        delete getOPartnerStream();
        __own_partner_stream = false;
      }
      
      if(partner_stream) {
        /* deep copy partner stream */
        __partner_stream->setValue(partner_stream->copy());
        __own_partner_stream = true;
      } else {
        __partner_stream->setValue(0);
      }
    }
  }
  
}

#include "nmm/nmm_suffix.hpp"
