/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "SerializeLocalTime.hpp"

#include "nmm/utils/time/LocalTime.hpp"

#include "nmm/comm/serialize/IValueStream.hpp"
#include "nmm/comm/serialize/OValueStream.hpp"
#include "nmm/comm/serialize/TValueConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  OValueStream& operator<<(OValueStream& s, const LocalTime& l_time) {
    s.beginType("LocalTime", "NMM");
    
    nmm_uchar second = l_time.getSecond();
    nmm_uchar minute = l_time.getMinute();
    nmm_uchar hour = l_time.getHour();
    nmm_uchar day_of_month = l_time.getDayOfMonth();
    nmm_uchar day_of_week = l_time.getDayOfWeek();
    nmm_uchar day_of_year = l_time.getDayOfYear();
    nmm_uchar month = l_time.getMonth();
    nmm_uint16 gmt_offset = l_time.getGMTOffset();
    nmm_uint32 year = l_time.getYear();
    nmm_bool is_dst = l_time.isDaylightSavingsTime();
    std::string time_zone = l_time.getTimeZone();
    
    s << second;
    s << minute;
    s << hour;
    s << day_of_month;
    s << day_of_week;
    s << day_of_year;
    s << month;
    s << gmt_offset;
    s << year;
    s << is_dst;
    s << time_zone;
    
    s.endType();
    return s;
  }
  
  IValueStream& operator>>(IValueStream& s, LocalTime& l_time) {
    if ( (s.getCurrentType() != "LocalTime") || (s.getCurrentNamespace() != "NMM")) {
      GLOBAL_ERROR_STREAM("Invalid Type! Can not assign " << s.getCurrentType()<< " to an NMM::LocalTime. " << endl);
      s.nextElement();
      return s;
    }
    
    if (!s.goDown()) {
      GLOBAL_ERROR_STREAM("Could not access the interval values " << endl);
      s.nextElement();
      return s;
    }
    
    nmm_uchar second;
    nmm_uchar minute;
    nmm_uchar hour;
    nmm_uchar day_of_month;
    nmm_uchar day_of_week;
    nmm_uchar day_of_year;
    nmm_uchar month;
    nmm_uint16 gmt_offset;
    nmm_uint32 year;
    nmm_bool is_dst;
    std::string time_zone;
    
    s >> second;
    s >> minute;
    s >> hour;
    s >> day_of_month;
    s >> day_of_week;
    s >> day_of_year;
    s >> month;
    s >> gmt_offset;
    s >> year;
    s >> is_dst;
    s >> time_zone;

    // TODO: fw: This doesn't set all fields of the structure!
    // TODO: fw: It was all out-commented before. Maybe someone can look into this
    // TODO: fw: (Then again I am not quite sure why anyone would want to serialize a *Local*Time ...
    // TODO: fw: unless it's in GMT of course. but why should *Local*Time be represented in GMT?
    // TODO: fw: Wouldn't that make it a "GlobalTime" or a "GMTime"?
    // TODO: fw: Whoever thought about this, I wonder what he was smoking!
    // TODO: fw: Or maybe he was just too lazy to create a new class and therefore hijacked mine for his own evil purposes)
    l_time.setSecond(second);
    l_time.setMinute(minute);
    l_time.setHour(hour);
    l_time.setDayOfMonth(day_of_month);
//    l_time.setDayOfWeek(day_of_week);
//    l_time.setDayOfYear(day_of_year);
    l_time.setDayOfMonth(day_of_month);
    l_time.setMonth(month);
    l_time.setGMTOffset(gmt_offset);
    l_time.setYear(year);
    l_time.setDaylightSavingsTime(is_dst);
//    l_time.setTimeZone(time_zone);
    
    s.goUp();
    return s;
  }
  
  TValueConstructor<LocalTime> local_time_prototype("LocalTime", "NMM");

}

#include "nmm/nmm_suffix.hpp"
