/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "SerializeUtilsTime.hpp"

#include "nmm/utils/time/Time.hpp"
#include "nmm/utils/time/Interval.hpp"
#include "nmm/utils/time/Rational.hpp"
#include "nmm/utils/time/UserTime.hpp"

#include "nmm/comm/serialize/IValueStream.hpp"
#include "nmm/comm/serialize/OValueStream.hpp"
#include "nmm/comm/serialize/TValueConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  OValueStream& operator<<(OValueStream& s, const Time& time) {
    s.beginType("Time", "NMM");
    s << time.sec;
    s << time.nsec;
    s.endType();
    
    return s;
  }
  
  IValueStream& operator>>(IValueStream& s, Time& time) {
    if (s.getCurrentType() != "Time" && s.getCurrentNamespace() != "NMM") {
      GLOBAL_ERROR_STREAM("Invalid Type! Can not assign " << s.getCurrentType()<< " to an NMM::Time. "<< endl);
      s.nextElement();
      return s;
    }
    
    if (!s.goDown()) {
      GLOBAL_ERROR_STREAM("Could not access the time values " << endl);
      s.nextElement();
      return s;
    }
    s >> time.sec;
    
    s >> time.nsec;
    
    s.goUp();
    return s;
  }
  
  TValueConstructor<Time> nmm_time_prototype("Time", "NMM");
  
  
  OValueStream& operator<<(OValueStream& s, const Interval& interval) {
    s.beginType("Interval", "NMM");
    s << interval.sec;
    s << interval.nsec;
    s.endType();
    
    return s;
  };
  
  IValueStream& operator>>(IValueStream& s,Interval& interval)  {
    if(s.getCurrentType() != "Interval" && s.getCurrentNamespace() != "NMM") {
      GLOBAL_ERROR_STREAM("Invalid Type! Can not assign " << s.getCurrentType()<< " to an NMM::Interval. " << endl);
      s.nextElement();
      return s;
    }
    
    if (!s.goDown()) {
      GLOBAL_ERROR_STREAM("Could not access the interval values " << endl);
      s.nextElement();
      return s;
    }
    s >> interval.sec;
    s >> interval.nsec;
    
    s.goUp();
    
    return s;
  };

  TValueConstructor<Interval> interval_prototype("Interval", "NMM");
  
  
  OValueStream& operator<<(OValueStream& s, const Rational& r) {
    s.beginType("Rational", "NMM");
    s << r.getNumerator();
    s << r.getDenominator();
    s.endType();
    return s;
  }
  
  IValueStream& operator>>(IValueStream& s, Rational& r) {
    if (s.getCurrentTypeInfo() != TypeInfo("Rational", "NMM") ){
      GLOBAL_ERROR_STREAM("Invalid Type! Can not assign " << s.getCurrentType()<< " to an NMM::Rational. " << endl);
      s.nextElement();
      return s;
    }
    
    if (!s.goDown()) {
      GLOBAL_ERROR_STREAM("Could not access the interval values " << endl);
      s.nextElement();
      return s;
    }

    nmm_int64 num;
    nmm_int64 denom;
    s >> num;
    s >> denom;
    r = Rational(num, denom);
    
    s.goUp();
    
    return s;
  }
  
  TValueConstructor<Rational> __rational_value("Rational", "NMM");
  
  
  OValueStream& operator<<(OValueStream& ov, const UserTime& u_time) {
    ov.beginType("UserTime", "NMM");
    ov << u_time.hour;
    ov << u_time.min;
    ov << u_time.sec;
    ov << u_time.msec;
    ov.endType();
    return ov;
  }
  
  IValueStream& operator>>(IValueStream& s, UserTime& u_time) {
    if ( (s.getCurrentType() != "UserTime") || (s.getCurrentNamespace() != "NMM")) {
      GLOBAL_ERROR_STREAM("Invalid Type! Can not assign " << s.getCurrentType()<< " to an NMM::UserTime. " << endl);
      s.nextElement();
      return s;
    }
    
    if (!s.goDown()) {
      GLOBAL_ERROR_STREAM("Could not access the interval values " << endl);
      s.nextElement();
      return s;
    }
    
    s >> u_time.hour;
    s >> u_time.min;
    s >> u_time.sec;
    s >> u_time.msec;
    
    s.goUp();
    return s;
  }
  
  TValueConstructor<UserTime> user_time_prototype("UserTime", "NMM");
  
}

#include "nmm/nmm_suffix.hpp"
