/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "SerializedException.hpp"
#include "TConstructor.hpp"

#include <iostream>

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  SerializedException::SerializedException()
    : SerializedValue("Exception", "NMM")
  {
    parent = 0;
    comment.setValue("Exception");
    insertArgument(&comment);
  }
  
  SerializedException::SerializedException(const std::string& s)
    : SerializedValue("Exception", "NMM")
  {
    parent = 0;
    comment = "Exception : " + s;
    insertArgument(&comment);
  }
  
  SerializedException::SerializedException(const NamedObject* n, const std::string& s)
    : SerializedValue("Exception", "NMM")
  {
    parent = n;
    comment = "Exception : " + s;
    insertArgument(&comment);
  }
  
  SerializedException::SerializedException(const SerializedException& exception)
    : SerializedValue(exception)
  {
    *this = exception;
    insertArgument(&comment);
  }
  
  SerializedException* SerializedException::copy() const {
    return new SerializedException(*this);
  }
  
  std::ostream& operator<<(std::ostream& os, const SerializedException& e) {
    return os << e.getComment();
  }
  
  TConstructor<SerializedException> __nmm__default_exception("SerializedException", "NMM");
  
}

#include "nmm/nmm_suffix.hpp"
