/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "StdIStream.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  StdIStream::StdIStream(std::istream* stream)
    : IStream(TypeInfo("StdIStream", "NMM")),
      __stream(stream)
  {}

  StdIStream* StdIStream::copy() const {
    return new StdIStream(*this);
  }

  StdIStream& StdIStream::operator>>(nmm_uint16& value)        {
    *__stream >> value;
    return *this;
  }
  StdIStream& StdIStream::operator>>(nmm_int16& value){
    *__stream >> value;
    return *this;
  }
  StdIStream& StdIStream::operator>>(nmm_uint32& value  ){
    *__stream >> value;
    return *this;
  }
  StdIStream& StdIStream::operator>>(nmm_int32& value  ){
    *__stream >> value;
    return *this;
  }

  StdIStream& StdIStream::operator>>(nmm_int64& value  ){
    *__stream >> value;
    return *this;
  }
  StdIStream& StdIStream::operator>>(nmm_uint64& value  ){
    *__stream >> value;
    return *this;
  }
  StdIStream& StdIStream::operator>>(nmm_bool& value  ){
    *__stream >> value;
    return *this;
  }

  StdIStream& StdIStream::operator>>(nmm_double& value ){
    *__stream >> value;
    return *this;
  }

  StdIStream& StdIStream::operator>>(nmm_float& value ){
    *__stream >> value;
    return *this;
  }
  StdIStream& StdIStream::operator>>(nmm_char& value ){
    readBuffer(&value, 1);
    return *this;
  }
  StdIStream& StdIStream::operator>>(nmm_uchar& value){
    readBuffer(&value, 1);
    return *this;
  }
  StdIStream& StdIStream::operator>>(string& value){
    size_t size = 0;
    *__stream >> size;
    value.resize(size);
    __stream->get(); //skip the blank
    if (size > 0)
      __stream->read(&value[0], size);
    return *this;
  }
  void StdIStream::readBuffer(char* value, nmm_uint32 size)           {
    __stream->get(); //skip the blank
    __stream->read(value, size);
  }
  void StdIStream::readBuffer(unsigned char* value, nmm_uint32 size){
    __stream->get(); //skip the blank
    __stream->read(reinterpret_cast<char*>(value), size);
  }     
     
  bool StdIStream::isEmpty() const {
    return true;
  }
  
}

#include "nmm/nmm_suffix.hpp"
