/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "StdOStream.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  StdOStream::StdOStream(std::ostream* stream)
    : OStream(TypeInfo("StdOStream", "NMM")),
      __stream(stream),
      __fin(0x20)
  {}
  
  StdOStream* StdOStream::copy() const {
    return new StdOStream(*this);
  }

  StdOStream::~StdOStream() {
    if (__stream)
      __stream->flush();

    __stream = 0;
  }

  StdOStream& StdOStream::operator<<(const nmm_uint16& value) {
    *__stream << value << __fin;
    return *this;
  }
  StdOStream& StdOStream::operator<<(const nmm_int16& value   ) {
    *__stream << value << __fin;
    return *this;
  }
  StdOStream& StdOStream::operator<<(const nmm_uint32& value) {
    *__stream << value << __fin;
    return *this;
  }
  StdOStream& StdOStream::operator<<(const nmm_int32& value   ) {
    *__stream << value << __fin;
    return *this;
  }
  StdOStream& StdOStream::operator<<(const nmm_uint64& value ) {
    *__stream << value << __fin;
    return *this;
  }
  StdOStream& StdOStream::operator<<(const nmm_int64& value ) {
    *__stream << value << __fin;
    return *this;
  }
  StdOStream& StdOStream::operator<<(const nmm_bool& value ) {
    *__stream << value << __fin;
    return *this;
  }
  StdOStream& StdOStream::operator<<(const nmm_double& value ) {
    *__stream << value << __fin;
    return *this;
  }
  StdOStream& StdOStream::operator<<(const nmm_float& value ) {
    *__stream << value << __fin;
    return *this;
  }
  StdOStream& StdOStream::operator<<(const nmm_char& value ) {
    writeBuffer(&value, 1);
    return *this;
  }
  StdOStream& StdOStream::operator<<(const nmm_uchar& value) {
    writeBuffer(&value, 1);
    return *this;
  }
  StdOStream& StdOStream::operator<<(const string& value) {
    *__stream << value.size() << __fin << value << __fin;
    return *this;
  }

  void StdOStream::writeBuffer(const char* value, nmm_uint32 size)  {
    __stream->write(value,size);
    __stream->write(&__fin,1);
  }

  void StdOStream::writeBuffer(const unsigned char* value, nmm_uint32 size) {
    __stream->write(reinterpret_cast<const char*>(value),size);
    __stream->write(reinterpret_cast<char*>(&__fin),1);
  }

  void StdOStream::writeData() {
    if (__stream)
      __stream->flush();
  }

}

#include "nmm/nmm_suffix.hpp"
