/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "StdValue.hpp"

#include "StdTypeInfo.hpp"
#include "TValueConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  // Value constructors for C++ POD types
  TValueConstructor<nmm_char>    __nmm_char_constructor     (nmm_char_type_info);
  TValueConstructor<nmm_uchar>   __nmm_uchar_constructor    (nmm_uchar_type_info);
  TValueConstructor<nmm_int16>   __nmm_int16_constructor    (nmm_int16_type_info);
  TValueConstructor<nmm_int32>   __nmm_int32_constructor    (nmm_int32_type_info);
  TValueConstructor<nmm_int64>   __nmm_int64_constructor    (nmm_int64_type_info);
  TValueConstructor<nmm_uint16>  __nmm_uint16_constructor   (nmm_uint16_type_info);
  TValueConstructor<nmm_uint32>  __nmm_uint32_constructor   (nmm_uint32_type_info);
  TValueConstructor<nmm_uint64>  __nmm_uint64_constructor   (nmm_uint64_type_info);
  TValueConstructor<nmm_bool>    __nmm_bool_constructor     (nmm_bool_type_info);
  TValueConstructor<nmm_float>   __nmm_float_constructor    (nmm_float_type_info);
  TValueConstructor<nmm_double>  __nmm_double_constructor   (nmm_double_type_info);

  // Value constructors for std::string
  TValueConstructor<std::string> __std_string_constructor   (string_type_info);



  OValueStream & operator<<(OValueStream& ov, const wstring& ws) {
    ov.beginType("wstring", "std");
    nmm_uint32 sof = sizeof(wchar_t);
    nmm_uint32 sos = static_cast<nmm_uint32>(ws.size());
    ov << sof;
    ov << sos;

    if (sos > 0) {
      ov.writeBuffer(reinterpret_cast<const char*>(ws.c_str()), static_cast<nmm_uint32>(sizeof(wchar_t) * ws.size()));
    }

    ov.endType();
    return ov;
  } 

  IValueStream & operator>>(IValueStream& iv, wstring& ws) {
    if ( iv.getCurrentTypeInfo() != TypeInfo("wstring", "std") ) 
    {
      GLOBAL_ERROR_STREAM("wstring> Invalid type assignment" << endl);
      GLOBAL_ERROR_STREAM ("wstring> Current type : " << iv.getCurrentType() << "                Expected type: " << "wstring" << endl);
      GLOBAL_ERROR_STREAM ("wstring> Current namespace : " << iv.getCurrentNamespace() << "      Expected namespace: " << "std" << endl);

      iv.nextElement();
      return iv;
    }
    /* End< */

    /* Begin> Access internal attributes */
    if (!iv.goDown()) {
      GLOBAL_DEBUG_STREAM("String seems to be empty" << endl);
      iv.nextElement();
      return iv;
    }
    /* End< */

    nmm_uint32 size_of_wchar_t = 0;
    nmm_uint32 size_of_string = 0;
   
    iv >> size_of_wchar_t;
    iv >> size_of_string;

    if ( size_of_string == 0 ) {
      ws = wstring(); // assign an empty string
    } else {
      if (size_of_wchar_t != sizeof(wchar_t)) {

		// Read data from network 
        char * tmp_buffer = new char[size_of_wchar_t * size_of_string];
        iv.readBuffer(tmp_buffer, size_of_wchar_t * size_of_string);
    
        ws.resize(size_of_string);
        // Copy data 
        for (unsigned int i = 0, j = 0; i < size_of_string; ++i, j = j+size_of_wchar_t) {
          if ( sizeof(wchar_t) > size_of_wchar_t ) {
            memcpy( &ws[i], &tmp_buffer[j], size_of_wchar_t );
          } else {
            memcpy( &ws[i], &tmp_buffer[j], sizeof(wchar_t) );
          }
        }

        delete[] tmp_buffer;
      } else {
        ws.resize(size_of_string);
        iv.readBuffer(reinterpret_cast<char*>(&ws[0]), static_cast<nmm_uint32>(sizeof(wchar_t) * ws.size()));
      }
    }

    iv.goUp();
    return iv;
  } 


  OValueStream& operator<<(OValueStream& ov, const list<string>& string_list) {
    return writeList<string>(ov, string_list, "string");
  }
  
  IValueStream& operator>>(IValueStream& ov, list<string>& string_list) {
    return readList<string>(ov, string_list, "string");
  }

  OValueStream& operator<<(OValueStream& ov, const list<nmm_float>& string_list){
    return writeList<nmm_float>(ov, string_list, "float");
  }

  IValueStream& operator>>(IValueStream& ov,list<nmm_float>& string_list) {
    return readList<nmm_float>(ov, string_list, "float");
  }


  OValueStream& operator<<(OValueStream& ov, const list<nmm_double>& string_list) {
    return writeList<nmm_double>(ov, string_list, "double");
  }

  IValueStream& operator>>(IValueStream& ov,list<nmm_double>& string_list)  {
    return readList<nmm_double>(ov, string_list, "double");
  }

  OValueStream& operator<<(OValueStream& ov, const list<nmm_uint32>& string_list) {
    return writeList<nmm_uint32>(ov, string_list, "unsigned_int");
  }

  IValueStream& operator>>(IValueStream& ov,list<nmm_uint32>& string_list) {
    return readList<nmm_uint32>(ov, string_list, "unsigned_int");
  }

  OValueStream& operator<<(OValueStream& ov, const list<nmm_uint16>& string_list) {
    return writeList<nmm_uint16>(ov, string_list, "short_unsigned_int");
  }

  IValueStream& operator>>(IValueStream& ov,list<nmm_uint16>& string_list){
    return readList<nmm_uint16>(ov, string_list, "short_unsigned_int");
  }

  OValueStream& operator<<(OValueStream& ov, const list<nmm_int32>& string_list){
    return writeList<nmm_int32>(ov, string_list, "int");
  }

  IValueStream& operator>>(IValueStream& ov,list<nmm_int32>& string_list) {
    return readList<nmm_int32>(ov, string_list, "int");
  }

  OValueStream& operator<<(OValueStream& ov, const list<nmm_int16>& string_list) {
    return writeList<nmm_int16>(ov, string_list, "short_int");
  }

  IValueStream& operator>>(IValueStream& ov,list<nmm_int16>& string_list) {
    return readList<nmm_int16>(ov, string_list, "short_int");
  }


  OValueStream& operator<<(OValueStream& ov, const list<pair<string, int> >& string_list) {
    return writeList<pair<string, int> >(ov, string_list, "pair_string_int");
  }

  IValueStream& operator>>(IValueStream& ov,list<pair<string, int> >& string_list) {
    return readList<pair<string, int> >(ov, string_list, "pair_string_int");
  }
  
  OValueStream& operator<<(OValueStream& ov, const pair<nmm_int32, nmm_int32>& string_list) {
    return writePair<nmm_int32, nmm_int32>(ov, string_list, "int", "int");
  }

  IValueStream& operator>>(IValueStream& ov, pair<nmm_int32, nmm_int32>& string_list) {
    return readPair<nmm_int32, nmm_int32>(ov, string_list, "int", "int");
  }

  OValueStream& operator<<(OValueStream& ov, const pair<nmm_uint64, nmm_uint64>& string_list) {
    return writePair<nmm_uint64, nmm_uint64>(ov, string_list, "unsigned_long_long", "unsigned_long_long");
  }

  IValueStream& operator>>(IValueStream& ov, pair<nmm_uint64, nmm_uint64>& string_list) {
    return readPair<nmm_uint64, nmm_uint64>(ov, string_list, "unsigned_long_long", "unsigned_long_long");
  }


  OValueStream& operator<<(OValueStream& ov, const pair<string, nmm_int32>& string_list) {
    return writePair<string, nmm_int32>(ov, string_list, "string", "int");
  }

  IValueStream& operator>>(IValueStream& ov, pair<string, nmm_int32>& string_list) {
    return readPair<string, nmm_int32>(ov, string_list, "string", "int");
  }


  OValueStream& operator<<(OValueStream& ov, const map<string, string>& string_map) {
    return writeMap<string, string>(ov, string_map, string_type_info, string_type_info);
  }

  IValueStream& operator>>(IValueStream& ov, map<string, string>& string_map) {
    return readMap<string, string>(ov, string_map, string_type_info, string_type_info);
  }

  OValueStream& operator<<(OValueStream& ov, const map<string, nmm_int32>& string_int_map) {
    return writeMap<string, nmm_int32>(ov, string_int_map, string_type_info, nmm_int32_type_info);
  }

  IValueStream& operator>>(IValueStream& ov, map<string, nmm_int32>& string_int_map) {
    return readMap<string, nmm_int32>(ov, string_int_map, string_type_info, nmm_int32_type_info);
  }

  OValueStream& operator<<(OValueStream& ov, const vector<nmm_bool>& bool_vector) {
    return writeVector<nmm_bool>(ov, bool_vector, nmm_bool_type_info.t_name);
  }

  IValueStream& operator>>(IValueStream& ov, vector<nmm_bool>& bool_vector) {
    return readBoolVector(ov, bool_vector, nmm_bool_type_info.t_name);
  }
  
  OValueStream& operator<<(OValueStream& ov, const map<nmm_int32, list<nmm_int32> >& string_map)
  {
    return writeMap<nmm_int32, list<nmm_int32> >(ov, string_map, NMM::nmm_int32_type_info, NMM::TypeInfo("list_int", "std"));
  }

  IValueStream& operator>>(IValueStream& ov, map<nmm_int32, list<nmm_int32> > & string_map)
  {
    return readMap<nmm_int32, list<nmm_int32> >(ov, string_map, NMM::nmm_int32_type_info, NMM::TypeInfo("list_int", "std"));
  }

   
  TValueConstructor<wstring>                        __nmm_wstring_value("wstring", "std");
  TValueConstructor<list<string> >                  __nmm_list_string_value("list_string", "std");
  TValueConstructor<list<nmm_float> >               __nmm_list_float_value ("list_float", "std");
  TValueConstructor<list<nmm_double> >              __nmm_list_nmm_double_value("list_double", "std");
  TValueConstructor<list<nmm_uint32> >              __nmm_list_u_int_value ("list_unsigned_int", "std");
  TValueConstructor<list<nmm_uint16> >              __nmm_list_su_int_value("list_short_unsigned_int", "std");
  TValueConstructor<list<nmm_int32> >               __nmm_list_int_value   ("list_int", "std");
  TValueConstructor<list<nmm_int16> >               __nmm_list_s_int_value ("list_short_int", "std");
/*  TValueConstructor<list<long> >                    __nmm_list_long_value  ("list_long", "std");
  TValueConstructor<list<unsigned long> >           __nmm_list_u_long_value ("list_unsigned_long", "std");*/
  TValueConstructor<list<pair<string, nmm_int32> > >      __nmm_list_pair_string_int ("list_pair_string_int", "std");
  
  TValueConstructor<pair<nmm_int32, nmm_int32> >    __nmm_pair_int_int_value ("pair_int_int", "std");
  TValueConstructor<pair<nmm_uint64, nmm_uint64> >  __nmm_pair_unigned_long_long_unsigned_long_long ("pair_unsigned_long_long_unsigned_long_long", "std");
  TValueConstructor<pair<string, int> >             __nmm_pair_string_int ("pair_string_int", "std");
  
  TValueConstructor<map<string, string> >           __nmm_map_string_string ("map_string_string", "std");
  TValueConstructor<map<string, nmm_int32> >        __nmm_map_string_int ("map_string_int", "std");
  
  TValueConstructor<vector<nmm_bool> >              __nmm_vector_bool ("vector_bool", "std");
  
  
  IValueStream& readBoolVector(IValueStream& iv, vector<nmm_bool>& v_vector, const string& type_name)
  {
    /* Begin> Type checking */
    if (iv.getCurrentType() != "vector_"+type_name) {
      GLOBAL_ERROR_STREAM("List> Invalid type assignment" << endl);
      GLOBAL_ERROR_STREAM ("List> Current type : " << iv.getCurrentType() << "                Expected type: " << "vector_"+type_name << endl);
      GLOBAL_ERROR_STREAM ("List> Current namespace : " << iv.getCurrentNamespace() << "      Expected namespace: " << "std" << endl);

      iv.nextElement();
      return iv;
    }
    /* End< */

    /* Begin> Access internal attributes */
    if (!iv.goDown()) {
      GLOBAL_DEBUG_STREAM("List seems to be empty" << endl);
      iv.nextElement();
      return iv;
    }
    /* End< */

    nmm_uint32 size;

    /* read the size of the vector */
    iv >> size;

    v_vector.resize(size);

    /* Begin> While the Stream contains elements, read them */
    bool tmp;
    for (size_t i = 0; i < size; ++i) {
      iv >> tmp;
      v_vector[i] = tmp;
    }
    /* End< */

    if (!iv.isEmpty()) {
      GLOBAL_ERROR_STREAM("Unhandled elements from vector_" << type_name << " in IValueStream" << endl);
    }

    iv.goUp();

    return iv;
  }
  
}

#include "nmm/nmm_suffix.hpp"
