/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "XMLOValueStream.hpp"
#include "XMLIValueStream.hpp" // for TValueStreamConstructor

#include "nmm/comm/serialize/StdTypeInfo.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"
#include "nmm/comm/serialize/TValueStreamConstructor.hpp"

#include "nmm/utils/xml/XMLNode.hpp"

#include "nmm/utils/NamedObject.hpp"

// XXX: fw: needed for UTF8toisolat1 (should use XMLEncoder instead)
// XXX: fw: whoever fixes this should also remove @XML_CFLAGS@ from
// XXX: fw: Makefile.am
#include <libxml/encoding.h>

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  string XMLOValueStream::convertToString(const int& i) {
    strstream stream;
    stream << i;
    string r_value;
    stream >> r_value;
    return r_value;
  }
  
  string XMLOValueStream::convertToString(const nmm_int16& i) {
    strstream stream;
    stream << i;
    string r_value;
    stream >> r_value;
    return r_value;
  }
  
  
  XMLOValueStream::XMLOValueStream()
    : OValueStream(TypeInfo("XMLOValueStream", "NMM"))
  {
    XMLNode* root_node = new XMLNode("XMLValueStream");
    tree.set_root(root_node);
    latest_node = root_node;
    parent_stack.push_front(latest_node);
  }

  XMLOValueStream::XMLOValueStream(const XMLOValueStream& stream)
    : OValueStream(stream)
  {
    XMLNode* root_node = new XMLNode("XMLValueStream");
    tree.set_root(root_node);
    latest_node = root_node;
  
  }
  XMLOValueStream::~XMLOValueStream() {}

  void XMLOValueStream::beginType(const string& type, const string& ns) {
    latest_node = latest_node->add_child("Compound");
    latest_node->add_property("type", type);    
    if (!ns.empty()) {
      latest_node->add_property("c_ns", ns);
    }
    parent_stack.push_front( latest_node );
  }

  void XMLOValueStream::endType() {
    if (parent_stack.empty()) {
      GLOBAL_ERROR_STREAM("XMLOValueStream : end error" << endl);
      return;
    }
    parent_stack.pop_front();
    latest_node = (*parent_stack.begin());
  }

  string XMLOValueStream::toString() const {
    return tree.write_buffer();
  }

  void XMLOValueStream::writeBuffer(const char* buffer, nmm_uint32 size) {
    XMLNode* tmp_node = latest_node->add_child("simple");
    tmp_node->add_property("type", "char_buffer");
    string s;
    unsigned int new_size = size * 2;
    s.resize(new_size);
    /* convert the buffer to UTF8 */
    isolat1ToUTF8(reinterpret_cast<unsigned char*>(&s[0]), reinterpret_cast<int*>(&new_size), reinterpret_cast<const unsigned char*>(buffer), reinterpret_cast<int*>(&size));
    s.resize(new_size);
    tmp_node->add_property("Value", s);
  }

  void XMLOValueStream::writeData()  {
    if (getOPartnerStream()) {
      string s = toString();
      nmm_int32 size = s.size();
      (*getOPartnerStream()) << size;
      getOPartnerStream()->writeBuffer(&s[0], size);
      getOPartnerStream()->writeData();
    }
    parent_stack.clear();
    XMLNode* root_node = new XMLNode("XMLValueStream");
    tree.set_root(root_node);
    latest_node = root_node;
    parent_stack.push_front(latest_node);
  }
  void XMLOValueStream::writeBuffer(const unsigned char* buffer, nmm_uint32 size) {
    XMLNode* tmp_node = latest_node->add_child("simple");
    tmp_node->add_property("type", "unsigned_char_buffer");

    XMLProperty* property = tmp_node->add_property("Value", string());
    string& s = property->value();

    unsigned int new_size = size * 2;
    s.resize(new_size);
    /* convert the buffer to UTF8 */
    isolat1ToUTF8(reinterpret_cast<unsigned char*>(&s[0]), reinterpret_cast<int*>(&new_size), reinterpret_cast<const unsigned char*>(buffer), reinterpret_cast<int*>(&size));
    s.resize(new_size);
  }
  
  void XMLOValueStream::writeConvertedType(const string& value, const string& type) {
    XMLNode* tmp_node = latest_node->add_child("simple");
    tmp_node->add_property("type", type);
    tmp_node->add_property("Value", value);
  }
	
  void XMLOValueStream::writeConvertedType(const string& value, const string& type, const string& ns) {
    XMLNode* tmp_node = latest_node->add_child("simple");
    tmp_node->add_property("type", type);	
    tmp_node->add_property("Value", value);
    tmp_node->add_property("c_ns", ns);
  }

  XMLOValueStream& XMLOValueStream::operator<<(const nmm_uint16 &i) {
    writeType<nmm_uint16>(i, nmm_uint16_type_info.t_name);
    return *this;
  }


  XMLOValueStream& XMLOValueStream::operator<<(const nmm_int16 &i) {
    writeType<nmm_int16>(i, nmm_int16_type_info.t_name);
    return *this;
  }

  XMLOValueStream& XMLOValueStream::operator<<(const nmm_uint32 &i) {
    writeType<nmm_uint32>(i, nmm_uint32_type_info.t_name);
    return *this;
  }

  XMLOValueStream& XMLOValueStream::operator<<(const nmm_int32&  i) {
    writeType<nmm_int32>(i, nmm_int32_type_info.t_name);
    return *this;
  }

  XMLOValueStream& XMLOValueStream::operator<<(const nmm_uint64 &i) {
    writeType<nmm_uint64>(i, nmm_uint64_type_info.t_name);
    return *this;
  }

  XMLOValueStream& XMLOValueStream::operator<<(const nmm_int64&  i) {
    writeType<nmm_int64>(i, nmm_int64_type_info.t_name);
    return *this;
  }


  XMLOValueStream& XMLOValueStream::operator<<(const nmm_double& i) {
    writeType<nmm_double>(i, nmm_double_type_info.t_name);
    return *this;
  }

  XMLOValueStream& XMLOValueStream::operator<<(const nmm_float& i) {
    writeType<nmm_float>(i, nmm_float_type_info.t_name);
    return *this;
  }

  XMLOValueStream& XMLOValueStream::operator<<(const nmm_char& i) {
    writeType<nmm_char>(i, nmm_char_type_info.t_name);
    return *this;
  }
  XMLOValueStream& XMLOValueStream::operator<<(const nmm_uchar& i) {
    writeType<nmm_uchar>(i, nmm_uchar_type_info.t_name);
    return *this;
  }


  XMLOValueStream& XMLOValueStream::operator<<(const string& i) {
    writeType<string>(i, string_type_info.t_name, string_type_info.t_namespace);
    return *this;
  }

  XMLOValueStream& XMLOValueStream::operator<<(const nmm_bool& i) {
    writeType<nmm_bool>(i, nmm_bool_type_info.t_name);
    return *this;
  }
  
  const char* XMLOValueStream::__o_value_stream_type = "XMLOValueStream";
  
  template<>
  class TConstructor<XMLOValueStream> : public ValueConstructor {
  public:
    TConstructor(const TypeInfo& t_info)
      : ValueConstructor(t_info)
    {}
    virtual XMLOValueStream*          createValue()          const {return new XMLOValueStream();}
    virtual TValue<XMLOValueStream>*  createTValue()         const {return new TValue<XMLOValueStream>();}
    
    virtual TValue<OValueStream*>*  createTPointerValue()  const {return new TValue<OValueStream*>();}
    
    virtual TConstructor<XMLOValueStream>* copy()  const {return new TConstructor(*this); }
  private:
    string  __type_name;
    string  __namespace;
  };

  TConstructor<XMLOValueStream> xml_o_value_stream(TypeInfo("XMLOValueStream", "NMM"));
  
  TValueStreamConstructor<XMLIValueStream, XMLOValueStream> xml_value_stream(XMLIValueStream::__i_value_stream_type, XMLOValueStream::__o_value_stream_type);  
  
}

#include "nmm/nmm_suffix.hpp"
