/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "AudioCDURLObject.hpp"

#include "nmm/multimedia/NodeDescription.hpp"
#include "nmm/plugins/file/cdda/ICDDA.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  // ****************** AudioCDURLObject ***************************
  void AudioCDURLObject::configureNodeDescription(NodeDescription &node){
    string device = __url->getURIPath();
    if (device == "") {
      device = string("/dev/cdrom");
    }
    node.addNodeEvent(Event(ICDDA::setDevice_event, new TInValue<string>(device)), INode::CONSTRUCTED);

    unsigned int track = 0;
    string track_str = __url->getOption("track");
    if (track_str == "") {
      track = 0;
    } 
    else {
      track = static_cast<unsigned int>(atoi(track_str.c_str()));
    }
    node.addNodeEvent(Event(ICDDA::setTrack_event, new TInValue<nmm_int32>(track)), INode::OUTPUT_INITIALIZED);
    node.addNodeEvent(Event(ICDDA::setOneTrackOnly_event, new TInValue<nmm_bool>(true)), INode::OUTPUT_INITIALIZED);
    node.addNodeEvent(Event(ICDDA::setRepeat_event, new TInValue<nmm_bool>(false)), INode::OUTPUT_INITIALIZED);
  }

  list<string> AudioCDURLObject::getInterfaces() const 
  {
    list<string> l;
    l.push_back(ICDDA::ICDDA_type);
    return l;
  }

  Format* AudioCDURLObject::getFormatFromProtocol() const
  {
    return new Format(Format::audio_raw_type);
  }

}

#include "nmm/nmm_suffix.hpp"
