/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "DVDURLObject.hpp"

#include "nmm/multimedia/NodeDescription.hpp"
#include "nmm/plugins/file/dvd/interfaces/IDVD.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  // ****************** DVDURLObject ***************************
  void  DVDURLObject::configureNodeDescription(NodeDescription &node){
    string device = __url->getURIPath();
    if (device == "") {
      device = string("/dev/cdrom");
    }   
    node.addNodeEvent(Event(IDVD::setDevice_event, new TInValue<string>(device)), INode::CONSTRUCTED);

    unsigned int title = 0;
    string title_str = __url->getOption("title");
    if (title_str == "") {
      title = 1;
    } 
    else {
      title = static_cast<unsigned int>(atoi(title_str.c_str()));
    }

    unsigned int chapter = 0;
    string chapter_str = __url->getOption("chapter");
    if (chapter_str == "") {
      chapter = 1;
    } 
    else {
      chapter = static_cast<unsigned int>(atoi(chapter_str.c_str()));
    }

    unsigned int angle = 0;
    string angle_str = __url->getOption("angle");
    if (angle_str == "") {
      angle = 1;
    } 
    else {
      angle = static_cast<unsigned int>(atoi(angle_str.c_str()));
    }
    
    node.addNodeEvent(Event(IDVD::select_event, new TInValue<nmm_uint32>(title),
                            new TInValue<nmm_uint32>(chapter),
                            new TInValue<nmm_uint32>(angle)),
                      INode::OUTPUT_INITIALIZED);

  }
  list<string> DVDURLObject::getInterfaces() const 
  {
    list<string> l;
    l.push_back(IDVD::IDVD_type);
    return l;
  }

  Format* DVDURLObject::getFormatFromProtocol() const
  {
    return new Format(Format::av_mpegps_type);
  }

}

#include "nmm/nmm_suffix.hpp"
