/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "FileURLObject.hpp"

#include "nmm/multimedia/NodeDescription.hpp"
#include "nmm/plugins/interfaces/IFileHandler.hpp"
#include "nmm/plugins/file/generic/utils/FileUtils.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  // ****************** FileURLObject ***************************

  string FileURLObject::getNodeNameFromExtension() const
  {
    string node_name = "";

    if(( __url->getURIExtension() == "mpg")  ||
       ( __url->getURIExtension() == "mpeg") ||
       ( __url->getURIExtension() == "ogg")  ||
       ( __url->getURIExtension() == "ac3")  ||
       ( __url->getURIExtension() == "vdr")  ||
       ( __url->getURIExtension() == "vob")) {
      node_name = "GenericReadNode";
    }
    else if( __url->getURIExtension() == "mp3") {
	node_name = "MP3ReadNode";
    }
    else if(__url->getURIExtension() == "avi") {
      node_name = "AVIReadNode";
    }  
    else if((__url->getURIExtension() == "wmv") ||
            (__url->getURIExtension() == "wma")) {  
      node_name = "ASFReadNode";
    }
		else if((__url->getURIExtension() == "wav")) {
			node_name = "WavReadNode";
		}
    return node_name;
  }

  string FileURLObject::getNodeNameFromContent() const
  {
    string node_name = "";
    string mime_type = FileUtils::getMIMETypeFromFile(__url->getURIPath());

    // image data -> MagickReadNode
    if(mime_type.find("image") != string::npos) {
      node_name = "MagickReadNode";
    }
    return node_name;
  }

  Format* FileURLObject::getFormatFromContent() const
  {
    string format_type = "";

    string mime_type = FileUtils::getMIMETypeFromFile(__url->getURIPath());

    // 'file' will return audio/x-ogg for all ogg/ogm -> check again
    if(mime_type == Format::audio_x_ogg_type) {
      string mime_type2 = FileUtils::getMIMETypeFromFile(__url->getURIPath(), false);
      if(mime_type2.find("Vorbis audio") != string::npos) {
	format_type = Format::audio_x_ogg_type;
      }
      else {
	format_type = Format::av_ogm_type;
      }
    }

    // 'file' will return video/mpeg for all mpeg a/v files -> reset to av/mpegps
    else if(mime_type == Format::video_mpeg_type) {
      format_type = Format::av_mpegps_type;
    }

    // audio/x-wav -> audio/raw
    else if(mime_type.find(Format::audio_x_wav_type) != string::npos) {
      format_type = Format::audio_raw_type;
    }

    // avi files will output msvideo -> check again for AVI
    else if(mime_type.find("msvideo") != string::npos) {
      string mime_type2 = FileUtils::getMIMETypeFromFile(__url->getURIPath(), false);
      if(mime_type2.find("AVI") != string::npos) {
	format_type = Format::av_avi_type;
      }
    }

    if(format_type.empty()) {
      return 0;
    }
    else {
      return new Format(format_type);
    }
  }

  list<string> FileURLObject::getInterfaces() const
  {
    list<string> l;
    l.push_back(IFileHandler::IFileHandler_type);
    return l;
  }

  void FileURLObject::configureNodeDescription(NodeDescription &node)
  {
    node.addNodeEvent(Event(IFileHandler::setFilename_event, new TInValue<string>(__url->getURIPath())), INode::CONSTRUCTED);
  }

}

#include "nmm/nmm_suffix.hpp"
