/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "GenericTVURLObject.hpp"

#include "nmm/multimedia/NodeDescription.hpp"
#include "nmm/plugins/video/tv/interfaces/ITVCard.hpp"
#include "nmm/plugins/video/tv/interfaces/IChannelInfo.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  // ******************** GenericTVURLObject ******************************
  void GenericTVURLObject::configureNodeDescription(NodeDescription &node){
    string device = __url->getURIPath();
    if (device == "") {
      device = string("/dev/video0");
    }   

    node.addNodeEvent(Event(ITVCard::setDevice_event, new TInValue<string>(device)), INode::CONSTRUCTED);

    unsigned int channel = 0;
    string channel_str = __url->getOption("channel");
    if (channel_str == "") {
      channel = 0;
    }else {
      channel = atoi(channel_str.c_str());
    }

    node.addNodeEvent(Event(ITVCard::setChannel_event, new TInValue<nmm_int32>(channel)), INode::INITIALIZED);

    string channelConfig = __url->getOption("channelConfig");
    if(!channelConfig.empty()){
      node.addNodeEvent(Event(IChannelInfo::setChannelConfigFile_event,
                              new TInValue<string>(channelConfig)), INode::CONSTRUCTED);
    }

  }


  list<string> GenericTVURLObject::getInterfaces() const 
  {
    list<string> l;
    l.push_back(ITVCard::ITVCard_type);
    return l;
  }

}

#include "nmm/nmm_suffix.hpp"
