/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "LircEvents.hpp"

#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IEventReceiver.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

struct lirc_config* config;
    int lirc_fd;
    bool send_extended_events;
  
  LircProducer::LircProducer()
    : config(NULL),
      lirc_fd(-1),
      send_extended_events(false)
  {}
  
  LircProducer::~LircProducer(){
    lirc_freeconfig(config);
    lirc_deinit();
  }
  
  bool LircProducer::readConfig(string fname) {
    if (-1 != lirc_fd) {
      if (config) {
        lirc_freeconfig(config);
        config = NULL;
      }
      lirc_deinit();
    }
    if ((lirc_fd = lirc_init((char*)"nmmlirc", 1)) == -1)
    {
      GLOBAL_ERROR_STREAM( "LircProducer: lirc_init failed" << endl);
      return false;
    }
    
    setFileDescriptor(lirc_fd);
    
    if(lirc_readconfig((char*)fname.c_str(), &config, NULL)!=0)
    {
      GLOBAL_ERROR_STREAM( "LircProducer: lirc_readconfig failed: " << fname << endl);
      return false;
    }
    return true;
  }
  
  void LircProducer::setSendExtendedEvents(bool sendExEv) {
    send_extended_events = sendExEv;
  }
  
  bool LircProducer::getSendExtendedEvents() {
    return send_extended_events;
  }
  
  void LircProducer::handleInput() {
    char* code;
    char* c;
    int   ret;
    
    if (lirc_nextcode(&code)==0) {
      if(code==NULL) return;
      while ((ret=lirc_code2char(config,code,&c))==0 && c!=NULL) {
        GLOBAL_DEBUG_STREAM( "Got New Event: " << c << endl);
        if (send_extended_events) {
          Event event(c, new TInValue<string>(c));
	  if (getEventReceiver()) {
            getEventReceiver()->receiveEvent(event, this);
	  } else {
            GLOBAL_DEBUG_STREAM( "No Event receiver ! " <<  endl);	     
	  }
        }
        else {
          Event event(c);
	  if (getEventReceiver()) {
            getEventReceiver()->receiveEvent(event, this);
	  } else {
	    GLOBAL_DEBUG_STREAM( "No Event receiver ! " <<  endl);	     
	  }
        }
      }
      free(code);
      if(ret==-1) {
        GLOBAL_ERROR_STREAM("LircProducer> Error computing char from code!" << endl);
      }
    } // endif (lirc_nextcode(&code)==0)
  }
  
}


#include "nmm/nmm_suffix.hpp"
