/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef _NMM_LIRCEVENTS_HPP_
#define _NMM_LIRCEVENTS_HPP_

#include "nmm/nmm_prefix.hpp"

#include <lirc/lirc_client.h>

#include "nmm/multimedia/InputProducer.hpp"

namespace NMM {
  
  class LircProducer
    : public InputProducer {
    
  private:
    struct lirc_config* config;
    int lirc_fd;
    bool send_extended_events;
    
    /**
     * The event-handling function
     * Events can be received by all registered input-devices
     * (like X-Display, Console, mouse, ...)
    */
    void handleInput();
    
  public:
    LircProducer();
    ~LircProducer();
    
    /**
     * Reads a LIRC-configuration file.
     * In the configuration file are informations how key presses
     * will be mapped to the corresponding event-strings
     */
    bool readConfig(std::string);
    
    /**
     * Sets the flag to decide whether to send an extended event
     */
    void setSendExtendedEvents(bool);
    
    /**
     * Gets the flag to decide whether to send an extended event
     */
    bool getSendExtendedEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
