/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#ifndef NMM_FLUSH_EVENT_HPP
#define NMM_FLUSH_EVENT_HPP NMM_FLUSH_EVENT_HPP

#include "nmm/nmm_prefix.hpp"

#include "visibility.hpp"
#include "nmm/comm/messaging/CEvent.hpp"

namespace NMM {

  /** Flushing composite event.
   * This is a special composite event which, when inserted into a StreamQueue, causes the StreamQueue
   * to be flushed. This means that all messages in the queue with a timestamp older than the timestamp
   * of the FlushEvent are deleted. Like a regular CEvent, a FlushEvent also carries instream events
   * which are not deleted upon flushing of the queue.
   */
  class DLLEXPORT FlushEvent : public CEvent {
  public:
    /** Create a new flush event.
     */
    FlushEvent();

    /** Copy constructor.
     *
     * @param e The original object
     */
    FlushEvent(const FlushEvent& e);

    /** Virtual destructor.
     */
    ~FlushEvent();

    /** Virtual copy constructor.
     *
     * @return A copy of this object which has the exact same type
     */
    virtual FlushEvent* copy() const  {return new FlushEvent(*this);}

    /** Type name of the FlushEvent type. This is "FlushEvent". */
    static const std::string type_name;
  };

}
#include "nmm/nmm_suffix.hpp"

#endif
