/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "GenericConverterNode.hpp"
#include "nmm/comm/messaging/Buffer.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

void GenericConverterNode::addConverter(NMM::Format* i_format, NMM::Format* o_format, Converter converter)
{
  i_format->setIOPartner(o_format);
  o_format->setIOPartner(i_format);

  map<pair<const Format*,const Format*>, Converter >::iterator tmp_iterator;

  DEBUG_STREAM2( "searching for existing converter" << endl);
  pair<const Format*, const Format*> format(i_format,o_format);

  for(tmp_iterator = converter_map.begin();tmp_iterator != converter_map.end(); tmp_iterator++)
  {
    if ( (*tmp_iterator).first.first->isFormatMatching(i_format) && (*tmp_iterator).first.second->isFormatMatching(o_format) )
    {
      WARNING_STREAM2( "Replace existing converter" << endl);
      converter_map.erase(tmp_iterator);
    }
  }
  converter_map.insert( pair< const pair<const Format*, const Format*>, const Converter> (format, converter) );
}

void GenericConverterNode::addConverter(Format* i_format, Format* o_format, ConverterObject* converter)
{
  i_format->setIOPartner(o_format);
  o_format->setIOPartner(i_format);

  map<pair<const Format*,const Format*>, ConverterObject* >::iterator it;

  DEBUG_STREAM2( "searching for existing converter" << endl);
  pair<const Format*, const Format*> format(i_format,o_format);

  for(it = object_converter_map.begin(); it != object_converter_map.end(); it++)
  {
    if ( (*it).first.first->isFormatMatching(i_format) && (*it).first.second->isFormatMatching(o_format) )
    {
      WARNING_STREAM2( "Replace existing converter" << endl);
      object_converter_map.erase(it);
    }
  }
  object_converter_map.insert( pair< const pair<const Format*, const Format*>, ConverterObject*> (format, converter) );
}

Message* GenericConverterNode::processBuffer(Buffer* in_buffer) {
  return converter(in_buffer, this);
}


void GenericConverterNode::activate() 
  throw (const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&) 
{
  GenericProcessorNode::activate();

  DEBUG_STREAM("Doing additional things in GenericConverterNode::activate()...");
  map<pair<const Format*, const Format*>, Converter >::iterator tmp_iterator;
  if (getInputFormat() == 0)  {
    ERROR_STREAM2( "Invalid input-format (NULL ptr)" << endl);
    throw StateTransitionFailedException("Invalid input-format (NULL ptr)");
  }
  if (getOutputFormat() == 0) {
    ERROR_STREAM2( "Invalid output-format (NULL ptr)" << endl);
    throw StateTransitionFailedException("Invalid input-format (NULL ptr)");
  }

  // FIXME this might not be needed anymore, since getInput|OutputJack only sets the formats if a matching format can be found!
  for(tmp_iterator = converter_map.begin();tmp_iterator != converter_map.end(); tmp_iterator++)
  {
    if ( ((*tmp_iterator).first.first->isFormatMatching(getInputFormat()) ) && ( (*tmp_iterator).first.second->isFormatMatching(getOutputFormat()) ) )
    {
      DEBUG_STREAM2( "Found a valid converter" << endl);
      converter = (*tmp_iterator).second;
      DEBUG_STREAM("Additional things in GenericConverterNode::activate() done.");
      return;
    }
  }
  throw StateTransitionFailedException("No valid converter in converter_map");
}
}//end namespace

#include "nmm/nmm_suffix.hpp"
