/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_GENERIC_DEMULTIPLEXER_NODE_HPP
#define NMM_GENERIC_DEMULTIPLEXER_NODE_HPP NMM_GENERIC_DEMULTIPLEXER_NODE_HPP

#include "nmm/nmm_prefix.hpp"

#include "GenericNode.hpp"

#include "visibility.hpp"

namespace NMM {

  /** Generic demultiplexer node.
   * This is the abstract base class of all demultiplexer nodes.
   * A demultiplexer node has exactly one input stream, which is named "default", and multiple output streams.
   * The node type of a GenericDemultiplexerNode is always INode::DEMULTIPLEXER.
   */
  class DLLEXPORT GenericDemultiplexerNode: public GenericNode {

  private:

  protected:

  public :

    /** Create a new demultiplexer node.
     *
     * @param name Name of the node
     * @param queue_mode Initial queue mode for all input queues
     * @param size Initial maximum queue size for all input queues
     */
    GenericDemultiplexerNode(const char* name = "GenericDemultiplexerNode",
                             const StreamQueue::Mode queue_mode = StreamQueue::DEFAULT_MODE,
                             const int size = StreamQueue::DEFAULT_SIZE);

    /** Virtual destructor.
     */
    virtual ~GenericDemultiplexerNode();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
