/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_GENERIC_FILTER_NODE_HPP
#define NMM_GENERIC_FILTER_NODE_HPP NMM_GENERIC_FILTER_NODE_HPP

#include "nmm/nmm_prefix.hpp"

#include "GenericNode.hpp"
#include "IFilterNodeImpl.hpp"

#include "visibility.hpp"

namespace NMM {

  /** Generic filter node.
   * This is the abstract base class of all filter nodes. A filter node has exactly one input stream
   * and one output stream, both named "default". The node type of a GenericFilterNode is always
   * INode::FILTER.
   *
   * The output format of a filter node is always identical to the input format.
   * Filter nodes can be enabled or disabled. A disabled filter node simply passes buffers through unchanged.
   */
  class DLLEXPORT GenericFilterNode: public GenericNode, public IFilterNodeImpl {

  private:
    bool enabled;       ///< Filter is enabled
  protected:
    // IFilterNode interface
    /** Enable or disable this node.
     * @param enable @c true to enable or @c false to disable this node
     */
    virtual void setEnabled(const nmm_bool& enable) { enabled = enable; }

    /** Check if this node is enabled.
     * @return @c true if this node is enabled, otherwise @c false
     */
    virtual nmm_bool isEnabled()  { return enabled; }

  public :

    /** Create a new filter node.
     *
     * @param name Name of the node
     * @param queue_mode Initial queue mode for all input queues
     * @param size Initial maximum queue size for all input queues
     */
    GenericFilterNode(const char* name = "GenericFilterNode",
                      const StreamQueue::Mode queue_mode = StreamQueue::DEFAULT_MODE,
                      const int size = StreamQueue::DEFAULT_SIZE);

    /** Virtual destructor.
     */
    virtual ~GenericFilterNode();
    
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
