/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_GENERIC_MULTIPLEXER_NODE_HPP
#define NMM_GENERIC_MULTIPLEXER_NODE_HPP

#include "nmm/nmm_prefix.hpp"

#include "GenericNode.hpp"

#include "visibility.hpp"

namespace NMM {

  /** Generic multiplexer node.
   * This is the abstract base class of all multiplexer nodes.
   * A multiplexer node has multiple input stream and exactly one output stream, which is named "default".
   * The node type of a GenericMultiplexerNode is always INode::MULTIPLEXER.
   */
  class DLLEXPORT GenericMultiplexerNode: public GenericNode {
  public:
    /** Create a new multiplexer node.
     *
     * @param name Name of the node
     * @param queue_mode Initial queue mode for all input queues
     * @param size Initial maximum queue size for all input queues
     */
    GenericMultiplexerNode(const char* name = "GenericMultiplexerNode",
                           const StreamQueue::Mode queue_mode = StreamQueue::DEFAULT_MODE,
                           const int size = StreamQueue::DEFAULT_SIZE);

    /** Virtual destructor.
     */
    virtual ~GenericMultiplexerNode();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
