/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#ifndef NMM_GENERIC_PRODUCER_HPP
#define NMM_GENERIC_PRODUCER_HPP NMM_GENERIC_PRODUCER_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/utils/thread/ThreadMutex.hpp"
#include "nmm/comm/messaging/IEventSender.hpp"

#include "visibility.hpp"

namespace NMM {

  class IEventReceiver;

  /** Generic Producer.
   * A producer is an object which generates NMM messages. This class is the base class of all
   * producer classes. Producers implement the IEventSender interface and send events to an object
   * which implements the IEventReceiver interface. Producers can be started and stopped. A producer
   * only sends events when it is started.
   */
  class DLLEXPORT GenericProducer : public IEventSender {
  public:
    /** Default constructor.
     * The producer is initially stopped.
     */
    GenericProducer();

    /** Destructor.
     * The destructor does not stop the producer. The producer must be stopped explicitly
     * before it is destroyed.
     */
    virtual ~GenericProducer();

    /** Start producing events.
     */
    virtual void start() = 0;

    /** Stop producing events.
     */
    virtual void stop() = 0;

  private:
    ThreadMutex mutex;        ///< An unused mutex
  };
  
}

#include "nmm/nmm_suffix.hpp"

#endif
