/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IBiDirQueuedNode.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/serialize/SerializeUtilsTime.hpp"

namespace NMM
{
  void IBiDirQueuedNode::used() {} 
  
  
  nmm_uint32 IBiDirQueuedNode::getDownstreamSize(const string& id /* = "default" */) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::TInValue<string > id_value(id, false);
    NMM::Event event("IBiDirQueuedNode::getDownstreamSize", &id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IBiDirQueuedNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 IBiDirQueuedNode::getDownstreamMaxSize(const string& id /* = "default" */) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::TInValue<string > id_value(id, false);
    NMM::Event event("IBiDirQueuedNode::getDownstreamMaxSize", &id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IBiDirQueuedNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IQueued::Mode IBiDirQueuedNode::getDownstreamMode(const string& id /* = "default" */) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<IQueued::Mode > return_value;
    NMM::TInValue<string > id_value(id, false);
    NMM::Event event("IBiDirQueuedNode::getDownstreamMode", &id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IBiDirQueuedNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IBiDirQueuedNode::setDownstreamMaxSize(const nmm_uint32& size, const string& id /* = "default" */) /* limited(INode::CONSTRUCTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_uint32 > size_value(size, false);
    NMM::TInValue<string > id_value(id, false);
    NMM::Event event("IBiDirQueuedNode::setDownstreamMaxSize", &size_value, &id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IBiDirQueuedNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IBiDirQueuedNode::setDownstreamMaxDuration(const Interval& duration, const string& id /* = "default" */) /* limited(INode::CONSTRUCTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<Interval > duration_value(duration, false);
    NMM::TInValue<string > id_value(id, false);
    NMM::Event event("IBiDirQueuedNode::setDownstreamMaxDuration", &duration_value, &id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IBiDirQueuedNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IBiDirQueuedNode::setDownstreamMode(const IQueued::Mode& mode, const string& id /* = "default" */) /* limited(INode::CONSTRUCTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<IQueued::Mode > mode_value(mode, false);
    NMM::TInValue<string > id_value(id, false);
    NMM::Event event("IBiDirQueuedNode::setDownstreamMode", &mode_value, &id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IBiDirQueuedNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IBiDirQueuedNode::unlimitDownstreamSize() /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("IBiDirQueuedNode::unlimitDownstreamSize");
    event.setReturnValue(&return_value);
    if(const_cast<IBiDirQueuedNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IBiDirQueuedNode::limitDownstreamSize() /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("IBiDirQueuedNode::limitDownstreamSize");
    event.setReturnValue(&return_value);
    if(const_cast<IBiDirQueuedNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 IBiDirQueuedNode::getUpstreamSize(const string& id /* = "default" */) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::TInValue<string > id_value(id, false);
    NMM::Event event("IBiDirQueuedNode::getUpstreamSize", &id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IBiDirQueuedNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 IBiDirQueuedNode::getUpstreamMaxSize(const string& id /* = "default" */) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::TInValue<string > id_value(id, false);
    NMM::Event event("IBiDirQueuedNode::getUpstreamMaxSize", &id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IBiDirQueuedNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IQueued::Mode IBiDirQueuedNode::getUpstreamMode(const string& id /* = "default" */) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<IQueued::Mode > return_value;
    NMM::TInValue<string > id_value(id, false);
    NMM::Event event("IBiDirQueuedNode::getUpstreamMode", &id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IBiDirQueuedNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IBiDirQueuedNode::setUpstreamMaxSize(const nmm_uint32& size, const string& id /* = "default" */) /* limited(INode::CONSTRUCTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_uint32 > size_value(size, false);
    NMM::TInValue<string > id_value(id, false);
    NMM::Event event("IBiDirQueuedNode::setUpstreamMaxSize", &size_value, &id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IBiDirQueuedNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IBiDirQueuedNode::setUpstreamMode(const IQueued::Mode& mode, const string& id /* = "default" */) /* limited(INode::CONSTRUCTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<IQueued::Mode > mode_value(mode, false);
    NMM::TInValue<string > id_value(id, false);
    NMM::Event event("IBiDirQueuedNode::setUpstreamMode", &mode_value, &id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IBiDirQueuedNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IBiDirQueuedNode::setMaxSize(const nmm_uint32& size, const string& id /* = "default" */) /* limited(INode::CONSTRUCTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_uint32 > size_value(size, false);
    NMM::TInValue<string > id_value(id, false);
    NMM::Event event("IBiDirQueuedNode::setMaxSize", &size_value, &id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IBiDirQueuedNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IBiDirQueuedNode::setMode(const IQueued::Mode& mode, const string& id /* = "default" */) /* limited(INode::CONSTRUCTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<IQueued::Mode > mode_value(mode, false);
    NMM::TInValue<string > id_value(id, false);
    NMM::Event event("IBiDirQueuedNode::setMode", &mode_value, &id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IBiDirQueuedNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IBiDirQueuedNode::IBiDirQueuedNode_type = "IBiDirQueuedNode";
  NMM::TConstructor<IBiDirQueuedNode > __IBiDirQueuedNode_prototype("IBiDirQueuedNode","NMM");


  const char* IBiDirQueuedNode::getDownstreamSize_event = "IBiDirQueuedNode::getDownstreamSize";
  const char* IBiDirQueuedNode::getDownstreamMaxSize_event = "IBiDirQueuedNode::getDownstreamMaxSize";
  const char* IBiDirQueuedNode::getDownstreamMode_event = "IBiDirQueuedNode::getDownstreamMode";
  const char* IBiDirQueuedNode::setDownstreamMaxSize_event = "IBiDirQueuedNode::setDownstreamMaxSize";
  const char* IBiDirQueuedNode::setDownstreamMaxDuration_event = "IBiDirQueuedNode::setDownstreamMaxDuration";
  const char* IBiDirQueuedNode::setDownstreamMode_event = "IBiDirQueuedNode::setDownstreamMode";
  const char* IBiDirQueuedNode::unlimitDownstreamSize_event = "IBiDirQueuedNode::unlimitDownstreamSize";
  NMM::Event* IBiDirQueuedNode::create_unlimitDownstreamSize()
  {

    return new NMM::Event("IBiDirQueuedNode::unlimitDownstreamSize");
  }

  const char* IBiDirQueuedNode::limitDownstreamSize_event = "IBiDirQueuedNode::limitDownstreamSize";
  NMM::Event* IBiDirQueuedNode::create_limitDownstreamSize()
  {

    return new NMM::Event("IBiDirQueuedNode::limitDownstreamSize");
  }

  const char* IBiDirQueuedNode::getUpstreamSize_event = "IBiDirQueuedNode::getUpstreamSize";
  const char* IBiDirQueuedNode::getUpstreamMaxSize_event = "IBiDirQueuedNode::getUpstreamMaxSize";
  const char* IBiDirQueuedNode::getUpstreamMode_event = "IBiDirQueuedNode::getUpstreamMode";
  const char* IBiDirQueuedNode::setUpstreamMaxSize_event = "IBiDirQueuedNode::setUpstreamMaxSize";
  const char* IBiDirQueuedNode::setUpstreamMode_event = "IBiDirQueuedNode::setUpstreamMode";
  const char* IBiDirQueuedNode::setMaxSize_event = "IBiDirQueuedNode::setMaxSize";
  const char* IBiDirQueuedNode::setMode_event = "IBiDirQueuedNode::setMode";


}


#include "nmm/nmm_suffix.hpp"

