/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IBiDirQueuedNodeImpl.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/messaging/TEDObject.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/serialize/SerializeUtilsTime.hpp"

namespace NMM
{
  
  void IBiDirQueuedNodeImpl::registerEvents()
  {
  
    getEventDispatcher().registerEvent(IBiDirQueuedNode::getDownstreamSize_event, new NMM::TEDObject1<IBiDirQueuedNodeImpl, string, nmm_uint32(IBiDirQueuedNodeImpl:: * )(const string&) const, nmm_uint32 >(this, &IBiDirQueuedNodeImpl::getDownstreamSize));
  
    getEventDispatcher().registerEvent(IBiDirQueuedNode::getDownstreamMaxSize_event, new NMM::TEDObject1<IBiDirQueuedNodeImpl, string, nmm_uint32(IBiDirQueuedNodeImpl:: * )(const string&) const, nmm_uint32 >(this, &IBiDirQueuedNodeImpl::getDownstreamMaxSize));
  
    getEventDispatcher().registerEvent(IBiDirQueuedNode::getDownstreamMode_event, new NMM::TEDObject1<IBiDirQueuedNodeImpl, string, IQueued::Mode(IBiDirQueuedNodeImpl:: * )(const string&) const, IQueued::Mode >(this, &IBiDirQueuedNodeImpl::getDownstreamMode));
  
    getEventDispatcher().registerEvent(IBiDirQueuedNode::setDownstreamMaxSize_event, new NMM::TEDObject2<IBiDirQueuedNodeImpl, nmm_uint32, string, Result(IBiDirQueuedNodeImpl:: * )(const nmm_uint32&, const string&), Result >(this, &IBiDirQueuedNodeImpl::setDownstreamMaxSize));
    getEventDispatcher().addLimited(IBiDirQueuedNode::setDownstreamMaxSize_event, INode::CONSTRUCTED);
  
    getEventDispatcher().registerEvent(IBiDirQueuedNode::setDownstreamMaxDuration_event, new NMM::TEDObject2<IBiDirQueuedNodeImpl, Interval, string, Result(IBiDirQueuedNodeImpl:: * )(const Interval&, const string&), Result >(this, &IBiDirQueuedNodeImpl::setDownstreamMaxDuration));
    getEventDispatcher().addLimited(IBiDirQueuedNode::setDownstreamMaxDuration_event, INode::CONSTRUCTED);
  
    getEventDispatcher().registerEvent(IBiDirQueuedNode::setDownstreamMode_event, new NMM::TEDObject2<IBiDirQueuedNodeImpl, IQueued::Mode, string, Result(IBiDirQueuedNodeImpl:: * )(const IQueued::Mode&, const string&), Result >(this, &IBiDirQueuedNodeImpl::setDownstreamMode));
    getEventDispatcher().addLimited(IBiDirQueuedNode::setDownstreamMode_event, INode::CONSTRUCTED);
  
    getEventDispatcher().registerEvent(IBiDirQueuedNode::unlimitDownstreamSize_event, new NMM::TEDObject0<IBiDirQueuedNodeImpl, Result(IBiDirQueuedNodeImpl:: * )(), Result >(this, &IBiDirQueuedNodeImpl::unlimitDownstreamSize));
  
    getEventDispatcher().registerEvent(IBiDirQueuedNode::limitDownstreamSize_event, new NMM::TEDObject0<IBiDirQueuedNodeImpl, Result(IBiDirQueuedNodeImpl:: * )(), Result >(this, &IBiDirQueuedNodeImpl::limitDownstreamSize));
  
    getEventDispatcher().registerEvent(IBiDirQueuedNode::getUpstreamSize_event, new NMM::TEDObject1<IBiDirQueuedNodeImpl, string, nmm_uint32(IBiDirQueuedNodeImpl:: * )(const string&) const, nmm_uint32 >(this, &IBiDirQueuedNodeImpl::getUpstreamSize));
  
    getEventDispatcher().registerEvent(IBiDirQueuedNode::getUpstreamMaxSize_event, new NMM::TEDObject1<IBiDirQueuedNodeImpl, string, nmm_uint32(IBiDirQueuedNodeImpl:: * )(const string&) const, nmm_uint32 >(this, &IBiDirQueuedNodeImpl::getUpstreamMaxSize));
  
    getEventDispatcher().registerEvent(IBiDirQueuedNode::getUpstreamMode_event, new NMM::TEDObject1<IBiDirQueuedNodeImpl, string, IQueued::Mode(IBiDirQueuedNodeImpl:: * )(const string&) const, IQueued::Mode >(this, &IBiDirQueuedNodeImpl::getUpstreamMode));
  
    getEventDispatcher().registerEvent(IBiDirQueuedNode::setUpstreamMaxSize_event, new NMM::TEDObject2<IBiDirQueuedNodeImpl, nmm_uint32, string, Result(IBiDirQueuedNodeImpl:: * )(const nmm_uint32&, const string&), Result >(this, &IBiDirQueuedNodeImpl::setUpstreamMaxSize));
    getEventDispatcher().addLimited(IBiDirQueuedNode::setUpstreamMaxSize_event, INode::CONSTRUCTED);
  
    getEventDispatcher().registerEvent(IBiDirQueuedNode::setUpstreamMode_event, new NMM::TEDObject2<IBiDirQueuedNodeImpl, IQueued::Mode, string, Result(IBiDirQueuedNodeImpl:: * )(const IQueued::Mode&, const string&), Result >(this, &IBiDirQueuedNodeImpl::setUpstreamMode));
    getEventDispatcher().addLimited(IBiDirQueuedNode::setUpstreamMode_event, INode::CONSTRUCTED);
  
    getEventDispatcher().registerEvent(IBiDirQueuedNode::setMaxSize_event, new NMM::TEDObject2<IBiDirQueuedNodeImpl, nmm_uint32, string, Result(IBiDirQueuedNodeImpl:: * )(const nmm_uint32&, const string&), Result >(this, &IBiDirQueuedNodeImpl::setMaxSize));
    getEventDispatcher().addLimited(IBiDirQueuedNode::setMaxSize_event, INode::CONSTRUCTED);
  
    getEventDispatcher().registerEvent(IBiDirQueuedNode::setMode_event, new NMM::TEDObject2<IBiDirQueuedNodeImpl, IQueued::Mode, string, Result(IBiDirQueuedNodeImpl:: * )(const IQueued::Mode&, const string&), Result >(this, &IBiDirQueuedNodeImpl::setMode));
    getEventDispatcher().addLimited(IBiDirQueuedNode::setMode_event, INode::CONSTRUCTED);

  }

}


#include "nmm/nmm_suffix.hpp"

