/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IBufferManager.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IBufferManager::used() {} 
  
  
  NMMReference IBufferManager::getBufferManager()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<NMMReference > return_value;
    NMM::Event event("IBufferManager::getBufferManager");
    event.setReturnValue(&return_value);
    if(const_cast<IBufferManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IBufferManager::useBufferManager(const IBufferManager& manager)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<IBufferManager > manager_value(manager, false);
    NMM::Event event("IBufferManager::useBufferManager", &manager_value);
    event.setReturnValue(&return_value);
    if(const_cast<IBufferManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IBufferManager::releaseBufferManager()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IBufferManager::releaseBufferManager");
    if(const_cast<IBufferManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IBufferManager::IBufferManager_type = "IBufferManager";
  NMM::TConstructor<IBufferManager > __IBufferManager_prototype("IBufferManager","NMM");


  const char* IBufferManager::getBufferManager_event = "IBufferManager::getBufferManager";
  const char* IBufferManager::useBufferManager_event = "IBufferManager::useBufferManager";
  const char* IBufferManager::releaseBufferManager_event = "IBufferManager::releaseBufferManager";


}


#include "nmm/nmm_suffix.hpp"

