/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IDelayMeasurement.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/serialize/SerializeUtilsTime.hpp"

namespace NMM
{
  void IDelayMeasurement::used() {} 
  
  
  void IDelayMeasurement::setDelayStartEnabled(const IDelayMeasurement::StartTimepoint& start_timepoint, const nmm_bool& enabled)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<IDelayMeasurement::StartTimepoint > start_timepoint_value(start_timepoint, false);
    NMM::TInValue<nmm_bool > enabled_value(enabled, false);
    NMM::Event event("IDelayMeasurement::setDelayStartEnabled", &start_timepoint_value, &enabled_value);
    if(const_cast<IDelayMeasurement*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IDelayMeasurement::setDelayStopEnabled(const IDelayMeasurement::StopTimepoint& stop_timepoint, const nmm_bool& enabled)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<IDelayMeasurement::StopTimepoint > stop_timepoint_value(stop_timepoint, false);
    NMM::TInValue<nmm_bool > enabled_value(enabled, false);
    NMM::Event event("IDelayMeasurement::setDelayStopEnabled", &stop_timepoint_value, &enabled_value);
    if(const_cast<IDelayMeasurement*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IDelayMeasurement::setDelayStartEnabled2(const string& start_timepoint, const nmm_bool& enabled)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > start_timepoint_value(start_timepoint, false);
    NMM::TInValue<nmm_bool > enabled_value(enabled, false);
    NMM::Event event("IDelayMeasurement::setDelayStartEnabled2", &start_timepoint_value, &enabled_value);
    if(const_cast<IDelayMeasurement*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IDelayMeasurement::setDelayStopEnabled2(const string& stop_timepoint, const nmm_bool& enabled)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > stop_timepoint_value(stop_timepoint, false);
    NMM::TInValue<nmm_bool > enabled_value(enabled, false);
    NMM::Event event("IDelayMeasurement::setDelayStopEnabled2", &stop_timepoint_value, &enabled_value);
    if(const_cast<IDelayMeasurement*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_bool IDelayMeasurement::isDelayStartEnabled(const IDelayMeasurement::StartTimepoint& start_timepoint) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::TInValue<IDelayMeasurement::StartTimepoint > start_timepoint_value(start_timepoint, false);
    NMM::Event event("IDelayMeasurement::isDelayStartEnabled", &start_timepoint_value);
    event.setReturnValue(&return_value);
    if(const_cast<IDelayMeasurement*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool IDelayMeasurement::isDelayStopEnabled(const IDelayMeasurement::StopTimepoint& stop_timepoint) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::TInValue<IDelayMeasurement::StopTimepoint > stop_timepoint_value(stop_timepoint, false);
    NMM::Event event("IDelayMeasurement::isDelayStopEnabled", &stop_timepoint_value);
    event.setReturnValue(&return_value);
    if(const_cast<IDelayMeasurement*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Interval IDelayMeasurement::getAverageDelay(const IDelayMeasurement::StopTimepoint& stop_timepoint)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Interval > return_value;
    NMM::TInValue<IDelayMeasurement::StopTimepoint > stop_timepoint_value(stop_timepoint, false);
    NMM::Event event("IDelayMeasurement::getAverageDelay", &stop_timepoint_value);
    event.setReturnValue(&return_value);
    if(const_cast<IDelayMeasurement*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Interval IDelayMeasurement::getMinDelay(const IDelayMeasurement::StopTimepoint& stop_timepoint)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Interval > return_value;
    NMM::TInValue<IDelayMeasurement::StopTimepoint > stop_timepoint_value(stop_timepoint, false);
    NMM::Event event("IDelayMeasurement::getMinDelay", &stop_timepoint_value);
    event.setReturnValue(&return_value);
    if(const_cast<IDelayMeasurement*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Interval IDelayMeasurement::getMaxDelay(const IDelayMeasurement::StopTimepoint& stop_timepoint)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Interval > return_value;
    NMM::TInValue<IDelayMeasurement::StopTimepoint > stop_timepoint_value(stop_timepoint, false);
    NMM::Event event("IDelayMeasurement::getMaxDelay", &stop_timepoint_value);
    event.setReturnValue(&return_value);
    if(const_cast<IDelayMeasurement*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IDelayMeasurement::resetDelayMeasurement(const IDelayMeasurement::StopTimepoint& stop_timepoint)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<IDelayMeasurement::StopTimepoint > stop_timepoint_value(stop_timepoint, false);
    NMM::Event event("IDelayMeasurement::resetDelayMeasurement", &stop_timepoint_value);
    if(const_cast<IDelayMeasurement*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IDelayMeasurement::setNumberOfDelaySamples(const nmm_uint32& number_of_samples)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > number_of_samples_value(number_of_samples, false);
    NMM::Event event("IDelayMeasurement::setNumberOfDelaySamples", &number_of_samples_value);
    if(const_cast<IDelayMeasurement*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IDelayMeasurement::setDelayLoggingEnabled(const nmm_bool& enabled)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > enabled_value(enabled, false);
    NMM::Event event("IDelayMeasurement::setDelayLoggingEnabled", &enabled_value);
    if(const_cast<IDelayMeasurement*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IDelayMeasurement::setDelayLoggingInterval(const nmm_uint32& interval)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > interval_value(interval, false);
    NMM::Event event("IDelayMeasurement::setDelayLoggingInterval", &interval_value);
    if(const_cast<IDelayMeasurement*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IDelayMeasurement::IDelayMeasurement_type = "IDelayMeasurement";
  NMM::TConstructor<IDelayMeasurement > __IDelayMeasurement_prototype("IDelayMeasurement","NMM");
  TValueConstructor< IDelayMeasurement::StartTimepoint > __IDelayMeasurement_StartTimepoint_prototype(TypeInfo("IDelayMeasurement::StartTimepoint", "NMM"));

  NMM::OValueStream& operator<<(NMM::OValueStream& ov, const IDelayMeasurement::StartTimepoint& t)
  {
    ov.beginType(TypeInfo("IDelayMeasurement::StartTimepoint", "NMM"));
    nmm_int32 temp = static_cast<nmm_int32>(t);
    ov << temp;
    ov.endType();
    return ov;
  }

  NMM::IValueStream& operator>>(NMM::IValueStream& iv, IDelayMeasurement::StartTimepoint& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("IDelayMeasurement::StartTimepoint", "NMM")) {
      throw NMM::InvalidTypeException(TypeInfo("IDelayMeasurement::StartTimepoint", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    nmm_int32 temp;
    iv >> temp;
    t = static_cast<IDelayMeasurement::StartTimepoint>(temp);
    iv.goUp();
    return iv;
  }

  TValueConstructor< IDelayMeasurement::StopTimepoint > __IDelayMeasurement_StopTimepoint_prototype(TypeInfo("IDelayMeasurement::StopTimepoint", "NMM"));

  NMM::OValueStream& operator<<(NMM::OValueStream& ov, const IDelayMeasurement::StopTimepoint& t)
  {
    ov.beginType(TypeInfo("IDelayMeasurement::StopTimepoint", "NMM"));
    nmm_int32 temp = static_cast<nmm_int32>(t);
    ov << temp;
    ov.endType();
    return ov;
  }

  NMM::IValueStream& operator>>(NMM::IValueStream& iv, IDelayMeasurement::StopTimepoint& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("IDelayMeasurement::StopTimepoint", "NMM")) {
      throw NMM::InvalidTypeException(TypeInfo("IDelayMeasurement::StopTimepoint", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    nmm_int32 temp;
    iv >> temp;
    t = static_cast<IDelayMeasurement::StopTimepoint>(temp);
    iv.goUp();
    return iv;
  }



  const char* IDelayMeasurement::setDelayStartEnabled_event = "IDelayMeasurement::setDelayStartEnabled";
  const char* IDelayMeasurement::setDelayStopEnabled_event = "IDelayMeasurement::setDelayStopEnabled";
  const char* IDelayMeasurement::setDelayStartEnabled2_event = "IDelayMeasurement::setDelayStartEnabled2";
  const char* IDelayMeasurement::setDelayStopEnabled2_event = "IDelayMeasurement::setDelayStopEnabled2";
  const char* IDelayMeasurement::isDelayStartEnabled_event = "IDelayMeasurement::isDelayStartEnabled";
  const char* IDelayMeasurement::isDelayStopEnabled_event = "IDelayMeasurement::isDelayStopEnabled";
  const char* IDelayMeasurement::getAverageDelay_event = "IDelayMeasurement::getAverageDelay";
  const char* IDelayMeasurement::getMinDelay_event = "IDelayMeasurement::getMinDelay";
  const char* IDelayMeasurement::getMaxDelay_event = "IDelayMeasurement::getMaxDelay";
  const char* IDelayMeasurement::resetDelayMeasurement_event = "IDelayMeasurement::resetDelayMeasurement";
  const char* IDelayMeasurement::setNumberOfDelaySamples_event = "IDelayMeasurement::setNumberOfDelaySamples";
  const char* IDelayMeasurement::setDelayLoggingEnabled_event = "IDelayMeasurement::setDelayLoggingEnabled";
  const char* IDelayMeasurement::setDelayLoggingInterval_event = "IDelayMeasurement::setDelayLoggingInterval";


}


#include "nmm/nmm_suffix.hpp"

