/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IDelayMeasurementImpl.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/messaging/TEDObject.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/serialize/SerializeUtilsTime.hpp"

namespace NMM
{
  
  void IDelayMeasurementImpl::registerEvents()
  {
  
    getEventDispatcher().registerEvent(IDelayMeasurement::setDelayStartEnabled_event, new NMM::TEDObject2<IDelayMeasurementImpl, IDelayMeasurement::StartTimepoint, nmm_bool, void(IDelayMeasurementImpl:: * )(const IDelayMeasurement::StartTimepoint&, const nmm_bool&), void >(this, &IDelayMeasurementImpl::setDelayStartEnabled));
  
    getEventDispatcher().registerEvent(IDelayMeasurement::setDelayStopEnabled_event, new NMM::TEDObject2<IDelayMeasurementImpl, IDelayMeasurement::StopTimepoint, nmm_bool, void(IDelayMeasurementImpl:: * )(const IDelayMeasurement::StopTimepoint&, const nmm_bool&), void >(this, &IDelayMeasurementImpl::setDelayStopEnabled));
  
    getEventDispatcher().registerEvent(IDelayMeasurement::setDelayStartEnabled2_event, new NMM::TEDObject2<IDelayMeasurementImpl, string, nmm_bool, void(IDelayMeasurementImpl:: * )(const string&, const nmm_bool&), void >(this, &IDelayMeasurementImpl::setDelayStartEnabled2));
  
    getEventDispatcher().registerEvent(IDelayMeasurement::setDelayStopEnabled2_event, new NMM::TEDObject2<IDelayMeasurementImpl, string, nmm_bool, void(IDelayMeasurementImpl:: * )(const string&, const nmm_bool&), void >(this, &IDelayMeasurementImpl::setDelayStopEnabled2));
  
    getEventDispatcher().registerEvent(IDelayMeasurement::isDelayStartEnabled_event, new NMM::TEDObject1<IDelayMeasurementImpl, IDelayMeasurement::StartTimepoint, nmm_bool(IDelayMeasurementImpl:: * )(const IDelayMeasurement::StartTimepoint&) const, nmm_bool >(this, &IDelayMeasurementImpl::isDelayStartEnabled));
  
    getEventDispatcher().registerEvent(IDelayMeasurement::isDelayStopEnabled_event, new NMM::TEDObject1<IDelayMeasurementImpl, IDelayMeasurement::StopTimepoint, nmm_bool(IDelayMeasurementImpl:: * )(const IDelayMeasurement::StopTimepoint&) const, nmm_bool >(this, &IDelayMeasurementImpl::isDelayStopEnabled));
  
    getEventDispatcher().registerEvent(IDelayMeasurement::getAverageDelay_event, new NMM::TEDObject1<IDelayMeasurementImpl, IDelayMeasurement::StopTimepoint, Interval(IDelayMeasurementImpl:: * )(const IDelayMeasurement::StopTimepoint&), Interval >(this, &IDelayMeasurementImpl::getAverageDelay));
  
    getEventDispatcher().registerEvent(IDelayMeasurement::getMinDelay_event, new NMM::TEDObject1<IDelayMeasurementImpl, IDelayMeasurement::StopTimepoint, Interval(IDelayMeasurementImpl:: * )(const IDelayMeasurement::StopTimepoint&), Interval >(this, &IDelayMeasurementImpl::getMinDelay));
  
    getEventDispatcher().registerEvent(IDelayMeasurement::getMaxDelay_event, new NMM::TEDObject1<IDelayMeasurementImpl, IDelayMeasurement::StopTimepoint, Interval(IDelayMeasurementImpl:: * )(const IDelayMeasurement::StopTimepoint&), Interval >(this, &IDelayMeasurementImpl::getMaxDelay));
  
    getEventDispatcher().registerEvent(IDelayMeasurement::resetDelayMeasurement_event, new NMM::TEDObject1<IDelayMeasurementImpl, IDelayMeasurement::StopTimepoint, void(IDelayMeasurementImpl:: * )(const IDelayMeasurement::StopTimepoint&), void >(this, &IDelayMeasurementImpl::resetDelayMeasurement));
  
    getEventDispatcher().registerEvent(IDelayMeasurement::setNumberOfDelaySamples_event, new NMM::TEDObject1<IDelayMeasurementImpl, nmm_uint32, void(IDelayMeasurementImpl:: * )(const nmm_uint32&), void >(this, &IDelayMeasurementImpl::setNumberOfDelaySamples));
  
    getEventDispatcher().registerEvent(IDelayMeasurement::setDelayLoggingEnabled_event, new NMM::TEDObject1<IDelayMeasurementImpl, nmm_bool, void(IDelayMeasurementImpl:: * )(const nmm_bool&), void >(this, &IDelayMeasurementImpl::setDelayLoggingEnabled));
  
    getEventDispatcher().registerEvent(IDelayMeasurement::setDelayLoggingInterval_event, new NMM::TEDObject1<IDelayMeasurementImpl, nmm_uint32, void(IDelayMeasurementImpl:: * )(const nmm_uint32&), void >(this, &IDelayMeasurementImpl::setDelayLoggingInterval));

  }

}


#include "nmm/nmm_suffix.hpp"

