/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IDevice.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IDevice::used() {} 
  
  
  void IDevice::setDevice(const string& value) /* limited(INode::CONSTRUCTED) */ 
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&, const DeviceNotFoundException&, const WrongDeviceException&)
  {
    NMM::TInValue<string > value_value(value, false);
    NMM::Event event("IDevice::setDevice", &value_value);
    if(const_cast<IDevice*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        if(DeviceNotFoundException* ex = dynamic_cast<DeviceNotFoundException*>(event.getException())) {
          throw DeviceNotFoundException(*ex);
        }
        if(WrongDeviceException* ex = dynamic_cast<WrongDeviceException*>(event.getException())) {
          throw WrongDeviceException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  string IDevice::getDevice() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("IDevice::getDevice");
    event.setReturnValue(&return_value);
    if(const_cast<IDevice*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IDevice::IDevice_type = "IDevice";
  NMM::TConstructor<IDevice > __IDevice_prototype("IDevice","NMM");


  const char* IDevice::setDevice_event = "IDevice::setDevice";
  const char* IDevice::getDevice_event = "IDevice::getDevice";


}


#include "nmm/nmm_suffix.hpp"

