/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IDevice_HPP
#define NMM_IDevice_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "DeviceNotFoundException.hpp"
#include "WrongDeviceException.hpp"
#include "nmm/multimedia/INode.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Interface for setting or querying device IDs.
   *
   * This interface is implemented by nodes which encapsulate devices, such as audio devices or
   * TV cards. It is used for device management by the plug-in manager.
   */

  class DLLEXPORT IDevice : virtual public NMM::Interface { 
  public:
    IDevice() : NMM::Interface(IDevice::IDevice_type, "NMM") { };

    IDevice(NMM::NMMObject& parent) : NMM::Interface (parent, IDevice::IDevice_type, "NMM") { };

    virtual ~IDevice() {};

    static const char* IDevice_type;

    virtual  string getInterfaceType() const { return IDevice_type; }

    NMM::Interface* copy() const { return new IDevice(*this); }
    static void used(); 
    static const char* setDevice_event;
    static const char* getDevice_event;
    /** Sets the device ID.
     * This method sets the device ID of the device that should be used by the node.
     *
     * Normally, this method is called during plug-in registration.
     * It should not be used by applications, because it bypasses device management by the registry.
     *
     * @param value The device ID
     */

    virtual void setDevice(const string& value) /* limited(INode::CONSTRUCTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&, const DeviceNotFoundException&, const WrongDeviceException&);
    /** Returns the device ID.
     * This method returns the device ID that is used by the node.
     *
     * @return The device ID
     */

    virtual string getDevice() const
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IDevice> IDevice_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
