/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IDeviceList_HPP
#define NMM_IDeviceList_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/serialize/StdValue.hpp"
#include "IDevice.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Interface for requesting a list of available devices.
   */

  class DLLEXPORT IDeviceList : virtual public IDevice { 
  public:
    IDeviceList() : NMM::Interface(IDeviceList::IDeviceList_type, "NMM") { };

    IDeviceList(NMM::NMMObject& parent) : NMM::Interface (parent, IDeviceList::IDeviceList_type, "NMM") { };

    virtual ~IDeviceList() {};

    static const char* IDeviceList_type;

    virtual  string getInterfaceType() const { return IDeviceList_type; }

    NMM::Interface* copy() const { return new IDeviceList(*this); }
    static void used(); 
    static const char* getDeviceList_event;
    /** Requests list of available devices.
     *
     * @return list of device IDs of available devices
     */

    virtual list<string >  getDeviceList()
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IDeviceList> IDeviceList_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
