/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IFilterNode_HPP
#define NMM_IFilterNode_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{
  /** Interface for controlling a filter node.
   */

  class DLLEXPORT IFilterNode : virtual public NMM::Interface { 
  public:
    IFilterNode() : NMM::Interface(IFilterNode::IFilterNode_type, "NMM") { };

    IFilterNode(NMM::NMMObject& parent) : NMM::Interface (parent, IFilterNode::IFilterNode_type, "NMM") { };

    virtual ~IFilterNode() {};

    static const char* IFilterNode_type;

    virtual  string getInterfaceType() const { return IFilterNode_type; }

    NMM::Interface* copy() const { return new IFilterNode(*this); }
    static void used(); 
    static const char* setEnabled_event;
    static const char* isEnabled_event;
    /** Enable or disable this node.
     * @param enable @c true to enable or @c false to disable this node
     */

    virtual void setEnabled(const nmm_bool& enable)
      throw(const NMM::SerializedException&);
    /** Check if this node is enabled.
     * @return @c true if this node is enabled, otherwise @c false
     */

    virtual nmm_bool isEnabled()
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IFilterNode> IFilterNode_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
