/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IFilterNodeImpl_HPP
#define NMM_IFilterNodeImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IFilterNode.hpp"


#include "visibility.hpp"

namespace NMM
{
  /** Interface for controlling a filter node.
   */

  class DLLEXPORT IFilterNodeImpl  : public virtual NMM::NMMObject { 
  public:
    IFilterNodeImpl() { addInterface(new IFilterNode(*this)); registerEvents(); };

    virtual ~IFilterNodeImpl() {};

  protected:
    /** Enable or disable this node.
     * @param enable @c true to enable or @c false to disable this node
     */

    virtual void setEnabled(const nmm_bool& enable) = 0;
    /** Check if this node is enabled.
     * @return @c true if this node is enabled, otherwise @c false
     */

    virtual nmm_bool isEnabled() = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
