/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IFormatChange.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IFormatChange::used() {} 
  
  
  Result IFormatChange::setFormat(const Format& new_format) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<Format > new_format_value(new_format, false);
    NMM::Event event("IFormatChange::setFormat", &new_format_value);
    event.setReturnValue(&return_value);
    if(const_cast<IFormatChange*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IFormatChange::IFormatChange_type = "IFormatChange";
  NMM::TConstructor<IFormatChange > __IFormatChange_prototype("IFormatChange","NMM");


  const char* IFormatChange::setFormat_event = "IFormatChange::setFormat";
  NMM::Event* IFormatChange::create_setFormat(const Format& new_format)
  {
    NMM::TInValue<Format >* new_format_value = new NMM::TInValue<Format >();
    new_format_value->setValue(new_format);

    return new NMM::Event("IFormatChange::setFormat", new_format_value);
  }



}


#include "nmm/nmm_suffix.hpp"

