/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IFormatChange_HPP
#define NMM_IFormatChange_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/format/Format.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Instream format change notification.
   */

  class DLLEXPORT IFormatChange : virtual public NMM::Interface { 
  public:
    IFormatChange() : NMM::Interface(IFormatChange::IFormatChange_type, "NMM") { };

    IFormatChange(NMM::NMMObject& parent) : NMM::Interface (parent, IFormatChange::IFormatChange_type, "NMM") { };

    virtual ~IFormatChange() {};

    static const char* IFormatChange_type;

    virtual  string getInterfaceType() const { return IFormatChange_type; }

    NMM::Interface* copy() const { return new IFormatChange(*this); }
    static void used(); 
    static const char* setFormat_event;
    static NMM::Event* create_setFormat(const Format& new_format);
  /* instream (protected) and non-instream (public) */        /** The input format has changed (instream event).
     * This instream event notifies the receiver that the input format has changed at the
     * input jack from which the event was received.
     * @param new_format The new input format
     * @return Result of the instream event
     */

    virtual Result setFormat(const Format& new_format) /* in-stream */ 
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IFormatChange> IFormatChange_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
