/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IFormatChangeImpl_HPP
#define NMM_IFormatChangeImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IFormatChange.hpp"


#include "visibility.hpp"

namespace NMM
{
  /** Instream format change notification.
   */

  class DLLEXPORT IFormatChangeImpl  : public virtual NMM::NMMObject { 
  public:
    IFormatChangeImpl() { addInterface(new IFormatChange(*this)); registerEvents(); };

    virtual ~IFormatChangeImpl() {};

  protected:
    /** The input format has changed (instream event).
     * This instream event notifies the receiver that the input format has changed at the
     * input jack from which the event was received.
     * @param new_format The new input format
     * @return Result of the instream event
     */

    virtual Result setFormat(const Format& new_format) /* in-stream */  = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
