/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IInstream_HPP
#define NMM_IInstream_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{
  /** Interface for injecting instream events.
   */

  class DLLEXPORT IInstream : virtual public NMM::Interface { 
  public:
    IInstream() : NMM::Interface(IInstream::IInstream_type, "NMM") { };

    IInstream(NMM::NMMObject& parent) : NMM::Interface (parent, IInstream::IInstream_type, "NMM") { };

    virtual ~IInstream() {};

    static const char* IInstream_type;

    virtual  string getInterfaceType() const { return IInstream_type; }

    NMM::Interface* copy() const { return new IInstream(*this); }
    static void used(); 
    static const char* sendMessageUpstream_event;
    static const char* sendMessageDownstream_event;
    /** Inject an upstream event.
     * This method injects the given instream event into the stream in upstream direction.
     *
     * @param cevent An instream event
     */

    virtual Result sendMessageUpstream(const CEvent& cevent)
      throw(const NMM::SerializedException&);
    /** Inject a downstream event.
     * This method injects the given instream event into the stream in downstream direction.
     *
     * @param cevent An instream event
     */

    virtual Result sendMessageDownstream(const CEvent& cevent)
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IInstream> IInstream_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
