/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IInstreamImpl_HPP
#define NMM_IInstreamImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IInstream.hpp"


#include "visibility.hpp"

namespace NMM
{
  /** Interface for injecting instream events.
   */

  class DLLEXPORT IInstreamImpl  : public virtual NMM::NMMObject { 
  public:
    IInstreamImpl() { addInterface(new IInstream(*this)); registerEvents(); };

    virtual ~IInstreamImpl() {};

  protected:
    /** Inject an upstream event.
     * This method injects the given instream event into the stream in upstream direction.
     *
     * @param cevent An instream event
     */

    virtual Result sendMessageUpstream(const CEvent& cevent) = 0;
    /** Inject a downstream event.
     * This method injects the given instream event into the stream in downstream direction.
     *
     * @param cevent An instream event
     */

    virtual Result sendMessageDownstream(const CEvent& cevent) = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
