/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "INode.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/format/Property.hpp"
#include "nmm/comm/serialize/StdValue.hpp"
#include "nmm/comm/serialize/SerializeUtilsTime.hpp"

namespace NMM
{
  void INode::used() {} 
  
  
  void INode::reachState(const INode::State& state) 
      throw(const NMM::SerializedException&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    NMM::TInValue<INode::State > state_value(state, false);
    NMM::Event event("INode::reachState", &state_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void INode::reachConstructed() 
      throw(const NMM::SerializedException&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    NMM::Event event("INode::reachConstructed");
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void INode::reachInitialized() 
      throw(const NMM::SerializedException&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    NMM::Event event("INode::reachInitialized");
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void INode::reachOutputInitialized() 
      throw(const NMM::SerializedException&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    NMM::Event event("INode::reachOutputInitialized");
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void INode::reachActivated() 
      throw(const NMM::SerializedException&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    NMM::Event event("INode::reachActivated");
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void INode::reachStarted() 
      throw(const NMM::SerializedException&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    NMM::Event event("INode::reachStarted");
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void INode::setPrefetchQueueSize(const nmm_uint32& pre_fetch_queue_size) /* limited(INode::CONSTRUCTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_uint32 > pre_fetch_queue_size_value(pre_fetch_queue_size, false);
    NMM::Event event("INode::setPrefetchQueueSize", &pre_fetch_queue_size_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void INode::setPrefetchQueueDuration(const Interval& pre_fetch_queue_duration) /* limited(INode::CONSTRUCTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<Interval > pre_fetch_queue_duration_value(pre_fetch_queue_duration, false);
    NMM::Event event("INode::setPrefetchQueueDuration", &pre_fetch_queue_duration_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_uint32 INode::getPrefetchQueueSize()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("INode::getPrefetchQueueSize");
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void INode::waitPrefetchFinished() /* limited(INode::ACTIVATED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::Event event("INode::waitPrefetchFinished");
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void INode::init() /* limited(INode::CONSTRUCTED) */ 
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    NMM::Event event("INode::init");
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void INode::initOutput() /* limited(INode::INITIALIZED) */ 
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    NMM::Event event("INode::initOutput");
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void INode::activate() /* limited(INode::OUTPUT_INITIALIZED) */ 
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    NMM::Event event("INode::activate");
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  Result INode::flush() /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("INode::flush");
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void INode::start() /* limited(INode::ACTIVATED) */ 
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    NMM::Event event("INode::start");
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void INode::stop() /* limited(INode::STARTED) */ 
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    NMM::Event event("INode::stop");
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void INode::deactivate() /* limited(INode::ACTIVATED) */ 
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    NMM::Event event("INode::deactivate");
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void INode::deinitOutput() /* limited(INode::OUTPUT_INITIALIZED) */ 
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    NMM::Event event("INode::deinitOutput");
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void INode::deinit() /* limited(INode::INITIALIZED) */ 
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    NMM::Event event("INode::deinit");
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_bool INode::isConstructed() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("INode::isConstructed");
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool INode::isInitialized() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("INode::isInitialized");
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool INode::isOutputInitialized() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("INode::isOutputInitialized");
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool INode::isActivated() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("INode::isActivated");
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool INode::isStarted() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("INode::isStarted");
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IInputJack* INode::getInputJack(const Format& format, const string& jack_tag) /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<IInputJack* > return_value;
    NMM::TInValue<Format > format_value(format, false);
    NMM::TInValue<string > jack_tag_value(jack_tag, false);
    NMM::Event event("INode::getInputJack", &format_value, &jack_tag_value);
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IInputJack* INode::getInputStream(const string& jack_tag) /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<IInputJack* > return_value;
    NMM::TInValue<string > jack_tag_value(jack_tag, false);
    NMM::Event event("INode::getInputStream", &jack_tag_value);
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<string >  INode::getInputStreamTags() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<list<string >  > return_value;
    NMM::Event event("INode::getInputStreamTags");
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool INode::isInputConnected(const string& jack_tag /* = "default" */) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::TInValue<string > jack_tag_value(jack_tag, false);
    NMM::Event event("INode::isInputConnected", &jack_tag_value);
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Format* INode::getInputStreamFormat(const string& jack_tag /* = "default" */)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Format* > return_value;
    NMM::TInValue<string > jack_tag_value(jack_tag, false);
    NMM::Event event("INode::getInputStreamFormat", &jack_tag_value);
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IOutputJack* INode::getOutputJack(const Format& format, const string& jack_tag) /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<IOutputJack* > return_value;
    NMM::TInValue<Format > format_value(format, false);
    NMM::TInValue<string > jack_tag_value(jack_tag, false);
    NMM::Event event("INode::getOutputJack", &format_value, &jack_tag_value);
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IOutputJack* INode::getUnintializedOutputJack(const Format& format, const string& jack_tag) /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<IOutputJack* > return_value;
    NMM::TInValue<Format > format_value(format, false);
    NMM::TInValue<string > jack_tag_value(jack_tag, false);
    NMM::Event event("INode::getUnintializedOutputJack", &format_value, &jack_tag_value);
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IOutputJack* INode::getOutputStream(const string& jack_tag) /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<IOutputJack* > return_value;
    NMM::TInValue<string > jack_tag_value(jack_tag, false);
    NMM::Event event("INode::getOutputStream", &jack_tag_value);
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<string >  INode::getOutputStreamTags() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<list<string >  > return_value;
    NMM::Event event("INode::getOutputStreamTags");
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool INode::isOutputConnected(const string& jack_tag /* = "default" */) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::TInValue<string > jack_tag_value(jack_tag, false);
    NMM::Event event("INode::isOutputConnected", &jack_tag_value);
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IOutputJack* INode::cloneOutputJack(Format*& intersec_format, const string& jack_tag /* = "default" */) /* limited(INode::OUTPUT_INITIALIZED) */ /* limited(INode::ACTIVATED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<IOutputJack* > return_value;
    NMM::TOutValue<Format* > intersec_format_value(intersec_format);
    NMM::TInValue<string > jack_tag_value(jack_tag, false);
    NMM::Event event("INode::cloneOutputJack", &intersec_format_value, &jack_tag_value);
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    intersec_format = intersec_format_value.getRValue();
    return return_value.getRValue();
  }
  
  Format* INode::getOutputStreamFormat(const string& jack_tag /* = "default" */)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Format* > return_value;
    NMM::TInValue<string > jack_tag_value(jack_tag, false);
    NMM::Event event("INode::getOutputStreamFormat", &jack_tag_value);
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_float INode::getCost(const string& jack_tag) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_float > return_value;
    NMM::TInValue<string > jack_tag_value(jack_tag, false);
    NMM::Event event("INode::getCost", &jack_tag_value);
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  INode::Type INode::getType() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<INode::Type > return_value;
    NMM::Event event("INode::getType");
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  INode::State INode::getState() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<INode::State > return_value;
    NMM::Event event("INode::getState");
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Property* INode::getInputProperty()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Property* > return_value;
    NMM::Event event("INode::getInputProperty");
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Property* INode::getWorkingInputProperty()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Property* > return_value;
    NMM::Event event("INode::getWorkingInputProperty");
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Property* INode::getOutputProperty()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Property* > return_value;
    NMM::Event event("INode::getOutputProperty");
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Property* INode::getWorkingOutputProperty()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Property* > return_value;
    NMM::Event event("INode::getWorkingOutputProperty");
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ComputationConfig* INode::getComputationConfig() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ComputationConfig* > return_value;
    NMM::Event event("INode::getComputationConfig");
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ComputationConfig* INode::getWorkingComputationConfig() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ComputationConfig* > return_value;
    NMM::Event event("INode::getWorkingComputationConfig");
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void INode::setComputationConfig(const ComputationConfig* computation_config)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<ComputationConfig* > computation_config_value (const_cast<ComputationConfig* >(computation_config), false);
    NMM::Event event("INode::setComputationConfig", &computation_config_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void INode::setWorkingComputationConfig(const ComputationConfig* computation_config)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<ComputationConfig* > computation_config_value (const_cast<ComputationConfig* >(computation_config), false);
    NMM::Event event("INode::setWorkingComputationConfig", &computation_config_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_float INode::getInternalCost() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_float > return_value;
    NMM::Event event("INode::getInternalCost");
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result INode::sendData() /* limited(INode::OUTPUT_INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* in-stream */ 
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("INode::sendData");
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result INode::sendDataFailed() /* limited(INode::INITIALIZED) */ /* in-stream */ 
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("INode::sendDataFailed");
    event.setReturnValue(&return_value);
    if(const_cast<INode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* INode::INode_type = "INode";
  NMM::TConstructor<INode > __INode_prototype("INode","NMM");
  TValueConstructor< INode::Type > __INode_Type_prototype(TypeInfo("INode::Type", "NMM"));

  NMM::OValueStream& operator<<(NMM::OValueStream& ov, const INode::Type& t)
  {
    ov.beginType(TypeInfo("INode::Type", "NMM"));
    nmm_int32 temp = static_cast<nmm_int32>(t);
    ov << temp;
    ov.endType();
    return ov;
  }

  NMM::IValueStream& operator>>(NMM::IValueStream& iv, INode::Type& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("INode::Type", "NMM")) {
      throw NMM::InvalidTypeException(TypeInfo("INode::Type", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    nmm_int32 temp;
    iv >> temp;
    t = static_cast<INode::Type>(temp);
    iv.goUp();
    return iv;
  }

  TValueConstructor< INode::State > __INode_State_prototype(TypeInfo("INode::State", "NMM"));

  NMM::OValueStream& operator<<(NMM::OValueStream& ov, const INode::State& t)
  {
    ov.beginType(TypeInfo("INode::State", "NMM"));
    nmm_int32 temp = static_cast<nmm_int32>(t);
    ov << temp;
    ov.endType();
    return ov;
  }

  NMM::IValueStream& operator>>(NMM::IValueStream& iv, INode::State& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("INode::State", "NMM")) {
      throw NMM::InvalidTypeException(TypeInfo("INode::State", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    nmm_int32 temp;
    iv >> temp;
    t = static_cast<INode::State>(temp);
    iv.goUp();
    return iv;
  }



  const char* INode::reachState_event = "INode::reachState";
  const char* INode::reachConstructed_event = "INode::reachConstructed";
  const char* INode::reachInitialized_event = "INode::reachInitialized";
  const char* INode::reachOutputInitialized_event = "INode::reachOutputInitialized";
  const char* INode::reachActivated_event = "INode::reachActivated";
  const char* INode::reachStarted_event = "INode::reachStarted";
  const char* INode::setPrefetchQueueSize_event = "INode::setPrefetchQueueSize";
  const char* INode::setPrefetchQueueDuration_event = "INode::setPrefetchQueueDuration";
  const char* INode::getPrefetchQueueSize_event = "INode::getPrefetchQueueSize";
  const char* INode::waitPrefetchFinished_event = "INode::waitPrefetchFinished";
  const char* INode::init_event = "INode::init";
  const char* INode::initOutput_event = "INode::initOutput";
  const char* INode::activate_event = "INode::activate";
  const char* INode::flush_event = "INode::flush";
  const char* INode::start_event = "INode::start";
  const char* INode::stop_event = "INode::stop";
  const char* INode::deactivate_event = "INode::deactivate";
  const char* INode::deinitOutput_event = "INode::deinitOutput";
  const char* INode::deinit_event = "INode::deinit";
  const char* INode::isConstructed_event = "INode::isConstructed";
  const char* INode::isInitialized_event = "INode::isInitialized";
  const char* INode::isOutputInitialized_event = "INode::isOutputInitialized";
  const char* INode::isActivated_event = "INode::isActivated";
  const char* INode::isStarted_event = "INode::isStarted";
  const char* INode::getInputJack_event = "INode::getInputJack";
  const char* INode::getInputStream_event = "INode::getInputStream";
  const char* INode::getInputStreamTags_event = "INode::getInputStreamTags";
  const char* INode::isInputConnected_event = "INode::isInputConnected";
  const char* INode::getInputStreamFormat_event = "INode::getInputStreamFormat";
  const char* INode::getOutputJack_event = "INode::getOutputJack";
  const char* INode::getUnintializedOutputJack_event = "INode::getUnintializedOutputJack";
  const char* INode::getOutputStream_event = "INode::getOutputStream";
  const char* INode::getOutputStreamTags_event = "INode::getOutputStreamTags";
  const char* INode::isOutputConnected_event = "INode::isOutputConnected";
  const char* INode::cloneOutputJack_event = "INode::cloneOutputJack";
  const char* INode::getOutputStreamFormat_event = "INode::getOutputStreamFormat";
  const char* INode::getCost_event = "INode::getCost";
  const char* INode::getType_event = "INode::getType";
  const char* INode::getState_event = "INode::getState";
  const char* INode::getInputProperty_event = "INode::getInputProperty";
  const char* INode::getWorkingInputProperty_event = "INode::getWorkingInputProperty";
  const char* INode::getOutputProperty_event = "INode::getOutputProperty";
  const char* INode::getWorkingOutputProperty_event = "INode::getWorkingOutputProperty";
  const char* INode::getComputationConfig_event = "INode::getComputationConfig";
  const char* INode::getWorkingComputationConfig_event = "INode::getWorkingComputationConfig";
  const char* INode::setComputationConfig_event = "INode::setComputationConfig";
  const char* INode::setWorkingComputationConfig_event = "INode::setWorkingComputationConfig";
  const char* INode::getInternalCost_event = "INode::getInternalCost";
  const char* INode::sendData_event = "INode::sendData";
  NMM::Event* INode::create_sendData()
  {

    return new NMM::Event("INode::sendData");
  }

  const char* INode::sendDataFailed_event = "INode::sendDataFailed";
  NMM::Event* INode::create_sendDataFailed()
  {

    return new NMM::Event("INode::sendDataFailed");
  }



}


#include "nmm/nmm_suffix.hpp"

