/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "INodeImpl.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/messaging/TEDObject.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/format/Property.hpp"
#include "nmm/comm/serialize/StdValue.hpp"
#include "nmm/comm/serialize/SerializeUtilsTime.hpp"

namespace NMM
{
  
  void INodeImpl::registerEvents()
  {
  
    getEventDispatcher().registerEvent(INode::reachState_event, new NMM::TEDObject1<INodeImpl, INode::State, void(INodeImpl:: * )(const INode::State&), void >(this, &INodeImpl::reachState));
  
    getEventDispatcher().registerEvent(INode::reachConstructed_event, new NMM::TEDObject0<INodeImpl, void(INodeImpl:: * )(), void >(this, &INodeImpl::reachConstructed));
  
    getEventDispatcher().registerEvent(INode::reachInitialized_event, new NMM::TEDObject0<INodeImpl, void(INodeImpl:: * )(), void >(this, &INodeImpl::reachInitialized));
  
    getEventDispatcher().registerEvent(INode::reachOutputInitialized_event, new NMM::TEDObject0<INodeImpl, void(INodeImpl:: * )(), void >(this, &INodeImpl::reachOutputInitialized));
  
    getEventDispatcher().registerEvent(INode::reachActivated_event, new NMM::TEDObject0<INodeImpl, void(INodeImpl:: * )(), void >(this, &INodeImpl::reachActivated));
  
    getEventDispatcher().registerEvent(INode::reachStarted_event, new NMM::TEDObject0<INodeImpl, void(INodeImpl:: * )(), void >(this, &INodeImpl::reachStarted));
  
    getEventDispatcher().registerEvent(INode::setPrefetchQueueSize_event, new NMM::TEDObject1<INodeImpl, nmm_uint32, void(INodeImpl:: * )(const nmm_uint32&), void >(this, &INodeImpl::setPrefetchQueueSize));
    getEventDispatcher().addLimited(INode::setPrefetchQueueSize_event, INode::CONSTRUCTED);
  
    getEventDispatcher().registerEvent(INode::setPrefetchQueueDuration_event, new NMM::TEDObject1<INodeImpl, Interval, void(INodeImpl:: * )(const Interval&), void >(this, &INodeImpl::setPrefetchQueueDuration));
    getEventDispatcher().addLimited(INode::setPrefetchQueueDuration_event, INode::CONSTRUCTED);
  
    getEventDispatcher().registerEvent(INode::getPrefetchQueueSize_event, new NMM::TEDObject0<INodeImpl, nmm_uint32(INodeImpl:: * )(), nmm_uint32 >(this, &INodeImpl::getPrefetchQueueSize));
  
    getEventDispatcher().registerEvent(INode::waitPrefetchFinished_event, new NMM::TEDObject0<INodeImpl, void(INodeImpl:: * )(), void >(this, &INodeImpl::waitPrefetchFinished));
    getEventDispatcher().addLimited(INode::waitPrefetchFinished_event, INode::ACTIVATED);
  
    getEventDispatcher().registerEvent(INode::init_event, new NMM::TEDObject0<INodeImpl, void(INodeImpl:: * )(), void >(this, &INodeImpl::init));
    getEventDispatcher().addLimited(INode::init_event, INode::CONSTRUCTED);
  
    getEventDispatcher().registerEvent(INode::initOutput_event, new NMM::TEDObject0<INodeImpl, void(INodeImpl:: * )(), void >(this, &INodeImpl::initOutput));
    getEventDispatcher().addLimited(INode::initOutput_event, INode::INITIALIZED);
  
    getEventDispatcher().registerEvent(INode::activate_event, new NMM::TEDObject0<INodeImpl, void(INodeImpl:: * )(), void >(this, &INodeImpl::activate));
    getEventDispatcher().addLimited(INode::activate_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(INode::flush_event, new NMM::TEDObject0<INodeImpl, Result(INodeImpl:: * )(), Result >(this, &INodeImpl::flush));
    getEventDispatcher().addLimited(INode::flush_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(INode::flush_event, INode::STARTED);
  
    getEventDispatcher().registerEvent(INode::start_event, new NMM::TEDObject0<INodeImpl, void(INodeImpl:: * )(), void >(this, &INodeImpl::start));
    getEventDispatcher().addLimited(INode::start_event, INode::ACTIVATED);
  
    getEventDispatcher().registerEvent(INode::stop_event, new NMM::TEDObject0<INodeImpl, void(INodeImpl:: * )(), void >(this, &INodeImpl::stop));
    getEventDispatcher().addLimited(INode::stop_event, INode::STARTED);
  
    getEventDispatcher().registerEvent(INode::deactivate_event, new NMM::TEDObject0<INodeImpl, void(INodeImpl:: * )(), void >(this, &INodeImpl::deactivate));
    getEventDispatcher().addLimited(INode::deactivate_event, INode::ACTIVATED);
  
    getEventDispatcher().registerEvent(INode::deinitOutput_event, new NMM::TEDObject0<INodeImpl, void(INodeImpl:: * )(), void >(this, &INodeImpl::deinitOutput));
    getEventDispatcher().addLimited(INode::deinitOutput_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(INode::deinit_event, new NMM::TEDObject0<INodeImpl, void(INodeImpl:: * )(), void >(this, &INodeImpl::deinit));
    getEventDispatcher().addLimited(INode::deinit_event, INode::INITIALIZED);
  
    getEventDispatcher().registerEvent(INode::isConstructed_event, new NMM::TEDObject0<INodeImpl, nmm_bool(INodeImpl:: * )() const, nmm_bool >(this, &INodeImpl::isConstructed));
  
    getEventDispatcher().registerEvent(INode::isInitialized_event, new NMM::TEDObject0<INodeImpl, nmm_bool(INodeImpl:: * )() const, nmm_bool >(this, &INodeImpl::isInitialized));
  
    getEventDispatcher().registerEvent(INode::isOutputInitialized_event, new NMM::TEDObject0<INodeImpl, nmm_bool(INodeImpl:: * )() const, nmm_bool >(this, &INodeImpl::isOutputInitialized));
  
    getEventDispatcher().registerEvent(INode::isActivated_event, new NMM::TEDObject0<INodeImpl, nmm_bool(INodeImpl:: * )() const, nmm_bool >(this, &INodeImpl::isActivated));
  
    getEventDispatcher().registerEvent(INode::isStarted_event, new NMM::TEDObject0<INodeImpl, nmm_bool(INodeImpl:: * )() const, nmm_bool >(this, &INodeImpl::isStarted));
  
    getEventDispatcher().registerEvent(INode::getInputJack_event, new NMM::TEDObject2<INodeImpl, Format, string, IInputJack*(INodeImpl:: * )(const Format&, const string&), IInputJack* >(this, &INodeImpl::getInputJack));
    getEventDispatcher().addLimited(INode::getInputJack_event, INode::INITIALIZED);
  
    getEventDispatcher().registerEvent(INode::getInputStream_event, new NMM::TEDObject1<INodeImpl, string, IInputJack*(INodeImpl:: * )(const string&), IInputJack* >(this, &INodeImpl::getInputStream));
    getEventDispatcher().addLimited(INode::getInputStream_event, INode::INITIALIZED);
  
    getEventDispatcher().registerEvent(INode::getInputStreamTags_event, new NMM::TEDObject0<INodeImpl, list<string > (INodeImpl:: * )() const, list<string >  >(this, &INodeImpl::getInputStreamTags));
  
    getEventDispatcher().registerEvent(INode::isInputConnected_event, new NMM::TEDObject1<INodeImpl, string, nmm_bool(INodeImpl:: * )(const string&) const, nmm_bool >(this, &INodeImpl::isInputConnected));
  
    getEventDispatcher().registerEvent(INode::getInputStreamFormat_event, new NMM::TEDObject1<INodeImpl, string, Format*(INodeImpl:: * )(const string&), Format* >(this, &INodeImpl::getInputStreamFormat));
  
    getEventDispatcher().registerEvent(INode::getOutputJack_event, new NMM::TEDObject2<INodeImpl, Format, string, IOutputJack*(INodeImpl:: * )(const Format&, const string&), IOutputJack* >(this, &INodeImpl::getOutputJack));
    getEventDispatcher().addLimited(INode::getOutputJack_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(INode::getUnintializedOutputJack_event, new NMM::TEDObject2<INodeImpl, Format, string, IOutputJack*(INodeImpl:: * )(const Format&, const string&), IOutputJack* >(this, &INodeImpl::getUnintializedOutputJack));
    getEventDispatcher().addLimited(INode::getUnintializedOutputJack_event, INode::INITIALIZED);
  
    getEventDispatcher().registerEvent(INode::getOutputStream_event, new NMM::TEDObject1<INodeImpl, string, IOutputJack*(INodeImpl:: * )(const string&), IOutputJack* >(this, &INodeImpl::getOutputStream));
    getEventDispatcher().addLimited(INode::getOutputStream_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(INode::getOutputStreamTags_event, new NMM::TEDObject0<INodeImpl, list<string > (INodeImpl:: * )() const, list<string >  >(this, &INodeImpl::getOutputStreamTags));
  
    getEventDispatcher().registerEvent(INode::isOutputConnected_event, new NMM::TEDObject1<INodeImpl, string, nmm_bool(INodeImpl:: * )(const string&) const, nmm_bool >(this, &INodeImpl::isOutputConnected));
  
    getEventDispatcher().registerEvent(INode::cloneOutputJack_event, new NMM::TEDObject2<INodeImpl, Format*, string, IOutputJack*(INodeImpl:: * )(Format*&, const string&), IOutputJack* >(this, &INodeImpl::cloneOutputJack));
    getEventDispatcher().addLimited(INode::cloneOutputJack_event, INode::OUTPUT_INITIALIZED);
    getEventDispatcher().addLimited(INode::cloneOutputJack_event, INode::ACTIVATED);
  
    getEventDispatcher().registerEvent(INode::getOutputStreamFormat_event, new NMM::TEDObject1<INodeImpl, string, Format*(INodeImpl:: * )(const string&), Format* >(this, &INodeImpl::getOutputStreamFormat));
  
    getEventDispatcher().registerEvent(INode::getCost_event, new NMM::TEDObject1<INodeImpl, string, nmm_float(INodeImpl:: * )(const string&) const, nmm_float >(this, &INodeImpl::getCost));
  
    getEventDispatcher().registerEvent(INode::getType_event, new NMM::TEDObject0<INodeImpl, INode::Type(INodeImpl:: * )() const, INode::Type >(this, &INodeImpl::getType));
  
    getEventDispatcher().registerEvent(INode::getState_event, new NMM::TEDObject0<INodeImpl, INode::State(INodeImpl:: * )() const, INode::State >(this, &INodeImpl::getState));
  
    getEventDispatcher().registerEvent(INode::getInputProperty_event, new NMM::TEDObject0<INodeImpl, Property*(INodeImpl:: * )(), Property* >(this, &INodeImpl::getInputProperty));
  
    getEventDispatcher().registerEvent(INode::getWorkingInputProperty_event, new NMM::TEDObject0<INodeImpl, Property*(INodeImpl:: * )(), Property* >(this, &INodeImpl::getWorkingInputProperty));
  
    getEventDispatcher().registerEvent(INode::getOutputProperty_event, new NMM::TEDObject0<INodeImpl, Property*(INodeImpl:: * )(), Property* >(this, &INodeImpl::getOutputProperty));
  
    getEventDispatcher().registerEvent(INode::getWorkingOutputProperty_event, new NMM::TEDObject0<INodeImpl, Property*(INodeImpl:: * )(), Property* >(this, &INodeImpl::getWorkingOutputProperty));
  
    getEventDispatcher().registerEvent(INode::getComputationConfig_event, new NMM::TEDObject0<INodeImpl, ComputationConfig*(INodeImpl:: * )() const, ComputationConfig* >(this, &INodeImpl::getComputationConfig));
  
    getEventDispatcher().registerEvent(INode::getWorkingComputationConfig_event, new NMM::TEDObject0<INodeImpl, ComputationConfig*(INodeImpl:: * )() const, ComputationConfig* >(this, &INodeImpl::getWorkingComputationConfig));
  
    getEventDispatcher().registerEvent(INode::setComputationConfig_event, new NMM::TEDObject1<INodeImpl, ComputationConfig*, void(INodeImpl:: * )(const ComputationConfig*), void >(this, &INodeImpl::setComputationConfig));
  
    getEventDispatcher().registerEvent(INode::setWorkingComputationConfig_event, new NMM::TEDObject1<INodeImpl, ComputationConfig*, void(INodeImpl:: * )(const ComputationConfig*), void >(this, &INodeImpl::setWorkingComputationConfig));
  
    getEventDispatcher().registerEvent(INode::getInternalCost_event, new NMM::TEDObject0<INodeImpl, nmm_float(INodeImpl:: * )() const, nmm_float >(this, &INodeImpl::getInternalCost));
  
    getEventDispatcher().registerEvent(INode::sendData_event, new NMM::TEDObject0<INodeImpl, Result(INodeImpl:: * )(), Result >(this, &INodeImpl::sendData));
    getEventDispatcher().addLimited(INode::sendData_event, INode::OUTPUT_INITIALIZED);
    getEventDispatcher().addLimited(INode::sendData_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(INode::sendData_event, INode::STARTED);
  
    getEventDispatcher().registerEvent(INode::sendDataFailed_event, new NMM::TEDObject0<INodeImpl, Result(INodeImpl:: * )(), Result >(this, &INodeImpl::sendDataFailed));
    getEventDispatcher().addLimited(INode::sendDataFailed_event, INode::INITIALIZED);

  }

}


#include "nmm/nmm_suffix.hpp"

