/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#ifndef NMM_IPLUGINMANAGERHANDLE_HPP
#define NMM_IPLUGINMANAGERHANDLE_HPP

#include "nmm/nmm_prefix.hpp"

#include <string>

namespace NMM {

  /** Interface for accessing the plug-in manager.
   * This abstract class allows a Plugin object to unregister itself at the plug-in manager.
   */
  class IPluginManagerHandle
  {
  public:
    /** Virtual destructor.
     */
    virtual ~IPluginManagerHandle() {}

    /** Unregister a plug-in.
     * This method is called by a Plugin object when the plug-in is unregistered.
     *
     * @param plugin_name The name of the plugin
     */
    virtual void unregisterPlugin(const std::string& plugin_name) = 0;
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
