/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IReferenceCount.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IReferenceCount::used() {} 
  
  
  void IReferenceCount::increaseCount(const nmm_uint32& i /* = 1 */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > i_value(i, false);
    NMM::Event event("IReferenceCount::increaseCount", &i_value);
    if(const_cast<IReferenceCount*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IReferenceCount::decreaseCount(const nmm_uint32& i /* = 1 */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > i_value(i, false);
    NMM::Event event("IReferenceCount::decreaseCount", &i_value);
    if(const_cast<IReferenceCount*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_uint32 IReferenceCount::getCount()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("IReferenceCount::getCount");
    event.setReturnValue(&return_value);
    if(const_cast<IReferenceCount*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IReferenceCount::IReferenceCount_type = "IReferenceCount";
  NMM::TConstructor<IReferenceCount > __IReferenceCount_prototype("IReferenceCount","NMM");


  const char* IReferenceCount::increaseCount_event = "IReferenceCount::increaseCount";
  const char* IReferenceCount::decreaseCount_event = "IReferenceCount::decreaseCount";
  const char* IReferenceCount::getCount_event = "IReferenceCount::getCount";


}


#include "nmm/nmm_suffix.hpp"

