/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IReferenceCount_HPP
#define NMM_IReferenceCount_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{
  /** Interface for reference-counted objects.
   * This interface is implemented by objects whose lifetime is determined by its reference count.
   * It provides methods for accessing the reference counter.
   */

  class DLLEXPORT IReferenceCount : virtual public NMM::Interface { 
  public:
    IReferenceCount() : NMM::Interface(IReferenceCount::IReferenceCount_type, "NMM") { };

    IReferenceCount(NMM::NMMObject& parent) : NMM::Interface (parent, IReferenceCount::IReferenceCount_type, "NMM") { };

    virtual ~IReferenceCount() {};

    static const char* IReferenceCount_type;

    virtual  string getInterfaceType() const { return IReferenceCount_type; }

    NMM::Interface* copy() const { return new IReferenceCount(*this); }
    static void used(); 
    static const char* increaseCount_event;
    static const char* decreaseCount_event;
    static const char* getCount_event;
    /** Increase the reference count by the given number.
     *
     * @param i Number of references to add
     */

    virtual void increaseCount(const nmm_uint32& i = 1)
      throw(const NMM::SerializedException&);
    /** Decrease the reference count by the given number.
     *
     * @param i Number of references to remove
     */

    virtual void decreaseCount(const nmm_uint32& i = 1)
      throw(const NMM::SerializedException&);
    /** Get the current reference count.
     *
     * @return Number of references
     */

    virtual nmm_uint32 getCount()
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IReferenceCount> IReferenceCount_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
