/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IReferenceCountImpl_HPP
#define NMM_IReferenceCountImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IReferenceCount.hpp"


#include "visibility.hpp"

namespace NMM
{
  /** Interface for reference-counted objects.
   * This interface is implemented by objects whose lifetime is determined by its reference count.
   * It provides methods for accessing the reference counter.
   */

  class DLLEXPORT IReferenceCountImpl  : public virtual NMM::NMMObject { 
  public:
    IReferenceCountImpl() { addInterface(new IReferenceCount(*this)); registerEvents(); };

    virtual ~IReferenceCountImpl() {};

  protected:
    /** Increase the reference count by the given number.
     *
     * @param i Number of references to add
     */

    virtual void increaseCount(const nmm_uint32& i = 1) = 0;
    /** Decrease the reference count by the given number.
     *
     * @param i Number of references to remove
     */

    virtual void decreaseCount(const nmm_uint32& i = 1) = 0;
    /** Get the current reference count.
     *
     * @return Number of references
     */

    virtual nmm_uint32 getCount() = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
