/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ISwitch.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ISwitch::used() {} 
  
  
  void ISwitch::switchOutputs()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("ISwitch::switchOutputs");
    if(const_cast<ISwitch*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ISwitch::flushOutputs(const Timestamp& ts)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<Timestamp > ts_value(ts, false);
    NMM::Event event("ISwitch::flushOutputs", &ts_value);
    if(const_cast<ISwitch*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ISwitch::setSwitchOutputsEnabled(const nmm_bool& enable)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > enable_value(enable, false);
    NMM::Event event("ISwitch::setSwitchOutputsEnabled", &enable_value);
    if(const_cast<ISwitch*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_bool ISwitch::getSwitchOutputsEnabled()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ISwitch::getSwitchOutputsEnabled");
    event.setReturnValue(&return_value);
    if(const_cast<ISwitch*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ISwitch::ISwitch_type = "ISwitch";
  NMM::TConstructor<ISwitch > __ISwitch_prototype("ISwitch","NMM");


  const char* ISwitch::switchOutputs_event = "ISwitch::switchOutputs";
  const char* ISwitch::flushOutputs_event = "ISwitch::flushOutputs";
  const char* ISwitch::setSwitchOutputsEnabled_event = "ISwitch::setSwitchOutputsEnabled";
  const char* ISwitch::getSwitchOutputsEnabled_event = "ISwitch::getSwitchOutputsEnabled";


}


#include "nmm/nmm_suffix.hpp"

