/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ISyncReset_HPP
#define NMM_ISyncReset_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{
  /** Interface for resetting the synchronization.
   */

  class DLLEXPORT ISyncReset : virtual public NMM::Interface { 
  public:
    ISyncReset() : NMM::Interface(ISyncReset::ISyncReset_type, "NMM") { };

    ISyncReset(NMM::NMMObject& parent) : NMM::Interface (parent, ISyncReset::ISyncReset_type, "NMM") { };

    virtual ~ISyncReset() {};

    static const char* ISyncReset_type;

    virtual  string getInterfaceType() const { return ISyncReset_type; }

    NMM::Interface* copy() const { return new ISyncReset(*this); }
    static void used(); 
    static const char* syncReset_event;
    static NMM::Event* create_syncReset();
  /* instream (protected) and non-instream (public) */        /** Synchronization reset (instream event).
     *
     * @return Result of the instream event
     */

    virtual Result syncReset() /* in-stream */ 
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ISyncReset> ISyncReset_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
