/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IThreadedNode_HPP
#define NMM_IThreadedNode_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "INode.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Interface for controlling a threaded node.
   */

  class DLLEXPORT IThreadedNode : virtual public NMM::Interface { 
  public:
    IThreadedNode() : NMM::Interface(IThreadedNode::IThreadedNode_type, "NMM") { };

    IThreadedNode(NMM::NMMObject& parent) : NMM::Interface (parent, IThreadedNode::IThreadedNode_type, "NMM") { };

    virtual ~IThreadedNode() {};

    static const char* IThreadedNode_type;

    virtual  string getInterfaceType() const { return IThreadedNode_type; }

    NMM::Interface* copy() const { return new IThreadedNode(*this); }
    static void used(); 
    static const char* lowerPriority_event;
    /** Lower thread priority.
     * This method lowers the priority of the node thread.
     *
     * This method merely requests the node to lower the priority of its thread.
     * The actual functionality might not be implemented on some platforms. In this case, this method
     * does nothing.
     *
     * @param value Priority value, must be between 1-20
     */

    virtual void lowerPriority(const nmm_uint32& value) /* limited(INode::OUTPUT_INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);


  };

  typedef auto_ptr<IThreadedNode> IThreadedNode_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
