/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IUsage.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IUsage::used() {} 
  
  
  void IUsage::setUsage(const nmm_int32& usage, const IUsageNotify* user)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > usage_value(usage, false);
    NMM::TInValue<IUsageNotify* > user_value (const_cast<IUsageNotify* >(user), false);
    NMM::Event event("IUsage::setUsage", &usage_value, &user_value);
    if(const_cast<IUsage*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IUsage::unsetUsage(const nmm_int32& usage, const IUsageNotify* user)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > usage_value(usage, false);
    NMM::TInValue<IUsageNotify* > user_value (const_cast<IUsageNotify* >(user), false);
    NMM::Event event("IUsage::unsetUsage", &usage_value, &user_value);
    if(const_cast<IUsage*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_int32 IUsage::getUsage()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("IUsage::getUsage");
    event.setReturnValue(&return_value);
    if(const_cast<IUsage*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 IUsage::getRank(const nmm_int32& usage, const IUsageNotify* user)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::TInValue<nmm_int32 > usage_value(usage, false);
    NMM::TInValue<IUsageNotify* > user_value (const_cast<IUsageNotify* >(user), false);
    NMM::Event event("IUsage::getRank", &usage_value, &user_value);
    event.setReturnValue(&return_value);
    if(const_cast<IUsage*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IUsage::isMaster(const nmm_int32& usage, const IUsageNotify* user)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > usage_value(usage, false);
    NMM::TInValue<IUsageNotify* > user_value (const_cast<IUsageNotify* >(user), false);
    NMM::Event event("IUsage::isMaster", &usage_value, &user_value);
    event.setReturnValue(&return_value);
    if(const_cast<IUsage*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IUsage::IUsage_type = "IUsage";
  NMM::TConstructor<IUsage > __IUsage_prototype("IUsage","NMM");


  const char* IUsage::setUsage_event = "IUsage::setUsage";
  const char* IUsage::unsetUsage_event = "IUsage::unsetUsage";
  const char* IUsage::getUsage_event = "IUsage::getUsage";
  const char* IUsage::getRank_event = "IUsage::getRank";
  const char* IUsage::isMaster_event = "IUsage::isMaster";


}


#include "nmm/nmm_suffix.hpp"

