/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IUsageNotify_HPP
#define NMM_IUsageNotify_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{
  /** Usage notification interface.
   *
   * Interfaces for notifying a user of a shared node about changes of usage status.
   *
   * @deprecated Usage management is not supported
   */

  class DLLEXPORT IUsageNotify : virtual public NMM::Interface { 
  public:
    IUsageNotify() : NMM::Interface(IUsageNotify::IUsageNotify_type, "NMM") { };

    IUsageNotify(NMM::NMMObject& parent) : NMM::Interface (parent, IUsageNotify::IUsageNotify_type, "NMM") { };

    virtual ~IUsageNotify() {};

    static const char* IUsageNotify_type;

    virtual  string getInterfaceType() const { return IUsageNotify_type; }

    NMM::Interface* copy() const { return new IUsageNotify(*this); }
    static void used(); 
    static const char* becomeMaster_event;
    static const char* becomeSlave_event;
    /** The user has become the master of the node.
     *
     * @return SUCCESS
     */

    virtual Result becomeMaster()
      throw(const NMM::SerializedException&);
    /** The user has become a slave of the node.
     *
     * @return SUCCESS
     */

    virtual Result becomeSlave()
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IUsageNotify> IUsageNotify_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
