/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "ReferenceCount.hpp"
#include "nmm/utils/thread/MutexGuard.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM 
{

  ReferenceCount::ReferenceCount() : __counter(0)
  {
  }

  ReferenceCount::~ReferenceCount() 
  {
  }

  void ReferenceCount::increaseCount(const nmm_uint32& i)
  {
    MutexGuard mg(__mutex);
    __counter += i;
  }

  void ReferenceCount::decreaseCount(const nmm_uint32& i)
  {
    MutexGuard mg(__mutex);
    if(__counter < i) {
      GLOBAL_ERROR_STREAM("ReferenceCount is " << __counter << " and to small to decrease by " << i << ", setting to 0");
      __counter = 0;
    }
    else {
      __counter -= i;
    }
  }

  nmm_uint32 ReferenceCount::getCount()
  {
    return __counter;
  }

}

#include "nmm/nmm_suffix.hpp"
