/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#ifndef NMM_REFERENCE_COUNT_HPP
#define NMM_REFERENCE_COUNT_HPP NMM_REFERENCE_COUNT_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/multimedia/IReferenceCountImpl.hpp"

#include "visibility.hpp"

namespace NMM {

  /** Reference counted object.
   * This class is a simple implementation of the IReferenceCountImpl interface.
   * It can be used to implement objects whose lifetime is determine by its reference count.
   * The reference count can be changed through the IReferenceCountImpl object.
   * Access to the reference counter is thread-safe.
   */
  class DLLEXPORT ReferenceCount : public IReferenceCountImpl {
  public:
    /** Default constructor.
     * The reference count is initially zero.
     */
    ReferenceCount();

    /** Virtual destructor.
     */
    virtual ~ReferenceCount();

  protected:

    /** Increase the reference count by the given number.
     *
     * @param i Number of references to add
     */
    virtual void increaseCount(const nmm_uint32& i = 1);

    /** Decrease the reference count by the given number.
     *
     * @param i Number of references to remove
     */
    virtual void decreaseCount(const nmm_uint32& i = 1) ;

    /** Get the current reference count.
     *
     * @return Number of references
     */
    virtual nmm_uint32 getCount();

  private:
    unsigned int __counter;     ///< Reference counter
    ThreadMutex __mutex;        ///< Mutex for protecting the reference counter
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
