/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_STATE_TRANSITION_FAILED_EXCEPTIONS_HPP
#define NMM_STATE_TRANSITION_FAILED_EXCEPTIONS_HPP NMM_STATE_TRANSITION_FAILED_EXCEPTIONS_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/serialize/SerializedException.hpp"

#include "visibility.hpp"

namespace NMM {

  /** A state transition has failed.
   *
   * This Exception is thrown if there was an error during a state transition.
   * For nodes, this exception is thrown if a 'do' method, such as GenericNode::doInit,
   * has returned @c FAILURE.
   */
  class DLLEXPORT StateTransitionFailedException : public SerializedException {
  public:
    /** Create a new %StateTransitionFailedException.
     */
    StateTransitionFailedException() 
      : SerializedException("StateTransitionFailedExcpetion") { setTypeName("StateTransitionFailedException"); };

    /** Create a new %StateTransitionFailedException.
     *
     * @param s Error message
     */
    StateTransitionFailedException (const string& s) 
      : SerializedException("StateTransitionFailedException : " + s) {setTypeName("StateTransitionFailedException");};

    /** Create a new %StateTransitionFailedException within a method of a NamedObject.
     *
     * @param parent NamedObject from which the exception is thrown
     * @param s Error message
     */
    StateTransitionFailedException (const NamedObject* parent, const string& s = "")
      : SerializedException(parent, ((s == "") ? "StateTransitionFailedException" : "StateTransitionFailedException : " + s)) {setTypeName("StateTransitionFailedException");};

    /** Virtual copy constructor.
     *
     * @return A copy of this object
     */
    StateTransitionFailedException* copy() const { return new StateTransitionFailedException(*this); }
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
