/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "connect.hpp"
#include "nmm/comm/format/Property.hpp"
#include "nmm/comm/INamed.hpp"
#include "nmm/comm/ProxyObject.hpp"
#include "nmm/comm/serialize/SerializedException.hpp"
#include <sstream>
#include <iostream>

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  class InstreamChannel;
  
  /********************** INode connect *********************/
  void connect(INode* n1, const string& out_jack,
	       INode* n2, const string& in_jack,
	       const Format* format)
    throw(const BindingException&)
  {
    connect(n1, out_jack, n2, in_jack, format, 0); 
  }

  void connect(INode* n1,  
	       INode* n2,  
	       const Format* format)
    throw(const BindingException&) 
  {
    connect(n1, "default", n2, "default", format, 0); 
  }

  void connect(INode* n1, const string& out_jack, 
	       INode* n2,  
	       const Format* format)
    throw(const BindingException&) 
  { 
    connect(n1, out_jack, n2, "default", format, 0); 
  }

  void connect(INode* n1,  
	       INode* n2, const string& in_jack, 
	       const Format* format)
    throw(const BindingException&) 
  { 
    connect(n1, "default", n2, in_jack, format, 0);
  }

  void connect(INode* n1, const string& out_jack, 
	       INode* n2, const string& in_jack)
    throw(const BindingException&) 
  { 
    connect(n1, out_jack, n2, in_jack, 0, 0); 
  }

  void connect(INode* n1,  
	       INode* n2)
    throw(const BindingException&) 
  { 
    connect(n1, "default", n2, "default", 0, 0); 
  }

  void connect(INode* n1, const string& out_jack, 
	       INode* n2)
    throw(const BindingException&) 
  { 
    connect(n1, out_jack, n2, "default", 0, 0); 
  }                                           

  void connect(INode* n1,  
	       INode* n2, const string& in_jack)
    throw(const BindingException&) 
  { 
    connect(n1, "default", n2, in_jack, 0, 0); 
  }


  /********************** INode_var connect *********************/

  void connect(INode_var& n1, const string& out_jack, 
	       INode_var& n2, const string& in_jack, 
	       const Format* format)
    throw(const BindingException&) 
  { 
    connect(n1.get(), out_jack, n2.get(), in_jack, format, 0); 
  }

  void connect(INode_var& n1,  
	       INode_var& n2,  
	       const Format* format)
    throw(const BindingException&) 
  {
    connect(n1.get(), "default", n2.get(), "default", format, 0); 
  }

  void connect(INode_var& n1, const string& out_jack, 
	       INode_var& n2,  
	       const Format* format)
    throw(const BindingException&) 
  { 
    connect(n1.get(), out_jack, n2.get(), "default", format, 0); 
  }

  void connect(INode_var& n1,  
	       INode_var& n2, const string& in_jack, 
	       const Format* format)
    throw(const BindingException&) 
  { 
    connect(n1.get(), "default", n2.get(), in_jack, format, 0); 
  }

  void connect(INode_var& n1, const string& out_jack, 
	       INode_var& n2, const string& in_jack)
    throw(const BindingException&) 
  { 
    connect(n1.get(), out_jack, n2.get(), in_jack, 0, 0); 
  }

  void connect(INode_var& n1,  
	       INode_var& n2)
    throw(const BindingException&) 
  { 
    connect(n1.get(), "default", n2.get(), "default", 0, 0); 
  }

  void connect(INode_var& n1, const string& out_jack, 
	       INode_var& n2)
    throw(const BindingException&) 
  { 
    connect(n1.get(), out_jack, n2.get(), "default", 0, 0); 
  }

  void connect(INode_var& n1,  
	       INode_var& n2, const string& in_jack)
    throw(const BindingException&) 
  { 
    connect(n1.get(), "default", n2.get(), in_jack, 0, 0); 
  }

  /************************* real_unchecked_c_connect **********************************/

  InstreamChannel* real_unchecked_c_connect(INode* n1, const string& out_jack, 
						 INode* n2, const string& in_jack, 
						 const Format* given_format, 
						 bool setup_comchannel) 
    throw(const BindingException&)
  {
    string name1 = "UndefNode1";
    string name2 = "UndefNode2";
    try {
      // check node n1 
      if(n1 == 0) {
	GLOBAL_ERROR_STREAM1("Node 1 is 0" << endl);
	throw BindingException();
      }
      if(n2 == 0) {
	GLOBAL_ERROR_STREAM1("Node 2 is 0" << endl);
	throw BindingException();
      }

      INamed_var named_n1(n1->getParentObject()->getInterface<INamed>());
      if(named_n1.get()) {
	name1 = named_n1->getName();
      }
      INamed_var named_n2(n2->getParentObject()->getInterface<INamed>());
      if(named_n2.get()) {
	name2 = named_n2 ->getName();
      }

      name1 = name1 + "::" + out_jack;
      name2 = name2 + "::" + in_jack;

      if(given_format == 0) {
	GLOBAL_ERROR_STREAM1("uncheked_c_connect needs format, cannot connect " << name1 << " with " << name2 << endl);
	throw BindingException();
      }
      else {
	GLOBAL_DEBUG_STREAM1("Trying to connect " << name1 << " with " << name2 << " taking given format " 
			     << endl << *given_format << endl);
      }

      if(given_format->hasFormatWildcards() || given_format->hasFormatRanges()) {
	GLOBAL_WARNING_STREAM("Given format has wildcards or ranges: " << endl <<  (*given_format) << endl);
	throw BindingException();
      }

      // try to get jacks
      IOutputJack_var out(n1->getUnintializedOutputJack(*given_format, out_jack));
      IInputJack_var in(n2->getInputJack(*given_format, in_jack));
    
      if (in.get() == 0) {
	GLOBAL_ERROR_STREAM1(name2 << ": Could not get input jack " << in_jack << endl);
	throw BindingException();
      }
      if(out.get() == 0) {
	GLOBAL_ERROR_STREAM1(name1 << ": Could not get output jack " << out_jack << endl);
	throw BindingException();
      }

      // in jack already connected?
      if(in->isConnected()) {
	GLOBAL_ERROR_STREAM1(name2 << ": Input jack " << in_jack << " already connected " << endl);
	throw BindingException();
      }
      // out jack already connected? 
      if(out->isConnected()) {
	GLOBAL_ERROR_STREAM1(name1 << ": Output jack " << out_jack << " already connected " << endl);
	throw BindingException();
      }

      // finally: try to connect jacks
      InstreamChannel* channel = 0;
      try {
	channel = out->connectTo(*in);
      } catch (const BindingException& e) {
	GLOBAL_ERROR_STREAM1(name1 << ": Could not connect out jack to other in jack" << endl);
	// release channel because exception will be thrown
	delete channel;
	throw e; 
      }
      try {
	InstreamChannel_var (in->connectTo(*out));
      } catch (const BindingException& e) {
	GLOBAL_ERROR_STREAM1(name2 << ": Could not connect in jack to other out jack" << endl);
	// disconnect already connected jack out again
	out->disconnect();
	throw e; 
      }
      
      if(setup_comchannel) {
        /* establish binding */
        channel->establishBinding();
      }
      
      // always: return channel (this is the real_c_connect method!)
      return channel;

    }
    catch(const SerializedException& e) {
      GLOBAL_ERROR_STREAM("Exception during c_connect " << name1 << "->" << name2 << ": " << e << endl);
      ostringstream o;
      o << "Exception during c_connect " << name1 << "->" << name2 << ": " << e;
      throw BindingException(o.str());
    }
    catch(...) {
      GLOBAL_ERROR_STREAM("Unknown exception during c_connect " << name1 << "->" << name2 << endl);
      throw BindingException("Unknown exception during c_connect " + name1 + "->" + name2);
    }
  }

  void unchecked_connect(INode* n1, const string& out_jack, 
			 INode* n2, const string& in_jack, 
			 const Format* given_format)
    throw(const BindingException&)
  {
    InstreamChannel_var c(real_unchecked_c_connect(n1, out_jack, n2, in_jack, given_format, true));
  }

  InstreamChannel* unchecked_c_connect(INode* n1, const string& out_jack, 
					    INode* n2, const string& in_jack, 
					    const Format* given_format)
    throw(const BindingException&)
  {
    return (real_unchecked_c_connect(n1, out_jack, n2, in_jack, given_format, false));
  }

  /************************* real_c_connect **********************************/

  InstreamChannel* real_c_connect(INode* n1, const string& out_jack, 
				       INode* n2, const string& in_jack, 
				       const Format* given_format, Format* connection_format,
				       bool setup_comchannel) 
    throw(const BindingException&)
  {
    string name1 = "UndefNode1";
    string name2 = "UndefNode2";
    try {
      // check node n1 
      if(n1 == 0) {
	GLOBAL_ERROR_STREAM1("Node 1 is 0" << endl);
	throw BindingException();
      }
      if(n2 == 0) {
	GLOBAL_ERROR_STREAM1("Node 2 is 0" << endl);
	throw BindingException();
      }

      INamed_var named_n1(n1->getParentObject()->getInterface<INamed>());
      if(named_n1.get()) {
	name1 = named_n1->getName();
      }
      INamed_var named_n2(n2->getParentObject()->getInterface<INamed>());
      if(named_n2.get()) {
	name2 = named_n2 ->getName();
      }

      name1 = name1 + "::" + out_jack;
      name2 = name2 + "::" + in_jack;

      if(given_format == 0) {
	GLOBAL_DEBUG_STREAM1("Trying to connect " << name1 << " with " << name2 << endl);
      }
      else {
	GLOBAL_DEBUG_STREAM1("Trying to connect " << name1 << " with " << name2 << " taking given format " 
			     << endl << *given_format << endl);
      }

      Format_var intersec_format; // holds the final intersection format

      Property_var w_out_property(n1->getWorkingOutputProperty());

      Property_var in_property(n2->getInputProperty());

      // *** first, try to match both default formats 
      //     (default formats were not copied!)
      const Format* def_out_format = 0;
      def_out_format = w_out_property->getDefaultFormat(out_jack);
      const Format* def_in_format = 0;
      def_in_format  = in_property->getDefaultFormat(in_jack);

      if(def_out_format && def_in_format) {
	Format_var io_intersec_format(def_out_format->createIntersectionFormat(def_in_format));

	if(io_intersec_format.get()) {
	  // if format was given -> need to create some intersection 
	  if(given_format) {
	    intersec_format.reset(io_intersec_format->createIntersectionFormat(given_format));
	    if(intersec_format.get()) {
	      GLOBAL_DEBUG_STREAM1("default formats intersected with given format " << endl << *intersec_format << endl);
	      // if format still has wildwards or ranges -> forget it
	      if (intersec_format->hasFormatWildcards() || intersec_format->hasFormatRanges()) {
		intersec_format.reset(NULL);
	      }
	    }
	    else { // no total intersection -> try subset intersection
	      intersec_format.reset(io_intersec_format->createSubsetIntersectionFormat(given_format));
	      if(intersec_format.get()) {
		GLOBAL_DEBUG_STREAM1("default formats subset-intersected with given format " << endl << *intersec_format << endl);
		// if format still has wildwards or ranges -> forget it
		if (intersec_format->hasFormatWildcards() || intersec_format->hasFormatRanges()) {
		  intersec_format.reset(NULL);
		}
	      }
	    }
	  }
	  else { // no format was given -> still check for wildcards 
	    intersec_format = io_intersec_format; //using operator= to copy the pointer.
	    if(intersec_format.get()) {
	      GLOBAL_DEBUG_STREAM1("default formats intersected only " << endl << *intersec_format << endl);
	      // if format still has wildwards or ranges -> forget it
	      if (intersec_format->hasFormatWildcards() || intersec_format->hasFormatRanges()) {
		intersec_format.reset(NULL);
	      }
	    }
	  }
	}
      }
      else {
	if(!def_out_format) {
	  GLOBAL_DEBUG_STREAM1("No def out format at " << name1 << endl);
	}
	if(!def_in_format) {
	  GLOBAL_DEBUG_STREAM1("No def in format at " << name2 << endl);
	}
      }

      if(!intersec_format.get()) {
	// *** try to match default out-format with input-property (not including default in format)
	const_format_list in_format_list;
	if(def_out_format) {
	  in_format_list = in_property->createIntersectionFormats(def_out_format, in_jack, false);
	  in_format_list.sort();
	  GLOBAL_DEBUG_STREAM1( "in_format_list " << in_format_list.size() << endl);
	  // printFormatList(in_format_list);
	}

	// *** try to match default in-format with working-output-property (not including default out format)
	const_format_list w_out_format_list;
	if(def_in_format) {
	  w_out_format_list = w_out_property->createIntersectionFormats(def_in_format, out_jack, false);
	  w_out_format_list.sort();
	  GLOBAL_DEBUG_STREAM1( "w_out_format_list " << w_out_format_list.size() << endl);
	  // printFormatList(w_out_format_list);
	}
    
	// *** try to match all formats of in- and working-out-property (not including any default formats)
	const_format_list io_format_list = 
	  w_out_property->createIntersectionFormats(out_jack, in_property.get(), in_jack, false);
	io_format_list.sort();
	GLOBAL_DEBUG_STREAM1("io_format_list " << io_format_list.size() << endl);
	// printFormatList(io_format_list);

	// *** splice all list together: io_format_list, w_out_format_list, in_format_list (remain sorted)
	//     -> the last element of in_format_list should be found first (since reverse iterator is taken)
	io_format_list.splice(io_format_list.end(), w_out_format_list);
	io_format_list.splice(io_format_list.end(), in_format_list);

	// *** if format was given as parameter -> find first intersection with given format 
	if(given_format) {
	  const_format_list::const_reverse_iterator i_end = io_format_list.rend();
	  for(const_format_list::const_reverse_iterator i = io_format_list.rbegin(); i != i_end; ++i) {
	    intersec_format.reset((*i)->createIntersectionFormat(given_format));
	    // check if full intersection format exits -> if not try subset-intersection
	    if(!intersec_format.get()) {
	      intersec_format.reset((*i)->createSubsetIntersectionFormat(given_format));
	    }
	    // found format that does not contain any wildcards or ranges -> then take this format
	    if(intersec_format.get() && !(intersec_format->hasFormatWildcards()) && !(intersec_format->hasFormatRanges())) {
	      break;
	    }
	  }
	}
	// *** otherwise, take last format in list, check for wildcards, and create a copy (because all other cases own their own copy)
	else {
	  while (!io_format_list.empty() && !intersec_format.get()) {
            // take the last Format from the io_format_list (this is a
            // different one on every iteration as this while loop
            // always removes the last one if it cannot find a
            // appropriate Format and otherwise, it stops iterating!)
            const_format_list::reverse_iterator curr_format = io_format_list.rbegin();
            if ((*curr_format)->hasFormatWildcards() || (*curr_format)->hasFormatRanges()) {
              // this Format has wildcards or ranges and so is not useful as
              // intersection Format (== connection Format), so we
              // remove it from the list as it is no longer needed
              delete (*curr_format);
              io_format_list.erase(--const_format_list::iterator(curr_format.base()));
            } else {
              // we found a useful Format in the io_format_list, so
              // take it. This will also stop the while-loop!
              intersec_format.reset(new Format(*(*curr_format)));
            }
	  }
	}

	// *** clean-up some memory
	// delete only formats in io_format_list, since other list use the same memory
	for(const_format_list::const_iterator i = io_format_list.begin(); i != io_format_list.end(); ++i) {
	  delete (*i);
	}
      } // end: if(!intersec_format)

      // *** clean-up some memory

      if(!intersec_format.get()) {
	GLOBAL_WARNING_STREAM("Could not create an intersection format for " << name1 << "->" << name2 << endl);
	throw BindingException();
      }

      if(intersec_format->hasFormatWildcards() || intersec_format->hasFormatRanges()) {
	GLOBAL_WARNING_STREAM("Intersection format still has wildcards or ranges: " << endl <<  (*intersec_format.get()) << endl);
	throw BindingException();
      }

      GLOBAL_DEBUG_STREAM1("Taking intersection format " << (*intersec_format.get()) << endl);

      // try to get jacks
      IOutputJack_var out(n1->getOutputJack(*intersec_format.get(), out_jack));
      IInputJack_var in(n2->getInputJack(*intersec_format.get(), in_jack));
    
      if (in.get() == 0) {
	GLOBAL_ERROR_STREAM1(name2 << ": Could not get input jack " << in_jack << endl);
	throw BindingException();
      }
      if(out.get() == 0) {
	GLOBAL_ERROR_STREAM1(name1 << ": Could not get output jack " << out_jack << endl);
	throw BindingException();
      }

      // in jack already connected?
      if(in->isConnected()) {
	GLOBAL_ERROR_STREAM1(name2 << ": Input jack " << in_jack << " already connected " << endl);
	throw BindingException();
      }
      // out jack already connected? 
      if(out->isConnected()) {
	GLOBAL_ERROR_STREAM1(name1 << ": Output jack " << out_jack << " already connected " << endl);
	throw BindingException();
      }

      // finally: try to connect jacks
      InstreamChannel* channel = 0;
      try {
	channel = out->connectTo(*in);
      } catch (const BindingException& e) {
	GLOBAL_ERROR_STREAM1(name1 << ": Could not connect out jack to other in jack" << endl);
	// release channel because exception will be thrown
	delete channel;
	throw e; 
      }
      try {
	InstreamChannel_var (in->connectTo(*out));
      } catch (const BindingException& e) {
	GLOBAL_ERROR_STREAM1(name2 << ": Could not connect in jack to other out jack" << endl);
	// disconnect already connected jack out again
	out->disconnect();
	throw e; 
      }

      if(setup_comchannel) {
	// take first strategy in list
	channel->establishBinding();
      }

      if (connection_format) {
        // store the used connection format in connection_format
        *connection_format = *intersec_format.get();
      }
      // always: return channel (this is the real_c_connect method!)
      return channel;

    }
    catch(const SerializedException& e) {
      GLOBAL_ERROR_STREAM("Exception during c_connect " << name1 << "->" << name2 << ": " << e << endl);
      ostringstream o;
      o << "Exception during c_connect " << name1 << "->" << name2 << ": " << e;
      throw BindingException(o.str());
    }
    catch(...) {
      GLOBAL_ERROR_STREAM("Unknown exception during c_connect " << name1 << "->" << name2 << endl);
      throw BindingException("Unknown exception during c_connect " + name1 + "->" + name2);
    }
  }

  /**************************************************************/

  void connect(INode* n1, const string& out_jack, 
	       INode* n2, const string& in_jack, 
	       const Format* format, Format* connection_format)
    throw(const BindingException&) 
  {
    // true means: setup com channel
    InstreamChannel_var c(real_c_connect(n1, out_jack, n2, in_jack, format, connection_format, true));
  }

  /********************** INode c_connect *********************/

  InstreamChannel* c_connect(INode* n1, const string& out_jack,
				  INode* n2, const string& in_jack,
				  const Format* format, Format* connection_format)
    throw(const BindingException&)
  {
    // false means: do not setup com channel
    return real_c_connect(n1, out_jack, n2, in_jack, format, connection_format, false);
  }

  InstreamChannel* c_connect(INode* n1, const string& out_jack, 
				  INode* n2, const string& in_jack, 
				  const Format* format) 
    throw(const BindingException&) 
  {
    // false means: do not setup com channel
    return c_connect(n1, out_jack, n2, in_jack, format, 0);
  }

  InstreamChannel* c_connect(INode* n1,  
				  INode* n2,  
				  const Format* format)
    throw(const BindingException&) 
  {
    return c_connect(n1, "default", n2, "default", format, 0); 
  }

  InstreamChannel* c_connect(INode* n1, const string& out_jack, 
				  INode* n2,  
				  const Format* format)
    throw(const BindingException&) 
  { 
    return c_connect(n1, out_jack, n2, "default", format, 0); 
  }

  InstreamChannel* c_connect(INode* n1,  
				  INode* n2, const string& in_jack, 
				  const Format* format)
    throw(const BindingException&) 
  { 
    return c_connect(n1, "default", n2, in_jack, format, 0); 
  }

  InstreamChannel* c_connect(INode* n1, const string& out_jack, 
				  INode* n2, const string& in_jack)
    throw(const BindingException&) 
  { 
    return c_connect(n1, out_jack, n2, in_jack, 0, 0); 
  }

  InstreamChannel* c_connect(INode* n1,  
				  INode* n2)
    throw(const BindingException&) 
  { 
    return c_connect(n1, "default", n2, "default", 0, 0); 
  }

  InstreamChannel* c_connect(INode* n1, const string& out_jack, 
				  INode* n2)
    throw(const BindingException&) 
  { 
    return c_connect(n1, out_jack, n2, "default", 0, 0); 
  }

  InstreamChannel* c_connect(INode* n1,  
				  INode* n2, const string& in_jack)
    throw(const BindingException&) 
  { 
    return c_connect(n1, "default", n2, in_jack, 0, 0); 
  }


  /********************** INode_var connect *********************/

  InstreamChannel* c_connect(INode_var& n1, const string& out_jack, 
				  INode_var& n2, const string& in_jack, 
				  const Format* format)
    throw(const BindingException&) 
  { 
    return c_connect(n1.get(), out_jack, n2.get(), in_jack, format, 0); 
  }

  InstreamChannel* c_connect(INode_var& n1,  
				  INode_var& n2,  
				  const Format* format)
    throw(const BindingException&) 
  {
    return c_connect(n1.get(), "default", n2.get(), "default", format, 0); 
  }

  InstreamChannel* c_connect(INode_var& n1, const string& out_jack, 
				  INode_var& n2,  
				  const Format* format)
    throw(const BindingException&) 
  { 
    return c_connect(n1.get(), out_jack, n2.get(), "default", format, 0); 
  }

  InstreamChannel* c_connect(INode_var& n1,  
				  INode_var& n2, const string& in_jack, 
				  const Format* format)
    throw(const BindingException&) 
  { 
    return c_connect(n1.get(), "default", n2.get(), in_jack, format, 0); 
  }

  InstreamChannel* c_connect(INode_var& n1, const string& out_jack, 
				  INode_var& n2, const string& in_jack)
    throw(const BindingException&) 
  { 
    return c_connect(n1.get(), out_jack, n2.get(), in_jack, 0, 0); 
  }

  InstreamChannel* c_connect(INode_var& n1,  
				  INode_var& n2)
    throw(const BindingException&) 
  { 
    return c_connect(n1.get(), "default", n2.get(), "default", 0, 0); 
  }

  InstreamChannel* c_connect(INode_var& n1, const string& out_jack, 
				  INode_var& n2)
    throw(const BindingException&) 
  { 
    return c_connect(n1.get(), out_jack, n2.get(), "default", 0, 0); 
  }

  InstreamChannel* c_connect(INode_var& n1,  
				  INode_var& n2, const string& in_jack)
    throw(const BindingException&) 
  { 
    return c_connect(n1.get(), "default", n2.get(), in_jack, 0, 0); 
  }


  /**************************************************************/

  /********************** disconnect *********************/

  void quick_disconnect_output(INode_var& n, const string& out_jack /* = "default" */ )
    throw(const UnbindingException&)
  {
    quick_disconnect_output(n.get(), out_jack);
  }

  void quick_disconnect_output(INode* n, const string& out_jack /* = "default" */ )
    throw(const UnbindingException&)
  {
    string name = "UndefNode";
    try {
      IOutputJack_var out(n->getOutputStream(out_jack));
      if (out.get() == 0) {
        GLOBAL_ERROR_STREAM2(name << ": could not get output jack " << out_jack << endl);
        throw UnbindingException(name + ": could not get output jack " + out_jack);
      }
      
      IInputJack_var in(out->getDataPartner());
      
      if (in.get() == 0) {
        GLOBAL_ERROR_STREAM2(name << ": Could not get partner jack of output jack " << out_jack << endl);
        throw UnbindingException(name + ": Could not get partner jack of output jack " + out_jack);
      }
      
      in->disconnect();
      out->disconnect();
      
      GLOBAL_DEBUG_STREAM2(name << ": Disconnect of output jack " << out_jack << " successful" << endl);
    }
    catch(const SerializedException& e) {
      GLOBAL_ERROR_STREAM(name << ": Exception during disconnect_output " << e << endl);
      ostringstream o;
      o << name << ": Exception during disconnect_output " << e;
      throw UnbindingException(o.str());
    }
    catch(...) {
      GLOBAL_ERROR_STREAM(name << ": Unknown exception during disconnect_output " << endl);
      throw UnbindingException(name + ": Unknown exception during disconnect_output");
    }
  }

  void disconnect_output(INode_var& n, const string& out_jack /* = "default" */ )
    throw(const UnbindingException&)
  {
    disconnect_output(n.get(), out_jack);
  }

  void disconnect_output(INode* n, const string& out_jack /* = "default" */ )
    throw(const UnbindingException&)
  {
    string name = "UndefNode";
    try {
      INamed_var named_n(n->getParentObject()->getInterface<INamed>());
      if(named_n.get()) {
        name = named_n->getName();
      }
      
      IOutputJack_var out(n->getOutputStream(out_jack));
      if (out.get() == 0) {
        GLOBAL_ERROR_STREAM2(name << ": could not get output jack " << out_jack << endl);
        throw UnbindingException(name + ": could not get output jack " + out_jack);
      }
      
      /* get connected jack : get partner jack, then the parent of the partner jack, 
         and then again the jack from the node with the jack tag
         this forces to call getInputStream and automatically checks for the correct
         node state
      */
      IInputJack_var in(out->getDataPartner());
      // INode_var n2(in->getIParent());
      // string tag = in->getJackTag();
      // in.reset(n2->getInputStream(tag));
      
      if (in.get() == 0) {
        GLOBAL_ERROR_STREAM2(name << ": Could not get partner jack of output jack " << out_jack << endl);
        throw UnbindingException(name + ": Could not get partner jack of output jack " + out_jack);
      }
      
      in->disconnect();
      out->disconnect();
      
      GLOBAL_DEBUG_STREAM2(name << ": Disconnect of output jack " << out_jack << " successful" << endl);
    }
    catch(const SerializedException& e) {
      GLOBAL_ERROR_STREAM(name << ": Exception during disconnect_output " << e << endl);
      ostringstream o;
      o << name << ": Exception during disconnect_output " << e;
      throw UnbindingException(o.str());
    }
    catch(...) {
      GLOBAL_ERROR_STREAM(name << ": Unknown exception during disconnect_output " << endl);
      throw UnbindingException(name + ": Unknown exception during disconnect_output");
    }
  }

  void disconnect_input(INode_var& n, const string& in_jack /* = "default" */ )
    throw(const UnbindingException&)
  {
    disconnect_input(n.get(), in_jack);
  }

  void disconnect_input(INode* n, const string& in_jack /* = "default" */ )
    throw(const UnbindingException&)
  {
    string name = "UndefNode";
    try {
      INamed_var named_n(n->getParentObject()->getInterface<INamed>());
      if(named_n.get()) {
	name = named_n->getName();
      }

      IInputJack_var in(n->getInputStream(in_jack));
      if (in.get() == 0) {
	GLOBAL_ERROR_STREAM2(name << ": could not get input jack " << in_jack << endl);
	throw UnbindingException(name + ": could not get input jack " + in_jack);
      }

      /* get connected jack : get partner jack, then the parent of the partner jack, 
	 and then again the jack from the node with the jack tag
	 this forces to call getOutputStream and automatically checks for the correct
	 node state
      */
      IOutputJack_var out(in->getDataPartner());
      // INode_var n2(out->getIParent());
      // string tag = out->getJackTag();
      // out.reset(n2->getOutputStream(tag));

      if (out.get() == 0) {
	GLOBAL_ERROR_STREAM2(name << ": Could not get partner jack of input jack " << in_jack << endl);
	throw UnbindingException(name + ": Could not get partner jack of input jack " + in_jack);
      }

      in->disconnect();
      out->disconnect();
        
      GLOBAL_DEBUG_STREAM2(name << ": Disconnect of input jack " << in_jack << " successful" << endl);
    }
    catch(const SerializedException& e) {
      GLOBAL_ERROR_STREAM(name << ": Exception during disconnect_input " << e << endl);
      ostringstream o;
      o << name << ": Exception during disconnect_input " << e;
      throw UnbindingException(o.str());
    }
    catch(...) {
      GLOBAL_ERROR_STREAM(name << ": Unknown exception during disconnect_input " << endl);
      throw UnbindingException(name + ": Unknown exception during disconnect_input");
    }
  }

  /********************** s_connect ***********************/
  
  InstreamChannel* real_s_connect(INode* n1, const string& out_jack, 
				       INode* n2, const string& in_jack,
                                       Format* connection_format,
				       bool setup_comchannel)
    throw(const BindingException&)
  {
    string name1 = "UndefNode1";
    string name2 = "UndefNode2";
    try {
      // check node n1 
      if(n1 == 0) {
	GLOBAL_ERROR_STREAM1("Node 1 is 0" << endl);
	throw BindingException();
      }
      if(n2 == 0) {
	GLOBAL_ERROR_STREAM1("Node 2 is 0" << endl);
	throw BindingException();
      }

      INamed_var named_n1(n1->getParentObject()->getInterface<INamed>());
      if(named_n1.get()) {
	name1 = named_n1->getName();
      }

      INamed_var named_n2(n2->getParentObject()->getInterface<INamed>());
      if(named_n2.get()) {
	name2 = named_n2 ->getName();
      }

      name1 = name1 + "::" + out_jack;
      name2 = name2 + "::" + in_jack;

      GLOBAL_DEBUG_STREAM1("Trying to create shared connection: " << name1 << " with " << name2 << endl);
      // try to get jacks
      Format* f = 0;
      IOutputJack_var out(n1->cloneOutputJack(f, out_jack));
      if(f == 0) {
	GLOBAL_ERROR_STREAM1(name1 << ": Could not clone output jack " << out_jack << endl);
	throw BindingException();
      }
      Format_var intersec_format(f);
      IInputJack_var in(n2->getInputJack(*intersec_format.get(), in_jack));
    
      if (in.get() == 0) {
	GLOBAL_ERROR_STREAM1(name2 << ": Could not get input jack " << in_jack << endl);
	throw BindingException();
      }
      if(out.get() == 0) {
	GLOBAL_ERROR_STREAM1(name1 << ": Could not get output jack " << out_jack << endl);
	throw BindingException();
      }

      // in jack already connected?
      if(in->isConnected()) {
	GLOBAL_ERROR_STREAM1(name2 << ": Input jack " << in_jack << " already connected " << endl);
	throw BindingException();
      }
      // out jack already connected? 
      if(out->isConnected()) {
	GLOBAL_ERROR_STREAM1(name1 << ": Output jack " << out_jack << " already connected, this should not happen!" << endl);
	throw BindingException();
      }

      // finally: try to connect jacks
      InstreamChannel* channel = 0;
      try {
	channel = out->connectTo(*in);
      } catch (const BindingException& e) {
	GLOBAL_ERROR_STREAM1(name1 << ": Could not connect out jack to other in jack" << endl);
	// release channel because exception will be thrown
	delete channel;
	throw e; 
      }
      try {
	InstreamChannel_var (in->connectTo(*out));
      } catch (const BindingException& e) {
	GLOBAL_ERROR_STREAM1(name2 << ": Could not connect in jack to other out jack" << endl);
	// disconnect already connected jack out again
	out->disconnect();
	throw e; 
      }

      if(setup_comchannel) {
	// take first strategy in list
	channel->establishBinding();
      }
      if (connection_format) {
        *connection_format = *intersec_format.get();
      }
      // always: return channel (this is the real_c_connect method!)
      return channel;

    }
    catch(const SerializedException& e) {
      GLOBAL_ERROR_STREAM("Exception during c_connect " << name1 << "->" << name2 << ": " << e << endl);
      ostringstream o;
      o << "Exception during c_connect " << name1 << "->" << name2 << ": " << e;
      throw BindingException(o.str());
    }
    catch(...) {
      GLOBAL_ERROR_STREAM("Unknown exception during c_connect " << name1 << "->" << name2 << endl);
      throw BindingException("Unknown exception during c_connect " + name1 + "->" + name2);
    }
  }

  void s_connect(INode* n1, const string& out_jack, 
		 INode* n2, const string& in_jack, Format* connection_format) 
    throw(const BindingException&) 
  {
    // true means: setup com channel
    InstreamChannel_var c(real_s_connect(n1, out_jack, n2, in_jack, connection_format, true));
  }

  InstreamChannel* c_s_connect(INode* n1, const string& out_jack,
		 INode* n2, const string& in_jack, Format* connection_format)
    throw(const BindingException&)
  {
    // false means: don't setup com channel
    return real_s_connect(n1, out_jack, n2, in_jack, connection_format, false);
  }

}


#include "nmm/nmm_suffix.hpp"
