/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IMute.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IMute::used() {} 
  
  
  void IMute::mute() /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IMute::mute");
    if(const_cast<IMute*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMute::unmute() /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IMute::unmute");
    if(const_cast<IMute*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IMute::IMute_type = "IMute";
  NMM::TConstructor<IMute > __IMute_prototype("IMute","NMM");


  const char* IMute::mute_event = "IMute::mute";
  NMM::Event* IMute::create_mute()
  {

    return new NMM::Event("IMute::mute");
  }

  const char* IMute::unmute_event = "IMute::unmute";
  NMM::Event* IMute::create_unmute()
  {

    return new NMM::Event("IMute::unmute");
  }



}


#include "nmm/nmm_suffix.hpp"

