/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IMuteImpl_HPP
#define NMM_IMuteImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IMute.hpp"


#include "visibility.hpp"

namespace NMM
{
  /** An interface for muting an audio source or sink.
   *
   * This interface defines instream events which cause an audio source or sink to be muted
   * when it receives the event.
   */

  class DLLEXPORT IMuteImpl  : public virtual NMM::NMMObject { 
  public:
    IMuteImpl() { addInterface(new IMute(*this)); registerEvents(); };

    virtual ~IMuteImpl() {};

  protected:
    /** Mute an audio source or sink.
     */

    virtual void mute() /* in-stream */  = 0;
    /** Un-mute an audio source or sink.
     */

    virtual void unmute() /* in-stream */  = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
