/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IProgressListener.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IProgressListener::used() {} 
  
  
  void IProgressListener::setProgress(const nmm_uint64& numerator, const nmm_uint64& denominator) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint64 > numerator_value(numerator, false);
    NMM::TInValue<nmm_uint64 > denominator_value(denominator, false);
    NMM::Event event("IProgressListener::setProgress", &numerator_value, &denominator_value);
    if(const_cast<IProgressListener*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IProgressListener::IProgressListener_type = "IProgressListener";
  NMM::TConstructor<IProgressListener > __IProgressListener_prototype("IProgressListener","NMM");


  const char* IProgressListener::setProgress_event = "IProgressListener::setProgress";
  NMM::Event* IProgressListener::create_setProgress(const nmm_uint64& numerator, const nmm_uint64& denominator)
  {
    NMM::TInValue<nmm_uint64 >* numerator_value = new NMM::TInValue<nmm_uint64 >();
    numerator_value->setValue(numerator);
    NMM::TInValue<nmm_uint64 >* denominator_value = new NMM::TInValue<nmm_uint64 >();
    denominator_value->setValue(denominator);

    return new NMM::Event("IProgressListener::setProgress", numerator_value, denominator_value);
  }



}


#include "nmm/nmm_suffix.hpp"

