/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IProgressListener_HPP
#define NMM_IProgressListener_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{
  /** A listener for progress events.
   *
   * This interface defines instream events that carry progress information.
   * Progress information is sent by nodes that implement the IProgress interface.
   * Applications that need progress information typically register listeners
   * for progress events.
   */

  class DLLEXPORT IProgressListener : virtual public NMM::Interface { 
  public:
    IProgressListener() : NMM::Interface(IProgressListener::IProgressListener_type, "NMM") { };

    IProgressListener(NMM::NMMObject& parent) : NMM::Interface (parent, IProgressListener::IProgressListener_type, "NMM") { };

    virtual ~IProgressListener() {};

    static const char* IProgressListener_type;

    virtual  string getInterfaceType() const { return IProgressListener_type; }

    NMM::Interface* copy() const { return new IProgressListener(*this); }
    static void used(); 
    static const char* setProgress_event;
    static NMM::Event* create_setProgress(const nmm_uint64& numerator, const nmm_uint64& denominator);
  /* instream (protected) and non-instream (public) */        /** Progress information (instream event).
     * Instream event containing the current progress state in the form
     * @c numerator / @c denominator.
     *
     * @param numerator Progress numerator (current progress)
     * @param denominator Progress denominator (maximum progress)
     */

    virtual void setProgress(const nmm_uint64& numerator, const nmm_uint64& denominator) /* in-stream */ 
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IProgressListener> IProgressListener_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
