/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ITimeProgress.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/serialize/SerializeUtilsTime.hpp"

namespace NMM
{
  void ITimeProgress::used() {} 
  
  
  void ITimeProgress::setProgressUpdateInterval(const Interval& update_interval)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<Interval > update_interval_value(update_interval, false);
    NMM::Event event("ITimeProgress::setProgressUpdateInterval", &update_interval_value);
    if(const_cast<ITimeProgress*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITimeProgress::setProgressUpdateEnabled(const nmm_bool& enable_update)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > enable_update_value(enable_update, false);
    NMM::Event event("ITimeProgress::setProgressUpdateEnabled", &enable_update_value);
    if(const_cast<ITimeProgress*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* ITimeProgress::ITimeProgress_type = "ITimeProgress";
  NMM::TConstructor<ITimeProgress > __ITimeProgress_prototype("ITimeProgress","NMM");


  const char* ITimeProgress::setProgressUpdateInterval_event = "ITimeProgress::setProgressUpdateInterval";
  const char* ITimeProgress::setProgressUpdateEnabled_event = "ITimeProgress::setProgressUpdateEnabled";


}


#include "nmm/nmm_suffix.hpp"

