/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ITimeProgressListener.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/serialize/SerializeUtilsTime.hpp"

namespace NMM
{
  void ITimeProgressListener::used() {} 
  
  
  void ITimeProgressListener::setTimeProgress(const Time& current_media_time) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<Time > current_media_time_value(current_media_time, false);
    NMM::Event event("ITimeProgressListener::setTimeProgress", &current_media_time_value);
    if(const_cast<ITimeProgressListener*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* ITimeProgressListener::ITimeProgressListener_type = "ITimeProgressListener";
  NMM::TConstructor<ITimeProgressListener > __ITimeProgressListener_prototype("ITimeProgressListener","NMM");


  const char* ITimeProgressListener::setTimeProgress_event = "ITimeProgressListener::setTimeProgress";
  NMM::Event* ITimeProgressListener::create_setTimeProgress(const Time& current_media_time)
  {
    NMM::TInValue<Time >* current_media_time_value = new NMM::TInValue<Time >();
    current_media_time_value->setValue(current_media_time);

    return new NMM::Event("ITimeProgressListener::setTimeProgress", current_media_time_value);
  }



}


#include "nmm/nmm_suffix.hpp"

