/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ITimeProgressListener_HPP
#define NMM_ITimeProgressListener_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/utils/time/Time.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** A listener for time progress events.
   *
   * This interface defines instream events that carry time progress information.
   * Time progress information is sent by nodes that implement the ITimeProgress interface.
   * Applications that need time progress information typically register listeners
   * for time progress events.
   */

  class DLLEXPORT ITimeProgressListener : virtual public NMM::Interface { 
  public:
    ITimeProgressListener() : NMM::Interface(ITimeProgressListener::ITimeProgressListener_type, "NMM") { };

    ITimeProgressListener(NMM::NMMObject& parent) : NMM::Interface (parent, ITimeProgressListener::ITimeProgressListener_type, "NMM") { };

    virtual ~ITimeProgressListener() {};

    static const char* ITimeProgressListener_type;

    virtual  string getInterfaceType() const { return ITimeProgressListener_type; }

    NMM::Interface* copy() const { return new ITimeProgressListener(*this); }
    static void used(); 
    static const char* setTimeProgress_event;
    static NMM::Event* create_setTimeProgress(const Time& current_media_time);
  /* instream (protected) and non-instream (public) */        /** Time progress information (instream event).
     * Instream event containing the current media time
     *
     * @param current_media_time Current media time
     */

    virtual void setTimeProgress(const Time& current_media_time) /* in-stream */ 
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ITimeProgressListener> ITimeProgressListener_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
