/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ITimeProgressListenerImpl_HPP
#define NMM_ITimeProgressListenerImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "ITimeProgressListener.hpp"


#include "visibility.hpp"

namespace NMM
{
  /** A listener for time progress events.
   *
   * This interface defines instream events that carry time progress information.
   * Time progress information is sent by nodes that implement the ITimeProgress interface.
   * Applications that need time progress information typically register listeners
   * for time progress events.
   */

  class DLLEXPORT ITimeProgressListenerImpl  : public virtual NMM::NMMObject { 
  public:
    ITimeProgressListenerImpl() { addInterface(new ITimeProgressListener(*this)); registerEvents(); };

    virtual ~ITimeProgressListenerImpl() {};

  protected:
    /** Time progress information (instream event).
     * Instream event containing the current media time
     *
     * @param current_media_time Current media time
     */

    virtual void setTimeProgress(const Time& current_media_time) /* in-stream */  = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
