/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ITrack.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/serialize/SerializeUtilsTime.hpp"
#include "nmm/utils/time/Time.hpp"

namespace NMM
{
  void ITrack::used() {} 
  
  
  Result ITrack::startTrack(const string& filename) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > filename_value(filename, false);
    NMM::Event event("ITrack::startTrack", &filename_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITrack*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITrack::endTrack() /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("ITrack::endTrack");
    event.setReturnValue(&return_value);
    if(const_cast<ITrack*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ITrack::ITrack_type = "ITrack";
  NMM::TConstructor<ITrack > __ITrack_prototype("ITrack","NMM");


  const char* ITrack::startTrack_event = "ITrack::startTrack";
  NMM::Event* ITrack::create_startTrack(const string& filename)
  {
    NMM::TInValue<string >* filename_value = new NMM::TInValue<string >();
    filename_value->setValue(filename);

    return new NMM::Event("ITrack::startTrack", filename_value);
  }

  const char* ITrack::endTrack_event = "ITrack::endTrack";
  NMM::Event* ITrack::create_endTrack()
  {

    return new NMM::Event("ITrack::endTrack");
  }



}


#include "nmm/nmm_suffix.hpp"

