/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ITrack_HPP
#define NMM_ITrack_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/utils/time/Rational.hpp"
#include "nmm/utils/time/Interval.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Track change events.
   *
   * This interface defines instream events that inform a listener about track changes.
   * Events are generated when a track or a list of tracks starts or ends.
   */

  class DLLEXPORT ITrack : virtual public NMM::Interface { 
  public:
    ITrack() : NMM::Interface(ITrack::ITrack_type, "NMM") { };

    ITrack(NMM::NMMObject& parent) : NMM::Interface (parent, ITrack::ITrack_type, "NMM") { };

    virtual ~ITrack() {};

    static const char* ITrack_type;

    virtual  string getInterfaceType() const { return ITrack_type; }

    NMM::Interface* copy() const { return new ITrack(*this); }
    static void used(); 
    static const char* startTrack_event;
    static NMM::Event* create_startTrack(const string& filename);
    static const char* endTrack_event;
    static NMM::Event* create_endTrack();
  /* instream (protected) and non-instream (public) */        /** Instream event that marks the start of a track.
     *
     * @param filename Name of the track
     * @return Result of the instream event
     */

    virtual Result startTrack(const string& filename) /* in-stream */ 
      throw(const NMM::SerializedException&);
        /** Instream event that marks the end of a track.
     *
     * @return Result of the instream event
     */

    virtual Result endTrack() /* in-stream */ 
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ITrack> ITrack_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
