/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ITrackDuration.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/serialize/SerializeUtilsTime.hpp"

namespace NMM
{
  void ITrackDuration::used() {} 
  
  
  Result ITrackDuration::trackDuration(const Interval& track_duration) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<Interval > track_duration_value(track_duration, false);
    NMM::Event event("ITrackDuration::trackDuration", &track_duration_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITrackDuration*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ITrackDuration::ITrackDuration_type = "ITrackDuration";
  NMM::TConstructor<ITrackDuration > __ITrackDuration_prototype("ITrackDuration","NMM");


  const char* ITrackDuration::trackDuration_event = "ITrackDuration::trackDuration";
  NMM::Event* ITrackDuration::create_trackDuration(const Interval& track_duration)
  {
    NMM::TInValue<Interval >* track_duration_value = new NMM::TInValue<Interval >();
    track_duration_value->setValue(track_duration);

    return new NMM::Event("ITrackDuration::trackDuration", track_duration_value);
  }



}


#include "nmm/nmm_suffix.hpp"

