/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ITrackDuration_HPP
#define NMM_ITrackDuration_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/utils/time/Interval.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** A Listener for Track Duration Events.
   *
   * This interface defines instream events that carry the duration of a track.
   * Applications or nodes that need to know the duration of every new track
   * should register a listener for this event.
   */

  class DLLEXPORT ITrackDuration : virtual public NMM::Interface { 
  public:
    ITrackDuration() : NMM::Interface(ITrackDuration::ITrackDuration_type, "NMM") { };

    ITrackDuration(NMM::NMMObject& parent) : NMM::Interface (parent, ITrackDuration::ITrackDuration_type, "NMM") { };

    virtual ~ITrackDuration() {};

    static const char* ITrackDuration_type;

    virtual  string getInterfaceType() const { return ITrackDuration_type; }

    NMM::Interface* copy() const { return new ITrackDuration(*this); }
    static void used(); 
    static const char* trackDuration_event;
    static NMM::Event* create_trackDuration(const Interval& track_duration);
  /* instream (protected) and non-instream (public) */        /** Track duration (instream event).
     *
     * This instream event is sent by a source node whenever the duration of a new
     * track is known.
     *
     * @param track_duration Duration of the current track
     * @return Result of the instream event
     */

    virtual Result trackDuration(const Interval& track_duration) /* in-stream */ 
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ITrackDuration> ITrackDuration_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
