/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ITrackDurationImpl_HPP
#define NMM_ITrackDurationImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "ITrackDuration.hpp"


#include "visibility.hpp"

namespace NMM
{
  /** A Listener for Track Duration Events.
   *
   * This interface defines instream events that carry the duration of a track.
   * Applications or nodes that need to know the duration of every new track
   * should register a listener for this event.
   */

  class DLLEXPORT ITrackDurationImpl  : public virtual NMM::NMMObject { 
  public:
    ITrackDurationImpl() { addInterface(new ITrackDuration(*this)); registerEvents(); };

    virtual ~ITrackDurationImpl() {};

  protected:
    /** Track duration (instream event).
     *
     * This instream event is sent by a source node whenever the duration of a new
     * track is known.
     *
     * @param track_duration Duration of the current track
     * @return Result of the instream event
     */

    virtual Result trackDuration(const Interval& track_duration) /* in-stream */  = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
