/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ITrackImpl_HPP
#define NMM_ITrackImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "ITrack.hpp"


#include "visibility.hpp"

namespace NMM
{
  /** Track change events.
   *
   * This interface defines instream events that inform a listener about track changes.
   * Events are generated when a track or a list of tracks starts or ends.
   */

  class DLLEXPORT ITrackImpl  : public virtual NMM::NMMObject { 
  public:
    ITrackImpl() { addInterface(new ITrack(*this)); registerEvents(); };

    virtual ~ITrackImpl() {};

  protected:
    /** Instream event that marks the start of a track.
     *
     * @param filename Name of the track
     * @return Result of the instream event
     */

    virtual Result startTrack(const string& filename) /* in-stream */  = 0;
    /** Instream event that marks the end of a track.
     *
     * @return Result of the instream event
     */

    virtual Result endTrack() /* in-stream */  = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
