/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ITrackList_HPP
#define NMM_ITrackList_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{

  class DLLEXPORT ITrackList : virtual public NMM::Interface { 
  public:
    ITrackList() : NMM::Interface(ITrackList::ITrackList_type, "NMM") { };

    ITrackList(NMM::NMMObject& parent) : NMM::Interface (parent, ITrackList::ITrackList_type, "NMM") { };

    virtual ~ITrackList() {};

    static const char* ITrackList_type;

    virtual  string getInterfaceType() const { return ITrackList_type; }

    NMM::Interface* copy() const { return new ITrackList(*this); }
    static void used(); 
    static const char* startTrackList_event;
    static NMM::Event* create_startTrackList();
    static const char* endTrackList_event;
    static NMM::Event* create_endTrackList();
  /* instream (protected) and non-instream (public) */    
    virtual Result startTrackList() /* in-stream */ 
      throw(const NMM::SerializedException&);
    
    virtual Result endTrackList() /* in-stream */ 
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ITrackList> ITrackList_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
