/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ITrackListDuration_HPP
#define NMM_ITrackListDuration_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/utils/time/Interval.hpp"

#include "visibility.hpp"

namespace NMM
{
  /**
   *  @brief TrackListDuration events
   *
   *  The trackListDuration event is sent by read nodes. It contains the duration of the
   *  audio/video track list that is being processed. A track list can be a Playlist, CD, DVD or similar.
   *  Applications that need to know the duration of the trackList 
   *  can register a listener for this event.
   */

  class DLLEXPORT ITrackListDuration : virtual public NMM::Interface { 
  public:
    ITrackListDuration() : NMM::Interface(ITrackListDuration::ITrackListDuration_type, "NMM") { };

    ITrackListDuration(NMM::NMMObject& parent) : NMM::Interface (parent, ITrackListDuration::ITrackListDuration_type, "NMM") { };

    virtual ~ITrackListDuration() {};

    static const char* ITrackListDuration_type;

    virtual  string getInterfaceType() const { return ITrackListDuration_type; }

    NMM::Interface* copy() const { return new ITrackListDuration(*this); }
    static void used(); 
    static const char* trackListDuration_event;
    static NMM::Event* create_trackListDuration(const Interval& i);
  /* instream (protected) and non-instream (public) */        /**
     *  @brief Duration of the current track list
     *  @param duration of the track list (in seconds and nanoseconds)
     *
     *  Instream event containing the duration of the current track list.
     */

    virtual Result trackListDuration(const Interval& i) /* in-stream */ 
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ITrackListDuration> ITrackListDuration_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
