/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ITrackListDurationImpl_HPP
#define NMM_ITrackListDurationImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "ITrackListDuration.hpp"


#include "visibility.hpp"

namespace NMM
{
  /**
   *  @brief TrackListDuration events
   *
   *  The trackListDuration event is sent by read nodes. It contains the duration of the
   *  audio/video track list that is being processed. A track list can be a Playlist, CD, DVD or similar.
   *  Applications that need to know the duration of the trackList 
   *  can register a listener for this event.
   */

  class DLLEXPORT ITrackListDurationImpl  : public virtual NMM::NMMObject { 
  public:
    ITrackListDurationImpl() { addInterface(new ITrackListDuration(*this)); registerEvents(); };

    virtual ~ITrackListDurationImpl() {};

  protected:
    /**
     *  @brief Duration of the current track list
     *  @param duration of the track list (in seconds and nanoseconds)
     *
     *  Instream event containing the duration of the current track list.
     */

    virtual Result trackListDuration(const Interval& i) /* in-stream */  = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
