/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#ifndef _NMM_DIFF_TIMER_HPP_
#define _NMM_DIFF_TIMER_HPP_

#include "nmm/nmm_prefix.hpp"

#include "nmm/utils/NamedObject.hpp"
#include "nmm/utils/time/TimedElement.hpp"

namespace NMM {

  /** Helper class for measuring a time interval.
   */
  class DiffTimer : public TimedElement,
                    public NamedObject {

  private:
    Time start_time;    ///< System time when measurement was started

  public:
    /** Default constructor.
     */
    DiffTimer();

    /** Virtual destructor.
     */
    virtual ~DiffTimer();

    /** Reset the timer.
     * This method stores the current system time as the start time of this timer.
     */
    void reset();

    /** Get the system time difference.
     * This method returns the difference between the current system time and the start time.
     *
     * @return System time difference in seconds
     */
    float getDiff();
  };
}

#include "nmm/nmm_suffix.hpp"

#endif // _NMM_DIFF_TIMER_HPP_
