/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#ifndef _NMM_GENERIC_UNBUFFERED_SYNC_SINK_NODE_HPP_
#define _NMM_GENERIC_UNBUFFERED_SYNC_SINK_NODE_HPP_

#include "nmm/nmm_prefix.hpp"

#include "GenericSyncSinkNode.hpp"

namespace NMM {

  //! This class is a superclass for synchronized sink nodes.
  /*!
   *  Synchronization can be switched on and off by the setSynchronized -
   *  method. By default the switch is turned off, so an application 
   *  that don't uses synchronization should not care about it.
   *  In synchronized mode, be careful that incoming buffers are 
   *  timestamped: The GenericUSyncSinkNode tries to present them according
   *  to their sync_time in the timestamps. 
   */
  class DLLEXPORT GenericUSyncSinkNode : public GenericSyncSinkNode {
  public:
    //! The main constructor.
    GenericUSyncSinkNode(const char* name,
			 const StreamQueue::Mode queue_mode 
			 = StreamQueue::DEFAULT_MODE,
			 const int size = StreamQueue::DEFAULT_SIZE);
    
    /** Check if sink is buffered (from ISynchronizedSink) */
    virtual nmm_bool isBuffered() const {
      return false;
    }
    
    //! The main destructor.
    virtual ~GenericUSyncSinkNode();
  };
}

#include "nmm/nmm_suffix.hpp"

#endif // _NMM_GENERIC_UNBUFFERED_SYNC_SINK_NODE_HPP_


