/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IController.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/serialize/SerializeUtilsTime.hpp"

namespace NMM
{
  void IPing::used() {} 
  
  
  Time IPing::ping()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Time > return_value;
    NMM::Event event("IPing::ping");
    event.setReturnValue(&return_value);
    if(const_cast<IPing*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IPing::IPing_type = "IPing";
  NMM::TConstructor<IPing > __IPing_prototype("IPing","NMM");


  const char* IPing::ping_event = "IPing::ping";
  void IController::used() {} 
  
  
  Result IController::setMode(const IController::Mode& mode)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<IController::Mode > mode_value(mode, false);
    NMM::Event event("IController::setMode", &mode_value);
    event.setReturnValue(&return_value);
    if(const_cast<IController*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IController::permitDiscardBuffers(const nmm_bool& enable)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_bool > enable_value(enable, false);
    NMM::Event event("IController::permitDiscardBuffers", &enable_value);
    event.setReturnValue(&return_value);
    if(const_cast<IController*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IController::pause()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("IController::pause");
    event.setReturnValue(&return_value);
    if(const_cast<IController*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IController::wakeup()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("IController::wakeup");
    event.setReturnValue(&return_value);
    if(const_cast<IController*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IController::reset()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("IController::reset");
    event.setReturnValue(&return_value);
    if(const_cast<IController*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IController::setBuffersInvalid(const nmm_bool& enable)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > enable_value(enable, false);
    NMM::Event event("IController::setBuffersInvalid", &enable_value);
    if(const_cast<IController*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IController::setBuffersInvalidTimed(const nmm_bool& enable, const Timestamp& timestamp)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > enable_value(enable, false);
    NMM::TInValue<Timestamp > timestamp_value(timestamp, false);
    NMM::Event event("IController::setBuffersInvalidTimed", &enable_value, &timestamp_value);
    if(const_cast<IController*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IController::IController_type = "IController";
  NMM::TConstructor<IController > __IController_prototype("IController","NMM");
  TValueConstructor< IController::Mode > __IController_Mode_prototype(TypeInfo("IController::Mode", "NMM"));

  NMM::OValueStream& operator<<(NMM::OValueStream& ov, const IController::Mode& t)
  {
    ov.beginType(TypeInfo("IController::Mode", "NMM"));
    nmm_int32 temp = static_cast<nmm_int32>(t);
    ov << temp;
    ov.endType();
    return ov;
  }

  NMM::IValueStream& operator>>(NMM::IValueStream& iv, IController::Mode& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("IController::Mode", "NMM")) {
      throw NMM::InvalidTypeException(TypeInfo("IController::Mode", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    nmm_int32 temp;
    iv >> temp;
    t = static_cast<IController::Mode>(temp);
    iv.goUp();
    return iv;
  }



  const char* IController::setMode_event = "IController::setMode";
  const char* IController::permitDiscardBuffers_event = "IController::permitDiscardBuffers";
  const char* IController::pause_event = "IController::pause";
  const char* IController::wakeup_event = "IController::wakeup";
  const char* IController::reset_event = "IController::reset";
  const char* IController::setBuffersInvalid_event = "IController::setBuffersInvalid";
  const char* IController::setBuffersInvalidTimed_event = "IController::setBuffersInvalidTimed";


}


#include "nmm/nmm_suffix.hpp"

