/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IMasterSlaveSourceSynchronizer.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/utils/time/Interval.hpp"
#include "nmm/comm/serialize/SerializeUtilsTime.hpp"

namespace NMM
{
  void IMasterSlaveSourceSynchronizer::used() {} 
  
  
  Result IMasterSlaveSourceSynchronizer::setStartDelay(const Interval& interval)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<Interval > interval_value(interval, false);
    NMM::Event event("IMasterSlaveSourceSynchronizer::setStartDelay", &interval_value);
    event.setReturnValue(&return_value);
    if(const_cast<IMasterSlaveSourceSynchronizer*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IMasterSlaveSourceSynchronizer::setMasterController(const ISourceController* value)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<ISourceController* > value_value (const_cast<ISourceController* >(value), false);
    NMM::Event event("IMasterSlaveSourceSynchronizer::setMasterController", &value_value);
    event.setReturnValue(&return_value);
    if(const_cast<IMasterSlaveSourceSynchronizer*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IMasterSlaveSourceSynchronizer::addSlaveController(const ISourceController* value)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<ISourceController* > value_value (const_cast<ISourceController* >(value), false);
    NMM::Event event("IMasterSlaveSourceSynchronizer::addSlaveController", &value_value);
    event.setReturnValue(&return_value);
    if(const_cast<IMasterSlaveSourceSynchronizer*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IMasterSlaveSourceSynchronizer::removeSlaveController(const ISourceController* value)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<ISourceController* > value_value (const_cast<ISourceController* >(value), false);
    NMM::Event event("IMasterSlaveSourceSynchronizer::removeSlaveController", &value_value);
    event.setReturnValue(&return_value);
    if(const_cast<IMasterSlaveSourceSynchronizer*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IMasterSlaveSourceSynchronizer::resetAll()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("IMasterSlaveSourceSynchronizer::resetAll");
    event.setReturnValue(&return_value);
    if(const_cast<IMasterSlaveSourceSynchronizer*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IMasterSlaveSourceSynchronizer::IMasterSlaveSourceSynchronizer_type = "IMasterSlaveSourceSynchronizer";
  NMM::TConstructor<IMasterSlaveSourceSynchronizer > __IMasterSlaveSourceSynchronizer_prototype("IMasterSlaveSourceSynchronizer","NMM");


  const char* IMasterSlaveSourceSynchronizer::setStartDelay_event = "IMasterSlaveSourceSynchronizer::setStartDelay";
  const char* IMasterSlaveSourceSynchronizer::setMasterController_event = "IMasterSlaveSourceSynchronizer::setMasterController";
  const char* IMasterSlaveSourceSynchronizer::addSlaveController_event = "IMasterSlaveSourceSynchronizer::addSlaveController";
  const char* IMasterSlaveSourceSynchronizer::removeSlaveController_event = "IMasterSlaveSourceSynchronizer::removeSlaveController";
  const char* IMasterSlaveSourceSynchronizer::resetAll_event = "IMasterSlaveSourceSynchronizer::resetAll";


}


#include "nmm/nmm_suffix.hpp"

