/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IMultiAudioSinkSynchronizer.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IMultiAudioSinkSynchronizer::used() {} 
  
  
  nmm_int32 IMultiAudioSinkSynchronizer::addAudioSinkController(const ISinkController* controller)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::TInValue<ISinkController* > controller_value (const_cast<ISinkController* >(controller), false);
    NMM::Event event("IMultiAudioSinkSynchronizer::addAudioSinkController", &controller_value);
    event.setReturnValue(&return_value);
    if(const_cast<IMultiAudioSinkSynchronizer*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IMultiAudioSinkSynchronizer::IMultiAudioSinkSynchronizer_type = "IMultiAudioSinkSynchronizer";
  NMM::TConstructor<IMultiAudioSinkSynchronizer > __IMultiAudioSinkSynchronizer_prototype("IMultiAudioSinkSynchronizer","NMM");


  const char* IMultiAudioSinkSynchronizer::addAudioSinkController_event = "IMultiAudioSinkSynchronizer::addAudioSinkController";


}


#include "nmm/nmm_suffix.hpp"

