/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IMultiAudioVideoSynchronizer.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/utils/time/Interval.hpp"
#include "nmm/comm/serialize/SerializeUtilsTime.hpp"

namespace NMM
{
  void IMultiAudioVideoSynchronizer::used() {} 
  
  
  Result IMultiAudioVideoSynchronizer::setMasterAudioSink(const nmm_int32& id)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > id_value(id, false);
    NMM::Event event("IMultiAudioVideoSynchronizer::setMasterAudioSink", &id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IMultiAudioVideoSynchronizer*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IMultiAudioVideoSynchronizer::setAudioSkew(const nmm_int32& id, const Interval& skew)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > id_value(id, false);
    NMM::TInValue<Interval > skew_value(skew, false);
    NMM::Event event("IMultiAudioVideoSynchronizer::setAudioSkew", &id_value, &skew_value);
    event.setReturnValue(&return_value);
    if(const_cast<IMultiAudioVideoSynchronizer*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IMultiAudioVideoSynchronizer::setVideoSkew(const nmm_int32& id, const Interval& skew)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > id_value(id, false);
    NMM::TInValue<Interval > skew_value(skew, false);
    NMM::Event event("IMultiAudioVideoSynchronizer::setVideoSkew", &id_value, &skew_value);
    event.setReturnValue(&return_value);
    if(const_cast<IMultiAudioVideoSynchronizer*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IMultiAudioVideoSynchronizer::IMultiAudioVideoSynchronizer_type = "IMultiAudioVideoSynchronizer";
  NMM::TConstructor<IMultiAudioVideoSynchronizer > __IMultiAudioVideoSynchronizer_prototype("IMultiAudioVideoSynchronizer","NMM");


  const char* IMultiAudioVideoSynchronizer::setMasterAudioSink_event = "IMultiAudioVideoSynchronizer::setMasterAudioSink";
  const char* IMultiAudioVideoSynchronizer::setAudioSkew_event = "IMultiAudioVideoSynchronizer::setAudioSkew";
  const char* IMultiAudioVideoSynchronizer::setVideoSkew_event = "IMultiAudioVideoSynchronizer::setVideoSkew";


}


#include "nmm/nmm_suffix.hpp"

