/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IMultiAudioVideoSynchronizerImpl_HPP
#define NMM_IMultiAudioVideoSynchronizerImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IMultiAudioVideoSynchronizer.hpp"

#include "IMultiAudioSinkSynchronizerImpl.hpp"
#include "IMultiVideoSinkSynchronizerImpl.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Interface of a synchronizer for multiple audio and video sinks.
   */

  class DLLEXPORT IMultiAudioVideoSynchronizerImpl  : virtual public ISinkSynchronizerImpl, virtual public IMultiAudioSinkSynchronizerImpl, virtual public IMultiVideoSinkSynchronizerImpl { 
  public:
    IMultiAudioVideoSynchronizerImpl() { addInterface(new IMultiAudioVideoSynchronizer(*this)); registerEvents(); };

    virtual ~IMultiAudioVideoSynchronizerImpl() {};

  protected:
    /** Set the master audio sink.
     * This method sets an audio sink as master audio sink.
     * If this method is not called, the first audio sink is the master by default.
     *
     * @param id audio sink id
     * @return SUCCESS
     */

    virtual Result setMasterAudioSink(const nmm_int32& id) = 0;
    /** Set skew of an audio sink.
     * This method sets the skew of an audio sink relative to the master audio sink.
     *
     * @param id Sink id
     * @param skew Skew of this sink relative to master
     * @return SUCCESS
     */

    virtual Result setAudioSkew(const nmm_int32& id, const Interval& skew) = 0;
    /** Set skew of a video sink.
     * This method sets the skew of a video sink relative to the master audio sink.
     *
     * @param id Sink id
     * @param skew Skew of this sink relative to master
     * @return SUCCESS
     */

    virtual Result setVideoSkew(const nmm_int32& id, const Interval& skew) = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
