/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IMultiVideoSinkSynchronizer_HPP
#define NMM_IMultiVideoSinkSynchronizer_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "ISinkSynchronizer.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Interface of a synchronizer for multiple video sinks.
   */

  class DLLEXPORT IMultiVideoSinkSynchronizer : virtual public NMM::Interface { 
  public:
    IMultiVideoSinkSynchronizer() : NMM::Interface(IMultiVideoSinkSynchronizer::IMultiVideoSinkSynchronizer_type, "NMM") { };

    IMultiVideoSinkSynchronizer(NMM::NMMObject& parent) : NMM::Interface (parent, IMultiVideoSinkSynchronizer::IMultiVideoSinkSynchronizer_type, "NMM") { };

    virtual ~IMultiVideoSinkSynchronizer() {};

    static const char* IMultiVideoSinkSynchronizer_type;

    virtual  string getInterfaceType() const { return IMultiVideoSinkSynchronizer_type; }

    NMM::Interface* copy() const { return new IMultiVideoSinkSynchronizer(*this); }
    static void used(); 
    static const char* addVideoSinkController_event;
    /** Add a video sink controller.
     * This method adds a the controller for a video sink to the synchronizer.
     *
     * @param controller A video sink controller
     * @return Sink ID of the video controller
     */

    virtual nmm_int32 addVideoSinkController(const ISinkController* controller)
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IMultiVideoSinkSynchronizer> IMultiVideoSinkSynchronizer_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
