/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ISinkController.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "ISinkSynchronizer.hpp"
#include "nmm/utils/time/Interval.hpp"
#include "nmm/comm/serialize/SerializeUtilsTime.hpp"

namespace NMM
{
  void ISinkController::used() {} 
  
  
  Result ISinkController::setMaxDelay(const Interval& max_delay)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<Interval > max_delay_value(max_delay, false);
    NMM::Event event("ISinkController::setMaxDelay", &max_delay_value);
    event.setReturnValue(&return_value);
    if(const_cast<ISinkController*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISinkController::setMaxAverageDelay(const Interval& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<Interval > value_value(value, false);
    NMM::Event event("ISinkController::setMaxAverageDelay", &value_value);
    event.setReturnValue(&return_value);
    if(const_cast<ISinkController*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISinkController::setTheoLatency(const Interval& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<Interval > value_value(value, false);
    NMM::Event event("ISinkController::setTheoLatency", &value_value);
    event.setReturnValue(&return_value);
    if(const_cast<ISinkController*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISinkController::setDefaultTheoLatency(const Interval& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<Interval > value_value(value, false);
    NMM::Event event("ISinkController::setDefaultTheoLatency", &value_value);
    event.setReturnValue(&return_value);
    if(const_cast<ISinkController*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISinkController::setSinkSynchronizer(const ISinkSynchronizer* sync, const nmm_int32& sink_id)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<ISinkSynchronizer* > sync_value (const_cast<ISinkSynchronizer* >(sync), false);
    NMM::TInValue<nmm_int32 > sink_id_value(sink_id, false);
    NMM::Event event("ISinkController::setSinkSynchronizer", &sync_value, &sink_id_value);
    event.setReturnValue(&return_value);
    if(const_cast<ISinkController*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISinkController::setAverageCount(const nmm_int32& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > value_value(value, false);
    NMM::Event event("ISinkController::setAverageCount", &value_value);
    event.setReturnValue(&return_value);
    if(const_cast<ISinkController*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISinkController::setMaxDiscardingCount(const nmm_int32& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > value_value(value, false);
    NMM::Event event("ISinkController::setMaxDiscardingCount", &value_value);
    event.setReturnValue(&return_value);
    if(const_cast<ISinkController*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISinkController::waitForReset()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("ISinkController::waitForReset");
    event.setReturnValue(&return_value);
    if(const_cast<ISinkController*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISinkController::restart()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("ISinkController::restart");
    event.setReturnValue(&return_value);
    if(const_cast<ISinkController*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISinkController::enableWaitToPresent(const nmm_bool& enable)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_bool > enable_value(enable, false);
    NMM::Event event("ISinkController::enableWaitToPresent", &enable_value);
    event.setReturnValue(&return_value);
    if(const_cast<ISinkController*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISinkController::setMinWaitInterval(const Interval& min_wait)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<Interval > min_wait_value(min_wait, false);
    NMM::Event event("ISinkController::setMinWaitInterval", &min_wait_value);
    event.setReturnValue(&return_value);
    if(const_cast<ISinkController*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ISinkController::enableFirstBufferInTime(const nmm_bool& enable)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > enable_value(enable, false);
    NMM::Event event("ISinkController::enableFirstBufferInTime", &enable_value);
    if(const_cast<ISinkController*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ISinkController::enableBlocking(const nmm_bool& enable)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > enable_value(enable, false);
    NMM::Event event("ISinkController::enableBlocking", &enable_value);
    if(const_cast<ISinkController*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* ISinkController::ISinkController_type = "ISinkController";
  NMM::TConstructor<ISinkController > __ISinkController_prototype("ISinkController","NMM");


  const char* ISinkController::setMaxDelay_event = "ISinkController::setMaxDelay";
  const char* ISinkController::setMaxAverageDelay_event = "ISinkController::setMaxAverageDelay";
  const char* ISinkController::setTheoLatency_event = "ISinkController::setTheoLatency";
  const char* ISinkController::setDefaultTheoLatency_event = "ISinkController::setDefaultTheoLatency";
  const char* ISinkController::setSinkSynchronizer_event = "ISinkController::setSinkSynchronizer";
  const char* ISinkController::setAverageCount_event = "ISinkController::setAverageCount";
  const char* ISinkController::setMaxDiscardingCount_event = "ISinkController::setMaxDiscardingCount";
  const char* ISinkController::waitForReset_event = "ISinkController::waitForReset";
  const char* ISinkController::restart_event = "ISinkController::restart";
  const char* ISinkController::enableWaitToPresent_event = "ISinkController::enableWaitToPresent";
  const char* ISinkController::setMinWaitInterval_event = "ISinkController::setMinWaitInterval";
  const char* ISinkController::enableFirstBufferInTime_event = "ISinkController::enableFirstBufferInTime";
  const char* ISinkController::enableBlocking_event = "ISinkController::enableBlocking";


}


#include "nmm/nmm_suffix.hpp"

