/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ISinkSynchronizer.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/utils/time/Interval.hpp"
#include "nmm/comm/serialize/SerializeUtilsTime.hpp"

namespace NMM
{
  void ISinkSynchronizer::used() {} 
  
  
  void ISinkSynchronizer::setAverageLatency(const Interval& value, const nmm_int32& sink)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<Interval > value_value(value, false);
    NMM::TInValue<nmm_int32 > sink_value(sink, false);
    NMM::Event event("ISinkSynchronizer::setAverageLatency", &value_value, &sink_value);
    if(const_cast<ISinkSynchronizer*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  Result ISinkSynchronizer::setFirstBufferInTimeAverageLatency(const Interval& average_latency, const nmm_int32& sink, const Timestamp& first_ts)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<Interval > average_latency_value(average_latency, false);
    NMM::TInValue<nmm_int32 > sink_value(sink, false);
    NMM::TInValue<Timestamp > first_ts_value(first_ts, false);
    NMM::Event event("ISinkSynchronizer::setFirstBufferInTimeAverageLatency", &average_latency_value, &sink_value, &first_ts_value);
    event.setReturnValue(&return_value);
    if(const_cast<ISinkSynchronizer*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISinkSynchronizer::removeSinkController(const ISinkController* controller)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<ISinkController* > controller_value (const_cast<ISinkController* >(controller), false);
    NMM::Event event("ISinkSynchronizer::removeSinkController", &controller_value);
    event.setReturnValue(&return_value);
    if(const_cast<ISinkSynchronizer*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ISinkSynchronizer::ISinkSynchronizer_type = "ISinkSynchronizer";
  NMM::TConstructor<ISinkSynchronizer > __ISinkSynchronizer_prototype("ISinkSynchronizer","NMM");


  const char* ISinkSynchronizer::setAverageLatency_event = "ISinkSynchronizer::setAverageLatency";
  const char* ISinkSynchronizer::setFirstBufferInTimeAverageLatency_event = "ISinkSynchronizer::setFirstBufferInTimeAverageLatency";
  const char* ISinkSynchronizer::removeSinkController_event = "ISinkSynchronizer::removeSinkController";


}


#include "nmm/nmm_suffix.hpp"

