/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ISinkSynchronizer_HPP
#define NMM_ISinkSynchronizer_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "ISynchronizer.hpp"
#include "ISinkController.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Interface of a sink synchronizer.
   */

  class DLLEXPORT ISinkSynchronizer : virtual public ISynchronizer { 
  public:
    ISinkSynchronizer() : NMM::Interface(ISinkSynchronizer::ISinkSynchronizer_type, "NMM") { };

    ISinkSynchronizer(NMM::NMMObject& parent) : NMM::Interface (parent, ISinkSynchronizer::ISinkSynchronizer_type, "NMM") { };

    virtual ~ISinkSynchronizer() {};

    static const char* ISinkSynchronizer_type;

    virtual  string getInterfaceType() const { return ISinkSynchronizer_type; }

    NMM::Interface* copy() const { return new ISinkSynchronizer(*this); }
    static void used(); 
    static const char* setAverageLatency_event;
    static const char* setFirstBufferInTimeAverageLatency_event;
    static const char* removeSinkController_event;
    /** Set average latency.
     * With this method, the sink controllers tell the
     * synchronizer the average latency of their incoming buffers.
     *
     * @param average_latency Average latency computed by the controller
     * @param sink Sink ID
     */

    virtual void setAverageLatency(const Interval& value, const nmm_int32& sink)
      throw(const NMM::SerializedException&);
    /** Set the average latency of the first buffer in tmie.
     * With this method, the sink controllers tell the
     * synchronizer the average latency of their first buffer in time after syncReset.
     *
     * @param average_latency The average latency of incoming buffers computed by the Controller
     * @param sink Sink ID
     * @param first_ts Timestamp of the first buffer in time
     * @return SUCCESS if value is good enough compared to the theoretical latency, FAILURE otherwise
     */

    virtual Result setFirstBufferInTimeAverageLatency(const Interval& average_latency, const nmm_int32& sink, const Timestamp& first_ts)
      throw(const NMM::SerializedException&);
    /** Remove a sink controller.
     * This method removes the sink controller with the specified interface from the synchronizer.
     *
     * @param controller Interface of sink controller to remove
     */

    virtual Result removeSinkController(const ISinkController* controller)
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ISinkSynchronizer> ISinkSynchronizer_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
