/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ISinkSynchronizerImpl_HPP
#define NMM_ISinkSynchronizerImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "ISinkSynchronizer.hpp"

#include "ISynchronizerImpl.hpp"
#include "ISinkControllerImpl.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Interface of a sink synchronizer.
   */

  class DLLEXPORT ISinkSynchronizerImpl  : virtual public ISynchronizerImpl { 
  public:
    ISinkSynchronizerImpl() { addInterface(new ISinkSynchronizer(*this)); registerEvents(); };

    virtual ~ISinkSynchronizerImpl() {};

  protected:
    /** Set average latency.
     * With this method, the sink controllers tell the
     * synchronizer the average latency of their incoming buffers.
     *
     * @param average_latency Average latency computed by the controller
     * @param sink Sink ID
     */

    virtual void setAverageLatency(const Interval& value, const nmm_int32& sink) = 0;
    /** Set the average latency of the first buffer in tmie.
     * With this method, the sink controllers tell the
     * synchronizer the average latency of their first buffer in time after syncReset.
     *
     * @param average_latency The average latency of incoming buffers computed by the Controller
     * @param sink Sink ID
     * @param first_ts Timestamp of the first buffer in time
     * @return SUCCESS if value is good enough compared to the theoretical latency, FAILURE otherwise
     */

    virtual Result setFirstBufferInTimeAverageLatency(const Interval& average_latency, const nmm_int32& sink, const Timestamp& first_ts) = 0;
    /** Remove a sink controller.
     * This method removes the sink controller with the specified interface from the synchronizer.
     *
     * @param controller Interface of sink controller to remove
     */

    virtual Result removeSinkController(const ISinkController* controller) = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
