/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ISourceController.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "ISourceSynchronizer.hpp"
#include "nmm/utils/time/Interval.hpp"
#include "nmm/comm/serialize/SerializeUtilsTime.hpp"

namespace NMM
{
  void ISourceController::used() {} 
  
  
  void ISourceController::setMasterSkew(const Interval& skew)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<Interval > skew_value(skew, false);
    NMM::Event event("ISourceController::setMasterSkew", &skew_value);
    if(const_cast<ISourceController*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ISourceController::setStartTime(const Time& time)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<Time > time_value(time, false);
    NMM::Event event("ISourceController::setStartTime", &time_value);
    if(const_cast<ISourceController*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  Result ISourceController::setSourceSynchronizer(const ISourceSynchronizer* value)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<ISourceSynchronizer* > value_value (const_cast<ISourceSynchronizer* >(value), false);
    NMM::Event event("ISourceController::setSourceSynchronizer", &value_value);
    event.setReturnValue(&return_value);
    if(const_cast<ISourceController*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ISourceController::setSyncMaster(const nmm_bool& master)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > master_value(master, false);
    NMM::Event event("ISourceController::setSyncMaster", &master_value);
    if(const_cast<ISourceController*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* ISourceController::ISourceController_type = "ISourceController";
  NMM::TConstructor<ISourceController > __ISourceController_prototype("ISourceController","NMM");


  const char* ISourceController::setMasterSkew_event = "ISourceController::setMasterSkew";
  const char* ISourceController::setStartTime_event = "ISourceController::setStartTime";
  const char* ISourceController::setSourceSynchronizer_event = "ISourceController::setSourceSynchronizer";
  const char* ISourceController::setSyncMaster_event = "ISourceController::setSyncMaster";


}


#include "nmm/nmm_suffix.hpp"

