/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ISourceSynchronizer.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/utils/time/Interval.hpp"
#include "nmm/comm/serialize/SerializeUtilsTime.hpp"

namespace NMM
{
  void ISourceSynchronizer::used() {} 
  
  
  Result ISourceSynchronizer::setMasterSkew(const Interval& skew)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<Interval > skew_value(skew, false);
    NMM::Event event("ISourceSynchronizer::setMasterSkew", &skew_value);
    event.setReturnValue(&return_value);
    if(const_cast<ISourceSynchronizer*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ISourceSynchronizer::ISourceSynchronizer_type = "ISourceSynchronizer";
  NMM::TConstructor<ISourceSynchronizer > __ISourceSynchronizer_prototype("ISourceSynchronizer","NMM");


  const char* ISourceSynchronizer::setMasterSkew_event = "ISourceSynchronizer::setMasterSkew";


}


#include "nmm/nmm_suffix.hpp"

