/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ISourceSynchronizer_HPP
#define NMM_ISourceSynchronizer_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "ISynchronizer.hpp"
#include "ISourceController.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Interface of a source synchronizer.
   */

  class DLLEXPORT ISourceSynchronizer : virtual public ISynchronizer { 
  public:
    ISourceSynchronizer() : NMM::Interface(ISourceSynchronizer::ISourceSynchronizer_type, "NMM") { };

    ISourceSynchronizer(NMM::NMMObject& parent) : NMM::Interface (parent, ISourceSynchronizer::ISourceSynchronizer_type, "NMM") { };

    virtual ~ISourceSynchronizer() {};

    static const char* ISourceSynchronizer_type;

    virtual  string getInterfaceType() const { return ISourceSynchronizer_type; }

    NMM::Interface* copy() const { return new ISourceSynchronizer(*this); }
    static void used(); 
    static const char* setMasterSkew_event;
    /** Set the master skew.
     * This method sets the skew between audio and video output.
     * The video output is presented later than the audio output by the given skew interval.
     *
     * @param skew Master skew
     * @return SUCCESS
     */

    virtual Result setMasterSkew(const Interval& skew)
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ISourceSynchronizer> ISourceSynchronizer_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
