/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ISynchronized.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ISynchronized::used() {} 
  
  
  Result ISynchronized::setSynchronized(const nmm_bool& toggle) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_bool > toggle_value(toggle, false);
    NMM::Event event("ISynchronized::setSynchronized", &toggle_value);
    event.setReturnValue(&return_value);
    if(const_cast<ISynchronized*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool ISynchronized::isSynchronized()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ISynchronized::isSynchronized");
    event.setReturnValue(&return_value);
    if(const_cast<ISynchronized*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ISynchronized::ISynchronized_type = "ISynchronized";
  NMM::TConstructor<ISynchronized > __ISynchronized_prototype("ISynchronized","NMM");


  const char* ISynchronized::setSynchronized_event = "ISynchronized::setSynchronized";
  NMM::Event* ISynchronized::create_setSynchronized(const nmm_bool& toggle)
  {
    NMM::TInValue<nmm_bool >* toggle_value = new NMM::TInValue<nmm_bool >();
    toggle_value->setValue(toggle);

    return new NMM::Event("ISynchronized::setSynchronized", toggle_value);
  }

  const char* ISynchronized::isSynchronized_event = "ISynchronized::isSynchronized";


}


#include "nmm/nmm_suffix.hpp"

