/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ISynchronized_HPP
#define NMM_ISynchronized_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "nmm/multimedia/ISyncReset.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Interface of a synchronized node.
   * This method is implemented by all nodes which can be synchronized.
   * It provides methods for enabling or disabling intra-stream synchronization for the node.
   */

  class DLLEXPORT ISynchronized : virtual public ISyncReset { 
  public:
    ISynchronized() : NMM::Interface(ISynchronized::ISynchronized_type, "NMM") { };

    ISynchronized(NMM::NMMObject& parent) : NMM::Interface (parent, ISynchronized::ISynchronized_type, "NMM") { };

    virtual ~ISynchronized() {};

    static const char* ISynchronized_type;

    virtual  string getInterfaceType() const { return ISynchronized_type; }

    NMM::Interface* copy() const { return new ISynchronized(*this); }
    static void used(); 
    static const char* setSynchronized_event;
    static NMM::Event* create_setSynchronized(const nmm_bool& toggle);
    static const char* isSynchronized_event;
  /* instream (protected) and non-instream (public) */        /** Enable or disable intra-stream synchronization.
     *
     * @return @c true to enable, @c false to disable intra-stream synchronization
     * @return SUCCESS
     */

    virtual Result setSynchronized(const nmm_bool& toggle) /* in-stream */ 
      throw(const NMM::SerializedException&);
  /* non-instream (public) */        /** Check if intra-stream synchronization is enabled.
     *
     * @return @c true if intra-stream synchronization is enabled, @c false otherwise
     */

    virtual nmm_bool isSynchronized()
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ISynchronized> ISynchronized_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
