/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ISynchronizedImpl_HPP
#define NMM_ISynchronizedImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "ISynchronized.hpp"

#include "nmm/multimedia/ISyncResetImpl.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Interface of a synchronized node.
   * This method is implemented by all nodes which can be synchronized.
   * It provides methods for enabling or disabling intra-stream synchronization for the node.
   */

  class DLLEXPORT ISynchronizedImpl  : virtual public ISyncResetImpl { 
  public:
    ISynchronizedImpl() { addInterface(new ISynchronized(*this)); registerEvents(); };

    virtual ~ISynchronizedImpl() {};

  protected:
    /** Enable or disable intra-stream synchronization.
     *
     * @return @c true to enable, @c false to disable intra-stream synchronization
     * @return SUCCESS
     */

    virtual Result setSynchronized(const nmm_bool& toggle) /* in-stream */  = 0;
    /** Check if intra-stream synchronization is enabled.
     *
     * @return @c true if intra-stream synchronization is enabled, @c false otherwise
     */

    virtual nmm_bool isSynchronized() = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
