/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ISynchronizedSink.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ISynchronizedSink::used() {} 
  
  
  ISinkController* ISynchronizedSink::getController()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ISinkController* > return_value;
    NMM::Event event("ISynchronizedSink::getController");
    event.setReturnValue(&return_value);
    if(const_cast<ISynchronizedSink*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool ISynchronizedSink::isBuffered() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ISynchronizedSink::isBuffered");
    event.setReturnValue(&return_value);
    if(const_cast<ISynchronizedSink*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ISynchronizedSink::ISynchronizedSink_type = "ISynchronizedSink";
  NMM::TConstructor<ISynchronizedSink > __ISynchronizedSink_prototype("ISynchronizedSink","NMM");


  const char* ISynchronizedSink::getController_event = "ISynchronizedSink::getController";
  const char* ISynchronizedSink::isBuffered_event = "ISynchronizedSink::isBuffered";


}


#include "nmm/nmm_suffix.hpp"

