/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ISynchronizedSink_HPP
#define NMM_ISynchronizedSink_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "ISynchronized.hpp"
#include "ISinkController.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Interface of a synchronized sink node.
   * This interface is implemented by sink nodes that can be synchronized.
   */

  class DLLEXPORT ISynchronizedSink : virtual public ISynchronized { 
  public:
    ISynchronizedSink() : NMM::Interface(ISynchronizedSink::ISynchronizedSink_type, "NMM") { };

    ISynchronizedSink(NMM::NMMObject& parent) : NMM::Interface (parent, ISynchronizedSink::ISynchronizedSink_type, "NMM") { };

    virtual ~ISynchronizedSink() {};

    static const char* ISynchronizedSink_type;

    virtual  string getInterfaceType() const { return ISynchronizedSink_type; }

    NMM::Interface* copy() const { return new ISynchronizedSink(*this); }
    static void used(); 
    static const char* getController_event;
    static const char* isBuffered_event;
    /** Get the sink controller of this sink node.
     *
     * @return Sink controller interface of sink controller
     */

    virtual ISinkController* getController()
      throw(const NMM::SerializedException&);
    /** Check if this sink is a buffered sink.
     * Typically, audio sinks are buffered sinks, and video sinks are unbuffered sinks.
     *
     * @return @c true if the sink is buffered, @c false if the sink is unbuffered
     */

    virtual nmm_bool isBuffered() const
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ISynchronizedSink> ISynchronizedSink_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
