/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ISynchronizedSinkImpl_HPP
#define NMM_ISynchronizedSinkImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "ISynchronizedSink.hpp"

#include "ISynchronizedImpl.hpp"
#include "ISinkControllerImpl.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Interface of a synchronized sink node.
   * This interface is implemented by sink nodes that can be synchronized.
   */

  class DLLEXPORT ISynchronizedSinkImpl  : virtual public ISynchronizedImpl { 
  public:
    ISynchronizedSinkImpl() { addInterface(new ISynchronizedSink(*this)); registerEvents(); };

    virtual ~ISynchronizedSinkImpl() {};

  protected:
    /** Get the sink controller of this sink node.
     *
     * @return Sink controller interface of sink controller
     */

    virtual ISinkController* getController() = 0;
    /** Check if this sink is a buffered sink.
     * Typically, audio sinks are buffered sinks, and video sinks are unbuffered sinks.
     *
     * @return @c true if the sink is buffered, @c false if the sink is unbuffered
     */

    virtual nmm_bool isBuffered() const = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
