/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ISynchronizedSource.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ISynchronizedSource::used() {} 
  
  
  ISourceController* ISynchronizedSource::getController()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ISourceController* > return_value;
    NMM::Event event("ISynchronizedSource::getController");
    event.setReturnValue(&return_value);
    if(const_cast<ISynchronizedSource*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ISynchronizedSource::setSyncMaster(const nmm_bool& master)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > master_value(master, false);
    NMM::Event event("ISynchronizedSource::setSyncMaster", &master_value);
    if(const_cast<ISynchronizedSource*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_bool ISynchronizedSource::isSyncMaster()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ISynchronizedSource::isSyncMaster");
    event.setReturnValue(&return_value);
    if(const_cast<ISynchronizedSource*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ISynchronizedSource::setConstantRateProducer(const nmm_bool& const_rate)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > const_rate_value(const_rate, false);
    NMM::Event event("ISynchronizedSource::setConstantRateProducer", &const_rate_value);
    if(const_cast<ISynchronizedSource*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_bool ISynchronizedSource::isConstantRateProducer()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ISynchronizedSource::isConstantRateProducer");
    event.setReturnValue(&return_value);
    if(const_cast<ISynchronizedSource*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ISynchronizedSource::ISynchronizedSource_type = "ISynchronizedSource";
  NMM::TConstructor<ISynchronizedSource > __ISynchronizedSource_prototype("ISynchronizedSource","NMM");


  const char* ISynchronizedSource::getController_event = "ISynchronizedSource::getController";
  const char* ISynchronizedSource::setSyncMaster_event = "ISynchronizedSource::setSyncMaster";
  const char* ISynchronizedSource::isSyncMaster_event = "ISynchronizedSource::isSyncMaster";
  const char* ISynchronizedSource::setConstantRateProducer_event = "ISynchronizedSource::setConstantRateProducer";
  const char* ISynchronizedSource::isConstantRateProducer_event = "ISynchronizedSource::isConstantRateProducer";


}


#include "nmm/nmm_suffix.hpp"

