/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ISynchronizedSourceImpl_HPP
#define NMM_ISynchronizedSourceImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "ISynchronizedSource.hpp"

#include "ISynchronizedImpl.hpp"
#include "ISourceControllerImpl.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Interface of a synchronized source node.
   * This interface is implemented by source nodes that can be synchronized.
   */

  class DLLEXPORT ISynchronizedSourceImpl  : virtual public ISynchronizedImpl { 
  public:
    ISynchronizedSourceImpl() { addInterface(new ISynchronizedSource(*this)); registerEvents(); };

    virtual ~ISynchronizedSourceImpl() {};

  protected:
    /** Get the source controller of this sink node.
     *
     * @return Source controller interface of source controller
     */

    virtual ISourceController* getController() = 0;
    /** Set whether this node is the source synchronization master.
     *
     * @param master @c true if the node is the source synchronization master, @c false otherwise
     */

    virtual void setSyncMaster(const nmm_bool& master) = 0;
    /** Check if this node is the synchronization master.
     *
     * @return @c true if the node is the source synchronization master, @c false otherwise
     */

    virtual nmm_bool isSyncMaster() = 0;
    /** Set if the node is a constant rate producer.
     * With this method one can set whether the underlying node produces its data
     * at a constant rate or not.
     *
     * @param const_rate @c true if the node is a constant rate producer, @c false otherwise
     */

    virtual void setConstantRateProducer(const nmm_bool& const_rate) = 0;
    /** Check if the node is a constant rate producer.
     * With this method one can check whether the underlying node produces its data
     * at a constant rate or not.
     *
     * @return @c true if the node is a constant rate producer, @c false otherwise
     */

    virtual nmm_bool isConstantRateProducer() = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
