/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ISynchronizer_HPP
#define NMM_ISynchronizer_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{
  /** Interface of a synchronizer.
   * A synchronizer is an object which synchronizes a number of sources and sinks.
   * This is the common interface for all types of synchronizers.
   */

  class DLLEXPORT ISynchronizer : virtual public NMM::Interface { 
  public:
    ISynchronizer() : NMM::Interface(ISynchronizer::ISynchronizer_type, "NMM") { };

    ISynchronizer(NMM::NMMObject& parent) : NMM::Interface (parent, ISynchronizer::ISynchronizer_type, "NMM") { };

    virtual ~ISynchronizer() {};

    static const char* ISynchronizer_type;

    virtual  string getInterfaceType() const { return ISynchronizer_type; }

    NMM::Interface* copy() const { return new ISynchronizer(*this); }
    static void used(); 
    static const char* pause_event;
    static const char* wakeup_event;
    static const char* reset_event;
    /** Pause the synchronizer.
     * This method pauses all sinks synchronized by the synchronizer.
     *
     * @return @c SUCCESS if the operation succeeded, @c FAILURE otherwise
     */

    virtual Result pause()
      throw(const NMM::SerializedException&);
    /** Wake up the synchronizer after pause.
     * This method wakes up all sinks synchronized by the synchronizer after pause.
     *
     * @return @c SUCCESS if the operation succeeded, @c FAILURE otherwise
     */

    virtual Result wakeup()
      throw(const NMM::SerializedException&);
    /** Reset the synchronizer.
     * This method resets all sinks synchronized by the synchronizer.
     * It is called by a sink which received a syncReset event. Only all other sinks are reset
     * explicitly by the synchronizer.
     *
     * @param sink_id Sink ID of sink which received the initial syncReset
     */

    virtual Result reset(const nmm_int32& sink_id)
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ISynchronizer> ISynchronizer_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
