/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ISynchronizerImpl_HPP
#define NMM_ISynchronizerImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "ISynchronizer.hpp"


#include "visibility.hpp"

namespace NMM
{
  /** Interface of a synchronizer.
   * A synchronizer is an object which synchronizes a number of sources and sinks.
   * This is the common interface for all types of synchronizers.
   */

  class DLLEXPORT ISynchronizerImpl  : public virtual NMM::NMMObject { 
  public:
    ISynchronizerImpl() { addInterface(new ISynchronizer(*this)); registerEvents(); };

    virtual ~ISynchronizerImpl() {};

  protected:
    /** Pause the synchronizer.
     * This method pauses all sinks synchronized by the synchronizer.
     *
     * @return @c SUCCESS if the operation succeeded, @c FAILURE otherwise
     */

    virtual Result pause() = 0;
    /** Wake up the synchronizer after pause.
     * This method wakes up all sinks synchronized by the synchronizer after pause.
     *
     * @return @c SUCCESS if the operation succeeded, @c FAILURE otherwise
     */

    virtual Result wakeup() = 0;
    /** Reset the synchronizer.
     * This method resets all sinks synchronized by the synchronizer.
     * It is called by a sink which received a syncReset event. Only all other sinks are reset
     * explicitly by the synchronizer.
     *
     * @param sink_id Sink ID of sink which received the initial syncReset
     */

    virtual Result reset(const nmm_int32& sink_id) = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
