/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#include "nmm/multimedia/TDevicePlugin.hpp"

#include "nmm/plugins/audio/interfaces/IAudioDevice.hpp"
#include "nmm/comm/ProxyObject.hpp"

namespace NMM {
  
  class ALSAPlaybackNode;
  
  /** Special plugin class for ALSAPlaybackNode.
   */
  template<class T>
  class ALSAPlaybackNodePlugin
    : public TDevicePlugin<T>
  {
  public:
    ALSAPlaybackNodePlugin(const string& node_name, int max_instance = -1);
    
  protected:
    virtual void checkForStandardAuxDevice(IDevice& device) throw();
    virtual void checkForAuxDevice(IDevice& device, unsigned device_num) throw();
  };  
  
  
  template<class T>
  ALSAPlaybackNodePlugin<T>::ALSAPlaybackNodePlugin (const string& node_name, int max_instance)
    : TDevicePlugin<T>("/dev/dsp", true, node_name, max_instance)
  {}
  
  template<class T>
  void ALSAPlaybackNodePlugin<T>::checkForStandardAuxDevice(IDevice& device) throw() {
    /* we get the IAudio device to specify the node capabilities on the system */
    IAudioDevice_var i_audio(device.getParentObject()->template getCheckedInterface<IAudioDevice>());
    
    /* then we try to setup the mixer device. This device must not available for
       a soundcard, but if we wish to use an availble mixer we must setup the device. */
    try {
      i_audio->setMixerDevice("/dev/mixer");
    } catch (const DeviceNotFoundException& de ) {
      GLOBAL_WARNING_STREAM(de << endl);
    }
  }
  
  template<class T>
  void ALSAPlaybackNodePlugin<T>::checkForAuxDevice(IDevice& device, unsigned device_num) throw() {
    /* we get the IAudio device to specify the node capabilities on the system */
    IAudioDevice_var i_audio(device.getParentObject()->template getCheckedInterface<IAudioDevice>());
    
    string next_mixer_device = string("/dev/mixer") + device_num;
    try {
      i_audio->setMixerDevice(next_mixer_device);
    } catch (const DeviceNotFoundException& de ) {
      GLOBAL_WARNING_STREAM(de << endl);
    }
  }
  
}
