/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IALSADevice.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IALSADevice::used() {} 
  
  
  void IALSADevice::setPCMDevice(const string& device)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > device_value(device, false);
    NMM::Event event("IALSADevice::setPCMDevice", &device_value);
    if(const_cast<IALSADevice*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  string IALSADevice::getPCMDevice() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("IALSADevice::getPCMDevice");
    event.setReturnValue(&return_value);
    if(const_cast<IALSADevice*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IALSADevice::setBufferTime(const nmm_uint32& buffertime)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > buffertime_value(buffertime, false);
    NMM::Event event("IALSADevice::setBufferTime", &buffertime_value);
    if(const_cast<IALSADevice*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IALSADevice::setPeriodTime(const nmm_uint32& periodtime)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > periodtime_value(periodtime, false);
    NMM::Event event("IALSADevice::setPeriodTime", &periodtime_value);
    if(const_cast<IALSADevice*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IALSADevice::setXferAlign(const nmm_uint32& xferalign)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > xferalign_value(xferalign, false);
    NMM::Event event("IALSADevice::setXferAlign", &xferalign_value);
    if(const_cast<IALSADevice*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IALSADevice::setMicroInVolume(const nmm_int32& vol)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > vol_value(vol, false);
    NMM::Event event("IALSADevice::setMicroInVolume", &vol_value);
    if(const_cast<IALSADevice*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_int32 IALSADevice::getMicroInVolume()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("IALSADevice::getMicroInVolume");
    event.setReturnValue(&return_value);
    if(const_cast<IALSADevice*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IALSADevice::IALSADevice_type = "IALSADevice";
  NMM::TConstructor<IALSADevice > __IALSADevice_prototype("IALSADevice","NMM");


  const char* IALSADevice::setPCMDevice_event = "IALSADevice::setPCMDevice";
  const char* IALSADevice::getPCMDevice_event = "IALSADevice::getPCMDevice";
  const char* IALSADevice::setBufferTime_event = "IALSADevice::setBufferTime";
  const char* IALSADevice::setPeriodTime_event = "IALSADevice::setPeriodTime";
  const char* IALSADevice::setXferAlign_event = "IALSADevice::setXferAlign";
  const char* IALSADevice::setMicroInVolume_event = "IALSADevice::setMicroInVolume";
  const char* IALSADevice::getMicroInVolume_event = "IALSADevice::getMicroInVolume";


}


#include "nmm/nmm_suffix.hpp"

