/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IALSADevice_HPP
#define NMM_IALSADevice_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "nmm/multimedia/INode.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Interface for configuring alsa devices.
   */

  class DLLEXPORT IALSADevice : virtual public NMM::Interface { 
  public:
    IALSADevice() : NMM::Interface(IALSADevice::IALSADevice_type, "NMM") { };

    IALSADevice(NMM::NMMObject& parent) : NMM::Interface (parent, IALSADevice::IALSADevice_type, "NMM") { };

    virtual ~IALSADevice() {};

    static const char* IALSADevice_type;

    virtual  string getInterfaceType() const { return IALSADevice_type; }

    NMM::Interface* copy() const { return new IALSADevice(*this); }
    static void used(); 
    static const char* setPCMDevice_event;
    static const char* getPCMDevice_event;
    static const char* setBufferTime_event;
    static const char* setPeriodTime_event;
    static const char* setXferAlign_event;
    static const char* setMicroInVolume_event;
    static const char* getMicroInVolume_event;
    /** Sets name of pcm device.
     *
     * @param device name of pcm device
     */

    virtual void setPCMDevice(const string& device)
      throw(const NMM::SerializedException&);
    /** Returns name of pcm device.
     *
     * @return name of pcm device
     */

    virtual string getPCMDevice() const
      throw(const NMM::SerializedException&);
    /** Sets buffer time.
     *
     * @param buffertime Buffertime.
     */

    virtual void setBufferTime(const nmm_uint32& buffertime)
      throw(const NMM::SerializedException&);
    /** Sets period time.
     *
     * @param periodtime Periodtime.
     */

    virtual void setPeriodTime(const nmm_uint32& periodtime)
      throw(const NMM::SerializedException&);
    /** Sets transfer alignment.
     *
     * @param xferalign transfer alignment
     */

    virtual void setXferAlign(const nmm_uint32& xferalign)
      throw(const NMM::SerializedException&);

    virtual void setMicroInVolume(const nmm_int32& vol)
      throw(const NMM::SerializedException&);

    virtual nmm_int32 getMicroInVolume()
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IALSADevice> IALSADevice_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
