/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IALSADeviceImpl_HPP
#define NMM_IALSADeviceImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IALSADevice.hpp"

#include "nmm/multimedia/INodeImpl.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Interface for configuring alsa devices.
   */

  class DLLEXPORT IALSADeviceImpl  : public virtual NMM::NMMObject { 
  public:
    IALSADeviceImpl() { addInterface(new IALSADevice(*this)); registerEvents(); };

    virtual ~IALSADeviceImpl() {};

  protected:
    /** Sets name of pcm device.
     *
     * @param device name of pcm device
     */

    virtual void setPCMDevice(const string& device) = 0;
    /** Returns name of pcm device.
     *
     * @return name of pcm device
     */

    virtual string getPCMDevice() const = 0;
    /** Sets buffer time.
     *
     * @param buffertime Buffertime.
     */

    virtual void setBufferTime(const nmm_uint32& buffertime) = 0;
    /** Sets period time.
     *
     * @param periodtime Periodtime.
     */

    virtual void setPeriodTime(const nmm_uint32& periodtime) = 0;
    /** Sets transfer alignment.
     *
     * @param xferalign transfer alignment
     */

    virtual void setXferAlign(const nmm_uint32& xferalign) = 0;

    virtual void setMicroInVolume(const nmm_int32& vol) = 0;

    virtual nmm_int32 getMicroInVolume() = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
