/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#include "DummyAudioSinkNode.hpp"
#include "nmm/plugins/NMMRegisterPlugin.hpp"

NMM_REGISTER_PLUGIN(DummyAudioSinkNode, NMM, "DummyAudioSinkNode")

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  DummyAudioSinkNode::DummyAudioSinkNode(const char* name,
                                         const StreamQueue::Mode queue_mode,
                                         const int size)
    : GenericSinkNode(name, queue_mode, size)
  {}

  DummyAudioSinkNode::~DummyAudioSinkNode() {}

  Result DummyAudioSinkNode::doInit()
  {
    // Most general raw audio format
    Format format;
    format.setFormatType(Format::audio_raw_type);
    format.addWildcard(Format::encoding_param);
    format.addWildcard(Format::sign_param);
    format.addWildcard(Format::bitpersample_param);
    format.addWildcard(Format::endian_param);
    format.addWildcard(Format::channels_param);
    format.addWildcard(Format::sample_rate_param);

    // Add to property
    getOwnInputProperty()->addNewFormat(new Format(format));

    return SUCCESS;
  }

  Message* DummyAudioSinkNode::processBuffer(Buffer *in_buffer)
  {
    // Release buffer
    in_buffer->release();

    return 0;
  }

}

#include "nmm/nmm_suffix.hpp"
