/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#ifndef NMM_DUMMYAUDIOSINKNODE_HPP
#define NMM_DUMMYAUDIOSINKNODE_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/multimedia/GenericSinkNode.hpp"

namespace NMM {

  /** A dummy sink node which only accepts raw audio formats.
   *
   * This node is used for testing the graph builder.
   * It is not a real audio sink, i.e. no synchronized sink. It is only a sink node which accepts
   * certain formats and acts as a DevNullNode.
   */
  class DummyAudioSinkNode :
    public GenericSinkNode
  {
  public:
    /** Constructor.
     */
    DummyAudioSinkNode(const char* name = "DummyAudioSinkNode",
                       const StreamQueue::Mode queue_mode = StreamQueue::DEFAULT_MODE,
                       const int size = StreamQueue::DEFAULT_SIZE);

    /** Destructor.
     */
    ~DummyAudioSinkNode();

  private:
    /** Change state from COSTRUCTED to INITIALIZED.
     *
     * This implementation sets the input property to accept only audio formats.
     *
     * @return SUCCESS
     */
    virtual Result doInit();

    /** Process a buffer.
     *
     * This implementation discards the buffer and does nothing else.
     *
     * @return 0
     */
    virtual Message* processBuffer(Buffer *in_buffer);
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
