/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#ifndef always_inline
#define always_inline inline
#endif

#include <sys/time.h>
#include "FFMpegAudioDecodeNode.hpp"
#include "nmm/comm/format/Property.hpp"
#include "nmm/multimedia/Plugin.hpp"
#include "nmm/multimedia/TPlugin.hpp"
#include <math.h>
#include <iostream>
#include "nmm/comm/serialize/SerializeUtilsTime.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/utils/time/Rational.hpp"
#include "nmm/utils/time/Interval.hpp"

#include "nmm/utils/thread/MutexGuard.hpp"

NMM_REGISTER_PLUGIN(FFMpegAudioDecodeNode, NMM, "FFMpegAudioDecodeNode")

#include "nmm/nmm_prefix.hpp"

namespace NMM {

static int bufcount= 0;
  FFMpegAudioDecodeNode::FFMpegHandle::FFMpegHandle() : ffAVCodec(NULL), ffavcc(NULL), parser(NULL) {}

  FFMpegAudioDecodeNode::FFMpegAudioDecodeNode(const char *name, StreamQueue::Mode qmode, int qsize)
    : GenericProcessorNode(name, qmode, qsize),
      ffmpeg(new FFMpegHandle()),
      first_timestamp(true),
      timestamp(),
      in_buf_ts(),
      /*my_delay(0),
      my_delay_num(0),
      my_delay_den(0),
      num_samples(0), */
      //last_buffer(NULL),
      out_buffer(0),
      out_buffer_size(0),
      sample_rate(0),
      channels(0),
      size(0),
      inbuf_ptr(0),
      outbuf_ptr(0),      
      inbuf(0),
      codecs()
  {
  }
  
  FFMpegAudioDecodeNode::~FFMpegAudioDecodeNode() {
    delete ffmpeg;
  }

  Result FFMpegAudioDecodeNode::startTrack(const string&) {
    DEBUG_STREAM("Received Event: startTrack");
    setWorkingFlag(false);
    return SUCCESS;
  }

  Result FFMpegAudioDecodeNode::endTrack() {
    DEBUG_STREAM("Received Event: endTrack");
    return SUCCESS;
  }

  Result FFMpegAudioDecodeNode::finished() {
    DEBUG_STREAM("Received Event: finished");
    return SUCCESS;
  }

  Result FFMpegAudioDecodeNode::doInit() {
    MutexGuard ffmpeg_guard(ffmpeg_mutex);
    DEBUG_STREAM("Got avcodec version: " << avcodec_version() << endl);
    DEBUG_STREAM("Got avcodec build: " << avcodec_build() << endl);
    
    /* initialize library */
    avcodec_init();

    /* register all the codecs */
    avcodec_register_all();

    /* detect all codecs */
    codecs.clear();

    //default_codec = "";
    detectAllCodecs();

    /* check if any codec was found */
    if(codecs.empty()) {
      ERROR_STREAM("No codecs with known FOURCC code available" << std::endl);
      return FAILURE;
    }

    /* general input format with bitrate known */
/*    Format* in_format = getOwnInputProperty()->addNewFormat(Format::audio_wma_type);
    in_format->addWildcard(Format::audio_codec_param);
    in_format->addWildcard(Format::channels_param);
    in_format->addWildcard(Format::sample_rate_param);
    in_format->addWildcard(Format::bitrate_param);
    in_format->addWildcard(Format::extradata_size_param);
    in_format->addWildcard(Format::blockalign_param);
    in_format->addWildcard(Format::extradata_param);
    in_format->addWildcard(Format::version_param);
    in_format->addWildcard(Format::compressor_param);
*/  
    // format without bitrate
    Format *in_format1 = getOwnInputProperty()->addNewFormat(Format::audio_mpeg_type);
    in_format1->addIntParamValue(Format::audio_codec_param, 0x55);
    in_format1->addWildcard(Format::layer_param);
    in_format1->addIntParamValue(Format::channels_param, 1);
    in_format1->addIntParamValue(Format::channels_param, 2);
    in_format1->addIntParamValue(Format::sample_rate_param, 8000);
    in_format1->addIntParamValue(Format::sample_rate_param, 11025);
    in_format1->addIntParamValue(Format::sample_rate_param, 12000);
    in_format1->addIntParamValue(Format::sample_rate_param, 16000);
    in_format1->addIntParamValue(Format::sample_rate_param, 22050);
    in_format1->addIntParamValue(Format::sample_rate_param, 44100);
    in_format1->addIntParamValue(Format::sample_rate_param, 48000);
    in_format1->addIntParamValue(Format::bitpersample_param, 8);
    in_format1->addIntParamValue(Format::bitpersample_param, 16);

    getOwnInputProperty()->setDefaultFormat(in_format1);    
    
    // format with bitrate
    Format *in_format2 = getOwnInputProperty()->addNewFormat(Format::audio_mpeg_type);
    in_format2->addIntParamValue(Format::audio_codec_param, 0x55);
    in_format2->addWildcard(Format::layer_param);
    in_format2->addWildcard(Format::bitrate_param);
    in_format2->addIntParamValue(Format::channels_param, 1);
    in_format2->addIntParamValue(Format::channels_param, 2);
    in_format2->addIntParamValue(Format::sample_rate_param, 8000);
    in_format2->addIntParamValue(Format::sample_rate_param, 11025);
    in_format2->addIntParamValue(Format::sample_rate_param, 12000);
    in_format2->addIntParamValue(Format::sample_rate_param, 16000);
    in_format2->addIntParamValue(Format::sample_rate_param, 22050);
    in_format2->addIntParamValue(Format::sample_rate_param, 44100);
    in_format2->addIntParamValue(Format::sample_rate_param, 48000);
    in_format2->addIntParamValue(Format::bitpersample_param, 8);
    in_format2->addIntParamValue(Format::bitpersample_param, 16);
    
    Format *in_format3 = getOwnInputProperty()->addNewFormat(Format::audio_mpeg_type);
    in_format3->addIntParamValue(Format::audio_codec_param, 0x55);
    in_format3->addWildcard(Format::layer_param);
    in_format3->addWildcard(Format::bitrate_param);

    Format *in_format4 = getOwnInputProperty()->addNewFormat(Format::audio_mpeg_type);
    in_format4->addIntParamValue(Format::audio_codec_param, 0x55);
    in_format4->addWildcard(Format::layer_param);
    
    // this format is needed so that a GenericReadNode with
    // output format = mime type of read file can be connected
    // *** GenericReadNode is NOT working ***
    Format *in_format5 = getOwnInputProperty()->addNewFormat(Format::audio_mpeg_type);
       
    Format* in_format6 = getOwnInputProperty()->addNewFormat(Format::audio_amr_type);
    in_format6->addIntParamValue(Format::channels_param, 1);
    in_format6->addIntParamValue(Format::sample_rate_param, 8000);
    in_format6->addIntParamValue(Format::bitpersample_param, 16);
    in_format6->addIntParamValue(Format::bitrate_param, 4750);
    in_format6->addIntParamValue(Format::bitrate_param, 5150);
    in_format6->addIntParamValue(Format::bitrate_param, 5900);
    in_format6->addIntParamValue(Format::bitrate_param, 6700);
    in_format6->addIntParamValue(Format::bitrate_param, 7400);
    in_format6->addIntParamValue(Format::bitrate_param, 7950);
    in_format6->addIntParamValue(Format::bitrate_param, 10200);
    in_format6->addIntParamValue(Format::bitrate_param, 12200);
    in_format6->addStringParamValue(Format::encoding_param, "amr_nb");
    
    getOwnInputProperty()->setDefaultFormat(in_format6);

    Format* in_format7 = getOwnInputProperty()->addNewFormat(Format::audio_amr_type);
    in_format7->addIntParamValue(Format::channels_param, 1);
    in_format7->addIntParamValue(Format::sample_rate_param, 16000);
    in_format7->addIntParamValue(Format::bitpersample_param, 16);
    //in_format7->addWildcard(Format::bitrate_param);
    in_format7->addIntParamValue(Format::bitrate_param, 6600);
    in_format7->addIntParamValue(Format::bitrate_param, 8850);
    in_format7->addIntParamValue(Format::bitrate_param, 12650);
    in_format7->addIntParamValue(Format::bitrate_param, 14250);
    in_format7->addIntParamValue(Format::bitrate_param, 15850);
    in_format7->addIntParamValue(Format::bitrate_param, 18250);
    in_format7->addIntParamValue(Format::bitrate_param, 19850);
    in_format7->addIntParamValue(Format::bitrate_param, 23050);
    in_format7->addIntParamValue(Format::bitrate_param, 23850);    
    in_format7->addStringParamValue(Format::encoding_param, "amr_wb");

    Format* out_format = getOwnOutputProperty()->addNewFormat(Format::audio_raw_type);
    out_format->addStringParamValue(Format::endian_param, "little");
    out_format->addStringParamValue(Format::encoding_param, "linear");
    out_format->addStringParamValue(Format::sign_param, "signed");
    out_format->addIntParamValue(Format::sample_rate_param, 8000);
    out_format->addIntParamValue(Format::sample_rate_param, 11025);
    out_format->addIntParamValue(Format::sample_rate_param, 12000);
    out_format->addIntParamValue(Format::sample_rate_param, 16000);
    out_format->addIntParamValue(Format::sample_rate_param, 22050);
    out_format->addIntParamValue(Format::sample_rate_param, 44100);
    out_format->addIntParamValue(Format::sample_rate_param, 48000);
    out_format->addIntParamValue(Format::channels_param, 1);
    out_format->addIntParamValue(Format::channels_param, 2);
    out_format->addIntParamValue(Format::bitpersample_param, 16);
    getOwnOutputProperty()->setDefaultFormat(out_format);
    
    return SUCCESS;
  }

  Result FFMpegAudioDecodeNode::doInitOutput() {
    /* get input format */
    const Format* in_format = getInputFormat();
    if (in_format == NULL) {
      ERROR_STREAM("Missing input format" << std::endl);
      return FAILURE;
    }
    // set output format
    // put input format in working property
    Format* i_format = new Format(*in_format);
    getOwnWorkingInputProperty()->addNewFormat(i_format);
    getOwnWorkingInputProperty()->setDefaultFormat(i_format);
   
    // need channels and sample rate
    if (in_format->isParameterSpecified(Format::sample_rate_param) &&
      in_format->isParameterSpecified(Format::channels_param)) {
          
      sample_rate = in_format->getIntValue(Format::sample_rate_param);
      channels = in_format->getIntValue(Format::channels_param);  
           
      // build output format from input format.
      setWOPInternal();
     
     } else {
       sample_rate = 0;
       channels = 0;
         
       // build output format from data analysis
       DEBUG_STREAM("No fully specified input format, need to analyze data" << endl);
       
       // *** GenericReadNode *NOT* supported yet...stopping here
       ERROR_STREAM("No sample rate and channels specified...stopping." << endl);     
       return FAILURE;
       
       /*
       setMaxAnalysisIterations(250);
       if (!analyseData()) {
	 ERROR_STREAM("analyseData failed in doInitOutput" << endl);
	 return FAILURE;
       }
       */
     }
     
    // set I/O partners
    const_cast<Format*>(getOwnWorkingInputProperty()->
			getDefaultFormat())->
      setIOPartner(const_cast<Format*>(getOwnWorkingOutputProperty()->
				       getDefaultFormat()));
    
    const_cast<Format*>(getOwnWorkingOutputProperty()->
			getDefaultFormat())->
      setIOPartner(const_cast<Format*>(getOwnWorkingInputProperty()->
				       getDefaultFormat()));
        
                                       
    // cerr << "FFMpegAudioDecodeNode InputFormat: " << *(getOwnWorkingInputProperty()->getDefaultFormat()) << endl;
    // cerr << "FFMpegAudioDecodeNode OutputFormat: " << *(getOwnWorkingOutputProperty()->getDefaultFormat()) << endl;
    
    return SUCCESS;
  }

  Result FFMpegAudioDecodeNode::doActivate() {
    /*eget input format */
    const Format* in_format = getInputFormat();
    if (in_format == NULL) {
      ERROR_STREAM("Missing input format" << std::endl);
      return FAILURE;
    }
       
    if (in_format->getFormatType() == Format::audio_mpeg_type) {
      compressor = "MP3";
    }
    else if ((sample_rate == 8000) && (channels == 1)){
      compressor = "AMR_NB"; }
    else if ((sample_rate == 16000) && (channels == 1)){
      compressor = "AMR_WB"; }
    else {
      /* SHOULD NOT HAPPEN */
      ERROR_STREAM("Invalid input format type" << endl);
      return FAILURE;
    }

    cerr << "FFMpegAudioDecodeNode: Decoder: " << compressor << endl;
    
    // find the correct decoder
    CodecConstIterator found = codecs.find(compressor);
   
    if(found == codecs.end()) {
      /* should not happen */
      ERROR_STREAM("FATAL: No codec listed for FourCC code " << compressor << std::endl);
      return FAILURE;
    }
    
    // find a codec with this ID
    {
      MutexGuard ffmpeg_guard(ffmpeg_mutex);
      ffmpeg->ffAVCodec = avcodec_find_decoder(static_cast<CodecID>(found->second));
    }
    if (!ffmpeg->ffAVCodec) {
      /* should not happen */
      ERROR_STREAM("FATAL: Unsupported codec " << std::endl);
      return FAILURE;
    }
  
    /* allocate ffmpeg context */
    {
      MutexGuard ffmpeg_guard(ffmpeg_mutex);
      ffmpeg->ffavcc = avcodec_alloc_context();
    }

    ffmpeg->ffavcc->sample_rate = sample_rate; 
    ffmpeg->ffavcc->channels = channels; 
    
    if (compressor == "MP3")
      if (in_format->isParameterSpecified(Format::bitrate_param))
        ffmpeg->ffavcc->bit_rate = in_format->getIntValue(Format::bitrate_param);
                  
    {
      MutexGuard ffmpeg_guard(ffmpeg_mutex);
      if(avcodec_open(ffmpeg->ffavcc, ffmpeg->ffAVCodec) < 0) {
        ERROR_STREAM("Could not open codec" << std::endl);
    
        if(ffmpeg->ffavcc) {
        /* release ffmpeg context */
          free(ffmpeg->ffavcc);
          ffmpeg->ffavcc = NULL;
        }
        
        DEBUG_STREAM("about to return FAILURE" << endl);
        return FAILURE;
      }
    }
    
    if (compressor == "MP3") {
      {
        MutexGuard ffmpeg_guard(ffmpeg_mutex);
        ffmpeg->parser = av_parser_init(ffmpeg->ffavcc->codec_id);
      }
      if (ffmpeg->parser == 0) {
        DEBUG_STREAM("about to return FAILURE" << endl);
        return FAILURE;
      }
    }
    
    return SUCCESS;
  }
  
  Result FFMpegAudioDecodeNode::doDeactivate() {
  
    free(ffmpeg->ffavcc->extradata);
	  
    /* flush ffmpeg buffers */
    if(ffmpeg->ffavcc) {
      MutexGuard ffmpeg_guard(ffmpeg_mutex);
      avcodec_flush_buffers(ffmpeg->ffavcc);
    }
    
    /* close codec */
    {
      MutexGuard ffmpeg_guard(ffmpeg_mutex);
      avcodec_close(ffmpeg->ffavcc);
    }
      
    /* release ffmpeg context */
    if(ffmpeg->ffavcc) {
      free(ffmpeg->ffavcc);
      ffmpeg->ffavcc = NULL;
    }
    
    if(ffmpeg->parser) {
      free(ffmpeg->parser);
      ffmpeg->parser = NULL;
    }
    
    return SUCCESS;
  }
    
  Message* FFMpegAudioDecodeNode::processBuffer(Buffer* in_buffer) {
    DEBUG_STREAM("Entered processBuffer!" << endl);
    out_buffer = getNewBuffer(AVCODEC_MAX_AUDIO_FRAME_SIZE);
    bool buffer_released = false;
    
    const Format* in_format = getInputFormat();
    if (in_format == NULL) {
      ERROR_STREAM("FFMpegAudioDecodeNode: Missing input format" << std::endl);
      return 0;
    }  
       
    if(!getWorkingFlag()) {

      //start processing new buffer
      
      size = in_buffer->getUsed();    
      
      //cerr << "Size: " << size << endl;
      inbuf = reinterpret_cast<UINT8*>(in_buffer->getData());
      inbuf_ptr = inbuf;
           
      if (!in_buf_ts.is_valid) {
        // Get the timestamp of the incoming buffer
        // only if we don't have one yet
        in_buf_ts = in_buffer->getTimestamp();
      }
    }
    
    if (size > 0) {
            
      uint8_t* frame = 0;
      int framesize = 0;
      long bytes_decoded = 0;  
      int decode_result = 0;
    
      if (compressor == "MP3") {        
        MutexGuard ffmpeg_guard(ffmpeg_mutex);
        // need to parse MP3 buffers
        bytes_decoded = av_parser_parse(ffmpeg->parser, ffmpeg->ffavcc, &frame, &framesize,              reinterpret_cast<uint8_t *>(inbuf_ptr), size, 0 ,0);	//pts, dts
                 
        decode_result = avcodec_decode_audio(ffmpeg->ffavcc, 
                        reinterpret_cast<int16_t *>(out_buffer->getData()), 
                        &out_buffer_size, 
                        frame, 
                        framesize);
      } else if (compressor == "AMR_NB" || compressor == "AMR_WB") {
        
        MutexGuard ffmpeg_guard(ffmpeg_mutex);
        bytes_decoded  = avcodec_decode_audio(ffmpeg->ffavcc, 
                                reinterpret_cast<short *>(out_buffer->getData()), 
                                &out_buffer_size,
                                inbuf_ptr, 
                                size);
      } else {
        ERROR_STREAM("FFMpegAudioDecodeNode: Unsupported compressor!!!" << endl);
      }
                 
      DEBUG_STREAM("FFMpegAudioDecodeNode: Bytes decoded: " << bytes_decoded  << endl);
      DEBUG_STREAM("FFMpegAudioDecodeNode: Size: " << size  << endl);
      DEBUG_STREAM("FFMpegAudioDecodeNode: FrameSize: " << framesize  << endl);
      DEBUG_STREAM("FFMpegAudioDecodeNode: OutBufferSize: " << out_buffer_size  << endl);
      DEBUG_STREAM("FFMpegAudioDecodeNode: DecodeResult: " << decode_result  << endl);
      
      bool discard_frame = false;
      
      if (bytes_decoded < 0) {
	ERROR_STREAM("FFMpegAudioDecodeNode: Bytes decoded was negative!" << endl);      
        cerr << "Bytes decoded was negative!" << endl;
        discard_frame = true;
      } else {
      
        //update buffer pointers
        size -= bytes_decoded;
        inbuf_ptr += bytes_decoded;
        
        // we have decoded a frame
        out_buffer->setUsed(out_buffer_size);
        
        DEBUG_STREAM("out_buffer->setUsed(" << out_buffer_size << ");"<< endl);
        //cerr << "out_buffer->setUsed(" << out_buffer_size << ");"<< endl;
        DEBUG_STREAM("out_buffer_size = " << out_buffer_size << endl);
        
    
        // is there data left in the buffer?
        if(size > 0) {
                
          if(!getWorkingFlag()) { // do we need to copy data into our own buffer?
            //last_buffer = new unsigned char[AVCODEC_MAX_AUDIO_FRAME_SIZE];
            setWorkingFlag(true);
            DEBUG_STREAM("about to memmove " << size << " bytes." <<  endl);
            memmove(last_buffer, inbuf_ptr, size);
            inbuf_ptr = last_buffer;
    
            // remember to not call release again!
            buffer_released = true;
            
            // the compressed frame is not needed any more
            in_buffer->release();
          }
          
        }
        else { // all the buffer is decoded. need to get new input buffer
          setWorkingFlag(false);
          //DEBUG_STREAM("about to release last_buffer." <<  endl);
          /* if (last_buffer) {
            delete[] last_buffer;
            last_buffer = 0;
            }
            */
        } 
      }
        
      if (out_buffer_size <= 0) {        
      ERROR_STREAM("FFMpegAudioDecodeNode: out_buffer_size <=0. Drop Frame." << endl);
      cerr << "FFMpegAudioDecodeNode: out_buffer_size <=0. Drop Frame." << endl;
      discard_frame = true;
      }
          
      if (decode_result < 0) {
        ERROR_STREAM("Bytes decoded was negative! Drop Frame." << endl);   
        cerr << "Bytes decoded was negative! Drop Frame." << endl;
        discard_frame = true;
      }
      
      DEBUG_STREAM("about to read number of samples." <<  endl);
        //XXX: need to carry this value forward as a format parameter from the ASFReadNode, even though its always 1 (i think)
        //num_samples = ffmpeg->ffavcc->frame_size;
/*	num_samples = 1;
        if(num_samples < 0) {
          DEBUG_STREAM("Number of samples invalid. Can not compute timestamp." << endl);
        }
        else {
        
          my_delay_num = num_samples;
          DEBUG_STREAM("Number of samples: " << num_samples << endl);
          my_delay_den = ffmpeg->ffavcc->sample_rate;
          DEBUG_STREAM("Sample rate: " << my_delay_den << endl);
          my_delay = (my_delay_num * 1000000) / my_delay_den; 
          my_delay *= 1000;
          DEBUG_STREAM("Duration of this buffer is " << my_delay << " nanoseconds." << endl);
        }
        */

      // the creation of the timestamps is mostly taken from the MPEGAudioDecodeNode code
      if(in_buf_ts.is_set) { // if timestamp was set by predecessor (e.g. MPEGDemuxNode)
        if(in_buf_ts.is_valid) {                              // if the incoming timestamp is valid
          DEBUG_STREAM("in_buf_ts is VALID!!!" << endl);
          timestamp.sync_time = in_buf_ts.sync_time;          // take it
          timestamp.is_valid  = true;                         // remember that it was valid
          in_buf_ts.is_valid = false;                         // and make sure that it is taken only once
          first_timestamp = false;                            // also, remember that we have a valid timestamp
        }
        else {
          // if the first incoming timestamp after syncReset is invalid
          if(first_timestamp) {
            DEBUG_STREAM("FIRST in_buf_ts is NOT VALID!!!" << endl);
            timestamp.sync_time = in_buf_ts.sync_time;       // take it anyway
            timestamp.is_valid  = false;                     // but remember that it was invalid
            first_timestamp = false;                         // also, remember that we have some first (invalid) timestamp
          }
          else { // not valid, but found a valid ts before -> new ts will be computed by taking valid ts
            DEBUG_STREAM("in_buf_ts is NOT VALID but we already had a valid" << endl);
          }
        }
      }
      else { // else: timestamp was *not* set by predecessor (e.g. GenericReadNode) :
        timestamp.is_valid = true; // will generate timestamps internally -> valid timestamps
        // do not need to do anything else : timestamp should be zero, and will be incremented for every outgoing buffer
      }
      out_buffer->setTimestamp(timestamp);           // Finally set the timestamp
      DEBUG_STREAM("Set timestamp to: " << timestamp << endl);
      
      
      // Do already the computations for the next outgoing buffer's timestamp
      timestamp.stream_counter++;             
      
      if (compressor == "MP3") {         
        double samples_per_sec = sample_rate * channels * getOutputFormat()->getIntValue(Format::bitpersample_param)/8;
        
        timestamp.sync_time += static_cast<long int>(( static_cast<double>(out_buffer->getUsed())/samples_per_sec) * 1000000000);
      }
      else {
        // AMR frames: 20 ms; we need nanosec
        timestamp.sync_time += 20*1000000;
      }
      
      //cerr << "FFMpegAudioDecodeNode Sync time: " << timestamp.sync_time << endl;
      
      //cout << "Audio timestamp " << timestamp << endl;
      DEBUG_STREAM("Returning from processBuffer." << endl);

      if (discard_frame) {
        out_buffer->release();
        return 0;
      }
            
      bufcount++;
      return out_buffer;     
    }
    
    DEBUG_STREAM("Buffer boundary reached, but no frame was decoded. Need new buffer."  << endl);
    if(getWorkingFlag()) {
      // need to get new input
      setWorkingFlag(false);
      /*
      if(last_buffer) {
	delete[] last_buffer;
        last_buffer = 0;
      }
      */
    }
    else 
    {
      if(!buffer_released)
        // Whole packet read. No frame decoded???? Is this possible?
        in_buffer->release();
      
      ERROR_STREAM("Not Working. This should not have happened!!" << endl);
    }
    DEBUG_STREAM("Returning from processBuffer." << endl);
    
    return 0;
  }
  
    
  Result FFMpegAudioDecodeNode::doFlush() {
    setWorkingFlag(false);
    return SUCCESS;
  }

  void FFMpegAudioDecodeNode::setWOPInternal() {

    Format* o_format = getOwnWorkingOutputProperty()->addNewFormat(Format::audio_raw_type);
    o_format->addStringParamValue(Format::endian_param, "little");
    o_format->addStringParamValue(Format::encoding_param, "linear");
    o_format->addStringParamValue(Format::sign_param, "signed");
    o_format->addIntParamValue(Format::sample_rate_param, sample_rate);
    o_format->addIntParamValue(Format::channels_param, channels);
    o_format->addIntParamValue(Format::bitpersample_param, 16); // marc always scales to 16
    getOwnWorkingOutputProperty()->setDefaultFormat(o_format);
  }


  void FFMpegAudioDecodeNode::detectAllCodecs() throw() {
    /* get all available codecs */
    AVCodec *p = first_avcodec;
    DEBUG_STREAM("Available Decoders:" << endl);
    while (p) {
      if (p->decode) {
        DEBUG_STREAM("  " << p->name << endl);
        switch(p->id) {
        case CODEC_ID_AMR_NB:
          addCodec("AMR_NB", p->id);
          break;
	case CODEC_ID_AMR_WB:
	  addCodec("AMR_WB", p->id);
	  break;
        case CODEC_ID_MP3:
          addCodec("MP3", p->id);
          break;
        default:
          break;
        }
      }

      p = p->next;
    }
    DEBUG_STREAM("Reached end of detectAllCodecs." << endl);
  }

  void FFMpegAudioDecodeNode::addCodec(const std::string& fourcc, int id) {
    /* add codec to map */
    codecs.insert(std::pair<std::string, int>(fourcc, id));
    DEBUG_STREAM("    " << fourcc << std::endl);
  }
  
}



#include "nmm/nmm_suffix.hpp"

