/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IIcecast.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IIcecast::used() {} 
  
  
  Result IIcecast::setServerIP(const string& value) /* limited(INode::INITIALIZED) */ /* required(INode::INITIALIZED, INode::ACTIVATED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > value_value(value, false);
    NMM::Event event("IIcecast::setServerIP", &value_value);
    event.setReturnValue(&return_value);
    if(const_cast<IIcecast*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IIcecast::setServerPort(const nmm_int32& value) /* limited(INode::INITIALIZED) */ /* required(INode::INITIALIZED, INode::ACTIVATED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > value_value(value, false);
    NMM::Event event("IIcecast::setServerPort", &value_value);
    event.setReturnValue(&return_value);
    if(const_cast<IIcecast*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IIcecast::setServerPassword(const string& value) /* limited(INode::INITIALIZED) */ /* required(INode::INITIALIZED, INode::ACTIVATED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > value_value(value, false);
    NMM::Event event("IIcecast::setServerPassword", &value_value);
    event.setReturnValue(&return_value);
    if(const_cast<IIcecast*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IIcecast::setUser(const string& value) /* limited(INode::INITIALIZED) */ /* required(INode::INITIALIZED, INode::ACTIVATED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > value_value(value, false);
    NMM::Event event("IIcecast::setUser", &value_value);
    event.setReturnValue(&return_value);
    if(const_cast<IIcecast*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IIcecast::setMountpoint(const string& value) /* limited(INode::INITIALIZED) */ /* required(INode::INITIALIZED, INode::ACTIVATED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > value_value(value, false);
    NMM::Event event("IIcecast::setMountpoint", &value_value);
    event.setReturnValue(&return_value);
    if(const_cast<IIcecast*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IIcecast::setTitle(const string& value) /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > value_value(value, false);
    NMM::Event event("IIcecast::setTitle", &value_value);
    event.setReturnValue(&return_value);
    if(const_cast<IIcecast*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IIcecast::setStreamName(const string& value) /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > value_value(value, false);
    NMM::Event event("IIcecast::setStreamName", &value_value);
    event.setReturnValue(&return_value);
    if(const_cast<IIcecast*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IIcecast::setGenre(const string& value) /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > value_value(value, false);
    NMM::Event event("IIcecast::setGenre", &value_value);
    event.setReturnValue(&return_value);
    if(const_cast<IIcecast*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IIcecast::setDescription(const string& value) /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > value_value(value, false);
    NMM::Event event("IIcecast::setDescription", &value_value);
    event.setReturnValue(&return_value);
    if(const_cast<IIcecast*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IIcecast::setURL(const string& value) /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > value_value(value, false);
    NMM::Event event("IIcecast::setURL", &value_value);
    event.setReturnValue(&return_value);
    if(const_cast<IIcecast*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IIcecast::setPublic(const nmm_int32& value) /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > value_value(value, false);
    NMM::Event event("IIcecast::setPublic", &value_value);
    event.setReturnValue(&return_value);
    if(const_cast<IIcecast*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IIcecast::setSendTitle(const nmm_bool& value) /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_bool > value_value(value, false);
    NMM::Event event("IIcecast::setSendTitle", &value_value);
    event.setReturnValue(&return_value);
    if(const_cast<IIcecast*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IIcecast::IIcecast_type = "IIcecast";
  NMM::TConstructor<IIcecast > __IIcecast_prototype("IIcecast","NMM");


  const char* IIcecast::setServerIP_event = "IIcecast::setServerIP";
  const char* IIcecast::setServerPort_event = "IIcecast::setServerPort";
  const char* IIcecast::setServerPassword_event = "IIcecast::setServerPassword";
  const char* IIcecast::setUser_event = "IIcecast::setUser";
  const char* IIcecast::setMountpoint_event = "IIcecast::setMountpoint";
  const char* IIcecast::setTitle_event = "IIcecast::setTitle";
  const char* IIcecast::setStreamName_event = "IIcecast::setStreamName";
  const char* IIcecast::setGenre_event = "IIcecast::setGenre";
  const char* IIcecast::setDescription_event = "IIcecast::setDescription";
  const char* IIcecast::setURL_event = "IIcecast::setURL";
  const char* IIcecast::setPublic_event = "IIcecast::setPublic";
  const char* IIcecast::setSendTitle_event = "IIcecast::setSendTitle";


}


#include "nmm/nmm_suffix.hpp"

