/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IIcecast_HPP
#define NMM_IIcecast_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "nmm/multimedia/INode.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Interface for IceCast sink node.
   */

  class DLLEXPORT IIcecast : virtual public NMM::Interface { 
  public:
    IIcecast() : NMM::Interface(IIcecast::IIcecast_type, "NMM") { };

    IIcecast(NMM::NMMObject& parent) : NMM::Interface (parent, IIcecast::IIcecast_type, "NMM") { };

    virtual ~IIcecast() {};

    static const char* IIcecast_type;

    virtual  string getInterfaceType() const { return IIcecast_type; }

    NMM::Interface* copy() const { return new IIcecast(*this); }
    static void used(); 
    static const char* setServerIP_event;
    static const char* setServerPort_event;
    static const char* setServerPassword_event;
    static const char* setUser_event;
    static const char* setMountpoint_event;
    static const char* setTitle_event;
    static const char* setStreamName_event;
    static const char* setGenre_event;
    static const char* setDescription_event;
    static const char* setURL_event;
    static const char* setPublic_event;
    static const char* setSendTitle_event;
    /** Set the IP address of the IceCast streaming server.
     */

    virtual Result setServerIP(const string& value) /* limited(INode::INITIALIZED) */ /* required(INode::INITIALIZED, INode::ACTIVATED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /** Set the port of the IceCast streaming server.
     */

    virtual Result setServerPort(const nmm_int32& value) /* limited(INode::INITIALIZED) */ /* required(INode::INITIALIZED, INode::ACTIVATED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /** Set the source password for the IceCast streaming server.
     */

    virtual Result setServerPassword(const string& value) /* limited(INode::INITIALIZED) */ /* required(INode::INITIALIZED, INode::ACTIVATED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /** Set the user name for the IceCast streaming server.
     */

    virtual Result setUser(const string& value) /* limited(INode::INITIALIZED) */ /* required(INode::INITIALIZED, INode::ACTIVATED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /** Set the mount point.
     */

    virtual Result setMountpoint(const string& value) /* limited(INode::INITIALIZED) */ /* required(INode::INITIALIZED, INode::ACTIVATED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /** Set the source title.
     */

    virtual Result setTitle(const string& value) /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /** Set the source name.
     */

    virtual Result setStreamName(const string& value) /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /** Set the source genre.
     */

    virtual Result setGenre(const string& value) /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /** Set the source description.
     */

    virtual Result setDescription(const string& value) /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /** Set the source URL.
     */

    virtual Result setURL(const string& value) /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /** Set public mode.
     */

    virtual Result setPublic(const nmm_int32& value) /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /** Set whether to send title.
     */

    virtual Result setSendTitle(const nmm_bool& value) /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);


  };

  typedef auto_ptr<IIcecast> IIcecast_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
