/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IIcecastImpl.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/messaging/TEDObject.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  
  void IIcecastImpl::registerEvents()
  {
  
    getEventDispatcher().registerEvent(IIcecast::setServerIP_event, new NMM::TEDObject1<IIcecastImpl, string, Result(IIcecastImpl:: * )(const string&), Result >(this, &IIcecastImpl::setServerIP));
    getEventDispatcher().addLimited(IIcecast::setServerIP_event, INode::INITIALIZED);
    getEventDispatcher().addRequired(IIcecast::setServerIP_event, INode::INITIALIZED, INode::ACTIVATED);
  
    getEventDispatcher().registerEvent(IIcecast::setServerPort_event, new NMM::TEDObject1<IIcecastImpl, nmm_int32, Result(IIcecastImpl:: * )(const nmm_int32&), Result >(this, &IIcecastImpl::setServerPort));
    getEventDispatcher().addLimited(IIcecast::setServerPort_event, INode::INITIALIZED);
    getEventDispatcher().addRequired(IIcecast::setServerPort_event, INode::INITIALIZED, INode::ACTIVATED);
  
    getEventDispatcher().registerEvent(IIcecast::setServerPassword_event, new NMM::TEDObject1<IIcecastImpl, string, Result(IIcecastImpl:: * )(const string&), Result >(this, &IIcecastImpl::setServerPassword));
    getEventDispatcher().addLimited(IIcecast::setServerPassword_event, INode::INITIALIZED);
    getEventDispatcher().addRequired(IIcecast::setServerPassword_event, INode::INITIALIZED, INode::ACTIVATED);
  
    getEventDispatcher().registerEvent(IIcecast::setUser_event, new NMM::TEDObject1<IIcecastImpl, string, Result(IIcecastImpl:: * )(const string&), Result >(this, &IIcecastImpl::setUser));
    getEventDispatcher().addLimited(IIcecast::setUser_event, INode::INITIALIZED);
    getEventDispatcher().addRequired(IIcecast::setUser_event, INode::INITIALIZED, INode::ACTIVATED);
  
    getEventDispatcher().registerEvent(IIcecast::setMountpoint_event, new NMM::TEDObject1<IIcecastImpl, string, Result(IIcecastImpl:: * )(const string&), Result >(this, &IIcecastImpl::setMountpoint));
    getEventDispatcher().addLimited(IIcecast::setMountpoint_event, INode::INITIALIZED);
    getEventDispatcher().addRequired(IIcecast::setMountpoint_event, INode::INITIALIZED, INode::ACTIVATED);
  
    getEventDispatcher().registerEvent(IIcecast::setTitle_event, new NMM::TEDObject1<IIcecastImpl, string, Result(IIcecastImpl:: * )(const string&), Result >(this, &IIcecastImpl::setTitle));
    getEventDispatcher().addLimited(IIcecast::setTitle_event, INode::INITIALIZED);
  
    getEventDispatcher().registerEvent(IIcecast::setStreamName_event, new NMM::TEDObject1<IIcecastImpl, string, Result(IIcecastImpl:: * )(const string&), Result >(this, &IIcecastImpl::setStreamName));
    getEventDispatcher().addLimited(IIcecast::setStreamName_event, INode::INITIALIZED);
  
    getEventDispatcher().registerEvent(IIcecast::setGenre_event, new NMM::TEDObject1<IIcecastImpl, string, Result(IIcecastImpl:: * )(const string&), Result >(this, &IIcecastImpl::setGenre));
    getEventDispatcher().addLimited(IIcecast::setGenre_event, INode::INITIALIZED);
  
    getEventDispatcher().registerEvent(IIcecast::setDescription_event, new NMM::TEDObject1<IIcecastImpl, string, Result(IIcecastImpl:: * )(const string&), Result >(this, &IIcecastImpl::setDescription));
    getEventDispatcher().addLimited(IIcecast::setDescription_event, INode::INITIALIZED);
  
    getEventDispatcher().registerEvent(IIcecast::setURL_event, new NMM::TEDObject1<IIcecastImpl, string, Result(IIcecastImpl:: * )(const string&), Result >(this, &IIcecastImpl::setURL));
    getEventDispatcher().addLimited(IIcecast::setURL_event, INode::INITIALIZED);
  
    getEventDispatcher().registerEvent(IIcecast::setPublic_event, new NMM::TEDObject1<IIcecastImpl, nmm_int32, Result(IIcecastImpl:: * )(const nmm_int32&), Result >(this, &IIcecastImpl::setPublic));
    getEventDispatcher().addLimited(IIcecast::setPublic_event, INode::INITIALIZED);
  
    getEventDispatcher().registerEvent(IIcecast::setSendTitle_event, new NMM::TEDObject1<IIcecastImpl, nmm_bool, Result(IIcecastImpl:: * )(const nmm_bool&), Result >(this, &IIcecastImpl::setSendTitle));
    getEventDispatcher().addLimited(IIcecast::setSendTitle_event, INode::INITIALIZED);

  }

}


#include "nmm/nmm_suffix.hpp"

