/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IIcecastImpl_HPP
#define NMM_IIcecastImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IIcecast.hpp"

#include "nmm/multimedia/INodeImpl.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Interface for IceCast sink node.
   */

  class DLLEXPORT IIcecastImpl  : public virtual NMM::NMMObject { 
  public:
    IIcecastImpl() { addInterface(new IIcecast(*this)); registerEvents(); };

    virtual ~IIcecastImpl() {};

  protected:
    /** Set the IP address of the IceCast streaming server.
     */

    virtual Result setServerIP(const string& value) /* limited(INode::INITIALIZED) */ /* required(INode::INITIALIZED, INode::ACTIVATED) */ = 0;
    /** Set the port of the IceCast streaming server.
     */

    virtual Result setServerPort(const nmm_int32& value) /* limited(INode::INITIALIZED) */ /* required(INode::INITIALIZED, INode::ACTIVATED) */ = 0;
    /** Set the source password for the IceCast streaming server.
     */

    virtual Result setServerPassword(const string& value) /* limited(INode::INITIALIZED) */ /* required(INode::INITIALIZED, INode::ACTIVATED) */ = 0;
    /** Set the user name for the IceCast streaming server.
     */

    virtual Result setUser(const string& value) /* limited(INode::INITIALIZED) */ /* required(INode::INITIALIZED, INode::ACTIVATED) */ = 0;
    /** Set the mount point.
     */

    virtual Result setMountpoint(const string& value) /* limited(INode::INITIALIZED) */ /* required(INode::INITIALIZED, INode::ACTIVATED) */ = 0;
    /** Set the source title.
     */

    virtual Result setTitle(const string& value) /* limited(INode::INITIALIZED) */ = 0;
    /** Set the source name.
     */

    virtual Result setStreamName(const string& value) /* limited(INode::INITIALIZED) */ = 0;
    /** Set the source genre.
     */

    virtual Result setGenre(const string& value) /* limited(INode::INITIALIZED) */ = 0;
    /** Set the source description.
     */

    virtual Result setDescription(const string& value) /* limited(INode::INITIALIZED) */ = 0;
    /** Set the source URL.
     */

    virtual Result setURL(const string& value) /* limited(INode::INITIALIZED) */ = 0;
    /** Set public mode.
     */

    virtual Result setPublic(const nmm_int32& value) /* limited(INode::INITIALIZED) */ = 0;
    /** Set whether to send title.
     */

    virtual Result setSendTitle(const nmm_bool& value) /* limited(INode::INITIALIZED) */ = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
