/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ICECASTNODE_HPP
#define NMM_ICECASTNODE_HPP NMM_ICECASTNODE_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/multimedia/GenericSinkNode.hpp"
#include "nmm/multimedia/interfaces/ITrackImpl.hpp"
#include "nmm/plugins/audio/icecast/IIcecastImpl.hpp"

#include <shout/shout.h>

namespace NMM {

  /**
   * IcecastNode
   */
  class IcecastNode :
    public GenericSinkNode,
    public IIcecastImpl,
    public ITrackImpl
  {
  private:
    string title;                     // title send as metadata
    bool send_title;                  // if <-this flag is set
    
    bool ignore_incoming;             // if set, the node will ignore all incoming data (id3 tags for example)
    
  protected:
    shout_t   *shout_connection; // connection data structure
    shout_metadata_t *shout_metadata;
    
    virtual Result doInit();
    virtual Result doActivate();
    virtual Result doDeactivate();
    virtual Buffer *processBuffer(Buffer *buffer);
    
    // ITrack interface
    virtual Result started(const Interval& overall_duration);
    virtual Result startTrack(const string& track);
    virtual Result endTrack();
    virtual Result finished();
    
  public:
    IcecastNode(const char *name = "IcecastNode", 
		const StreamQueue::Mode queue_mode = StreamQueue::MODE_SUSPEND, 
		const int queue_size = 1);
    ~IcecastNode();
    
    // IIcecast interface
    virtual Result setServerIP(const string& value);
    virtual Result setServerPort(const nmm_int32& value);
    virtual Result setServerPassword(const string& value);
    virtual Result setUser(const string& value);
    virtual Result setMountpoint(const string& value);
    virtual Result setTitle(const string& value);
    virtual Result setStreamName(const string& value);
    virtual Result setGenre(const string& value);
    virtual Result setDescription(const string& value);
    virtual Result setURL(const string& value);
    virtual Result setPublic(const nmm_int32& value);
    virtual Result setSendTitle(const nmm_bool& value);
    
  private:
    void sendMetadata(const string& name, const string& value);
  };
  
}

#include "nmm/nmm_suffix.hpp"

#endif
