/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IAudioDevice.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IAudioDevice::used() {} 
  
  
  void IAudioDevice::setMixerDevice(const string& value) 
      throw(const NMM::SerializedException&, const DeviceNotFoundException&)
  {
    NMM::TInValue<string > value_value(value, false);
    NMM::Event event("IAudioDevice::setMixerDevice", &value_value);
    if(const_cast<IAudioDevice*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(DeviceNotFoundException* ex = dynamic_cast<DeviceNotFoundException*>(event.getException())) {
          throw DeviceNotFoundException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IAudioDevice::setMixerVolumeElement(const string& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > value_value(value, false);
    NMM::Event event("IAudioDevice::setMixerVolumeElement", &value_value);
    if(const_cast<IAudioDevice*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  string IAudioDevice::getMixerDevice() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("IAudioDevice::getMixerDevice");
    event.setReturnValue(&return_value);
    if(const_cast<IAudioDevice*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IAudioDevice::increaseVolume()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IAudioDevice::increaseVolume");
    if(const_cast<IAudioDevice*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IAudioDevice::decreaseVolume()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IAudioDevice::decreaseVolume");
    if(const_cast<IAudioDevice*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IAudioDevice::setVolume(const nmm_int32& volume)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > volume_value(volume, false);
    NMM::Event event("IAudioDevice::setVolume", &volume_value);
    if(const_cast<IAudioDevice*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_int32 IAudioDevice::getVolume()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("IAudioDevice::getVolume");
    event.setReturnValue(&return_value);
    if(const_cast<IAudioDevice*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IAudioDevice::IAudioDevice_type = "IAudioDevice";
  NMM::TConstructor<IAudioDevice > __IAudioDevice_prototype("IAudioDevice","NMM");


  const char* IAudioDevice::setMixerDevice_event = "IAudioDevice::setMixerDevice";
  const char* IAudioDevice::setMixerVolumeElement_event = "IAudioDevice::setMixerVolumeElement";
  const char* IAudioDevice::getMixerDevice_event = "IAudioDevice::getMixerDevice";
  const char* IAudioDevice::increaseVolume_event = "IAudioDevice::increaseVolume";
  const char* IAudioDevice::decreaseVolume_event = "IAudioDevice::decreaseVolume";
  const char* IAudioDevice::setVolume_event = "IAudioDevice::setVolume";
  const char* IAudioDevice::getVolume_event = "IAudioDevice::getVolume";


}


#include "nmm/nmm_suffix.hpp"

