/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IAudioDeviceImpl_HPP
#define NMM_IAudioDeviceImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IAudioDevice.hpp"

#include "nmm/multimedia/IDeviceImpl.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Interface for configuring audio devices.
   */

  class DLLEXPORT IAudioDeviceImpl  : virtual public IDeviceImpl { 
  public:
    IAudioDeviceImpl() { addInterface(new IAudioDevice(*this)); registerEvents(); };

    virtual ~IAudioDeviceImpl() {};

  protected:
    /** Sets mixer device.
     *
     * @param value name of device 
     */

    virtual void setMixerDevice(const string& value) 
      throw(const DeviceNotFoundException&) = 0;
    /** Sets the element name of the mixer element. 
     *
     * Sets the element name of the mixer element. used by setVolume, increaseVolume and decreaseVolume.
     * The interpretation of the element name is device-specific.
     *
     * @param value Name of mixer element.
     */

    virtual void setMixerVolumeElement(const string& value) = 0;
    /** Returns the element name of the mixer element used by setVolume, increaseVolume and decreaseVolume-
     *
     * @return Name of mixer element.
     */

    virtual string getMixerDevice() const = 0;
    /** Increases Volume.
     */

    virtual void increaseVolume() = 0;
    /** Decreases Volume.
     */

    virtual void decreaseVolume() = 0;
    /** Sets volume.
     *
     * @param volume Volume to set.
     */

    virtual void setVolume(const nmm_int32& volume) = 0;
    /** Returns current volume value.
     *
     * @return current volume value.
     */

    virtual nmm_int32 getVolume() = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
