/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IVolumeControl.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/serialize/SerializeUtilsTime.hpp"

namespace NMM
{
  void IVolumeControl::used() {} 
  
  
  Rational IVolumeControl::setLinearVolume(const Rational& linear_volume)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Rational > return_value;
    NMM::TInValue<Rational > linear_volume_value(linear_volume, false);
    NMM::Event event("IVolumeControl::setLinearVolume", &linear_volume_value);
    event.setReturnValue(&return_value);
    if(const_cast<IVolumeControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Rational IVolumeControl::setLinearVolumeRelative(const Rational& relative_linear_volume)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Rational > return_value;
    NMM::TInValue<Rational > relative_linear_volume_value(relative_linear_volume, false);
    NMM::Event event("IVolumeControl::setLinearVolumeRelative", &relative_linear_volume_value);
    event.setReturnValue(&return_value);
    if(const_cast<IVolumeControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Rational IVolumeControl::getLinearVolume() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Rational > return_value;
    NMM::Event event("IVolumeControl::getLinearVolume");
    event.setReturnValue(&return_value);
    if(const_cast<IVolumeControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_double IVolumeControl::setLogarithmicVolume(const nmm_double& volume_in_db)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_double > return_value;
    NMM::TInValue<nmm_double > volume_in_db_value(volume_in_db, false);
    NMM::Event event("IVolumeControl::setLogarithmicVolume", &volume_in_db_value);
    event.setReturnValue(&return_value);
    if(const_cast<IVolumeControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_double IVolumeControl::setLogarithmicVolumeRelative(const nmm_double& relative_volume_in_db)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_double > return_value;
    NMM::TInValue<nmm_double > relative_volume_in_db_value(relative_volume_in_db, false);
    NMM::Event event("IVolumeControl::setLogarithmicVolumeRelative", &relative_volume_in_db_value);
    event.setReturnValue(&return_value);
    if(const_cast<IVolumeControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_double IVolumeControl::getLogarithmicVolume() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_double > return_value;
    NMM::Event event("IVolumeControl::getLogarithmicVolume");
    event.setReturnValue(&return_value);
    if(const_cast<IVolumeControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IVolumeControl::getLinearVolumeRange(Rational& min_linear_volume, Rational& max_linear_volume) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Rational > min_linear_volume_value(min_linear_volume);
    NMM::TOutValue<Rational > max_linear_volume_value(max_linear_volume);
    NMM::Event event("IVolumeControl::getLinearVolumeRange", &min_linear_volume_value, &max_linear_volume_value);
    if(const_cast<IVolumeControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    min_linear_volume = min_linear_volume_value.getRValue();
    max_linear_volume = max_linear_volume_value.getRValue();
  }
  
  void IVolumeControl::getLogarithmicVolumeRange(nmm_double& min_volume_in_db, nmm_double& max_volume_in_db) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_double > min_volume_in_db_value(min_volume_in_db);
    NMM::TOutValue<nmm_double > max_volume_in_db_value(max_volume_in_db);
    NMM::Event event("IVolumeControl::getLogarithmicVolumeRange", &min_volume_in_db_value, &max_volume_in_db_value);
    if(const_cast<IVolumeControl*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    min_volume_in_db = min_volume_in_db_value.getRValue();
    max_volume_in_db = max_volume_in_db_value.getRValue();
  }

  const char* IVolumeControl::IVolumeControl_type = "IVolumeControl";
  NMM::TConstructor<IVolumeControl > __IVolumeControl_prototype("IVolumeControl","NMM");


  const char* IVolumeControl::setLinearVolume_event = "IVolumeControl::setLinearVolume";
  const char* IVolumeControl::setLinearVolumeRelative_event = "IVolumeControl::setLinearVolumeRelative";
  const char* IVolumeControl::getLinearVolume_event = "IVolumeControl::getLinearVolume";
  const char* IVolumeControl::setLogarithmicVolume_event = "IVolumeControl::setLogarithmicVolume";
  const char* IVolumeControl::setLogarithmicVolumeRelative_event = "IVolumeControl::setLogarithmicVolumeRelative";
  const char* IVolumeControl::getLogarithmicVolume_event = "IVolumeControl::getLogarithmicVolume";
  const char* IVolumeControl::getLinearVolumeRange_event = "IVolumeControl::getLinearVolumeRange";
  const char* IVolumeControl::getLogarithmicVolumeRange_event = "IVolumeControl::getLogarithmicVolumeRange";


}


#include "nmm/nmm_suffix.hpp"

