/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IAudioMux.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/serialize/SerializeUtilsTime.hpp"
#include "nmm/utils/time/Time.hpp"

namespace NMM
{
  void IAudioMux::used() {} 
  
  
  Time IAudioMux::getCurrentSyncTime()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Time > return_value;
    NMM::Event event("IAudioMux::getCurrentSyncTime");
    event.setReturnValue(&return_value);
    if(const_cast<IAudioMux*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IAudioMux::setVolumeControl(const string& input_jack_tag, const Time& start_time, const Time& end_time, const nmm_int32& start_value, const nmm_int32& end_value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > input_jack_tag_value(input_jack_tag, false);
    NMM::TInValue<Time > start_time_value(start_time, false);
    NMM::TInValue<Time > end_time_value(end_time, false);
    NMM::TInValue<nmm_int32 > start_value_value(start_value, false);
    NMM::TInValue<nmm_int32 > end_value_value(end_value, false);
    NMM::Event event("IAudioMux::setVolumeControl", &input_jack_tag_value, &start_time_value, &end_time_value, &start_value_value, &end_value_value);
    if(const_cast<IAudioMux*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_int32 IAudioMux::setVolume(const string& input_jack_tag, const nmm_int32& volume)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::TInValue<string > input_jack_tag_value(input_jack_tag, false);
    NMM::TInValue<nmm_int32 > volume_value(volume, false);
    NMM::Event event("IAudioMux::setVolume", &input_jack_tag_value, &volume_value);
    event.setReturnValue(&return_value);
    if(const_cast<IAudioMux*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 IAudioMux::getVolume(const string& input_jack_tag)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::TInValue<string > input_jack_tag_value(input_jack_tag, false);
    NMM::Event event("IAudioMux::getVolume", &input_jack_tag_value);
    event.setReturnValue(&return_value);
    if(const_cast<IAudioMux*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IAudioMux::IAudioMux_type = "IAudioMux";
  NMM::TConstructor<IAudioMux > __IAudioMux_prototype("IAudioMux","NMM");


  const char* IAudioMux::getCurrentSyncTime_event = "IAudioMux::getCurrentSyncTime";
  const char* IAudioMux::setVolumeControl_event = "IAudioMux::setVolumeControl";
  const char* IAudioMux::setVolume_event = "IAudioMux::setVolume";
  const char* IAudioMux::getVolume_event = "IAudioMux::getVolume";


}


#include "nmm/nmm_suffix.hpp"

