/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IAudioMuxImpl_HPP
#define NMM_IAudioMuxImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IAudioMux.hpp"


#include "visibility.hpp"

namespace NMM
{
  /** Audio mux interface. */

  class DLLEXPORT IAudioMuxImpl  : public virtual NMM::NMMObject { 
  public:
    IAudioMuxImpl() { addInterface(new IAudioMux(*this)); registerEvents(); };

    virtual ~IAudioMuxImpl() {};

  protected:
    /**
     * Returns the current sync_time of the latest timestamp in the media stream
     */

    virtual Time getCurrentSyncTime() = 0;
    /**
     * Sets start/end time and start/end value for volume control for specified input
     */

    virtual void setVolumeControl(const string& input_jack_tag, const Time& start_time, const Time& end_time, const nmm_int32& start_value, const nmm_int32& end_value) = 0;
    /**
     * Sets the current volume for specified input
     */

    virtual nmm_int32 setVolume(const string& input_jack_tag, const nmm_int32& volume) = 0;
    /**
     * Returns the current volume for specified input
     */

    virtual nmm_int32 getVolume(const string& input_jack_tag) = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
