/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#ifndef NMM_OGGVORBISDECODENODE_HPP
#define NMM_OGGVORBISDECODENODE_HPP NMM_OGGVORBISDECODENODE_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/multimedia/GenericProcessorNode.hpp"

#include "nmm/plugins/interfaces/ITrackSizeImpl.hpp"
#include "nmm/multimedia/ISyncResetImpl.hpp"
#include "nmm/plugins/audio/oggvorbis/IOggVorbisImpl.hpp"

#include <vorbis/vorbisfile.h>

namespace NMM {

  class OggVorbisDecodeNode
    : public ISyncResetImpl,
      public IOggVorbisImpl,
      public ITrackSizeImpl,
      public GenericProcessorNode {
  private:
    Buffer* read_buffer;
    size_t bytes_read;
    bool ov_initialized;
    Buffer* merge_buffer;
    OggVorbis_File oggvorbis;
    size_t out_buffer_size;
    string info_user_comments;
    string info_vendor;
    Timestamp timestamp;
    Timestamp in_timestamp;
    int64_t old_pos;
    int sample_rate;
    int channels;
    string title,album,artist,year,genre;
    u_int64_t track_size;
    bool send_track_duration;

    size_t readFunc(void*, size_t, size_t);
    static size_t readFuncOggStruct(void* ptr, size_t size, size_t nmemb, void* datasource) { 
      return reinterpret_cast<OggVorbisDecodeNode*>(datasource)->readFunc(ptr, size, nmemb); 
    }

    int seekFunc(ogg_int64_t, int);
    static int seekFuncOggStruct(void* datasource, ogg_int64_t offset, int whence) { 
      return reinterpret_cast<OggVorbisDecodeNode*>(datasource)->seekFunc(offset, whence); 
    }

    int closeFunc();
    static int closeFuncOggStruct(void* datasource) { 
      return reinterpret_cast<OggVorbisDecodeNode*>(datasource)->closeFunc(); 
    }

    long int tellFunc();
    static long int tellFuncOggStruct(void* datasource) { 
      return reinterpret_cast<OggVorbisDecodeNode*>(datasource)->tellFunc(); 
    }

    bool initOggVorbis();

  protected:

    Result doDeactivate();
    Result doInit();
    Result doInitOutput();
    
    //from ISyncResetImpl
    virtual Result syncReset();
    
    //from IOggVorbisImpl
    string getUserComments();
    string getVendor();

    //from ITrackSizeImpl
    Result trackSize(const nmm_uint64& bytes);
    
   public:
    OggVorbisDecodeNode(const char* = "OggVorbisDecodeNode", 
			StreamQueue::Mode = StreamQueue::MODE_SUSPEND, 
			int = 20);

    ~OggVorbisDecodeNode();

    Message* processBuffer(Buffer*);

  };

}

#include "nmm/nmm_suffix.hpp"

#endif //NMM_OGGVORBISDECODENODE_HPP



