/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IFragmentSize_HPP
#define NMM_IFragmentSize_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "nmm/multimedia/INode.hpp"

#include "visibility.hpp"

namespace NMM
{

  class DLLEXPORT IFragmentSize : virtual public NMM::Interface { 
  public:
    IFragmentSize() : NMM::Interface(IFragmentSize::IFragmentSize_type, "NMM") { };

    IFragmentSize(NMM::NMMObject& parent) : NMM::Interface (parent, IFragmentSize::IFragmentSize_type, "NMM") { };

    virtual ~IFragmentSize() {};

    static const char* IFragmentSize_type;

    virtual  string getInterfaceType() const { return IFragmentSize_type; }

    NMM::Interface* copy() const { return new IFragmentSize(*this); }
    static void used(); 
    static const char* getFragmentSize_event;
    static const char* setFragmentSize_event;
    /**
     * Returns FragmentSize in bytes
     */

    virtual nmm_int32 getFragmentSize() /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /*
     * Tries to set 2^value as FragmentSize in bytes 
     */

    virtual void setFragmentSize(const nmm_int32& value) /* limited(INode::CONSTRUCTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);


  };

  typedef auto_ptr<IFragmentSize> IFragmentSize_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
