/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "PCMEncodeNode.hpp"
#include "nmm/plugins/NMMRegisterPlugin.hpp"
#include <iostream>

NMM_REGISTER_PLUGIN(PCMEncodeNode, NMM, "PCMEncodeNode")

#include "g711/g711.h"

#include "nmm/nmm_prefix.hpp"

  namespace NMM {

  

    PCMEncodeNode::PCMEncodeNode(const char* name /*="PCMEncodeNode"*/,
                                 const StreamQueue::Mode queue_mode /*= StreamQueue::MODE_SUSPEND*/,
                                 const int queue_size /*= StreamQueue::DEFAULT_SIZE*/) :
      GenericProcessorNode(name,queue_mode,queue_size) {
      codec = PCMU;
    } // PCMEncodeNode::PCMEncodeNode  


    PCMEncodeNode::~PCMEncodeNode(void) {
    }  // PCMEncodeNode::~PCMEncodeNode

    Result PCMEncodeNode::doInit(void) {
      // input format -> raw audio samples
      Format *in_format = getOwnInputProperty()->addNewFormat(Format::audio_raw_type);
      in_format->addIntParamValue(Format::sample_rate_param, 44100);
      in_format->addIntParamValue(Format::channels_param, 2);
      in_format->addIntParamValue(Format::bitpersample_param, 16);
      in_format->addStringParamValue(Format::endian_param, "little");
      in_format->addStringParamValue(Format::encoding_param, "linear");
      in_format->addStringParamValue(Format::sign_param, "signed");
    
      Format *in_format2 = getOwnInputProperty()->addNewFormat(Format::audio_raw_type);
      in_format2->addIntParamValue(Format::sample_rate_param, 8000);
      in_format2->addIntParamValue(Format::channels_param, 1);
      in_format2->addIntParamValue(Format::bitpersample_param,  16);
      in_format2->addStringParamValue(Format::endian_param, "little");
      in_format2->addStringParamValue(Format::encoding_param, "linear");
      in_format2->addStringParamValue(Format::sign_param, "signed");
      getOwnInputProperty()->setDefaultFormat(in_format2);

      Format *in_format3 = getOwnInputProperty()->addNewFormat(Format::audio_raw_type);
      in_format3->addIntParamValue(Format::sample_rate_param, 48000);
      in_format3->addIntParamValue(Format::channels_param, 2);
      in_format3->addIntParamValue(Format::bitpersample_param, 16);
      in_format3->addStringParamValue(Format::endian_param, "little");
      in_format3->addStringParamValue(Format::encoding_param, "linear");
      in_format3->addStringParamValue(Format::sign_param, "signed");

      // output format -> PCMU
      Format *out_format = getOwnOutputProperty()->addNewFormat(Format::audio_raw_type);
      out_format->addIntParamValue(Format::sample_rate_param, 8000);
      out_format->addIntParamValue(Format::channels_param, 1);
      out_format->addIntParamValue(Format::bitpersample_param, 8);
      out_format->addStringParamValue(Format::endian_param, "little");
      out_format->addStringParamValue(Format::encoding_param, "mu_law");
      out_format->addStringParamValue(Format::sign_param, "signed");
      getOwnOutputProperty()->setDefaultFormat(out_format);

      Format *out_format2 = getOwnOutputProperty()->addNewFormat(Format::audio_raw_type);
      out_format2->addIntParamValue(Format::sample_rate_param, 44100);
      out_format2->addIntParamValue(Format::channels_param, 2);
      out_format2->addIntParamValue(Format::bitpersample_param, 8);
      out_format2->addStringParamValue(Format::endian_param, "little");
      out_format2->addStringParamValue(Format::encoding_param, "mu_law");
      out_format2->addStringParamValue(Format::sign_param, "signed");

      Format *out_format3 = getOwnOutputProperty()->addNewFormat(Format::audio_raw_type);
      out_format3->addIntParamValue(Format::sample_rate_param, 48000);
      out_format3->addIntParamValue(Format::channels_param, 2);
      out_format3->addIntParamValue(Format::bitpersample_param, 8);
      out_format3->addStringParamValue(Format::endian_param, "little");
      out_format3->addStringParamValue(Format::encoding_param, "mu_law");
      out_format3->addStringParamValue(Format::sign_param, "signed");

      // output format -> PCMA
      Format *out_format4 = getOwnOutputProperty()->addNewFormat(Format::audio_raw_type);
      out_format4->addIntParamValue(Format::sample_rate_param, 8000);
      out_format4->addIntParamValue(Format::channels_param, 1);
      out_format4->addIntParamValue(Format::bitpersample_param, 8);
      out_format4->addStringParamValue(Format::endian_param, "little");
      out_format4->addStringParamValue(Format::encoding_param, "a_law");
      out_format4->addStringParamValue(Format::sign_param, "signed");
      
      Format *out_format5 = getOwnOutputProperty()->addNewFormat(Format::audio_raw_type);
      out_format5->addIntParamValue(Format::sample_rate_param, 44100);
      out_format5->addIntParamValue(Format::channels_param, 2);
      out_format5->addIntParamValue(Format::bitpersample_param, 8);
      out_format5->addStringParamValue(Format::endian_param, "little");
      out_format5->addStringParamValue(Format::encoding_param, "a_law");
      out_format5->addStringParamValue(Format::sign_param, "signed");

      Format *out_format6 = getOwnOutputProperty()->addNewFormat(Format::audio_raw_type);
      out_format6->addIntParamValue(Format::sample_rate_param, 48000);
      out_format6->addIntParamValue(Format::channels_param, 2);
      out_format6->addIntParamValue(Format::bitpersample_param, 8);
      out_format6->addStringParamValue(Format::endian_param, "little");
      out_format6->addStringParamValue(Format::encoding_param, "a_law");
      out_format6->addStringParamValue(Format::sign_param, "signed");

      return SUCCESS;
    } // PCMEncodeNode::doInit
  

    Result PCMEncodeNode::doDeinit(void) {
      getOwnInputProperty()->erase();
      getOwnOutputProperty()->erase();

      return SUCCESS;
    } // PCMEncodeNode::doDeinit
  

    Result PCMEncodeNode::doInitOutput(void) {
      const Format *in_format = getInputFormat();
      if (!in_format) { 
        ERROR_STREAM("no input format specified" << std::endl);
        return FAILURE; 
      }

      Format *i_format = new Format(*in_format);
      getOwnWorkingInputProperty()->addNewFormat(i_format);
      getOwnWorkingInputProperty()->setDefaultFormat(i_format);

      int channels = i_format->getIntValue(Format::channels_param);
      int sample_rate = i_format->getIntValue(Format::sample_rate_param);
      
      // output format -> pcmu
      Format *out_format = getOwnWorkingOutputProperty()->addNewFormat(Format::audio_raw_type);
      out_format->addIntParamValue(Format::sample_rate_param, sample_rate);
      out_format->addIntParamValue(Format::channels_param, channels);
      out_format->addIntParamValue(Format::bitpersample_param, 8);
      out_format->addStringParamValue(Format::endian_param, "little");
      out_format->addStringParamValue(Format::encoding_param, "mu_law");
      out_format->addStringParamValue(Format::sign_param, "signed");
      getOwnWorkingOutputProperty()->setDefaultFormat(out_format);

      Format *out_format2 = getOwnWorkingOutputProperty()->addNewFormat(Format::audio_raw_type);
      out_format2->addIntParamValue(Format::sample_rate_param, sample_rate);
      out_format2->addIntParamValue(Format::channels_param, channels);
      out_format2->addIntParamValue(Format::bitpersample_param, 8);
      out_format2->addStringParamValue(Format::endian_param, "little");
      out_format2->addStringParamValue(Format::encoding_param, "a_law");
      out_format2->addStringParamValue(Format::sign_param, "signed");
      

      DEBUG_STREAM(endl << "PCMEncodeNode WorkingOutputProperty: " << 
                   endl << "sample_rate = " << sample_rate << 
                   endl << "channels = " << channels << 
                   endl << endl);

      i_format->setIOPartner(out_format);
      out_format->setIOPartner(i_format);
      i_format->setIOPartner(out_format2);
      out_format2->setIOPartner(i_format);

      return SUCCESS;
    } // PCMEncodeNode::doInitOutput
  

    Result PCMEncodeNode::doDeinitOutput(void) {

      getWorkingInputProperty()->erase();
      getWorkingOutputProperty()->erase();
      return SUCCESS;
    } // PCMEncodeNode::doDeinitOutput
  

    Result PCMEncodeNode::doActivate(void) {
      const Format *out_format = getOutputFormat();
      if (!out_format) { 
        ERROR_STREAM("no output format specified" << std::endl);
        return FAILURE; 
      }
    
      string codec_string = out_format->getStringValue(Format::encoding_param);
      
      if(codec_string == "mu_law"){
        codec = PCMU;
        cout << "Using PCMUUUUUUUUUUU Encoding!" << endl;
      }else if(codec_string == "a_law"){
        codec = PCMA;
        cout << "Using PCMAAAAAAAAAAA Encoding!" << endl;
      }else{
        ERROR_STREAM("Unknown encoding_param " << codec_string << " in input format!" << endl);
      }
     
      return SUCCESS;
    } // PCMEncodeNode::doActivate
  

    Result PCMEncodeNode::doDeactivate(void) {
    
      return SUCCESS;
    } //PCMEncodeNode::doDeactivate 
  

    Message* PCMEncodeNode::processBuffer(Buffer *in_buffer) {
      if (!in_buffer) { 
        return 0; 
      }
    
      Buffer* out;
    
      int in_buffer_size = in_buffer->getUsed();
      //int out_size = out->getSize();
      //DEBUG_STREAM("in_buffer->getUsed() = " << in_buffer_size << endl);
      short *samples = NULL;
      unsigned char *dst = NULL;
      //DEBUG_STREAM("Using G.711 encoding." << endl);
      out = getNewBuffer(in_buffer_size,in_buffer);
      // ERROR_STREAM("out->getSize() = " << out_size << endl);
      // ERROR_STREAM("out->getData() = " << out->getData() << endl <<  "in_buffer->getData() " << in_buffer->getData() << endl);
      samples = reinterpret_cast<short*>(in_buffer->getData());
      //cout << samples << " *samples: " << *samples << endl;
      dst = reinterpret_cast<unsigned char*>(out->getData());
      //cout << dst << " *dst: " << *dst << endl;
      switch(codec){
      case PCMU:
        //cout << "Encoding to pcmu " << endl;
        for(int i=0;i<in_buffer_size;){
          //unsigned char frame = 
          short value = *samples;
          //ERROR_STREAM("actual encoding sample with value " << value << endl);
          unsigned char sample = linear2ulaw(value);
          dst[0]=sample;
          dst++;
          samples++;
          i+=2;
        }
        break;
      case PCMA:
        //cout << "Encoding to pcma " << endl;
        for(int i=0;i<in_buffer_size;){
          //unsigned char frame = 
          short value = *samples;
          //ERROR_STREAM("actual encoding sample with value " << value << endl);
          unsigned char sample = linear2alaw(value);
          dst[0]=sample;
          dst++;
          samples++;
          i+=2;
        }
        break;
      }
      out->setUsed(in_buffer_size/2);
      //DEBUG_STREAM("Setting used value of out_buffer to " << in_buffer_size/2 << endl);
    
      in_buffer->release();

      return out;
    } // PCMEncodeNode::processBuffer 

#include "nmm/nmm_prefix.hpp"

  }; //namespace NMM

#include "nmm/nmm_suffix.hpp"
