/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#ifndef NMM_PCMENCODENODE_HPP
#define NMM_PCMENCODENODE_HPP NMM_PCMENCODENODE_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/multimedia/GenericProcessorNode.hpp"

namespace NMM {

  /**
   * @brief Encode audio/raw with mu-law or a-law.
   * Encode audio/raw with mu-law or a-law.
   * Format to use is estimated by connection format.
   * If no connection format is given, mu-law encoding ist used.
   */
  class PCMEncodeNode : 
    public GenericProcessorNode {
  private:
    enum codec_type{PCMU, PCMA};
    codec_type codec;
    
  public:
    PCMEncodeNode(const char* name="PCMEncodeNode",
			  const StreamQueue::Mode queue_mode = StreamQueue::MODE_SUSPEND,
			  const int queue_size = StreamQueue::DEFAULT_SIZE);
    ~PCMEncodeNode(void);

  protected:
    //+++ INode +++
    virtual Result doInit(void);
    virtual Result doDeinit(void);
    virtual Result doInitOutput(void);
    virtual Result doDeinitOutput(void);
    virtual Result doActivate(void);
    virtual Result doDeactivate(void);
    virtual Message* processBuffer(Buffer *in_buffer);

  }; 

}; // namespace NMM

#include "nmm/nmm_suffix.hpp"

#endif // NMM_PCMUENCODENODE_HPP
