/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IScope_HPP
#define NMM_IScope_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{

  class DLLEXPORT IScope : virtual public NMM::Interface { 
  public:
    IScope() : NMM::Interface(IScope::IScope_type, "NMM") { };

    IScope(NMM::NMMObject& parent) : NMM::Interface (parent, IScope::IScope_type, "NMM") { };

    virtual ~IScope() {};

    static const char* IScope_type;

    virtual  string getInterfaceType() const { return IScope_type; }

    NMM::Interface* copy() const { return new IScope(*this); }
    static void used(); 
    static const char* setBlurEnabled_event;
    static const char* setBlurStep_event;
    /**
     * this method enables or disables the blur effect.
     */

    virtual Result setBlurEnabled(const nmm_bool& toggle)
      throw(const NMM::SerializedException&);
    /**
     * sets the intensity of the blur effect.
     */

    virtual Result setBlurStep(const nmm_int32& value)
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IScope> IScope_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
