/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IMPEGDemux.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IMPEGDemux::used() {} 
  
  
  void IMPEGDemux::setVideoStream(const nmm_int32& stream)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > stream_value(stream, false);
    NMM::Event event("IMPEGDemux::setVideoStream", &stream_value);
    if(const_cast<IMPEGDemux*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMPEGDemux::setAudioStream(const nmm_int32& stream)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > stream_value(stream, false);
    NMM::Event event("IMPEGDemux::setAudioStream", &stream_value);
    if(const_cast<IMPEGDemux*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMPEGDemux::setSPUStream(const nmm_int32& stream)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > stream_value(stream, false);
    NMM::Event event("IMPEGDemux::setSPUStream", &stream_value);
    if(const_cast<IMPEGDemux*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMPEGDemux::setDVDMode(const nmm_bool& stream)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > stream_value(stream, false);
    NMM::Event event("IMPEGDemux::setDVDMode", &stream_value);
    if(const_cast<IMPEGDemux*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IMPEGDemux::IMPEGDemux_type = "IMPEGDemux";
  NMM::TConstructor<IMPEGDemux > __IMPEGDemux_prototype("IMPEGDemux","NMM");


  const char* IMPEGDemux::setVideoStream_event = "IMPEGDemux::setVideoStream";
  const char* IMPEGDemux::setAudioStream_event = "IMPEGDemux::setAudioStream";
  const char* IMPEGDemux::setSPUStream_event = "IMPEGDemux::setSPUStream";
  const char* IMPEGDemux::setDVDMode_event = "IMPEGDemux::setDVDMode";


}


#include "nmm/nmm_suffix.hpp"

