/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IMPEGDemux_HPP
#define NMM_IMPEGDemux_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{

  class DLLEXPORT IMPEGDemux : virtual public NMM::Interface { 
  public:
    IMPEGDemux() : NMM::Interface(IMPEGDemux::IMPEGDemux_type, "NMM") { };

    IMPEGDemux(NMM::NMMObject& parent) : NMM::Interface (parent, IMPEGDemux::IMPEGDemux_type, "NMM") { };

    virtual ~IMPEGDemux() {};

    static const char* IMPEGDemux_type;

    virtual  string getInterfaceType() const { return IMPEGDemux_type; }

    NMM::Interface* copy() const { return new IMPEGDemux(*this); }
    static void used(); 
    static const char* setVideoStream_event;
    static const char* setAudioStream_event;
    static const char* setSPUStream_event;
    static const char* setDVDMode_event;

    virtual void setVideoStream(const nmm_int32& stream)
      throw(const NMM::SerializedException&);

    virtual void setAudioStream(const nmm_int32& stream)
      throw(const NMM::SerializedException&);

    virtual void setSPUStream(const nmm_int32& stream)
      throw(const NMM::SerializedException&);

    virtual void setDVDMode(const nmm_bool& stream)
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IMPEGDemux> IMPEGDemux_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
