/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "MPEGDemuxNode.hpp"
#include "nmm/plugins/NMMRegisterPlugin.hpp"

// #include "nmm/comm/format/Property.hpp"
// #include "nmm/multimedia/OutputJack.hpp"
// #include "nmm/comm/messaging/Buffer.hpp"
// #include "nmm/multimedia/Plugin.hpp"


NMM_REGISTER_PLUGIN(MPEGDemuxNode, NMM, "MPEGDemuxNode")


#include "nmm/nmm_prefix.hpp"

namespace NMM {

  MPEGDemuxNode::MPEGDemuxNode(const char *name, StreamQueue::Mode qmode, int qsize) 
    : GenericDemultiplexerNode(name, qmode, qsize),
      analysed_buffers(0),
      mpeg_version(0) {
    
    tmpbuffer     = NULL;
    fragment_size = 0xffff;
    inbuffer_size = fragment_size;
    rest_length   = 0;

    skip_audio = false;
    send_event = false;
    
    syncReset();
    
    video_stream_number = 0;
    audio_stream_number = 0;
    spu_stream_number   = 0;
    dvd_mode            = false;

    temp = 0;
  }
  
  MPEGDemuxNode::~MPEGDemuxNode() {
  }
  
  Result MPEGDemuxNode::doInit() {
    Format* format;
    Property* property = getOwnInputProperty();
    
    // init input property
    // simple mpeg format
    format = getOwnInputProperty()->addNewFormat(Format::av_mpegps_type);

    format = getOwnInputProperty()->addNewFormat(Format::av_mpegps_type);
    format->addIntParamValue(Format::version_param, 1);
    format->addIntParamValue(Format::version_param, 2);
    
    // detailed mpeg format
    format = property->addNewFormat(Format::av_mpegps_type);
    format->addIntParamValue(Format::version_param, 1);
    format->addIntParamValue(Format::version_param, 2);
    // maps of available streams for each stream type
    format->addWildcard(Format::map_mpeg_audio_streams_param);
    format->addWildcard(Format::map_ac3_audio_streams_param);
    format->addWildcard(Format::map_mpeg_video_streams_param);
    format->addWildcard(Format::map_dts_audio_streams_param);
    format->addWildcard(Format::map_lpcm_audio_streams_param);
    format->addWildcard(Format::map_spu_streams_param);

    // formats for every stream type
    Format* sub_format = new Format(Format::audio_mpeg_type);
    sub_format->addIntParamValue(Format::audio_codec_param, 0x55);
    sub_format->addWildcard(Format::layer_param);
    sub_format->addIntParamValue(Format::channels_param, 1);
    sub_format->addIntParamValue(Format::channels_param, 2);
    sub_format->addIntParamValue(Format::sample_rate_param, 8000);
    sub_format->addIntParamValue(Format::sample_rate_param, 11025);
    sub_format->addIntParamValue(Format::sample_rate_param, 12000);
    sub_format->addIntParamValue(Format::sample_rate_param, 16000);
    sub_format->addIntParamValue(Format::sample_rate_param, 22050);
    sub_format->addIntParamValue(Format::sample_rate_param, 44100);
    sub_format->addIntParamValue(Format::sample_rate_param, 48000);
    sub_format->addIntParamValue(Format::bitpersample_param, 8);
    sub_format->addIntParamValue(Format::bitpersample_param, 16);
    format->addFPointerParamValue(Format::format_mpeg_audio_streams_param, sub_format);
    sub_format = new Format(Format::audio_mpeg_type);
    sub_format->addIntParamValue(Format::audio_codec_param, 0x55);
    sub_format->addWildcard(Format::layer_param);
    sub_format->addWildcard(Format::bitrate_param);
    sub_format->addIntParamValue(Format::channels_param, 1);
    sub_format->addIntParamValue(Format::channels_param, 2);
    sub_format->addIntParamValue(Format::sample_rate_param, 8000);
    sub_format->addIntParamValue(Format::sample_rate_param, 11025);
    sub_format->addIntParamValue(Format::sample_rate_param, 12000);
    sub_format->addIntParamValue(Format::sample_rate_param, 16000);
    sub_format->addIntParamValue(Format::sample_rate_param, 22050);
    sub_format->addIntParamValue(Format::sample_rate_param, 44100);
    sub_format->addIntParamValue(Format::sample_rate_param, 48000);
    sub_format->addIntParamValue(Format::bitpersample_param, 8);
    sub_format->addIntParamValue(Format::bitpersample_param, 16);
    format->addFPointerParamValue(Format::format_mpeg_audio_streams_param, sub_format);
    sub_format = new Format(Format::audio_mpeg_type);
    sub_format->addIntParamValue(Format::audio_codec_param, 0x55);
    sub_format->addWildcard(Format::layer_param);
    sub_format->addWildcard(Format::bitrate_param);
    format->addFPointerParamValue(Format::format_mpeg_audio_streams_param, sub_format);
    sub_format = new Format(Format::audio_mpeg_type);
    sub_format->addIntParamValue(Format::audio_codec_param, 0x55);
    sub_format->addWildcard(Format::layer_param);
    format->addFPointerParamValue(Format::format_mpeg_audio_streams_param, sub_format);

    sub_format = new Format("audio/ac3");
    sub_format->addWildcard(Format::bitrate_param);
    format->addFPointerParamValue(Format::format_ac3_audio_streams_param, sub_format);
    sub_format = new Format("audio/ac3");
    format->addFPointerParamValue(Format::format_ac3_audio_streams_param, sub_format);

    sub_format = new Format("video/mpeg");
    sub_format->addIntParamValue(Format::version_param, 1);
    sub_format->addIntParamValue(Format::version_param, 2);
    sub_format->addWildcard(Format::x_resolution_param);
    sub_format->addWildcard(Format::y_resolution_param);
    sub_format->addWildcard(Format::ratio_x_param);
    sub_format->addWildcard(Format::ratio_y_param);
    sub_format->addWildcard(Format::framerate_param);
    format->addFPointerParamValue(Format::format_mpeg_video_streams_param, sub_format);
    sub_format = new Format("video/mpeg");
    sub_format->addIntParamValue(Format::version_param, 1);
    sub_format->addIntParamValue(Format::version_param, 2);
    format->addFPointerParamValue(Format::format_mpeg_video_streams_param, sub_format);

    sub_format = new Format("audio/dts");
    format->addFPointerParamValue(Format::format_dts_audio_streams_param, sub_format);

    sub_format = new Format("audio/lpcm");
    format->addFPointerParamValue(Format::format_lpcm_audio_streams_param, sub_format);

    sub_format = new Format("video/spu");
    format->addFPointerParamValue(Format::format_spu_streams_param, sub_format);
    
    property->setDefaultFormat(format);
    
    // init output property
    property = getOwnOutputProperty();
    
    unsigned int stream;
    for( stream=0; stream<MAX_MPEG_AUDIO; stream++ ) {
      mpeg_audio_string[stream] = string("mpeg_audio") + stream;
      format = property->addNewFormat("audio/mpeg");
      format->addIntParamValue(Format::audio_codec_param, 0x55);
      format->addIntParamValue(Format::layer_param, 2);
      format->setJackTag(mpeg_audio_string[stream]);
      property->setDefaultFormat(format, mpeg_audio_string[stream]);

      // also offer a detailed format
      format = property->addNewFormat("audio/mpeg");
      format->addIntParamValue(Format::audio_codec_param, 0x55);
      format->addIntParamValue(Format::layer_param, 2);
      format->addIntParamValue(Format::channels_param, 1);
      format->addIntParamValue(Format::channels_param, 2);
      format->addIntParamValue(Format::sample_rate_param, 8000);
      format->addIntParamValue(Format::sample_rate_param, 11025);
      format->addIntParamValue(Format::sample_rate_param, 12000);
      format->addIntParamValue(Format::sample_rate_param, 16000);
      format->addIntParamValue(Format::sample_rate_param, 22050);
      format->addIntParamValue(Format::sample_rate_param, 44100);
      format->addIntParamValue(Format::sample_rate_param, 48000);
      format->addIntParamValue(Format::bitpersample_param, 8);
      format->addIntParamValue(Format::bitpersample_param, 16);
      format->setJackTag(mpeg_audio_string[stream]);
    }
    
    for( stream=0; stream<MAX_AC3_AUDIO; stream++ ) {
      ac3_audio_string[stream] = string("ac3_audio") + stream;
      format = property->addNewFormat("audio/ac3");
      format->setJackTag(ac3_audio_string[stream] );
      property->setDefaultFormat(format, ac3_audio_string[stream]);
    }
    
    for( stream=0; stream<MAX_MPEG_VIDEO; stream++ ) {
      mpeg_video_string[stream] = string("mpeg_video") + stream;
      format = property->addNewFormat("video/mpeg");
      format->addIntParamValue(Format::version_param, 1);
      format->addIntParamValue(Format::version_param, 2);
      format->setJackTag(mpeg_video_string[stream]);
      property->setDefaultFormat(format, mpeg_video_string[stream]);

      // also offer a detailed format
      format = property->addNewFormat("video/mpeg");
      format->addIntParamValue(Format::version_param, 1);
      format->addIntParamValue(Format::version_param, 2);
      format->addWildcard(Format::x_resolution_param);
      format->addWildcard(Format::y_resolution_param);
      format->addWildcard(Format::ratio_x_param);
      format->addWildcard(Format::ratio_y_param);
      format->addWildcard(Format::framerate_param);
      format->setJackTag(mpeg_video_string[stream]);
    }
  
    for( stream=0; stream<MAX_SPU; stream++ ) {
      spu_string[stream] = string("spu") + stream;
      format = property->addNewFormat("video/spu");
      format->setJackTag(spu_string[stream]);
      property->setDefaultFormat(format, spu_string[stream]);
    }
    
    return SUCCESS;
  }
  
  Result MPEGDemuxNode::doInitOutput() {
    // get input format
    const Format* in_format = getInputFormat();
    if (in_format == NULL) { 
      ERROR_STREAM("InputFormat is NULL!" << endl);
      return FAILURE; 
    }
    
    // init working input property
    Format* i_format = new Format(*in_format);
    getOwnWorkingInputProperty()->addNewFormat(i_format);
    getOwnWorkingInputProperty()->setDefaultFormat(i_format);
    
    Format* format_mpeg_audio = NULL;
    Format* format_mpeg_audio2 = NULL;
    Format* format_ac3_audio = NULL;
    // the following 2 lines are there for the case, that this Node support DTS or LPCM audio
//     Format* format_dts_audio = NULL;
//     Format* format_lpcm_audio = NULL;
    Format* format_mpeg_video = NULL;
    Format* format_mpeg_video2 = NULL;
    Format* format_spu = NULL;
    
    // get mpeg version
    if (in_format->isParameterIncluded(Format::version_param)) {
      mpeg_version = in_format->getIntValue(Format::version_param);
    }

    // get input format parameters
    if (in_format->isParameterIncluded(Format::map_mpeg_audio_streams_param)) {
      
      int map_mpeg_audio;
      int map_ac3_audio;
      // the following 2 lines are there for the case, that this Node support DTS or LPCM audio
//       int map_dts_audio;
//       int map_lpcm_audio;
      int map_mpeg_video;
      int map_spu;

      try { // maybe a param is not what it should be!
	// detailed format, take the values
	map_mpeg_audio = in_format->getIntValue(Format::map_mpeg_audio_streams_param);
	map_ac3_audio  = in_format->getIntValue(Format::map_ac3_audio_streams_param); 
	// the following 2 lines are there for the case, that this Node support DTS or LPCM audio
// 	map_dts_audio  = in_format->getIntValue(Format::map_ac3_audio_streams_param); 
// 	map_lpcm_audio = in_format->getIntValue(Format::map_ac3_audio_streams_param); 
	map_mpeg_video = in_format->getIntValue(Format::map_mpeg_video_streams_param);
	map_spu        = in_format->getIntValue(Format::map_spu_streams_param);

	format_mpeg_audio = in_format->getFPointerValue(Format::format_mpeg_audio_streams_param); 
	format_ac3_audio  = in_format->getFPointerValue(Format::format_ac3_audio_streams_param); 
	// the following 2 lines are there for the case, that this Node support DTS or LPCM audio
// 	format_dts_audio  = in_format->getFPointerValue(Format::format_ac3_audio_streams_param); 
// 	format_lpcm_audio = in_format->getFPointerValue(Format::format_ac3_audio_streams_param); 
	format_mpeg_video = in_format->getFPointerValue(Format::format_mpeg_video_streams_param); 
	format_spu        = in_format->getFPointerValue(Format::format_spu_streams_param);

      } catch (const FormatException& fe) {
	ERROR_STREAM("Error reading a param from the detailed input format!" << endl);
	ERROR_STREAM(fe << endl);
	return FAILURE;
      } catch (...) {
	ERROR_STREAM("Something strange went wrong while reading the detailed input format!" << endl);
	return FAILURE;
      }
      
      if (dvd_mode) {
        
        // in dvd mode we only have three output jacks
        addOutputStream(mpeg_video_string[0].c_str());
        addOutputStream(ac3_audio_string[0].c_str());
        addOutputStream(spu_string[0].c_str());
        
      } else {
        
        /* Begin> add output streams for all stream of each type available */
        int stream_mask;
        string current_jack_tag("");
      
        stream_mask = 1;
        for (unsigned int stream=0; stream<MAX_MPEG_AUDIO; stream++) {
          if (map_mpeg_audio & stream_mask) {
            current_jack_tag = mpeg_audio_string[stream];
            addOutputStream(current_jack_tag.c_str());
          }
          stream_mask = stream_mask << 1;
        }

        stream_mask = 1;
        for (unsigned int stream=0; stream<MAX_AC3_AUDIO; stream++) {
          if (map_ac3_audio & stream_mask) {
            current_jack_tag = ac3_audio_string[stream];
            addOutputStream(current_jack_tag.c_str());
          }
          stream_mask = stream_mask << 1;
        }

	// the following 2 blocks are there for the case, that this Node support DTS or LPCM audio
// 	stream_mask = 1;
// 	for (unsigned int stream=0; stream<MAX_DTS_AUDIO; stream++) {
// 	  current_jack_tag = dts_audio_string[stream];
// 	  if (map_dts_audio & stream_mask) {
// 	    addOutputStream(current_jack_tag.c_str());
// 	  }
// 	  stream_mask = stream_mask << 1;
// 	}

// 	stream_mask = 1;
// 	for (unsigned int stream=0; stream<MAX_LPCM_AUDIO; stream++) {
// 	  current_jack_tag = lpcm_audio_string[stream];
// 	  if (map_lpcm_audio & stream_mask) {
// 	    addOutputStream(current_jack_tag.c_str());
// 	  }
// 	  stream_mask = stream_mask << 1;
// 	}

        stream_mask = 1;
        for (unsigned int stream=0; stream<MAX_MPEG_VIDEO; stream++) {
          if (map_mpeg_video & stream_mask) {
	  current_jack_tag = mpeg_video_string[stream];
	  addOutputStream(current_jack_tag.c_str());
          }
          stream_mask = stream_mask << 1;
        }
        
        stream_mask = 1;
        for (unsigned int stream=0; stream<MAX_SPU; stream++) {
          if (map_spu & stream_mask) {
            current_jack_tag = spu_string[stream];
            addOutputStream(current_jack_tag.c_str());
          }
          stream_mask = stream_mask << 1;
        }
        /* End< add output streams for all stream of each type available */
      }
      
    } else {
      
      if (dvd_mode) {
        
        // in dvd mode we only have three output jacks
        addOutputStream(mpeg_video_string[0].c_str());
        addOutputStream(ac3_audio_string[0].c_str());
        addOutputStream(spu_string[0].c_str());
        
      } else {
        
        mpeg_version = 0;  // find a new mpeg_version
        analysed_buffers = 0;
        setMaxAnalysisIterations(100);
        if (!analyseData()) {
          ERROR_STREAM("analyseData() failed!" << endl);
          getWorkingInputProperty()->erase();
          return FAILURE;
        }
        
      }
      
      /* simple mpeg audio format */
      format_mpeg_audio = new Format("audio/mpeg");
      format_mpeg_audio->addIntParamValue(Format::audio_codec_param, 0x55);
      format_mpeg_audio->addIntParamValue(Format::layer_param, 2);
      
      /* unknown detailed mpeg audio format */
      format_mpeg_audio2 = new Format("audio/mpeg");
      format_mpeg_audio2->addIntParamValue(Format::audio_codec_param, 0x55);
      format_mpeg_audio2->addIntParamValue(Format::layer_param, 2);
      format_mpeg_audio2->addIntParamValue(Format::channels_param, 1);
      format_mpeg_audio2->addIntParamValue(Format::channels_param, 2);
      format_mpeg_audio2->addIntParamValue(Format::sample_rate_param, 8000);
      format_mpeg_audio2->addIntParamValue(Format::sample_rate_param, 11025);
      format_mpeg_audio2->addIntParamValue(Format::sample_rate_param, 12000);
      format_mpeg_audio2->addIntParamValue(Format::sample_rate_param, 16000);
      format_mpeg_audio2->addIntParamValue(Format::sample_rate_param, 22050);
      format_mpeg_audio2->addIntParamValue(Format::sample_rate_param, 44100);
      format_mpeg_audio2->addIntParamValue(Format::sample_rate_param, 48000);
      format_mpeg_audio2->addIntParamValue(Format::bitpersample_param, 8);
      format_mpeg_audio2->addIntParamValue(Format::bitpersample_param, 16);
      
      format_ac3_audio  = new Format("audio/ac3");
      
      // the following 2 lines are there for the case, that this Node support DTS or LPCM audio
      //       format_dts_audio  =
      //       format_lpcm_audio =
      
      /* simple mpeg video format */
      format_mpeg_video = new Format("video/mpeg");
      format_mpeg_video->addIntParamValue(Format::version_param, mpeg_version);
      format_spu        = new Format("video/spu");
      
      /* unknown detailed mpeg video format */
      format_mpeg_video2 = new Format("video/mpeg");
      format_mpeg_video2->addIntParamValue(Format::version_param, mpeg_version);
      format_mpeg_video2->addWildcard(Format::x_resolution_param);
      format_mpeg_video2->addWildcard(Format::y_resolution_param);
      format_mpeg_video2->addWildcard(Format::ratio_x_param);
      format_mpeg_video2->addWildcard(Format::ratio_y_param);
      format_mpeg_video2->addWildcard(Format::framerate_param);
      
      format_spu        = new Format("video/spu");
    }
    
    // init working output property
    Property* property = getOwnWorkingOutputProperty();
    Format* format;
    
    unsigned int stream;
    string  current_jack_tag;
    for( stream=0; stream<MAX_MPEG_AUDIO; stream++ ) {
      current_jack_tag = mpeg_audio_string[stream];
      if (getOwnOutputStream(current_jack_tag)) {
        /* add simple mpeg audio format */
	Format* format = new Format(*format_mpeg_audio);
        property->addNewFormat(format);
	format->setJackTag(current_jack_tag);
	property->setDefaultFormat(format, current_jack_tag);
        
        /* add unknown detailed mpeg audio format */
	if (format_mpeg_audio2) {
	  Format* format2 = new Format(*format_mpeg_audio2);
          property->addNewFormat(format2);
	  format2->setJackTag(current_jack_tag);
	}
      }
    }
    
    for( stream=0; stream<MAX_AC3_AUDIO; stream++ ) {
      current_jack_tag = ac3_audio_string[stream];
      if (getOwnOutputStream(current_jack_tag)) {
	format = new Format(*format_ac3_audio);
	format->setJackTag(current_jack_tag);
        property->addNewFormat(format);
	property->setDefaultFormat(format, current_jack_tag);
      }
    }
    
    // the following 2 blocks are there for the case, that this Node support DTS or LPCM audio
//     for( stream=0; stream<MAX_DTS_AUDIO; stream++ ) {
//       current_jack_tag = dts_audio_string[stream];
//       if (getOwnOutputStream(current_jack_tag)) {
// 	format = new Format(*format_dts_audio);
// 	format->setJackTag(current_jack_tag);
//         property->addNewFormat(format);
// 	property->setDefaultFormat(format, current_jack_tag);
//       }
//     }
    
//     for( stream=0; stream<MAX_LPCM_AUDIO; stream++ ) {
//       current_jack_tag = lpcm_audio_string[stream];
//       if (getOwnOutputStream(current_jack_tag)) {
// 	format = new Format(*format_lpcm_audio);
// 	format->setJackTag(current_jack_tag);
//         property->addNewFormat(format);
// 	property->setDefaultFormat(format, current_jack_tag);
//       }
//     }
    
    for( stream=0; stream<MAX_MPEG_VIDEO; stream++ ) {
      current_jack_tag = mpeg_video_string[stream];
      if (getOwnOutputStream(current_jack_tag)) {
        /* add simple mpeg video format */
	Format* format = new Format(*format_mpeg_video);
	format->setJackTag(current_jack_tag);
        property->addNewFormat(format);
	property->setDefaultFormat(format, current_jack_tag);

        /* add detailed unknown mpeg video format */
	if (format_mpeg_video2) {
	  Format* format2 = new Format(*format_mpeg_video2);
	  format2->setJackTag(current_jack_tag);
          property->addNewFormat(format2);
	}
      }
    }
    
    for( stream=0; stream<MAX_SPU; stream++ ) {
      current_jack_tag = spu_string[stream];
      if (getOwnOutputStream(current_jack_tag)) {
	format = new Format(*format_spu);
	format->setJackTag(current_jack_tag);
        property->addNewFormat(format);
	property->setDefaultFormat(format, current_jack_tag);
      }
    }
    
    if (!in_format->isParameterIncluded(Format::map_mpeg_audio_streams_param)) {
      // if input format is not detailed, Format pointer are allocated with new, so deallocate them
      delete format_mpeg_audio;
      delete format_mpeg_audio2;
      delete format_ac3_audio;
      // the following 2 lines are there for the case, that this Node support DTS or LPCM audio
//       delete format_dts_audio;
//       delete format_lpcm_audio;
      delete format_mpeg_video;
      delete format_mpeg_video2;
      delete format_spu;
    }

    return SUCCESS;
  }
  
  Result MPEGDemuxNode::doDeinitOutput() {
    /* Begin> remove all output streams */
    unsigned int stream;
    string current_jack_tag;
    for( stream=0; stream<MAX_MPEG_AUDIO; stream++ ) {
      current_jack_tag = mpeg_audio_string[stream];
      if (getOwnOutputStream(current_jack_tag)) {
        removeOutputStream(current_jack_tag.c_str());
      } 
    }
    
    for( stream=0; stream<MAX_AC3_AUDIO; stream++ ) {
      current_jack_tag = ac3_audio_string[stream];
      if (getOwnOutputStream(current_jack_tag)) {
        removeOutputStream(current_jack_tag.c_str());
      } 
    }
    
// the following 2 blocks are there for the case, that this Node support DTS or LPCM audio
//     for( stream=0; stream<MAX_DTS_AUDIO; stream++ ) {
//       current_jack_tag = dts_audio_string[stream];
//       if (getOwnOutputStream(current_jack_tag)) {
//         removeOutputStream(current_jack_tag.c_str());
//       } 
//     }
    
//     for( stream=0; stream<MAX_MPCM_AUDIO; stream++ ) {
//       current_jack_tag = lpcm_audio_string[stream];
//       if (getOwnOutputStream(current_jack_tag)) {
//         removeOutputStream(current_jack_tag.c_str());
//       } 
//     }
    
    for( stream=0; stream<MAX_MPEG_VIDEO; stream++ ) {
      current_jack_tag = mpeg_video_string[stream];
      if (getOwnOutputStream(current_jack_tag)) {
        removeOutputStream(current_jack_tag.c_str());
      } 
    }
    
    for( stream=0; stream<MAX_SPU; stream++ ) {
      current_jack_tag = spu_string[stream];
      if (getOwnOutputStream(current_jack_tag)) {
        removeOutputStream(current_jack_tag.c_str());
      } 
    }
    /* End< remove all output streams */
    
    return SUCCESS;
  }
  
  void MPEGDemuxNode::setVideoStream( const nmm_int32& s ) {
    video_stream_number = s;
  }
  
  void MPEGDemuxNode::setAudioStream( const nmm_int32& s ) {
    audio_stream_number = s;
  }
  
  void MPEGDemuxNode::setSPUStream( const nmm_int32& s ) {
    spu_stream_number = s;
  }
  
  void MPEGDemuxNode::setDVDMode( const nmm_bool& m ) {
    dvd_mode = m;
  }

  struct timeval MPEGDemuxNode::readTimecode(uint8_t *pointer) {
    double timestamp;
    uint8_t hibit; uint32_t lowbytes;

    hibit = (pointer[0]>>3)&0x01;
    lowbytes = (((uint32_t)pointer[0] >> 1) & 0x03) << 30;
    lowbytes |= (uint32_t)pointer[1] << 22;
    lowbytes |= ((uint32_t)pointer[2] >> 1) << 15;
    lowbytes |= (uint32_t)pointer[3] << 7;
    lowbytes |= ((uint32_t)pointer[4]) >> 1;
    timestamp = (double)hibit*FLOAT_0x10000*FLOAT_0x10000+(double)lowbytes;
    timestamp /= STD_SYSTEM_CLOCK_FREQ;

    struct timeval now;
    now.tv_sec = static_cast<long>(timestamp);
    now.tv_usec = static_cast<long> (( timestamp - now.tv_sec ) * 1000000);

    return now;
  }

  /* some helpers for demux() */
  inline bool isProgramEnd(const unsigned int code) { return (code == 0xb9); }
  inline bool isPackHeader(const unsigned int code) { return (code == 0xba); }
  inline bool isMPEGVideo(const unsigned int code) { return ((0xe0 <= code) && (code < 0xf0)); }
  inline bool isMPEGAudio(const unsigned int code) { return ((0xc0 <= code) && (code < 0xe0)); }
  inline bool isAC3OrSPU(const unsigned int code) { return ((code == 0xbd)); }
  inline bool isAC3(const uint8_t data_start) { return ((0x80 <= data_start) && (data_start < 0x88)); }
  inline bool isSPU(const uint8_t data_start) { return ((0x20 <= data_start) && (data_start < 0x40)); }

  inline Buffer* MPEGDemuxNode::sendMPEGPacket(const uint8_t* data_start, const uint8_t* packet_end, const string jack_tag, const long int counter) {
    Buffer* out = NULL;
    timestamp.stream_counter = counter;
    if ( data_start < packet_end ) {
      int buff_size = packet_end - data_start;
      out = getNewBuffer( buff_size );
      memcpy( out->getData(), data_start, buff_size );
      out->setUsed( buff_size );
      out->setTimestamp( timestamp );
      // cerr << "LOGGER " << jack_tag.c_str() << " set timestamp " << timestamp << endl;

      DEBUG_STREAM(jack_tag << ": set timestamp " << timestamp << endl);
      DEBUG_STREAM(jack_tag << ": buffer size: " << buff_size << endl);
      setCurrentSendOutputStream(jack_tag.c_str());
    }
    return out;
  }

  OutputJack* MPEGDemuxNode::getMPEGOutputStream(const string& jack_tag /*= "default"*/ )
  {
    OutputJack* out_jack = GenericDemultiplexerNode::getOwnOutputStream(jack_tag);
    if (getAnalysingFlag() && !out_jack) {
      DEBUG_STREAM("MPEGDemuxNode::getMPEGOutputStream(): Adding output stream " << jack_tag << endl);
      if (SUCCESS != addOutputStream(jack_tag.c_str())) {
	return NULL;
      }
      out_jack = GenericDemultiplexerNode::getOwnOutputStream(jack_tag);
    }
    return out_jack;
  }

  bool MPEGDemuxNode::demux() {
    unsigned int code;
    uint8_t* data_start;
    uint8_t* packet_end;
    bool     start_code_ok;
    
    out = NULL;
    start_code_ok = false;
 
    /* Begin> check if enough data exists to hold min. 23 0-Bits and 1 1-Bit -> "packet start code prefix" */
    if ( buf + 3 < end  ) {

      /* Begin> check start code, if found, <buf> points to the begin of the packet */
      while ( buf + 3 < end ) {
	if ( buf[0] || buf[1] || (buf[2] != 0x01) ) {
	  buf++;
	} else {
	  start_code_ok = true; break; 
	}
      }
      if ( (!start_code_ok) || (buf+14 > end) ) {
	return( false );
      }
      code = buf[3];
      /* program end code */
      if ( isProgramEnd(code) ) {
	/* skip */
	packet_end = buf + 6 + (buf[4] << 8) + buf[5];
	if ( packet_end > end) return( false );
	/* set buffer pointer to next packet */
	buf = packet_end;
	return( true );
      }
      /* End< check start code, if found, <buf> points to the begin of the packet */

      /* Begin> pack header */
      if ( isPackHeader(code) ) {
	/* skip */
	if ((buf[4] & 0xc0) == 0x40) {	/* mpeg2 */
	  packet_end = buf + 14 + (buf[13] & 7);
	} else {
	  packet_end = buf + 12;
	}
	if ( packet_end > end ) {
	  return( false );
	}
	/* set buffer pointer to next packet */
	buf = packet_end;
	return( true );
      } // packet header
      /* End< pack header */
	  
      /* Begin> mpeg video */
      if ( isMPEGVideo(code) ) {
	code -= 0xe0; // calculate video stream number

	if ( dvd_mode ) {
	  /* the selected stream number is output to mpeg_video0 */
	  if ( code == video_stream_number ) code = 0;
	  else code = 1;
	}
	OutputJack* mpeg_video_jack = getMPEGOutputStream( mpeg_video_string[code] );
	if ( mpeg_video_jack != NULL && 
	     ( isAnyOutputStreamConnected(mpeg_video_string[code]) ||
	       getAnalysingFlag() )) {
	  packet_end = buf + 6 + (buf[4] << 8) + buf[5];
	  if ( packet_end > end ) {
	    return( false );
	  }

	  if ( (buf[6] & 0xc0) == 0x80 ) { /* Begin> mpeg2 */
	    if (getAnalysingFlag() && !mpeg_version) {
	      if (2 != mpeg_version) {
		ERROR_STREAM("Found MPEG-Video1 AND MPEG-Video2 in on MPEG-System stream!" << endl);
	      }
	      mpeg_version = 2;
	    }
	    data_start = buf + 9 + buf[8];
	    // check, if PTS present
	    if ( (buf[7] & 0xc0) >= 0x80 ) {
	      if(first_pts) {
		first_pts = false;
		start_time = readTimecode( &buf[9] );
		DEBUG_STREAM("First PTS at MPEG Video : " << start_time.tv_sec << " " << start_time.tv_usec << endl);
	      }
	      now = readTimecode( &buf[9] );
	      timestamp.sync_time = Time(now);
	      timestamp.is_valid = true;
	      DEBUG_STREAM("Video PTS found: " << timestamp.sync_time << endl );
	    } else {
	      timestamp.is_valid = false;
	    }
	  } /* End< mpeg2 */
	  else { /* Begin> mpeg1 */
	    if (getAnalysingFlag() && !mpeg_version) {
	      if (1 != mpeg_version) {
		ERROR_STREAM("Found MPEG-Video1 AND MPEG-Video2 in on MPEG-System stream!" << endl);
	      }
	      mpeg_version = 1;
	    }
	    int i;
	    for (i = 6; buf[i] == 0xff; i++);
	    if ((buf[i] & 0xc0) == 0x40) i += 2;
	    data_start = buf + i + 1;
	    if (buf[i] & 0x20) { // PTS present
	      data_start += 4;
	      if(first_pts) {
		first_pts = false;
		start_time = readTimecode( &buf[i] );
		DEBUG_STREAM("First PTS at MPEG Video : " << start_time.tv_sec << " " << start_time.tv_usec << endl);
	      }
	      now = readTimecode( &buf[i] );
	      timestamp.sync_time = Time(now);
	      timestamp.is_valid = true;
	      DEBUG_STREAM("Video PTS found: " << timestamp.sync_time << endl );
	    } else {
	      timestamp.is_valid = false;
	    }
	    if (buf[i] & 0x10) { // DTS present
	      data_start += 5;
	    } 
	  }
	  /* End< mpeg1 */

	  /* set buffer pointer to next packet */
	  buf = packet_end;
	  // Video Stream verschicken
	  out = sendMPEGPacket(data_start, packet_end, mpeg_video_string[code], mpeg_video_count[code]++);
	  return( true );
	}
      } // video
      /* End< mpeg video */
      
      /* Begin> mpeg audio */
      if ( code != 0xbe && isMPEGAudio(code) && !skip_audio ) {
	code -= 0xc0; // calculate mpeg audio stream number
	if ( dvd_mode ) {
	  /* the selected stream number is output to mpeg_audio0 */
	  if ( code == audio_stream_number ) code = 0;
	  else code = 1;
	}
	OutputJack* mpeg_audio_jack = getMPEGOutputStream( mpeg_audio_string[code] );
	if ( mpeg_audio_jack != NULL && 
	     ( isAnyOutputStreamConnected(mpeg_audio_string[code]) ||
	       getAnalysingFlag() )) {
	  packet_end = buf + 6 + (buf[4] << 8) + buf[5];
	  if ( packet_end > end ) {
	    return( false );
	  }

	  if ((buf[6] & 0xc0) == 0x80) {	/* mpeg2 */
	    data_start = buf + 9 + buf[8];
	    // check, if PTS present
	    if ( (buf[7] & 0xc0) >= 0x80 ) {
	      if(first_pts) {
		first_pts = false;
		start_time = readTimecode( &buf[9] );
		DEBUG_STREAM("First PTS at MPEG Audio : " << start_time.tv_sec << " " << start_time.tv_usec << endl);
	      }
	      now = readTimecode( &buf[9] );
	      timestamp.sync_time = Time(now);
	      timestamp.is_valid = true;
	      DEBUG_STREAM("MPEG Audio PTS found: " << timestamp.sync_time << endl );
	    } else {
	      timestamp.is_valid = false;
	    }
	  } else {
	    int i;
	    for (i = 6; buf[i] == 0xff; i++);
	    if ((buf[i] & 0xc0) == 0x40) i += 2;
	    data_start = buf + i + 1;
	    if (buf[i] & 0x20) {  // PTS present
              //cout << "## " << (buf[i] & 0x20) << endl;

	      data_start += 4;
	      if(first_pts) {
		first_pts = false;
		start_time = readTimecode( &buf[i] );
		DEBUG_STREAM("First PTS at MPEG Audio : " << start_time.tv_sec << " " << start_time.tv_usec << endl);
	      }
	      now = readTimecode( &buf[i] );
	      timestamp.sync_time = Time(now);
	      timestamp.is_valid = true;

	      DEBUG_STREAM("Audio PTS found: " << timestamp.sync_time << endl );
	    } else {
	      timestamp.is_valid = false;
	    }
	    if (buf[i] & 0x10) { // DTS present
	      data_start += 5;
	    }
	  }

	  buf = packet_end;
	  // Audio Stream verschicken
	  out = sendMPEGPacket(data_start, packet_end, mpeg_audio_string[code], mpeg_audio_count[code]++);
	  return( true );
	}
      } // audio (mpeg)
      /* End< mpeg audio */

      /* Begin> Private_stream_1: there you can find the ac3 and spu streams */
      if ( isAC3OrSPU(code) ) {
	packet_end = buf + 6 + (buf[4] << 8) + buf[5];
	data_start = buf + 9 + buf[8];
	if ( packet_end > end) {
	  return( false );
	}
	// check, if PTS present
	if ( (buf[7] & 0xc0) >= 0x80 ) {
	  if(first_pts) {
	    first_pts = false;
	    start_time = readTimecode( &buf[9] );
	    DEBUG_STREAM("First PTS at AC3 Audio/SPU : " << start_time.tv_sec << " " << start_time.tv_usec << endl);
	  }
	  now = readTimecode( &buf[9] );
	  timestamp.sync_time = Time(now);
	  timestamp.is_valid = true;
	  DEBUG_STREAM("AC3 Audio/SPU PTS found: " << Time(now) << endl );
	} else {
	  timestamp.is_valid = false;
	}
	buf = packet_end;

	/* Begin> ac3 audio */
	if (  isAC3(*data_start) && !skip_audio ) {
	  code = *data_start-0x80; // calculate ac3 audio stream number
	  if ( dvd_mode ) {
	    /* the selected stream number is output to ac3_audio0 */
	    if ( code == audio_stream_number ) code = 0;
	    else code = 1;
	  }
	  OutputJack* ac3_audio_jack = getMPEGOutputStream( ac3_audio_string[code] );
	  if ( ac3_audio_jack != NULL && 
	       ( isAnyOutputStreamConnected(ac3_audio_string[code]) ||
		 getAnalysingFlag() )) {
	    // skip the 4 mystery ac3 bytes
	    data_start += 4;
	    // Audio Stream verschicken
	    out = sendMPEGPacket(data_start, packet_end, ac3_audio_string[code], ac3_audio_count[code]++);
	  }
	}  /* End< ac3 audio */
	/* Begin> subtitle */
	else if ( isSPU(*data_start) ) { // subtitle unit
	  code = *data_start-0x20; // calculate spu stream number
	  if ( dvd_mode ) {
	    /* the selected stream number is output to spu0 */
	    if ( code == spu_stream_number ) code = 0;
	    else code = 1;
	  }
	  OutputJack* spu_jack = getMPEGOutputStream( spu_string[code] );
	  if ( spu_jack != NULL && 
	       ( isAnyOutputStreamConnected(spu_string[code]) ||
		 getAnalysingFlag() )) {
	    DEBUG_STREAM( "Found Subtitle #" << (unsigned int) *data_start << endl );
	    data_start+=1;
	    out = sendMPEGPacket(data_start, packet_end, spu_string[code], spu_count[code]++);
	  }
	}
	/* End< subtitle */
	
	return( true );
      } // private stream1
      /* End< Private_stream_1: there you can find the ac3 and spu streams */
          
      //default:
      if (code < 0xb9) {
	DEBUG_STREAM( "looks like a video stream, not system stream" << endl );
      }
          
      /* skip */
      packet_end = buf + 6 + (buf[4] << 8) + buf[5];
      if ( packet_end > end ) return( false );

      buf = packet_end;
      return( true );
    } // if (enough data)
    /* End< check if enough data exists to hold min. 23 0-Bits and 1 1-Bit -> "packet start code prefix" */

    return( false );
  }


  Message* MPEGDemuxNode::processBuffer( Buffer* in_buffer ) {
    if(!getWorkingFlag()) { // start processing new buffer

      /* save incoming buffer size */
      cur_buffer_length = in_buffer->getUsed();
      
      /*
       * since we must copy some undemultiplexed data from the last buffer
       * (because not enought data was available to demultiplex the last packet)
       * we need some space before the incoming buffer to put this data in it
       * that's why we copy the incoming buffer in an extra buffer that is a bit larger
       * usually the incoming buffers has got the same size, so we adjust our extra
       * buffer only if the incoming buffer is larger, so we save some memory allocation
       */
      if ( cur_buffer_length > (inbuffer_size - fragment_size) ) {
        inbuffer_size = cur_buffer_length + fragment_size;
        if ( tmpbuffer ) delete[] tmpbuffer;

        /* our extra buffer with some extra space */
        tmpbuffer = new uint8_t[inbuffer_size];
        DEBUG_STREAM("New temp. buffer created with size:" << inbuffer_size << endl);
      }

      buf = tmpbuffer;

      /* pointer to our end data */
      end  = tmpbuffer + cur_buffer_length + rest_length;

      /*
       * copy incoming buffer into extra buffer and skip some bytes before
       * because here we find the undecoded data from the last buffer
       */
      memcpy( tmpbuffer+rest_length, in_buffer->getData(), cur_buffer_length);

      /* now we don't need the incoming buffer anymore */
      in_buffer->release();

      setWorkingFlag( true );

    }

    if ( send_event ) {
      send_event = false;
      return( new CEvent(Event(ISyncReset::syncReset_event)) );
    }
    
    do {
      /* demux stream */
      if ( demux() == false ) {
        /* we get here if we have not enough data to demux a packet */

        /* calculate the size of the data that could not be demuxed */
        rest_length = end - buf;

        /**
        * copy this piece of data to the begin of our extra buffer, so the next time
        * enought data should be there to demux the packet
        * (should we here use better a memmove instead of memcpy?)
        */
        memcpy( tmpbuffer, buf, rest_length );

        setWorkingFlag( false );
        
        return( 0 );
        /* returns no buffer */
      }
      /* if no packet could be demuxed and there is enough data, demux next packet...*/
    } while( !out );

    if (getAnalysingFlag()) {
      if (++analysed_buffers > 140) {
	setAnalysingFlag(false);
      }
    }
      

    return( out );
  }

  Result MPEGDemuxNode::doFlush() {
    setWorkingFlag( false );
    rest_length = 0;

    return( SUCCESS );
  }
  
  Result MPEGDemuxNode::syncReset() {
    first_pts = true;

    memset( mpeg_video_count, 0, MAX_MPEG_VIDEO );
    memset( mpeg_audio_count, 0, MAX_MPEG_AUDIO );
    memset( ac3_audio_count,  0, MAX_AC3_AUDIO );
    memset( spu_count,        0, MAX_SPU );

    timestamp.sync_time = Time(0);
    timestamp.is_valid = false;

    DEBUG_STREAM("sync_reset completed." << endl);

    return SUCCESS;
  };
  
  Result MPEGDemuxNode::sendIFramesOnly(const nmm_int32& trigger) {
    DEBUG_STREAM("event i_frames_only: "<< trigger << endl);
    if (trigger == 0) {
      skip_audio  = false;
      DEBUG_STREAM("normal processing mode" << endl);
      return SUCCESS;
    }

    skip_audio  = true;
    DEBUG_STREAM("sending only video-data" << endl);

    return SUCCESS;
  };

  Result MPEGDemuxNode::fastForward( const nmm_int32& speed ) {
   
    /* stop itself */
    try{ stop(); }
    catch(...){ DEBUG_STREAM("Node already stopped" << endl); };
    
    sendIFramesOnly( speed );
    
    if ( speed > 0 ) {
      DEBUG_STREAM("fast forward event: on" << endl);
     
      /* send flush event to sucessor */
      // removed sendSet|GetEvent (was implemented as return FAILURE; anyway!)
      // if ( successor ) successor->sendSetEvent(Event(INode::flush_event));
    } else {
      DEBUG_STREAM("fast forward event: off" << endl);
      
      syncReset();
      
      /* flag to send sync_reset event in processBuffer */
      send_event = true;
    }
    
    try{ start(); }
    catch(...){ DEBUG_STREAM("Node already started" << endl); };

    return( SUCCESS );
  }

  Result MPEGDemuxNode::streamJump( const nmm_int32& speed ) {
    
    DEBUG_STREAM("stream jump event, speed = " << speed << endl);
    
    /* stop itself */
    try{ stop(); }
    catch(...){ DEBUG_STREAM("Node already stopped" << endl); };
    
    flush();
    
    
    /* send flush event to sucessor */
    // removed sendSet|GetEvent (was implemented as return FAILURE; anyway!)
    // if ( successor ) successor->sendSetEvent(Event(INode::flush_event));
    
    /* send upstream event to predecessor */
    // removed sendSet|GetEvent (was implemented as return FAILURE; anyway!)
    // if ( predecessor ) predecessor->sendSetEvent(Event(IStreamJump::streamJump_event, new TInValue<nmm_int32>( speed )) );
    
    try{ start(); }
    catch(...){ DEBUG_STREAM("Node already started" << endl); };
    
    return( SUCCESS );
  }

}

#include "nmm/nmm_suffix.hpp"
