/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
 

#include "MPEGVSHDetectionNode.hpp"
#include "nmm/comm/format/Property.hpp"
#include "nmm/multimedia/Plugin.hpp"
#include "nmm/plugins/NMMRegisterPlugin.hpp"
#include "nmm/utils/time/Interval.hpp"

#include <iostream>

NMM_REGISTER_PLUGIN(MPEGVSHDetectionNode, NMM, "MPEGVSHDetectionNode")

using namespace std;

#include "nmm/nmm_prefix.hpp"

namespace NMM{
  
  MPEGVSHDetectionNode::MPEGVSHDetectionNode(const char* name, const StreamQueue::Mode queue_mode, const int queue_size) 
    : GenericMuxDemuxNode(name, queue_mode, queue_size) ,
      has_seen_first_vsh(false),
      buf(NULL),
      cur_inbuffer_length(0),
      tmpbuffer(NULL),
      inbuffer_size(0),
      end(NULL),
      rest_buffer(NULL),
      rest_buffer_length(0),
      out(NULL),
      has_audio(false),
      has_video(false)
  {}
  
  MPEGVSHDetectionNode::~MPEGVSHDetectionNode(){
  }  

  Result MPEGVSHDetectionNode::doInit(){
    setWorkingFlag(false);

    addInputStream("audio_input_stream");   // audio input stream
    addInputStream("video_input_stream");   // video input stream
    addOutputStream("audio_output_stream"); // audio output stream
    addOutputStream("video_output_stream"); // video output stream

    // input formats
    Format* audio_input_format;

    audio_input_format = getOwnInputProperty()->addNewFormat(Format::audio_mpeg_type);
    audio_input_format->setJackTag("audio_input_stream");
    audio_input_format->addIntParamValue(Format::layer_param, 2);
    audio_input_format->addIntParamValue(Format::layer_param, 3);
    audio_input_format->addIntParamValue(Format::audio_codec_param, 0x55);
    audio_input_format->addWildcard(Format::sample_rate_param);
    audio_input_format->addWildcard(Format::channels_param);
    audio_input_format->addWildcard(Format::bitpersample_param);
    audio_input_format->addWildcard(Format::bitrate_param);

    audio_input_format = getOwnInputProperty()->addNewFormat(Format::audio_mpeg_type);
    audio_input_format->setJackTag("audio_input_stream");
    audio_input_format->addIntParamValue(Format::layer_param, 2);
    audio_input_format->addIntParamValue(Format::layer_param, 3);
    audio_input_format->addIntParamValue(Format::audio_codec_param, 0x55);
    audio_input_format->addWildcard(Format::sample_rate_param);
    audio_input_format->addWildcard(Format::channels_param);
    audio_input_format->addWildcard(Format::bitpersample_param);

    audio_input_format = getOwnInputProperty()->addNewFormat(Format::audio_mpeg_type);
    audio_input_format->setJackTag("audio_input_stream");
    audio_input_format->addIntParamValue(Format::layer_param, 2);
    audio_input_format->addIntParamValue(Format::layer_param, 3);
    audio_input_format->addIntParamValue(Format::audio_codec_param, 0x55);

    audio_input_format = getOwnInputProperty()->addNewFormat(Format::audio_mpeg_type);
    audio_input_format->setJackTag("audio_input_stream");
    getOwnInputProperty()->setDefaultFormat(audio_input_format,"audio_input_stream");
    

    Format* video_input_format;

    video_input_format = getOwnInputProperty()->addNewFormat(Format::video_mpeg_type);
    video_input_format->setJackTag("video_input_stream");
    video_input_format->addIntParamValue(Format::version_param, 1);
    video_input_format->addIntParamValue(Format::version_param, 2);
    video_input_format->addWildcard(Format::x_resolution_param);
    video_input_format->addWildcard(Format::y_resolution_param);
    video_input_format->addWildcard(Format::ratio_x_param);
    video_input_format->addWildcard(Format::ratio_y_param);
    video_input_format->addWildcard(Format::framerate_param);

    video_input_format = getOwnInputProperty()->addNewFormat(Format::video_mpeg_type);
    video_input_format->setJackTag("video_input_stream");
    video_input_format->addIntParamValue(Format::version_param, 1);
    video_input_format->addIntParamValue(Format::version_param, 2);
    getOwnInputProperty()->setDefaultFormat(video_input_format,"video_input_stream");


    // output formats
    Format* audio_output_format;

    audio_output_format = getOwnOutputProperty()->addNewFormat(Format::audio_mpeg_type);
    audio_output_format->setJackTag("audio_output_stream");
    audio_output_format->addIntParamValue(Format::layer_param, 2);
    audio_output_format->addIntParamValue(Format::layer_param, 3);
    audio_output_format->addIntParamValue(Format::audio_codec_param, 0x55);
    audio_output_format->addWildcard(Format::sample_rate_param);
    audio_output_format->addWildcard(Format::channels_param);
    audio_output_format->addWildcard(Format::bitpersample_param);
    audio_output_format->addWildcard(Format::bitrate_param);

    audio_output_format = getOwnOutputProperty()->addNewFormat(Format::audio_mpeg_type);
    audio_output_format->setJackTag("audio_output_stream");
    audio_output_format->addIntParamValue(Format::layer_param, 2);
    audio_output_format->addIntParamValue(Format::layer_param, 3);
    audio_output_format->addIntParamValue(Format::audio_codec_param, 0x55);
    audio_output_format->addWildcard(Format::sample_rate_param);
    audio_output_format->addWildcard(Format::channels_param);
    audio_output_format->addWildcard(Format::bitpersample_param);

    audio_output_format = getOwnOutputProperty()->addNewFormat(Format::audio_mpeg_type);
    audio_output_format->setJackTag("audio_output_stream");
    audio_output_format->addIntParamValue(Format::layer_param, 2);
    audio_output_format->addIntParamValue(Format::layer_param, 3);
    audio_output_format->addIntParamValue(Format::audio_codec_param, 0x55);
    getOwnOutputProperty()->setDefaultFormat(audio_output_format,"audio_output_stream");


    Format* video_output_format;

    video_output_format = getOwnOutputProperty()->addNewFormat(Format::video_mpeg_type);
    video_output_format->setJackTag("video_output_stream");
    video_output_format->addIntParamValue(Format::version_param, 1);
    video_output_format->addIntParamValue(Format::version_param, 2);
    video_output_format->addWildcard(Format::x_resolution_param);
    video_output_format->addWildcard(Format::y_resolution_param);
    video_output_format->addWildcard(Format::ratio_x_param);
    video_output_format->addWildcard(Format::ratio_y_param);
    video_output_format->addWildcard(Format::framerate_param);

    video_output_format = getOwnOutputProperty()->addNewFormat(Format::video_mpeg_type);
    video_output_format->setJackTag("video_output_stream");
    video_output_format->addIntParamValue(Format::version_param, 1);
    video_output_format->addIntParamValue(Format::version_param, 2);
    getOwnOutputProperty()->setDefaultFormat(video_output_format,"video_output_stream");
	    
    return SUCCESS;
  };

    
  Result MPEGVSHDetectionNode::doInitOutput(){

    // get input formats and make sure they exist
    const Format* audio_in_format = getInputFormat("audio_input_stream");
    const Format* video_in_format = getInputFormat("video_input_stream");

    if (audio_in_format == NULL && video_in_format == NULL) {
      ERROR_STREAM("Failure at input streams. No audio and video input format!" << endl);
      return FAILURE;
    }

    // Audio formats
    if (audio_in_format) {
      has_audio = true;

      // working input property
      Format* working_audio_in_format = new Format(*audio_in_format);	
      getOwnWorkingInputProperty()->addNewFormat(working_audio_in_format);
      getOwnWorkingInputProperty()->setDefaultFormat(working_audio_in_format,"audio_input_stream");
      
      // set output format
      // this is equal to input format
      Format* audio_out_format = new Format(*working_audio_in_format);
      getOwnWorkingOutputProperty()->addNewFormat(audio_out_format);
      getOwnWorkingOutputProperty()->setDefaultFormat(audio_out_format, "audio_output_stream");
      
      // set I/O partners
      working_audio_in_format->setIOPartner(audio_out_format);
      audio_out_format->setIOPartner(working_audio_in_format);
    } else {
      ERROR_STREAM("getInputFormat(\"audio_input_stream\") failed!" << endl);
      ERROR_STREAM("FAILURE at input streams" << endl); 
    }
        
    // Video formats
    if (video_in_format) {
      has_video = true;

      // working input property
      Format* working_video_in_format = new Format(*video_in_format);
      getOwnWorkingInputProperty()->addNewFormat(working_video_in_format);
      getOwnWorkingInputProperty()->setDefaultFormat(working_video_in_format,"video_input_stream");
      
      // set output format
      // this is equal to input format
      Format* video_out_format = new Format(*working_video_in_format);
      getOwnWorkingOutputProperty()->addNewFormat(video_out_format);
      getOwnWorkingOutputProperty()->setDefaultFormat(video_out_format, "video_output_stream");
      
      // set I/O partners
      working_video_in_format->setIOPartner(video_out_format);
      video_out_format->setIOPartner(working_video_in_format);
    } else { 
      ERROR_STREAM("getInputFormat(\"video_input_stream\") failed!" << endl);
      ERROR_STREAM("FAILURE at input streams" << endl); 
    }

    return SUCCESS;
  };
  
  
  Message* MPEGVSHDetectionNode::processBuffer(Buffer* in_buffer) {

    if ( isInputConnected("audio_input_stream") && 
	 !isInputConnected("video_input_stream") ) {
      setCurrentSendOutputStream("audio_output_stream");
      return in_buffer;
    }

    if ( isInputConnected("video_input_stream") && 
	 !isInputConnected("audio_input_stream") ) {
      setCurrentSendOutputStream("video_output_stream");
      return in_buffer;
    }

    if (!getWorkingFlag()) {
      if ( strcmp(getCurrentRecvInputStream(), "audio_input_stream" ) == 0) {
	setCurrentSendOutputStream("audio_output_stream");
	if (has_seen_first_vsh) {
	  Time inbuffer_ts = in_buffer->getTimestamp().sync_time;
	  Interval distance = inbuffer_ts - vsh_ts;
	  if (distance >= Interval(0) ) {
	    return in_buffer;
	  } else {
	    in_buffer->release();
	    return NULL;
	  }
	} else {
	  in_buffer->release();
	  return NULL;
	}
      } else if ( strcmp(getCurrentRecvInputStream(), "video_input_stream" ) == 0) {
	setCurrentSendOutputStream("video_output_stream");
	if (has_seen_first_vsh) {
	  return in_buffer;
	} else {
	  
	  cur_inbuffer_length = in_buffer->getUsed();
	  
	  if (rest_buffer) {
	    inbuffer_size = rest_buffer_length + cur_inbuffer_length;
	  } else {
	    inbuffer_size = cur_inbuffer_length;
	  }
	  if (tmpbuffer) delete[] tmpbuffer;
	  tmpbuffer = new uint8_t[inbuffer_size];
	  
	  /* start point of the new data buffer */
	  buf = tmpbuffer;
	  /* end point of the new data buffer */
	  end = tmpbuffer + inbuffer_size;
	  
	  /* copy the rest_buffer and the new incoming buffer into the tmpbuffer */
	  if (rest_buffer) {
	    memcpy(tmpbuffer, rest_buffer, rest_buffer_length);
	    memcpy(tmpbuffer + rest_buffer_length, in_buffer->getData(), cur_inbuffer_length);
	  } else {
	    memcpy(tmpbuffer, in_buffer->getData(), cur_inbuffer_length);
	  }
	  
	  setWorkingFlag(true);
	  
	  if (searchVSH() == false) {
	    /* If we get here we have not yet found a VSH. Now we copy the last 
	     * three bytes of the in_buffer into the rest_buffer. If we have a 
	     * new incoming buffer we append his data to this rest_buffer.
	     */
	    if (rest_buffer) delete[] rest_buffer;
	    rest_buffer = new uint8_t[rest_buffer_length];
	    memcpy(rest_buffer, in_buffer->getData()+(cur_inbuffer_length-rest_buffer_length), rest_buffer_length);
	    
	    setWorkingFlag(false);
	    in_buffer->release();
	    
	    return NULL;
	  } else {
	    
	    // If we get here a VSH was found and it starts at position buf
	    if (buf < end) {
	      vsh_ts = in_buffer->getTimestamp().sync_time;
	      int buffer_size = end - buf;
	      out = getNewBuffer( buffer_size );
	      memcpy( out->getData(), buf, buffer_size);
	      out->setUsed( buffer_size);
	      out->setTimestamp(in_buffer->getTimestamp());
	      DEBUG_STREAM("Created new outgoing buffer with size " << buffer_size << endl);
	      DEBUG_STREAM("Set timestamp in new outgoing buffer to " << out->getTimestamp() << endl);
	      
	      setWorkingFlag(false);
	      in_buffer->release();
	      
	      return out;
	    }
	  }
	}
      }
    }
    return NULL;
  }
 
  Result MPEGVSHDetectionNode::doDeactivate(){
    return SUCCESS;
  };

  bool MPEGVSHDetectionNode::searchVSH() {

    unsigned int code;
    bool start_code_ok;

    start_code_ok = false;
    
    /* Begin> check if enough data exists to hold min. 23 0-Bits and 1 1-Bit -> "packet start code prefix" */
    if ( buf+3 < end) {
      /* Begin> check start code, if found, <buf> points to the begin of the packet */
      while ( buf + 3 < end ) {
	if ( buf[0] || buf[1] || (buf[2] != 0x01) ) {
	  buf++;
	} else { 
	  start_code_ok = true; break; 
	}
      }
      if ( (!start_code_ok) || (buf+14 > end) ) {
	// if start code not okay or rest of buffer is not long enough return false
	return( false );
      }
      code = buf[3];
      /* check VSH */
      if ( isVideoSequenceHeader( code ) ) {
	has_seen_first_vsh = true;
	return true;
      }
    } // if (enough data)
    /* End> check if enough data exists to hold min. 23 0-Bits and 1 1-Bit -> "packet start code prefix" */
    return false;
  }
  
  bool MPEGVSHDetectionNode::isVideoSequenceHeader(const unsigned int code) {
    return (code == 0xb3);
  }

}

#include "nmm/nmm_suffix.hpp"
