/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IFrameCount_HPP
#define NMM_IFrameCount_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{
  /**
   * Requesting of frame count.
   *
   * Defines a method that returns the frame count of some video stream.
   */

  class DLLEXPORT IFrameCount : virtual public NMM::Interface { 
  public:
    IFrameCount() : NMM::Interface(IFrameCount::IFrameCount_type, "NMM") { };

    IFrameCount(NMM::NMMObject& parent) : NMM::Interface (parent, IFrameCount::IFrameCount_type, "NMM") { };

    virtual ~IFrameCount() {};

    static const char* IFrameCount_type;

    virtual  string getInterfaceType() const { return IFrameCount_type; }

    NMM::Interface* copy() const { return new IFrameCount(*this); }
    static void used(); 
    static const char* getFrameCount_event;
    /**
     * Get the framecount
     */

    virtual nmm_int32 getFrameCount() const
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IFrameCount> IFrameCount_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
