/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ICDDA.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/serialize/SerializeUtilsTime.hpp"

namespace NMM
{
  void ICDDA::used() {} 
  
  
  Result ICDDA::setTrack(const nmm_int32& track)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > track_value(track, false);
    NMM::Event event("ICDDA::setTrack", &track_value);
    event.setReturnValue(&return_value);
    if(const_cast<ICDDA*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 ICDDA::getTrack()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("ICDDA::getTrack");
    event.setReturnValue(&return_value);
    if(const_cast<ICDDA*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 ICDDA::getTotalTracks()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("ICDDA::getTotalTracks");
    event.setReturnValue(&return_value);
    if(const_cast<ICDDA*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ICDDA::numberTracks(const nmm_bool& number)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > number_value(number, false);
    NMM::Event event("ICDDA::numberTracks", &number_value);
    if(const_cast<ICDDA*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  Interval ICDDA::getTrackDuration(const nmm_int32& track)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Interval > return_value;
    NMM::TInValue<nmm_int32 > track_value(track, false);
    NMM::Event event("ICDDA::getTrackDuration", &track_value);
    event.setReturnValue(&return_value);
    if(const_cast<ICDDA*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Interval ICDDA::getTotalDuration()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Interval > return_value;
    NMM::Event event("ICDDA::getTotalDuration");
    event.setReturnValue(&return_value);
    if(const_cast<ICDDA*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool ICDDA::isFinished()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ICDDA::isFinished");
    event.setReturnValue(&return_value);
    if(const_cast<ICDDA*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ICDDA::setRepeat(const nmm_bool& repeat)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > repeat_value(repeat, false);
    NMM::Event event("ICDDA::setRepeat", &repeat_value);
    if(const_cast<ICDDA*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_bool ICDDA::getRepeat()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ICDDA::getRepeat");
    event.setReturnValue(&return_value);
    if(const_cast<ICDDA*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ICDDA::setOneTrackOnly(const nmm_bool& enable)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > enable_value(enable, false);
    NMM::Event event("ICDDA::setOneTrackOnly", &enable_value);
    if(const_cast<ICDDA*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_bool ICDDA::getOneTrackOnly()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ICDDA::getOneTrackOnly");
    event.setReturnValue(&return_value);
    if(const_cast<ICDDA*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ICDDA::ICDDA_type = "ICDDA";
  NMM::TConstructor<ICDDA > __ICDDA_prototype("ICDDA","NMM");


  const char* ICDDA::setTrack_event = "ICDDA::setTrack";
  const char* ICDDA::getTrack_event = "ICDDA::getTrack";
  const char* ICDDA::getTotalTracks_event = "ICDDA::getTotalTracks";
  const char* ICDDA::numberTracks_event = "ICDDA::numberTracks";
  const char* ICDDA::getTrackDuration_event = "ICDDA::getTrackDuration";
  const char* ICDDA::getTotalDuration_event = "ICDDA::getTotalDuration";
  const char* ICDDA::isFinished_event = "ICDDA::isFinished";
  const char* ICDDA::setRepeat_event = "ICDDA::setRepeat";
  const char* ICDDA::getRepeat_event = "ICDDA::getRepeat";
  const char* ICDDA::setOneTrackOnly_event = "ICDDA::setOneTrackOnly";
  const char* ICDDA::getOneTrackOnly_event = "ICDDA::getOneTrackOnly";


}


#include "nmm/nmm_suffix.hpp"

