/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ICDDB.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ICDDB::used() {} 
  
  
  Result ICDDB::setCDDBServer(const string& host, const nmm_int32& port)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > host_value(host, false);
    NMM::TInValue<nmm_int32 > port_value(port, false);
    NMM::Event event("ICDDB::setCDDBServer", &host_value, &port_value);
    event.setReturnValue(&return_value);
    if(const_cast<ICDDB*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string ICDDB::getTrackName(const nmm_int32& track)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::TInValue<nmm_int32 > track_value(track, false);
    NMM::Event event("ICDDB::getTrackName", &track_value);
    event.setReturnValue(&return_value);
    if(const_cast<ICDDB*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string ICDDB::getDiscTitle()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("ICDDB::getDiscTitle");
    event.setReturnValue(&return_value);
    if(const_cast<ICDDB*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string ICDDB::getDiscArtist()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("ICDDB::getDiscArtist");
    event.setReturnValue(&return_value);
    if(const_cast<ICDDB*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ICDDB::setCDDB(const nmm_bool& enabled)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > enabled_value(enabled, false);
    NMM::Event event("ICDDB::setCDDB", &enabled_value);
    if(const_cast<ICDDB*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  bool ICDDB::getCDDB()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<bool > return_value;
    NMM::Event event("ICDDB::getCDDB");
    event.setReturnValue(&return_value);
    if(const_cast<ICDDB*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ICDDB::ICDDB_type = "ICDDB";
  NMM::TConstructor<ICDDB > __ICDDB_prototype("ICDDB","NMM");


  const char* ICDDB::setCDDBServer_event = "ICDDB::setCDDBServer";
  const char* ICDDB::getTrackName_event = "ICDDB::getTrackName";
  const char* ICDDB::getDiscTitle_event = "ICDDB::getDiscTitle";
  const char* ICDDB::getDiscArtist_event = "ICDDB::getDiscArtist";
  const char* ICDDB::setCDDB_event = "ICDDB::setCDDB";
  const char* ICDDB::getCDDB_event = "ICDDB::getCDDB";


}


#include "nmm/nmm_suffix.hpp"

