/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ICDDB_HPP
#define NMM_ICDDB_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{

  class DLLEXPORT ICDDB : virtual public NMM::Interface { 
  public:
    ICDDB() : NMM::Interface(ICDDB::ICDDB_type, "NMM") { };

    ICDDB(NMM::NMMObject& parent) : NMM::Interface (parent, ICDDB::ICDDB_type, "NMM") { };

    virtual ~ICDDB() {};

    static const char* ICDDB_type;

    virtual  string getInterfaceType() const { return ICDDB_type; }

    NMM::Interface* copy() const { return new ICDDB(*this); }
    static void used(); 
    static const char* setCDDBServer_event;
    static const char* getTrackName_event;
    static const char* getDiscTitle_event;
    static const char* getDiscArtist_event;
    static const char* setCDDB_event;
    static const char* getCDDB_event;

    virtual Result setCDDBServer(const string& host, const nmm_int32& port)
      throw(const NMM::SerializedException&);

    virtual string getTrackName(const nmm_int32& track)
      throw(const NMM::SerializedException&);

    virtual string getDiscTitle()
      throw(const NMM::SerializedException&);

    virtual string getDiscArtist()
      throw(const NMM::SerializedException&);

    virtual void setCDDB(const nmm_bool& enabled)
      throw(const NMM::SerializedException&);

    virtual bool getCDDB()
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ICDDB> ICDDB_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
