/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include <stdio.h>

#include "nmm/utils/NamedObject.hpp"

#include "TableOfContents.hpp"

namespace NMM {

  TableOfContents::TableOfContents() {
    title = "unknown title";
    artist = "unknown artist";
    size_set = false;
    cddb_server = "";
    cddb_port = 0;
  };

  TableOfContents::~TableOfContents() {
    for (unsigned int i=0; i<size(); i++) {
      delete toc[i];
    }
  };



  unsigned int TableOfContents::size() {
    return toc.size();
  }

  Result TableOfContents::setSize(unsigned int size) {
    if (size_set) {
      return FAILURE;
    }

    char track_no[3];
    toc = vector<TocEntry*>(size);
    for (unsigned int i=0; i<toc.size(); i++) {
      toc[i] = new TocEntry();
      toc[i]->is_audio = true;
      
      sprintf(track_no,"%02d",i+1);
      toc[i]->name = "Track" + string(track_no);
    }
    
    size_set = true;
    return SUCCESS;
  }


  void TableOfContents::setStartSector(int num, long sec) {
//      if ((num < 0) || (num >= toc.size()))
//        throw OutOfTocList;

     toc[num]->start_sector = sec;
  }; // setStartSector

  long TableOfContents::getStartSector(int num) {
//      if ((num < 0) || (num >= size()))
//        return -1;

    return toc[num]->start_sector;
  }; // setStartSector


  void TableOfContents::setEndSector(int num, long sec) {
//      if ((num < 0) || (num >= toc.size()))
//        throw OutOfTocList;

    toc[num]->end_sector = sec;
  }; // setEndSector

  long TableOfContents::getEndSector(int num) {
//      if ((num < 0) || (num >= toc.size()))
//        throw OutOfTocList;

    return toc[num]->end_sector;
  }; // getEndSector



  long TableOfContents::getSectors(int num) {
//      if ((num < 0) || (num >= toc.size()))
//        throw OutOfTocList;

    return (toc[num]->end_sector - toc[num]->start_sector + 1);
  }; // getSectors


  long TableOfContents::getSeconds(int num) {
//      if ((num < 0) || (num >= toc.size()))
//        throw OutOfTocList;

    return (long) (getSectors(num)/75);
  }; // getSeconds


  void TableOfContents::setName(int num, string name) {
//      if ((num < 0) || (num >= toc.size()))
//        throw OutOfTocList;

    toc[num]->name = name;
  }; // setName

  string TableOfContents::getName(int num) {
//      if ((num < 0) || (num >= toc.size()))
//        throw OutOfTocList;

    return toc[num]->name;
  }; // setName

  string TableOfContents::getYear() {
    return year;
  }
   
  string TableOfContents::getGenre() {
    return genre;
  }

  string TableOfContents::getTitle() {
//      if ((num < 0) || (num >= toc.size()))
//        throw OutOfTocList;

    return title;
  }; // getTitle


  string TableOfContents::getArtist() {
//      if ((num < 0) || (num >= toc.size()))
//        throw OutOfTocList;

    return artist;
  }; // setArtist


  void TableOfContents::setAudio(int num, bool audio) {
//      if ((num < 0) || (num >= toc.size()))
//        throw OutOfTocList;

    toc[num]->is_audio = audio;
  }; // setName

  bool TableOfContents::getAudio(int num) {
//      if ((num < 0) || (num >= toc.size()))
//        throw OutOfTocList;

    return toc[num]->is_audio;
  }; // setName


  int TableOfContents::getTrackBySector(long sector) {
    if (getEndSector(size()-1) < sector ||
        getStartSector(0) > sector) {
      return -1;
    }

    int track=0;
    for(unsigned int i = 0; i < size(); i++) {
      if (sector >= getStartSector(i)) {
          track = i;
      } else {
        break;
      }
    }
    return track;
  }; // getTrackBySector


  string TableOfContents::toString() {
    string ret = "Title: \"" + title + "\"\n" + "Artist: \"" + artist + "\"\n";
    
    for (unsigned int i=0; i<toc.size(); i++) {
      char tmp_char[256];
      string is_it_audio;
      sprintf(tmp_char,
              "%02d\t[%d:%02d]\t%01ld\t%01ld\t",
              i+1,
              (int) ((getSectors(i)/75)/60),
              (int) (getSectors(i)/75)%60,
              toc[i]->start_sector,
              toc[i]->end_sector);
      if (toc[i]->is_audio) {
        is_it_audio = "Audio";
      } else {
        is_it_audio = "Data";
      }

      ret = ret 
        + string(tmp_char) 
        + is_it_audio 
        + "\t\""
        + toc[i]->name
        + "\"\n";
    }

    return string(ret.begin(),ret.end()-1);
  }; // toString



  long TableOfContents::getCDDBFirstFrames() {
    return (150 + toc[0]->start_sector);
  }; // getCDDBFirstFrames

  long TableOfContents::getCDDBFrames(int num) {
    return (151 + toc[num]->end_sector);
  }; // getCDDBFrames

  long TableOfContents::getCDDBFirstSeconds() {
    return (long) (getCDDBFirstFrames()/75);
  }; // getCDDBFirstSeconds

  long TableOfContents::getCDDBSeconds(int num) {
    return (long) (getCDDBFrames(num)/75);
  }; // getCDDBSeconds


  Result TableOfContents::queryCDDBServer() {
    CDDBServer cddb(cddb_server, cddb_port);

    cddb.addTrack(getCDDBFirstSeconds(), getCDDBFirstFrames());
    for (unsigned int i=0; i<size(); i++) {
      cddb.addTrack(getCDDBSeconds(i), getCDDBFrames(i));
    }

    if (cddb.doQuery() == FAILURE) {
      GLOBAL_ERROR_STREAM("Failure querying server " << cddb_server << " on port " << cddb_port << "!" << endl);
    } else {
      title = cddb.getTitle();
      artist = cddb.getArtist();
      year = cddb.getYear();
      genre = cddb.getGenre();
      for (unsigned int i=0; i < size(); i++) {
	setName(i, cddb.getName(i));
      }
    }

    return SUCCESS;
  }


  void TableOfContents::addCDDBServer(string s, int p) {
    cddb_server = s;
    cddb_port = p;
  } 

} // namespace NMM
