/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

/*
 * Code based on example (play_title.c, Copyright (C) 2001 Billy Biggs <vektor@dumbterm.net>)
 * of libdvdread package
 */

#include <iostream>

#include "DVDNavReadNode.hpp"
#include "nmm/plugins/NMMRegisterPlugin.hpp"
#include "nmm/comm/format/Property.hpp"
#include "nmm/multimedia/sync/interfaces/ISynchronizedSink.hpp"
#include "nmm/multimedia/interfaces/ITrack.hpp"
#include "nmm/plugins/interfaces/IStillFrame.hpp"
//#include "nmm/interfaces/video/filter/IOverlay.hpp"
#include "nmm/plugins/av/mpeg/IMPEGDemux.hpp"
#include "nmm/plugins/file/dvd/interfaces/ISPUDecoder.hpp"
#include "nmm/multimedia/interfaces/IProgressListener.hpp"
#include "nmm/utils/thread/MutexGuard.hpp"
#include "nmm/multimedia/FlushEvent.hpp"
#include "nmm/comm/messaging/CEvent.hpp"
#include "nmm/comm/messaging/Event.hpp"

NMM_REGISTER_PLUGIN(DVDNavReadNode, NMM, "DVDNavReadNode")


#include "nmm/nmm_prefix.hpp"

namespace NMM {

  DVDNavReadNode::DVDNavReadNode(const char * name /* = "DVDNavReadNode" */,
                                 StreamQueue::Mode mode /* = StreamQueue::MODE_SUSPEND */,
                                 int queue_size /* = StreamQueue::DEFAULT_SIZE */)
    : GenericSourceNode(name, mode, queue_size),
      domain(DOMAIN_UNKNOWN),
      devicename("/dev/cdrom"),
      dvdnav(NULL),
      sendevent(false),
      send_sync_reset_ok(false),
      send_flush(false),
      need_buttons(false),
      selected_button(0),
      send_selected_button(false),
      sec_sleep(0),
      is_still_frame(false),
      dvd_event(0),
      last_event(false),
      saved_buffer(NULL),
      send_progress(false),
      buffers_before_progress(100),
      buffers_progress(0),
      dvdnav_wait(false)

  {}

  DVDNavReadNode::~DVDNavReadNode() {
  }

  Result DVDNavReadNode::doInit() {


    /* Begin> Init output property */
    Format* format;
    Property* property = getOwnOutputProperty();

    // init output property
    // simple mpeg format
    format = property->addNewFormat("av/mpegps");
    format->addIntParamValue(Format::version_param, 1);
    format->addIntParamValue(Format::version_param, 2);
    property->setDefaultFormat(format);

    // detailed mpeg format
    format = property->addNewFormat("av/mpegps");
    format->addIntParamValue(Format::version_param, 1);
    format->addIntParamValue(Format::version_param, 2);
    // maps of available streams for each stream type
    format->addWildcard(Format::map_mpeg_audio_streams_param);
    format->addWildcard(Format::map_ac3_audio_streams_param);
    format->addWildcard(Format::map_mpeg_video_streams_param);
    format->addWildcard(Format::map_dts_audio_streams_param);
    format->addWildcard(Format::map_lpcm_audio_streams_param);
    format->addWildcard(Format::map_spu_streams_param);

    // formats for every stream type
    Format* sub_format = new Format(Format::audio_mpeg_type);
    sub_format->addIntParamValue(Format::audio_codec_param, 0x55);
    sub_format->addWildcard(Format::layer_param);
    sub_format->addIntParamValue(Format::channels_param, 1);
    sub_format->addIntParamValue(Format::channels_param, 2);
    sub_format->addIntParamValue(Format::sample_rate_param, 8000);
    sub_format->addIntParamValue(Format::sample_rate_param, 11025);
    sub_format->addIntParamValue(Format::sample_rate_param, 12000);
    sub_format->addIntParamValue(Format::sample_rate_param, 16000);
    sub_format->addIntParamValue(Format::sample_rate_param, 22050);
    sub_format->addIntParamValue(Format::sample_rate_param, 44100);
    sub_format->addIntParamValue(Format::sample_rate_param, 48000);
    sub_format->addIntParamValue(Format::bitpersample_param, 8);
    sub_format->addIntParamValue(Format::bitpersample_param, 16);
    format->addFPointerParamValue(Format::format_mpeg_audio_streams_param, sub_format);
    sub_format = new Format(Format::audio_mpeg_type);
    sub_format->addIntParamValue(Format::audio_codec_param, 0x55);
    sub_format->addWildcard(Format::layer_param);
    sub_format->addWildcard(Format::bitrate_param);
    sub_format->addIntParamValue(Format::channels_param, 1);
    sub_format->addIntParamValue(Format::channels_param, 2);
    sub_format->addIntParamValue(Format::sample_rate_param, 8000);
    sub_format->addIntParamValue(Format::sample_rate_param, 11025);
    sub_format->addIntParamValue(Format::sample_rate_param, 12000);
    sub_format->addIntParamValue(Format::sample_rate_param, 16000);
    sub_format->addIntParamValue(Format::sample_rate_param, 22050);
    sub_format->addIntParamValue(Format::sample_rate_param, 44100);
    sub_format->addIntParamValue(Format::sample_rate_param, 48000);
    sub_format->addIntParamValue(Format::bitpersample_param, 8);
    sub_format->addIntParamValue(Format::bitpersample_param, 16);
    format->addFPointerParamValue(Format::format_mpeg_audio_streams_param, sub_format);
    sub_format = new Format(Format::audio_mpeg_type);
    sub_format->addIntParamValue(Format::audio_codec_param, 0x55);
    sub_format->addWildcard(Format::layer_param);
    sub_format->addWildcard(Format::bitrate_param);
    format->addFPointerParamValue(Format::format_mpeg_audio_streams_param, sub_format);
    sub_format = new Format(Format::audio_mpeg_type);
    sub_format->addIntParamValue(Format::audio_codec_param, 0x55);
    sub_format->addWildcard(Format::layer_param);
    format->addFPointerParamValue(Format::format_mpeg_audio_streams_param, sub_format);

    sub_format = new Format("audio/ac3");
    sub_format->addWildcard(Format::bitrate_param);
    format->addFPointerParamValue(Format::format_ac3_audio_streams_param, sub_format);
    sub_format = new Format("audio/ac3");
    format->addFPointerParamValue(Format::format_ac3_audio_streams_param, sub_format);

    sub_format = new Format("video/mpeg");
    sub_format->addIntParamValue(Format::version_param, 1);
    sub_format->addIntParamValue(Format::version_param, 2);
    sub_format->addWildcard(Format::x_resolution_param);
    sub_format->addWildcard(Format::y_resolution_param);
    sub_format->addWildcard(Format::ratio_x_param);
    sub_format->addWildcard(Format::ratio_y_param);
    sub_format->addWildcard(Format::framerate_param);
    format->addFPointerParamValue(Format::format_mpeg_video_streams_param, sub_format);
    sub_format = new Format("video/mpeg");
    sub_format->addIntParamValue(Format::version_param, 1);
    sub_format->addIntParamValue(Format::version_param, 2);
    format->addFPointerParamValue(Format::format_mpeg_video_streams_param, sub_format);

    sub_format = new Format("audio/dts");
    format->addFPointerParamValue(Format::format_dts_audio_streams_param, sub_format);

    sub_format = new Format("audio/lpcm");
    format->addFPointerParamValue(Format::format_lpcm_audio_streams_param, sub_format);

    sub_format = new Format("video/spu");
    format->addFPointerParamValue(Format::format_spu_streams_param, sub_format);

    // simple mpeg video format
    format = property->addNewFormat("video/mpeg");
    format->addIntParamValue(Format::version_param, 1);
    format->addIntParamValue(Format::version_param, 2);

    // detailed mpeg video format
    format = property->addNewFormat("video/mpeg");
    format->addIntParamValue(Format::version_param, 1);
    format->addIntParamValue(Format::version_param, 2);
    format->addWildcard(Format::x_resolution_param);
    format->addWildcard(Format::y_resolution_param);
    format->addWildcard(Format::ratio_x_param);
    format->addWildcard(Format::ratio_y_param);
    format->addWildcard(Format::framerate_param);
    /* End< init output property */

    return SUCCESS;
  }

  Result DVDNavReadNode::doDeinitOutput() {
    if (dvdnav) {
      dvdnav_close( dvdnav ); //cause a segfault if dvdvnav is 0
      dvdnav = NULL;
    }

    return SUCCESS;
  }

  Result DVDNavReadNode::doInitOutput() {
    bool   mpeg_audio;
    bool   ac3_audio;
    bool   mpeg2_video;
    bool   mpeg1_video;
    int    width;
    int    height;
    int    frame_rate_code;
    double frame_rate;
    int    ratio_x;
    int    ratio_y;

    dvdnav_status_t  dvd_status;
    selected_button = 0;

    /*Begin> Open the disc */
    if ( dvdnav ) {
       dvdnav_close( dvdnav );
       dvdnav = NULL;
    }

    dvd_status = dvdnav_open( &dvdnav, devicename.c_str() );

    if ( dvd_status == DVDNAV_STATUS_ERR )
    	return FAILURE;
    /*End< */

    dvdnav_set_readahead_flag( dvdnav, 1 );

    need_buttons = true;

    send_flush = false;
    sendevent  = false;
    send_sync_reset_ok = false;
    sec_sleep = 0;
    is_still_frame = false;

    domain = DOMAIN_UNKNOWN;

    /* Begin> Check the Format of the MPEG stream */
    if (CheckFormat(mpeg_audio,
		    ac3_audio,
		    mpeg2_video,
		    mpeg1_video,
		    width,
		    height,
		    frame_rate_code,
		    frame_rate,
		    ratio_x,
		    ratio_y) == FAILURE) {
      ERROR_STREAM("Failure while checking format" << endl);
      return FAILURE;
    }
    /* End< Check the format of the MPEG stream */

    if ( mpeg_audio )  DEBUG_STREAM("Found MPEG Audio Stream" << endl);
    if ( ac3_audio )   DEBUG_STREAM("Found AC3 Audio Stream" << endl);
    if ( mpeg2_video ) DEBUG_STREAM("Found MPEG2 Video Stream" << endl);
    if ( mpeg1_video ) DEBUG_STREAM("Found MPEG1 Video Stream" << endl);

    // XXX: set the masks

    /* Begin> Setup working output property */
    DEBUG_STREAM("Found MPEG program stream" << endl);
    Format *format = getOwnWorkingOutputProperty()->addNewFormat("av/mpegps");
    format->addIntParamValue("version", mpeg1_video ? 1 : 2);
    getOwnWorkingOutputProperty()->setDefaultFormat(format);
    /* End< */

    return SUCCESS;
  }

  DVDNavReadNode::domain_e DVDNavReadNode::checkDomain() {
    if ( dvdnav_is_domain_vts(dvdnav) == 1 ) {
      return( DOMAIN_VTS );
    }
    if ( dvdnav_is_domain_fp(dvdnav) == 1 ) {
      return( DOMAIN_FP );
    }
    if ( dvdnav_is_domain_vmgm(dvdnav) == 1 ) {
      return( DOMAIN_VMGM );
    }
    if ( dvdnav_is_domain_vtsm(dvdnav) == 1 ) {
      return( DOMAIN_VTSM );
    }

    return( DOMAIN_UNKNOWN );
  }

  void DVDNavReadNode::setDevice(const string& value) throw()
  {
    devicename = value;
    DEBUG_STREAM("Set device to " << devicename << endl);
  }

  string DVDNavReadNode::getDevice() const {
    return devicename;
  }

  Result DVDNavReadNode::select(const nmm_uint32& title, const nmm_uint32& chapter, const nmm_uint32& angle) {
    MutexGuard process_guard(process_buffer_mutex);

    if (DVDNAV_STATUS_ERR == dvdnav_part_play( dvdnav, title, chapter )) {
      ERROR_STREAM("Failed to select title/chapter " << title << "/" << chapter << endl);
      return FAILURE;
    }
    if (DVDNAV_STATUS_ERR == dvdnav_angle_change( dvdnav, angle )) {
      ERROR_STREAM("Failed to select view angle " << angle << endl);
      return FAILURE;
    }

    last_event = false;

    return SUCCESS;
  }

  nmm_uint32 DVDNavReadNode::getCurrentChapter() const {
    int title, chapter;
    if (SUCCESS == getInfo(&title, &chapter)) {
      return chapter;
    } else {
      GLOBAL_DEBUG_STREAM("Could not get current chapter");
      return 0;
    }
  }

  nmm_uint32 DVDNavReadNode::getCurrentTitle() const {
    int title, chapter;
    if (SUCCESS == getInfo(&title, &chapter)) {
      return title;
    } else {
      GLOBAL_DEBUG_STREAM("Could not get current title");
      return 0;
    }
  }

  nmm_uint32 DVDNavReadNode::getTitles() const {
    int32_t titles;
    dvdnav_get_number_of_titles( dvdnav, &titles );
    return ( titles );
  }

  nmm_uint32 DVDNavReadNode::getChapters(const nmm_uint32& title) const {
    int32_t chapters;
    dvdnav_get_number_of_parts( dvdnav, title, &chapters );
    return ( chapters );
  }

  nmm_uint32 DVDNavReadNode::getAngles(const nmm_uint32& title) const {
    if (!title || title != getCurrentTitle()) {
      ERROR_STREAM("Sorry, not implemented: Getting the number of angles in a title which is not the current title" << endl);
      return 0;
    }

    int32_t current, count;
    dvdnav_get_angle_info( dvdnav, &current, &count );
    return ( count );
  }

  Result DVDNavReadNode::selectButton(const nmm_int32& button) {
    pci_t* pci = dvdnav_get_current_nav_pci(dvdnav);
    if ( dvdnav_button_select(dvdnav, pci, button) == DVDNAV_STATUS_OK ) {
      selected_button = button;
      send_selected_button = true;
      return SUCCESS;
    }

    return FAILURE;
  }

  Result DVDNavReadNode::activateButton(const nmm_int32& button) {
    pci_t* pci = dvdnav_get_current_nav_pci(dvdnav);
    if ( dvdnav_button_select_and_activate(dvdnav, pci, button) == DVDNAV_STATUS_OK ) {
      return SUCCESS;
    }
    return FAILURE;
  }

  Result DVDNavReadNode::activateSelectedButton() {
    pci_t* pci = dvdnav_get_current_nav_pci(dvdnav);
    if ( dvdnav_button_activate( dvdnav, pci ) == DVDNAV_STATUS_OK )
      return SUCCESS;
    return FAILURE;
  }

  Result DVDNavReadNode::goUp() {
    if ( dvdnav_go_up( dvdnav ) == DVDNAV_STATUS_OK ) { return SUCCESS; }
    return FAILURE;
  }

  Result DVDNavReadNode::buttonUp() {
    pci_t* pci = dvdnav_get_current_nav_pci(dvdnav);
    if ( dvdnav_upper_button_select( dvdnav, pci ) == DVDNAV_STATUS_OK ) {
      int32_t selected_button;
      if ( dvdnav_get_current_highlight( dvdnav, &selected_button ) == DVDNAV_STATUS_ERR )
      	selected_button = 0;
      this->selected_button = selected_button;
      send_selected_button = true;
      return SUCCESS;
    }
    return FAILURE;
  }

  Result DVDNavReadNode::buttonDown() {
    pci_t* pci = dvdnav_get_current_nav_pci(dvdnav);
    if ( dvdnav_lower_button_select( dvdnav, pci ) == DVDNAV_STATUS_OK ) {
      int32_t selected_button;
      if ( dvdnav_get_current_highlight( dvdnav, &selected_button ) == DVDNAV_STATUS_ERR )
      	selected_button = 0;
      this->selected_button = selected_button;
      send_selected_button = true;
      return SUCCESS;
    }
    return FAILURE;
  }

  Result DVDNavReadNode::buttonRight() {
    pci_t* pci = dvdnav_get_current_nav_pci(dvdnav);

    if ( dvdnav_right_button_select( dvdnav, pci ) == DVDNAV_STATUS_OK ) {
      int32_t selected_button;
      if ( dvdnav_get_current_highlight( dvdnav, &selected_button ) == DVDNAV_STATUS_ERR )
      	selected_button = 0;
      this->selected_button = selected_button;
      send_selected_button = true;
      return SUCCESS;
    }
    return FAILURE;
  }

  Result DVDNavReadNode::buttonLeft() {
    pci_t* pci = dvdnav_get_current_nav_pci(dvdnav);
    if ( dvdnav_left_button_select( dvdnav, pci ) == DVDNAV_STATUS_OK ) {
      int32_t selected_button;
      if ( dvdnav_get_current_highlight( dvdnav, &selected_button ) == DVDNAV_STATUS_ERR )
      	selected_button = 0;
      this->selected_button = selected_button;
      send_selected_button = true;
      return SUCCESS;
    }
    return FAILURE;
  }

  nmm_int32 DVDNavReadNode::getSelectedButton() {
    int32_t button;
    if ( dvdnav_get_current_highlight( dvdnav, &button ) == DVDNAV_STATUS_ERR )
    	button = 0;
    return( button );
  }


  Result DVDNavReadNode::getInfo( int* title, int* chapter ) const {
    int32_t temp_title;
    int32_t temp_chapter;
    dvdnav_current_title_info(dvdnav, &temp_title, &temp_chapter);
    *title = temp_title;
    *chapter = temp_chapter;
    return( SUCCESS );
  }


  // maybe we want that too ?!
  //   Result DVDNavReadNode::chapterSelect(int title, int chapter, int chapternr) {
  //     MutexGuard process_guard(process_buffer_mutex);

  //     dvdnav_part_play_auto_stop(dvdnav, title, chapter, chapternr);
  //     return SUCCESS;
  //   }

  void DVDNavReadNode::reset() {
    dvdnav_reset( dvdnav );
  }

  void DVDNavReadNode::sendFlush() {
    MutexGuard process_guard(process_buffer_mutex);

    send_flush = true;
  }

  Message* DVDNavReadNode::processBuffer(Buffer *) {
    MutexGuard process_guard(process_buffer_mutex);

    if (getWorkingFlag()) {
      if (send_progress && buffers_progress > buffers_before_progress) {
        buffers_progress = 0;

        uint32_t pos = 0;
        uint32_t len = 0;
        dvdnav_get_position_in_title(dvdnav, &pos, &len);

        CEvent* cevent(new CEvent(IProgressListener::create_setProgress(pos, len)));
        cevent->setMandatoryBit(false);
        return cevent;
      }

      if (dvdnav_wait)
        dvdnav_wait_skip(dvdnav);

      setWorkingFlag(false);
      buffers_progress++;

      /* saved buffer contains some DVDNavBlocks which need to be returned */
      if (dvd_event == DVDNAV_BLOCK_OK) {
        return saved_buffer;
      }
      else {
        return NULL;
      }
    }

    if ( sendevent && send_sync_reset_ok ) {
      if ( send_flush ) {
        send_flush = false;
        return( new FlushEvent() );
      }
      sendevent = false;
      send_sync_reset_ok = false;

      CEvent* event = new CEvent();
      event->insertEvent( 0, Event(ISynchronized::setSynchronized_event, new TInValue<nmm_bool>(true)) );
      event->insertEvent( 1, Event(ISyncReset::syncReset_event) );

      // make sure, progress is send next
      buffers_progress = buffers_before_progress + 1;

      return ( event );
    }

    /* send only button information, if we are in a menu */
    if ( send_selected_button && domain != DOMAIN_VTS ) {
      send_selected_button = false;
      Event event( ISPUDecoder::setDVDSelectedButton_event, 1 );
      event.setValue( 0, new TInValue<nmm_int32>(selected_button-1) );
      return new CEvent( event );
    }


    domain_e new_domain = checkDomain();
    if ( new_domain != domain ) {
      domain = new_domain;
      switch( domain ) {
      case DOMAIN_FP:
        /* we are in the First Play domain. (Menu) */
        //selectButton( 1 );
        //break;

      case DOMAIN_VMGM:
        /* we are in the Video management Menu domain. (Menu) */
        //selectButton( 1 );
        //break;

      case DOMAIN_VTSM:
        /* we are in the Video Title Menu domain (Menu) */
        //selectButton( 1 );
        //break;

      case DOMAIN_VTS:
        /* we are in the Video Title domain (playing movie) */
        break;

      default:
        DEBUG_STREAM2("Unknown domain" << std::endl);
        break;
      }
    }

    if ( sec_sleep ) {
      sleep( sec_sleep );
      sec_sleep = 0;
      dvdnav_still_skip(dvdnav);
      activateSelectedButton();
    }

    Buffer* out;
    int32_t len;
    int     offset=0;
    // fw: get next still frame flag
    int next_still_flag = dvdnav_get_next_still_flag(dvdnav);


    /* are we ready to read some new blocks or do we have an old event to handle? */
    if (!last_event) {
      out = getNewBuffer(21000); //enough space for 10 blocks
      for (int i=1;i<10;++i) {
        dvdnav_get_next_block( dvdnav, (unsigned char*) out->getData()+offset, &dvd_event, &len );
        out->setUsed( len+offset );
        offset+=len;
        /* if we find another event than normal DVDNav_Block we have to do some work */
        if (dvd_event!=DVDNAV_BLOCK_OK) {
	  // trim the out-buffer to the size of the DVDNav-Blocks
          out->setUsed(offset-len);
          send_sync_reset_ok = true;
          last_event = true;
          setWorkingFlag(true);
	  if (dvd_event == DVDNAV_WAIT)
	    dvdnav_wait = true;
          //get a new buffer to backup the last (non DVDNav-Block) event
          saved_buffer = getNewBuffer(len);
          memcpy(saved_buffer->getData(),out->getData()+offset-len,len);
          saved_buffer->setUsed(len);
          //do not return a out buffer with a size of 0, it will lead to errors
          if (out->getUsed()==0) {
            out->release();
            return NULL;
          } else
	    return out;
          break;
        }
      }
    }

    last_event=false;

    switch(dvd_event) {
      case DVDNAV_BLOCK_OK:
      {
	// to avoid sending sync_reset if no data is sent but several cell changes occured
        send_sync_reset_ok = true;
        //saved_buffer = getNewBuffer(offset);
        saved_buffer = out;
        break;
      }
      case DVDNAV_CELL_CHANGE:
      {
	sendevent = true;
	send_sync_reset_ok = false;
        saved_buffer->release();
        return(0);
      }
      case DVDNAV_HOP_CHANNEL:
      {
  	need_buttons = true;
        saved_buffer->release();

	CEvent* event = new CEvent();
	event->insertEvent( 0, Event(ISynchronized::setSynchronized_event, new TInValue<nmm_bool>(true)) );
	//event->insertEvent( 1, Event(IOverlay::disableOverlay_event) );
        return ( event );
      }
      case DVDNAV_STILL_FRAME:
      {
        dvdnav_still_event_t* still_frame = (dvdnav_still_event_t *) saved_buffer->getData();

        if ( still_frame->length != 255 )
          sec_sleep = still_frame->length;
        saved_buffer->release();

        if (is_still_frame) {
          CEvent* event = new CEvent();
          event->insertEvent( 0, Event(ISynchronized::setSynchronized_event, new TInValue<nmm_bool>(false)) );
          event->insertEvent( 1, Event(IStillFrame::setStillFrame_event, new TInValue<nmm_bool>(false)) );

          is_still_frame = false;

          return( event );
        } else {
          return( NULL );
        }
      }
      case DVDNAV_SPU_CLUT_CHANGE:
      {
	CEvent* event = new CEvent();
        unsigned int* clut = (unsigned int*) saved_buffer->getData();
        union {
          u_int32_t i32;
          unsigned char yuva[4];
        } un;
	for( unsigned int i=0; i<16; i++ ) {

          un.i32 = clut[i];

	  Event event0( ISPUDecoder::setPaletteItem_event, 5 );
	  event0.setValue( 0, new TInValue<nmm_int32>(i) );
	  event0.setValue( 1, new TInValue<nmm_int32>(un.yuva[0]) );
	  event0.setValue( 2, new TInValue<nmm_int32>(un.yuva[1]) );
	  event0.setValue( 3, new TInValue<nmm_int32>(un.yuva[2]) );
	  event0.setValue( 4, new TInValue<nmm_int32>(un.yuva[3]) );
	  event->insertEvent( i, event0 );
	}
	saved_buffer->release();
	return( event );
      }
      case DVDNAV_HIGHLIGHT:
      {
	saved_buffer->release();
	return( 0 );
      }
      case DVDNAV_NAV_PACKET:
      {
        hl_gi_t* hl_gi = &dvdnav_get_current_nav_pci(dvdnav)->hli.hl_gi;

        if(hl_gi->btn_ns > 0) {
	  unsigned int button;
	  unsigned int event_nr = 0;

	  if ( need_buttons ) {
	    CEvent* event = new CEvent();

	    unsigned int* btn_coli = (unsigned int*) (&dvdnav_get_current_nav_pci(dvdnav)->hli.btn_colit.btn_coli);
            for( int i=1; i<=3; i++ ) {
	      Event event0( ISPUDecoder::initColorScheme_event, 3 );
	      event0.setValue( 0, new TValue<nmm_int32>( i ) );
	      event0.setValue( 1, new TValue<nmm_int32>( *btn_coli ) );
	      btn_coli++;
	      event0.setValue( 2, new TValue<nmm_int32>( *btn_coli ) );
	      btn_coli++;
	      event->insertEvent( event_nr++, event0 );
	    }

	    Event event0( ISPUDecoder::setDVDButtonCount_event, 1 );
	    event0.setValue( 0, new TValue<nmm_int32>(dvdnav_get_current_nav_pci(dvdnav)->hli.hl_gi.btn_ns) );
	    event->insertEvent( event_nr++, event0 );

	    int auto_select_button   =  hl_gi->fosl_btnn;
	    int auto_activate_button =  hl_gi->foac_btnn;

	    if ( auto_select_button )
	      selectButton( auto_select_button );
	    else
	      selectButton(1);

	    if ( auto_activate_button )
	      activateButton( auto_activate_button );

	    for (button = 0; button < hl_gi->btn_ns; ++button) {
	      btni_t *btni = &(dvdnav_get_current_nav_pci(dvdnav)->hli.btnit[button]);
              Event event1( ISPUDecoder::setDVDButtonCoords_event, 4 );
              event1.setValue( 0, new TValue<nmm_int32>(btni->x_start) );
              event1.setValue( 1, new TValue<nmm_int32>(btni->y_start) );
	      event1.setValue( 2, new TValue<nmm_int32>(btni->x_end) );
	      event1.setValue( 3, new TValue<nmm_int32>(btni->y_end) );
	      event->insertEvent( event_nr++, event1 );

	      Event event2( ISPUDecoder::setDVDButtonScheme_event, 1 );
	      event2.setValue( 0, new TValue<nmm_int32>(btni->btn_coln) );
              event->insertEvent( event_nr++, event2 );
            }

	    need_buttons = false;
	    saved_buffer->release();
	    return( event );
          }
        }

       // fw: it looked like a bug to return the saved_buffer buffer here, and it works well both ways
        saved_buffer->release();
        return 0;
      }
      case DVDNAV_SPU_STREAM_CHANGE:
      {
        dvdnav_spu_stream_change_event_t* spu_stream;
        spu_stream = (dvdnav_spu_stream_change_event_t*) saved_buffer->getData();

	DEBUG_STREAM2("SPU physical mpeg stream number (widescreen): " << spu_stream->physical_wide << endl);
        DEBUG_STREAM2("SPU physical mpeg stream number (letterbox):  " << spu_stream->physical_letterbox << endl);
        DEBUG_STREAM2("SPU physical mpeg stream number (pan&scan):   " << spu_stream->physical_pan_scan << endl);
        DEBUG_STREAM2("SPU logical mpeg stream number:               " << spu_stream->logical << endl);

	saved_buffer->release();

        Event e = Event(IMPEGDemux::setSPUStream_event, new TInValue<nmm_int32>( spu_stream->physical_wide ) );
        return new CEvent(e);
      }
      case DVDNAV_AUDIO_STREAM_CHANGE:
      {
        dvdnav_audio_stream_change_event_t* audio_stream;
	audio_stream = (dvdnav_audio_stream_change_event_t*) saved_buffer->getData();

	audio_stream = (dvdnav_audio_stream_change_event_t*) saved_buffer->getData();

	DEBUG_STREAM2("Audio physical mpeg stream number: " << audio_stream->physical << endl);
        DEBUG_STREAM2("Audio logical mpeg stream number:  " << audio_stream->logical << endl);
	saved_buffer->release();

	Event e = Event(IMPEGDemux::setAudioStream_event, new TInValue<nmm_int32>( audio_stream->physical ) );
        return new CEvent(e);
      }
      case DVDNAV_STOP:
      {
        DEBUG_STREAM("DVD Read stopped" << endl);
        /* so, no produceBuffer will be called */
        setProducingFlag(false);
        saved_buffer->release();

        /* send event */
        return new CEvent( Event( ITrack::endTrack_event ) );
      }
      default:
        saved_buffer->release();
        return( 0 );
    }

    // errror 20030513: changed buffer is always saved and returned in next call to be able to count data buffers properly
    setWorkingFlag(true);

    if (!is_still_frame && next_still_flag) {
      CEvent* event = new CEvent();
      event->insertEvent( 0, Event(IStillFrame::setStillFrame_event, new TInValue<nmm_bool>(true)) );
      is_still_frame = true;

      return event;
    }

    return NULL;
  }

  Result DVDNavReadNode::CheckFormat(bool&   mpeg_audio,
				     bool&   ac3_audio,
				     bool&   mpeg2_video,
				     bool&   mpeg1_video,
				     int&    width,
				     int&    height,
				     int&    frame_rate_code,
				     double& /*frame_rate*/,
				     int&    ratio_x,
				     int&    ratio_y) {

    mpeg_audio      = false;
    ac3_audio       = true;
    mpeg2_video     = true;
    mpeg1_video     = false;
    width           = 0;
    height          = 0;
    frame_rate_code = 0;

    /* check aspect ratio */
    switch( dvdnav_get_video_aspect( dvdnav ) ) {
      // 4:3
      case 0:
        ratio_x = 400;
        ratio_y = 300;
        break;

      // 16:9
      case 2:
      // if there are more values in future, let's have a defined default value
      default:
        ratio_x = 1600;
        ratio_y = 900;
        break;
    }

    return SUCCESS;
  }

  void DVDNavReadNode::sendProgressInformation(const nmm_bool& in) {
    send_progress = in;
  }

  void DVDNavReadNode::setProgressInterval(const nmm_int32& buffers) {
    buffers_before_progress = buffers;
  }

}

#include "nmm/nmm_suffix.hpp"
