/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

/*
 * Code based on example (play_title.c, Copyright (C) 2001 Billy Biggs <vektor@dumbterm.net>)
 * of libdvdread package
 */

#include "DVDReadNode.hpp"
#include "nmm/comm/format/Property.hpp"
#include "nmm/multimedia/ISyncReset.hpp"
#include "nmm/multimedia/interfaces/ITrack.hpp"
#include "nmm/multimedia/interfaces/IProgressListener.hpp"
#include "nmm/plugins/NMMRegisterPlugin.hpp"
#include "nmm/utils/thread/MutexGuard.hpp"
#include <inttypes.h>
#include <stdint.h>

extern "C" {
#include <dvdread/ifo_read.h>
#include <dvdread/nav_read.h>
}

NMM_REGISTER_PLUGIN(DVDReadNode, NMM, "DVDReadNode")

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  DVDReadNode::DVDReadNode(const char * name /* = "DVDReadNode" */,
			   StreamQueue::Mode mode /* = StreamQueue::MODE_SUSPEND */,
			   int queue_size /* = StreamQueue::DEFAULT_SIZE */)
    : GenericSourceNode(name, mode, queue_size),
      __device("/dev/cdrom"),
      __title(0),
      __chapter(0),
      __angle(0),
      __pgc_id(0),
      __cell_finished(false),
      __start_cell(0),
      __current_cell(0),
      __next_cell(0),
      __current_sector(0),
      __ttn(0),
      __pgn(0),
      __read_buffer(),
      __dvd_reader(NULL),
      __dvd_file(NULL),
      __vmg_file(NULL),
      __tt_srpt(NULL),
      __vts_file(NULL),
      __vts_ptt_srpt(NULL),
      __pgc(NULL),
      __select_mutex(),
      __event_state(EVENT_START_TRACK),
      __send_sync_reset(false),
      __send_progress(false),
      __save_buffer(NULL),
      __buffers_before_progress(2),
      __buffers_progress(0),
      __title_length(0),
      __title_position(0)
  {}
  
  Result DVDReadNode::doInit() {    
    /*Begin> Init static output property */
    /* MPEG audio subformat */
    // XXX: fw: not used yet because format analysis is incomplete
//     Format mpeg_audio_format(Format::audio_mpeg_type);
//     mpeg_audio_format.addIntParamValue(Format::audio_codec_param, 0x55);
//     mpeg_audio_format.addIntParamValue(Format::layer_param, 2);
//     mpeg_audio_format.addIntParamValue(Format::layer_param, 3);
//     mpeg_audio_format.addIntParamValue(Format::channels_param, 1);
//     mpeg_audio_format.addIntParamValue(Format::channels_param, 2);
//     mpeg_audio_format.addIntParamValue(Format::sample_rate_param, 8000);
//     mpeg_audio_format.addIntParamValue(Format::sample_rate_param, 11025);
//     mpeg_audio_format.addIntParamValue(Format::sample_rate_param, 12000);
//     mpeg_audio_format.addIntParamValue(Format::sample_rate_param, 16000);
//     mpeg_audio_format.addIntParamValue(Format::sample_rate_param, 22050);
//     mpeg_audio_format.addIntParamValue(Format::sample_rate_param, 44100);
//     mpeg_audio_format.addIntParamValue(Format::sample_rate_param, 48000);
//     mpeg_audio_format.addIntParamValue(Format::bitpersample_param, 8);
//     mpeg_audio_format.addIntParamValue(Format::bitpersample_param, 16);
    
    /* MPEG audio subformat with bitrate */
    // XXX: fw: not used yet because format analysis is incomplete
//     Format mpeg_audio_bitrate_format(Format::audio_mpeg_type);
//     mpeg_audio_bitrate_format.addIntParamValue(Format::audio_codec_param, 0x55);
//     mpeg_audio_bitrate_format.addIntParamValue(Format::layer_param, 2);
//     mpeg_audio_bitrate_format.addIntParamValue(Format::layer_param, 3);
//     mpeg_audio_bitrate_format.addWildcard(Format::bitrate_param);
//     mpeg_audio_bitrate_format.addIntParamValue(Format::channels_param, 1);
//     mpeg_audio_bitrate_format.addIntParamValue(Format::channels_param, 2);
//     mpeg_audio_bitrate_format.addIntParamValue(Format::sample_rate_param, 8000);
//     mpeg_audio_bitrate_format.addIntParamValue(Format::sample_rate_param, 11025);
//     mpeg_audio_bitrate_format.addIntParamValue(Format::sample_rate_param, 12000);
//     mpeg_audio_bitrate_format.addIntParamValue(Format::sample_rate_param, 16000);
//     mpeg_audio_bitrate_format.addIntParamValue(Format::sample_rate_param, 22050);
//     mpeg_audio_bitrate_format.addIntParamValue(Format::sample_rate_param, 44100);
//     mpeg_audio_bitrate_format.addIntParamValue(Format::sample_rate_param, 48000);
//     mpeg_audio_bitrate_format.addIntParamValue(Format::bitpersample_param, 8);
//     mpeg_audio_bitrate_format.addIntParamValue(Format::bitpersample_param, 16);
    
    /* Simple MPEG audio subformat */
    // XXX: fw: not used yet because format analysis is incomplete
//     Format mpeg_audio_simple_format(Format::audio_mpeg_type);
//     mpeg_audio_simple_format.addIntParamValue(Format::audio_codec_param, 0x55);
//     mpeg_audio_simple_format.addIntParamValue(Format::layer_param, 2);
//     mpeg_audio_simple_format.addIntParamValue(Format::layer_param, 3);
    
    /* Simple MPEG audio subformat with bitrate */
    // XXX: fw: not used yet because format analysis is incomplete
//     Format mpeg_audio_simple_bitrate_format(Format::audio_mpeg_type);
//     mpeg_audio_simple_bitrate_format.addIntParamValue(Format::audio_codec_param, 0x55);
//     mpeg_audio_simple_bitrate_format.addIntParamValue(Format::layer_param, 2);
//     mpeg_audio_simple_bitrate_format.addIntParamValue(Format::layer_param, 3);
//     mpeg_audio_simple_bitrate_format.addWildcard(Format::bitrate_param);
    
    /* AC3 audio subformat */
    // XXX: fw: not used yet because format analysis is incomplete
//     Format ac3_audio_format(Format::audio_ac3_type);

    /* AC3 audio subformat with bitrate */
    // XXX: fw: not used yet because format analysis is incomplete
//     Format ac3_audio_format(Format::audio_ac3_type);
//     ac3_audio_format.addWildcard(Format::bitrate_param);
    
    /* MPEG video subformat */
    Format mpeg_video_format(Format::video_mpeg_type);
    mpeg_video_format.addIntParamValue(Format::version_param, 1);
    mpeg_video_format.addIntParamValue(Format::version_param, 2);
    mpeg_video_format.addWildcard(Format::x_resolution_param);
    mpeg_video_format.addWildcard(Format::y_resolution_param);
    mpeg_video_format.addWildcard(Format::ratio_x_param);
    mpeg_video_format.addWildcard(Format::ratio_y_param);
    mpeg_video_format.addWildcard(Format::framerate_param);

    /* MPEG video subformat with bitrate */
    // XXX: fw: not used yet because format analysis is incomplete
//     Format mpeg_video_bitrate_format(Format::video_mpeg_type);
//     mpeg_video_bitrate_format.addIntParamValue(Format::version_param, 1);
//     mpeg_video_bitrate_format.addIntParamValue(Format::version_param, 2);
//     mpeg_video_bitrate_format.addWildcard(Format::x_resolution_param);
//     mpeg_video_bitrate_format.addWildcard(Format::y_resolution_param);
//     mpeg_video_bitrate_format.addWildcard(Format::ratio_x_param);
//     mpeg_video_bitrate_format.addWildcard(Format::ratio_y_param);
//     mpeg_video_bitrate_format.addWildcard(Format::framerate_param);
//     mpeg_video_bitrate_format.addWildcard(Format::bitrate_param);
    
    /* Simple MPEG video subformat */
    // XXX: fw: not used yet because format analysis is incomplete
//     Format mpeg_video_simple_format(Format::video_mpeg_type);
//     mpeg_video_simple_format.addIntParamValue(Format::version_param, 1);
//     mpeg_video_simple_format.addIntParamValue(Format::version_param, 2);

    /* Simple MPEG video subformat with bitrate */
    // XXX: fw: not used yet because format analysis is incomplete
//     Format mpeg_video_simple_bitrate_format(Format::video_mpeg_type);
//     mpeg_video_simple_bitrate_format.addIntParamValue(Format::version_param, 1);
//     mpeg_video_simple_bitrate_format.addIntParamValue(Format::version_param, 2);
//     mpeg_video_simple_bitrate_format.addWildcard(Format::bitrate_param);
    
    /* Dummy DTS audio subformat */
    // XXX: fw: not used yet because format analysis is incomplete
//     Format dts_audio_format(Format::audio_dts_type);
    
    /* Dummy LPCM audio subformat */
    // XXX: fw: not used yet because format analysis is incomplete
//     Format lpcm_audio_format(Format::audio_lpcm_type);
    
    /* SPU subformat */
    // XXX: fw: not used yet because format analysis is incomplete
//     Format spu_format(Format::video_spu_type);
    
    /* MPEG program stream format */
    // XXX: fw: not used yet because format analysis is incomplete
//     Format mpegps_format(Format::av_mpegps_type);
//     mpegps_format.addIntParamValue(Format::version_param, 1);
//     mpegps_format.addIntParamValue(Format::version_param, 2);
//     mpegps_format.addWildcard(Format::map_mpeg_audio_streams_param);
//     mpegps_format.addWildcard(Format::map_ac3_audio_streams_param);
//     mpegps_format.addWildcard(Format::map_mpeg_video_streams_param);
//     mpegps_format.addWildcard(Format::map_dts_audio_streams_param);
//     mpegps_format.addWildcard(Format::map_lpcm_audio_streams_param);
//     mpegps_format.addWildcard(Format::map_spu_streams_param);
//     mpegps_format.addFPointerParamValue(Format::format_mpeg_audio_streams_param, new Format(mpeg_audio_format));
//     mpegps_format.addFPointerParamValue(Format::format_mpeg_audio_streams_param, new Format(mpeg_audio_bitrate_format));
//     mpegps_format.addFPointerParamValue(Format::format_mpeg_audio_streams_param, new Format(mpeg_audio_simple_format));
//     mpegps_format.addFPointerParamValue(Format::format_mpeg_audio_streams_param, new Format(mpeg_audio_simple_bitrate_format));
//     mpegps_format.addFPointerParamValue(Format::format_ac3_audio_streams_param, new Format(ac3_audio_format));
//     mpegps_format.addFPointerParamValue(Format::format_ac3_audio_streams_param, new Format(ac3_audio_bitrate_format));
//     mpegps_format.addFPointerParamValue(Format::format_mpeg_video_streams_param, new Format(mpeg_video_format));
//     mpegps_format.addFPointerParamValue(Format::format_mpeg_video_streams_param, new Format(mpeg_video_bitrate_format));
//     mpegps_format.addFPointerParamValue(Format::format_mpeg_video_streams_param, new Format(mpeg_video_simple_format));
//     mpegps_format.addFPointerParamValue(Format::format_mpeg_video_streams_param, new Format(mpeg_video_simple_bitrate_format));
//     mpegps_format.addFPointerParamValue(Format::format_dts_audio_streams_param, new Format(dts_audio_format));
//     mpegps_format.addFPointerParamValue(Format::format_lpcm_audio_streams_param, new Format(lpcm_audio_format));
//     mpegps_format.addFPointerParamValue(Format::format_spu_streams_param, new Format(spu_format));
    
    /* Simple MPEG program stream format */
    Format mpegps_simple_format(Format::av_mpegps_type);
    mpegps_simple_format.addIntParamValue(Format::version_param, 1);
    mpegps_simple_format.addIntParamValue(Format::version_param, 2);
    
    /* Add output formats */
    // XXX: fw: not used yet because format analysis is incomplete
//     getOwnOutputProperty()->addNewFormat(new Format(mpegps_format));
    getOwnOutputProperty()->addNewFormat(new Format(mpegps_simple_format));
    getOwnOutputProperty()->addNewFormat(new Format(mpeg_video_format));
    // XXX: fw: not used yet because format analysis is incomplete
//     getOwnOutputProperty()->addNewFormat(new Format(mpeg_video_simple_format));
//     getOwnOutputProperty()->addNewFormat(new Format(mpeg_video_bitrate_format));
//     getOwnOutputProperty()->addNewFormat(new Format(mpeg_video_simple_bitrate_format));
    /*End< */
    
    return SUCCESS;
  }
  
  Result DVDReadNode::doInitOutput() {
    /* First try to open the DVD */
    if (FAILURE == openDVD()) {
      return FAILURE;
    }
    
    try {  
      /*Begin> Check the Format of the MPEG stream */
      bool   pes_stream;
      bool   mpeg_audio;
      bool   ac3_audio;
      bool   mpeg2_video;
      bool   mpeg1_video;
      int    width;
      int    height;
      int    frame_rate_code;
      double frame_rate;
      int    ratio_x;
      int    ratio_y;
      if (FAILURE == checkFormat(pes_stream,
                                 mpeg_audio,
                                 ac3_audio,
                                 mpeg2_video,
                                 mpeg1_video,
                                 width,
                                 height,
                                 frame_rate_code,
                                 frame_rate,
                                 ratio_x,
                                 ratio_y)) {
        throw Exception("Failed to check DVD format");
      }
      /*End< */
      
      /*Begin> Setup working output property */
      if (!pes_stream) {
        DEBUG_STREAM("Output format is MPEG program stream" << endl);
        
        /* Simple MPEG program stream format */
        Format *mpegps_simple_format = getOwnWorkingOutputProperty()->addNewFormat(Format::av_mpegps_type);
        mpegps_simple_format->addIntParamValue(Format::version_param, mpeg1_video ? 1 : 2);
        getOwnWorkingOutputProperty()->setDefaultFormat(mpegps_simple_format);
        
        // XXX: fw: determine the stream maps
        // XXX: fw: determine the sub formats
      } else {
        DEBUG_STREAM("Output format is MPEG packed elementary stream" << endl);
        
        /* MPEG video format */
        Format* format = getOwnWorkingOutputProperty()->addNewFormat(Format::video_mpeg_type);
        format->addIntParamValue(Format::version_param, mpeg1_video ? 1 : 2);
        format->addIntParamValue(Format::x_resolution_param, width);
        format->addIntParamValue(Format::y_resolution_param, height);
        format->addIntParamValue(Format::ratio_x_param, ratio_x);
        format->addIntParamValue(Format::ratio_y_param, ratio_y);
        format->addFloatParamValue(Format::framerate_param, frame_rate);
        getOwnWorkingOutputProperty()->setDefaultFormat(format);
      }
      /*End< */
      
      /* Reset the DVD to the first title/chapter/view angle */
      reset();
      
      return SUCCESS;
    } catch(const Exception& e) {
      ERROR_STREAM(e.getComment() << endl);
      closeDVD();
      return FAILURE;
    }
  }

  Result DVDReadNode::doDeinitOutput() {
    closeDVD();
    return SUCCESS;
  }
  
  Result DVDReadNode::doActivate() {
    /* Init event state */
    __send_sync_reset = false;
    __event_state = EVENT_START_TRACK;
    
    /* Allocate read buffer */
    __read_buffer = vector<unsigned char>(1024 * DVD_VIDEO_LB_LEN);
    
    return SUCCESS;
  }
  
  Result DVDReadNode::doDeactivate() {
    /* Deallocate read buffer */
    __read_buffer = vector<unsigned char>();
    
    return SUCCESS;
  }
  
  nmm_uint32 DVDReadNode::getCurrentTitle() const {
    return __title;
  }
  
  nmm_uint32 DVDReadNode::getCurrentChapter() const {
    return __chapter;
  }
  
  nmm_uint32 DVDReadNode::getTitles() const {
    return (__tt_srpt->nr_of_srpts);
  }
  
  nmm_uint32 DVDReadNode::getChapters(const nmm_uint32& title) const {
    if ( title < 1 || title > __tt_srpt->nr_of_srpts ) { return 0; }
    return ( __tt_srpt->title[ title-1 ].nr_of_ptts );
  }
  
  nmm_uint32 DVDReadNode::getAngles(const nmm_uint32& title) const {
    if ( title < 1 || title > __tt_srpt->nr_of_srpts ) { return 0; }
    return ( __tt_srpt->title[ title-1 ].nr_of_angles );  
  }
  
  Result DVDReadNode::select(const nmm_uint32& title, const nmm_uint32& chapter, const nmm_uint32& angle) {
    MutexGuard guard(__select_mutex);
    DEBUG_STREAM("Select title " << title << " chapter " << chapter << " angle " << angle << endl);
    
    /* Make sure the title is valid */
    if(title < 1 || title > __tt_srpt->nr_of_srpts) {
      ERROR_STREAM( "Invalid title " << title << endl );
      return FAILURE;
    }
    
    /* Make sure the angle number is valid for the title we want */
    if(angle < 1 || angle > __tt_srpt->title[title-1].nr_of_angles) {
      ERROR_STREAM( "Invalid view angle " << angle << endl );
      return FAILURE;
    }
    
    /* Make sure the chapter number is valid for the title we want */
    if(chapter < 1 || chapter > __tt_srpt->title[title-1].nr_of_ptts ) {
      ERROR_STREAM( "Invalid chapter " << chapter << endl );
      return FAILURE;
    }
    
    /* Check if title has changed */
    if (__dvd_file == NULL || __title != title) {
      /* Open the requested title */
      if (FAILURE == openDVDTitle(title)) {
        return FAILURE;
      }
    }
    
    /* Set new view angle */
    __angle = angle;
    
    /* Set new chapter */
    __chapter = chapter;
    
    /* Determine which program chain we want to watch. This is based on the chapter number. */
    __pgc_id = __vts_ptt_srpt->title[__ttn - 1].ptt[__chapter - 1].pgcn;
    __pgn = __vts_ptt_srpt->title[__ttn - 1].ptt[__chapter - 1].pgn;
    __pgc = __vts_file->vts_pgcit->pgci_srp[__pgc_id - 1].pgc;
    __start_cell = __pgc->program_map[__pgn - 1] - 1;
    
    /* Compute the length of the title and the starting position */
    __title_length = 0;
    __title_position = 0;
    __next_cell = 0;
    while(__next_cell<__pgc->nr_of_cells) {
      if (!nextCell()) {
        /* Something's wrong... */
        __title_length = 0;
        __title_position = 0;
        break;
      }
      
      /* If this is the cell where we start to play, set the position accordingly */
      if (static_cast<int>(__current_cell) == __start_cell) {
        __title_position = __title_length;
      }
      
      /* Count sectors of this cell */
      __title_length += __pgc->cell_playback[__current_cell].last_sector - __pgc->cell_playback[__current_cell].first_sector;
    }
    
    /* Initialize cell iteratorss */
    __next_cell     = __start_cell;
    __current_cell  = 0;
    __cell_finished = true;
    
    /* Change event state.
     * if we are already running, send endTrack, then startTrack.
     * otherwise, only send startTrack.
     */
    if (__event_state != EVENT_START_TRACK) {
      __event_state = EVENT_NEXT_TRACK;
    }
    
    /* Create event to reset synchronisation. */
    __send_sync_reset = true;
    
    /* If the node is in STARTED, make sure the producing flag is set */
    if (isStarted() && !getProducingFlag()) {
      setProducingFlag(true);
    }
    
    return SUCCESS;
  }
  
  Message* DVDReadNode::processBuffer(Buffer*) {
    /* If the working flag was set, output the saved buffer */
    if(getWorkingFlag()) {
      setWorkingFlag(false);
      Buffer* out = __save_buffer;
      __save_buffer = NULL;
      return out;
    }
    
    MutexGuard guard(__select_mutex);
    
    /* Check if a file is opened */
    if (!__dvd_file) {
      /* No DVD file, stop producing */
      DEBUG_STREAM("No DVD file opened, stopping producing" << endl);
      __event_state = EVENT_START_TRACK;
      setProducingFlag(false);
    }
    
    /* if chapter, title, angle was changed, send event to resynchronize */
    switch(__event_state) {
    case EVENT_START_TRACK:
      __event_state = EVENT_NONE;
      return new CEvent(ITrack::create_startTrack(string("Title ") + __title));
      
    case EVENT_NEXT_TRACK:
      __event_state = EVENT_START_TRACK;
      return new CEvent(ITrack::create_endTrack());
      
    case EVENT_NONE:
      break;
      
    default:
      ERROR_STREAM("Invalid event state!!" << endl);
      return NULL;
    }
    
    /* Send syncReset if needed */
    if(__send_sync_reset) {
      __send_sync_reset = false;
      return new CEvent(ISyncReset::create_syncReset());
    }
    
    /* Check if we are at the end of the cell */
    if(__cell_finished) {
      /* Check if we are at the end of the title */
      if(__next_cell >= __pgc->nr_of_cells) {
        DEBUG_STREAM("End of title" << endl);
        
        /* Stop producing */
        setProducingFlag(false);
        
        /* Next event must be startTrack */
        __event_state = EVENT_START_TRACK;
        
        /* create new event to indicate the end of DVD reading */
        return new CEvent(ITrack::create_endTrack());
      }
      
      /* Update progress */
      if (0 != __next_cell) {
        __title_position +=
          __pgc->cell_playback[__current_cell].last_sector -
          __pgc->cell_playback[__current_cell].first_sector;
      }
      
      /* Go to next cell */
      if (!nextCell()) {
        setProducingFlag(false);
        return NULL;
      }
      
      /* Start at first sector in the cell */
      __current_sector = __pgc->cell_playback[__current_cell].first_sector;
      
      /* We have a valid cell now */
      __cell_finished = false;
    }
      
    /* Read NAV packet */
    int len = DVDReadBlocks(__dvd_file, static_cast<int>(__current_sector), 1, &__read_buffer[0]);
    if(len != 1) {
      DEBUG_STREAM("Read failed for cell " << __current_cell << ", sector " << __current_sector << endl);
      return NULL;
    }
    
    /* Parse the contained dsi packet */
    dsi_t dsi_pack;
    navRead_DSI(&dsi_pack, &__read_buffer[DSI_START_BYTE]);
    
    /* Determine where we go next.  These values are the ones we mostly care about. */
    // fw: unused, but kept here in case we might need it some day
//     unsigned int next_ilvu_start = __current_sector + dsi_pack.sml_agli.data[__angle-1].address;
    unsigned int cur_output_size = dsi_pack.dsi_gi.vobu_ea;

    /* If we're not at the end of this cell, we can determine the next
     * VOBU to display using the VOBU_SRI information section of the
     * DSI.  Using this value correctly follows the current angle,
     * avoiding the doubled scenes in The Matrix, and makes our life
     * really happy.
     *
     * Otherwise, we set our next address past the end of this cell to
     * force the code above to go to the next cell in the program.
     */
    unsigned int next_vobu;
    if(dsi_pack.vobu_sri.next_vobu != SRI_END_OF_CELL) {
      next_vobu = __current_sector + (dsi_pack.vobu_sri.next_vobu & 0x7fffffff);
    } else {
      next_vobu = __current_sector + cur_output_size + 1;
    }
    ++__current_sector;
    
    DEBUG_STREAM("Output size: " << (cur_output_size * DVD_VIDEO_LB_LEN) << endl);
    
    /* Allocate output buffer */
    Buffer* out_buffer = getNewBuffer( cur_output_size * DVD_VIDEO_LB_LEN);
    out_buffer->setUsed(cur_output_size * DVD_VIDEO_LB_LEN);
    
    /* Read data into buffer */
    len = DVDReadBlocks(__dvd_file, static_cast<int>(__current_sector), cur_output_size,
                        reinterpret_cast<unsigned char*>(out_buffer->getData()));
    if(len != static_cast<int>(cur_output_size)) {
      ERROR_STREAM("Read failed for cell " << __current_cell << ", sector " << __current_sector << " blocks=" << cur_output_size << endl);
      out_buffer->release();
      /* Do not return yet. Try to get to the next sector at least and maybe skip over the broken one */
    }
    
    /* Go to next sector */
    __current_sector = next_vobu;
    
    /* Check if the next sector is out of the cell */
    if(__current_sector >= __pgc->cell_playback[__current_cell].last_sector) {
      __cell_finished = true;
    }
    
    /* Check if we have to send a progress event */
    if (__send_progress) {
      /* Send progress every __buffers_before_progress buffers */
      if (__buffers_progress++ > __buffers_before_progress) {
        __buffers_progress = 0;
        
        if (out_buffer) {        
          __save_buffer = out_buffer;
          setWorkingFlag(true);
        }
        
        unsigned progress = __title_position + // current position is the position in the completly played cells
          (__current_sector - __pgc->cell_playback[__current_cell].first_sector); // plus the position in the current cell
        
        CEvent* progress_event(new CEvent(IProgressListener::create_setProgress(progress, __title_length)));
        progress_event->setMandatoryBit(false);
        return progress_event;
      }
    }
    
    /* Return buffer */
    return out_buffer;
  }
  
  bool DVDReadNode::nextCell() {
    /* Find the next cell (this is a mix between ogle code and old NMM code).
     * Invariant: __next_cell is either not in a block or at the beginning of a block
     * The following code sets __current_cell and __next_cell such that the invariant is preserved
     * and the right cell is played next.
     */
    switch(__pgc->cell_playback[__next_cell].block_mode) {
    case BLOCK_MODE_NOT_IN_BLOCK:
      /* If we are not entering a block, the block type must be BLOCK_TYPE_NONE */
      if (__pgc->cell_playback[__next_cell].block_type != BLOCK_TYPE_NONE) {
        ERROR_STREAM("Unexpected: cell has BLOCK_MODE_NOT_IN_BLOCK but not BLOCK_TYPE_NONE!" << endl);
        return false;
      }
      
      /* Just go to next cell */
      __current_cell = __next_cell;
      __next_cell += 1;
      return true;
    case BLOCK_MODE_FIRST_CELL:
      /* A block begins here. Check what type of block it is. We can only handle BLOCK_TYPE_ANGLE_BLOCK atm */
      switch(__pgc->cell_playback[__next_cell].block_type) {
      case BLOCK_TYPE_ANGLE_BLOCK:
        /* Go to the selected angle */
        __current_cell = __next_cell + __angle - 1;
        
        /* Check if the next cell is valid */
        if(__current_cell > __pgc->nr_of_cells) {
          ERROR_STREAM("Unexpected: Angle cell is out of program chain!" << endl);
          return false;
        }
        if(__pgc->cell_playback[__current_cell].block_mode == BLOCK_MODE_NOT_IN_BLOCK) {
          ERROR_STREAM("Unexpected: Angle cell has BLOCK_MODE_NOT_IN_BLOCK" << endl);
          return false;
        }
        if(__pgc->cell_playback[__current_cell].block_type != BLOCK_TYPE_ANGLE_BLOCK) {
          ERROR_STREAM("Unexpected: Angle cell doesn't have BLOCK_TYPE_ANGLE_BLOCK" << endl);
          return false;
        }
        
        /* Skip the other angle cells in the block */
        for(__next_cell = __current_cell+1; __next_cell < __pgc->nr_of_cells &&
              __pgc->cell_playback[__next_cell].block_mode >= BLOCK_MODE_IN_BLOCK;
            ++__next_cell) {
        }
        return true;
      default:
        /* Skip this block completely since we don't know how to handle it */
//         for(__next_cell = __next_cell+1; __next_cell < __pgc->nr_of_cells &&
//               __pgc->cell_playback[__next_cell].block_mode >= BLOCK_MODE_IN_BLOCK;
//             ++__next_cell) {
//         }
        
        ERROR_STREAM("Unknown block type for BLOCK_MODE_FIRST_CELL!" << endl);
        return false;
      }
      break;
    case BLOCK_MODE_IN_BLOCK:
    case BLOCK_MODE_LAST_CELL:
      /* This should never happen */
      ERROR_STREAM("Unexpected: Next cell is in a block but not at the beginning of it" << endl);
      return false;
    default:
      /* This should never happen */
      ERROR_STREAM("Unexpected: Unknown block mode" << endl);
      return false;
    }
  }
  
  void DVDReadNode::setDevice(const string& device)
    throw()
  {
    __device = device;
  }
  
  string DVDReadNode::getDevice() const {
    return __device;
  }
  
  Result DVDReadNode::openDVD() {
    /* Close DVD fiest */
    closeDVD();
    
    try {
      /* Create DVD reader */
      __dvd_reader = DVDOpen(__device.c_str());
      if ( !__dvd_reader ) {
        throw Exception("Can't open DVD");
      }
      DEBUG_STREAM("Successfully opened DVD." << endl );
      
      /* Parse VMG (video management) file */
      __vmg_file = ifoOpen(__dvd_reader, 0);
      if( !__vmg_file ) {
        throw Exception("Can't open VMG info.");
      }
      
      /* Get search pointer table */
      __tt_srpt = __vmg_file->tt_srpt;
      
      return SUCCESS;
    } catch(const Exception& e) {
      ERROR_STREAM("Failed to open DVD: " << e.getComment() << endl);
      closeDVD();
      return FAILURE;
    }
  }
  
  void DVDReadNode::closeDVD() {
    /* Close DVD title first */
    closeDVDTitle();
    
    /* Search pointer table is invalid */
    __tt_srpt = NULL;
    
    /* Close video manager file */
    if(__vmg_file) {
      DEBUG_STREAM("Close VMG file..." << endl);
      ifoClose(__vmg_file);
      __vmg_file = NULL;
    }
    
    /* Close DVD reader */
    if(__dvd_reader) {
      DEBUG_STREAM("Close DVD..." << endl);
      DVDClose(__dvd_reader);
      __dvd_reader = NULL;
    }
  }
  
  Result DVDReadNode::openDVDTitle(int title) {
    /* Check if the DVD has been opened */
    if (!__dvd_reader) {
      ERROR_STREAM("DVD not opened!");
      return FAILURE;
    }
    
    /* Make sure given title number is valid */
    if(title < 1 || title > __tt_srpt->nr_of_srpts) {
      ERROR_STREAM("Invalid title " << title << endl);
      return FAILURE;
    }
    
    /* Close the current title if it is already opened */
    closeDVDTitle();
    
    try {
      int title_set_nr = __tt_srpt->title[title-1].title_set_nr;
      
      /* Load the VTS information for the title set our title is in */
      __vts_file = ifoOpen(__dvd_reader, title_set_nr);
      if(!__vts_file) {
        throw Exception(string("Failed to open VTS file for title set ") + title_set_nr);
      }
      
      /* We've got enough info, time to open the title set data */
      __dvd_file = DVDOpenFile(__dvd_reader, title_set_nr, DVD_READ_TITLE_VOBS);
      if(!__dvd_file) {
        throw Exception(string("Can't open title VOBS (VTS_") + title_set_nr + "_1.VOB)");
      }
    } catch(const Exception& e) {
      ERROR_STREAM("Failed to open DVD title " << title << ": " << e.getComment() << endl);
      closeDVDTitle();
      return FAILURE;
    }
    
    /* Get additional information for title */
    __ttn = __tt_srpt->title[title-1].vts_ttn;
    __vts_ptt_srpt = __vts_file->vts_ptt_srpt;
    
    /* Set current title ID */
    __title = title;
    
    return SUCCESS;
  }
  
  void DVDReadNode::closeDVDTitle() {
    /* Current title is invalid */
    __title = 0;
    
    /* Additional information is invalid */
    __ttn = 0;
    __vts_ptt_srpt = NULL;
    
    /* Close video title set (VTS) file for current title */
    if(__vts_file) {
      DEBUG_STREAM("Close VTS file for current title..." << endl);
      ifoClose(__vts_file);
      __vts_file = NULL;
    }
    
    /* Close DVD file for current title */
    if(__dvd_file) {
      DEBUG_STREAM("Close DVD file for current title..." << endl);
      DVDCloseFile( __dvd_file );
      __dvd_file = NULL;
    }
  }
  
  void DVDReadNode::reset() {
    select(1,1,1);
  }
  
  void DVDReadNode::sendFlush() {
    // XXX: (fw) NOT IMPLEMENTED
  }
  
  Result DVDReadNode::checkFormat(bool&   pes_stream,
				  bool&   mpeg_audio,
				  bool&   ac3_audio,
				  bool&   mpeg2_video,
				  bool&   mpeg1_video,
				  int&    width,
				  int&    height,
				  int&    frame_rate_code,
				  double& frame_rate,
				  int&    ratio_x,
				  int&    ratio_y)
  {
    audio_attr_t *a_attr;
    
    /* Set default values */
    pes_stream      = false;
    mpeg_audio      = false;
    ac3_audio       = false;
    mpeg2_video     = false;
    mpeg1_video     = false;
    width           = 0;
    height          = 0;
    frame_rate_code = 0;
    frame_rate      = 25.0f;
    
    // XXX: fw: frame rate code and frame rate are NOT correctly determined! (TODO!)
    WARNING_STREAM("Sorry, not implemented: Determine video frame rate of DVD. Assuming PAL (25 fps)" << endl);
    
    /* Must have a VMG file */
    if ( !__vmg_file ) {
      ERROR_STREAM("No VMG file! Can't check format!" << endl);
      return FAILURE;
    }
    
    /* Get video attributes and check if they are valid */
    video_attr_t *v_attr = &__vmg_file->vmgi_mat->vmgm_video_attr;
    /*if(v_attr->mpeg_version == 0
       && v_attr->video_format == 0
       && v_attr->display_aspect_ratio == 0
       && v_attr->permitted_df == 0
       && v_attr->unknown1 == 0
       && v_attr->line21_cc_1 == 0
       && v_attr->line21_cc_2 == 0
       && v_attr->video_format == 0
       && v_attr->letterboxed == 0
       && v_attr->film_mode == 0) {
      ERROR_STREAM("DVD Video Format unspecified" << endl);
      return FAILURE;
      }*/
    
    /* Determine MPEG video version */
    switch(v_attr->mpeg_version) {
    case 0:
      DEBUG_STREAM("Found MPEG-1 video stream" << endl);
      mpeg1_video = true;
      break;
    case 1:
      DEBUG_STREAM("Found MPEG-2 video stream" << endl);
      mpeg2_video = true;
      break;
    default:
      ERROR_STREAM("No MPEG stream or failed to determine MPEG version" << endl);
      return FAILURE;
    }
    
    /* Determine aspect ration */
    switch(v_attr->display_aspect_ratio) {
      /* 4:3 */
    case 0:
      DEBUG_STREAM("Video aspect ratio is 4:3" << endl);
      ratio_x = 400;
      ratio_y = 300;
      break;
      
      /* 16:9 */
    case 3:
      DEBUG_STREAM("Video aspect ratio is 16:9" << endl);
      ratio_x = 1600;
      ratio_y = 900;
      break;
    default:
      ERROR_STREAM("Failed to determine video aspect ratio" << endl);
      return FAILURE;
    }
    
    /* Determine resolution */
    height = 480;
    if(v_attr->video_format != 0) {
      height = 576;
    }
    switch(v_attr->picture_size) {
    case 0:
      width = 720;
      break;
    case 1:
      width = 704;
      break;
    case 2:
      width = 352;
      break;
    case 3:
      width = 352;
      height /= 2;
      break;
    default:
      ERROR_STREAM("Failed to determine video resolution" << endl);
      return FAILURE;
    }
    DEBUG_STREAM("Video resolution is " << width << "x" << height << endl);
    
    /* Determine number of audio streams */
    if ( __vmg_file->vmgi_mat->nr_of_vmgm_audio_streams <= 0 ) {
      /* Only one AC3 audio stream, so we are done */
      DEBUG_STREAM("No information about DVD audio streams found, assuming one AC3 stream" << endl);
      ac3_audio = true;
      return SUCCESS;
    }
    
    /* Get audio attributes and check if they are valid */
    a_attr = &__vmg_file->vmgi_mat->vmgm_audio_attr;
    if(a_attr->audio_format == 0
       && a_attr->multichannel_extension == 0
       && a_attr->lang_type == 0
       && a_attr->application_mode == 0
       && a_attr->quantization == 0
       && a_attr->sample_frequency == 0
       && a_attr->channels == 0
       && a_attr->lang_extension == 0
       && a_attr->unknown1 == 0
       && a_attr->unknown1 == 0) {
      ERROR_STREAM("DVD Audio Format unspecified" << endl);
      return FAILURE;
    }
    
    /* Determine audio format */
    switch(a_attr->audio_format) {
    case 0:
      DEBUG_STREAM("Found AC3 audio stream" << endl);
      ac3_audio = true;
      break;
    case 2:
      DEBUG_STREAM("Found MPEG audio stream (audio_format=2)" << endl);
      mpeg_audio = true;
      break;
    case 3:
      DEBUG_STREAM("Found MPEG audio stream (audio_format=3)" << endl);
      mpeg_audio = true;
      break;
    case 4:
      ERROR_STREAM("Sorry, not supported: DVD with LPCM audio" << endl);
      return FAILURE;
    case 6:
      ERROR_STREAM("Sorry, not supported: DVD with DTS audio" << endl);
      return FAILURE;
    default:
      ERROR_STREAM("Failed to determine audio format" << endl);
      return FAILURE;
    }
    
    // fw: not used, but kept here for future implementation
//     switch(attr->lang_type) {
//     case 0:
//       // not specified
//       assert(attr->lang_code == 0);
//       break;
//     case 1:
//       printf("%c%c ", attr->lang_code>>8, attr->lang_code & 0xff);
//       break;
//     default:
//         printf("(please send a bug report) ");
//     }
    
    // fw: not used, but kept here for future implementation
//     switch(attr->quantization) {
//     case 0:
//       printf("16bit ");
//       break;
//     case 1:
//       printf("20bit ");
//       break;
//     case 2:
//       printf("24bit ");
//       break;
//     case 3:
//       printf("drc ");
//       break;
//     default:
//       printf("(please send a bug report) ");
//     }
    
    return SUCCESS;
  }
  
  void DVDReadNode::sendProgressInformation(const nmm_bool& in) {
    __send_progress = in;
  }

  void DVDReadNode::setProgressInterval(const nmm_int32& buffers) {
    __buffers_before_progress = buffers;
  }
}

#include "nmm/nmm_suffix.hpp"
