/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IDVD.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IDVD::used() {} 
  
  
  Result IDVD::select(const nmm_uint32& title, const nmm_uint32& chapter, const nmm_uint32& angle)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_uint32 > title_value(title, false);
    NMM::TInValue<nmm_uint32 > chapter_value(chapter, false);
    NMM::TInValue<nmm_uint32 > angle_value(angle, false);
    NMM::Event event("IDVD::select", &title_value, &chapter_value, &angle_value);
    event.setReturnValue(&return_value);
    if(const_cast<IDVD*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 IDVD::getCurrentTitle() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("IDVD::getCurrentTitle");
    event.setReturnValue(&return_value);
    if(const_cast<IDVD*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 IDVD::getCurrentChapter() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("IDVD::getCurrentChapter");
    event.setReturnValue(&return_value);
    if(const_cast<IDVD*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 IDVD::getTitles() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("IDVD::getTitles");
    event.setReturnValue(&return_value);
    if(const_cast<IDVD*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 IDVD::getChapters(const nmm_uint32& title) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::TInValue<nmm_uint32 > title_value(title, false);
    NMM::Event event("IDVD::getChapters", &title_value);
    event.setReturnValue(&return_value);
    if(const_cast<IDVD*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 IDVD::getAngles(const nmm_uint32& title) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::TInValue<nmm_uint32 > title_value(title, false);
    NMM::Event event("IDVD::getAngles", &title_value);
    event.setReturnValue(&return_value);
    if(const_cast<IDVD*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IDVD::sendFlush()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IDVD::sendFlush");
    if(const_cast<IDVD*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IDVD::reset()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IDVD::reset");
    if(const_cast<IDVD*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IDVD::IDVD_type = "IDVD";
  NMM::TConstructor<IDVD > __IDVD_prototype("IDVD","NMM");


  const char* IDVD::select_event = "IDVD::select";
  const char* IDVD::getCurrentTitle_event = "IDVD::getCurrentTitle";
  const char* IDVD::getCurrentChapter_event = "IDVD::getCurrentChapter";
  const char* IDVD::getTitles_event = "IDVD::getTitles";
  const char* IDVD::getChapters_event = "IDVD::getChapters";
  const char* IDVD::getAngles_event = "IDVD::getAngles";
  const char* IDVD::sendFlush_event = "IDVD::sendFlush";
  const char* IDVD::reset_event = "IDVD::reset";


}


#include "nmm/nmm_suffix.hpp"

