/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IDVDNav.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IDVDNav::used() {} 
  
  
  Result IDVDNav::selectButton(const nmm_int32& button)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > button_value(button, false);
    NMM::Event event("IDVDNav::selectButton", &button_value);
    event.setReturnValue(&return_value);
    if(const_cast<IDVDNav*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IDVDNav::activateButton(const nmm_int32& button)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > button_value(button, false);
    NMM::Event event("IDVDNav::activateButton", &button_value);
    event.setReturnValue(&return_value);
    if(const_cast<IDVDNav*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IDVDNav::activateSelectedButton()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("IDVDNav::activateSelectedButton");
    event.setReturnValue(&return_value);
    if(const_cast<IDVDNav*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IDVDNav::goUp()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("IDVDNav::goUp");
    event.setReturnValue(&return_value);
    if(const_cast<IDVDNav*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IDVDNav::buttonUp()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("IDVDNav::buttonUp");
    event.setReturnValue(&return_value);
    if(const_cast<IDVDNav*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IDVDNav::buttonDown()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("IDVDNav::buttonDown");
    event.setReturnValue(&return_value);
    if(const_cast<IDVDNav*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IDVDNav::buttonRight()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("IDVDNav::buttonRight");
    event.setReturnValue(&return_value);
    if(const_cast<IDVDNav*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IDVDNav::buttonLeft()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("IDVDNav::buttonLeft");
    event.setReturnValue(&return_value);
    if(const_cast<IDVDNav*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 IDVDNav::getSelectedButton()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("IDVDNav::getSelectedButton");
    event.setReturnValue(&return_value);
    if(const_cast<IDVDNav*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IDVDNav::IDVDNav_type = "IDVDNav";
  NMM::TConstructor<IDVDNav > __IDVDNav_prototype("IDVDNav","NMM");


  const char* IDVDNav::selectButton_event = "IDVDNav::selectButton";
  const char* IDVDNav::activateButton_event = "IDVDNav::activateButton";
  const char* IDVDNav::activateSelectedButton_event = "IDVDNav::activateSelectedButton";
  const char* IDVDNav::goUp_event = "IDVDNav::goUp";
  const char* IDVDNav::buttonUp_event = "IDVDNav::buttonUp";
  const char* IDVDNav::buttonDown_event = "IDVDNav::buttonDown";
  const char* IDVDNav::buttonRight_event = "IDVDNav::buttonRight";
  const char* IDVDNav::buttonLeft_event = "IDVDNav::buttonLeft";
  const char* IDVDNav::getSelectedButton_event = "IDVDNav::getSelectedButton";


}


#include "nmm/nmm_suffix.hpp"

