/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IDVDNav_HPP
#define NMM_IDVDNav_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{

  class DLLEXPORT IDVDNav : virtual public NMM::Interface { 
  public:
    IDVDNav() : NMM::Interface(IDVDNav::IDVDNav_type, "NMM") { };

    IDVDNav(NMM::NMMObject& parent) : NMM::Interface (parent, IDVDNav::IDVDNav_type, "NMM") { };

    virtual ~IDVDNav() {};

    static const char* IDVDNav_type;

    virtual  string getInterfaceType() const { return IDVDNav_type; }

    NMM::Interface* copy() const { return new IDVDNav(*this); }
    static void used(); 
    static const char* selectButton_event;
    static const char* activateButton_event;
    static const char* activateSelectedButton_event;
    static const char* goUp_event;
    static const char* buttonUp_event;
    static const char* buttonDown_event;
    static const char* buttonRight_event;
    static const char* buttonLeft_event;
    static const char* getSelectedButton_event;

    virtual Result selectButton(const nmm_int32& button)
      throw(const NMM::SerializedException&);

    virtual Result activateButton(const nmm_int32& button)
      throw(const NMM::SerializedException&);

    virtual Result activateSelectedButton()
      throw(const NMM::SerializedException&);

    virtual Result goUp()
      throw(const NMM::SerializedException&);

    virtual Result buttonUp()
      throw(const NMM::SerializedException&);

    virtual Result buttonDown()
      throw(const NMM::SerializedException&);

    virtual Result buttonRight()
      throw(const NMM::SerializedException&);

    virtual Result buttonLeft()
      throw(const NMM::SerializedException&);

    virtual nmm_int32 getSelectedButton()
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IDVDNav> IDVDNav_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
