/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ISPUDecoder.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ISPUDecoder::used() {} 
  
  
  Result ISPUDecoder::setPaletteItem(const nmm_int32& index, const nmm_int32& color_y, const nmm_int32& color_u, const nmm_int32& color_v, const nmm_int32& alpha) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > index_value(index, false);
    NMM::TInValue<nmm_int32 > color_y_value(color_y, false);
    NMM::TInValue<nmm_int32 > color_u_value(color_u, false);
    NMM::TInValue<nmm_int32 > color_v_value(color_v, false);
    NMM::TInValue<nmm_int32 > alpha_value(alpha, false);
    NMM::Event event("ISPUDecoder::setPaletteItem", &index_value, &color_y_value, &color_u_value, &color_v_value, &alpha_value);
    event.setReturnValue(&return_value);
    if(const_cast<ISPUDecoder*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISPUDecoder::initColorScheme(const nmm_int32& scheme_nr, const nmm_int32& selection, const nmm_int32& activation) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > scheme_nr_value(scheme_nr, false);
    NMM::TInValue<nmm_int32 > selection_value(selection, false);
    NMM::TInValue<nmm_int32 > activation_value(activation, false);
    NMM::Event event("ISPUDecoder::initColorScheme", &scheme_nr_value, &selection_value, &activation_value);
    event.setReturnValue(&return_value);
    if(const_cast<ISPUDecoder*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISPUDecoder::setDVDSelectedButton(const nmm_int32& button) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > button_value(button, false);
    NMM::Event event("ISPUDecoder::setDVDSelectedButton", &button_value);
    event.setReturnValue(&return_value);
    if(const_cast<ISPUDecoder*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISPUDecoder::setDVDButtonCount(const nmm_int32& buttons) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > buttons_value(buttons, false);
    NMM::Event event("ISPUDecoder::setDVDButtonCount", &buttons_value);
    event.setReturnValue(&return_value);
    if(const_cast<ISPUDecoder*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISPUDecoder::setDVDButtonCoords(const nmm_int32& start_x, const nmm_int32& start_y, const nmm_int32& end_x, const nmm_int32& end_y) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > start_x_value(start_x, false);
    NMM::TInValue<nmm_int32 > start_y_value(start_y, false);
    NMM::TInValue<nmm_int32 > end_x_value(end_x, false);
    NMM::TInValue<nmm_int32 > end_y_value(end_y, false);
    NMM::Event event("ISPUDecoder::setDVDButtonCoords", &start_x_value, &start_y_value, &end_x_value, &end_y_value);
    event.setReturnValue(&return_value);
    if(const_cast<ISPUDecoder*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISPUDecoder::setDVDButtonScheme(const nmm_int32& scheme) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > scheme_value(scheme, false);
    NMM::Event event("ISPUDecoder::setDVDButtonScheme", &scheme_value);
    event.setReturnValue(&return_value);
    if(const_cast<ISPUDecoder*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ISPUDecoder::ISPUDecoder_type = "ISPUDecoder";
  NMM::TConstructor<ISPUDecoder > __ISPUDecoder_prototype("ISPUDecoder","NMM");


  const char* ISPUDecoder::setPaletteItem_event = "ISPUDecoder::setPaletteItem";
  NMM::Event* ISPUDecoder::create_setPaletteItem(const nmm_int32& index, const nmm_int32& color_y, const nmm_int32& color_u, const nmm_int32& color_v, const nmm_int32& alpha)
  {
    NMM::TInValue<nmm_int32 >* index_value = new NMM::TInValue<nmm_int32 >();
    index_value->setValue(index);
    NMM::TInValue<nmm_int32 >* color_y_value = new NMM::TInValue<nmm_int32 >();
    color_y_value->setValue(color_y);
    NMM::TInValue<nmm_int32 >* color_u_value = new NMM::TInValue<nmm_int32 >();
    color_u_value->setValue(color_u);
    NMM::TInValue<nmm_int32 >* color_v_value = new NMM::TInValue<nmm_int32 >();
    color_v_value->setValue(color_v);
    NMM::TInValue<nmm_int32 >* alpha_value = new NMM::TInValue<nmm_int32 >();
    alpha_value->setValue(alpha);

    return new NMM::Event("ISPUDecoder::setPaletteItem", index_value, color_y_value, color_u_value, color_v_value, alpha_value);
  }

  const char* ISPUDecoder::initColorScheme_event = "ISPUDecoder::initColorScheme";
  NMM::Event* ISPUDecoder::create_initColorScheme(const nmm_int32& scheme_nr, const nmm_int32& selection, const nmm_int32& activation)
  {
    NMM::TInValue<nmm_int32 >* scheme_nr_value = new NMM::TInValue<nmm_int32 >();
    scheme_nr_value->setValue(scheme_nr);
    NMM::TInValue<nmm_int32 >* selection_value = new NMM::TInValue<nmm_int32 >();
    selection_value->setValue(selection);
    NMM::TInValue<nmm_int32 >* activation_value = new NMM::TInValue<nmm_int32 >();
    activation_value->setValue(activation);

    return new NMM::Event("ISPUDecoder::initColorScheme", scheme_nr_value, selection_value, activation_value);
  }

  const char* ISPUDecoder::setDVDSelectedButton_event = "ISPUDecoder::setDVDSelectedButton";
  NMM::Event* ISPUDecoder::create_setDVDSelectedButton(const nmm_int32& button)
  {
    NMM::TInValue<nmm_int32 >* button_value = new NMM::TInValue<nmm_int32 >();
    button_value->setValue(button);

    return new NMM::Event("ISPUDecoder::setDVDSelectedButton", button_value);
  }

  const char* ISPUDecoder::setDVDButtonCount_event = "ISPUDecoder::setDVDButtonCount";
  NMM::Event* ISPUDecoder::create_setDVDButtonCount(const nmm_int32& buttons)
  {
    NMM::TInValue<nmm_int32 >* buttons_value = new NMM::TInValue<nmm_int32 >();
    buttons_value->setValue(buttons);

    return new NMM::Event("ISPUDecoder::setDVDButtonCount", buttons_value);
  }

  const char* ISPUDecoder::setDVDButtonCoords_event = "ISPUDecoder::setDVDButtonCoords";
  NMM::Event* ISPUDecoder::create_setDVDButtonCoords(const nmm_int32& start_x, const nmm_int32& start_y, const nmm_int32& end_x, const nmm_int32& end_y)
  {
    NMM::TInValue<nmm_int32 >* start_x_value = new NMM::TInValue<nmm_int32 >();
    start_x_value->setValue(start_x);
    NMM::TInValue<nmm_int32 >* start_y_value = new NMM::TInValue<nmm_int32 >();
    start_y_value->setValue(start_y);
    NMM::TInValue<nmm_int32 >* end_x_value = new NMM::TInValue<nmm_int32 >();
    end_x_value->setValue(end_x);
    NMM::TInValue<nmm_int32 >* end_y_value = new NMM::TInValue<nmm_int32 >();
    end_y_value->setValue(end_y);

    return new NMM::Event("ISPUDecoder::setDVDButtonCoords", start_x_value, start_y_value, end_x_value, end_y_value);
  }

  const char* ISPUDecoder::setDVDButtonScheme_event = "ISPUDecoder::setDVDButtonScheme";
  NMM::Event* ISPUDecoder::create_setDVDButtonScheme(const nmm_int32& scheme)
  {
    NMM::TInValue<nmm_int32 >* scheme_value = new NMM::TInValue<nmm_int32 >();
    scheme_value->setValue(scheme);

    return new NMM::Event("ISPUDecoder::setDVDButtonScheme", scheme_value);
  }



}


#include "nmm/nmm_suffix.hpp"

