/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IFileModeHandler.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IFileModeHandler::used() {} 
  
  
  void IFileModeHandler::setMode(const IFileModeHandler::FileMode& mode)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<IFileModeHandler::FileMode > mode_value(mode, false);
    NMM::Event event("IFileModeHandler::setMode", &mode_value);
    if(const_cast<IFileModeHandler*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  IFileModeHandler::FileMode IFileModeHandler::getMode()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<IFileModeHandler::FileMode > return_value;
    NMM::Event event("IFileModeHandler::getMode");
    event.setReturnValue(&return_value);
    if(const_cast<IFileModeHandler*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IFileModeHandler::setFormat(const IFileModeHandler::FileFormat& format)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<IFileModeHandler::FileFormat > format_value(format, false);
    NMM::Event event("IFileModeHandler::setFormat", &format_value);
    if(const_cast<IFileModeHandler*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  IFileModeHandler::FileFormat IFileModeHandler::getFormat()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<IFileModeHandler::FileFormat > return_value;
    NMM::Event event("IFileModeHandler::getFormat");
    event.setReturnValue(&return_value);
    if(const_cast<IFileModeHandler*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IFileModeHandler::setFilenameMode(const IFileModeHandler::FilenameMode& mode)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<IFileModeHandler::FilenameMode > mode_value(mode, false);
    NMM::Event event("IFileModeHandler::setFilenameMode", &mode_value);
    if(const_cast<IFileModeHandler*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  IFileModeHandler::FilenameMode IFileModeHandler::getFilenameMode()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<IFileModeHandler::FilenameMode > return_value;
    NMM::Event event("IFileModeHandler::getFilenameMode");
    event.setReturnValue(&return_value);
    if(const_cast<IFileModeHandler*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IFileModeHandler::setLoop(const nmm_bool& d)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > d_value(d, false);
    NMM::Event event("IFileModeHandler::setLoop", &d_value);
    if(const_cast<IFileModeHandler*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IFileModeHandler::IFileModeHandler_type = "IFileModeHandler";
  NMM::TConstructor<IFileModeHandler > __IFileModeHandler_prototype("IFileModeHandler","NMM");
  TValueConstructor< IFileModeHandler::FileMode > __IFileModeHandler_FileMode_prototype(TypeInfo("IFileModeHandler::FileMode", "NMM"));

  NMM::OValueStream& operator<<(NMM::OValueStream& ov, const IFileModeHandler::FileMode& t)
  {
    ov.beginType(TypeInfo("IFileModeHandler::FileMode", "NMM"));
    nmm_int32 temp = static_cast<nmm_int32>(t);
    ov << temp;
    ov.endType();
    return ov;
  }

  NMM::IValueStream& operator>>(NMM::IValueStream& iv, IFileModeHandler::FileMode& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("IFileModeHandler::FileMode", "NMM")) {
      throw NMM::InvalidTypeException(TypeInfo("IFileModeHandler::FileMode", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    nmm_int32 temp;
    iv >> temp;
    t = static_cast<IFileModeHandler::FileMode>(temp);
    iv.goUp();
    return iv;
  }

  TValueConstructor< IFileModeHandler::FileFormat > __IFileModeHandler_FileFormat_prototype(TypeInfo("IFileModeHandler::FileFormat", "NMM"));

  NMM::OValueStream& operator<<(NMM::OValueStream& ov, const IFileModeHandler::FileFormat& t)
  {
    ov.beginType(TypeInfo("IFileModeHandler::FileFormat", "NMM"));
    nmm_int32 temp = static_cast<nmm_int32>(t);
    ov << temp;
    ov.endType();
    return ov;
  }

  NMM::IValueStream& operator>>(NMM::IValueStream& iv, IFileModeHandler::FileFormat& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("IFileModeHandler::FileFormat", "NMM")) {
      throw NMM::InvalidTypeException(TypeInfo("IFileModeHandler::FileFormat", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    nmm_int32 temp;
    iv >> temp;
    t = static_cast<IFileModeHandler::FileFormat>(temp);
    iv.goUp();
    return iv;
  }

  TValueConstructor< IFileModeHandler::FilenameMode > __IFileModeHandler_FilenameMode_prototype(TypeInfo("IFileModeHandler::FilenameMode", "NMM"));

  NMM::OValueStream& operator<<(NMM::OValueStream& ov, const IFileModeHandler::FilenameMode& t)
  {
    ov.beginType(TypeInfo("IFileModeHandler::FilenameMode", "NMM"));
    nmm_int32 temp = static_cast<nmm_int32>(t);
    ov << temp;
    ov.endType();
    return ov;
  }

  NMM::IValueStream& operator>>(NMM::IValueStream& iv, IFileModeHandler::FilenameMode& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("IFileModeHandler::FilenameMode", "NMM")) {
      throw NMM::InvalidTypeException(TypeInfo("IFileModeHandler::FilenameMode", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    nmm_int32 temp;
    iv >> temp;
    t = static_cast<IFileModeHandler::FilenameMode>(temp);
    iv.goUp();
    return iv;
  }



  const char* IFileModeHandler::setMode_event = "IFileModeHandler::setMode";
  const char* IFileModeHandler::getMode_event = "IFileModeHandler::getMode";
  const char* IFileModeHandler::setFormat_event = "IFileModeHandler::setFormat";
  const char* IFileModeHandler::getFormat_event = "IFileModeHandler::getFormat";
  const char* IFileModeHandler::setFilenameMode_event = "IFileModeHandler::setFilenameMode";
  const char* IFileModeHandler::getFilenameMode_event = "IFileModeHandler::getFilenameMode";
  const char* IFileModeHandler::setLoop_event = "IFileModeHandler::setLoop";


}


#include "nmm/nmm_suffix.hpp"

