/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IFileModeHandler_HPP
#define NMM_IFileModeHandler_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{
/** Interface specifying modes for creating files.
   */

  class DLLEXPORT IFileModeHandler : virtual public NMM::Interface { 
  public:
  enum FileMode { 
    FILE_PER_FRAME = 0 , 
    FILE_PER_STREAM  };

  enum FileFormat { 
    DATA_ONLY = 0 , 
    SIZE_DATA , 
    SIZE_TIMESTAMP_DATA  };

  enum FilenameMode { 
    FILENAME_ONLY = 0 , 
    FILENAME_BUFCOUNT , 
    TIMESTAMP_AS_NAME  };

    IFileModeHandler() : NMM::Interface(IFileModeHandler::IFileModeHandler_type, "NMM") { };

    IFileModeHandler(NMM::NMMObject& parent) : NMM::Interface (parent, IFileModeHandler::IFileModeHandler_type, "NMM") { };

    virtual ~IFileModeHandler() {};

    static const char* IFileModeHandler_type;

    virtual  string getInterfaceType() const { return IFileModeHandler_type; }

    NMM::Interface* copy() const { return new IFileModeHandler(*this); }
    static void used(); 
    static const char* setMode_event;
    static const char* getMode_event;
    static const char* setFormat_event;
    static const char* getFormat_event;
    static const char* setFilenameMode_event;
    static const char* getFilenameMode_event;
    static const char* setLoop_event;
    /** Sets file mode to use.
     *
     * @param mode File mode to be used.
     */

    virtual void setMode(const IFileModeHandler::FileMode& mode)
      throw(const NMM::SerializedException&);
    /** Returns current file mode.
     *
     * @return Current chosen file mode.
     */

    virtual IFileModeHandler::FileMode getMode()
      throw(const NMM::SerializedException&);
    /** Sets format to be written.
     *
     * @return Current chosen format.
     */

    virtual void setFormat(const IFileModeHandler::FileFormat& format)
      throw(const NMM::SerializedException&);
    /** Returns current chosen format.
     *
     * @return Current chosen format.
     */

    virtual IFileModeHandler::FileFormat getFormat()
      throw(const NMM::SerializedException&);
    /** Sets filename mode to use.
     *
     * @param mode Filename mode to be used.
     */

    virtual void setFilenameMode(const IFileModeHandler::FilenameMode& mode)
      throw(const NMM::SerializedException&);
    /** Returns current chosen filename mode.
     *
     * @return Current chosen filename mode.
     */

    virtual IFileModeHandler::FilenameMode getFilenameMode()
      throw(const NMM::SerializedException&);
    /** Switches loop on or off.
     *
     * @param d True if loop is switched on, else otherwise.
     */

    virtual void setLoop(const nmm_bool& d)
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IFileModeHandler> IFileModeHandler_var;

  DLLEXPORT NMM::OValueStream& operator<<(NMM::OValueStream& ov, const IFileModeHandler::FileMode& t);
  DLLEXPORT NMM::IValueStream& operator>>(NMM::IValueStream& iv, IFileModeHandler::FileMode& t);
  DLLEXPORT NMM::OValueStream& operator<<(NMM::OValueStream& ov, const IFileModeHandler::FileFormat& t);
  DLLEXPORT NMM::IValueStream& operator>>(NMM::IValueStream& iv, IFileModeHandler::FileFormat& t);
  DLLEXPORT NMM::OValueStream& operator<<(NMM::OValueStream& ov, const IFileModeHandler::FilenameMode& t);
  DLLEXPORT NMM::IValueStream& operator>>(NMM::IValueStream& iv, IFileModeHandler::FilenameMode& t);



}

#include "nmm/nmm_suffix.hpp"

#endif
