/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IFileModeHandlerImpl_HPP
#define NMM_IFileModeHandlerImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IFileModeHandler.hpp"


#include "visibility.hpp"

namespace NMM
{
/** Interface specifying modes for creating files.
   */

  class DLLEXPORT IFileModeHandlerImpl  : public virtual NMM::NMMObject { 
  public:
    IFileModeHandlerImpl() { addInterface(new IFileModeHandler(*this)); registerEvents(); };

    virtual ~IFileModeHandlerImpl() {};

  protected:
    /** Sets file mode to use.
     *
     * @param mode File mode to be used.
     */

    virtual void setMode(const IFileModeHandler::FileMode& mode) = 0;
    /** Returns current file mode.
     *
     * @return Current chosen file mode.
     */

    virtual IFileModeHandler::FileMode getMode() = 0;
    /** Sets format to be written.
     *
     * @return Current chosen format.
     */

    virtual void setFormat(const IFileModeHandler::FileFormat& format) = 0;
    /** Returns current chosen format.
     *
     * @return Current chosen format.
     */

    virtual IFileModeHandler::FileFormat getFormat() = 0;
    /** Sets filename mode to use.
     *
     * @param mode Filename mode to be used.
     */

    virtual void setFilenameMode(const IFileModeHandler::FilenameMode& mode) = 0;
    /** Returns current chosen filename mode.
     *
     * @return Current chosen filename mode.
     */

    virtual IFileModeHandler::FilenameMode getFilenameMode() = 0;
    /** Switches loop on or off.
     *
     * @param d True if loop is switched on, else otherwise.
     */

    virtual void setLoop(const nmm_bool& d) = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
