/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IHeader_HPP
#define NMM_IHeader_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{
  /** Interface to send file header.
   *
   * Nodes implementing this interface send file header when they open a new file.
   */

  class DLLEXPORT IHeader : virtual public NMM::Interface { 
  public:
    IHeader() : NMM::Interface(IHeader::IHeader_type, "NMM") { };

    IHeader(NMM::NMMObject& parent) : NMM::Interface (parent, IHeader::IHeader_type, "NMM") { };

    virtual ~IHeader() {};

    static const char* IHeader_type;

    virtual  string getInterfaceType() const { return IHeader_type; }

    NMM::Interface* copy() const { return new IHeader(*this); }
    static void used(); 
    static const char* sendHeader_event;
    static const char* FileHeader_event;
    static NMM::Event* create_FileHeader(const string& val);
    /** Enables sending of file header.
     *
     * @param value  True if sending of file header is enabled, false otherwise.
     */

    virtual void sendHeader(const nmm_bool& value)
      throw(const NMM::SerializedException&);
  /* instream (protected) and non-instream (public) */        /** Event which contains file header as string.
     *
     * @param val File Header 
     */

    virtual void FileHeader(const string& val) /* in-stream */ 
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IHeader> IHeader_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
