/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IHeaderImpl_HPP
#define NMM_IHeaderImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IHeader.hpp"


#include "visibility.hpp"

namespace NMM
{
  /** Interface to send file header.
   *
   * Nodes implementing this interface send file header when they open a new file.
   */

  class DLLEXPORT IHeaderImpl  : public virtual NMM::NMMObject { 
  public:
    IHeaderImpl() { addInterface(new IHeader(*this)); registerEvents(); };

    virtual ~IHeaderImpl() {};

  protected:
    /** Enables sending of file header.
     *
     * @param value  True if sending of file header is enabled, false otherwise.
     */

    virtual void sendHeader(const nmm_bool& value) = 0;
    /** Event which contains file header as string.
     *
     * @param val File Header 
     */

    virtual void FileHeader(const string& val) /* in-stream */  = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
