/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IMetaData.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IMetaData::used() {} 
  
  
  void IMetaData::getMetaData(string& title, string& album, string& artist, string& year, string& genre)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > title_value(title);
    NMM::TOutValue<string > album_value(album);
    NMM::TOutValue<string > artist_value(artist);
    NMM::TOutValue<string > year_value(year);
    NMM::TOutValue<string > genre_value(genre);
    NMM::Event event("IMetaData::getMetaData", &title_value, &album_value, &artist_value, &year_value, &genre_value);
    if(const_cast<IMetaData*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    title = title_value.getRValue();
    album = album_value.getRValue();
    artist = artist_value.getRValue();
    year = year_value.getRValue();
    genre = genre_value.getRValue();
  }

  const char* IMetaData::IMetaData_type = "IMetaData";
  NMM::TConstructor<IMetaData > __IMetaData_prototype("IMetaData","NMM");


  const char* IMetaData::getMetaData_event = "IMetaData::getMetaData";


}


#include "nmm/nmm_suffix.hpp"

