/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IMetaData_HPP
#define NMM_IMetaData_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{
/** Interface to request metadata of files.
   */

  class DLLEXPORT IMetaData : virtual public NMM::Interface { 
  public:
    IMetaData() : NMM::Interface(IMetaData::IMetaData_type, "NMM") { };

    IMetaData(NMM::NMMObject& parent) : NMM::Interface (parent, IMetaData::IMetaData_type, "NMM") { };

    virtual ~IMetaData() {};

    static const char* IMetaData_type;

    virtual  string getInterfaceType() const { return IMetaData_type; }

    NMM::Interface* copy() const { return new IMetaData(*this); }
    static void used(); 
    static const char* getMetaData_event;
   /** Returns meta information of currently opened file.
     *
     * @param title Title 
     * @param album Album title
     * @param artist Artists name
     * @param year Year of production 
     * @param genre Genre of the multimedia file
     */

    virtual void getMetaData(string& title, string& album, string& artist, string& year, string& genre)
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IMetaData> IMetaData_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
