/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ISeekable.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/serialize/SerializeUtilsTime.hpp"
#include "nmm/utils/time/Time.hpp"

namespace NMM
{
  void ISeekable::used() {} 
  
  
  nmm_bool ISeekable::hasPercentSeek() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ISeekable::hasPercentSeek");
    event.setReturnValue(&return_value);
    if(const_cast<ISeekable*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ISeekable::seekPercent(const Rational& percent)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<Rational > percent_value(percent, false);
    NMM::Event event("ISeekable::seekPercent", &percent_value);
    if(const_cast<ISeekable*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ISeekable::seekPercentTo(const Rational& percent)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<Rational > percent_value(percent, false);
    NMM::Event event("ISeekable::seekPercentTo", &percent_value);
    if(const_cast<ISeekable*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_bool ISeekable::hasBytesSeek() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ISeekable::hasBytesSeek");
    event.setReturnValue(&return_value);
    if(const_cast<ISeekable*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ISeekable::seekBytes(const nmm_int64& offset)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int64 > offset_value(offset, false);
    NMM::Event event("ISeekable::seekBytes", &offset_value);
    if(const_cast<ISeekable*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ISeekable::seekBytesTo(const nmm_uint64& bytes) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint64 > bytes_value(bytes, false);
    NMM::Event event("ISeekable::seekBytesTo", &bytes_value);
    if(const_cast<ISeekable*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_bool ISeekable::hasTimeSeek() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ISeekable::hasTimeSeek");
    event.setReturnValue(&return_value);
    if(const_cast<ISeekable*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ISeekable::seekTime(const Interval& interval)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<Interval > interval_value(interval, false);
    NMM::Event event("ISeekable::seekTime", &interval_value);
    if(const_cast<ISeekable*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ISeekable::seekTimeTo(const Time& time)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<Time > time_value(time, false);
    NMM::Event event("ISeekable::seekTimeTo", &time_value);
    if(const_cast<ISeekable*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* ISeekable::ISeekable_type = "ISeekable";
  NMM::TConstructor<ISeekable > __ISeekable_prototype("ISeekable","NMM");


  const char* ISeekable::hasPercentSeek_event = "ISeekable::hasPercentSeek";
  const char* ISeekable::seekPercent_event = "ISeekable::seekPercent";
  const char* ISeekable::seekPercentTo_event = "ISeekable::seekPercentTo";
  const char* ISeekable::hasBytesSeek_event = "ISeekable::hasBytesSeek";
  const char* ISeekable::seekBytes_event = "ISeekable::seekBytes";
  const char* ISeekable::seekBytesTo_event = "ISeekable::seekBytesTo";
  NMM::Event* ISeekable::create_seekBytesTo(const nmm_uint64& bytes)
  {
    NMM::TInValue<nmm_uint64 >* bytes_value = new NMM::TInValue<nmm_uint64 >();
    bytes_value->setValue(bytes);

    return new NMM::Event("ISeekable::seekBytesTo", bytes_value);
  }

  const char* ISeekable::hasTimeSeek_event = "ISeekable::hasTimeSeek";
  const char* ISeekable::seekTime_event = "ISeekable::seekTime";
  const char* ISeekable::seekTimeTo_event = "ISeekable::seekTimeTo";


}


#include "nmm/nmm_suffix.hpp"

