/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ISendFilenameAndTimestamp.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ISendFilenameAndTimestamp::used() {} 
  
  
  Result ISendFilenameAndTimestamp::sendFilenameAndTimestamp(const string& fn, const Timestamp& ts) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > fn_value(fn, false);
    NMM::TInValue<Timestamp > ts_value(ts, false);
    NMM::Event event("ISendFilenameAndTimestamp::sendFilenameAndTimestamp", &fn_value, &ts_value);
    event.setReturnValue(&return_value);
    if(const_cast<ISendFilenameAndTimestamp*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ISendFilenameAndTimestamp::ISendFilenameAndTimestamp_type = "ISendFilenameAndTimestamp";
  NMM::TConstructor<ISendFilenameAndTimestamp > __ISendFilenameAndTimestamp_prototype("ISendFilenameAndTimestamp","NMM");


  const char* ISendFilenameAndTimestamp::sendFilenameAndTimestamp_event = "ISendFilenameAndTimestamp::sendFilenameAndTimestamp";
  NMM::Event* ISendFilenameAndTimestamp::create_sendFilenameAndTimestamp(const string& fn, const Timestamp& ts)
  {
    NMM::TInValue<string >* fn_value = new NMM::TInValue<string >();
    fn_value->setValue(fn);
    NMM::TInValue<Timestamp >* ts_value = new NMM::TInValue<Timestamp >();
    ts_value->setValue(ts);

    return new NMM::Event("ISendFilenameAndTimestamp::sendFilenameAndTimestamp", fn_value, ts_value);
  }



}


#include "nmm/nmm_suffix.hpp"

