/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ISendFilenameAndTimestamp_HPP
#define NMM_ISendFilenameAndTimestamp_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/messaging/Timestamp.hpp"

#include "visibility.hpp"

namespace NMM
{
  /**
   * Sending of timestamps.
   *
   * Defines an event containing a filename and a timestamp that can
   * be sent by any node and caught by the application.
   */

  class DLLEXPORT ISendFilenameAndTimestamp : virtual public NMM::Interface { 
  public:
    ISendFilenameAndTimestamp() : NMM::Interface(ISendFilenameAndTimestamp::ISendFilenameAndTimestamp_type, "NMM") { };

    ISendFilenameAndTimestamp(NMM::NMMObject& parent) : NMM::Interface (parent, ISendFilenameAndTimestamp::ISendFilenameAndTimestamp_type, "NMM") { };

    virtual ~ISendFilenameAndTimestamp() {};

    static const char* ISendFilenameAndTimestamp_type;

    virtual  string getInterfaceType() const { return ISendFilenameAndTimestamp_type; }

    NMM::Interface* copy() const { return new ISendFilenameAndTimestamp(*this); }
    static void used(); 
    static const char* sendFilenameAndTimestamp_event;
    static NMM::Event* create_sendFilenameAndTimestamp(const string& fn, const Timestamp& ts);
  /* instream (protected) and non-instream (public) */        /** Sends the filename and timestamp.
     * 
     * @param fn filename to be sent
     * @param ts the timestamp to be sent
     * @return Result
     */

    virtual Result sendFilenameAndTimestamp(const string& fn, const Timestamp& ts) /* in-stream */ 
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ISendFilenameAndTimestamp> ISendFilenameAndTimestamp_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
