/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ISendFilenameAndTimestampImpl_HPP
#define NMM_ISendFilenameAndTimestampImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "ISendFilenameAndTimestamp.hpp"


#include "visibility.hpp"

namespace NMM
{
  /**
   * Sending of timestamps.
   *
   * Defines an event containing a filename and a timestamp that can
   * be sent by any node and caught by the application.
   */

  class DLLEXPORT ISendFilenameAndTimestampImpl  : public virtual NMM::NMMObject { 
  public:
    ISendFilenameAndTimestampImpl() { addInterface(new ISendFilenameAndTimestamp(*this)); registerEvents(); };

    virtual ~ISendFilenameAndTimestampImpl() {};

  protected:
    /** Sends the filename and timestamp.
     * 
     * @param fn filename to be sent
     * @param ts the timestamp to be sent
     * @return Result
     */

    virtual Result sendFilenameAndTimestamp(const string& fn, const Timestamp& ts) /* in-stream */  = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
