/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ISinkFileHandler.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ISinkFileHandler::used() {} 
  
  
  void ISinkFileHandler::setWorkingDirectory(const string& directory)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > directory_value(directory, false);
    NMM::Event event("ISinkFileHandler::setWorkingDirectory", &directory_value);
    if(const_cast<ISinkFileHandler*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  string ISinkFileHandler::getWorkingDirectory()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("ISinkFileHandler::getWorkingDirectory");
    event.setReturnValue(&return_value);
    if(const_cast<ISinkFileHandler*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ISinkFileHandler::setFileExtension(const string& extension)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > extension_value(extension, false);
    NMM::Event event("ISinkFileHandler::setFileExtension", &extension_value);
    if(const_cast<ISinkFileHandler*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  string ISinkFileHandler::getFileExtension()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("ISinkFileHandler::getFileExtension");
    event.setReturnValue(&return_value);
    if(const_cast<ISinkFileHandler*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ISinkFileHandler::ISinkFileHandler_type = "ISinkFileHandler";
  NMM::TConstructor<ISinkFileHandler > __ISinkFileHandler_prototype("ISinkFileHandler","NMM");


  const char* ISinkFileHandler::setWorkingDirectory_event = "ISinkFileHandler::setWorkingDirectory";
  const char* ISinkFileHandler::getWorkingDirectory_event = "ISinkFileHandler::getWorkingDirectory";
  const char* ISinkFileHandler::setFileExtension_event = "ISinkFileHandler::setFileExtension";
  const char* ISinkFileHandler::getFileExtension_event = "ISinkFileHandler::getFileExtension";


}


#include "nmm/nmm_suffix.hpp"

