/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ISinkFileHandler_HPP
#define NMM_ISinkFileHandler_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "nmm/plugins/interfaces/IFileHandler.hpp"
#include "nmm/multimedia/interfaces/ITrack.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Interface especially for configuring FileSinkNodes.
   */

  class DLLEXPORT ISinkFileHandler : virtual public IFileHandler { 
  public:
    ISinkFileHandler() : NMM::Interface(ISinkFileHandler::ISinkFileHandler_type, "NMM") { };

    ISinkFileHandler(NMM::NMMObject& parent) : NMM::Interface (parent, ISinkFileHandler::ISinkFileHandler_type, "NMM") { };

    virtual ~ISinkFileHandler() {};

    static const char* ISinkFileHandler_type;

    virtual  string getInterfaceType() const { return ISinkFileHandler_type; }

    NMM::Interface* copy() const { return new ISinkFileHandler(*this); }
    static void used(); 
    static const char* setWorkingDirectory_event;
    static const char* getWorkingDirectory_event;
    static const char* setFileExtension_event;
    static const char* getFileExtension_event;
    /** Sets working directory.
     *
     * @param directory  Working directory to work in.
     */

    virtual void setWorkingDirectory(const string& directory)
      throw(const NMM::SerializedException&);
    /** Returns current working directory.
     *
     * @return Current Working directory.
     */

    virtual string getWorkingDirectory()
      throw(const NMM::SerializedException&);
    /** Sets file extension of file to be written.
     *
     * @param extension  file extension of file to be written.
     */

    virtual void setFileExtension(const string& extension)
      throw(const NMM::SerializedException&);
     /** Returns file extension of file to be written
     *
     * @return File extension of file to be written.
     */

    virtual string getFileExtension()
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ISinkFileHandler> ISinkFileHandler_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
