/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ISinkFileHandlerImpl_HPP
#define NMM_ISinkFileHandlerImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "ISinkFileHandler.hpp"

#include "nmm/plugins/interfaces/IFileHandlerImpl.hpp"
#include "nmm/multimedia/interfaces/ITrackImpl.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Interface especially for configuring FileSinkNodes.
   */

  class DLLEXPORT ISinkFileHandlerImpl  : virtual public IFileHandlerImpl { 
  public:
    ISinkFileHandlerImpl() { addInterface(new ISinkFileHandler(*this)); registerEvents(); };

    virtual ~ISinkFileHandlerImpl() {};

  protected:
    /** Sets working directory.
     *
     * @param directory  Working directory to work in.
     */

    virtual void setWorkingDirectory(const string& directory) = 0;
    /** Returns current working directory.
     *
     * @return Current Working directory.
     */

    virtual string getWorkingDirectory() = 0;
    /** Sets file extension of file to be written.
     *
     * @param extension  file extension of file to be written.
     */

    virtual void setFileExtension(const string& extension) = 0;
     /** Returns file extension of file to be written
     *
     * @return File extension of file to be written.
     */

    virtual string getFileExtension() = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
