/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ISourceFileHandler.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ISourceFileHandler::used() {} 
  
  
  nmm_uint64 ISourceFileHandler::getFilesize() /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<nmm_uint64 > return_value;
    NMM::Event event("ISourceFileHandler::getFilesize");
    event.setReturnValue(&return_value);
    if(const_cast<ISourceFileHandler*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ISourceFileHandler::ISourceFileHandler_type = "ISourceFileHandler";
  NMM::TConstructor<ISourceFileHandler > __ISourceFileHandler_prototype("ISourceFileHandler","NMM");


  const char* ISourceFileHandler::getFilesize_event = "ISourceFileHandler::getFilesize";


}


#include "nmm/nmm_suffix.hpp"

