/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ISourceFileHandler_HPP
#define NMM_ISourceFileHandler_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "nmm/multimedia/INode.hpp"
#include "nmm/plugins/interfaces/IFileHandler.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Interface especially for configuring FileSourceNodes.
   */

  class DLLEXPORT ISourceFileHandler : virtual public IFileHandler { 
  public:
    ISourceFileHandler() : NMM::Interface(ISourceFileHandler::ISourceFileHandler_type, "NMM") { };

    ISourceFileHandler(NMM::NMMObject& parent) : NMM::Interface (parent, ISourceFileHandler::ISourceFileHandler_type, "NMM") { };

    virtual ~ISourceFileHandler() {};

    static const char* ISourceFileHandler_type;

    virtual  string getInterfaceType() const { return ISourceFileHandler_type; }

    NMM::Interface* copy() const { return new ISourceFileHandler(*this); }
    static void used(); 
    static const char* getFilesize_event;
    /** Returns file size of currently opened file.
     *
     * @return File size of currently opened file.
     */

    virtual nmm_uint64 getFilesize() /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);


  };

  typedef auto_ptr<ISourceFileHandler> ISourceFileHandler_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
