/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "MPEGCheckFormat.hpp"

#include <string.h>
#include <iostream>
using namespace std;

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  MPEGCheckFormat::MPEGCheckFormat()
   { 
    const char * ari_str[8] = {
	"Invalid Aspect Ratio",
	"1:1",
	"4:3",
	"16:9",
	"2.21:1",
	"Invalid Aspect Ratio",
	"Invalid Aspect Ratio",
	"Invalid Aspect Ratio"
    };
    
    const char * fr_str[16] = {
	"Invalid frame_rate_code",
	"23.976", "24", "25" , "29.97",
	"30" , "50", "59.94", "60" ,
	"Invalid frame_rate_code", "Invalid frame_rate_code",
	"Invalid frame_rate_code", "Invalid frame_rate_code",
	"Invalid frame_rate_code", "Invalid frame_rate_code",
	"Invalid frame_rate_code"
    };
    
    int skip_table[16] = {
	     1, 0xffff,      5,     10, 0xffff, 0xffff, 0xffff, 0xffff,
	0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff
    };


    int i;
    
    for( i=0; i<8; i++ ) {
      aspect_ratio_information_str[i] = new char[30];
      strcpy( aspect_ratio_information_str[i], ari_str[i] );
    }
    
    for( i=0; i<16; i++ ) {
      frame_rate_str[i] = new char[30];
      strcpy( frame_rate_str[i], fr_str[i] );
    }
    
    for( i=0; i<16; i++ )
      mpeg1_skip_table[i] = skip_table[i];    

    INIT_BUFFER_SIZE=262144;
    max_packets = 5000;
   }


  MPEGCheckFormat::~MPEGCheckFormat() {
  }

  bool MPEGCheckFormat::CheckFormat( mpeg_file_format *form ) {
    u_int8_t* buffer;
    int      packets;
    bool     finished;

    code = 0xff;
    chunk_ptr = chunk_buffer;
    shift = 0;

    found_pes_stream  = false;
    found_mpeg_audio  = false;
    found_ac3_audio   = false;
    found_mpeg2_video = false;
    found_mpeg1_video = false;
    found_sequence    = false;

    packets = 0;
    finished = false;

    u_int8_t * buf;
    u_int8_t * end;
    u_int8_t * tmp1;
    u_int8_t * tmp2;

    buffer = new u_int8_t[INIT_BUFFER_SIZE];
  
    buf = buffer;

    do {
        end = buf + MPEG_Read_Buffer( buf, buffer + INIT_BUFFER_SIZE - buf );
	buf = buffer;

	while (buf + 4 <= end) {
	    if (buf[0] || buf[1] || (buf[2] != 0x01)) {
		buf++;
		continue;
	    }

            /* Here we can assume that we have a pes-stream */
            found_pes_stream = true;
            packets++;
            if ( packets > max_packets ) { finished = true; break; }

	    switch (buf[3]) {
	    case 0xb9:	/* program end code */
		continue;

	    case 0xba:	/* pack header */
  		/* skip */
		if ((buf[4] & 0xc0) == 0x40) /* mpeg2 */
		    tmp1 = buf + 14 + (buf[13] & 7);
		else if ((buf[4] & 0xf0) == 0x20) /* mpeg1 */
		    tmp1 = buf + 12;
		else if (buf + 5 > end)
		    goto copy;
		else {
		    // weird pack header -> exit?
		}
		if (tmp1 > end)
		    goto copy;
		buf = tmp1;
		break;


	    case 0xe0:	/* video */
		tmp2 = buf + 6 + (buf[4] << 8) + buf[5];
		if (tmp2 > end)
		    goto copy;
                /* mpeg2 */
		if ((buf[6] & 0xc0) == 0x80) {
		    tmp1 = buf + 9 + buf[8];
                    found_mpeg2_video = true;
                }
		else {	/* mpeg1 */
                    found_mpeg1_video = true;
		    for (tmp1 = buf + 6; *tmp1 == 0xff; tmp1++)
			if (tmp1 == buf + 6 + 16) {
			    // too much stuffing
			    buf = tmp2;
			    break;
			}
		    if ((*tmp1 & 0xc0) == 0x40)
			tmp1 += 2;
		    tmp1 += mpeg1_skip_table [*tmp1 >> 4];
		}

                if ( !found_sequence ) {

		  mpeg2_info( tmp1, tmp2 );
		  finished = true;
		  break;
		}

		buf = tmp2;
		break;


	    case 0xc0:	 //mpeg2 audio
		tmp2 = buf + 6 + (buf[4] << 8) + buf[5];
		if (tmp2 > end)
		    goto copy;

		if ((buf[6] & 0xc0) == 0x80)
		    tmp1 = buf + 9 + buf[8];

                found_mpeg_audio = true;

		buf = tmp2;
		break;

            /* Private_stream_1: there you can find the ac3 streams */
            case 0xbd:
		tmp2 = buf + 6 + (buf[4] << 8) + buf[5];
		tmp1 = buf + 9 + buf[8];

                if ( (*tmp1 >= 0x80) && (*tmp1 <= 0x87) ) found_ac3_audio = true;

		buf = tmp2;
		break;


	    default:
		if (buf[3] < 0xb9) {
		  //looks like a video stream, not system stream
                  found_pes_stream = false;
		}
		/* skip */
		tmp1 = buf + 6 + (buf[4] << 8) + buf[5];
		if (tmp1 > end)
		    goto copy;
		buf = tmp1;
		break;
	    }
	}

	copy:
	if (buf < end) {
	    /* we only pass here for mpeg1 ps streams */
	    memmove (buffer, buf, end - buf);
	}
	buf = buffer + (end - buf);

    } while ( (end == buffer + INIT_BUFFER_SIZE) && !finished );


    /* if no pes-stream was found, we can try to check if it's an es-stream */
    if ( !found_pes_stream ) {
      // fileposition to startposition
      MPEG_Seek_Begin();
      end = buffer + MPEG_Read_Buffer( buffer, INIT_BUFFER_SIZE );
      mpeg2_info( buffer, end );
    }

    MPEG_Seek_Begin();

    delete[] buffer;

    if ( found_sequence ) {
      form->pes_stream      = found_pes_stream;
      form->mpeg_audio      = found_mpeg_audio;
      form->ac3_audio       = found_ac3_audio;
      form->mpeg2_video     = found_mpeg2_video;
      form->mpeg1_video     = found_mpeg1_video;
      form->width           = horizontal_size;
      form->height          = vertical_size;
      form->frame_rate_code = frame_rate_code;
      // convert framerate code to "readable" double value
      form->frame_rate      = atof( frame_rate_str [frame_rate_code] );
      switch( aspect_ratio_information ) {
	case 2:
	  form->ratio_x = 400;
	  form->ratio_y = 300;
	break;

	case 3:
	  form->ratio_x = 1600;
	  form->ratio_y = 900;
	break;

	case 4:
	  form->ratio_x = 221;
	  form->ratio_y = 100;
	break;

	default:
	  form->ratio_x = 100;
	  form->ratio_y = 100;
	break;
      }

      return true;
    }

    return false;
  }
  
  void MPEGCheckFormat::mpeg2_info(u_int8_t *current, u_int8_t *end) {
    u_int8_t code = 0xff;
    u_int8_t chunk_buffer[65536];
    u_int8_t *chunk_ptr = chunk_buffer;
    u_int32_t shift = 0;
    u_int8_t byte;

    while (current != end) {
	while (1) {
	    byte = *current++;
	    if (shift != 0x00000100) {
		*chunk_ptr++ = byte;
		shift = (shift | byte) << 8;
		if (current != end)
		    continue;
		return;
	    }
	    break;
	}

        /* found start_code following chunk */
        check_sequence(code, chunk_buffer);


        code = byte;
        chunk_ptr = chunk_buffer;
        shift = 0xffffff00;
/*        if ( found_sequence )
          break;*/
    }
  }
  
  void MPEGCheckFormat::check_sequence(int code, u_int8_t * buffer) {
    int bit_rate_value;
    int vbv_buffer_size_value;
    int constrained_parameters_flag;
    int load_intra_quantizer_matrix;
    int load_non_intra_quantizer_matrix;

    /* Here all information is extracted out of mpeg stream */
    if ( code == 0xb3 ) {
      found_sequence = true;
      vertical_size = (buffer[0] << 16) | (buffer[1] << 8) | buffer[2];
      horizontal_size = vertical_size >> 12;
      vertical_size &= 0xfff;
      aspect_ratio_information = buffer[3] >> 4;
      frame_rate_code = buffer[3] & 15;
      bit_rate_value = (buffer[4] << 10) | (buffer[5] << 2) | (buffer[6] >> 6);
      vbv_buffer_size_value = ((buffer[6] << 5) | (buffer[7] >> 3)) & 0x3ff;
      constrained_parameters_flag = buffer[7] & 4;
      load_intra_quantizer_matrix = buffer[7] & 2;
      if (load_intra_quantizer_matrix) buffer += 64;
      load_non_intra_quantizer_matrix = buffer[7] & 1;

/*      printf (" (seq) %dx%d %s, %s fps, %5.0f kbps\n",
	     horizontal_size, vertical_size,
	     aspect_ratio_information_str [aspect_ratio_information],
	     frame_rate_str [frame_rate_code],
	     bit_rate_value * 400.0 / 1000.0);*/
   }
  }
}

#include "nmm/nmm_suffix.hpp"
