/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ICreateImage.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ICreateImage::used() {} 
  
  
  Result ICreateImage::createImage(const nmm_uint32& color, const nmm_uint32& w, const nmm_uint32& h)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_uint32 > color_value(color, false);
    NMM::TInValue<nmm_uint32 > w_value(w, false);
    NMM::TInValue<nmm_uint32 > h_value(h, false);
    NMM::Event event("ICreateImage::createImage", &color_value, &w_value, &h_value);
    event.setReturnValue(&return_value);
    if(const_cast<ICreateImage*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ICreateImage::ICreateImage_type = "ICreateImage";
  NMM::TConstructor<ICreateImage > __ICreateImage_prototype("ICreateImage","NMM");


  const char* ICreateImage::createImage_event = "ICreateImage::createImage";


}


#include "nmm/nmm_suffix.hpp"

