/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IMultipleSourceFileHandler.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IMultipleSourceFileHandler::used() {} 
  
  
  void IMultipleSourceFileHandler::addFilename(const string& name) /* limited(INode::CONSTRUCTED) */ /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<string > name_value(name, false);
    NMM::Event event("IMultipleSourceFileHandler::addFilename", &name_value);
    if(const_cast<IMultipleSourceFileHandler*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IMultipleSourceFileHandler::IMultipleSourceFileHandler_type = "IMultipleSourceFileHandler";
  NMM::TConstructor<IMultipleSourceFileHandler > __IMultipleSourceFileHandler_prototype("IMultipleSourceFileHandler","NMM");


  const char* IMultipleSourceFileHandler::addFilename_event = "IMultipleSourceFileHandler::addFilename";


}


#include "nmm/nmm_suffix.hpp"

