/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ITimeshifting.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/serialize/SerializeUtilsTime.hpp"

namespace NMM
{
  void ITimeshifting::used() {} 
  
  
  void ITimeshifting::enableMemBuf(const nmm_uint32& size /* = 60 */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > size_value(size, false);
    NMM::Event event("ITimeshifting::enableMemBuf", &size_value);
    if(const_cast<ITimeshifting*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITimeshifting::disableMemBuf(const nmm_bool& play /* = false */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > play_value(play, false);
    NMM::Event event("ITimeshifting::disableMemBuf", &play_value);
    if(const_cast<ITimeshifting*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_bool ITimeshifting::isMemBuf()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ITimeshifting::isMemBuf");
    event.setReturnValue(&return_value);
    if(const_cast<ITimeshifting*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITimeshifting::keepData(const nmm_bool& keep)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > keep_value(keep, false);
    NMM::Event event("ITimeshifting::keepData", &keep_value);
    if(const_cast<ITimeshifting*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  ITimeshifting::Mode ITimeshifting::getMode() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ITimeshifting::Mode > return_value;
    NMM::Event event("ITimeshifting::getMode");
    event.setReturnValue(&return_value);
    if(const_cast<ITimeshifting*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ITimeshifting::PlayMode ITimeshifting::getPlayMode() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ITimeshifting::PlayMode > return_value;
    NMM::Event event("ITimeshifting::getPlayMode");
    event.setReturnValue(&return_value);
    if(const_cast<ITimeshifting*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITimeshifting::setModeLive(const nmm_bool& enable_memory_buffer /* = true */, const nmm_bool& send_sync_event /* = true */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > enable_memory_buffer_value(enable_memory_buffer, false);
    NMM::TInValue<nmm_bool > send_sync_event_value(send_sync_event, false);
    NMM::Event event("ITimeshifting::setModeLive", &enable_memory_buffer_value, &send_sync_event_value);
    if(const_cast<ITimeshifting*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITimeshifting::setModeFBLive(const nmm_bool& send_sync_event /* = true */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > send_sync_event_value(send_sync_event, false);
    NMM::Event event("ITimeshifting::setModeFBLive", &send_sync_event_value);
    if(const_cast<ITimeshifting*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITimeshifting::setModePlay(const nmm_bool& send_sync_reset /* = true */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > send_sync_reset_value(send_sync_reset, false);
    NMM::Event event("ITimeshifting::setModePlay", &send_sync_reset_value);
    if(const_cast<ITimeshifting*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITimeshifting::setModePause()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("ITimeshifting::setModePause");
    if(const_cast<ITimeshifting*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITimeshifting::setModeFastForward(const nmm_int32& speed /* = 5 */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > speed_value(speed, false);
    NMM::Event event("ITimeshifting::setModeFastForward", &speed_value);
    if(const_cast<ITimeshifting*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITimeshifting::setModeRewind(const nmm_int32& speed /* = -5 */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > speed_value(speed, false);
    NMM::Event event("ITimeshifting::setModeRewind", &speed_value);
    if(const_cast<ITimeshifting*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITimeshifting::setFilename(const string& filename)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > filename_value(filename, false);
    NMM::Event event("ITimeshifting::setFilename", &filename_value);
    if(const_cast<ITimeshifting*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITimeshifting::setFileBufferThreshhold(const nmm_uint32& threshold) /* limited(INode::CONSTRUCTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_uint32 > threshold_value(threshold, false);
    NMM::Event event("ITimeshifting::setFileBufferThreshhold", &threshold_value);
    if(const_cast<ITimeshifting*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  Result ITimeshifting::setBufferSize(const nmm_uint32& size)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_uint32 > size_value(size, false);
    NMM::Event event("ITimeshifting::setBufferSize", &size_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITimeshifting*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITimeshifting::startRecord()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("ITimeshifting::startRecord");
    if(const_cast<ITimeshifting*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITimeshifting::stopRecord()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("ITimeshifting::stopRecord");
    if(const_cast<ITimeshifting*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_bool ITimeshifting::isRecording()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ITimeshifting::isRecording");
    event.setReturnValue(&return_value);
    if(const_cast<ITimeshifting*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITimeshifting::saveToFile(const string& filename)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > filename_value(filename, false);
    NMM::Event event("ITimeshifting::saveToFile", &filename_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITimeshifting*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Time ITimeshifting::getReadPosition() const /* limited(INode::STARTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Time > return_value;
    NMM::Event event("ITimeshifting::getReadPosition");
    event.setReturnValue(&return_value);
    if(const_cast<ITimeshifting*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Time ITimeshifting::getStartTime() const /* limited(INode::STARTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Time > return_value;
    NMM::Event event("ITimeshifting::getStartTime");
    event.setReturnValue(&return_value);
    if(const_cast<ITimeshifting*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Time ITimeshifting::getLivePosition() const /* limited(INode::STARTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Time > return_value;
    NMM::Event event("ITimeshifting::getLivePosition");
    event.setReturnValue(&return_value);
    if(const_cast<ITimeshifting*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Interval ITimeshifting::getLiveOffset() const /* limited(INode::STARTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Interval > return_value;
    NMM::Event event("ITimeshifting::getLiveOffset");
    event.setReturnValue(&return_value);
    if(const_cast<ITimeshifting*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Interval ITimeshifting::getFileLength() const /* limited(INode::STARTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Interval > return_value;
    NMM::Event event("ITimeshifting::getFileLength");
    event.setReturnValue(&return_value);
    if(const_cast<ITimeshifting*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ITimeshifting::ITimeshifting_type = "ITimeshifting";
  NMM::TConstructor<ITimeshifting > __ITimeshifting_prototype("ITimeshifting","NMM");
  TValueConstructor< ITimeshifting::Mode > __ITimeshifting_Mode_prototype(TypeInfo("ITimeshifting::Mode", "NMM"));

  NMM::OValueStream& operator<<(NMM::OValueStream& ov, const ITimeshifting::Mode& t)
  {
    ov.beginType(TypeInfo("ITimeshifting::Mode", "NMM"));
    nmm_int32 temp = static_cast<nmm_int32>(t);
    ov << temp;
    ov.endType();
    return ov;
  }

  NMM::IValueStream& operator>>(NMM::IValueStream& iv, ITimeshifting::Mode& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("ITimeshifting::Mode", "NMM")) {
      throw NMM::InvalidTypeException(TypeInfo("ITimeshifting::Mode", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    nmm_int32 temp;
    iv >> temp;
    t = static_cast<ITimeshifting::Mode>(temp);
    iv.goUp();
    return iv;
  }

  TValueConstructor< ITimeshifting::PlayMode > __ITimeshifting_PlayMode_prototype(TypeInfo("ITimeshifting::PlayMode", "NMM"));

  NMM::OValueStream& operator<<(NMM::OValueStream& ov, const ITimeshifting::PlayMode& t)
  {
    ov.beginType(TypeInfo("ITimeshifting::PlayMode", "NMM"));
    nmm_int32 temp = static_cast<nmm_int32>(t);
    ov << temp;
    ov.endType();
    return ov;
  }

  NMM::IValueStream& operator>>(NMM::IValueStream& iv, ITimeshifting::PlayMode& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("ITimeshifting::PlayMode", "NMM")) {
      throw NMM::InvalidTypeException(TypeInfo("ITimeshifting::PlayMode", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    nmm_int32 temp;
    iv >> temp;
    t = static_cast<ITimeshifting::PlayMode>(temp);
    iv.goUp();
    return iv;
  }



  const char* ITimeshifting::enableMemBuf_event = "ITimeshifting::enableMemBuf";
  const char* ITimeshifting::disableMemBuf_event = "ITimeshifting::disableMemBuf";
  const char* ITimeshifting::isMemBuf_event = "ITimeshifting::isMemBuf";
  const char* ITimeshifting::keepData_event = "ITimeshifting::keepData";
  const char* ITimeshifting::getMode_event = "ITimeshifting::getMode";
  const char* ITimeshifting::getPlayMode_event = "ITimeshifting::getPlayMode";
  const char* ITimeshifting::setModeLive_event = "ITimeshifting::setModeLive";
  const char* ITimeshifting::setModeFBLive_event = "ITimeshifting::setModeFBLive";
  const char* ITimeshifting::setModePlay_event = "ITimeshifting::setModePlay";
  const char* ITimeshifting::setModePause_event = "ITimeshifting::setModePause";
  const char* ITimeshifting::setModeFastForward_event = "ITimeshifting::setModeFastForward";
  const char* ITimeshifting::setModeRewind_event = "ITimeshifting::setModeRewind";
  const char* ITimeshifting::setFilename_event = "ITimeshifting::setFilename";
  const char* ITimeshifting::setFileBufferThreshhold_event = "ITimeshifting::setFileBufferThreshhold";
  const char* ITimeshifting::setBufferSize_event = "ITimeshifting::setBufferSize";
  const char* ITimeshifting::startRecord_event = "ITimeshifting::startRecord";
  const char* ITimeshifting::stopRecord_event = "ITimeshifting::stopRecord";
  const char* ITimeshifting::isRecording_event = "ITimeshifting::isRecording";
  const char* ITimeshifting::saveToFile_event = "ITimeshifting::saveToFile";
  const char* ITimeshifting::getReadPosition_event = "ITimeshifting::getReadPosition";
  const char* ITimeshifting::getStartTime_event = "ITimeshifting::getStartTime";
  const char* ITimeshifting::getLivePosition_event = "ITimeshifting::getLivePosition";
  const char* ITimeshifting::getLiveOffset_event = "ITimeshifting::getLiveOffset";
  const char* ITimeshifting::getFileLength_event = "ITimeshifting::getFileLength";
  void ITimeshiftingListener::used() {} 
  
  
  Result ITimeshiftingListener::timeshiftingBegin() /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("ITimeshiftingListener::timeshiftingBegin");
    event.setReturnValue(&return_value);
    if(const_cast<ITimeshiftingListener*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITimeshiftingListener::live() /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("ITimeshiftingListener::live");
    event.setReturnValue(&return_value);
    if(const_cast<ITimeshiftingListener*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITimeshiftingListener::setProgress(const Time& ts_time) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<Time > ts_time_value(ts_time, false);
    NMM::Event event("ITimeshiftingListener::setProgress", &ts_time_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITimeshiftingListener*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ITimeshiftingListener::ITimeshiftingListener_type = "ITimeshiftingListener";
  NMM::TConstructor<ITimeshiftingListener > __ITimeshiftingListener_prototype("ITimeshiftingListener","NMM");


  const char* ITimeshiftingListener::timeshiftingBegin_event = "ITimeshiftingListener::timeshiftingBegin";
  NMM::Event* ITimeshiftingListener::create_timeshiftingBegin()
  {

    return new NMM::Event("ITimeshiftingListener::timeshiftingBegin");
  }

  const char* ITimeshiftingListener::live_event = "ITimeshiftingListener::live";
  NMM::Event* ITimeshiftingListener::create_live()
  {

    return new NMM::Event("ITimeshiftingListener::live");
  }

  const char* ITimeshiftingListener::setProgress_event = "ITimeshiftingListener::setProgress";
  NMM::Event* ITimeshiftingListener::create_setProgress(const Time& ts_time)
  {
    NMM::TInValue<Time >* ts_time_value = new NMM::TInValue<Time >();
    ts_time_value->setValue(ts_time);

    return new NMM::Event("ITimeshiftingListener::setProgress", ts_time_value);
  }



}


#include "nmm/nmm_suffix.hpp"

