/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ITimeshiftingImpl.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/messaging/TEDObject.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/serialize/SerializeUtilsTime.hpp"

namespace NMM
{
  
  void ITimeshiftingImpl::registerEvents()
  {
  
    getEventDispatcher().registerEvent(ITimeshifting::enableMemBuf_event, new NMM::TEDObject1<ITimeshiftingImpl, nmm_uint32, void(ITimeshiftingImpl:: * )(const nmm_uint32&), void >(this, &ITimeshiftingImpl::enableMemBuf));
  
    getEventDispatcher().registerEvent(ITimeshifting::disableMemBuf_event, new NMM::TEDObject1<ITimeshiftingImpl, nmm_bool, void(ITimeshiftingImpl:: * )(const nmm_bool&), void >(this, &ITimeshiftingImpl::disableMemBuf));
  
    getEventDispatcher().registerEvent(ITimeshifting::isMemBuf_event, new NMM::TEDObject0<ITimeshiftingImpl, nmm_bool(ITimeshiftingImpl:: * )(), nmm_bool >(this, &ITimeshiftingImpl::isMemBuf));
  
    getEventDispatcher().registerEvent(ITimeshifting::keepData_event, new NMM::TEDObject1<ITimeshiftingImpl, nmm_bool, void(ITimeshiftingImpl:: * )(const nmm_bool&), void >(this, &ITimeshiftingImpl::keepData));
  
    getEventDispatcher().registerEvent(ITimeshifting::getMode_event, new NMM::TEDObject0<ITimeshiftingImpl, ITimeshifting::Mode(ITimeshiftingImpl:: * )() const, ITimeshifting::Mode >(this, &ITimeshiftingImpl::getMode));
  
    getEventDispatcher().registerEvent(ITimeshifting::getPlayMode_event, new NMM::TEDObject0<ITimeshiftingImpl, ITimeshifting::PlayMode(ITimeshiftingImpl:: * )() const, ITimeshifting::PlayMode >(this, &ITimeshiftingImpl::getPlayMode));
  
    getEventDispatcher().registerEvent(ITimeshifting::setModeLive_event, new NMM::TEDObject2<ITimeshiftingImpl, nmm_bool, nmm_bool, void(ITimeshiftingImpl:: * )(const nmm_bool&, const nmm_bool&), void >(this, &ITimeshiftingImpl::setModeLive));
  
    getEventDispatcher().registerEvent(ITimeshifting::setModeFBLive_event, new NMM::TEDObject1<ITimeshiftingImpl, nmm_bool, void(ITimeshiftingImpl:: * )(const nmm_bool&), void >(this, &ITimeshiftingImpl::setModeFBLive));
  
    getEventDispatcher().registerEvent(ITimeshifting::setModePlay_event, new NMM::TEDObject1<ITimeshiftingImpl, nmm_bool, void(ITimeshiftingImpl:: * )(const nmm_bool&), void >(this, &ITimeshiftingImpl::setModePlay));
  
    getEventDispatcher().registerEvent(ITimeshifting::setModePause_event, new NMM::TEDObject0<ITimeshiftingImpl, void(ITimeshiftingImpl:: * )(), void >(this, &ITimeshiftingImpl::setModePause));
  
    getEventDispatcher().registerEvent(ITimeshifting::setModeFastForward_event, new NMM::TEDObject1<ITimeshiftingImpl, nmm_int32, void(ITimeshiftingImpl:: * )(const nmm_int32&), void >(this, &ITimeshiftingImpl::setModeFastForward));
  
    getEventDispatcher().registerEvent(ITimeshifting::setModeRewind_event, new NMM::TEDObject1<ITimeshiftingImpl, nmm_int32, void(ITimeshiftingImpl:: * )(const nmm_int32&), void >(this, &ITimeshiftingImpl::setModeRewind));
  
    getEventDispatcher().registerEvent(ITimeshifting::setFilename_event, new NMM::TEDObject1<ITimeshiftingImpl, string, void(ITimeshiftingImpl:: * )(const string&), void >(this, &ITimeshiftingImpl::setFilename));
  
    getEventDispatcher().registerEvent(ITimeshifting::setFileBufferThreshhold_event, new NMM::TEDObject1<ITimeshiftingImpl, nmm_uint32, void(ITimeshiftingImpl:: * )(const nmm_uint32&), void >(this, &ITimeshiftingImpl::setFileBufferThreshhold));
    getEventDispatcher().addLimited(ITimeshifting::setFileBufferThreshhold_event, INode::CONSTRUCTED);
  
    getEventDispatcher().registerEvent(ITimeshifting::setBufferSize_event, new NMM::TEDObject1<ITimeshiftingImpl, nmm_uint32, Result(ITimeshiftingImpl:: * )(const nmm_uint32&), Result >(this, &ITimeshiftingImpl::setBufferSize));
  
    getEventDispatcher().registerEvent(ITimeshifting::startRecord_event, new NMM::TEDObject0<ITimeshiftingImpl, void(ITimeshiftingImpl:: * )(), void >(this, &ITimeshiftingImpl::startRecord));
  
    getEventDispatcher().registerEvent(ITimeshifting::stopRecord_event, new NMM::TEDObject0<ITimeshiftingImpl, void(ITimeshiftingImpl:: * )(), void >(this, &ITimeshiftingImpl::stopRecord));
  
    getEventDispatcher().registerEvent(ITimeshifting::isRecording_event, new NMM::TEDObject0<ITimeshiftingImpl, nmm_bool(ITimeshiftingImpl:: * )(), nmm_bool >(this, &ITimeshiftingImpl::isRecording));
  
    getEventDispatcher().registerEvent(ITimeshifting::saveToFile_event, new NMM::TEDObject1<ITimeshiftingImpl, string, Result(ITimeshiftingImpl:: * )(const string&), Result >(this, &ITimeshiftingImpl::saveToFile));
  
    getEventDispatcher().registerEvent(ITimeshifting::getReadPosition_event, new NMM::TEDObject0<ITimeshiftingImpl, Time(ITimeshiftingImpl:: * )() const, Time >(this, &ITimeshiftingImpl::getReadPosition));
    getEventDispatcher().addLimited(ITimeshifting::getReadPosition_event, INode::STARTED);
  
    getEventDispatcher().registerEvent(ITimeshifting::getStartTime_event, new NMM::TEDObject0<ITimeshiftingImpl, Time(ITimeshiftingImpl:: * )() const, Time >(this, &ITimeshiftingImpl::getStartTime));
    getEventDispatcher().addLimited(ITimeshifting::getStartTime_event, INode::STARTED);
  
    getEventDispatcher().registerEvent(ITimeshifting::getLivePosition_event, new NMM::TEDObject0<ITimeshiftingImpl, Time(ITimeshiftingImpl:: * )() const, Time >(this, &ITimeshiftingImpl::getLivePosition));
    getEventDispatcher().addLimited(ITimeshifting::getLivePosition_event, INode::STARTED);
  
    getEventDispatcher().registerEvent(ITimeshifting::getLiveOffset_event, new NMM::TEDObject0<ITimeshiftingImpl, Interval(ITimeshiftingImpl:: * )() const, Interval >(this, &ITimeshiftingImpl::getLiveOffset));
    getEventDispatcher().addLimited(ITimeshifting::getLiveOffset_event, INode::STARTED);
  
    getEventDispatcher().registerEvent(ITimeshifting::getFileLength_event, new NMM::TEDObject0<ITimeshiftingImpl, Interval(ITimeshiftingImpl:: * )() const, Interval >(this, &ITimeshiftingImpl::getFileLength));
    getEventDispatcher().addLimited(ITimeshifting::getFileLength_event, INode::STARTED);

  }
  
  void ITimeshiftingListenerImpl::registerEvents()
  {
  
    getEventDispatcher().registerEvent(ITimeshiftingListener::timeshiftingBegin_event, new NMM::TEDObject0<ITimeshiftingListenerImpl, Result(ITimeshiftingListenerImpl:: * )(), Result >(this, &ITimeshiftingListenerImpl::timeshiftingBegin));
  
    getEventDispatcher().registerEvent(ITimeshiftingListener::live_event, new NMM::TEDObject0<ITimeshiftingListenerImpl, Result(ITimeshiftingListenerImpl:: * )(), Result >(this, &ITimeshiftingListenerImpl::live));
  
    getEventDispatcher().registerEvent(ITimeshiftingListener::setProgress_event, new NMM::TEDObject1<ITimeshiftingListenerImpl, Time, Result(ITimeshiftingListenerImpl:: * )(const Time&), Result >(this, &ITimeshiftingListenerImpl::setProgress));

  }

}


#include "nmm/nmm_suffix.hpp"

