/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_BUFFERSHAPINGNODE_HPP
#define NMM_BUFFERSHAPINGNODE_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/multimedia/GenericFilterNode.hpp"
#include "nmm/plugins/interfaces/IBufferSizeImpl.hpp"

namespace NMM {
  
  class BufferShapingNode
    : public GenericFilterNode,
      public IBufferSizeImpl
  {
  private:
    size_t frame_size;
    Buffer* incomplete_out_buffer;
    queue<Buffer* > complete_out_buffers;
    Timestamp current_ts;
    
  protected:
    // IBufferSize interface
    virtual void setBufferSize(const nmm_int32& value);
    virtual nmm_int32 getBufferSize();

    virtual Result doInit();
    virtual Result doInitOutput();
    
    virtual Message *processBuffer(Buffer *in_buffer);
    
  public:
    BufferShapingNode(const char* name = "BufferShapingNode",
		const StreamQueue::Mode queue_mode = StreamQueue::DEFAULT_MODE,
		const int size = StreamQueue::DEFAULT_SIZE);
    virtual ~BufferShapingNode() {};
  };
}

#include "nmm/nmm_suffix.hpp"

#endif //NMM_IDNODE_HPP
