/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "CopyNode.hpp"
#include "nmm/comm/messaging/Buffer.hpp"
#include "nmm/comm/format/Property.hpp"
#include <stdio.h>
#include "nmm/plugins/NMMRegisterPlugin.hpp"

NMM_REGISTER_PLUGIN(CopyNode, NMM, "CopyNode")

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  CopyNode::CopyNode(const char *name, StreamQueue::Mode qmode, int qsize) :
    GenericDemultiplexerNode(name, qmode, qsize) 
  {
    num_of_outputs = 2;
  }
  
  void CopyNode::setNumberOfOutputs(const nmm_int32 &value) {
    num_of_outputs = value;
  }
  
  Result CopyNode::doInit() {
    Format* any_format_i = new Format("*/*");
    getOwnInputProperty()->addNewFormat(any_format_i);
    
    Format* any_format_o = new Format("*/*");
    getOwnOutputProperty()->addNewFormat(any_format_o);
    
    return SUCCESS;
  }
  
  Result CopyNode::doInitOutput() {
    if(! (num_of_outputs > 0)) {
      ERROR_STREAM("Number of outputs is set to 0, cannot initialize 0 outputs!" << endl);
      return FAILURE;
    }

    /* get input format */
    if (!getInputFormat()) {
      GLOBAL_ERROR_STREAM("No Input Format! FAILURE" << endl);
      return FAILURE;
    }
    Format* i_format = new Format(*getInputFormat());
    getOwnWorkingInputProperty()->addNewFormat(i_format);
    getOwnWorkingInputProperty()->setDefaultFormat(i_format);
    
    for( unsigned short i = 0; i < num_of_outputs; i++ ) {
      string tag = string("") + i;
      DEBUG_STREAM("Adding stream with name: " << tag << " and with format: " << i_format->getFormatType() << endl);
      if(FAILURE == addOutputStream(tag.c_str())) {
	WARNING_STREAM("Could not add stream with name " << tag << endl);
      } else {
        Format* o_format = new Format(*i_format);
        o_format->setJackTag(tag);
        getOwnWorkingOutputProperty()->addNewFormat(o_format);
        getOwnWorkingOutputProperty()->setDefaultFormat(o_format, tag);
      }
    }
    
    return SUCCESS;
  }
  
  Result CopyNode::doDeinitOutput() {
    getOwnWorkingInputProperty()->erase();
    getOwnWorkingOutputProperty()->erase();
    
    for(unsigned short i = 0; i < num_of_outputs; i++) {
      string tag = string("") + i;
      DEBUG_STREAM("Removing stream with name " << tag << endl);
      if(FAILURE == removeOutputStream(tag.c_str())) {
	WARNING_STREAM("Could not remove stream with name " << tag << endl);
      }
    }
    return SUCCESS;
  }
  
  Message *CopyNode::processBuffer(Buffer *in) 
  {
    if(num_of_outputs) {
      return in;
  }
    else {
      in->release();
      return 0;
    }
  }

}

#include "nmm/nmm_suffix.hpp"
