/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef _NMM_COPYNODE_HPP_
#define _NMM_COPYNODE_HPP_

#include "nmm/nmm_prefix.hpp"

#include "nmm/plugins/general/ICopyImpl.hpp"
#include "nmm/multimedia/GenericDemultiplexerNode.hpp"

namespace NMM {

  /**
   * \class CopyNode
   * \brief This class provides the functionality to copy arbitrary input streams to 
   * up to 256 outputs, which are named "0", "1", "2", ...
   */
  class CopyNode :
    public GenericDemultiplexerNode,
    public ICopyImpl
  {
  private:
    unsigned short num_of_outputs;
    
  protected:
    /**
     * ICopy interface.
     * Set the number of outputs.
     */
    void setNumberOfOutputs(const nmm_int32 &value);
    
    /**
     * Here all formats are specified.
     */
    virtual Result doInit();
    
    /**
     * Here all output streams are added.
     */
    virtual Result doInitOutput();

    /**
     * Here all output streams are removed.
     */
    virtual Result doDeinitOutput();

    /**
     * The processBuffer method from superclass
     */
    virtual Message *processBuffer (Buffer *in);

  public:

    /**
     * The main constructor.
     */
    CopyNode(const char *name = "CopyNode", StreamQueue::Mode = StreamQueue::MODE_SUSPEND, int qsize = 5);
  };

}

#include "nmm/nmm_suffix.hpp"

#endif //_NMM_VIDEO_FILTER_OVERLAYNODE_HPP_



