/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IBufferDrop.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IBufferDrop::used() {} 
  
  
  void IBufferDrop::setBufferToDrop(const nmm_uint32& buffer_num)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > buffer_num_value(buffer_num, false);
    NMM::Event event("IBufferDrop::setBufferToDrop", &buffer_num_value);
    if(const_cast<IBufferDrop*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IBufferDrop::setDropRatio(const nmm_uint32& numerator, const nmm_uint32& denominator)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > numerator_value(numerator, false);
    NMM::TInValue<nmm_uint32 > denominator_value(denominator, false);
    NMM::Event event("IBufferDrop::setDropRatio", &numerator_value, &denominator_value);
    if(const_cast<IBufferDrop*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IBufferDrop::IBufferDrop_type = "IBufferDrop";
  NMM::TConstructor<IBufferDrop > __IBufferDrop_prototype("IBufferDrop","NMM");


  const char* IBufferDrop::setBufferToDrop_event = "IBufferDrop::setBufferToDrop";
  const char* IBufferDrop::setDropRatio_event = "IBufferDrop::setDropRatio";
  void ITimedBufferDrop::used() {} 
  
  
  void ITimedBufferDrop::addInOutPoint(const nmm_uint32& in_secs, const nmm_uint32& in_millisecs, const nmm_uint32& out_secs, const nmm_uint32& out_millisecs) /* limited(INode::CONSTRUCTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_uint32 > in_secs_value(in_secs, false);
    NMM::TInValue<nmm_uint32 > in_millisecs_value(in_millisecs, false);
    NMM::TInValue<nmm_uint32 > out_secs_value(out_secs, false);
    NMM::TInValue<nmm_uint32 > out_millisecs_value(out_millisecs, false);
    NMM::Event event("ITimedBufferDrop::addInOutPoint", &in_secs_value, &in_millisecs_value, &out_secs_value, &out_millisecs_value);
    if(const_cast<ITimedBufferDrop*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITimedBufferDrop::useRelativeInOutPoints(const nmm_bool& relative) /* limited(INode::CONSTRUCTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_bool > relative_value(relative, false);
    NMM::Event event("ITimedBufferDrop::useRelativeInOutPoints", &relative_value);
    if(const_cast<ITimedBufferDrop*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_bool ITimedBufferDrop::isRelativeInOutPoints()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ITimedBufferDrop::isRelativeInOutPoints");
    event.setReturnValue(&return_value);
    if(const_cast<ITimedBufferDrop*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ITimedBufferDrop::ITimedBufferDrop_type = "ITimedBufferDrop";
  NMM::TConstructor<ITimedBufferDrop > __ITimedBufferDrop_prototype("ITimedBufferDrop","NMM");


  const char* ITimedBufferDrop::addInOutPoint_event = "ITimedBufferDrop::addInOutPoint";
  const char* ITimedBufferDrop::useRelativeInOutPoints_event = "ITimedBufferDrop::useRelativeInOutPoints";
  const char* ITimedBufferDrop::isRelativeInOutPoints_event = "ITimedBufferDrop::isRelativeInOutPoints";


}


#include "nmm/nmm_suffix.hpp"

