/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#include "Camera1394.hpp"
//#include <stream.h>
#include <iostream>
#include <l1394/l1394_session.h>
#include "nmm/comm/messaging/EventDispatcher.hpp"
#include "nmm/plugins/video/camera/ICamera.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM{

  Camera1394::Camera1394(const char* name)
  : GenericFireWireNode(name), camera(0)
  {

  }


  Result Camera1394::doActivate() {
    if (GenericFireWireNode::doActivate() == FAILURE)
      return FAILURE;
    return SUCCESS;
  }

  Result Camera1394::doStart()  {
    if (GenericFireWireNode::doStart() == FAILURE)
      return FAILURE;
    return SUCCESS;
  }
  Result Camera1394::doStop() {
    if (GenericFireWireNode::doStop() == FAILURE)
      return FAILURE;
    return SUCCESS;
  }


  Result Camera1394::doInit() {
    if (GenericFireWireNode::doInit() == FAILURE)
      return FAILURE;
    camera = dynamic_cast<L1394::Camera*>(getInternalDevice());
    if (camera) {
//      camera->whiteBalance(1)->automatic(true);
//      automatic = true;
      return SUCCESS;
    }
    return FAILURE;
  }

  void Camera1394::setBrightness(const nmm_int32& value){
    camera->brightness()->setValue(value);
  }

  void Camera1394::setAutoExposure(const nmm_int32& value){
    camera->autoExposure()->setValue(value);
  }
  void Camera1394::setSharpness(const nmm_int32& value){
    camera->sharpness()->setValue(value) ;
  }
  void Camera1394::setWhiteBalance(const nmm_int32& value){
    camera->whiteBalance()->setValue(value) ;
  }

  void Camera1394::setBlueBalance(const nmm_int32& value) {
    camera->whiteBalance(RED)->setValue(value);
  }
  void Camera1394::setRedBalance(const nmm_int32& value) {
    camera->whiteBalance(BLUE)->setValue(value);
  }

  void Camera1394::setHue(const nmm_int32& value) {
    camera->hue()->setValue(value) ;
  }
  void Camera1394::setSaturation(const nmm_int32& value){
    camera->saturation()->setValue(value) ;
  }
  void Camera1394::setGamma(const nmm_int32& value)    {
    camera->gamma()->setValue(value) ;
  }
  void Camera1394::setShutter(const nmm_int32& value){
    camera->shutter()->setValue(value) ;
  }
  void Camera1394::setGain(const nmm_int32& value)    {
    camera->gain()->setValue(value) ;
  }
  void Camera1394::setIris(const nmm_int32& value)    {
    camera->iris()->setValue(value) ;
  }  ;
  void Camera1394::setFocus(const nmm_int32& value)    {
    camera->focus()->setValue(value) ;
  };
  void Camera1394::setZoom(const nmm_int32& value)     {
    camera->zoom()->setValue(value);
  };
  void Camera1394::onePush()              {
    camera->whiteBalance()->onePush();
  };



  nmm_int32 Camera1394::getBrightness()        const{
    return camera->brightness()->getValue();
  };
  nmm_int32 Camera1394::getAutoExposure()      const{
    return camera->autoExposure()->getValue();
  };
  nmm_int32 Camera1394::getSharpness()         const{
    return camera->sharpness()->getValue();
  };
  nmm_int32 Camera1394::getWhiteBalance()      const{
    return camera->whiteBalance()->getValue();
  };
  nmm_int32 Camera1394::getBlueBalance()       const {
    return camera->whiteBalance(BLUE)->getValue();
  }
  nmm_int32 Camera1394::getRedBalance()        const {
    return camera->whiteBalance(RED)->getValue();
  }
  nmm_int32 Camera1394::getHue()               const{
    return camera->hue()->getValue();
  };
  nmm_int32 Camera1394::getSaturation()        const{
    return camera->saturation()->getValue();
  };
  nmm_int32 Camera1394::getGamma()             const{
    return camera->gamma()->getValue();
  };
  nmm_int32 Camera1394::getShutter()           const{
    return camera->shutter()->getValue();
  };
  nmm_int32 Camera1394::getGain()              const{
    return camera->gain()->getValue();
  };
  nmm_int32 Camera1394::getIris()              const{
    return camera->iris()->getValue();
  };
  nmm_int32 Camera1394::getFocus()             const{
    return camera->focus()->getValue();
  };
  nmm_int32 Camera1394::getZoom()              const{
    return camera->zoom()->getValue();
  };


  pair<nmm_int32, nmm_int32> Camera1394::getBrightnessRange()    const{
    return pair<nmm_int32, nmm_int32>(camera->brightness()->getMinValue(),camera->brightness()->getMaxValue());
  };
  pair<nmm_int32, nmm_int32> Camera1394::getAutoExposureRange()  const{
    return pair<nmm_int32, nmm_int32>(camera->autoExposure()->getMinValue(),camera->autoExposure()->getMaxValue());
  }
  pair<nmm_int32, nmm_int32> Camera1394::getSharpnessRange()     const{
    return pair<nmm_int32, nmm_int32>(camera->sharpness()->getMinValue(),camera->sharpness()->getMaxValue());
  };
  pair<nmm_int32, nmm_int32> Camera1394::getWhiteBalanceRange()  const{
    return pair<nmm_int32, nmm_int32>(camera->whiteBalance()->getMinValue(),camera->whiteBalance()->getMaxValue());
  }
  pair<nmm_int32, nmm_int32> Camera1394::getHueRange()           const{
    return pair<nmm_int32, nmm_int32>(camera->hue()->getMinValue(),camera->hue()->getMaxValue());
  };
  pair<nmm_int32, nmm_int32> Camera1394::getSaturationRange()    const{
    return pair<nmm_int32, nmm_int32>(camera->saturation()->getMinValue(),camera->saturation()->getMaxValue());
  }
  pair<nmm_int32, nmm_int32> Camera1394::getGammaRange()         const{
    return pair<nmm_int32, nmm_int32>(camera->gamma()->getMinValue(),camera->gamma()->getMaxValue());
  };
  pair<nmm_int32, nmm_int32> Camera1394::getShutterRange()       const{
    return pair<nmm_int32, nmm_int32>(camera->shutter()->getMinValue(),camera->shutter()->getMaxValue());
  };
  pair<nmm_int32, nmm_int32> Camera1394::getGainRange()          const{
    return pair<nmm_int32, nmm_int32>(camera->gain()->getMinValue(),camera->gain()->getMaxValue());
  }
  pair<nmm_int32, nmm_int32> Camera1394::getIrisRange()          const{
    return pair<nmm_int32, nmm_int32>(camera->iris()->getMinValue(),camera->iris()->getMaxValue());
  };
  pair<nmm_int32, nmm_int32> Camera1394::getFocusRange()         const{
    return pair<nmm_int32, nmm_int32>(camera->focus()->getMinValue(),camera->focus()->getMaxValue());
  };
  pair<nmm_int32, nmm_int32> Camera1394::getZoomRange()          const{
    return pair<nmm_int32, nmm_int32>(camera->zoom()->getMinValue(),camera->zoom()->getMaxValue());
  };


  nmm_bool Camera1394::hasBrightness()     const{
    cout<< camera->brightness()->hasFeature()<<endl;
    return camera->brightness()->hasFeature();
  };
  nmm_bool Camera1394::hasAutoExposure()   const{
    cout<< camera->autoExposure()->hasFeature()<<endl;
    return camera->autoExposure()->hasFeature();
  };
  nmm_bool Camera1394::hasSharpness()      const{
    cout << camera->sharpness()->hasFeature() << endl;
    return camera->sharpness()->hasFeature();
  };
  nmm_bool Camera1394::hasWhiteBalance()   const{
    cout << camera->whiteBalance()->hasFeature() << endl;
    return camera->whiteBalance()->hasFeature();
  };
  nmm_bool Camera1394::hasHue()            const{
    return camera->hue()->hasFeature();
  };
  nmm_bool Camera1394::hasSaturation()     const{
    return camera->saturation()->hasFeature();
  };
  nmm_bool Camera1394::hasGamma()          const{
    cout << camera->gamma()->hasFeature() << endl;
    return camera->gamma()->hasFeature();
  };
  nmm_bool Camera1394::hasShutter()        const{
    cout << camera->shutter()->hasFeature() << endl;
    return camera->shutter()->hasFeature();
  };
  nmm_bool Camera1394::hasGain()           const{
    cout << camera->gain()->hasFeature() << endl;
    return camera->gain()->hasFeature();
  };
  nmm_bool Camera1394::hasIris()           const{
    cout << camera->iris()->hasFeature() << endl;
    return camera->iris()->hasFeature();
  };
  nmm_bool Camera1394::hasFocus()          const{
    cout << camera->focus()->hasFeature() << endl;
    return camera->focus()->hasFeature();
  };
  nmm_bool Camera1394::hasZoom()           const{
    cout << camera->zoom()->hasFeature() << endl;
    return camera->zoom()->hasFeature();
  };

    


} //end namespace


#include "nmm/nmm_suffix.hpp"
