/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IAVMux.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IAVMux::used() {} 
  
  
  void IAVMux::setSyncMode(const nmm_bool& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > value_value(value, false);
    NMM::Event event("IAVMux::setSyncMode", &value_value);
    if(const_cast<IAVMux*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IAVMux::IAVMux_type = "IAVMux";
  NMM::TConstructor<IAVMux > __IAVMux_prototype("IAVMux","NMM");
  TValueConstructor< IAVMux::Label > __IAVMux_Label_prototype(TypeInfo("IAVMux::Label", "NMM"));

  NMM::OValueStream& operator<<(NMM::OValueStream& ov, const IAVMux::Label& t)
  {
    ov.beginType(TypeInfo("IAVMux::Label", "NMM"));
    nmm_int32 temp = static_cast<nmm_int32>(t);
    ov << temp;
    ov.endType();
    return ov;
  }

  NMM::IValueStream& operator>>(NMM::IValueStream& iv, IAVMux::Label& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("IAVMux::Label", "NMM")) {
      throw NMM::InvalidTypeException(TypeInfo("IAVMux::Label", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    nmm_int32 temp;
    iv >> temp;
    t = static_cast<IAVMux::Label>(temp);
    iv.goUp();
    return iv;
  }



  const char* IAVMux::setSyncMode_event = "IAVMux::setSyncMode";


}


#include "nmm/nmm_suffix.hpp"

