/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IBufferSize.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IBufferSize::used() {} 
  
  
  void IBufferSize::setBufferSize(const nmm_int32& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > value_value(value, false);
    NMM::Event event("IBufferSize::setBufferSize", &value_value);
    if(const_cast<IBufferSize*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_int32 IBufferSize::getBufferSize()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("IBufferSize::getBufferSize");
    event.setReturnValue(&return_value);
    if(const_cast<IBufferSize*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IBufferSize::IBufferSize_type = "IBufferSize";
  NMM::TConstructor<IBufferSize > __IBufferSize_prototype("IBufferSize","NMM");


  const char* IBufferSize::setBufferSize_event = "IBufferSize::setBufferSize";
  const char* IBufferSize::getBufferSize_event = "IBufferSize::getBufferSize";


}


#include "nmm/nmm_suffix.hpp"

