/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IBufferSize_HPP
#define NMM_IBufferSize_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{
 /** Interface for setting/requesting buffer size.
   *
   * The buffer size to be used in a node can be requested or set with this interface. 
   */

  class DLLEXPORT IBufferSize : virtual public NMM::Interface { 
  public:
    IBufferSize() : NMM::Interface(IBufferSize::IBufferSize_type, "NMM") { };

    IBufferSize(NMM::NMMObject& parent) : NMM::Interface (parent, IBufferSize::IBufferSize_type, "NMM") { };

    virtual ~IBufferSize() {};

    static const char* IBufferSize_type;

    virtual  string getInterfaceType() const { return IBufferSize_type; }

    NMM::Interface* copy() const { return new IBufferSize(*this); }
    static void used(); 
    static const char* setBufferSize_event;
    static const char* getBufferSize_event;
    /** Sets buffer size to be used.
     *  
     *  @param value Buffer size to be used.
     */

    virtual void setBufferSize(const nmm_int32& value)
      throw(const NMM::SerializedException&);
    /** Requests buffer size that the node is currently using.
     *  
     *  @return Buffer size used by node.
     */

    virtual nmm_int32 getBufferSize()
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IBufferSize> IBufferSize_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
