/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IBufferSizeImpl_HPP
#define NMM_IBufferSizeImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IBufferSize.hpp"


#include "visibility.hpp"

namespace NMM
{
 /** Interface for setting/requesting buffer size.
   *
   * The buffer size to be used in a node can be requested or set with this interface. 
   */

  class DLLEXPORT IBufferSizeImpl  : public virtual NMM::NMMObject { 
  public:
    IBufferSizeImpl() { addInterface(new IBufferSize(*this)); registerEvents(); };

    virtual ~IBufferSizeImpl() {};

  protected:
    /** Sets buffer size to be used.
     *  
     *  @param value Buffer size to be used.
     */

    virtual void setBufferSize(const nmm_int32& value) = 0;
    /** Requests buffer size that the node is currently using.
     *  
     *  @return Buffer size used by node.
     */

    virtual nmm_int32 getBufferSize() = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
