/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IByteStream.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IByteStream::used() {} 
  
  
  void IByteStream::sendBuffer2(const TransferBuffer2& buffer)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<TransferBuffer2 > buffer_value(buffer, false);
    NMM::Event event("IByteStream::sendBuffer2", &buffer_value);
    if(const_cast<IByteStream*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_int32 IByteStream::getSize()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("IByteStream::getSize");
    event.setReturnValue(&return_value);
    if(const_cast<IByteStream*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 IByteStream::getMaxSize()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("IByteStream::getMaxSize");
    event.setReturnValue(&return_value);
    if(const_cast<IByteStream*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IByteStream::setMaxSize(const nmm_int32& size)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > size_value(size, false);
    NMM::Event event("IByteStream::setMaxSize", &size_value);
    if(const_cast<IByteStream*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IByteStream::IByteStream_type = "IByteStream";
  NMM::TConstructor<IByteStream > __IByteStream_prototype("IByteStream","NMM");


  const char* IByteStream::sendBuffer2_event = "IByteStream::sendBuffer2";
  const char* IByteStream::getSize_event = "IByteStream::getSize";
  const char* IByteStream::getMaxSize_event = "IByteStream::getMaxSize";
  const char* IByteStream::setMaxSize_event = "IByteStream::setMaxSize";


}


#include "nmm/nmm_suffix.hpp"

