/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IByteStream_HPP
#define NMM_IByteStream_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/messaging/TransferBuffer2.hpp"


#include "visibility.hpp"

namespace NMM
{

  class DLLEXPORT IByteStream : virtual public NMM::Interface { 
  public:
    IByteStream() : NMM::Interface(IByteStream::IByteStream_type, "NMM") { };

    IByteStream(NMM::NMMObject& parent) : NMM::Interface (parent, IByteStream::IByteStream_type, "NMM") { };

    virtual ~IByteStream() {};

    static const char* IByteStream_type;

    virtual  string getInterfaceType() const { return IByteStream_type; }

    NMM::Interface* copy() const { return new IByteStream(*this); }
    static void used(); 
    static const char* sendBuffer2_event;
    static const char* getSize_event;
    static const char* getMaxSize_event;
    static const char* setMaxSize_event;

    virtual void sendBuffer2(const TransferBuffer2& buffer)
      throw(const NMM::SerializedException&);

    virtual nmm_int32 getSize()
      throw(const NMM::SerializedException&);

    virtual nmm_int32 getMaxSize()
      throw(const NMM::SerializedException&);

    virtual void setMaxSize(const nmm_int32& size)
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IByteStream> IByteStream_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
