/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IEncode.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IEncode::used() {} 
  
  
  void IEncode::setQuality(const nmm_float& quality)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_float > quality_value(quality, false);
    NMM::Event event("IEncode::setQuality", &quality_value);
    if(const_cast<IEncode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IEncode::setBitrate(const nmm_int32& bitrate)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > bitrate_value(bitrate, false);
    NMM::Event event("IEncode::setBitrate", &bitrate_value);
    if(const_cast<IEncode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_float IEncode::getQuality()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_float > return_value;
    NMM::Event event("IEncode::getQuality");
    event.setReturnValue(&return_value);
    if(const_cast<IEncode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 IEncode::getBitrate()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("IEncode::getBitrate");
    event.setReturnValue(&return_value);
    if(const_cast<IEncode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IEncode::IEncode_type = "IEncode";
  NMM::TConstructor<IEncode > __IEncode_prototype("IEncode","NMM");


  const char* IEncode::setQuality_event = "IEncode::setQuality";
  const char* IEncode::setBitrate_event = "IEncode::setBitrate";
  const char* IEncode::getQuality_event = "IEncode::getQuality";
  const char* IEncode::getBitrate_event = "IEncode::getBitrate";


}


#include "nmm/nmm_suffix.hpp"

