/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IEncode_HPP
#define NMM_IEncode_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{
 /** Interface for setting/requesting quality or bitrate of encoding.
   *
   * Interface sets quality or bitrate of encoding in a encoding node.
   */

  class DLLEXPORT IEncode : virtual public NMM::Interface { 
  public:
    IEncode() : NMM::Interface(IEncode::IEncode_type, "NMM") { };

    IEncode(NMM::NMMObject& parent) : NMM::Interface (parent, IEncode::IEncode_type, "NMM") { };

    virtual ~IEncode() {};

    static const char* IEncode_type;

    virtual  string getInterfaceType() const { return IEncode_type; }

    NMM::Interface* copy() const { return new IEncode(*this); }
    static void used(); 
    static const char* setQuality_event;
    static const char* setBitrate_event;
    static const char* getQuality_event;
    static const char* getBitrate_event;
    /** Sets quality of encoding.
     *  
     *  @param quality Quality with which should be encoded.
     */

    virtual void setQuality(const nmm_float& quality)
      throw(const NMM::SerializedException&);
    /** Sets bitrate of encoding.
     *  
     *  @param bitrate Bitrate with which should be encoded.
     */

    virtual void setBitrate(const nmm_int32& bitrate)
      throw(const NMM::SerializedException&);
    /** Returns quality of encoding.
     *  
     * @return Quality with which should be encoded.
     *
     */

    virtual nmm_float getQuality()
      throw(const NMM::SerializedException&);
    /** Returns bitrate of encoding.
     *  
     *  @return Bitrate with which should be encoded.
     */

    virtual nmm_int32 getBitrate()
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IEncode> IEncode_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
