/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IEncodeImpl_HPP
#define NMM_IEncodeImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IEncode.hpp"


#include "visibility.hpp"

namespace NMM
{
 /** Interface for setting/requesting quality or bitrate of encoding.
   *
   * Interface sets quality or bitrate of encoding in a encoding node.
   */

  class DLLEXPORT IEncodeImpl  : public virtual NMM::NMMObject { 
  public:
    IEncodeImpl() { addInterface(new IEncode(*this)); registerEvents(); };

    virtual ~IEncodeImpl() {};

  protected:
    /** Sets quality of encoding.
     *  
     *  @param quality Quality with which should be encoded.
     */

    virtual void setQuality(const nmm_float& quality) = 0;
    /** Sets bitrate of encoding.
     *  
     *  @param bitrate Bitrate with which should be encoded.
     */

    virtual void setBitrate(const nmm_int32& bitrate) = 0;
    /** Returns quality of encoding.
     *  
     * @return Quality with which should be encoded.
     *
     */

    virtual nmm_float getQuality() = 0;
    /** Returns bitrate of encoding.
     *  
     *  @return Bitrate with which should be encoded.
     */

    virtual nmm_int32 getBitrate() = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
