/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IEncodeStats.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IEncodeStats::used() {} 
  
  
  void IEncodeStats::sendFrameCount(const nmm_bool& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > value_value(value, false);
    NMM::Event event("IEncodeStats::sendFrameCount", &value_value);
    if(const_cast<IEncodeStats*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IEncodeStats::IEncodeStats_type = "IEncodeStats";
  NMM::TConstructor<IEncodeStats > __IEncodeStats_prototype("IEncodeStats","NMM");


  const char* IEncodeStats::sendFrameCount_event = "IEncodeStats::sendFrameCount";
  void IEncodeStatsListener::used() {} 
  
  
  void IEncodeStatsListener::setFrameCount(const nmm_uint32& count, const nmm_float& fps) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > count_value(count, false);
    NMM::TInValue<nmm_float > fps_value(fps, false);
    NMM::Event event("IEncodeStatsListener::setFrameCount", &count_value, &fps_value);
    if(const_cast<IEncodeStatsListener*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IEncodeStatsListener::IEncodeStatsListener_type = "IEncodeStatsListener";
  NMM::TConstructor<IEncodeStatsListener > __IEncodeStatsListener_prototype("IEncodeStatsListener","NMM");


  const char* IEncodeStatsListener::setFrameCount_event = "IEncodeStatsListener::setFrameCount";
  NMM::Event* IEncodeStatsListener::create_setFrameCount(const nmm_uint32& count, const nmm_float& fps)
  {
    NMM::TInValue<nmm_uint32 >* count_value = new NMM::TInValue<nmm_uint32 >();
    count_value->setValue(count);
    NMM::TInValue<nmm_float >* fps_value = new NMM::TInValue<nmm_float >();
    fps_value->setValue(fps);

    return new NMM::Event("IEncodeStatsListener::setFrameCount", count_value, fps_value);
  }



}


#include "nmm/nmm_suffix.hpp"

