/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IEncodeStats_HPP
#define NMM_IEncodeStats_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{
  /** Interface for encoding statistics.
   *
   * Interface for encoders that provide statistics by sending events
   * to an IEncodeStatsListener.
   */

  class DLLEXPORT IEncodeStats : virtual public NMM::Interface { 
  public:
    IEncodeStats() : NMM::Interface(IEncodeStats::IEncodeStats_type, "NMM") { };

    IEncodeStats(NMM::NMMObject& parent) : NMM::Interface (parent, IEncodeStats::IEncodeStats_type, "NMM") { };

    virtual ~IEncodeStats() {};

    static const char* IEncodeStats_type;

    virtual  string getInterfaceType() const { return IEncodeStats_type; }

    NMM::Interface* copy() const { return new IEncodeStats(*this); }
    static void used(); 
    static const char* sendFrameCount_event;
    /** Toggles sending frame count events.
     * 
     * Turns sending frame count events on or off. A frame count event
     * contains the number of frames encoded so far and the encoding
     * framerate.
     *
     * @param value True if frame count should be sent, false otherwise.
     */

    virtual void sendFrameCount(const nmm_bool& value)
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IEncodeStats> IEncodeStats_var;


  /** Interface for handling/receiving encoding statistics.
   *
   * Used for handling instream events by an encoder that implements
   * IEncodeStats.
   */

  class DLLEXPORT IEncodeStatsListener : virtual public NMM::Interface { 
  public:
    IEncodeStatsListener() : NMM::Interface(IEncodeStatsListener::IEncodeStatsListener_type, "NMM") { };

    IEncodeStatsListener(NMM::NMMObject& parent) : NMM::Interface (parent, IEncodeStatsListener::IEncodeStatsListener_type, "NMM") { };

    virtual ~IEncodeStatsListener() {};

    static const char* IEncodeStatsListener_type;

    virtual  string getInterfaceType() const { return IEncodeStatsListener_type; }

    NMM::Interface* copy() const { return new IEncodeStatsListener(*this); }
    static void used(); 
    static const char* setFrameCount_event;
    static NMM::Event* create_setFrameCount(const nmm_uint32& count, const nmm_float& fps);
  /* instream (protected) and non-instream (public) */        /** Propagates encoded frames and current framerate.
     * 
     * Statistical event sent by the node, e.g. encoder.
     *
     * @param count number of frames encoded so far
     * @param fps current encoding framerate in frames per second
     */

    virtual void setFrameCount(const nmm_uint32& count, const nmm_float& fps) /* in-stream */ 
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IEncodeStatsListener> IEncodeStatsListener_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
