/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IEncodeStatsImpl_HPP
#define NMM_IEncodeStatsImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IEncodeStats.hpp"


#include "visibility.hpp"

namespace NMM
{
  /** Interface for encoding statistics.
   *
   * Interface for encoders that provide statistics by sending events
   * to an IEncodeStatsListener.
   */

  class DLLEXPORT IEncodeStatsImpl  : public virtual NMM::NMMObject { 
  public:
    IEncodeStatsImpl() { addInterface(new IEncodeStats(*this)); registerEvents(); };

    virtual ~IEncodeStatsImpl() {};

  protected:
    /** Toggles sending frame count events.
     * 
     * Turns sending frame count events on or off. A frame count event
     * contains the number of frames encoded so far and the encoding
     * framerate.
     *
     * @param value True if frame count should be sent, false otherwise.
     */

    virtual void sendFrameCount(const nmm_bool& value) = 0;


  private:
    void registerEvents();
  };
  /** Interface for handling/receiving encoding statistics.
   *
   * Used for handling instream events by an encoder that implements
   * IEncodeStats.
   */

  class DLLEXPORT IEncodeStatsListenerImpl  : public virtual NMM::NMMObject { 
  public:
    IEncodeStatsListenerImpl() { addInterface(new IEncodeStatsListener(*this)); registerEvents(); };

    virtual ~IEncodeStatsListenerImpl() {};

  protected:
    /** Propagates encoded frames and current framerate.
     * 
     * Statistical event sent by the node, e.g. encoder.
     *
     * @param count number of frames encoded so far
     * @param fps current encoding framerate in frames per second
     */

    virtual void setFrameCount(const nmm_uint32& count, const nmm_float& fps) /* in-stream */  = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
