/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IFileHandler_HPP
#define NMM_IFileHandler_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{
 /** Interface for setting filename in file reading nodes.
   */

  class DLLEXPORT IFileHandler : virtual public NMM::Interface { 
  public:
    IFileHandler() : NMM::Interface(IFileHandler::IFileHandler_type, "NMM") { };

    IFileHandler(NMM::NMMObject& parent) : NMM::Interface (parent, IFileHandler::IFileHandler_type, "NMM") { };

    virtual ~IFileHandler() {};

    static const char* IFileHandler_type;

    virtual  string getInterfaceType() const { return IFileHandler_type; }

    NMM::Interface* copy() const { return new IFileHandler(*this); }
    static void used(); 
    static const char* setFilename_event;
    /** Sets name of file to be opened.
     *
     * @param name Name of the file to be read.
     */

    virtual void setFilename(const string& name)
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IFileHandler> IFileHandler_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
