/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IProgress.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IProgress::used() {} 
  
  
  void IProgress::sendProgressInformation(const nmm_bool& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > value_value(value, false);
    NMM::Event event("IProgress::sendProgressInformation", &value_value);
    if(const_cast<IProgress*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IProgress::setProgressInterval(const nmm_int32& buffers)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > buffers_value(buffers, false);
    NMM::Event event("IProgress::setProgressInterval", &buffers_value);
    if(const_cast<IProgress*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IProgress::IProgress_type = "IProgress";
  NMM::TConstructor<IProgress > __IProgress_prototype("IProgress","NMM");


  const char* IProgress::sendProgressInformation_event = "IProgress::sendProgressInformation";
  const char* IProgress::setProgressInterval_event = "IProgress::setProgressInterval";


}


#include "nmm/nmm_suffix.hpp"

