/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ISeekReceiver.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ISeekReceiver::used() {} 
  
  
  Result ISeekReceiver::seeked(const nmm_uint64& pos) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_uint64 > pos_value(pos, false);
    NMM::Event event("ISeekReceiver::seeked", &pos_value);
    event.setReturnValue(&return_value);
    if(const_cast<ISeekReceiver*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ISeekReceiver::ISeekReceiver_type = "ISeekReceiver";
  NMM::TConstructor<ISeekReceiver > __ISeekReceiver_prototype("ISeekReceiver","NMM");


  const char* ISeekReceiver::seeked_event = "ISeekReceiver::seeked";
  NMM::Event* ISeekReceiver::create_seeked(const nmm_uint64& pos)
  {
    NMM::TInValue<nmm_uint64 >* pos_value = new NMM::TInValue<nmm_uint64 >();
    pos_value->setValue(pos);

    return new NMM::Event("ISeekReceiver::seeked", pos_value);
  }



}


#include "nmm/nmm_suffix.hpp"

