/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ISeekReceiver_HPP
#define NMM_ISeekReceiver_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{
  /** Interface for creating/handling the notification of a seeking operation.
   */

  class DLLEXPORT ISeekReceiver : virtual public NMM::Interface { 
  public:
    ISeekReceiver() : NMM::Interface(ISeekReceiver::ISeekReceiver_type, "NMM") { };

    ISeekReceiver(NMM::NMMObject& parent) : NMM::Interface (parent, ISeekReceiver::ISeekReceiver_type, "NMM") { };

    virtual ~ISeekReceiver() {};

    static const char* ISeekReceiver_type;

    virtual  string getInterfaceType() const { return ISeekReceiver_type; }

    NMM::Interface* copy() const { return new ISeekReceiver(*this); }
    static void used(); 
    static const char* seeked_event;
    static NMM::Event* create_seeked(const nmm_uint64& pos);
  /* instream (protected) and non-instream (public) */        /** Event is sent whenever sending node just did a seeking operation.
     *
     * This interface method can be used to propagate the information
     * about the new file position (in bytes) after a seeking
     * operation.
     *
     * @param pos The new position in the current file (in bytes)
     */

    virtual Result seeked(const nmm_uint64& pos) /* in-stream */ 
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ISeekReceiver> ISeekReceiver_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
