/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ISeekReceiverImpl_HPP
#define NMM_ISeekReceiverImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "ISeekReceiver.hpp"


#include "visibility.hpp"

namespace NMM
{
  /** Interface for creating/handling the notification of a seeking operation.
   */

  class DLLEXPORT ISeekReceiverImpl  : public virtual NMM::NMMObject { 
  public:
    ISeekReceiverImpl() { addInterface(new ISeekReceiver(*this)); registerEvents(); };

    virtual ~ISeekReceiverImpl() {};

  protected:
    /** Event is sent whenever sending node just did a seeking operation.
     *
     * This interface method can be used to propagate the information
     * about the new file position (in bytes) after a seeking
     * operation.
     *
     * @param pos The new position in the current file (in bytes)
     */

    virtual Result seeked(const nmm_uint64& pos) /* in-stream */  = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
