/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ISetBufferLabel.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ISetBufferLabel::used() {} 
  
  
  Result ISetBufferLabel::setBufferLabel(const string& stream, const nmm_int32& label) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > stream_value(stream, false);
    NMM::TInValue<nmm_int32 > label_value(label, false);
    NMM::Event event("ISetBufferLabel::setBufferLabel", &stream_value, &label_value);
    event.setReturnValue(&return_value);
    if(const_cast<ISetBufferLabel*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ISetBufferLabel::ISetBufferLabel_type = "ISetBufferLabel";
  NMM::TConstructor<ISetBufferLabel > __ISetBufferLabel_prototype("ISetBufferLabel","NMM");


  const char* ISetBufferLabel::setBufferLabel_event = "ISetBufferLabel::setBufferLabel";
  NMM::Event* ISetBufferLabel::create_setBufferLabel(const string& stream, const nmm_int32& label)
  {
    NMM::TInValue<string >* stream_value = new NMM::TInValue<string >();
    stream_value->setValue(stream);
    NMM::TInValue<nmm_int32 >* label_value = new NMM::TInValue<nmm_int32 >();
    label_value->setValue(label);

    return new NMM::Event("ISetBufferLabel::setBufferLabel", stream_value, label_value);
  }



}


#include "nmm/nmm_suffix.hpp"

