/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IStillFrame.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IStillFrame::used() {} 
  
  
  Result IStillFrame::setStillFrame(const nmm_bool& value) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_bool > value_value(value, false);
    NMM::Event event("IStillFrame::setStillFrame", &value_value);
    event.setReturnValue(&return_value);
    if(const_cast<IStillFrame*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IStillFrame::IStillFrame_type = "IStillFrame";
  NMM::TConstructor<IStillFrame > __IStillFrame_prototype("IStillFrame","NMM");


  const char* IStillFrame::setStillFrame_event = "IStillFrame::setStillFrame";
  NMM::Event* IStillFrame::create_setStillFrame(const nmm_bool& value)
  {
    NMM::TInValue<nmm_bool >* value_value = new NMM::TInValue<nmm_bool >();
    value_value->setValue(value);

    return new NMM::Event("IStillFrame::setStillFrame", value_value);
  }



}


#include "nmm/nmm_suffix.hpp"

