/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IStreamJump.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IStreamJump::used() {} 
  
  
  Result IStreamJump::fastForward(const nmm_int32& value) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > value_value(value, false);
    NMM::Event event("IStreamJump::fastForward", &value_value);
    event.setReturnValue(&return_value);
    if(const_cast<IStreamJump*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IStreamJump::streamJump(const nmm_int32& value) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > value_value(value, false);
    NMM::Event event("IStreamJump::streamJump", &value_value);
    event.setReturnValue(&return_value);
    if(const_cast<IStreamJump*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IStreamJump::sendIFramesOnly(const nmm_int32& n) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > n_value(n, false);
    NMM::Event event("IStreamJump::sendIFramesOnly", &n_value);
    event.setReturnValue(&return_value);
    if(const_cast<IStreamJump*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IStreamJump::IStreamJump_type = "IStreamJump";
  NMM::TConstructor<IStreamJump > __IStreamJump_prototype("IStreamJump","NMM");


  const char* IStreamJump::fastForward_event = "IStreamJump::fastForward";
  NMM::Event* IStreamJump::create_fastForward(const nmm_int32& value)
  {
    NMM::TInValue<nmm_int32 >* value_value = new NMM::TInValue<nmm_int32 >();
    value_value->setValue(value);

    return new NMM::Event("IStreamJump::fastForward", value_value);
  }

  const char* IStreamJump::streamJump_event = "IStreamJump::streamJump";
  NMM::Event* IStreamJump::create_streamJump(const nmm_int32& value)
  {
    NMM::TInValue<nmm_int32 >* value_value = new NMM::TInValue<nmm_int32 >();
    value_value->setValue(value);

    return new NMM::Event("IStreamJump::streamJump", value_value);
  }

  const char* IStreamJump::sendIFramesOnly_event = "IStreamJump::sendIFramesOnly";
  NMM::Event* IStreamJump::create_sendIFramesOnly(const nmm_int32& n)
  {
    NMM::TInValue<nmm_int32 >* n_value = new NMM::TInValue<nmm_int32 >();
    n_value->setValue(n);

    return new NMM::Event("IStreamJump::sendIFramesOnly", n_value);
  }



}


#include "nmm/nmm_suffix.hpp"

