/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IStreamJump_HPP
#define NMM_IStreamJump_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{

  class DLLEXPORT IStreamJump : virtual public NMM::Interface { 
  public:
    IStreamJump() : NMM::Interface(IStreamJump::IStreamJump_type, "NMM") { };

    IStreamJump(NMM::NMMObject& parent) : NMM::Interface (parent, IStreamJump::IStreamJump_type, "NMM") { };

    virtual ~IStreamJump() {};

    static const char* IStreamJump_type;

    virtual  string getInterfaceType() const { return IStreamJump_type; }

    NMM::Interface* copy() const { return new IStreamJump(*this); }
    static void used(); 
    static const char* fastForward_event;
    static NMM::Event* create_fastForward(const nmm_int32& value);
    static const char* streamJump_event;
    static NMM::Event* create_streamJump(const nmm_int32& value);
    static const char* sendIFramesOnly_event;
    static NMM::Event* create_sendIFramesOnly(const nmm_int32& n);
  /* instream (protected) and non-instream (public) */    
    virtual Result fastForward(const nmm_int32& value) /* in-stream */ 
      throw(const NMM::SerializedException&);
    
    virtual Result streamJump(const nmm_int32& value) /* in-stream */ 
      throw(const NMM::SerializedException&);
        /**
     * Send only n I-Frames.
     * Tells the decoder to send only the n decoded I-frames
     * (usefull for fast-forward)
     * n = 0 means normal buffer processing
     */

    virtual Result sendIFramesOnly(const nmm_int32& n) /* in-stream */ 
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IStreamJump> IStreamJump_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
