/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IStreamJumpImpl_HPP
#define NMM_IStreamJumpImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IStreamJump.hpp"


#include "visibility.hpp"

namespace NMM
{

  class DLLEXPORT IStreamJumpImpl  : public virtual NMM::NMMObject { 
  public:
    IStreamJumpImpl() { addInterface(new IStreamJump(*this)); registerEvents(); };

    virtual ~IStreamJumpImpl() {};

  protected:

    virtual Result fastForward(const nmm_int32& value) /* in-stream */  = 0;

    virtual Result streamJump(const nmm_int32& value) /* in-stream */  = 0;
    /**
     * Send only n I-Frames.
     * Tells the decoder to send only the n decoded I-frames
     * (usefull for fast-forward)
     * n = 0 means normal buffer processing
     */

    virtual Result sendIFramesOnly(const nmm_int32& n) /* in-stream */  = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
