/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ITagInfo.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ITagInfo::used() {} 
  
  
  Result ITagInfo::TagInfo(const string& title, const string& album, const string& artist, const string& year, const string& genre) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > title_value(title, false);
    NMM::TInValue<string > album_value(album, false);
    NMM::TInValue<string > artist_value(artist, false);
    NMM::TInValue<string > year_value(year, false);
    NMM::TInValue<string > genre_value(genre, false);
    NMM::Event event("ITagInfo::TagInfo", &title_value, &album_value, &artist_value, &year_value, &genre_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITagInfo*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ITagInfo::ITagInfo_type = "ITagInfo";
  NMM::TConstructor<ITagInfo > __ITagInfo_prototype("ITagInfo","NMM");


  const char* ITagInfo::TagInfo_event = "ITagInfo::TagInfo";
  NMM::Event* ITagInfo::create_TagInfo(const string& title, const string& album, const string& artist, const string& year, const string& genre)
  {
    NMM::TInValue<string >* title_value = new NMM::TInValue<string >();
    title_value->setValue(title);
    NMM::TInValue<string >* album_value = new NMM::TInValue<string >();
    album_value->setValue(album);
    NMM::TInValue<string >* artist_value = new NMM::TInValue<string >();
    artist_value->setValue(artist);
    NMM::TInValue<string >* year_value = new NMM::TInValue<string >();
    year_value->setValue(year);
    NMM::TInValue<string >* genre_value = new NMM::TInValue<string >();
    genre_value->setValue(genre);

    return new NMM::Event("ITagInfo::TagInfo", title_value, album_value, artist_value, year_value, genre_value);
  }



}


#include "nmm/nmm_suffix.hpp"

