/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ITagInfo_HPP
#define NMM_ITagInfo_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{
  /** Interface for creating/receiving instream events about meta data of current multimedia file.
   */

  class DLLEXPORT ITagInfo : virtual public NMM::Interface { 
  public:
    ITagInfo() : NMM::Interface(ITagInfo::ITagInfo_type, "NMM") { };

    ITagInfo(NMM::NMMObject& parent) : NMM::Interface (parent, ITagInfo::ITagInfo_type, "NMM") { };

    virtual ~ITagInfo() {};

    static const char* ITagInfo_type;

    virtual  string getInterfaceType() const { return ITagInfo_type; }

    NMM::Interface* copy() const { return new ITagInfo(*this); }
    static void used(); 
    static const char* TagInfo_event;
    static NMM::Event* create_TagInfo(const string& title, const string& album, const string& artist, const string& year, const string& genre);
  /* instream (protected) and non-instream (public) */        /** This interface method is used to handle meta data of current multimedia file.
     * 
     * @param title Title 
     * @param album Album title
     * @param artist Artists name
     * @param year Year of production 
     * @param genre Genre of the multimedia file
     */

    virtual Result TagInfo(const string& title, const string& album, const string& artist, const string& year, const string& genre) /* in-stream */ 
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ITagInfo> ITagInfo_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
