/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ITagInfoImpl_HPP
#define NMM_ITagInfoImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "ITagInfo.hpp"


#include "visibility.hpp"

namespace NMM
{
  /** Interface for creating/receiving instream events about meta data of current multimedia file.
   */

  class DLLEXPORT ITagInfoImpl  : public virtual NMM::NMMObject { 
  public:
    ITagInfoImpl() { addInterface(new ITagInfo(*this)); registerEvents(); };

    virtual ~ITagInfoImpl() {};

  protected:
    /** This interface method is used to handle meta data of current multimedia file.
     * 
     * @param title Title 
     * @param album Album title
     * @param artist Artists name
     * @param year Year of production 
     * @param genre Genre of the multimedia file
     */

    virtual Result TagInfo(const string& title, const string& album, const string& artist, const string& year, const string& genre) /* in-stream */  = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
