/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ITrackSizeImpl_HPP
#define NMM_ITrackSizeImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "ITrackSize.hpp"


#include "visibility.hpp"

namespace NMM
{
  /** Interface for creating/handling instream events representing the size of the current track.
   */

  class DLLEXPORT ITrackSizeImpl  : public virtual NMM::NMMObject { 
  public:
    ITrackSizeImpl() { addInterface(new ITrackSize(*this)); registerEvents(); };

    virtual ~ITrackSizeImpl() {};

  protected:
    /** Event which contains trackSize of current track.
     *
     * This interface method is used to handle the size (in bytes) of
     * a track. 
     *
     * @param bytes the size of the current track in bytes
     */

    virtual Result trackSize(const nmm_uint64& bytes) /* in-stream */  = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
