/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "INet.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/TSAddress.hpp"

namespace NMM
{
  void INet::used() {} 
  
  
  void INet::setTransportStrategy(const TSAddress* address) 
      throw(const NMM::SerializedException&, const BindingException&)
  {
    NMM::TInValue<TSAddress* > address_value (const_cast<TSAddress* >(address), false);
    NMM::Event event("INet::setTransportStrategy", &address_value);
    if(const_cast<INet*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(BindingException* ex = dynamic_cast<BindingException*>(event.getException())) {
          throw BindingException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  ITransportStrategy* INet::getTransportStrategy() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ITransportStrategy* > return_value;
    NMM::Event event("INet::getTransportStrategy");
    event.setReturnValue(&return_value);
    if(const_cast<INet*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void INet::setRTPStrategy(const string& address, const nmm_int32& rtp_port, const nmm_int32& rtcp_port) /* limited(INode::CONSTRUCTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<string > address_value(address, false);
    NMM::TInValue<nmm_int32 > rtp_port_value(rtp_port, false);
    NMM::TInValue<nmm_int32 > rtcp_port_value(rtcp_port, false);
    NMM::Event event("INet::setRTPStrategy", &address_value, &rtp_port_value, &rtcp_port_value);
    if(const_cast<INet*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void INet::setPayloadType(const nmm_int32& type)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > type_value(type, false);
    NMM::Event event("INet::setPayloadType", &type_value);
    if(const_cast<INet*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_int32 INet::getPayloadType() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("INet::getPayloadType");
    event.setReturnValue(&return_value);
    if(const_cast<INet*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* INet::INet_type = "INet";
  NMM::TConstructor<INet > __INet_prototype("INet","NMM");


  const char* INet::setTransportStrategy_event = "INet::setTransportStrategy";
  const char* INet::getTransportStrategy_event = "INet::getTransportStrategy";
  const char* INet::setRTPStrategy_event = "INet::setRTPStrategy";
  const char* INet::setPayloadType_event = "INet::setPayloadType";
  const char* INet::getPayloadType_event = "INet::getPayloadType";


}


#include "nmm/nmm_suffix.hpp"

