/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_INetImpl_HPP
#define NMM_INetImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "INet.hpp"

#include "nmm/multimedia/INodeImpl.hpp"
#include "nmm/comm/ITransportStrategyImpl.hpp"

#include "visibility.hpp"

namespace NMM
{

  class DLLEXPORT INetImpl  : public virtual NMM::NMMObject { 
  public:
    INetImpl() { addInterface(new INet(*this)); registerEvents(); };

    virtual ~INetImpl() {};

  protected:
    /**
     *  @brief TSAddress for connecting
     *  @param tsaddress address to connect
     *
     *  This method sets the @a tsaddress for the NetSinkNode. The type of 
     *  TSAddress (eg. UDPAddress, TCPAddress or RTPAddress) defines the type 
     *  of protocol to use (eg. UDP, TCP or RTP).
     */

    virtual void setTransportStrategy(const TSAddress* address) 
      throw(const BindingException&) = 0;
// Get pointer ot ITransportStrategy

    virtual ITransportStrategy* getTransportStrategy() const = 0;

    virtual void setRTPStrategy(const string& address, const nmm_int32& rtp_port, const nmm_int32& rtcp_port) /* limited(INode::CONSTRUCTED) */ = 0;

    virtual void setPayloadType(const nmm_int32& type) = 0;

    virtual nmm_int32 getPayloadType() const = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
