/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IRTPParameters.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IRTPParameters::used() {} 
  
  
  string IRTPParameters::getRTPSinkMediaType() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("IRTPParameters::getRTPSinkMediaType");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPParameters*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uchar IRTPParameters::getRTPSinkPayloadType() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uchar > return_value;
    NMM::Event event("IRTPParameters::getRTPSinkPayloadType");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPParameters*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string IRTPParameters::getRTPSinkMapLine() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("IRTPParameters::getRTPSinkMapLine");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPParameters*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string IRTPParameters::getRTPSinkAuxSDPLine() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("IRTPParameters::getRTPSinkAuxSDPLine");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPParameters*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint16 IRTPParameters::getRTPSeqNum() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint16 > return_value;
    NMM::Event event("IRTPParameters::getRTPSeqNum");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPParameters*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 IRTPParameters::getRTPTimestamp() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("IRTPParameters::getRTPTimestamp");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPParameters*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 IRTPParameters::getRTPPort() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("IRTPParameters::getRTPPort");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPParameters*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 IRTPParameters::getRTCPPort() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("IRTPParameters::getRTCPPort");
    event.setReturnValue(&return_value);
    if(const_cast<IRTPParameters*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IRTPParameters::IRTPParameters_type = "IRTPParameters";
  NMM::TConstructor<IRTPParameters > __IRTPParameters_prototype("IRTPParameters","NMM");


  const char* IRTPParameters::getRTPSinkMediaType_event = "IRTPParameters::getRTPSinkMediaType";
  const char* IRTPParameters::getRTPSinkPayloadType_event = "IRTPParameters::getRTPSinkPayloadType";
  const char* IRTPParameters::getRTPSinkMapLine_event = "IRTPParameters::getRTPSinkMapLine";
  const char* IRTPParameters::getRTPSinkAuxSDPLine_event = "IRTPParameters::getRTPSinkAuxSDPLine";
  const char* IRTPParameters::getRTPSeqNum_event = "IRTPParameters::getRTPSeqNum";
  const char* IRTPParameters::getRTPTimestamp_event = "IRTPParameters::getRTPTimestamp";
  const char* IRTPParameters::getRTPPort_event = "IRTPParameters::getRTPPort";
  const char* IRTPParameters::getRTCPPort_event = "IRTPParameters::getRTCPPort";


}


#include "nmm/nmm_suffix.hpp"

