/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IRTPParameters_HPP
#define NMM_IRTPParameters_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{

  class DLLEXPORT IRTPParameters : virtual public NMM::Interface { 
  public:
    IRTPParameters() : NMM::Interface(IRTPParameters::IRTPParameters_type, "NMM") { };

    IRTPParameters(NMM::NMMObject& parent) : NMM::Interface (parent, IRTPParameters::IRTPParameters_type, "NMM") { };

    virtual ~IRTPParameters() {};

    static const char* IRTPParameters_type;

    virtual  string getInterfaceType() const { return IRTPParameters_type; }

    NMM::Interface* copy() const { return new IRTPParameters(*this); }
    static void used(); 
    static const char* getRTPSinkMediaType_event;
    static const char* getRTPSinkPayloadType_event;
    static const char* getRTPSinkMapLine_event;
    static const char* getRTPSinkAuxSDPLine_event;
    static const char* getRTPSeqNum_event;
    static const char* getRTPTimestamp_event;
    static const char* getRTPPort_event;
    static const char* getRTCPPort_event;

    virtual string getRTPSinkMediaType() const
      throw(const NMM::SerializedException&);

    virtual nmm_uchar getRTPSinkPayloadType() const
      throw(const NMM::SerializedException&);

    virtual string getRTPSinkMapLine() const
      throw(const NMM::SerializedException&);

    virtual string getRTPSinkAuxSDPLine() const
      throw(const NMM::SerializedException&);

    virtual nmm_uint16 getRTPSeqNum() const
      throw(const NMM::SerializedException&);

    virtual nmm_uint32 getRTPTimestamp() const
      throw(const NMM::SerializedException&);

    virtual nmm_uint32 getRTPPort() const
      throw(const NMM::SerializedException&);

    virtual nmm_uint32 getRTCPPort() const
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IRTPParameters> IRTPParameters_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
