/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#ifndef NMM_NETSINKNODE_HPP
#define NMM_NETSINKNODE_HPP NMM_NETSINKNODE_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/multimedia/GenericSinkNode.hpp"

#include "INetImpl.hpp"
#include "IRTPParametersImpl.hpp"

#include "nmm/utils/time/Clock.hpp"

#include <list>
using std::list;

namespace NMM {
  
  class TransportStrategy;
  class ITSOneWayBufferReceiver;
  
  class NetSinkNode :
    public GenericSinkNode,
    public INetImpl,
    public IRTPParametersImpl
  {
  public: 
    NetSinkNode(const char* = "NetSinkNode");
    virtual ~NetSinkNode();
    
    Result enableInputStream();
  
  protected:
    virtual Message* processBuffer(Buffer*);
    
    virtual Result doInit();
    virtual Result doInitOutput();
    virtual Result doDeinitOutput();
    /**
     *  @name INet Interface
     */
    //@{

    /**
     *  @brief TSAddress for connecting
     *  @param tsaddress address to connect
     *  
     *  This method sets the @a tsaddress for the NetSinkNode. The type of 
     *  TSAddress (eg. UDPAddress, TCPAddress or RTPAddress) defines the type 
     *  of protocol to use (eg. UDP, TCP or RTP).
     */
    virtual void setTransportStrategy(const TSAddress*) throw(const BindingException&);
    
    virtual ITransportStrategy* getTransportStrategy() const;
    
    virtual void setRTPStrategy(const string&, const nmm_int32&, const nmm_int32&);
    
    virtual void setPayloadType(const nmm_int32&);
    
    virtual nmm_int32  getPayloadType() const;
    //@}        

    /**
     *  @name IRTPParameters Interface
     */
    //@{

    virtual string        getRTPSinkMediaType() const;

    virtual nmm_uchar getRTPSinkPayloadType() const;

    virtual string        getRTPSinkMapLine() const;

    virtual string        getRTPSinkAuxSDPLine() const;

    virtual nmm_uint16    getRTPSeqNum() const;

    virtual nmm_uint32    getRTPTimestamp() const;

    virtual nmm_uint32    getRTPPort() const;
    
    virtual nmm_uint32      getRTCPPort() const;

    virtual Result        setFormat(const Format& ) ;    
    //@}

  private:
    void setTransportStrategy(TransportStrategy*);
    
    TransportStrategy*       __transport_strategy;
    ITSOneWayBufferReceiver* __buffer_receiver;
    
    ThreadMutex          __internal_mutex;
    ThreadCondition      __internal_cond;

    int                  __payload_type;
    TSAddress*           __current_address;
    
    // required for sending reports more often    
    bool                   __first_buffer;
    Clock                  __clock;
    Time                   __last_report_time; 
  };

  /**
   * Class for receiving signals.
   */
  class NetSignalReceiver : public ISignalReceiver {
  private:
    NetSinkNode* __netsink;
    
  public:
    NetSignalReceiver(NetSinkNode* _netsink);
    virtual ~NetSignalReceiver();
    
    virtual void receiveSignal(Signal);
  };  
} //end namepsace

#include "nmm/nmm_suffix.hpp"

#endif
