/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#ifndef NMM_NETSOURCENODE_HPP
#define NMM_NETSOURCENODE_HPP NMM_NETSOURCENODE_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/multimedia/GenericSourceNode.hpp"
#include "nmm/comm/messaging/ITSOneWayBufferReceiver.hpp"

#include "nmm/plugins/net/INetImpl.hpp"

// HACK: fw: Required for sending reports more often
#include "nmm/utils/time/Clock.hpp"

#include <list>
using std::list;

namespace NMM {

  class TransportStrategy;
  class ITSOneWayBufferSender;

  class NetSourceNode :
    public GenericSourceNode,
    public INetImpl,
    public ITSOneWayBufferReceiver
  {
  public: 
    NetSourceNode(const char* = "NetSourceNode",
                  IQueued::Mode queue_mode = IQueued::MODE_SUSPEND,
                  unsigned int queue_size = 100);
    virtual ~NetSourceNode();

    Result enableOutputStream();
    Result disableOutputStream();

  protected:
    virtual Message* processBuffer(Buffer*);

    virtual Result doInit();
    virtual Result doInitOutput();
    virtual Result doActivate();

  public:
    // BEGIN> INet interface
    virtual void setTransportStrategy(const TSAddress*) throw(const BindingException&);

    virtual ITransportStrategy* getTransportStrategy() const;

    virtual void setRTPStrategy(const string&, const nmm_int32&, const nmm_int32&);

    virtual void setPayloadType(const nmm_int32& /*type*/);

    virtual nmm_int32 getPayloadType() const;
    virtual void flushITSReceiver();
    // END< INet interface

    // BEGIN> ITSOneWayBufferReceiver methods
    virtual Result receiveOneWayBuffer(Buffer*);
    virtual void configure(const Format&) throw(const FormatNotSupportedException&) {}
    // END< ITSOneWayBufferReceiver methods

  private:
    void setTransportStrategy(TransportStrategy*);

    TSAddress*             __address;
    TransportStrategy*     __transport_strategy;
    ITSOneWayBufferSender* __buffer_sender;

    Message*               __current_message;
    ThreadMutex            __current_message_mutex;

    StreamQueue            __stream_queue;

    int                    __payload_type;
    int                    __audio_channels;
    int                    __audio_bit_per_sample;
    int                    __audio_sample_rate;
    bool first;

    // HACK: fw: Required for sending reports more often
    bool                   __first_buffer;
    Clock                  __clock;
    Time                   __last_report_time;
  };


}

#include "nmm/nmm_suffix.hpp"

#endif
