/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IBufferCounter.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IBufferCounter::used() {} 
  
  
  nmm_int32 IBufferCounter::getBuffersPassed() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("IBufferCounter::getBuffersPassed");
    event.setReturnValue(&return_value);
    if(const_cast<IBufferCounter*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IBufferCounter::resetBufferCounter()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IBufferCounter::resetBufferCounter");
    if(const_cast<IBufferCounter*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IBufferCounter::IBufferCounter_type = "IBufferCounter";
  NMM::TConstructor<IBufferCounter > __IBufferCounter_prototype("IBufferCounter","NMM");


  const char* IBufferCounter::getBuffersPassed_event = "IBufferCounter::getBuffersPassed";
  const char* IBufferCounter::resetBufferCounter_event = "IBufferCounter::resetBufferCounter";


}


#include "nmm/nmm_suffix.hpp"

