/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IBufferCounter_HPP
#define NMM_IBufferCounter_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{
  /**
   * Interfae for counting the buffers processed by a node.
   */

  class DLLEXPORT IBufferCounter : virtual public NMM::Interface { 
  public:
    IBufferCounter() : NMM::Interface(IBufferCounter::IBufferCounter_type, "NMM") { };

    IBufferCounter(NMM::NMMObject& parent) : NMM::Interface (parent, IBufferCounter::IBufferCounter_type, "NMM") { };

    virtual ~IBufferCounter() {};

    static const char* IBufferCounter_type;

    virtual  string getInterfaceType() const { return IBufferCounter_type; }

    NMM::Interface* copy() const { return new IBufferCounter(*this); }
    static void used(); 
    static const char* getBuffersPassed_event;
    static const char* resetBufferCounter_event;
    /** Returns number of buffers processed.
     * 
     * Returns the current value of the buffer counter.
     *
     * @return Number of buffers processed.
     */

    virtual nmm_int32 getBuffersPassed() const
      throw(const NMM::SerializedException&);
    /** Resets the buffer counter.
     * 
     * Resets the buffer counter to zero buffers processed.
     */

    virtual void resetBufferCounter()
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IBufferCounter> IBufferCounter_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
