/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IBuffersPassedEventReceiver.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IBuffersPassedEventReceiver::used() {} 
  
  
  Result IBuffersPassedEventReceiver::buffersPassed(const nmm_int32& buffers) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > buffers_value(buffers, false);
    NMM::Event event("IBuffersPassedEventReceiver::buffersPassed", &buffers_value);
    event.setReturnValue(&return_value);
    if(const_cast<IBuffersPassedEventReceiver*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 IBuffersPassedEventReceiver::getUpstreamBuffersPassed() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("IBuffersPassedEventReceiver::getUpstreamBuffersPassed");
    event.setReturnValue(&return_value);
    if(const_cast<IBuffersPassedEventReceiver*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IBuffersPassedEventReceiver::resetUpstreamBufferCounter()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IBuffersPassedEventReceiver::resetUpstreamBufferCounter");
    if(const_cast<IBuffersPassedEventReceiver*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IBuffersPassedEventReceiver::IBuffersPassedEventReceiver_type = "IBuffersPassedEventReceiver";
  NMM::TConstructor<IBuffersPassedEventReceiver > __IBuffersPassedEventReceiver_prototype("IBuffersPassedEventReceiver","NMM");


  const char* IBuffersPassedEventReceiver::buffersPassed_event = "IBuffersPassedEventReceiver::buffersPassed";
  NMM::Event* IBuffersPassedEventReceiver::create_buffersPassed(const nmm_int32& buffers)
  {
    NMM::TInValue<nmm_int32 >* buffers_value = new NMM::TInValue<nmm_int32 >();
    buffers_value->setValue(buffers);

    return new NMM::Event("IBuffersPassedEventReceiver::buffersPassed", buffers_value);
  }

  const char* IBuffersPassedEventReceiver::getUpstreamBuffersPassed_event = "IBuffersPassedEventReceiver::getUpstreamBuffersPassed";
  const char* IBuffersPassedEventReceiver::resetUpstreamBufferCounter_event = "IBuffersPassedEventReceiver::resetUpstreamBufferCounter";


}


#include "nmm/nmm_suffix.hpp"

