/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IBuffersPassedEventReceiver_HPP
#define NMM_IBuffersPassedEventReceiver_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{
  /**
   * Interface impemented by objects that receive events containing the number of buffers passed.
   * These events are sent upstream from a node which implements IBuffersPassedEventSender.
   */

  class DLLEXPORT IBuffersPassedEventReceiver : virtual public NMM::Interface { 
  public:
    IBuffersPassedEventReceiver() : NMM::Interface(IBuffersPassedEventReceiver::IBuffersPassedEventReceiver_type, "NMM") { };

    IBuffersPassedEventReceiver(NMM::NMMObject& parent) : NMM::Interface (parent, IBuffersPassedEventReceiver::IBuffersPassedEventReceiver_type, "NMM") { };

    virtual ~IBuffersPassedEventReceiver() {};

    static const char* IBuffersPassedEventReceiver_type;

    virtual  string getInterfaceType() const { return IBuffersPassedEventReceiver_type; }

    NMM::Interface* copy() const { return new IBuffersPassedEventReceiver(*this); }
    static void used(); 
    static const char* buffersPassed_event;
    static NMM::Event* create_buffersPassed(const nmm_int32& buffers);
    static const char* getUpstreamBuffersPassed_event;
    static const char* resetUpstreamBufferCounter_event;
  /* instream (protected) and non-instream (public) */        /** Receives number of buffers processed recently.
     * 
     * This is an upstream event. The value is the number of buffers processed since the last event.
     *
     * @param buffers Number of buffers processed recently.
     */

    virtual Result buffersPassed(const nmm_int32& buffers) /* in-stream */ 
      throw(const NMM::SerializedException&);
  /* non-instream (public) */        /** Returns number of buffers processed below.
     * 
     * Returns the number of buffers processed below in the graph, according to the upstream events
     * we received.
     *
     * @return Number of buffers processed below.
     */

    virtual nmm_int32 getUpstreamBuffersPassed() const
      throw(const NMM::SerializedException&);
    /** Resets number of buffers processed below.
     */

    virtual void resetUpstreamBufferCounter()
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IBuffersPassedEventReceiver> IBuffersPassedEventReceiver_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
