/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IBuffersPassedEventSenderImpl_HPP
#define NMM_IBuffersPassedEventSenderImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IBuffersPassedEventSender.hpp"


#include "visibility.hpp"

namespace NMM
{
  /**
   * Interface to be implemented by objects that send the number of buffers passed upstream.
   * This interface is for setting the number of buffers after which to send an upstream event
   * containing the number of buffers passed. The event is received by a node that implements
   * IBuffersPassedEventReceiver.
   */

  class DLLEXPORT IBuffersPassedEventSenderImpl  : public virtual NMM::NMMObject { 
  public:
    IBuffersPassedEventSenderImpl() { addInterface(new IBuffersPassedEventSender(*this)); registerEvents(); };

    virtual ~IBuffersPassedEventSenderImpl() {};

  protected:
    /** Sets buffer count interval.
     * 
     * Sets the number of buffers after which to send the upstream event.
     * Setting it to zero disables sending the event. Buffers processed while the interval is zero
     * are not counted in upstream events.
     *
     * @param value Buffer count interval.
     */

    virtual void setBufferCountInterval(const nmm_int32& value) = 0;
    /** Returns buffer count interval.
     * 
     * Returns the number of buffers after which to send the upstream event.
     * Zero means no upstream events are sent.
     *
     * @return Buffer count interval.
     */

    virtual nmm_int32 getBufferCountInterval() const = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
