/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ISleep_HPP
#define NMM_ISleep_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{
  /**
   * Interface for setting the sleep time after receiving a buffer.
   * This functionality is for testing purposes and implemented, for example, in DevNullNode.
   */

  class DLLEXPORT ISleep : virtual public NMM::Interface { 
  public:
    ISleep() : NMM::Interface(ISleep::ISleep_type, "NMM") { };

    ISleep(NMM::NMMObject& parent) : NMM::Interface (parent, ISleep::ISleep_type, "NMM") { };

    virtual ~ISleep() {};

    static const char* ISleep_type;

    virtual  string getInterfaceType() const { return ISleep_type; }

    NMM::Interface* copy() const { return new ISleep(*this); }
    static void used(); 
    static const char* setSleepTime_event;
    static const char* getSleepTime_event;
    /** Sets the sleep time.
     * 
     * Set the time the node will wait after receiving a buffer, in seconds.
     *
     * @param value Time to sleep.
     */

    virtual void setSleepTime(const nmm_int32& value)
      throw(const NMM::SerializedException&);
    /** Returns the sleep time.
     *  
     * Returns the time the node will wait after receiving a buffer, in seconds.
     *
     * @return Time to sleep.
     */

    virtual nmm_int32 getSleepTime() const
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ISleep> ISleep_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
