/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#include "IdNode.hpp"

#include "nmm/comm/format/Property.hpp"

#include "nmm/plugins/NMMRegisterPlugin.hpp"
#include "nmm/utils/thread/Thread.hpp"

NMM_REGISTER_PLUGIN(IdNode, NMM, "IdNode")

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  IdNode::IdNode(const char* name,
                 const StreamQueue::Mode queue_mode,
                 const int size)
    : GenericFilterNode(name, queue_mode, size) 
  {
    buffers_passed = 0;
    upstream_passed = 0;
    sleep_time = 0;
  }
  
  void IdNode::setSleepTime(const nmm_int32& value) {
    sleep_time = value;
  }
  
  nmm_int32 IdNode::getSleepTime() const {
    return sleep_time;
  }
  
  nmm_int32 IdNode::getBuffersPassed() const {
    return buffers_passed;
  }
  
  void IdNode::resetBufferCounter() {
    buffers_passed = 0;
  }
  
  Result IdNode::doInit() {
    Format* format;
    
    format = getOwnInputProperty()->addNewFormat("*/*");
    getOwnInputProperty()->setDefaultFormat(format);
    
    format = getOwnOutputProperty()->addNewFormat("*/*");
    getOwnOutputProperty()->setDefaultFormat(format);
    
    return SUCCESS;
  }
  
  Result IdNode::doInitOutput() {
    const Format* in_format = getInputFormat();
    if (in_format == NULL) { return FAILURE; }
    Format* format;
    
    format = new Format( *in_format );
    getOwnWorkingInputProperty()->addNewFormat(format);
    getOwnWorkingInputProperty()->setDefaultFormat(format);
    
    format = new Format( *in_format );
    getOwnWorkingOutputProperty()->addNewFormat(format);
    getOwnWorkingOutputProperty()->setDefaultFormat(format);
    
    return SUCCESS;
  }
  
  Message* IdNode::processBuffer(Buffer *in_buffer) {
    buffers_passed++;
    if (sleep_time) {
		Thread::suspendThread(sleep_time/1000);
    }
    return in_buffer;
  }
  
  Result IdNode::buffersPassed(const nmm_int32& value) {
    upstream_passed += value;
    return SUCCESS;
  }
  
  nmm_int32 IdNode::getUpstreamBuffersPassed() const {
    return upstream_passed;
  }
  
  void IdNode::resetUpstreamBufferCounter() {
    upstream_passed = 0;
  }
}

#include "nmm/nmm_suffix.hpp"
