/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "RawNode.hpp"

#include "nmm/plugins/NMMRegisterPlugin.hpp"

NMM_REGISTER_PLUGIN(RawNode, NMM, "RawNode")

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  RawNode::RawNode(const char* name /* = "RawNode" */,
		   StreamQueue::Mode mode /* = StreamQueue::MODE_SUSPEND */,
		   int queue_size /* = StreamQueue::DEFAULT_SIZE */)
    : GenericSourceNode(name, mode, queue_size) {
    buffers_passed = 0L;
    upstream_passed = 0L;
    buffer_size = 1024;
  }
  
  nmm_int32 RawNode::getBuffersPassed() const {
    return buffers_passed;
  }
  
  void RawNode::resetBufferCounter() {
    buffers_passed = 0;
  }
  
  void RawNode::setBufferSize(const nmm_int32& value) {
    buffer_size = value;
  }
  
  nmm_int32 RawNode::getBufferSize() {
    return buffer_size;
  }
  
  Result RawNode::doInit() {
    Format* format = getOwnOutputProperty()->addNewFormat("*/*");
    getOwnOutputProperty()->setDefaultFormat(format);
    
    return SUCCESS;
  }
  
  Result RawNode::doInitOutput() {
    Format* format = getOwnWorkingOutputProperty()->addNewFormat("*/*");
    getOwnWorkingOutputProperty()->setDefaultFormat(format);
    
    return SUCCESS;
  }
  
  Message* RawNode::processBuffer(Buffer *) {
    Buffer *out_buffer = getNewBuffer(buffer_size);
    buffers_passed++;
    return out_buffer;
  }
  
  Result RawNode::buffersPassed(const nmm_int32& value) {
    upstream_passed += value;
    return SUCCESS;
  }
  
  nmm_int32 RawNode::getUpstreamBuffersPassed() const {
    return upstream_passed;
  }
  
  void RawNode::resetUpstreamBufferCounter() {
    upstream_passed = 0;
  }
}

#include "nmm/nmm_suffix.hpp"
