/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ICamera.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ICamera::used() {} 
  
  
  void ICamera::setBrightness(const nmm_int32& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > value_value(value, false);
    NMM::Event event("ICamera::setBrightness", &value_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ICamera::setAutoExposure(const nmm_int32& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > value_value(value, false);
    NMM::Event event("ICamera::setAutoExposure", &value_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ICamera::setSharpness(const nmm_int32& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > value_value(value, false);
    NMM::Event event("ICamera::setSharpness", &value_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ICamera::setWhiteBalance(const nmm_int32& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > value_value(value, false);
    NMM::Event event("ICamera::setWhiteBalance", &value_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ICamera::setBlueBalance(const nmm_int32& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > value_value(value, false);
    NMM::Event event("ICamera::setBlueBalance", &value_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ICamera::setRedBalance(const nmm_int32& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > value_value(value, false);
    NMM::Event event("ICamera::setRedBalance", &value_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ICamera::setHue(const nmm_int32& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > value_value(value, false);
    NMM::Event event("ICamera::setHue", &value_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ICamera::setSaturation(const nmm_int32& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > value_value(value, false);
    NMM::Event event("ICamera::setSaturation", &value_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ICamera::setGamma(const nmm_int32& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > value_value(value, false);
    NMM::Event event("ICamera::setGamma", &value_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ICamera::setShutter(const nmm_int32& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > value_value(value, false);
    NMM::Event event("ICamera::setShutter", &value_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ICamera::setGain(const nmm_int32& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > value_value(value, false);
    NMM::Event event("ICamera::setGain", &value_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ICamera::setIris(const nmm_int32& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > value_value(value, false);
    NMM::Event event("ICamera::setIris", &value_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ICamera::setFocus(const nmm_int32& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > value_value(value, false);
    NMM::Event event("ICamera::setFocus", &value_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ICamera::setZoom(const nmm_int32& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > value_value(value, false);
    NMM::Event event("ICamera::setZoom", &value_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ICamera::onePush()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("ICamera::onePush");
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_int32 ICamera::getBrightness() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("ICamera::getBrightness");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 ICamera::getAutoExposure() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("ICamera::getAutoExposure");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 ICamera::getSharpness() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("ICamera::getSharpness");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 ICamera::getWhiteBalance() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("ICamera::getWhiteBalance");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 ICamera::getBlueBalance() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("ICamera::getBlueBalance");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 ICamera::getRedBalance() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("ICamera::getRedBalance");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 ICamera::getHue() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("ICamera::getHue");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 ICamera::getSaturation() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("ICamera::getSaturation");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 ICamera::getGamma() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("ICamera::getGamma");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 ICamera::getShutter() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("ICamera::getShutter");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 ICamera::getGain() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("ICamera::getGain");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 ICamera::getIris() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("ICamera::getIris");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 ICamera::getFocus() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("ICamera::getFocus");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 ICamera::getZoom() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("ICamera::getZoom");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  pair<nmm_int32, nmm_int32 >  ICamera::getBrightnessRange() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<pair<nmm_int32, nmm_int32 >  > return_value;
    NMM::Event event("ICamera::getBrightnessRange");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  pair<nmm_int32, nmm_int32 >  ICamera::getAutoExposureRange() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<pair<nmm_int32, nmm_int32 >  > return_value;
    NMM::Event event("ICamera::getAutoExposureRange");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  pair<nmm_int32, nmm_int32 >  ICamera::getSharpnessRange() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<pair<nmm_int32, nmm_int32 >  > return_value;
    NMM::Event event("ICamera::getSharpnessRange");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  pair<nmm_int32, nmm_int32 >  ICamera::getWhiteBalanceRange() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<pair<nmm_int32, nmm_int32 >  > return_value;
    NMM::Event event("ICamera::getWhiteBalanceRange");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  pair<nmm_int32, nmm_int32 >  ICamera::getHueRange() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<pair<nmm_int32, nmm_int32 >  > return_value;
    NMM::Event event("ICamera::getHueRange");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  pair<nmm_int32, nmm_int32 >  ICamera::getSaturationRange() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<pair<nmm_int32, nmm_int32 >  > return_value;
    NMM::Event event("ICamera::getSaturationRange");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  pair<nmm_int32, nmm_int32 >  ICamera::getGammaRange() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<pair<nmm_int32, nmm_int32 >  > return_value;
    NMM::Event event("ICamera::getGammaRange");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  pair<nmm_int32, nmm_int32 >  ICamera::getShutterRange() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<pair<nmm_int32, nmm_int32 >  > return_value;
    NMM::Event event("ICamera::getShutterRange");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  pair<nmm_int32, nmm_int32 >  ICamera::getGainRange() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<pair<nmm_int32, nmm_int32 >  > return_value;
    NMM::Event event("ICamera::getGainRange");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  pair<nmm_int32, nmm_int32 >  ICamera::getIrisRange() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<pair<nmm_int32, nmm_int32 >  > return_value;
    NMM::Event event("ICamera::getIrisRange");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  pair<nmm_int32, nmm_int32 >  ICamera::getFocusRange() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<pair<nmm_int32, nmm_int32 >  > return_value;
    NMM::Event event("ICamera::getFocusRange");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  pair<nmm_int32, nmm_int32 >  ICamera::getZoomRange() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<pair<nmm_int32, nmm_int32 >  > return_value;
    NMM::Event event("ICamera::getZoomRange");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool ICamera::hasBrightness() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ICamera::hasBrightness");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool ICamera::hasAutoExposure() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ICamera::hasAutoExposure");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool ICamera::hasSharpness() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ICamera::hasSharpness");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool ICamera::hasWhiteBalance() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ICamera::hasWhiteBalance");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool ICamera::hasHue() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ICamera::hasHue");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool ICamera::hasSaturation() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ICamera::hasSaturation");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool ICamera::hasGamma() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ICamera::hasGamma");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool ICamera::hasShutter() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ICamera::hasShutter");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool ICamera::hasGain() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ICamera::hasGain");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool ICamera::hasIris() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ICamera::hasIris");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool ICamera::hasFocus() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ICamera::hasFocus");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool ICamera::hasZoom() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ICamera::hasZoom");
    event.setReturnValue(&return_value);
    if(const_cast<ICamera*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ICamera::ICamera_type = "ICamera";
  NMM::TConstructor<ICamera > __ICamera_prototype("ICamera","NMM");


  const char* ICamera::setBrightness_event = "ICamera::setBrightness";
  const char* ICamera::setAutoExposure_event = "ICamera::setAutoExposure";
  const char* ICamera::setSharpness_event = "ICamera::setSharpness";
  const char* ICamera::setWhiteBalance_event = "ICamera::setWhiteBalance";
  const char* ICamera::setBlueBalance_event = "ICamera::setBlueBalance";
  const char* ICamera::setRedBalance_event = "ICamera::setRedBalance";
  const char* ICamera::setHue_event = "ICamera::setHue";
  const char* ICamera::setSaturation_event = "ICamera::setSaturation";
  const char* ICamera::setGamma_event = "ICamera::setGamma";
  const char* ICamera::setShutter_event = "ICamera::setShutter";
  const char* ICamera::setGain_event = "ICamera::setGain";
  const char* ICamera::setIris_event = "ICamera::setIris";
  const char* ICamera::setFocus_event = "ICamera::setFocus";
  const char* ICamera::setZoom_event = "ICamera::setZoom";
  const char* ICamera::onePush_event = "ICamera::onePush";
  const char* ICamera::getBrightness_event = "ICamera::getBrightness";
  const char* ICamera::getAutoExposure_event = "ICamera::getAutoExposure";
  const char* ICamera::getSharpness_event = "ICamera::getSharpness";
  const char* ICamera::getWhiteBalance_event = "ICamera::getWhiteBalance";
  const char* ICamera::getBlueBalance_event = "ICamera::getBlueBalance";
  const char* ICamera::getRedBalance_event = "ICamera::getRedBalance";
  const char* ICamera::getHue_event = "ICamera::getHue";
  const char* ICamera::getSaturation_event = "ICamera::getSaturation";
  const char* ICamera::getGamma_event = "ICamera::getGamma";
  const char* ICamera::getShutter_event = "ICamera::getShutter";
  const char* ICamera::getGain_event = "ICamera::getGain";
  const char* ICamera::getIris_event = "ICamera::getIris";
  const char* ICamera::getFocus_event = "ICamera::getFocus";
  const char* ICamera::getZoom_event = "ICamera::getZoom";
  const char* ICamera::getBrightnessRange_event = "ICamera::getBrightnessRange";
  const char* ICamera::getAutoExposureRange_event = "ICamera::getAutoExposureRange";
  const char* ICamera::getSharpnessRange_event = "ICamera::getSharpnessRange";
  const char* ICamera::getWhiteBalanceRange_event = "ICamera::getWhiteBalanceRange";
  const char* ICamera::getHueRange_event = "ICamera::getHueRange";
  const char* ICamera::getSaturationRange_event = "ICamera::getSaturationRange";
  const char* ICamera::getGammaRange_event = "ICamera::getGammaRange";
  const char* ICamera::getShutterRange_event = "ICamera::getShutterRange";
  const char* ICamera::getGainRange_event = "ICamera::getGainRange";
  const char* ICamera::getIrisRange_event = "ICamera::getIrisRange";
  const char* ICamera::getFocusRange_event = "ICamera::getFocusRange";
  const char* ICamera::getZoomRange_event = "ICamera::getZoomRange";
  const char* ICamera::hasBrightness_event = "ICamera::hasBrightness";
  const char* ICamera::hasAutoExposure_event = "ICamera::hasAutoExposure";
  const char* ICamera::hasSharpness_event = "ICamera::hasSharpness";
  const char* ICamera::hasWhiteBalance_event = "ICamera::hasWhiteBalance";
  const char* ICamera::hasHue_event = "ICamera::hasHue";
  const char* ICamera::hasSaturation_event = "ICamera::hasSaturation";
  const char* ICamera::hasGamma_event = "ICamera::hasGamma";
  const char* ICamera::hasShutter_event = "ICamera::hasShutter";
  const char* ICamera::hasGain_event = "ICamera::hasGain";
  const char* ICamera::hasIris_event = "ICamera::hasIris";
  const char* ICamera::hasFocus_event = "ICamera::hasFocus";
  const char* ICamera::hasZoom_event = "ICamera::hasZoom";


}


#include "nmm/nmm_suffix.hpp"

