/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ICamera_HPP
#define NMM_ICamera_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/serialize/StdValue.hpp"

#include "visibility.hpp"

namespace NMM
{

  class DLLEXPORT ICamera : virtual public NMM::Interface { 
  public:
    ICamera() : NMM::Interface(ICamera::ICamera_type, "NMM") { };

    ICamera(NMM::NMMObject& parent) : NMM::Interface (parent, ICamera::ICamera_type, "NMM") { };

    virtual ~ICamera() {};

    static const char* ICamera_type;

    virtual  string getInterfaceType() const { return ICamera_type; }

    NMM::Interface* copy() const { return new ICamera(*this); }
    static void used(); 
    static const char* setBrightness_event;
    static const char* setAutoExposure_event;
    static const char* setSharpness_event;
    static const char* setWhiteBalance_event;
    static const char* setBlueBalance_event;
    static const char* setRedBalance_event;
    static const char* setHue_event;
    static const char* setSaturation_event;
    static const char* setGamma_event;
    static const char* setShutter_event;
    static const char* setGain_event;
    static const char* setIris_event;
    static const char* setFocus_event;
    static const char* setZoom_event;
    static const char* onePush_event;
    static const char* getBrightness_event;
    static const char* getAutoExposure_event;
    static const char* getSharpness_event;
    static const char* getWhiteBalance_event;
    static const char* getBlueBalance_event;
    static const char* getRedBalance_event;
    static const char* getHue_event;
    static const char* getSaturation_event;
    static const char* getGamma_event;
    static const char* getShutter_event;
    static const char* getGain_event;
    static const char* getIris_event;
    static const char* getFocus_event;
    static const char* getZoom_event;
    static const char* getBrightnessRange_event;
    static const char* getAutoExposureRange_event;
    static const char* getSharpnessRange_event;
    static const char* getWhiteBalanceRange_event;
    static const char* getHueRange_event;
    static const char* getSaturationRange_event;
    static const char* getGammaRange_event;
    static const char* getShutterRange_event;
    static const char* getGainRange_event;
    static const char* getIrisRange_event;
    static const char* getFocusRange_event;
    static const char* getZoomRange_event;
    static const char* hasBrightness_event;
    static const char* hasAutoExposure_event;
    static const char* hasSharpness_event;
    static const char* hasWhiteBalance_event;
    static const char* hasHue_event;
    static const char* hasSaturation_event;
    static const char* hasGamma_event;
    static const char* hasShutter_event;
    static const char* hasGain_event;
    static const char* hasIris_event;
    static const char* hasFocus_event;
    static const char* hasZoom_event;

    virtual void setBrightness(const nmm_int32& value)
      throw(const NMM::SerializedException&);

    virtual void setAutoExposure(const nmm_int32& value)
      throw(const NMM::SerializedException&);

    virtual void setSharpness(const nmm_int32& value)
      throw(const NMM::SerializedException&);

    virtual void setWhiteBalance(const nmm_int32& value)
      throw(const NMM::SerializedException&);

    virtual void setBlueBalance(const nmm_int32& value)
      throw(const NMM::SerializedException&);

    virtual void setRedBalance(const nmm_int32& value)
      throw(const NMM::SerializedException&);

    virtual void setHue(const nmm_int32& value)
      throw(const NMM::SerializedException&);

    virtual void setSaturation(const nmm_int32& value)
      throw(const NMM::SerializedException&);

    virtual void setGamma(const nmm_int32& value)
      throw(const NMM::SerializedException&);

    virtual void setShutter(const nmm_int32& value)
      throw(const NMM::SerializedException&);

    virtual void setGain(const nmm_int32& value)
      throw(const NMM::SerializedException&);

    virtual void setIris(const nmm_int32& value)
      throw(const NMM::SerializedException&);

    virtual void setFocus(const nmm_int32& value)
      throw(const NMM::SerializedException&);

    virtual void setZoom(const nmm_int32& value)
      throw(const NMM::SerializedException&);

    virtual void onePush()
      throw(const NMM::SerializedException&);

    virtual nmm_int32 getBrightness() const
      throw(const NMM::SerializedException&);

    virtual nmm_int32 getAutoExposure() const
      throw(const NMM::SerializedException&);

    virtual nmm_int32 getSharpness() const
      throw(const NMM::SerializedException&);

    virtual nmm_int32 getWhiteBalance() const
      throw(const NMM::SerializedException&);

    virtual nmm_int32 getBlueBalance() const
      throw(const NMM::SerializedException&);

    virtual nmm_int32 getRedBalance() const
      throw(const NMM::SerializedException&);

    virtual nmm_int32 getHue() const
      throw(const NMM::SerializedException&);

    virtual nmm_int32 getSaturation() const
      throw(const NMM::SerializedException&);

    virtual nmm_int32 getGamma() const
      throw(const NMM::SerializedException&);

    virtual nmm_int32 getShutter() const
      throw(const NMM::SerializedException&);

    virtual nmm_int32 getGain() const
      throw(const NMM::SerializedException&);

    virtual nmm_int32 getIris() const
      throw(const NMM::SerializedException&);

    virtual nmm_int32 getFocus() const
      throw(const NMM::SerializedException&);

    virtual nmm_int32 getZoom() const
      throw(const NMM::SerializedException&);

    virtual pair<nmm_int32, nmm_int32 >  getBrightnessRange() const
      throw(const NMM::SerializedException&);

    virtual pair<nmm_int32, nmm_int32 >  getAutoExposureRange() const
      throw(const NMM::SerializedException&);

    virtual pair<nmm_int32, nmm_int32 >  getSharpnessRange() const
      throw(const NMM::SerializedException&);

    virtual pair<nmm_int32, nmm_int32 >  getWhiteBalanceRange() const
      throw(const NMM::SerializedException&);

    virtual pair<nmm_int32, nmm_int32 >  getHueRange() const
      throw(const NMM::SerializedException&);

    virtual pair<nmm_int32, nmm_int32 >  getSaturationRange() const
      throw(const NMM::SerializedException&);

    virtual pair<nmm_int32, nmm_int32 >  getGammaRange() const
      throw(const NMM::SerializedException&);

    virtual pair<nmm_int32, nmm_int32 >  getShutterRange() const
      throw(const NMM::SerializedException&);

    virtual pair<nmm_int32, nmm_int32 >  getGainRange() const
      throw(const NMM::SerializedException&);

    virtual pair<nmm_int32, nmm_int32 >  getIrisRange() const
      throw(const NMM::SerializedException&);

    virtual pair<nmm_int32, nmm_int32 >  getFocusRange() const
      throw(const NMM::SerializedException&);

    virtual pair<nmm_int32, nmm_int32 >  getZoomRange() const
      throw(const NMM::SerializedException&);

    virtual nmm_bool hasBrightness() const
      throw(const NMM::SerializedException&);

    virtual nmm_bool hasAutoExposure() const
      throw(const NMM::SerializedException&);

    virtual nmm_bool hasSharpness() const
      throw(const NMM::SerializedException&);

    virtual nmm_bool hasWhiteBalance() const
      throw(const NMM::SerializedException&);

    virtual nmm_bool hasHue() const
      throw(const NMM::SerializedException&);

    virtual nmm_bool hasSaturation() const
      throw(const NMM::SerializedException&);

    virtual nmm_bool hasGamma() const
      throw(const NMM::SerializedException&);

    virtual nmm_bool hasShutter() const
      throw(const NMM::SerializedException&);

    virtual nmm_bool hasGain() const
      throw(const NMM::SerializedException&);

    virtual nmm_bool hasIris() const
      throw(const NMM::SerializedException&);

    virtual nmm_bool hasFocus() const
      throw(const NMM::SerializedException&);

    virtual nmm_bool hasZoom() const
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ICamera> ICamera_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
