/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ICameraImpl.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/messaging/TEDObject.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  
  void ICameraImpl::registerEvents()
  {
  
    getEventDispatcher().registerEvent(ICamera::setBrightness_event, new NMM::TEDObject1<ICameraImpl, nmm_int32, void(ICameraImpl:: * )(const nmm_int32&), void >(this, &ICameraImpl::setBrightness));
  
    getEventDispatcher().registerEvent(ICamera::setAutoExposure_event, new NMM::TEDObject1<ICameraImpl, nmm_int32, void(ICameraImpl:: * )(const nmm_int32&), void >(this, &ICameraImpl::setAutoExposure));
  
    getEventDispatcher().registerEvent(ICamera::setSharpness_event, new NMM::TEDObject1<ICameraImpl, nmm_int32, void(ICameraImpl:: * )(const nmm_int32&), void >(this, &ICameraImpl::setSharpness));
  
    getEventDispatcher().registerEvent(ICamera::setWhiteBalance_event, new NMM::TEDObject1<ICameraImpl, nmm_int32, void(ICameraImpl:: * )(const nmm_int32&), void >(this, &ICameraImpl::setWhiteBalance));
  
    getEventDispatcher().registerEvent(ICamera::setBlueBalance_event, new NMM::TEDObject1<ICameraImpl, nmm_int32, void(ICameraImpl:: * )(const nmm_int32&), void >(this, &ICameraImpl::setBlueBalance));
  
    getEventDispatcher().registerEvent(ICamera::setRedBalance_event, new NMM::TEDObject1<ICameraImpl, nmm_int32, void(ICameraImpl:: * )(const nmm_int32&), void >(this, &ICameraImpl::setRedBalance));
  
    getEventDispatcher().registerEvent(ICamera::setHue_event, new NMM::TEDObject1<ICameraImpl, nmm_int32, void(ICameraImpl:: * )(const nmm_int32&), void >(this, &ICameraImpl::setHue));
  
    getEventDispatcher().registerEvent(ICamera::setSaturation_event, new NMM::TEDObject1<ICameraImpl, nmm_int32, void(ICameraImpl:: * )(const nmm_int32&), void >(this, &ICameraImpl::setSaturation));
  
    getEventDispatcher().registerEvent(ICamera::setGamma_event, new NMM::TEDObject1<ICameraImpl, nmm_int32, void(ICameraImpl:: * )(const nmm_int32&), void >(this, &ICameraImpl::setGamma));
  
    getEventDispatcher().registerEvent(ICamera::setShutter_event, new NMM::TEDObject1<ICameraImpl, nmm_int32, void(ICameraImpl:: * )(const nmm_int32&), void >(this, &ICameraImpl::setShutter));
  
    getEventDispatcher().registerEvent(ICamera::setGain_event, new NMM::TEDObject1<ICameraImpl, nmm_int32, void(ICameraImpl:: * )(const nmm_int32&), void >(this, &ICameraImpl::setGain));
  
    getEventDispatcher().registerEvent(ICamera::setIris_event, new NMM::TEDObject1<ICameraImpl, nmm_int32, void(ICameraImpl:: * )(const nmm_int32&), void >(this, &ICameraImpl::setIris));
  
    getEventDispatcher().registerEvent(ICamera::setFocus_event, new NMM::TEDObject1<ICameraImpl, nmm_int32, void(ICameraImpl:: * )(const nmm_int32&), void >(this, &ICameraImpl::setFocus));
  
    getEventDispatcher().registerEvent(ICamera::setZoom_event, new NMM::TEDObject1<ICameraImpl, nmm_int32, void(ICameraImpl:: * )(const nmm_int32&), void >(this, &ICameraImpl::setZoom));
  
    getEventDispatcher().registerEvent(ICamera::onePush_event, new NMM::TEDObject0<ICameraImpl, void(ICameraImpl:: * )(), void >(this, &ICameraImpl::onePush));
  
    getEventDispatcher().registerEvent(ICamera::getBrightness_event, new NMM::TEDObject0<ICameraImpl, nmm_int32(ICameraImpl:: * )() const, nmm_int32 >(this, &ICameraImpl::getBrightness));
  
    getEventDispatcher().registerEvent(ICamera::getAutoExposure_event, new NMM::TEDObject0<ICameraImpl, nmm_int32(ICameraImpl:: * )() const, nmm_int32 >(this, &ICameraImpl::getAutoExposure));
  
    getEventDispatcher().registerEvent(ICamera::getSharpness_event, new NMM::TEDObject0<ICameraImpl, nmm_int32(ICameraImpl:: * )() const, nmm_int32 >(this, &ICameraImpl::getSharpness));
  
    getEventDispatcher().registerEvent(ICamera::getWhiteBalance_event, new NMM::TEDObject0<ICameraImpl, nmm_int32(ICameraImpl:: * )() const, nmm_int32 >(this, &ICameraImpl::getWhiteBalance));
  
    getEventDispatcher().registerEvent(ICamera::getBlueBalance_event, new NMM::TEDObject0<ICameraImpl, nmm_int32(ICameraImpl:: * )() const, nmm_int32 >(this, &ICameraImpl::getBlueBalance));
  
    getEventDispatcher().registerEvent(ICamera::getRedBalance_event, new NMM::TEDObject0<ICameraImpl, nmm_int32(ICameraImpl:: * )() const, nmm_int32 >(this, &ICameraImpl::getRedBalance));
  
    getEventDispatcher().registerEvent(ICamera::getHue_event, new NMM::TEDObject0<ICameraImpl, nmm_int32(ICameraImpl:: * )() const, nmm_int32 >(this, &ICameraImpl::getHue));
  
    getEventDispatcher().registerEvent(ICamera::getSaturation_event, new NMM::TEDObject0<ICameraImpl, nmm_int32(ICameraImpl:: * )() const, nmm_int32 >(this, &ICameraImpl::getSaturation));
  
    getEventDispatcher().registerEvent(ICamera::getGamma_event, new NMM::TEDObject0<ICameraImpl, nmm_int32(ICameraImpl:: * )() const, nmm_int32 >(this, &ICameraImpl::getGamma));
  
    getEventDispatcher().registerEvent(ICamera::getShutter_event, new NMM::TEDObject0<ICameraImpl, nmm_int32(ICameraImpl:: * )() const, nmm_int32 >(this, &ICameraImpl::getShutter));
  
    getEventDispatcher().registerEvent(ICamera::getGain_event, new NMM::TEDObject0<ICameraImpl, nmm_int32(ICameraImpl:: * )() const, nmm_int32 >(this, &ICameraImpl::getGain));
  
    getEventDispatcher().registerEvent(ICamera::getIris_event, new NMM::TEDObject0<ICameraImpl, nmm_int32(ICameraImpl:: * )() const, nmm_int32 >(this, &ICameraImpl::getIris));
  
    getEventDispatcher().registerEvent(ICamera::getFocus_event, new NMM::TEDObject0<ICameraImpl, nmm_int32(ICameraImpl:: * )() const, nmm_int32 >(this, &ICameraImpl::getFocus));
  
    getEventDispatcher().registerEvent(ICamera::getZoom_event, new NMM::TEDObject0<ICameraImpl, nmm_int32(ICameraImpl:: * )() const, nmm_int32 >(this, &ICameraImpl::getZoom));
  
    getEventDispatcher().registerEvent(ICamera::getBrightnessRange_event, new NMM::TEDObject0<ICameraImpl, pair<nmm_int32, nmm_int32 > (ICameraImpl:: * )() const, pair<nmm_int32, nmm_int32 >  >(this, &ICameraImpl::getBrightnessRange));
  
    getEventDispatcher().registerEvent(ICamera::getAutoExposureRange_event, new NMM::TEDObject0<ICameraImpl, pair<nmm_int32, nmm_int32 > (ICameraImpl:: * )() const, pair<nmm_int32, nmm_int32 >  >(this, &ICameraImpl::getAutoExposureRange));
  
    getEventDispatcher().registerEvent(ICamera::getSharpnessRange_event, new NMM::TEDObject0<ICameraImpl, pair<nmm_int32, nmm_int32 > (ICameraImpl:: * )() const, pair<nmm_int32, nmm_int32 >  >(this, &ICameraImpl::getSharpnessRange));
  
    getEventDispatcher().registerEvent(ICamera::getWhiteBalanceRange_event, new NMM::TEDObject0<ICameraImpl, pair<nmm_int32, nmm_int32 > (ICameraImpl:: * )() const, pair<nmm_int32, nmm_int32 >  >(this, &ICameraImpl::getWhiteBalanceRange));
  
    getEventDispatcher().registerEvent(ICamera::getHueRange_event, new NMM::TEDObject0<ICameraImpl, pair<nmm_int32, nmm_int32 > (ICameraImpl:: * )() const, pair<nmm_int32, nmm_int32 >  >(this, &ICameraImpl::getHueRange));
  
    getEventDispatcher().registerEvent(ICamera::getSaturationRange_event, new NMM::TEDObject0<ICameraImpl, pair<nmm_int32, nmm_int32 > (ICameraImpl:: * )() const, pair<nmm_int32, nmm_int32 >  >(this, &ICameraImpl::getSaturationRange));
  
    getEventDispatcher().registerEvent(ICamera::getGammaRange_event, new NMM::TEDObject0<ICameraImpl, pair<nmm_int32, nmm_int32 > (ICameraImpl:: * )() const, pair<nmm_int32, nmm_int32 >  >(this, &ICameraImpl::getGammaRange));
  
    getEventDispatcher().registerEvent(ICamera::getShutterRange_event, new NMM::TEDObject0<ICameraImpl, pair<nmm_int32, nmm_int32 > (ICameraImpl:: * )() const, pair<nmm_int32, nmm_int32 >  >(this, &ICameraImpl::getShutterRange));
  
    getEventDispatcher().registerEvent(ICamera::getGainRange_event, new NMM::TEDObject0<ICameraImpl, pair<nmm_int32, nmm_int32 > (ICameraImpl:: * )() const, pair<nmm_int32, nmm_int32 >  >(this, &ICameraImpl::getGainRange));
  
    getEventDispatcher().registerEvent(ICamera::getIrisRange_event, new NMM::TEDObject0<ICameraImpl, pair<nmm_int32, nmm_int32 > (ICameraImpl:: * )() const, pair<nmm_int32, nmm_int32 >  >(this, &ICameraImpl::getIrisRange));
  
    getEventDispatcher().registerEvent(ICamera::getFocusRange_event, new NMM::TEDObject0<ICameraImpl, pair<nmm_int32, nmm_int32 > (ICameraImpl:: * )() const, pair<nmm_int32, nmm_int32 >  >(this, &ICameraImpl::getFocusRange));
  
    getEventDispatcher().registerEvent(ICamera::getZoomRange_event, new NMM::TEDObject0<ICameraImpl, pair<nmm_int32, nmm_int32 > (ICameraImpl:: * )() const, pair<nmm_int32, nmm_int32 >  >(this, &ICameraImpl::getZoomRange));
  
    getEventDispatcher().registerEvent(ICamera::hasBrightness_event, new NMM::TEDObject0<ICameraImpl, nmm_bool(ICameraImpl:: * )() const, nmm_bool >(this, &ICameraImpl::hasBrightness));
  
    getEventDispatcher().registerEvent(ICamera::hasAutoExposure_event, new NMM::TEDObject0<ICameraImpl, nmm_bool(ICameraImpl:: * )() const, nmm_bool >(this, &ICameraImpl::hasAutoExposure));
  
    getEventDispatcher().registerEvent(ICamera::hasSharpness_event, new NMM::TEDObject0<ICameraImpl, nmm_bool(ICameraImpl:: * )() const, nmm_bool >(this, &ICameraImpl::hasSharpness));
  
    getEventDispatcher().registerEvent(ICamera::hasWhiteBalance_event, new NMM::TEDObject0<ICameraImpl, nmm_bool(ICameraImpl:: * )() const, nmm_bool >(this, &ICameraImpl::hasWhiteBalance));
  
    getEventDispatcher().registerEvent(ICamera::hasHue_event, new NMM::TEDObject0<ICameraImpl, nmm_bool(ICameraImpl:: * )() const, nmm_bool >(this, &ICameraImpl::hasHue));
  
    getEventDispatcher().registerEvent(ICamera::hasSaturation_event, new NMM::TEDObject0<ICameraImpl, nmm_bool(ICameraImpl:: * )() const, nmm_bool >(this, &ICameraImpl::hasSaturation));
  
    getEventDispatcher().registerEvent(ICamera::hasGamma_event, new NMM::TEDObject0<ICameraImpl, nmm_bool(ICameraImpl:: * )() const, nmm_bool >(this, &ICameraImpl::hasGamma));
  
    getEventDispatcher().registerEvent(ICamera::hasShutter_event, new NMM::TEDObject0<ICameraImpl, nmm_bool(ICameraImpl:: * )() const, nmm_bool >(this, &ICameraImpl::hasShutter));
  
    getEventDispatcher().registerEvent(ICamera::hasGain_event, new NMM::TEDObject0<ICameraImpl, nmm_bool(ICameraImpl:: * )() const, nmm_bool >(this, &ICameraImpl::hasGain));
  
    getEventDispatcher().registerEvent(ICamera::hasIris_event, new NMM::TEDObject0<ICameraImpl, nmm_bool(ICameraImpl:: * )() const, nmm_bool >(this, &ICameraImpl::hasIris));
  
    getEventDispatcher().registerEvent(ICamera::hasFocus_event, new NMM::TEDObject0<ICameraImpl, nmm_bool(ICameraImpl:: * )() const, nmm_bool >(this, &ICameraImpl::hasFocus));
  
    getEventDispatcher().registerEvent(ICamera::hasZoom_event, new NMM::TEDObject0<ICameraImpl, nmm_bool(ICameraImpl:: * )() const, nmm_bool >(this, &ICameraImpl::hasZoom));

  }

}


#include "nmm/nmm_suffix.hpp"

