/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ICameraImpl_HPP
#define NMM_ICameraImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "ICamera.hpp"


#include "visibility.hpp"

namespace NMM
{

  class DLLEXPORT ICameraImpl  : public virtual NMM::NMMObject { 
  public:
    ICameraImpl() { addInterface(new ICamera(*this)); registerEvents(); };

    virtual ~ICameraImpl() {};

  protected:

    virtual void setBrightness(const nmm_int32& value) = 0;

    virtual void setAutoExposure(const nmm_int32& value) = 0;

    virtual void setSharpness(const nmm_int32& value) = 0;

    virtual void setWhiteBalance(const nmm_int32& value) = 0;

    virtual void setBlueBalance(const nmm_int32& value) = 0;

    virtual void setRedBalance(const nmm_int32& value) = 0;

    virtual void setHue(const nmm_int32& value) = 0;

    virtual void setSaturation(const nmm_int32& value) = 0;

    virtual void setGamma(const nmm_int32& value) = 0;

    virtual void setShutter(const nmm_int32& value) = 0;

    virtual void setGain(const nmm_int32& value) = 0;

    virtual void setIris(const nmm_int32& value) = 0;

    virtual void setFocus(const nmm_int32& value) = 0;

    virtual void setZoom(const nmm_int32& value) = 0;

    virtual void onePush() = 0;

    virtual nmm_int32 getBrightness() const = 0;

    virtual nmm_int32 getAutoExposure() const = 0;

    virtual nmm_int32 getSharpness() const = 0;

    virtual nmm_int32 getWhiteBalance() const = 0;

    virtual nmm_int32 getBlueBalance() const = 0;

    virtual nmm_int32 getRedBalance() const = 0;

    virtual nmm_int32 getHue() const = 0;

    virtual nmm_int32 getSaturation() const = 0;

    virtual nmm_int32 getGamma() const = 0;

    virtual nmm_int32 getShutter() const = 0;

    virtual nmm_int32 getGain() const = 0;

    virtual nmm_int32 getIris() const = 0;

    virtual nmm_int32 getFocus() const = 0;

    virtual nmm_int32 getZoom() const = 0;

    virtual pair<nmm_int32, nmm_int32 >  getBrightnessRange() const = 0;

    virtual pair<nmm_int32, nmm_int32 >  getAutoExposureRange() const = 0;

    virtual pair<nmm_int32, nmm_int32 >  getSharpnessRange() const = 0;

    virtual pair<nmm_int32, nmm_int32 >  getWhiteBalanceRange() const = 0;

    virtual pair<nmm_int32, nmm_int32 >  getHueRange() const = 0;

    virtual pair<nmm_int32, nmm_int32 >  getSaturationRange() const = 0;

    virtual pair<nmm_int32, nmm_int32 >  getGammaRange() const = 0;

    virtual pair<nmm_int32, nmm_int32 >  getShutterRange() const = 0;

    virtual pair<nmm_int32, nmm_int32 >  getGainRange() const = 0;

    virtual pair<nmm_int32, nmm_int32 >  getIrisRange() const = 0;

    virtual pair<nmm_int32, nmm_int32 >  getFocusRange() const = 0;

    virtual pair<nmm_int32, nmm_int32 >  getZoomRange() const = 0;

    virtual nmm_bool hasBrightness() const = 0;

    virtual nmm_bool hasAutoExposure() const = 0;

    virtual nmm_bool hasSharpness() const = 0;

    virtual nmm_bool hasWhiteBalance() const = 0;

    virtual nmm_bool hasHue() const = 0;

    virtual nmm_bool hasSaturation() const = 0;

    virtual nmm_bool hasGamma() const = 0;

    virtual nmm_bool hasShutter() const = 0;

    virtual nmm_bool hasGain() const = 0;

    virtual nmm_bool hasIris() const = 0;

    virtual nmm_bool hasFocus() const = 0;

    virtual nmm_bool hasZoom() const = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
