/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IVISCACameraImpl.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/messaging/TEDObject.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  
  void IVISCACameraImpl::registerEvents()
  {
  
    getEventDispatcher().registerEvent(IVISCACamera::IF_Clear_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::IF_Clear));
  
    getEventDispatcher().registerEvent(IVISCACamera::AddressSet_event, new NMM::TEDObject0<IVISCACameraImpl, Result(IVISCACameraImpl:: * )(), Result >(this, &IVISCACameraImpl::AddressSet));
  
    getEventDispatcher().registerEvent(IVISCACamera::getNumberOfCameras_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(nmm_uchar&), Result >(this, &IVISCACameraImpl::getNumberOfCameras));
  
    getEventDispatcher().registerEvent(IVISCACamera::CommandCancel_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::CommandCancel));
  
    getEventDispatcher().registerEvent(IVISCACamera::PowerOn_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::PowerOn));
  
    getEventDispatcher().registerEvent(IVISCACamera::PowerOff_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::PowerOff));
  
    getEventDispatcher().registerEvent(IVISCACamera::AutoPowerOff_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_int32, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_int32&), Result >(this, &IVISCACameraImpl::AutoPowerOff));
  
    getEventDispatcher().registerEvent(IVISCACamera::ZoomStop_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::ZoomStop));
  
    getEventDispatcher().registerEvent(IVISCACamera::ZoomTeleStd_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::ZoomTeleStd));
  
    getEventDispatcher().registerEvent(IVISCACamera::ZoomWideStd_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::ZoomWideStd));
  
    getEventDispatcher().registerEvent(IVISCACamera::ZoomTeleVar_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::ZoomTeleVar));
  
    getEventDispatcher().registerEvent(IVISCACamera::ZoomWideVar_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::ZoomWideVar));
  
    getEventDispatcher().registerEvent(IVISCACamera::ZoomDirect_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_int32, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_int32&), Result >(this, &IVISCACameraImpl::ZoomDirect));
  
    getEventDispatcher().registerEvent(IVISCACamera::ZoomDigitalZoomOnOff_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_bool, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_bool&), Result >(this, &IVISCACameraImpl::ZoomDigitalZoomOnOff));
  
    getEventDispatcher().registerEvent(IVISCACamera::FocusStop_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::FocusStop));
  
    getEventDispatcher().registerEvent(IVISCACamera::FocusFarStd_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::FocusFarStd));
  
    getEventDispatcher().registerEvent(IVISCACamera::FocusNearStd_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::FocusNearStd));
  
    getEventDispatcher().registerEvent(IVISCACamera::FocusFarVar_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::FocusFarVar));
  
    getEventDispatcher().registerEvent(IVISCACamera::FocusNearVar_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::FocusNearVar));
  
    getEventDispatcher().registerEvent(IVISCACamera::FocusDirect_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uint16, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uint16&), Result >(this, &IVISCACameraImpl::FocusDirect));
  
    getEventDispatcher().registerEvent(IVISCACamera::FocusAutoFocus_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::FocusAutoFocus));
  
    getEventDispatcher().registerEvent(IVISCACamera::FocusManualFocus_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::FocusManualFocus));
  
    getEventDispatcher().registerEvent(IVISCACamera::FocusAutoManual_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::FocusAutoManual));
  
    getEventDispatcher().registerEvent(IVISCACamera::FocusOnePushTrigger_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::FocusOnePushTrigger));
  
    getEventDispatcher().registerEvent(IVISCACamera::FocusInfinity_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::FocusInfinity));
  
    getEventDispatcher().registerEvent(IVISCACamera::FocusAFSensHigh_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::FocusAFSensHigh));
  
    getEventDispatcher().registerEvent(IVISCACamera::FocusAFSensLow_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::FocusAFSensLow));
  
    getEventDispatcher().registerEvent(IVISCACamera::FocusNearLimit_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uint16, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uint16&), Result >(this, &IVISCACameraImpl::FocusNearLimit));
  
    getEventDispatcher().registerEvent(IVISCACamera::WhiteBalance_event, new NMM::TEDObject3<IVISCACameraImpl, nmm_uchar, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::WhiteBalance));
  
    getEventDispatcher().registerEvent(IVISCACamera::WhiteBalanceAuto_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::WhiteBalanceAuto));
  
    getEventDispatcher().registerEvent(IVISCACamera::WhiteBalanceIndoor_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::WhiteBalanceIndoor));
  
    getEventDispatcher().registerEvent(IVISCACamera::WhiteBalanceOutdoor_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::WhiteBalanceOutdoor));
  
    getEventDispatcher().registerEvent(IVISCACamera::WhiteBalanceOnePush_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::WhiteBalanceOnePush));
  
    getEventDispatcher().registerEvent(IVISCACamera::WhiteBalanceAT_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::WhiteBalanceAT));
  
    getEventDispatcher().registerEvent(IVISCACamera::WhiteBalanceManual_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::WhiteBalanceManual));
  
    getEventDispatcher().registerEvent(IVISCACamera::WhiteBalanceOnePushTrigger_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::WhiteBalanceOnePushTrigger));
  
    getEventDispatcher().registerEvent(IVISCACamera::RGainReset_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::RGainReset));
  
    getEventDispatcher().registerEvent(IVISCACamera::RGainUp_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::RGainUp));
  
    getEventDispatcher().registerEvent(IVISCACamera::RGainDown_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::RGainDown));
  
    getEventDispatcher().registerEvent(IVISCACamera::RGainDirect_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uint16, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uint16&), Result >(this, &IVISCACameraImpl::RGainDirect));
  
    getEventDispatcher().registerEvent(IVISCACamera::BGainReset_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::BGainReset));
  
    getEventDispatcher().registerEvent(IVISCACamera::BGainUp_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::BGainUp));
  
    getEventDispatcher().registerEvent(IVISCACamera::BGainDown_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::BGainDown));
  
    getEventDispatcher().registerEvent(IVISCACamera::BGainDirect_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uint16, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uint16&), Result >(this, &IVISCACameraImpl::BGainDirect));
  
    getEventDispatcher().registerEvent(IVISCACamera::AutomaticExposureFull_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::AutomaticExposureFull));
  
    getEventDispatcher().registerEvent(IVISCACamera::AutomaticExposureManual_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::AutomaticExposureManual));
  
    getEventDispatcher().registerEvent(IVISCACamera::AutomaticExposureShutterPriority_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::AutomaticExposureShutterPriority));
  
    getEventDispatcher().registerEvent(IVISCACamera::AutomaticExposureIrisPriority_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::AutomaticExposureIrisPriority));
  
    getEventDispatcher().registerEvent(IVISCACamera::AutomaticExposureGainPriority_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::AutomaticExposureGainPriority));
  
    getEventDispatcher().registerEvent(IVISCACamera::AutomaticExposureBright_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::AutomaticExposureBright));
  
    getEventDispatcher().registerEvent(IVISCACamera::AutomaticExposureShutterAuto_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::AutomaticExposureShutterAuto));
  
    getEventDispatcher().registerEvent(IVISCACamera::AutomaticExposureIrisAuto_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::AutomaticExposureIrisAuto));
  
    getEventDispatcher().registerEvent(IVISCACamera::AutomaticExposureGainAuto_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::AutomaticExposureGainAuto));
  
    getEventDispatcher().registerEvent(IVISCACamera::SlowShutter_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_bool, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_bool&), Result >(this, &IVISCACameraImpl::SlowShutter));
  
    getEventDispatcher().registerEvent(IVISCACamera::ShutterReset_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::ShutterReset));
  
    getEventDispatcher().registerEvent(IVISCACamera::ShutterUp_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::ShutterUp));
  
    getEventDispatcher().registerEvent(IVISCACamera::ShutterDown_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::ShutterDown));
  
    getEventDispatcher().registerEvent(IVISCACamera::ShutterDirect_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uint16, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uint16&), Result >(this, &IVISCACameraImpl::ShutterDirect));
  
    getEventDispatcher().registerEvent(IVISCACamera::IrisReset_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::IrisReset));
  
    getEventDispatcher().registerEvent(IVISCACamera::IrisUp_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::IrisUp));
  
    getEventDispatcher().registerEvent(IVISCACamera::IrisDown_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::IrisDown));
  
    getEventDispatcher().registerEvent(IVISCACamera::IrisDirect_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uint16, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uint16&), Result >(this, &IVISCACameraImpl::IrisDirect));
  
    getEventDispatcher().registerEvent(IVISCACamera::GainReset_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::GainReset));
  
    getEventDispatcher().registerEvent(IVISCACamera::GainUp_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::GainUp));
  
    getEventDispatcher().registerEvent(IVISCACamera::GainDown_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::GainDown));
  
    getEventDispatcher().registerEvent(IVISCACamera::GainDirect_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uint16, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uint16&), Result >(this, &IVISCACameraImpl::GainDirect));
  
    getEventDispatcher().registerEvent(IVISCACamera::BrightReset_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::BrightReset));
  
    getEventDispatcher().registerEvent(IVISCACamera::BrightUp_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::BrightUp));
  
    getEventDispatcher().registerEvent(IVISCACamera::BrightDown_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::BrightDown));
  
    getEventDispatcher().registerEvent(IVISCACamera::BrightDirect_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uint16, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uint16&), Result >(this, &IVISCACameraImpl::BrightDirect));
  
    getEventDispatcher().registerEvent(IVISCACamera::ExposureCompensationOnOff_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_bool, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_bool&), Result >(this, &IVISCACameraImpl::ExposureCompensationOnOff));
  
    getEventDispatcher().registerEvent(IVISCACamera::ExposureCompensationReset_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::ExposureCompensationReset));
  
    getEventDispatcher().registerEvent(IVISCACamera::ExposureCompensationUp_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::ExposureCompensationUp));
  
    getEventDispatcher().registerEvent(IVISCACamera::ExposureCompensationDown_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::ExposureCompensationDown));
  
    getEventDispatcher().registerEvent(IVISCACamera::ExposureCompensationDirect_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uint16, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uint16&), Result >(this, &IVISCACameraImpl::ExposureCompensationDirect));
  
    getEventDispatcher().registerEvent(IVISCACamera::BackLightOnOff_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_bool, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_bool&), Result >(this, &IVISCACameraImpl::BackLightOnOff));
  
    getEventDispatcher().registerEvent(IVISCACamera::ApertureReset_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::ApertureReset));
  
    getEventDispatcher().registerEvent(IVISCACamera::ApertureUp_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::ApertureUp));
  
    getEventDispatcher().registerEvent(IVISCACamera::ApertureDown_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::ApertureDown));
  
    getEventDispatcher().registerEvent(IVISCACamera::ApertureDirect_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uint16, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uint16&), Result >(this, &IVISCACameraImpl::ApertureDirect));
  
    getEventDispatcher().registerEvent(IVISCACamera::WideModeOff_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::WideModeOff));
  
    getEventDispatcher().registerEvent(IVISCACamera::WideModeCinema_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::WideModeCinema));
  
    getEventDispatcher().registerEvent(IVISCACamera::WideMode16to9Full_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::WideMode16to9Full));
  
    getEventDispatcher().registerEvent(IVISCACamera::LR_Reverse_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_bool, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_bool&), Result >(this, &IVISCACameraImpl::LR_Reverse));
  
    getEventDispatcher().registerEvent(IVISCACamera::Freeze_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_bool, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_bool&), Result >(this, &IVISCACameraImpl::Freeze));
  
    getEventDispatcher().registerEvent(IVISCACamera::PictureEffectOff_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::PictureEffectOff));
  
    getEventDispatcher().registerEvent(IVISCACamera::PictureEffectPastel_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::PictureEffectPastel));
  
    getEventDispatcher().registerEvent(IVISCACamera::PictureEffectNegArt_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::PictureEffectNegArt));
  
    getEventDispatcher().registerEvent(IVISCACamera::PictureEffectSepia_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::PictureEffectSepia));
  
    getEventDispatcher().registerEvent(IVISCACamera::PictureEffectBW_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::PictureEffectBW));
  
    getEventDispatcher().registerEvent(IVISCACamera::PictureEffectSolarize_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::PictureEffectSolarize));
  
    getEventDispatcher().registerEvent(IVISCACamera::PictureEffectMosaic_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::PictureEffectMosaic));
  
    getEventDispatcher().registerEvent(IVISCACamera::PictureEffectSlim_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::PictureEffectSlim));
  
    getEventDispatcher().registerEvent(IVISCACamera::PictureEffectStretch_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::PictureEffectStretch));
  
    getEventDispatcher().registerEvent(IVISCACamera::DigitalEffectOff_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::DigitalEffectOff));
  
    getEventDispatcher().registerEvent(IVISCACamera::DigitalEffectStill_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::DigitalEffectStill));
  
    getEventDispatcher().registerEvent(IVISCACamera::DigitalEffectFlash_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::DigitalEffectFlash));
  
    getEventDispatcher().registerEvent(IVISCACamera::DigitalEffectLumi_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::DigitalEffectLumi));
  
    getEventDispatcher().registerEvent(IVISCACamera::DigitalEffectTrail_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::DigitalEffectTrail));
  
    getEventDispatcher().registerEvent(IVISCACamera::DigitalEffectLevel_event, new NMM::TEDObject3<IVISCACameraImpl, nmm_uchar, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::DigitalEffectLevel));
  
    getEventDispatcher().registerEvent(IVISCACamera::Memory_event, new NMM::TEDObject3<IVISCACameraImpl, nmm_uchar, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::Memory));
  
    getEventDispatcher().registerEvent(IVISCACamera::MemoryReset_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::MemoryReset));
  
    getEventDispatcher().registerEvent(IVISCACamera::MemorySet_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::MemorySet));
  
    getEventDispatcher().registerEvent(IVISCACamera::MemoryRecall_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::MemoryRecall));
  
    getEventDispatcher().registerEvent(IVISCACamera::DataScreenOnOff_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_bool, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_bool&), Result >(this, &IVISCACameraImpl::DataScreenOnOff));
  
    getEventDispatcher().registerEvent(IVISCACamera::IRReceiveOnOff_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_bool, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_bool&), Result >(this, &IVISCACameraImpl::IRReceiveOnOff));
  
    getEventDispatcher().registerEvent(IVISCACamera::IRReceiveReturn_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_bool, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_bool&), Result >(this, &IVISCACameraImpl::IRReceiveReturn));
  
    getEventDispatcher().registerEvent(IVISCACamera::PanTiltDriveMotion_event, new NMM::TEDObject5<IVISCACameraImpl, nmm_uchar, nmm_uchar, nmm_uchar, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uchar&, const nmm_uchar&, const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::PanTiltDriveMotion));
  
    getEventDispatcher().registerEvent(IVISCACamera::PanTiltDriveUp_event, new NMM::TEDObject3<IVISCACameraImpl, nmm_uchar, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::PanTiltDriveUp));
  
    getEventDispatcher().registerEvent(IVISCACamera::PanTiltDriveDown_event, new NMM::TEDObject3<IVISCACameraImpl, nmm_uchar, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::PanTiltDriveDown));
  
    getEventDispatcher().registerEvent(IVISCACamera::PanTiltDriveLeft_event, new NMM::TEDObject3<IVISCACameraImpl, nmm_uchar, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::PanTiltDriveLeft));
  
    getEventDispatcher().registerEvent(IVISCACamera::PanTiltDriveRight_event, new NMM::TEDObject3<IVISCACameraImpl, nmm_uchar, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::PanTiltDriveRight));
  
    getEventDispatcher().registerEvent(IVISCACamera::PanTiltDriveUpLeft_event, new NMM::TEDObject3<IVISCACameraImpl, nmm_uchar, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::PanTiltDriveUpLeft));
  
    getEventDispatcher().registerEvent(IVISCACamera::PanTiltDriveUpRight_event, new NMM::TEDObject3<IVISCACameraImpl, nmm_uchar, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::PanTiltDriveUpRight));
  
    getEventDispatcher().registerEvent(IVISCACamera::PanTiltDriveDownLeft_event, new NMM::TEDObject3<IVISCACameraImpl, nmm_uchar, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::PanTiltDriveDownLeft));
  
    getEventDispatcher().registerEvent(IVISCACamera::PanTiltDriveDownRight_event, new NMM::TEDObject3<IVISCACameraImpl, nmm_uchar, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::PanTiltDriveDownRight));
  
    getEventDispatcher().registerEvent(IVISCACamera::PanTiltDriveStop_event, new NMM::TEDObject3<IVISCACameraImpl, nmm_uchar, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::PanTiltDriveStop));
  
    getEventDispatcher().registerEvent(IVISCACamera::PanTiltDrivePosition_event, new NMM::TEDObject6<IVISCACameraImpl, nmm_uchar, nmm_uchar, nmm_int32, nmm_int32, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uchar&, const nmm_int32&, const nmm_int32&, const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::PanTiltDrivePosition));
  
    getEventDispatcher().registerEvent(IVISCACamera::PanTiltDriveAbsolute_event, new NMM::TEDObject5<IVISCACameraImpl, nmm_uchar, nmm_int32, nmm_int32, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_int32&, const nmm_int32&, const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::PanTiltDriveAbsolute));
  
    getEventDispatcher().registerEvent(IVISCACamera::PanTiltDriveRelative_event, new NMM::TEDObject5<IVISCACameraImpl, nmm_uchar, nmm_int32, nmm_int32, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_int32&, const nmm_int32&, const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::PanTiltDriveRelative));
  
    getEventDispatcher().registerEvent(IVISCACamera::PanTiltDriveAbsoluteDeg_event, new NMM::TEDObject5<IVISCACameraImpl, nmm_uchar, nmm_int32, nmm_int32, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_int32&, const nmm_int32&, const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::PanTiltDriveAbsoluteDeg));
  
    getEventDispatcher().registerEvent(IVISCACamera::PanTiltDriveRelativeDeg_event, new NMM::TEDObject5<IVISCACameraImpl, nmm_uchar, nmm_int32, nmm_int32, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_int32&, const nmm_int32&, const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::PanTiltDriveRelativeDeg));
  
    getEventDispatcher().registerEvent(IVISCACamera::PanTiltDriveHome_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::PanTiltDriveHome));
  
    getEventDispatcher().registerEvent(IVISCACamera::PanTiltDriveReset_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&), Result >(this, &IVISCACameraImpl::PanTiltDriveReset));
  
    getEventDispatcher().registerEvent(IVISCACamera::PanTiltLimitSet_event, new NMM::TEDObject4<IVISCACameraImpl, nmm_uchar, nmm_uchar, nmm_int32, nmm_int32, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uchar&, const nmm_int32&, const nmm_int32&), Result >(this, &IVISCACameraImpl::PanTiltLimitSet));
  
    getEventDispatcher().registerEvent(IVISCACamera::PanTiltLimitClear_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, const nmm_uchar&), Result >(this, &IVISCACameraImpl::PanTiltLimitClear));
  
    getEventDispatcher().registerEvent(IVISCACamera::PowerInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_bool, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_bool&), Result >(this, &IVISCACameraImpl::PowerInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::AutoPowerOffInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uint16, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_uint16&), Result >(this, &IVISCACameraImpl::AutoPowerOffInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::DZoomModeInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_bool, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_bool&), Result >(this, &IVISCACameraImpl::DZoomModeInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::ZoomPosInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uint16, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_uint16&), Result >(this, &IVISCACameraImpl::ZoomPosInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::FocusModeInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_bool, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_bool&), Result >(this, &IVISCACameraImpl::FocusModeInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::FocusPosInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uint16, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_uint16&), Result >(this, &IVISCACameraImpl::FocusPosInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::AFModeInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_bool, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_bool&), Result >(this, &IVISCACameraImpl::AFModeInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::FocusNearLimitInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uint16, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_uint16&), Result >(this, &IVISCACameraImpl::FocusNearLimitInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::WhiteBalanceModeInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_uchar&), Result >(this, &IVISCACameraImpl::WhiteBalanceModeInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::RGainInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_uchar&), Result >(this, &IVISCACameraImpl::RGainInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::BGainInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uint16, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_uint16&), Result >(this, &IVISCACameraImpl::BGainInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::AutoExposureModeInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_uchar&), Result >(this, &IVISCACameraImpl::AutoExposureModeInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::SlowShutterModeInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_bool, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_bool&), Result >(this, &IVISCACameraImpl::SlowShutterModeInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::ShutterPosInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uint16, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_uint16&), Result >(this, &IVISCACameraImpl::ShutterPosInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::IrisPosInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uint16, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_uint16&), Result >(this, &IVISCACameraImpl::IrisPosInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::GainPosInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uint16, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_uint16&), Result >(this, &IVISCACameraImpl::GainPosInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::BrightPosInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uint16, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_uint16&), Result >(this, &IVISCACameraImpl::BrightPosInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::ExposureCompensationModeInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_bool, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_bool&), Result >(this, &IVISCACameraImpl::ExposureCompensationModeInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::ExposureCompensationPosInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uint16, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_uint16&), Result >(this, &IVISCACameraImpl::ExposureCompensationPosInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::BackLightModeInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_bool, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_bool&), Result >(this, &IVISCACameraImpl::BackLightModeInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::ApertureInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uint16, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_uint16&), Result >(this, &IVISCACameraImpl::ApertureInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::WideModeInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_uchar&), Result >(this, &IVISCACameraImpl::WideModeInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::LR_ReverseModeInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_bool, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_bool&), Result >(this, &IVISCACameraImpl::LR_ReverseModeInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::FreezeModeInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_bool, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_bool&), Result >(this, &IVISCACameraImpl::FreezeModeInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::PictureEffectModeInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_uchar&), Result >(this, &IVISCACameraImpl::PictureEffectModeInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::DigitalEffectModeInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_uchar&), Result >(this, &IVISCACameraImpl::DigitalEffectModeInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::DigitalEffectLevelInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_uchar&), Result >(this, &IVISCACameraImpl::DigitalEffectLevelInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::MemoryInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_uchar&), Result >(this, &IVISCACameraImpl::MemoryInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::DatascreenInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_bool, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_bool&), Result >(this, &IVISCACameraImpl::DatascreenInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::PanTiltModeInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uint16, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_uint16&), Result >(this, &IVISCACameraImpl::PanTiltModeInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::PanTiltMaxSpeedInq_event, new NMM::TEDObject3<IVISCACameraImpl, nmm_uchar, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_uchar&, nmm_uchar&), Result >(this, &IVISCACameraImpl::PanTiltMaxSpeedInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::PanTiltPosInq_event, new NMM::TEDObject3<IVISCACameraImpl, nmm_uchar, nmm_int32, nmm_int32, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_int32&, nmm_int32&), Result >(this, &IVISCACameraImpl::PanTiltPosInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::VideoSystemInq_event, new NMM::TEDObject2<IVISCACameraImpl, nmm_uchar, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_uchar&), Result >(this, &IVISCACameraImpl::VideoSystemInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::DeviceTypeInq_event, new NMM::TEDObject5<IVISCACameraImpl, nmm_uchar, nmm_uint16, nmm_uint16, nmm_uint16, nmm_uchar, Result(IVISCACameraImpl:: * )(const nmm_uchar&, nmm_uint16&, nmm_uint16&, nmm_uint16&, nmm_uchar&), Result >(this, &IVISCACameraImpl::DeviceTypeInq));
  
    getEventDispatcher().registerEvent(IVISCACamera::verbose_event, new NMM::TEDObject2<IVISCACameraImpl, string, string, void(IVISCACameraImpl:: * )(const string&, const string&), void >(this, &IVISCACameraImpl::verbose));
  
    getEventDispatcher().registerEvent(IVISCACamera::islocked_event, new NMM::TEDObject1<IVISCACameraImpl, nmm_uchar, nmm_bool(IVISCACameraImpl:: * )(const nmm_uchar&), nmm_bool >(this, &IVISCACameraImpl::islocked));

  }

}


#include "nmm/nmm_suffix.hpp"

