/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef _NMM_FramerateConverterNode_HPP_
#define _NMM_FramerateConverterNode_HPP_

#include "nmm/nmm_prefix.hpp"

#include "nmm/plugins/general/IBufferDropImpl.hpp"
#include "nmm/multimedia/GenericProcessorNode.hpp"
#include "nmm/utils/time/Rational.hpp"
#include "nmm/utils/time/Interval.hpp"
#include "nmm/utils/thread/ThreadMutex.hpp"

namespace NMM {

  /**
   * \class FramerateConverterNode
   */
  class FramerateConverterNode :
    public GenericProcessorNode,
    public IBufferDropImpl
  {
  private:
    unsigned int buffer_to_drop;
    unsigned int buffer_count;
    
    // for setDropRatio
    unsigned int drop_num;
    unsigned int drop_denom;
    Rational drop_ratio;
    Rational ratio_counter;
    bool notify_framerate_changed;
    bool changed_framerate;
    
    Interval adjust_timestamp_value;
    bool first_timestamp;
    Timestamp current_timestamp;
    ThreadMutex internal_mutex;
    Buffer* stored_buffer;

  protected:
    // IBufferDrop interface
    void setBufferToDrop(const nmm_uint32 &buffer_num);
    void setDropRatio(const nmm_uint32& nominator, const nmm_uint32& denominator);

    /**
     * Here all formats are specified.
     */
    virtual Result doInit();
    
    /**
     * Here all output streams are added.
     */
    virtual Result doInitOutput();
    
    /**
     * Set drop ratio based on input and output format
     */
    virtual Result doActivate();

    /**
     * The processBuffer method from superclass
     */
    virtual Message *processBuffer (Buffer *in);

  public:

    /**
     * The main constructor.
     */
    FramerateConverterNode(const char *name = "FramerateConverterNode", StreamQueue::Mode = StreamQueue::MODE_SUSPEND, int qsize = 5);
  };

}

#include "nmm/nmm_suffix.hpp"

#endif //_NMM_VIDEO_FILTER_OVERLAYNODE_HPP_



