/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ICrop.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ICrop::used() {} 
  
  
  void ICrop::setCrop(const nmm_int32& xOffset, const nmm_int32& yOffset, const nmm_int32& xRange, const nmm_int32& yRange) /* required(INode::INITIALIZED, INode::OUTPUT_INITIALIZED) */ /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_int32 > xOffset_value(xOffset, false);
    NMM::TInValue<nmm_int32 > yOffset_value(yOffset, false);
    NMM::TInValue<nmm_int32 > xRange_value(xRange, false);
    NMM::TInValue<nmm_int32 > yRange_value(yRange, false);
    NMM::Event event("ICrop::setCrop", &xOffset_value, &yOffset_value, &xRange_value, &yRange_value);
    if(const_cast<ICrop*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* ICrop::ICrop_type = "ICrop";
  NMM::TConstructor<ICrop > __ICrop_prototype("ICrop","NMM");


  const char* ICrop::setCrop_event = "ICrop::setCrop";


}


#include "nmm/nmm_suffix.hpp"

