/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IScaler.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IScaler::used() {} 
  
  
  void IScaler::setAutoAspect(const nmm_bool& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > value_value(value, false);
    NMM::Event event("IScaler::setAutoAspect", &value_value);
    if(const_cast<IScaler*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IScaler::setFiltering(const nmm_bool& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > value_value(value, false);
    NMM::Event event("IScaler::setFiltering", &value_value);
    if(const_cast<IScaler*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IScaler::setOutputSize(const nmm_int32& w, const nmm_int32& h)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > w_value(w, false);
    NMM::TInValue<nmm_int32 > h_value(h, false);
    NMM::Event event("IScaler::setOutputSize", &w_value, &h_value);
    if(const_cast<IScaler*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IScaler::getOutputSize(nmm_int32& w, nmm_int32& h) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > w_value(w);
    NMM::TOutValue<nmm_int32 > h_value(h);
    NMM::Event event("IScaler::getOutputSize", &w_value, &h_value);
    if(const_cast<IScaler*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    w = w_value.getRValue();
    h = h_value.getRValue();
  }
  
  nmm_bool IScaler::getAutoAspect() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("IScaler::getAutoAspect");
    event.setReturnValue(&return_value);
    if(const_cast<IScaler*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool IScaler::getFiltering() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("IScaler::getFiltering");
    event.setReturnValue(&return_value);
    if(const_cast<IScaler*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IScaler::IScaler_type = "IScaler";
  NMM::TConstructor<IScaler > __IScaler_prototype("IScaler","NMM");


  const char* IScaler::setAutoAspect_event = "IScaler::setAutoAspect";
  const char* IScaler::setFiltering_event = "IScaler::setFiltering";
  const char* IScaler::setOutputSize_event = "IScaler::setOutputSize";
  const char* IScaler::getOutputSize_event = "IScaler::getOutputSize";
  const char* IScaler::getAutoAspect_event = "IScaler::getAutoAspect";
  const char* IScaler::getFiltering_event = "IScaler::getFiltering";


}


#include "nmm/nmm_suffix.hpp"

