/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IScaler_HPP
#define NMM_IScaler_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{

  class DLLEXPORT IScaler : virtual public NMM::Interface { 
  public:
    IScaler() : NMM::Interface(IScaler::IScaler_type, "NMM") { };

    IScaler(NMM::NMMObject& parent) : NMM::Interface (parent, IScaler::IScaler_type, "NMM") { };

    virtual ~IScaler() {};

    static const char* IScaler_type;

    virtual  string getInterfaceType() const { return IScaler_type; }

    NMM::Interface* copy() const { return new IScaler(*this); }
    static void used(); 
    static const char* setAutoAspect_event;
    static const char* setFiltering_event;
    static const char* setOutputSize_event;
    static const char* getOutputSize_event;
    static const char* getAutoAspect_event;
    static const char* getFiltering_event;

    virtual void setAutoAspect(const nmm_bool& value)
      throw(const NMM::SerializedException&);

    virtual void setFiltering(const nmm_bool& value)
      throw(const NMM::SerializedException&);

    virtual void setOutputSize(const nmm_int32& w, const nmm_int32& h)
      throw(const NMM::SerializedException&);

    virtual void getOutputSize(nmm_int32& w, nmm_int32& h) const
      throw(const NMM::SerializedException&);

    virtual nmm_bool getAutoAspect() const
      throw(const NMM::SerializedException&);

    virtual nmm_bool getFiltering() const
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IScaler> IScaler_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
