/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#include "RGBtoYV12ConverterNode.hpp"

//#include "nmm/utils/utils.hpp"
//#include "nmm/base/format/Property.hpp"
//#include "nmm/base/memorymgr/Buffer.hpp"
//#include "nmm/base/registry/Plugin.hpp"

#include "nmm/plugins/NMMRegisterPlugin.hpp"

NMM_REGISTER_PLUGIN(RGBtoYV12ConverterNode, NMM, "RGBtoYV12ConverterNode")

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  RGBtoYV12ConverterNode::RGBtoYV12ConverterNode( const char * name, StreamQueue::Mode qmode, int qsize)
    : GenericProcessorNode(name, qmode, qsize)
  {
      hasmmx = false;
  }

  Result RGBtoYV12ConverterNode::doInit()
  {

    Format* i_format = getOwnInputProperty()->addNewFormat("video/raw");
    i_format->addWildcard(Format::x_resolution_param);
    i_format->addWildcard(Format::y_resolution_param);
    i_format->addWildcard(Format::ratio_x_param);
    i_format->addWildcard(Format::ratio_y_param);
    i_format->addWildcard(Format::framerate_param);
    i_format->addStringParamValue(Format::channelorder_param,"rgb");
    i_format->addStringParamValue(Format::endian_param, "little");
    //i_format->addStringParamValue(Format::colorspace_param,"rgba32");
    i_format->addStringParamValue(Format::colorspace_param,"rgb24");
    i_format->addIntParamValue(Format::bitperpixel_param,32);
    i_format->addStringParamValue(Format::format_param,"packet");
    getOwnInputProperty()->setDefaultFormat(i_format);

    Format* o_format = getOwnOutputProperty()->addNewFormat("video/raw");
    o_format->addWildcard(Format::x_resolution_param);
    o_format->addWildcard(Format::y_resolution_param);
    o_format->addWildcard(Format::ratio_x_param);
    o_format->addWildcard(Format::ratio_y_param);
    o_format->addWildcard(Format::framerate_param);
    o_format->addStringParamValue(Format::channelorder_param,"yvu");
    o_format->addStringParamValue(Format::endian_param, "little");
    o_format->addStringParamValue(Format::colorspace_param,"yv12");
    o_format->addIntParamValue(Format::bitperpixel_param,12);
    o_format->addStringParamValue(Format::format_param, "planar");
    getOwnOutputProperty()->setDefaultFormat(o_format);
    i_format->setIOPartner(o_format);
    o_format->setIOPartner(i_format);


    i_format = getOwnInputProperty()->addNewFormat("video/raw");
    i_format->addWildcard(Format::x_resolution_param);
    i_format->addWildcard(Format::y_resolution_param);
    i_format->addWildcard(Format::ratio_x_param);
    i_format->addWildcard(Format::ratio_y_param);
    i_format->addWildcard(Format::framerate_param);
    i_format->addStringParamValue(Format::channelorder_param,"rgb");
    i_format->addStringParamValue(Format::endian_param, "little");
    i_format->addStringParamValue(Format::colorspace_param,"rgb24");
    i_format->addIntParamValue(Format::bitperpixel_param,24);
    i_format->addStringParamValue(Format::format_param,"packet");

    o_format = getOwnOutputProperty()->addNewFormat("video/raw");
    o_format->addWildcard(Format::x_resolution_param);
    o_format->addWildcard(Format::y_resolution_param);
    o_format->addWildcard(Format::ratio_x_param);
    o_format->addWildcard(Format::ratio_y_param);
    o_format->addWildcard(Format::framerate_param);
    o_format->addStringParamValue(Format::channelorder_param,"yvu");
    o_format->addStringParamValue(Format::endian_param, "little");
    o_format->addStringParamValue(Format::colorspace_param,"yv12");
    o_format->addIntParamValue(Format::bitperpixel_param,12);
    o_format->addStringParamValue(Format::format_param, "planar");
    i_format->setIOPartner(o_format);
    o_format->setIOPartner(i_format);


    return SUCCESS;
  }

  Result RGBtoYV12ConverterNode::doInitOutput()
  {

    const Format* in_format = getInputFormat();
    if (in_format == NULL) {
      ERROR_STREAM("No Input format"<<endl);
      return FAILURE;
    }

    int bits = in_format->getIntValue( Format::bitperpixel_param );
    if( bits == 32 )
        __format = RGB32;
    else if( bits == 24 )
        __format = RGB24;
    else if( bits == 16 )
        __format = RGB565;
    else
        return FAILURE;

    Format* i_format = new Format( *in_format );
    getOwnWorkingInputProperty()->addNewFormat(i_format);
    getOwnWorkingInputProperty()->setDefaultFormat(i_format);

    Format* o_format = getOwnWorkingOutputProperty()->addNewFormat("video/raw");
    o_format->addIntParamValue(Format::x_resolution_param,in_format->getIntValue(Format::x_resolution_param));
    o_format->addIntParamValue(Format::y_resolution_param,in_format->getIntValue(Format::y_resolution_param));
    o_format->addIntParamValue(Format::ratio_x_param,in_format->getIntValue(Format::ratio_x_param));
    o_format->addIntParamValue(Format::ratio_y_param,in_format->getIntValue(Format::ratio_y_param));
    o_format->addFloatParamValue(Format::framerate_param,in_format->getFloatValue(Format::framerate_param));
    o_format->addStringParamValue(Format::channelorder_param,"yvu");
    o_format->addStringParamValue(Format::endian_param, "little");
    o_format->addStringParamValue(Format::colorspace_param,"yv12");
    o_format->addIntParamValue(Format::bitperpixel_param,12);
    o_format->addStringParamValue(Format::format_param, "planar");
    getOwnWorkingOutputProperty()->setDefaultFormat(o_format);

    i_format->setIOPartner(o_format);
    o_format->setIOPartner(i_format);

    return SUCCESS;
  }

  Result RGBtoYV12ConverterNode::doActivate()
  {

    const Format *out_f = getOutputFormat();
    if (out_f == NULL) {
      return FAILURE;
    }

    width = out_f->getIntValue(Format::x_resolution_param);
    height = out_f->getIntValue(Format::y_resolution_param);

    if ((width <= 0) || (height <= 0)) {
       return FAILURE;
    }

    return SUCCESS;
  }

  Result RGBtoYV12ConverterNode::doSetFormat(const Format& format)
  {

    DEBUG_STREAM("RGBtoYV12ConverterNode::doSetFormat" << endl);
    int res_x = format.getIntValue(Format::x_resolution_param);
    int res_y = format.getIntValue(Format::y_resolution_param);

    int ratio_x = format.getIntValue(Format::ratio_x_param);
    int ratio_y = format.getIntValue(Format::ratio_y_param);
    DEBUG_STREAM("SetFormat: " << res_x << " x " << res_y << ", ratio " << ratio_x << ":" << ratio_y << endl);
    return SUCCESS;
  }


void RGBtoYV12ConverterNode::rgb2yuv( unsigned char R, unsigned char G, unsigned char B, double *Y, double *U, double *V ) {
  *Y = R *  0.299 + G *  0.587 + B *  0.114;
  *U = R * -0.169 + G * -0.332 + B *  0.500 + 128;
  *V = R *  0.500 + G * -0.419 + B * -0.0813 + 128;
}

void RGBtoYV12ConverterNode::rgb32_to_YV12( unsigned char *rgb_image, unsigned char *yuv_image ) {

{
  
    int px, py, x, y, x_offset, y_offset;
    int v,w, i,j, r,g,b;
    int blocks_per_row, blocks_per_line;
    double u_block[2][2];
    double v_block[2][2];

    int r1,g1,b1;

    double t, u1, v1;

    unsigned char *y_plane;
    unsigned char *u_plane;
    unsigned char *v_plane;
    int plane_size;
    
    plane_size = width*height;
    
    y_plane = yuv_image;
    u_plane = yuv_image + plane_size;
    v_plane = yuv_image + plane_size + plane_size/4;
    
    blocks_per_line = width/2;
    if ( blocks_per_line%2 ) blocks_per_line++;
    
    blocks_per_row = height/2;
    if ( blocks_per_row%2 ) blocks_per_row++;
    
    x = y = x_offset = y_offset = 0;
    
    for( v=0; v < blocks_per_row ; v++) {
      for( w=0; w < blocks_per_line; w++ ) {
	
	y = y_offset;
	
	py = y/2;
	px = x/8;
	
	r1 = g1 = b1 = 0;
	
	// 2x2 Block konvertieren
	for( j=0; j<2; j++ ) {
	  x  = x_offset;
	  
	  for ( i=0; i<2; i++ ) {
	    r = rgb_image[x+0 + y*width*4];
	    g = rgb_image[x+1 + y*width*4];
	    b = rgb_image[x+2 + y*width*4];
	    
	    //rgb2yuv( r, g, b, &t, u_block[i,j], v_block[i,j] );
	    rgb2yuv( r, g, b, &t, u_block[i], v_block[i] );
	    r1 += r;
	    g1 += g;
	    b1 += b;
	    
	    y_plane[(x/4)+y*width] = (int) t;
	    x+=4;
	  } // for i
	  y++;
	} // for j
	// End 2x2 Block
	
	r1 = r1 /4;
	g1 = g1 /4;
	b1 = b1 /4;
	
	rgb2yuv( r1, g1, b1, &t, &u1, &v1 );
	u_plane[px + py*width/2] = (unsigned char) u1;
	v_plane[px + py*width/2] = (unsigned char) v1;
	
	x_offset = x;
      } // for w
      
      x_offset = 0;
      y_offset = y;
    } // for v
  }
}

void RGBtoYV12ConverterNode::rgb24_to_YV12( unsigned char *rgb_image, unsigned char *yuv_image ) {

{
  
    int px, py, x, y, x_offset, y_offset;
    int v,w, i,j, r,g,b;
    int blocks_per_row, blocks_per_line;
    double u_block[2][2];
    double v_block[2][2];
    
    int r1,g1,b1;
    
    double t, u1, v1;
    
    unsigned char *y_plane;
    unsigned char *u_plane;
    unsigned char *v_plane;
    int plane_size;
    
    plane_size = width*height;
    
    y_plane = yuv_image;
    u_plane = yuv_image + plane_size;
    v_plane = yuv_image + plane_size + plane_size/4;
    
    blocks_per_line = width/2;
    if ( blocks_per_line%2 ) blocks_per_line++;
    
    blocks_per_row = height/2;
    if ( blocks_per_row%2 ) blocks_per_row++;
    
    x = y = x_offset = y_offset = 0;
    
    for( v=0; v < blocks_per_row ; v++) {
      for( w=0; w < blocks_per_line; w++ ) {
	
	y = y_offset;
	
	py = y/2;
	px = x/6;
	
	r1 = g1 = b1 = 0;
	
	// 2x2 Block konvertieren
	for( j=0; j<2; j++ ) {
	  x  = x_offset;
	  
	  for ( i=0; i<2; i++ ) {
	    r = rgb_image[x+0 + y*width*3];
	    g = rgb_image[x+1 + y*width*3];
	    b = rgb_image[x+2 + y*width*3];
	    
	    //rgb2yuv( r, g, b, &t, u_block[i,j], v_block[i,j] );
	    rgb2yuv( r, g, b, &t, u_block[i], v_block[i] );
	    r1 += r;
	    g1 += g;
	    b1 += b;
	    
	    y_plane[(x/3)+y*width] = (int) t;
	    x+=3;
	  } // for i
	  y++;
	} // for j
	// End 2x2 Block
	
	r1 = r1 /4;
	g1 = g1 /4;
	b1 = b1 /4;
	
	rgb2yuv( r1, g1, b1, &t, &u1, &v1 );
	u_plane[px + py*width/2] = (unsigned char) u1;
	v_plane[px + py*width/2] = (unsigned char) v1;
	
	x_offset = x;
      } // for w
      
      x_offset = 0;
      y_offset = y;
    } // for v
  }
}

void RGBtoYV12ConverterNode::rgb565_to_YV12( unsigned char *rgb_image, unsigned char *yuv_image ) {

{ 
    int px, py, x, y, x_offset, y_offset;
    int v,w, i,j, r,g,b;
    int blocks_per_row, blocks_per_line;
    double u_block[2][2];
    double v_block[2][2];
    
    int r1,g1,b1;
    
    double t, u1, v1;
    
    unsigned char *y_plane;
    unsigned char *u_plane;
    unsigned char *v_plane;
    int plane_size;
    
    plane_size = width*height;
    
    y_plane = yuv_image;
    u_plane = yuv_image + plane_size;
    v_plane = yuv_image + plane_size + plane_size/4;
    
    blocks_per_line = width/2;
    if ( blocks_per_line%2 ) blocks_per_line++;
    
    blocks_per_row = height/2;
    if ( blocks_per_row%2 ) blocks_per_row++;
    
    x = y = x_offset = y_offset = 0;
    
    for( v=0; v < blocks_per_row ; v++) {
      for( w=0; w < blocks_per_line; w++ ) {
	
	y = y_offset;
	
	py = y/2;
	px = x/4;
	
	r1 = g1 = b1 = 0;
	
	// 2x2 Block konvertieren
	for( j=0; j<2; j++ ) {
	  x  = x_offset;
	  
	  for ( i=0; i<2; i++ ) {
	    int tmp = rgb_image[x+0 + y*width*2];
	    r = tmp;
	    r = r >> 3;
	    g = (tmp & 0x7);
	    g = g << 3;
	    tmp = rgb_image[x+1 + y*width*2];
	    g = g | (tmp >> 5);
	    b = tmp & 0x1f;
	    
	    //rgb2yuv( r, g, b, &t, u_block[i,j], v_block[i,j] );
	    rgb2yuv( r, g, b, &t, u_block[i], v_block[i] );
	    r1 += r;
	    g1 += g;
	    b1 += b;
	    
	    y_plane[(x/2)+y*width] = (int) t;
	    x+=2;
	  } // for i
	  y++;
	} // for j
	// End 2x2 Block
	
	r1 = r1 /4;
	g1 = g1 /4;
	b1 = b1 /4;
	
	rgb2yuv( r1, g1, b1, &t, &u1, &v1 );
	u_plane[px + py*width/2] = (unsigned char) u1;
	v_plane[px + py*width/2] = (unsigned char) v1;
	
	x_offset = x;
      } // for w
      
      x_offset = 0;
      y_offset = y;
    } // for v
  }
}



  Message* RGBtoYV12ConverterNode::processBuffer( Buffer *in_buffer) {
    Buffer *out_buffer;
    size_t frame_size;
    size_t out_size;

    frame_size = width * height;
    out_size = frame_size + (frame_size/4)*2; // 4Y + 1U + 1V

    out_buffer = getNewBuffer(out_size, in_buffer);

    if (out_buffer) {
        unsigned char *rgb_image = (unsigned char*) in_buffer->getData();
        unsigned char *yuv_image = (unsigned char*) out_buffer->getData();
        switch ( __format ) {
        case RGB32:
            rgb32_to_YV12( rgb_image, yuv_image );
            break;
        case RGB24:
            rgb24_to_YV12( rgb_image, yuv_image );
            break;
        case RGB565:
            rgb565_to_YV12( rgb_image, yuv_image );
            break;
        }
        out_buffer->setUsed( out_size );
    }

    in_buffer->release();
    return out_buffer;
  }
}



#include "nmm/nmm_suffix.hpp"
