/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include <iostream>
#include "YUVtoYUVConverterNode.hpp"
#include "nmm/comm/format/Property.hpp"
#include "nmm/comm/messaging/Buffer.hpp"
#include "nmm/multimedia/Plugin.hpp"
#include "nmm/plugins/NMMRegisterPlugin.hpp"

NMM_REGISTER_PLUGIN(NMM::YUVtoYUVConverterNode, NMM, "YUVtoYUVConverterNode")

namespace NMM {

  YUVtoYUVConverterNode::YUVtoYUVConverterNode(const char* name,
					       const StreamQueue::Mode queue_mode,
					       const int size)
    : GenericConverterNode(name, queue_mode, size)
  {
  }

  Result YUVtoYUVConverterNode::doInit()
  {
    Format* in_format_1   = 0;
    Format* out_format_1  = 0;
    Format* in_format_2   = 0;
    Format* out_format_2  = 0;
    Format* in_format_3   = 0;
    Format* out_format_3  = 0;
    Format* in_format_4   = 0;
    Format* out_format_4  = 0;
    Format* in_format_5   = 0;
    Format* out_format_5  = 0;
    Format* in_format_6   = 0;
    Format* out_format_6  = 0;

    /* Input format for function UYVY2YUYV */
    in_format_1 = getOwnInputProperty()->addNewFormat("video/raw");
    in_format_1->addWildcard(Format::x_resolution_param);
    in_format_1->addWildcard(Format::y_resolution_param);
    in_format_1->addWildcard(Format::ratio_x_param);
    in_format_1->addWildcard(Format::ratio_y_param);
    in_format_1->addWildcard(Format::framerate_param);
    in_format_1->addStringParamValue(Format::channelorder_param,"uyvy");
    in_format_1->addStringParamValue(Format::endian_param, "little");
    in_format_1->addStringParamValue(Format::colorspace_param,"yuv422");
    in_format_1->addIntParamValue(Format::bitperpixel_param,16);
    in_format_1->addStringParamValue(Format::format_param,"packet");
    getOwnInputProperty()->setDefaultFormat(in_format_1);

    /* Output format for function UYVY2YUYV */
    out_format_1 = getOwnOutputProperty()->addNewFormat("video/raw");
    out_format_1->addWildcard(Format::x_resolution_param);
    out_format_1->addWildcard(Format::y_resolution_param);
    out_format_1->addWildcard(Format::ratio_x_param);
    out_format_1->addWildcard(Format::ratio_y_param);
    out_format_1->addWildcard(Format::framerate_param);
    out_format_1->addStringParamValue(Format::channelorder_param,"yuyv");
    out_format_1->addStringParamValue(Format::endian_param, "little");
    out_format_1->addStringParamValue(Format::colorspace_param,"yuv422");
    out_format_1->addIntParamValue(Format::bitperpixel_param,16);
    out_format_1->addStringParamValue(Format::format_param,"packet");
    getOwnOutputProperty()->setDefaultFormat(out_format_1);

    /* Add the converter*/
    addConverter(in_format_1, out_format_1, &UYVY2YUYV);
    in_format_1->setIOPartner(out_format_1);
    out_format_1->setIOPartner(in_format_1);

    /* Input format for function YUV422_planar2UYVY422*/
    in_format_2 = getOwnInputProperty()->addNewFormat("video/raw");
    in_format_2->addWildcard(Format::x_resolution_param);
    in_format_2->addWildcard(Format::y_resolution_param);
    in_format_2->addWildcard(Format::ratio_x_param);
    in_format_2->addWildcard(Format::ratio_y_param);
    in_format_2->addWildcard(Format::framerate_param);
    in_format_2->addStringParamValue(Format::channelorder_param,"yuv");
    in_format_2->addStringParamValue(Format::endian_param, "little");
    in_format_2->addStringParamValue(Format::colorspace_param,"yuv422");
    in_format_2->addIntParamValue(Format::bitperpixel_param,16);
    in_format_2->addStringParamValue(Format::format_param,"planar");

    /* Output format for function YUV422_planar2UYVY422*/
    out_format_2 = getOwnOutputProperty()->addNewFormat("video/raw");
    out_format_2->addWildcard(Format::x_resolution_param);
    out_format_2->addWildcard(Format::y_resolution_param);
    out_format_2->addWildcard(Format::ratio_x_param);
    out_format_2->addWildcard(Format::ratio_y_param);
    out_format_2->addWildcard(Format::framerate_param);
    out_format_2->addStringParamValue(Format::channelorder_param,"uyvy");
    out_format_2->addStringParamValue(Format::endian_param, "little");
    out_format_2->addStringParamValue(Format::colorspace_param,"yuv422");
    out_format_2->addIntParamValue(Format::bitperpixel_param,16);
    out_format_2->addStringParamValue(Format::format_param,"packet");
    /* Add the converter*/
    addConverter(in_format_2, out_format_2,  &YUV422_planar2UYVY422);
    in_format_2->setIOPartner(out_format_2);
    out_format_2->setIOPartner(in_format_2);

    /* Input format for function UYVY422toYUV422_planar*/
    in_format_3 = getOwnInputProperty()->addNewFormat("video/raw");
    in_format_3->addWildcard(Format::x_resolution_param);
    in_format_3->addWildcard(Format::y_resolution_param);
    in_format_3->addWildcard(Format::ratio_x_param);
    in_format_3->addWildcard(Format::ratio_y_param);
    in_format_3->addWildcard(Format::framerate_param);
    in_format_3->addStringParamValue(Format::channelorder_param,"uyvy");
    in_format_3->addStringParamValue(Format::endian_param, "little");
    in_format_3->addStringParamValue(Format::colorspace_param,"yuv422");
    in_format_3->addIntParamValue(Format::bitperpixel_param,16);
    in_format_3->addStringParamValue(Format::format_param,"packet");

    /* Output format for function UYVY422toYUV422_planar*/
    out_format_3 = getOwnOutputProperty()->addNewFormat("video/raw");
    out_format_3->addWildcard(Format::x_resolution_param);
    out_format_3->addWildcard(Format::y_resolution_param);
    out_format_3->addWildcard(Format::ratio_x_param);
    out_format_3->addWildcard(Format::ratio_y_param);
    out_format_3->addWildcard(Format::framerate_param);
    out_format_3->addStringParamValue(Format::channelorder_param,"yuv");
    out_format_3->addStringParamValue(Format::endian_param, "little");
    out_format_3->addStringParamValue(Format::colorspace_param,"yuv422");
    out_format_3->addIntParamValue(Format::bitperpixel_param,16);
    out_format_3->addStringParamValue(Format::format_param,"planar");
    /* Add the converter*/
    addConverter(in_format_3, out_format_3, &UYVY422toYUV422_planar);
    in_format_3->setIOPartner(out_format_3);
    out_format_3->setIOPartner(in_format_3);


    /* Input format for function UYVY411_packetToYUV411_planar*/
    in_format_4 = getOwnInputProperty()->addNewFormat("video/raw");
    in_format_4->addWildcard(Format::x_resolution_param);
    in_format_4->addWildcard(Format::y_resolution_param);
    in_format_4->addWildcard(Format::ratio_x_param);
    in_format_4->addWildcard(Format::ratio_y_param);
    in_format_4->addWildcard(Format::framerate_param);
    in_format_4->addStringParamValue(Format::channelorder_param,"yvu");
    in_format_4->addStringParamValue(Format::endian_param, "little");
    in_format_4->addStringParamValue(Format::colorspace_param,"yuv411");
    in_format_4->addIntParamValue(Format::bitperpixel_param,12);
    in_format_4->addStringParamValue(Format::format_param,"packet");

    /* Output format for function UYVY411toYUV11_planar*/
    out_format_4 = getOwnOutputProperty()->addNewFormat("video/raw");
    out_format_4->addWildcard(Format::x_resolution_param);
    out_format_4->addWildcard(Format::y_resolution_param);
    out_format_4->addWildcard(Format::ratio_x_param);
    out_format_4->addWildcard(Format::ratio_y_param);
    out_format_4->addWildcard(Format::framerate_param);
    out_format_4->addStringParamValue(Format::channelorder_param,"yvu");
    out_format_4->addStringParamValue(Format::endian_param, "little");
    out_format_4->addStringParamValue(Format::colorspace_param,"yuv411");
    out_format_4->addIntParamValue(Format::bitperpixel_param,12);
    out_format_4->addStringParamValue(Format::format_param,"planar");
    /* Add the converter*/
    addConverter(in_format_4, out_format_4, &UYVY411_packetToYUV411_planar);
    in_format_4->setIOPartner(out_format_4);
    out_format_4->setIOPartner(in_format_4);

    /* Input format for function UYVY422_packetToYUV422_packet*/
    in_format_5 = getOwnInputProperty()->addNewFormat("video/raw");
    in_format_5->addWildcard(Format::x_resolution_param);
    in_format_5->addWildcard(Format::y_resolution_param);
    in_format_5->addWildcard(Format::ratio_x_param);
    in_format_5->addWildcard(Format::ratio_y_param);
    in_format_5->addWildcard(Format::framerate_param);
    in_format_5->addStringParamValue(Format::channelorder_param,"uyvy");
    in_format_5->addStringParamValue(Format::endian_param, "little");
    in_format_5->addStringParamValue(Format::colorspace_param,"yuv422");
    in_format_5->addIntParamValue(Format::bitperpixel_param,16);
    in_format_5->addStringParamValue(Format::format_param,"packet");

    /* Output format for function UYVY422_packetToYUV422_packet*/
    out_format_5 = getOwnOutputProperty()->addNewFormat("video/raw");
    out_format_5->addWildcard(Format::x_resolution_param);
    out_format_5->addWildcard(Format::y_resolution_param);
    out_format_5->addWildcard(Format::ratio_x_param);
    out_format_5->addWildcard(Format::ratio_y_param);
    out_format_5->addWildcard(Format::framerate_param);
    out_format_5->addStringParamValue(Format::channelorder_param,"yuv");
    out_format_5->addStringParamValue(Format::endian_param, "little");
    out_format_5->addStringParamValue(Format::colorspace_param,"yuv422");
    out_format_5->addIntParamValue(Format::bitperpixel_param,16);
    out_format_5->addStringParamValue(Format::format_param,"packet");
    /* Add the converter*/
    addConverter(in_format_5, out_format_5, &UYVY422_packetToYUV422_packet);
    in_format_5->setIOPartner(out_format_5);
    out_format_5->setIOPartner(in_format_5);

    /* Input format for function YUV422_packetToUYVY422_packet*/
    in_format_6 = getOwnInputProperty()->addNewFormat("video/raw");
    in_format_6->addWildcard(Format::x_resolution_param);
    in_format_6->addWildcard(Format::y_resolution_param);
    in_format_6->addWildcard(Format::ratio_x_param);
    in_format_6->addWildcard(Format::ratio_y_param);
    in_format_6->addWildcard(Format::framerate_param);
    in_format_6->addStringParamValue(Format::channelorder_param,"yuv");
    in_format_6->addStringParamValue(Format::endian_param, "little");
    in_format_6->addStringParamValue(Format::colorspace_param,"yuv422");
    in_format_6->addIntParamValue(Format::bitperpixel_param,16);
    in_format_6->addStringParamValue(Format::format_param,"packet");

    /* Output format for function YUV422_packetToUYVY422_packet*/
    out_format_6 = getOwnOutputProperty()->addNewFormat("video/raw");
    out_format_6->addWildcard(Format::x_resolution_param);
    out_format_6->addWildcard(Format::y_resolution_param);
    out_format_6->addWildcard(Format::ratio_x_param);
    out_format_6->addWildcard(Format::ratio_y_param);
    out_format_6->addWildcard(Format::framerate_param);
    out_format_6->addStringParamValue(Format::channelorder_param,"uyvy");
    out_format_6->addStringParamValue(Format::endian_param, "little");
    out_format_6->addStringParamValue(Format::colorspace_param,"yuv422");
    out_format_6->addIntParamValue(Format::bitperpixel_param,16);
    out_format_6->addStringParamValue(Format::format_param,"packet");
    /* Add the converter*/
    addConverter(in_format_6, out_format_6, &YUV422_packetToUYVY422_packet);
    in_format_6->setIOPartner(out_format_6);
    out_format_6->setIOPartner(in_format_6);

    return SUCCESS;
  }

  Buffer* YUVtoYUVConverterNode::UYVY2YUYV(Buffer* in,const GenericConverterNode* node)
  {
    // BUG
    // Buffer* out = node->getBufferManager()->requestBuffer(in->getUsed());
    // geht nicht
    // Buffer* out = node->getNewBuffer(in->getUsed(), in);

    // geht, aber unschoen..., oder?
    Buffer *out =
      const_cast<GenericConverterNode *>(node)->getNewBuffer(in->getUsed(), in);

    if(out != 0) {
      char *src = in->getData();
      char *dest = out->getData();

      for (unsigned int i = 0; i < in->getUsed(); i += 2) {
	dest[i]   = src[i+1];
	dest[i+1] = src[i];
      }
      out->setUsed(in->getUsed());
    }
    else {
      NODE_ERROR_STREAM(node,  " Could not get out buffer." << endl);
    }

    in->release();
    return out;
  }

  Buffer* YUVtoYUVConverterNode::YUV422_planar2UYVY422(Buffer* in, const GenericConverterNode* node)
  {
    // BUG
    // Buffer *out = node->getBufferManager()->requestBuffer(in->getUsed()*2);
    // Geht nicht
    // Buffer *out = node->getNewBuffer(in->getUsed()*2, in);

    // geht, aber unschoen..., oder?
    Buffer *out =
      const_cast<GenericConverterNode *>(node)->getNewBuffer(in->getUsed(), in);

    if (out) {
      char *y   = in->getData();
      char *u   = y + (in->getUsed())/2;
      char *v   = u + (in->getUsed())/4;
      char *tmp = out->getData();

      for(unsigned int i = 0; i < (in->getUsed())/4; ++i) { //fix it for other resolution!!!!!!!
	*tmp = *u; tmp++; u++;
	*tmp = *y; tmp++; y++;
	*tmp = *v; tmp++; v++;
	*tmp = *y; tmp++; y++;
      }
      out->setUsed(in->getUsed());
    }
    else {
      NODE_ERROR_STREAM(node,  " Could not get out buffer." << endl);
    }

    in->release();
    return out;
  }

  Buffer* YUVtoYUVConverterNode::UYVY422toYUV422_planar(Buffer *in, const GenericConverterNode* node)
  {
    Buffer *out =
      const_cast<GenericConverterNode *>(node)->getNewBuffer(in->getUsed(), in);

    if(out) {
      char *src = in->getData();
      char *dst = out->getData();
      int n = in->getUsed();

      char *end = src+n;
      int y1, y2, u, v;
      int dy, du, dv;

      dy = 0;
      du = n / 2;
      dv = du + (du/2);

      while(src!=end) {
	u  = src[0];
	y1 = src[1];
	v  = src[2];
	y2 = src[3];

	dst[dy++] = y1;
	dst[dy++] = y2;
	dst[du++] = u;
	dst[dv++] = v;

	src+=4;
      }
      out->setUsed(in->getUsed());
    }
    else {
      NODE_ERROR_STREAM(node,  " Could not get out buffer." << endl);
    }

    in->release();
    return out;
  }

  Buffer* YUVtoYUVConverterNode::UYVY411_packetToYUV411_planar(Buffer *in, const GenericConverterNode* node)
  {
    Buffer *out =
      const_cast<GenericConverterNode *>(node)->getNewBuffer(in->getUsed(), in);
/*yvu*/
    if(out) {
      char *src = in->getData();
      char *dst = out->getData();

      char *end = src + in->getUsed();
      int y11, y12, y22, y21, u, v;
      int dy, du, dv;

      dy = 0;
      dv = in->getUsed() - in->getUsed()/3;
      du = in->getUsed() - in->getUsed()/6;
      cout << ">> Begin converter While size : " << dec << in->getUsed() <<endl;
      while(src!=end) {
        u   = src[0];
        y11 = src[1];
        y12 = src[2];
        v   = src[3];
        y21 = src[4];
        y22 = src[5];

        dst[dy++] = y11;
        dst[dy++] = y12;
        dst[dy++] = y21;
        dst[dy++] = y22;
        dst[du++] = u;
        dst[dv++] = v;

        src+=6;
      }
      out->setUsed(in->getUsed());
      cout << "<< Begin converter While" <<endl;
    } else {
      NODE_ERROR_STREAM(node,  " Could not get out buffer." << endl);
    }

    in->release();
    return out;
  }

  Buffer* YUVtoYUVConverterNode::UYVY422_packetToYUV422_packet(Buffer *in, const GenericConverterNode* node)
  {
    Buffer *out =
      const_cast<GenericConverterNode *>(node)->getNewBuffer(in->getUsed(), in);


    if(out != 0) {
      char *src = in->getData();
      char *dest = out->getData();

      for (unsigned int i = 0; i < in->getUsed(); i += 4) {
        dest[i]   = src[i+1]; // y1
        dest[i+1] = src[i];   // u
        dest[i+2] = src[i+3]; // y2
        dest[i+3] = src[i+2]; // v
      }
      out->setUsed(in->getUsed());
    }
    else {
      NODE_ERROR_STREAM(node,  " Could not get out buffer." << endl);
    }

    in->release();
    return out;
  }

  Buffer* YUVtoYUVConverterNode::YUV422_packetToUYVY422_packet(Buffer *in, const GenericConverterNode* node)
  {
    Buffer *out =
      const_cast<GenericConverterNode *>(node)->getNewBuffer(in->getUsed(), in);

    if(out != 0) {
      char *src = in->getData();
      char *dest = out->getData();

      for (unsigned int i = 0; i < in->getUsed(); i += 4) {
        dest[i]   = src[i+1]; // u
        dest[i+1] = src[i];   // y1
        dest[i+2] = src[i+3]; // v
        dest[i+3] = src[i+2]; // y2
      }
      out->setUsed(in->getUsed());
    }
    else {
      NODE_ERROR_STREAM(node,  " Could not get out buffer." << endl);
    }

    in->release();
    return out;
  }


  Result YUVtoYUVConverterNode::doInitOutput() {
    // get input format
    const Format* input_format = getInputFormat();
    if (input_format == NULL) {
      ERROR_STREAM("input format is null"<< endl);
    return FAILURE; }

    // setup output format
    Format* in_format;
    Format* out_format;

    // get format parameters
    int x_resolution;
    int y_resolution;
    x_resolution = input_format->getIntValue(Format::ratio_x_param);
    y_resolution = input_format->getIntValue(Format::ratio_y_param);
    int ratio_x = input_format->getIntValue(Format::ratio_x_param);
    int ratio_y = input_format->getIntValue(Format::ratio_y_param);
    float framerate = input_format->getFloatValue(Format::framerate_param);
    string channelorder = input_format->getStringValue(Format::channelorder_param);
    string colorspace = input_format->getStringValue(Format::colorspace_param);
    string format = input_format->getStringValue(Format::format_param);
    string endian = input_format->getStringValue(Format::endian_param);


    if (colorspace == string("yuv422")) {
      if (channelorder == string("uyvy")) {

            /* Output format for function UYVY422toYUV422_packet*/
            out_format = getOwnWorkingOutputProperty()->addNewFormat("video/raw");
            out_format->addIntParamValue(Format::x_resolution_param,x_resolution);
            out_format->addIntParamValue(Format::y_resolution_param,y_resolution);
            out_format->addIntParamValue(Format::ratio_x_param,ratio_x);
            out_format->addIntParamValue(Format::ratio_y_param,ratio_y);
            out_format->addFloatParamValue(Format::framerate_param, framerate);
            out_format->addStringParamValue(Format::channelorder_param,"yuv");
            out_format->addStringParamValue(Format::endian_param, "little");
            out_format->addStringParamValue(Format::colorspace_param,"yuv422");
            out_format->addIntParamValue(Format::bitperpixel_param,16);
            out_format->addStringParamValue(Format::format_param,"packet");

	    in_format = input_format->duplicateFormat();
	    getOwnWorkingInputProperty()->addNewFormat(in_format);

            in_format->setIOPartner(out_format);
            out_format->setIOPartner(in_format);

	    getOwnWorkingInputProperty()->setDefaultFormat(in_format);
	    getOwnWorkingOutputProperty()->setDefaultFormat(out_format);

	    /* Output format for function UYVY422toYUV422_planar*/
            out_format = getOwnWorkingOutputProperty()->addNewFormat("video/raw");
            out_format->addIntParamValue(Format::x_resolution_param,x_resolution);
            out_format->addIntParamValue(Format::y_resolution_param,y_resolution);
            out_format->addIntParamValue(Format::ratio_x_param,ratio_x);
            out_format->addIntParamValue(Format::ratio_y_param,ratio_y);
            out_format->addFloatParamValue(Format::framerate_param, framerate);
            out_format->addStringParamValue(Format::channelorder_param,"yuv");
            out_format->addStringParamValue(Format::endian_param, "little");
            out_format->addStringParamValue(Format::colorspace_param,"yuv422");
            out_format->addIntParamValue(Format::bitperpixel_param,16);
            out_format->addStringParamValue(Format::format_param,"planar");

	    in_format = input_format->duplicateFormat();
	    getOwnWorkingInputProperty()->addNewFormat(in_format);

	    in_format->setIOPartner(out_format);
            out_format->setIOPartner(in_format);

	    /* Output format for function UYVY422toYUV422_planar*/
            out_format = getOwnWorkingOutputProperty()->addNewFormat("video/raw");
            out_format->addIntParamValue(Format::x_resolution_param,x_resolution);
            out_format->addIntParamValue(Format::y_resolution_param,y_resolution);
            out_format->addIntParamValue(Format::ratio_x_param,ratio_x);
            out_format->addIntParamValue(Format::ratio_y_param,ratio_y);
            out_format->addFloatParamValue(Format::framerate_param, framerate);
            out_format->addStringParamValue(Format::channelorder_param,"yuyv");
            out_format->addStringParamValue(Format::endian_param, "little");
            out_format->addStringParamValue(Format::colorspace_param,"yuv422");
            out_format->addIntParamValue(Format::bitperpixel_param,16);
            out_format->addStringParamValue(Format::format_param,"packet");

	    in_format = input_format->duplicateFormat();
	    getOwnWorkingInputProperty()->addNewFormat(in_format);

            in_format->setIOPartner(out_format);
            out_format->setIOPartner(in_format);

            return SUCCESS;
      }
      if ((channelorder == string("yuv")) && (format == string("planar"))) {
            /* Output format for function UYVY422toYUV422_planar*/
            out_format = getOwnWorkingOutputProperty()->addNewFormat("video/raw");
            out_format->addIntParamValue(Format::x_resolution_param,x_resolution);
            out_format->addIntParamValue(Format::y_resolution_param,y_resolution);
            out_format->addIntParamValue(Format::ratio_x_param,ratio_x);
            out_format->addIntParamValue(Format::ratio_y_param,ratio_y);
            out_format->addFloatParamValue(Format::framerate_param, framerate);
            out_format->addStringParamValue(Format::channelorder_param,"uyvy");
            out_format->addStringParamValue(Format::endian_param, "little");
            out_format->addStringParamValue(Format::colorspace_param,"yuv422");
            out_format->addIntParamValue(Format::bitperpixel_param,16);
            out_format->addStringParamValue(Format::format_param,"packet");

	    in_format = input_format->duplicateFormat();
	    getOwnWorkingInputProperty()->addNewFormat(in_format);

	    getOwnWorkingInputProperty()->setDefaultFormat(in_format);
            getOwnWorkingOutputProperty()->setDefaultFormat(out_format);

            in_format->setIOPartner(out_format);
            out_format->setIOPartner(in_format);

            return SUCCESS;
      }
      if ((channelorder == string("yuv")) && (format == string("packet"))) {
            /* Output format for function YUV422to_planarUYVY422*/
            out_format = getOwnWorkingOutputProperty()->addNewFormat("video/raw");
            out_format->addIntParamValue(Format::x_resolution_param,x_resolution);
            out_format->addIntParamValue(Format::y_resolution_param,y_resolution);
            out_format->addIntParamValue(Format::ratio_x_param,ratio_x);
            out_format->addIntParamValue(Format::ratio_y_param,ratio_y);
            out_format->addFloatParamValue(Format::framerate_param, framerate);
            out_format->addStringParamValue(Format::channelorder_param,"uyvy");
            out_format->addStringParamValue(Format::endian_param, "little");
            out_format->addStringParamValue(Format::colorspace_param,"yuv422");
            out_format->addIntParamValue(Format::bitperpixel_param,16);
            out_format->addStringParamValue(Format::format_param,"packet");

	    in_format = input_format->duplicateFormat();
	    getOwnWorkingInputProperty()->addNewFormat(in_format);

	    getOwnWorkingInputProperty()->setDefaultFormat(in_format);
            getOwnWorkingOutputProperty()->setDefaultFormat(out_format);

            in_format->setIOPartner(out_format);
            out_format->setIOPartner(in_format);
            return SUCCESS;
      }
      return FAILURE;
    }
    if (colorspace == string("yuv411")) {
      if (channelorder == string("yvu")) {
            /* Output format for function UYVY422toYUV422_planar*/
            out_format = getOwnWorkingOutputProperty()->addNewFormat("video/raw");
            out_format->addIntParamValue(Format::x_resolution_param,x_resolution);
            out_format->addIntParamValue(Format::y_resolution_param,y_resolution);
            out_format->addIntParamValue(Format::ratio_x_param,ratio_x);
            out_format->addIntParamValue(Format::ratio_y_param,ratio_y);
            out_format->addFloatParamValue(Format::framerate_param, framerate);
            out_format->addStringParamValue(Format::channelorder_param,"yvu");
            out_format->addStringParamValue(Format::endian_param, "little");
            out_format->addStringParamValue(Format::colorspace_param,"yuv422");
            out_format->addIntParamValue(Format::bitperpixel_param,12);
            out_format->addStringParamValue(Format::format_param,"planar");

	    in_format = input_format->duplicateFormat();
	    getOwnWorkingInputProperty()->addNewFormat(in_format);

	    getOwnWorkingInputProperty()->setDefaultFormat(in_format);
            getOwnWorkingOutputProperty()->setDefaultFormat(out_format);

	    in_format->setIOPartner(out_format);
            out_format->setIOPartner(in_format);
            return SUCCESS;
      }
    }

    return FAILURE;
  }

  Result YUVtoYUVConverterNode::doDeinitOutput() {
    // erase working properties
    getOwnWorkingInputProperty()->erase();
    getOwnWorkingOutputProperty()->erase();
    return SUCCESS;
  }

}

