/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef _NMM_VIDEOGRABNODE_HPP_
#define _NMM_VIDEOGRABNODE_HPP_

#include "nmm/multimedia/GenericSourceNode.hpp"
#include "nmm/multimedia/sync/StreamTimer.hpp"
#include "nmm/plugins/video/converter/ICropImpl.hpp"
#include "nmm/utils/time/Time.hpp"
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/XShm.h>
#include <unistd.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

namespace NMM {

  class VideoGrabNode : public GenericSourceNode, public ICropImpl {
  private:
    StreamTimer* stimer;
    
    int image_width;
    int image_height;
    int image_bpp;
    double image_fps;

    int x_offset;
    int y_offset;
    int x_range;
    int y_range;
    
    Time            __last_timer;     ///< stores the latest timespamp
    ThreadMutex     __cond_mutex;     ///< ThreadMutex, required for the ThreadCondition
    ThreadCondition __condition;      ///< used to wait until next buffer must be send

    Display *_display;
    Window _root;
    XShmSegmentInfo _shminfo;
    XImage *_ximage;

    void addSupportedFormats(Property* property);
  protected:
    virtual void setCrop(const nmm_int32& xOffset, const nmm_int32& yOffset, const nmm_int32& xRange, const nmm_int32& yRange);
    virtual Message* processBuffer(Buffer*);
    
    virtual Result doInit();
    virtual Result doInitOutput();
    virtual Result doDeinitOutput();
    virtual Result doActivate();
    virtual Result doStop();
    
  public:
    VideoGrabNode(const char* name = "VideoGrabNode",
                   StreamQueue::Mode = StreamQueue::MODE_SUSPEND,
                   int = StreamQueue::DEFAULT_SIZE);
    
    virtual ~VideoGrabNode();
  };
}

#endif //_NMM_VIDEOGRABNODE_HPP_

