/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "XImg.hpp"

#include "nmm/comm/messaging/Buffer.hpp"
#include "nmm/multimedia/Node.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  XImg::XImg(Node* parent,
             int iwidth, int iheight,
	     string ititle)
    : XVirtualImg(parent, iwidth, iheight, ititle),
      ximage(NULL) {
  }

  XImg::~XImg() {
    // this is not very clean: as freeImage() is virtual and might be
    // overloaded in derived classes (see XShmImg), we test here for a
    // NULL-Ptr and only call freeImage() if we still have a ximage
    if(ximage) {
      deinitImage();
    }
  }

  bool XImg::initImage() {
    // allocate ximage using "Default*" macros for visual and depth
    // and NULL-Ptr for the image data (will be replaced on present
    // with the char-Ptr of the incoming Buffer).Variables
    // starting with '_' are provided by XVirtualImg.
    ximage = XCreateImage(getDisplay(), //Display
                          DefaultVisual(getDisplay(), DefaultScreen(getDisplay())), //Visual
                          DefaultDepth(getDisplay(), DefaultScreen(getDisplay())), //depth
                          ZPixmap, //format
                          0, //offset
                          NULL, //data
                          getWidth(), //width
                          getHeight(), //height
                          8, //bitmap_pad
                          0); //bytes_per_line
    
    if(!ximage) {
      NODE_ERROR_STREAM(getParent(), "XImg> Could not allocate a XImage!" << endl);
      return false;
    }

    // sync the display, to be sure image creation is forwarded to the
    // XServer, not discarding incoming events
    XSync(getDisplay(), False);

    return true;
  }

  void XImg::deinitImage() {
    if (ximage) {
      XFree(ximage);
      ximage = NULL;
    }
  }

  void XImg::prepareOutput(Buffer* in_buffer) {
    // simple forward pointer
    ximage->data = in_buffer->getData();
  }

  void XImg::update(Buffer*) {
    NODE_DEBUG_STREAM(getParent(), "XImg> update begin" << endl);
    // put the image ximage to the Window _win on Display getDisplay(),
    // using context _gc, width _width and height _height. Variables
    // starting with '_' are provided by XVirtualImg. Presentation
    // will have zero offset in x and y direction (specified by src_x
    // and src_y parameters).
    XPutImage(getDisplay(), getWindow(), getContext(), ximage,
              0, 0, 0, 0, // src_x, src_y, dest_x, dest_y
              getWidth(), getHeight());
    // sync the display, to be sure image is forwarded to the
    // XServer, not discarding incoming events
    XSync(getDisplay(), False);
    // now, we can modify the data of the ximage again
    NODE_DEBUG_STREAM(getParent(), "XImg> update end" << endl);
  }
  
  char* XImg::getImage() {
    if(!ximage) {
      return NULL;
    }
    
    return ximage->data;
  }

#include "nmm/nmm_prefix.hpp"

} // namespace NMM

#include "nmm/nmm_suffix.hpp"
