/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_X_IMG_HPP
#define NMM_X_IMG_HPP NMM_X_IMG_HPP

#include "nmm/nmm_prefix.hpp"

#include "XVirtualImg.hpp"

namespace NMM {

  /** basic X image.
   *
   * This is a class for writing video frames to a X Display using
   * pure Xlib calls.
   */
  class XImg : public XVirtualImg {
  protected:
    //! pointer to the XImage used for putting the incomming images in
    XImage* ximage;

    // implementation of pure virtual methods from XVirtualImg
    virtual bool initImage();
    virtual void deinitImage();
    
  public:
    /** Constructor.
     *
     * \param parent Node* to the DisplayNode, this Img is running in
     * \param width The width of the video frames, to be presented
     * \param height The width of the video frames, to be presented
     * \param title The title used for the X-Window to be created
     */
    XImg(Node* parent,
         int width, int height,
         std::string title);

    /** Destructor.
     */
    virtual ~XImg();

    // implementation of pure virtual methods from XVirtualImg
    virtual void prepareOutput(Buffer*);
    virtual void update(Buffer*);

    virtual char* getImage();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
