/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_X_SHM_IMG_HPP
#define NMM_X_SHM_IMG_HPP NMM_X_SHM_IMG_HPP

#include "nmm/nmm_prefix.hpp"

#include "XImg.hpp"
#include "XShmManager.hpp"

namespace NMM {

  /** Shared memory X image.
   *
   * This is a class for writing video frames to a X Display using
   * XShm extension.
   */
  class XShmImg : public XImg {
  protected:
    //! the X shared memory manager for this class
    XShmManager shm;

    // reimplementation of virtual methods of XImg
    virtual bool initImage();
    virtual void deinitImage();

  public:

    /** Constructor.
     *
     * \param parent Node* to the DisplayNode, this Img is running in
     * \param width The width of the video frames, to be presented
     * \param height The width of the video frames, to be presented
     * \param title The title used for the X-Window to be created
     * \param manager A pointer to a ShmBufferManager or NULL if no
     *                ShmBufferManager should be used
     * \param pages The ammount of shared memory pages
     *              (with size with * height), which the shared memory
     *              manager should allocate/attach.
     */
    XShmImg(Node* parent,
            int width, int height,
            std::string title,
            ShmBufferManager* manager, unsigned int pages);

    /** Destructor.
     */
    virtual ~XShmImg();

    // reimplementation of virtual method of XVirtualImg to replace
    // its dummy implementation
    virtual void updateMemoryManager(ShmBufferManager*, unsigned int);

    // reimplementation of virtual methods of XImg
    virtual void prepareOutput(Buffer*);
    virtual void update(Buffer*);
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
