/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_XV_SHM_IMG_HPP
#define NMM_XV_SHM_IMG_HPP NMM_XV_SHM_IMG_HPP

#include "nmm/nmm_prefix.hpp"

#include "XShmManager.hpp"

// must come after the shm-includes, otherwise
// the XvShm-Stuff is not correctly declared
#include "XvImg.hpp"

namespace NMM {

  /** Shared memory Xv image.
   *
   * This is a class for writing video frames to a X Display using
   * XVideo extension in combination with XShm extension.
   */
  class XvShmImg : public XvImg {
  protected:
    //! the X shared memory manager for this class
    XShmManager shm;

    // reimplementation of virtual methods of XvImg
    virtual bool initImage();
    virtual void deinitImage();

  public:

    /** Constructor.
     *
     * \param parent Node* to the DisplayNode, this Img is running in
     * \param width The width of the video frames, to be presented
     * \param height The width of the video frames, to be presented
     * \param title The title used for the X-Window to be created
     * \param format The XVideo format description of the video frames
     *               to be presented
     * \param fullscreen Flag to indicate, that the window should be
     *                   opened directly in fullscreen mode
     * \param ratio_x The x-component of the aspect ratio to use for
     *                presenting the video frames
     * \param ratio_y The y-component of the aspect ratio to use for
     *                presenting the video frames
     * \param manager A pointer to a ShmBufferManager or NULL if no
     *                ShmBufferManager should be used
     * \param pages The ammount of shared memory pages
     *              (with size with * height), which the shared memory
     *              manager should allocate/attach.
     */
    XvShmImg(Node* parent,
             int width, int height,
             std::string title,
	     unsigned int format, bool fullscreen,
	     int ratio_x, int ratio_y,
	     ShmBufferManager* manager, unsigned int shared_pages);

    /** Destructor.
     */
    virtual ~XvShmImg();

    // reimplementation of virtual method of XVirtualImg to replace
    // its dummy implementation
    void updateMemoryManager(ShmBufferManager*, unsigned int);
    
    // reimplementation of virtual methods of XvImg
    virtual void prepareOutput( Buffer* in_buffer);
    virtual void update( Buffer* in_buffer );
  };

}

#include "nmm/nmm_suffix.hpp"

#endif //NMM_XVSHMIMG_HPP
