/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IDisplayEventSender.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IDisplayEventSender::used() {} 
  
  
  void IDisplayEventSender::enableDisplayEvents(const string& config_file /* = "" */) /* limited(INode::STARTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<string > config_file_value(config_file, false);
    NMM::Event event("IDisplayEventSender::enableDisplayEvents", &config_file_value);
    if(const_cast<IDisplayEventSender*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IDisplayEventSender::disableDisplayEvents()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IDisplayEventSender::disableDisplayEvents");
    if(const_cast<IDisplayEventSender*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IDisplayEventSender::IDisplayEventSender_type = "IDisplayEventSender";
  NMM::TConstructor<IDisplayEventSender > __IDisplayEventSender_prototype("IDisplayEventSender","NMM");


  const char* IDisplayEventSender::enableDisplayEvents_event = "IDisplayEventSender::enableDisplayEvents";
  const char* IDisplayEventSender::disableDisplayEvents_event = "IDisplayEventSender::disableDisplayEvents";


}


#include "nmm/nmm_suffix.hpp"

