/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IDisplayEventSender_HPP
#define NMM_IDisplayEventSender_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "nmm/multimedia/INode.hpp"

#include "visibility.hpp"

namespace NMM
{
  /**
   * Display event sender.
   * A node which can send methods from IDisplayEvents. This interface is for checking
   * whether a node can send such events and for enabling/disabling them.
   *
   * The node can send display events exactly if it supports this interface.
   * Display events are disabled by default.
   */

  class DLLEXPORT IDisplayEventSender : virtual public NMM::Interface { 
  public:
    IDisplayEventSender() : NMM::Interface(IDisplayEventSender::IDisplayEventSender_type, "NMM") { };

    IDisplayEventSender(NMM::NMMObject& parent) : NMM::Interface (parent, IDisplayEventSender::IDisplayEventSender_type, "NMM") { };

    virtual ~IDisplayEventSender() {};

    static const char* IDisplayEventSender_type;

    virtual  string getInterfaceType() const { return IDisplayEventSender_type; }

    NMM::Interface* copy() const { return new IDisplayEventSender(*this); }
    static void used(); 
    static const char* enableDisplayEvents_event;
    static const char* disableDisplayEvents_event;
    /**
     * Enable sending display events.
     * Display events are disabled by default.
     * @param config_file filename of configuration file that maps keys to aliases.
     */

    virtual void enableDisplayEvents(const string& config_file = "") /* limited(INode::STARTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /**
     * Disable sending display events.
     * Display events are disabled by default.
     */

    virtual void disableDisplayEvents()
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IDisplayEventSender> IDisplayEventSender_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
