/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IDisplayEvents.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IDisplayEvents::used() {} 
  
  
  Result IDisplayEvents::keyPressed(const string& key) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > key_value(key, false);
    NMM::Event event("IDisplayEvents::keyPressed", &key_value);
    event.setReturnValue(&return_value);
    if(const_cast<IDisplayEvents*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IDisplayEvents::keyReleased(const string& key) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > key_value(key, false);
    NMM::Event event("IDisplayEvents::keyReleased", &key_value);
    event.setReturnValue(&return_value);
    if(const_cast<IDisplayEvents*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IDisplayEvents::keysym_keyPressed(const nmm_uint32& keysym) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_uint32 > keysym_value(keysym, false);
    NMM::Event event("IDisplayEvents::keysym_keyPressed", &keysym_value);
    event.setReturnValue(&return_value);
    if(const_cast<IDisplayEvents*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IDisplayEvents::keysym_keyReleased(const nmm_uint32& keysym) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_uint32 > keysym_value(keysym, false);
    NMM::Event event("IDisplayEvents::keysym_keyReleased", &keysym_value);
    event.setReturnValue(&return_value);
    if(const_cast<IDisplayEvents*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IDisplayEvents::mouseButtonPressed(const nmm_int32& button, const nmm_int32& x, const nmm_int32& y, const nmm_int32& buttons, const nmm_bool& shift) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > button_value(button, false);
    NMM::TInValue<nmm_int32 > x_value(x, false);
    NMM::TInValue<nmm_int32 > y_value(y, false);
    NMM::TInValue<nmm_int32 > buttons_value(buttons, false);
    NMM::TInValue<nmm_bool > shift_value(shift, false);
    NMM::Event event("IDisplayEvents::mouseButtonPressed", &button_value, &x_value, &y_value, &buttons_value, &shift_value);
    event.setReturnValue(&return_value);
    if(const_cast<IDisplayEvents*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IDisplayEvents::mouseButtonReleased(const nmm_int32& button, const nmm_int32& x, const nmm_int32& y, const nmm_int32& buttons, const nmm_bool& shift) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > button_value(button, false);
    NMM::TInValue<nmm_int32 > x_value(x, false);
    NMM::TInValue<nmm_int32 > y_value(y, false);
    NMM::TInValue<nmm_int32 > buttons_value(buttons, false);
    NMM::TInValue<nmm_bool > shift_value(shift, false);
    NMM::Event event("IDisplayEvents::mouseButtonReleased", &button_value, &x_value, &y_value, &buttons_value, &shift_value);
    event.setReturnValue(&return_value);
    if(const_cast<IDisplayEvents*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IDisplayEvents::mouseMoved(const nmm_int32& x, const nmm_int32& y, const nmm_int32& buttons, const nmm_bool& shift) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > x_value(x, false);
    NMM::TInValue<nmm_int32 > y_value(y, false);
    NMM::TInValue<nmm_int32 > buttons_value(buttons, false);
    NMM::TInValue<nmm_bool > shift_value(shift, false);
    NMM::Event event("IDisplayEvents::mouseMoved", &x_value, &y_value, &buttons_value, &shift_value);
    event.setReturnValue(&return_value);
    if(const_cast<IDisplayEvents*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IDisplayEvents::windowExposed() /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("IDisplayEvents::windowExposed");
    event.setReturnValue(&return_value);
    if(const_cast<IDisplayEvents*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IDisplayEvents::windowClosed() /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("IDisplayEvents::windowClosed");
    event.setReturnValue(&return_value);
    if(const_cast<IDisplayEvents*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IDisplayEvents::windowResized(const nmm_int32& xsize, const nmm_int32& ysize) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > xsize_value(xsize, false);
    NMM::TInValue<nmm_int32 > ysize_value(ysize, false);
    NMM::Event event("IDisplayEvents::windowResized", &xsize_value, &ysize_value);
    event.setReturnValue(&return_value);
    if(const_cast<IDisplayEvents*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IDisplayEvents::IDisplayEvents_type = "IDisplayEvents";
  NMM::TConstructor<IDisplayEvents > __IDisplayEvents_prototype("IDisplayEvents","NMM");
  TValueConstructor< IDisplayEvents::MouseButtons > __IDisplayEvents_MouseButtons_prototype(TypeInfo("IDisplayEvents::MouseButtons", "NMM"));

  NMM::OValueStream& operator<<(NMM::OValueStream& ov, const IDisplayEvents::MouseButtons& t)
  {
    ov.beginType(TypeInfo("IDisplayEvents::MouseButtons", "NMM"));
    nmm_int32 temp = static_cast<nmm_int32>(t);
    ov << temp;
    ov.endType();
    return ov;
  }

  NMM::IValueStream& operator>>(NMM::IValueStream& iv, IDisplayEvents::MouseButtons& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("IDisplayEvents::MouseButtons", "NMM")) {
      throw NMM::InvalidTypeException(TypeInfo("IDisplayEvents::MouseButtons", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    nmm_int32 temp;
    iv >> temp;
    t = static_cast<IDisplayEvents::MouseButtons>(temp);
    iv.goUp();
    return iv;
  }



  const char* IDisplayEvents::keyPressed_event = "IDisplayEvents::keyPressed";
  NMM::Event* IDisplayEvents::create_keyPressed(const string& key)
  {
    NMM::TInValue<string >* key_value = new NMM::TInValue<string >();
    key_value->setValue(key);

    return new NMM::Event("IDisplayEvents::keyPressed", key_value);
  }

  const char* IDisplayEvents::keyReleased_event = "IDisplayEvents::keyReleased";
  NMM::Event* IDisplayEvents::create_keyReleased(const string& key)
  {
    NMM::TInValue<string >* key_value = new NMM::TInValue<string >();
    key_value->setValue(key);

    return new NMM::Event("IDisplayEvents::keyReleased", key_value);
  }

  const char* IDisplayEvents::keysym_keyPressed_event = "IDisplayEvents::keysym_keyPressed";
  NMM::Event* IDisplayEvents::create_keysym_keyPressed(const nmm_uint32& keysym)
  {
    NMM::TInValue<nmm_uint32 >* keysym_value = new NMM::TInValue<nmm_uint32 >();
    keysym_value->setValue(keysym);

    return new NMM::Event("IDisplayEvents::keysym_keyPressed", keysym_value);
  }

  const char* IDisplayEvents::keysym_keyReleased_event = "IDisplayEvents::keysym_keyReleased";
  NMM::Event* IDisplayEvents::create_keysym_keyReleased(const nmm_uint32& keysym)
  {
    NMM::TInValue<nmm_uint32 >* keysym_value = new NMM::TInValue<nmm_uint32 >();
    keysym_value->setValue(keysym);

    return new NMM::Event("IDisplayEvents::keysym_keyReleased", keysym_value);
  }

  const char* IDisplayEvents::mouseButtonPressed_event = "IDisplayEvents::mouseButtonPressed";
  NMM::Event* IDisplayEvents::create_mouseButtonPressed(const nmm_int32& button, const nmm_int32& x, const nmm_int32& y, const nmm_int32& buttons, const nmm_bool& shift)
  {
    NMM::TInValue<nmm_int32 >* button_value = new NMM::TInValue<nmm_int32 >();
    button_value->setValue(button);
    NMM::TInValue<nmm_int32 >* x_value = new NMM::TInValue<nmm_int32 >();
    x_value->setValue(x);
    NMM::TInValue<nmm_int32 >* y_value = new NMM::TInValue<nmm_int32 >();
    y_value->setValue(y);
    NMM::TInValue<nmm_int32 >* buttons_value = new NMM::TInValue<nmm_int32 >();
    buttons_value->setValue(buttons);
    NMM::TInValue<nmm_bool >* shift_value = new NMM::TInValue<nmm_bool >();
    shift_value->setValue(shift);

    return new NMM::Event("IDisplayEvents::mouseButtonPressed", button_value, x_value, y_value, buttons_value, shift_value);
  }

  const char* IDisplayEvents::mouseButtonReleased_event = "IDisplayEvents::mouseButtonReleased";
  NMM::Event* IDisplayEvents::create_mouseButtonReleased(const nmm_int32& button, const nmm_int32& x, const nmm_int32& y, const nmm_int32& buttons, const nmm_bool& shift)
  {
    NMM::TInValue<nmm_int32 >* button_value = new NMM::TInValue<nmm_int32 >();
    button_value->setValue(button);
    NMM::TInValue<nmm_int32 >* x_value = new NMM::TInValue<nmm_int32 >();
    x_value->setValue(x);
    NMM::TInValue<nmm_int32 >* y_value = new NMM::TInValue<nmm_int32 >();
    y_value->setValue(y);
    NMM::TInValue<nmm_int32 >* buttons_value = new NMM::TInValue<nmm_int32 >();
    buttons_value->setValue(buttons);
    NMM::TInValue<nmm_bool >* shift_value = new NMM::TInValue<nmm_bool >();
    shift_value->setValue(shift);

    return new NMM::Event("IDisplayEvents::mouseButtonReleased", button_value, x_value, y_value, buttons_value, shift_value);
  }

  const char* IDisplayEvents::mouseMoved_event = "IDisplayEvents::mouseMoved";
  NMM::Event* IDisplayEvents::create_mouseMoved(const nmm_int32& x, const nmm_int32& y, const nmm_int32& buttons, const nmm_bool& shift)
  {
    NMM::TInValue<nmm_int32 >* x_value = new NMM::TInValue<nmm_int32 >();
    x_value->setValue(x);
    NMM::TInValue<nmm_int32 >* y_value = new NMM::TInValue<nmm_int32 >();
    y_value->setValue(y);
    NMM::TInValue<nmm_int32 >* buttons_value = new NMM::TInValue<nmm_int32 >();
    buttons_value->setValue(buttons);
    NMM::TInValue<nmm_bool >* shift_value = new NMM::TInValue<nmm_bool >();
    shift_value->setValue(shift);

    return new NMM::Event("IDisplayEvents::mouseMoved", x_value, y_value, buttons_value, shift_value);
  }

  const char* IDisplayEvents::windowExposed_event = "IDisplayEvents::windowExposed";
  NMM::Event* IDisplayEvents::create_windowExposed()
  {

    return new NMM::Event("IDisplayEvents::windowExposed");
  }

  const char* IDisplayEvents::windowClosed_event = "IDisplayEvents::windowClosed";
  NMM::Event* IDisplayEvents::create_windowClosed()
  {

    return new NMM::Event("IDisplayEvents::windowClosed");
  }

  const char* IDisplayEvents::windowResized_event = "IDisplayEvents::windowResized";
  NMM::Event* IDisplayEvents::create_windowResized(const nmm_int32& xsize, const nmm_int32& ysize)
  {
    NMM::TInValue<nmm_int32 >* xsize_value = new NMM::TInValue<nmm_int32 >();
    xsize_value->setValue(xsize);
    NMM::TInValue<nmm_int32 >* ysize_value = new NMM::TInValue<nmm_int32 >();
    ysize_value->setValue(ysize);

    return new NMM::Event("IDisplayEvents::windowResized", xsize_value, ysize_value);
  }



}


#include "nmm/nmm_suffix.hpp"

