/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IDisplayEventsImpl.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/messaging/TEDObject.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  
  void IDisplayEventsImpl::registerEvents()
  {
  
    getEventDispatcher().registerEvent(IDisplayEvents::keyPressed_event, new NMM::TEDObject1<IDisplayEventsImpl, string, Result(IDisplayEventsImpl:: * )(const string&), Result >(this, &IDisplayEventsImpl::keyPressed));
  
    getEventDispatcher().registerEvent(IDisplayEvents::keyReleased_event, new NMM::TEDObject1<IDisplayEventsImpl, string, Result(IDisplayEventsImpl:: * )(const string&), Result >(this, &IDisplayEventsImpl::keyReleased));
  
    getEventDispatcher().registerEvent(IDisplayEvents::keysym_keyPressed_event, new NMM::TEDObject1<IDisplayEventsImpl, nmm_uint32, Result(IDisplayEventsImpl:: * )(const nmm_uint32&), Result >(this, &IDisplayEventsImpl::keysym_keyPressed));
  
    getEventDispatcher().registerEvent(IDisplayEvents::keysym_keyReleased_event, new NMM::TEDObject1<IDisplayEventsImpl, nmm_uint32, Result(IDisplayEventsImpl:: * )(const nmm_uint32&), Result >(this, &IDisplayEventsImpl::keysym_keyReleased));
  
    getEventDispatcher().registerEvent(IDisplayEvents::mouseButtonPressed_event, new NMM::TEDObject5<IDisplayEventsImpl, nmm_int32, nmm_int32, nmm_int32, nmm_int32, nmm_bool, Result(IDisplayEventsImpl:: * )(const nmm_int32&, const nmm_int32&, const nmm_int32&, const nmm_int32&, const nmm_bool&), Result >(this, &IDisplayEventsImpl::mouseButtonPressed));
  
    getEventDispatcher().registerEvent(IDisplayEvents::mouseButtonReleased_event, new NMM::TEDObject5<IDisplayEventsImpl, nmm_int32, nmm_int32, nmm_int32, nmm_int32, nmm_bool, Result(IDisplayEventsImpl:: * )(const nmm_int32&, const nmm_int32&, const nmm_int32&, const nmm_int32&, const nmm_bool&), Result >(this, &IDisplayEventsImpl::mouseButtonReleased));
  
    getEventDispatcher().registerEvent(IDisplayEvents::mouseMoved_event, new NMM::TEDObject4<IDisplayEventsImpl, nmm_int32, nmm_int32, nmm_int32, nmm_bool, Result(IDisplayEventsImpl:: * )(const nmm_int32&, const nmm_int32&, const nmm_int32&, const nmm_bool&), Result >(this, &IDisplayEventsImpl::mouseMoved));
  
    getEventDispatcher().registerEvent(IDisplayEvents::windowExposed_event, new NMM::TEDObject0<IDisplayEventsImpl, Result(IDisplayEventsImpl:: * )(), Result >(this, &IDisplayEventsImpl::windowExposed));
  
    getEventDispatcher().registerEvent(IDisplayEvents::windowClosed_event, new NMM::TEDObject0<IDisplayEventsImpl, Result(IDisplayEventsImpl:: * )(), Result >(this, &IDisplayEventsImpl::windowClosed));
  
    getEventDispatcher().registerEvent(IDisplayEvents::windowResized_event, new NMM::TEDObject2<IDisplayEventsImpl, nmm_int32, nmm_int32, Result(IDisplayEventsImpl:: * )(const nmm_int32&, const nmm_int32&), Result >(this, &IDisplayEventsImpl::windowResized));

  }

}


#include "nmm/nmm_suffix.hpp"

