/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IMouseCursor.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IMouseCursor::used() {} 
  
  
  void IMouseCursor::useDotCursor(const nmm_bool& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > value_value(value, false);
    NMM::Event event("IMouseCursor::useDotCursor", &value_value);
    if(const_cast<IMouseCursor*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMouseCursor::useStandardCursor()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IMouseCursor::useStandardCursor");
    if(const_cast<IMouseCursor*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMouseCursor::useTriangleCursor(const nmm_bool& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > value_value(value, false);
    NMM::Event event("IMouseCursor::useTriangleCursor", &value_value);
    if(const_cast<IMouseCursor*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IMouseCursor::IMouseCursor_type = "IMouseCursor";
  NMM::TConstructor<IMouseCursor > __IMouseCursor_prototype("IMouseCursor","NMM");


  const char* IMouseCursor::useDotCursor_event = "IMouseCursor::useDotCursor";
  const char* IMouseCursor::useStandardCursor_event = "IMouseCursor::useStandardCursor";
  const char* IMouseCursor::useTriangleCursor_event = "IMouseCursor::useTriangleCursor";


}


#include "nmm/nmm_suffix.hpp"

