/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IMouseCursor_HPP
#define NMM_IMouseCursor_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{
  /**
   * Interface for setting mouse cursor properties.
   */

  class DLLEXPORT IMouseCursor : virtual public NMM::Interface { 
  public:
    IMouseCursor() : NMM::Interface(IMouseCursor::IMouseCursor_type, "NMM") { };

    IMouseCursor(NMM::NMMObject& parent) : NMM::Interface (parent, IMouseCursor::IMouseCursor_type, "NMM") { };

    virtual ~IMouseCursor() {};

    static const char* IMouseCursor_type;

    virtual  string getInterfaceType() const { return IMouseCursor_type; }

    NMM::Interface* copy() const { return new IMouseCursor(*this); }
    static void used(); 
    static const char* useDotCursor_event;
    static const char* useStandardCursor_event;
    static const char* useTriangleCursor_event;
    /**
     * Set dot mouse cursor in window.
     * @param value true for dot mouse cursor, false for standard mouse cursor
     */

    virtual void useDotCursor(const nmm_bool& value)
      throw(const NMM::SerializedException&);
    /**
     * Use standard X mouse cursor in window.
     */

    virtual void useStandardCursor()
      throw(const NMM::SerializedException&);
    /**
     * Set cursor to pixmap specified in src with mask msk.
     * @param src the cursor is set to this pixmap values
     * @param msk the mask for the cursor
     */

    virtual void useTriangleCursor(const nmm_bool& value)
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IMouseCursor> IMouseCursor_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
