/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IMouseCursorImpl_HPP
#define NMM_IMouseCursorImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IMouseCursor.hpp"


#include "visibility.hpp"

namespace NMM
{
  /**
   * Interface for setting mouse cursor properties.
   */

  class DLLEXPORT IMouseCursorImpl  : public virtual NMM::NMMObject { 
  public:
    IMouseCursorImpl() { addInterface(new IMouseCursor(*this)); registerEvents(); };

    virtual ~IMouseCursorImpl() {};

  protected:
    /**
     * Set dot mouse cursor in window.
     * @param value true for dot mouse cursor, false for standard mouse cursor
     */

    virtual void useDotCursor(const nmm_bool& value) = 0;
    /**
     * Use standard X mouse cursor in window.
     */

    virtual void useStandardCursor() = 0;
    /**
     * Set cursor to pixmap specified in src with mask msk.
     * @param src the cursor is set to this pixmap values
     * @param msk the mask for the cursor
     */

    virtual void useTriangleCursor(const nmm_bool& value) = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
