/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IVideoDisplay.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IVideoDisplay::used() {} 
  
  
  void IVideoDisplay::setBenchmarkingEnabled(const nmm_bool& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > value_value(value, false);
    NMM::Event event("IVideoDisplay::setBenchmarkingEnabled", &value_value);
    if(const_cast<IVideoDisplay*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IVideoDisplay::setFullscreen(const nmm_bool& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > value_value(value, false);
    NMM::Event event("IVideoDisplay::setFullscreen", &value_value);
    if(const_cast<IVideoDisplay*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_bool IVideoDisplay::getFullscreen() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("IVideoDisplay::getFullscreen");
    event.setReturnValue(&return_value);
    if(const_cast<IVideoDisplay*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool IVideoDisplay::hasFullscreen() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("IVideoDisplay::hasFullscreen");
    event.setReturnValue(&return_value);
    if(const_cast<IVideoDisplay*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool IVideoDisplay::isFullscreenOnly() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("IVideoDisplay::isFullscreenOnly");
    event.setReturnValue(&return_value);
    if(const_cast<IVideoDisplay*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool IVideoDisplay::makeScreenshot(const string& filename) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::TInValue<string > filename_value(filename, false);
    NMM::Event event("IVideoDisplay::makeScreenshot", &filename_value);
    event.setReturnValue(&return_value);
    if(const_cast<IVideoDisplay*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IVideoDisplay::setDisplayPosition(const nmm_int32& xPos, const nmm_int32& yPos) /* limited(INode::CONSTRUCTED) */ /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_int32 > xPos_value(xPos, false);
    NMM::TInValue<nmm_int32 > yPos_value(yPos, false);
    NMM::Event event("IVideoDisplay::setDisplayPosition", &xPos_value, &yPos_value);
    if(const_cast<IVideoDisplay*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IVideoDisplay::setLeftWindowAlignment() /* limited(INode::CONSTRUCTED) */ /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::Event event("IVideoDisplay::setLeftWindowAlignment");
    if(const_cast<IVideoDisplay*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IVideoDisplay::setRightWindowAlignment() /* limited(INode::CONSTRUCTED) */ /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::Event event("IVideoDisplay::setRightWindowAlignment");
    if(const_cast<IVideoDisplay*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IVideoDisplay::setCenterWindowAlignment() /* limited(INode::CONSTRUCTED) */ /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::Event event("IVideoDisplay::setCenterWindowAlignment");
    if(const_cast<IVideoDisplay*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IVideoDisplay::IVideoDisplay_type = "IVideoDisplay";
  NMM::TConstructor<IVideoDisplay > __IVideoDisplay_prototype("IVideoDisplay","NMM");


  const char* IVideoDisplay::setBenchmarkingEnabled_event = "IVideoDisplay::setBenchmarkingEnabled";
  const char* IVideoDisplay::setFullscreen_event = "IVideoDisplay::setFullscreen";
  const char* IVideoDisplay::getFullscreen_event = "IVideoDisplay::getFullscreen";
  const char* IVideoDisplay::hasFullscreen_event = "IVideoDisplay::hasFullscreen";
  const char* IVideoDisplay::isFullscreenOnly_event = "IVideoDisplay::isFullscreenOnly";
  const char* IVideoDisplay::makeScreenshot_event = "IVideoDisplay::makeScreenshot";
  const char* IVideoDisplay::setDisplayPosition_event = "IVideoDisplay::setDisplayPosition";
  const char* IVideoDisplay::setLeftWindowAlignment_event = "IVideoDisplay::setLeftWindowAlignment";
  const char* IVideoDisplay::setRightWindowAlignment_event = "IVideoDisplay::setRightWindowAlignment";
  const char* IVideoDisplay::setCenterWindowAlignment_event = "IVideoDisplay::setCenterWindowAlignment";


}


#include "nmm/nmm_suffix.hpp"

