/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IVideoDisplayImpl.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/messaging/TEDObject.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  
  void IVideoDisplayImpl::registerEvents()
  {
  
    getEventDispatcher().registerEvent(IVideoDisplay::setBenchmarkingEnabled_event, new NMM::TEDObject1<IVideoDisplayImpl, nmm_bool, void(IVideoDisplayImpl:: * )(const nmm_bool&), void >(this, &IVideoDisplayImpl::setBenchmarkingEnabled));
  
    getEventDispatcher().registerEvent(IVideoDisplay::setFullscreen_event, new NMM::TEDObject1<IVideoDisplayImpl, nmm_bool, void(IVideoDisplayImpl:: * )(const nmm_bool&), void >(this, &IVideoDisplayImpl::setFullscreen));
  
    getEventDispatcher().registerEvent(IVideoDisplay::getFullscreen_event, new NMM::TEDObject0<IVideoDisplayImpl, nmm_bool(IVideoDisplayImpl:: * )() const, nmm_bool >(this, &IVideoDisplayImpl::getFullscreen));
  
    getEventDispatcher().registerEvent(IVideoDisplay::hasFullscreen_event, new NMM::TEDObject0<IVideoDisplayImpl, nmm_bool(IVideoDisplayImpl:: * )() const, nmm_bool >(this, &IVideoDisplayImpl::hasFullscreen));
  
    getEventDispatcher().registerEvent(IVideoDisplay::isFullscreenOnly_event, new NMM::TEDObject0<IVideoDisplayImpl, nmm_bool(IVideoDisplayImpl:: * )() const, nmm_bool >(this, &IVideoDisplayImpl::isFullscreenOnly));
  
    getEventDispatcher().registerEvent(IVideoDisplay::makeScreenshot_event, new NMM::TEDObject1<IVideoDisplayImpl, string, nmm_bool(IVideoDisplayImpl:: * )(const string&) const, nmm_bool >(this, &IVideoDisplayImpl::makeScreenshot));
  
    getEventDispatcher().registerEvent(IVideoDisplay::setDisplayPosition_event, new NMM::TEDObject2<IVideoDisplayImpl, nmm_int32, nmm_int32, void(IVideoDisplayImpl:: * )(const nmm_int32&, const nmm_int32&), void >(this, &IVideoDisplayImpl::setDisplayPosition));
    getEventDispatcher().addLimited(IVideoDisplay::setDisplayPosition_event, INode::CONSTRUCTED);
    getEventDispatcher().addLimited(IVideoDisplay::setDisplayPosition_event, INode::INITIALIZED);
  
    getEventDispatcher().registerEvent(IVideoDisplay::setLeftWindowAlignment_event, new NMM::TEDObject0<IVideoDisplayImpl, void(IVideoDisplayImpl:: * )(), void >(this, &IVideoDisplayImpl::setLeftWindowAlignment));
    getEventDispatcher().addLimited(IVideoDisplay::setLeftWindowAlignment_event, INode::CONSTRUCTED);
    getEventDispatcher().addLimited(IVideoDisplay::setLeftWindowAlignment_event, INode::INITIALIZED);
  
    getEventDispatcher().registerEvent(IVideoDisplay::setRightWindowAlignment_event, new NMM::TEDObject0<IVideoDisplayImpl, void(IVideoDisplayImpl:: * )(), void >(this, &IVideoDisplayImpl::setRightWindowAlignment));
    getEventDispatcher().addLimited(IVideoDisplay::setRightWindowAlignment_event, INode::CONSTRUCTED);
    getEventDispatcher().addLimited(IVideoDisplay::setRightWindowAlignment_event, INode::INITIALIZED);
  
    getEventDispatcher().registerEvent(IVideoDisplay::setCenterWindowAlignment_event, new NMM::TEDObject0<IVideoDisplayImpl, void(IVideoDisplayImpl:: * )(), void >(this, &IVideoDisplayImpl::setCenterWindowAlignment));
    getEventDispatcher().addLimited(IVideoDisplay::setCenterWindowAlignment_event, INode::CONSTRUCTED);
    getEventDispatcher().addLimited(IVideoDisplay::setCenterWindowAlignment_event, INode::INITIALIZED);

  }

}


#include "nmm/nmm_suffix.hpp"

