/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IVideoDisplayImpl_HPP
#define NMM_IVideoDisplayImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IVideoDisplay.hpp"

#include "nmm/multimedia/INodeImpl.hpp"

#include "visibility.hpp"

namespace NMM
{
  /**
   * General display interface.
   * This interface provides all general display methods.
   */

  class DLLEXPORT IVideoDisplayImpl  : public virtual NMM::NMMObject { 
  public:
    IVideoDisplayImpl() { addInterface(new IVideoDisplay(*this)); registerEvents(); };

    virtual ~IVideoDisplayImpl() {};

  protected:
    /**
     * Enable/disable presentation of fps on title of display/window
     * @param value true for enabling presentation, else false 
     */

    virtual void setBenchmarkingEnabled(const nmm_bool& value) = 0;
    /**
     * Set fullscreen mode.
     * @param value true for fullscreen, false for windowed display mode
     */

    virtual void setFullscreen(const nmm_bool& value) = 0;
    /**
     * Get fullscreen mode.
     * @return true if the display is in fullscreen mode, false if it is in windowed mode
     */

    virtual nmm_bool getFullscreen() const = 0;
    /**
     * test if display has fullscreen support.
     * @return true if fullscreen mode can be turned on, false otherwise
     */

    virtual nmm_bool hasFullscreen() const = 0;
    /**
     * Test if display is fullscreen only.
     * @return true if fullscren mode is the default and cannot be turned off.
     */

    virtual nmm_bool isFullscreenOnly() const = 0;

    virtual nmm_bool makeScreenshot(const string& filename) const = 0;
    /**
     * Set window alignment.
     */

    virtual void setDisplayPosition(const nmm_int32& xPos, const nmm_int32& yPos) /* limited(INode::CONSTRUCTED) */ /* limited(INode::INITIALIZED) */ = 0;

    virtual void setLeftWindowAlignment() /* limited(INode::CONSTRUCTED) */ /* limited(INode::INITIALIZED) */ = 0;

    virtual void setRightWindowAlignment() /* limited(INode::CONSTRUCTED) */ /* limited(INode::INITIALIZED) */ = 0;

    virtual void setCenterWindowAlignment() /* limited(INode::CONSTRUCTED) */ /* limited(INode::INITIALIZED) */ = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
