/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IXDisplayEventSender.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IXDisplayEventSender::used() {} 
  
  
  void IXDisplayEventSender::setSendingMouseEventsDirect(const nmm_bool& smed) /* limited(INode::STARTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_bool > smed_value(smed, false);
    NMM::Event event("IXDisplayEventSender::setSendingMouseEventsDirect", &smed_value);
    if(const_cast<IXDisplayEventSender*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_bool IXDisplayEventSender::isSendingMouseEventsDirect()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("IXDisplayEventSender::isSendingMouseEventsDirect");
    event.setReturnValue(&return_value);
    if(const_cast<IXDisplayEventSender*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IXDisplayEventSender::setUsingKeySymKeyEvents(const nmm_bool& ukke) /* limited(INode::STARTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_bool > ukke_value(ukke, false);
    NMM::Event event("IXDisplayEventSender::setUsingKeySymKeyEvents", &ukke_value);
    if(const_cast<IXDisplayEventSender*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_bool IXDisplayEventSender::isUsingKeySymKeyEvents()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("IXDisplayEventSender::isUsingKeySymKeyEvents");
    event.setReturnValue(&return_value);
    if(const_cast<IXDisplayEventSender*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IXDisplayEventSender::setXAutoRepeat(const nmm_bool& on)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > on_value(on, false);
    NMM::Event event("IXDisplayEventSender::setXAutoRepeat", &on_value);
    if(const_cast<IXDisplayEventSender*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_bool IXDisplayEventSender::isXAutoRepeat()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("IXDisplayEventSender::isXAutoRepeat");
    event.setReturnValue(&return_value);
    if(const_cast<IXDisplayEventSender*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IXDisplayEventSender::IXDisplayEventSender_type = "IXDisplayEventSender";
  NMM::TConstructor<IXDisplayEventSender > __IXDisplayEventSender_prototype("IXDisplayEventSender","NMM");


  const char* IXDisplayEventSender::setSendingMouseEventsDirect_event = "IXDisplayEventSender::setSendingMouseEventsDirect";
  const char* IXDisplayEventSender::isSendingMouseEventsDirect_event = "IXDisplayEventSender::isSendingMouseEventsDirect";
  const char* IXDisplayEventSender::setUsingKeySymKeyEvents_event = "IXDisplayEventSender::setUsingKeySymKeyEvents";
  const char* IXDisplayEventSender::isUsingKeySymKeyEvents_event = "IXDisplayEventSender::isUsingKeySymKeyEvents";
  const char* IXDisplayEventSender::setXAutoRepeat_event = "IXDisplayEventSender::setXAutoRepeat";
  const char* IXDisplayEventSender::isXAutoRepeat_event = "IXDisplayEventSender::isXAutoRepeat";


}


#include "nmm/nmm_suffix.hpp"

