/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IXDisplayEventSender_HPP
#define NMM_IXDisplayEventSender_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "nmm/plugins/video/display/interfaces/IDisplayEventSender.hpp"

#include "visibility.hpp"

namespace NMM
{
  /**
   * XDisplay event sender.
   * This Node extends IDisplayEventSender with methods to determine the kind mouseevents and keyevents are treated resp. to test,
   * which kind is choosen.
   */

  class DLLEXPORT IXDisplayEventSender : virtual public IDisplayEventSender { 
  public:
    IXDisplayEventSender() : NMM::Interface(IXDisplayEventSender::IXDisplayEventSender_type, "NMM") { };

    IXDisplayEventSender(NMM::NMMObject& parent) : NMM::Interface (parent, IXDisplayEventSender::IXDisplayEventSender_type, "NMM") { };

    virtual ~IXDisplayEventSender() {};

    static const char* IXDisplayEventSender_type;

    virtual  string getInterfaceType() const { return IXDisplayEventSender_type; }

    NMM::Interface* copy() const { return new IXDisplayEventSender(*this); }
    static void used(); 
    static const char* setSendingMouseEventsDirect_event;
    static const char* isSendingMouseEventsDirect_event;
    static const char* setUsingKeySymKeyEvents_event;
    static const char* isUsingKeySymKeyEvents_event;
    static const char* setXAutoRepeat_event;
    static const char* isXAutoRepeat_event;
    /**
     * Switches between processing only Mouseclicks, which are in predefined Screenrectangles or sending all MouseEvents.
     * @param smed boolean Value
     */

    virtual void setSendingMouseEventsDirect(const nmm_bool& smed) /* limited(INode::STARTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /**
     * Returns Value of SendingMouseEventsDirect.
     */

    virtual nmm_bool isSendingMouseEventsDirect()
      throw(const NMM::SerializedException&);
    /**
     * Switches between sending KeyEvents with strings or with KeySym representing the key triggering the event.
     * @param ukke boolean Value
     */

    virtual void setUsingKeySymKeyEvents(const nmm_bool& ukke) /* limited(INode::STARTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /**
     * Returns Value of boolean Variable UsingKeySymKeyEvents.
     */

    virtual nmm_bool isUsingKeySymKeyEvents()
      throw(const NMM::SerializedException&);
    /**
     * Turn XAutoRepeat on or off
     */

    virtual void setXAutoRepeat(const nmm_bool& on)
      throw(const NMM::SerializedException&);
    /**
     * Is XAutoRepeat on or off?
     */

    virtual nmm_bool isXAutoRepeat()
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IXDisplayEventSender> IXDisplayEventSender_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
