/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IOverlay.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IOverlay::used() {} 
  
  
  void IOverlay::setAlpha(const nmm_float& alpha)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_float > alpha_value(alpha, false);
    NMM::Event event("IOverlay::setAlpha", &alpha_value);
    if(const_cast<IOverlay*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_float IOverlay::getAlpha() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_float > return_value;
    NMM::Event event("IOverlay::getAlpha");
    event.setReturnValue(&return_value);
    if(const_cast<IOverlay*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IOverlay::setForeground(const nmm_int32& x, const nmm_int32& y, const nmm_int32& w, const nmm_int32& h)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > x_value(x, false);
    NMM::TInValue<nmm_int32 > y_value(y, false);
    NMM::TInValue<nmm_int32 > w_value(w, false);
    NMM::TInValue<nmm_int32 > h_value(h, false);
    NMM::Event event("IOverlay::setForeground", &x_value, &y_value, &w_value, &h_value);
    if(const_cast<IOverlay*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IOverlay::setForegroundSegment(const nmm_int32& x, const nmm_int32& y, const nmm_int32& w, const nmm_int32& h)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > x_value(x, false);
    NMM::TInValue<nmm_int32 > y_value(y, false);
    NMM::TInValue<nmm_int32 > w_value(w, false);
    NMM::TInValue<nmm_int32 > h_value(h, false);
    NMM::Event event("IOverlay::setForegroundSegment", &x_value, &y_value, &w_value, &h_value);
    if(const_cast<IOverlay*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  Result IOverlay::disableOverlay() /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("IOverlay::disableOverlay");
    event.setReturnValue(&return_value);
    if(const_cast<IOverlay*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IOverlay::IOverlay_type = "IOverlay";
  NMM::TConstructor<IOverlay > __IOverlay_prototype("IOverlay","NMM");


  const char* IOverlay::setAlpha_event = "IOverlay::setAlpha";
  const char* IOverlay::getAlpha_event = "IOverlay::getAlpha";
  const char* IOverlay::setForeground_event = "IOverlay::setForeground";
  const char* IOverlay::setForegroundSegment_event = "IOverlay::setForegroundSegment";
  const char* IOverlay::disableOverlay_event = "IOverlay::disableOverlay";
  NMM::Event* IOverlay::create_disableOverlay()
  {

    return new NMM::Event("IOverlay::disableOverlay");
  }



}


#include "nmm/nmm_suffix.hpp"

