/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IOverlay_HPP
#define NMM_IOverlay_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "ISetOffset.hpp"

#include "visibility.hpp"

namespace NMM
{

  class DLLEXPORT IOverlay : virtual public ISetOffset { 
  public:
    IOverlay() : NMM::Interface(IOverlay::IOverlay_type, "NMM") { };

    IOverlay(NMM::NMMObject& parent) : NMM::Interface (parent, IOverlay::IOverlay_type, "NMM") { };

    virtual ~IOverlay() {};

    static const char* IOverlay_type;

    virtual  string getInterfaceType() const { return IOverlay_type; }

    NMM::Interface* copy() const { return new IOverlay(*this); }
    static void used(); 
    static const char* setAlpha_event;
    static const char* getAlpha_event;
    static const char* setForeground_event;
    static const char* setForegroundSegment_event;
    static const char* disableOverlay_event;
    static NMM::Event* create_disableOverlay();

    virtual void setAlpha(const nmm_float& alpha)
      throw(const NMM::SerializedException&);

    virtual nmm_float getAlpha() const
      throw(const NMM::SerializedException&);
    /**
     * Set dimension and start coordinates of foreground image.
     * @param x x-position of upper left corner
     * @param y y-position of upper left corner
     * @param w width of foreground image
     * @param h height of foreground image
     */

    virtual void setForeground(const nmm_int32& x, const nmm_int32& y, const nmm_int32& w, const nmm_int32& h)
      throw(const NMM::SerializedException&);

    virtual void setForegroundSegment(const nmm_int32& x, const nmm_int32& y, const nmm_int32& w, const nmm_int32& h)
      throw(const NMM::SerializedException&);
  /* instream (protected) and non-instream (public) */    
    virtual Result disableOverlay() /* in-stream */ 
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IOverlay> IOverlay_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
