/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IOverlayImpl_HPP
#define NMM_IOverlayImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IOverlay.hpp"

#include "ISetOffsetImpl.hpp"

#include "visibility.hpp"

namespace NMM
{

  class DLLEXPORT IOverlayImpl  : virtual public ISetOffsetImpl { 
  public:
    IOverlayImpl() { addInterface(new IOverlay(*this)); registerEvents(); };

    virtual ~IOverlayImpl() {};

  protected:

    virtual void setAlpha(const nmm_float& alpha) = 0;

    virtual nmm_float getAlpha() const = 0;
    /**
     * Set dimension and start coordinates of foreground image.
     * @param x x-position of upper left corner
     * @param y y-position of upper left corner
     * @param w width of foreground image
     * @param h height of foreground image
     */

    virtual void setForeground(const nmm_int32& x, const nmm_int32& y, const nmm_int32& w, const nmm_int32& h) = 0;

    virtual void setForegroundSegment(const nmm_int32& x, const nmm_int32& y, const nmm_int32& w, const nmm_int32& h) = 0;

    virtual Result disableOverlay() /* in-stream */  = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
