/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ISetOffset.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ISetOffset::used() {} 
  
  
  void ISetOffset::setOffset(const nmm_int32& x, const nmm_int32& y)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > x_value(x, false);
    NMM::TInValue<nmm_int32 > y_value(y, false);
    NMM::Event event("ISetOffset::setOffset", &x_value, &y_value);
    if(const_cast<ISetOffset*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ISetOffset::getOffset(nmm_int32& x, nmm_int32& y) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > x_value(x);
    NMM::TOutValue<nmm_int32 > y_value(y);
    NMM::Event event("ISetOffset::getOffset", &x_value, &y_value);
    if(const_cast<ISetOffset*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    x = x_value.getRValue();
    y = y_value.getRValue();
  }

  const char* ISetOffset::ISetOffset_type = "ISetOffset";
  NMM::TConstructor<ISetOffset > __ISetOffset_prototype("ISetOffset","NMM");


  const char* ISetOffset::setOffset_event = "ISetOffset::setOffset";
  const char* ISetOffset::getOffset_event = "ISetOffset::getOffset";


}


#include "nmm/nmm_suffix.hpp"

