/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 */

/*  Sources taken from
 *
 *  ivtc.c
 *
 *  Copyright (C) Thomas Östreich - June 2001
 *
 *  This file is part of transcode, a video stream processing tool
 *
 *  transcode is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  transcode is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */

#include "yv12_deinterlace.h"

#ifdef WIN32
#include <windows.h>
#endif

inline void line_average(unsigned char *row1, unsigned char *row2, unsigned char *out, int bytes)
{
  // calculate the average of each color entry in two arrays and return
  // result in char *out
  int y;
  unsigned int tmp;
  
  for (y = 0; y<bytes; ++y) {
    tmp = ((unsigned int) row2[y] + (unsigned int) row1[y]) >>1;
    out[y] = (unsigned char) (tmp & 0xff);
  }
  return;
}

void yv12_deinterlace(char *image, unsigned int width_, unsigned int height_)
{
  unsigned char *in, *out;
  int y, block; 
  
  block = width_;
  in  = (unsigned char*) image;
  out = (unsigned char*) image;
  
  // convert half frame to full frame by simple interpolation
  out +=block;
  
  for (y = 0; y < (height_>>1)-1; y++) {
    line_average(in, in+(block<<1), out, block);
    in  += block<<1;
    out += block<<1;
  }
  
  if (!(height_ & 1)) {
    // clone last row
    memcpy(out, in, block);
  }
  
  return;
}
