; Network-Integrated Multimedia Middleware (NMM)
; http://www.motama.com/nmm.html
;  
; Copyright (C) 2006-2007
; Motama GmbH, Saarbruecken, Germany
; http://www.motama.com
;  
; This file is part of NMM. 
; 
; NMM is free software; you can redistribute it and/or modify it    
; under the terms of the GNU General Public License as published by 
; the Free Software Foundation; only version 2 of the License.      
; 
; NMM is distributed in the hope that it will be useful, but WITHOUT 
; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
; or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
; License for more details.  
; 
; You should have received a copy of the GNU General Public License 
; along with NMM; if not, write to the Free Software Foundation,    
; Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
;


; inline void yv12_deinterlace(char *image, unsigned int width, unsigned int height)
global yv12_deinterlace:function

section .text

; Assumptions: width must be a multiple of 4!!!
yv12_deinterlace:

	push ebp            ; safe on stack
        mov ebp, esp        ; stack pointer in ebp
	push ebx            ; safe ebx
	push esi            ; safe esi
	push edi            ; safe edi

	mov esi, [ebp + 8]  ; get memory address of first line ( = pointer of image at beginning)
	mov eax, [ebp + 12] ; get pixels per line
	mov edx, [ebp + 16] ; get number of lines
        ; as deinterlacing is only done with the lines of ONE field, and the average
	; from the actual line and the actul line + 2, loop from edx/2 - 1 to 0
	shr edx, 1          ; edx = edx / 2
        dec edx             ; edx = edx - 1
	mov edi, esi        ; edi = line 1
        add edi, eax        ; edi = line 2
	mov ebx, edi        ; ebx = line 2
        add ebx, eax        ; ebx = line 3

        pxor mm5, mm5       ; mm5 = 00 00 00 00 00 00 00 00

_l1:    ; for each line do
	mov ecx, eax        ; (re)set number of pixels per line
        shr ecx, 2          ; ecx = ecx / 4;

_l0:    ; for each pixel do
        movd mm0, [esi]     ; mm0 = 00 00 00 00 x1 x2 x3 x4
        movd mm1, [ebx]     ; mm1 = 00 00 00 00 y1 y2 y3 y4
        punpcklbw mm0, mm5  ; mm0 = 00 x1 00 x2 00 x3 00 x4
        punpcklbw mm1, mm5  ; mm1 = 00 y1 00 y2 00 y3 00 y4
        paddw mm0, mm1      ; mm0 = mm0 _+_ mm1
        psrlw mm0, 1        ; mm0 = mm0 / 2
        packuswb mm0, mm0   ; mm0 = z1 z2 z3 z4 z1 z2 z3 z4
        movd [edi], mm0     ; write result
        add esi, 4          ; increment in row1
        add ebx, 4          ; increment in row3
        add edi, 4          ; increment in row2
        loop _l0            ; loop...
	add esi, eax        ; get one line further
	add ebx, eax        ; ...
	add edi, eax        ; ...
	dec edx             ; edx = edx - 1
	jnz NEAR _l1        ; if edx != 0 jump to _l1

	; copy last line
	mov ecx, eax        ; (re)set number of pixels per line
        shr ecx, 2          ; ecx = ecx / 4;
	rep movsd           ; copy values from from esi to edi, increment by 4, repeat until ecx is 0

	emms                ; clean up

	pop edi
	pop esi
	mov ebx, [ebp - 4]

	leave
	ret
