/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IMagickManipulationNode.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IMagickManipulationNode::used() {} 
  
  
  void IMagickManipulationNode::setBorderColor(const nmm_double& r, const nmm_double& g, const nmm_double& b)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_double > r_value(r, false);
    NMM::TInValue<nmm_double > g_value(g, false);
    NMM::TInValue<nmm_double > b_value(b, false);
    NMM::Event event("IMagickManipulationNode::setBorderColor", &r_value, &g_value, &b_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::clearManipulators()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IMagickManipulationNode::clearManipulators");
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::adaptiveThreshold(const nmm_uint32& width, const nmm_uint32& height, const nmm_uint32& offset /* = 0 */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > width_value(width, false);
    NMM::TInValue<nmm_uint32 > height_value(height, false);
    NMM::TInValue<nmm_uint32 > offset_value(offset, false);
    NMM::Event event("IMagickManipulationNode::adaptiveThreshold", &width_value, &height_value, &offset_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::addNoise(const string& noise_type)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > noise_type_value(noise_type, false);
    NMM::Event event("IMagickManipulationNode::addNoise", &noise_type_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::annotate(const string& text, const string& bounding_area, const string& gravity, const nmm_double& degrees)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > text_value(text, false);
    NMM::TInValue<string > bounding_area_value(bounding_area, false);
    NMM::TInValue<string > gravity_value(gravity, false);
    NMM::TInValue<nmm_double > degrees_value(degrees, false);
    NMM::Event event("IMagickManipulationNode::annotate", &text_value, &bounding_area_value, &gravity_value, &degrees_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::blur(const nmm_double& radius /* = 1 */, const nmm_double& sigma /* = 0.5 */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_double > radius_value(radius, false);
    NMM::TInValue<nmm_double > sigma_value(sigma, false);
    NMM::Event event("IMagickManipulationNode::blur", &radius_value, &sigma_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::border(const string& geometry /* = "6x6+0+0" */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > geometry_value(geometry, false);
    NMM::Event event("IMagickManipulationNode::border", &geometry_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::channel(const string& layer)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > layer_value(layer, false);
    NMM::Event event("IMagickManipulationNode::channel", &layer_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::charcoal(const nmm_double& radius /* = 1 */, const nmm_double& sigma /* = 0.5 */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_double > radius_value(radius, false);
    NMM::TInValue<nmm_double > sigma_value(sigma, false);
    NMM::Event event("IMagickManipulationNode::charcoal", &radius_value, &sigma_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::chop(const string& geometry)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > geometry_value(geometry, false);
    NMM::Event event("IMagickManipulationNode::chop", &geometry_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::colorize(const nmm_uint32& opacity_red, const nmm_uint32& opacity_green, const nmm_uint32& opacity_blue, const nmm_double& pen_color_r, const nmm_double& pen_color_g, const nmm_double& pen_color_b)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > opacity_red_value(opacity_red, false);
    NMM::TInValue<nmm_uint32 > opacity_green_value(opacity_green, false);
    NMM::TInValue<nmm_uint32 > opacity_blue_value(opacity_blue, false);
    NMM::TInValue<nmm_double > pen_color_r_value(pen_color_r, false);
    NMM::TInValue<nmm_double > pen_color_g_value(pen_color_g, false);
    NMM::TInValue<nmm_double > pen_color_b_value(pen_color_b, false);
    NMM::Event event("IMagickManipulationNode::colorize", &opacity_red_value, &opacity_green_value, &opacity_blue_value, &pen_color_r_value, &pen_color_g_value, &pen_color_b_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::comment(const string& comment)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > comment_value(comment, false);
    NMM::Event event("IMagickManipulationNode::comment", &comment_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::contrast(const nmm_uint32& sharpen)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > sharpen_value(sharpen, false);
    NMM::Event event("IMagickManipulationNode::contrast", &sharpen_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::crop(const string& geometry)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > geometry_value(geometry, false);
    NMM::Event event("IMagickManipulationNode::crop", &geometry_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::cycleColormap(const nmm_int32& amount)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > amount_value(amount, false);
    NMM::Event event("IMagickManipulationNode::cycleColormap", &amount_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::despeckle()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IMagickManipulationNode::despeckle");
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::edge(const nmm_uint32& radius /* = 0.0 */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > radius_value(radius, false);
    NMM::Event event("IMagickManipulationNode::edge", &radius_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::emboss(const nmm_double& radius /* = 1 */, const nmm_double& sigma /* = 0.5 */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_double > radius_value(radius, false);
    NMM::TInValue<nmm_double > sigma_value(sigma, false);
    NMM::Event event("IMagickManipulationNode::emboss", &radius_value, &sigma_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::enhance()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IMagickManipulationNode::enhance");
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::equalize()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IMagickManipulationNode::equalize");
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::erase()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IMagickManipulationNode::erase");
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::flip()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IMagickManipulationNode::flip");
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::floodFillColor(const nmm_uint32& x, const nmm_uint32& y, const nmm_double& fill_color_r, const nmm_double& fill_color_g, const nmm_double& fill_color_b)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > x_value(x, false);
    NMM::TInValue<nmm_uint32 > y_value(y, false);
    NMM::TInValue<nmm_double > fill_color_r_value(fill_color_r, false);
    NMM::TInValue<nmm_double > fill_color_g_value(fill_color_g, false);
    NMM::TInValue<nmm_double > fill_color_b_value(fill_color_b, false);
    NMM::Event event("IMagickManipulationNode::floodFillColor", &x_value, &y_value, &fill_color_r_value, &fill_color_g_value, &fill_color_b_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::floodFillOpacity(const nmm_int64& x, const nmm_int64& y, const nmm_uint32& opacity, const string& method)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int64 > x_value(x, false);
    NMM::TInValue<nmm_int64 > y_value(y, false);
    NMM::TInValue<nmm_uint32 > opacity_value(opacity, false);
    NMM::TInValue<string > method_value(method, false);
    NMM::Event event("IMagickManipulationNode::floodFillOpacity", &x_value, &y_value, &opacity_value, &method_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::flop()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IMagickManipulationNode::flop");
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::frame(const nmm_uint32& width, const nmm_uint32& height, const nmm_int32& x, const nmm_int32& y, const nmm_int32& inner_bevel /* = 0 */, const nmm_int32& outer_bevel /* = 0 */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > width_value(width, false);
    NMM::TInValue<nmm_uint32 > height_value(height, false);
    NMM::TInValue<nmm_int32 > x_value(x, false);
    NMM::TInValue<nmm_int32 > y_value(y, false);
    NMM::TInValue<nmm_int32 > inner_bevel_value(inner_bevel, false);
    NMM::TInValue<nmm_int32 > outer_bevel_value(outer_bevel, false);
    NMM::Event event("IMagickManipulationNode::frame", &width_value, &height_value, &x_value, &y_value, &inner_bevel_value, &outer_bevel_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::gamma(const nmm_double& gamma_red, const nmm_double& gamma_green, const nmm_double& gamma_blue)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_double > gamma_red_value(gamma_red, false);
    NMM::TInValue<nmm_double > gamma_green_value(gamma_green, false);
    NMM::TInValue<nmm_double > gamma_blue_value(gamma_blue, false);
    NMM::Event event("IMagickManipulationNode::gamma", &gamma_red_value, &gamma_green_value, &gamma_blue_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::gaussianBlur(const nmm_double& width, const nmm_double& sigma)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_double > width_value(width, false);
    NMM::TInValue<nmm_double > sigma_value(sigma, false);
    NMM::Event event("IMagickManipulationNode::gaussianBlur", &width_value, &sigma_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::implode(const nmm_double& factor)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_double > factor_value(factor, false);
    NMM::Event event("IMagickManipulationNode::implode", &factor_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::label(const string& name)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > name_value(name, false);
    NMM::Event event("IMagickManipulationNode::label", &name_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::magnify()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IMagickManipulationNode::magnify");
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::matteFloodfill(const nmm_double& target_r, const nmm_double& target_g, const nmm_double& target_b, const nmm_uint32& opacity, const nmm_int32& x, const nmm_int32& y, const string& method)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_double > target_r_value(target_r, false);
    NMM::TInValue<nmm_double > target_g_value(target_g, false);
    NMM::TInValue<nmm_double > target_b_value(target_b, false);
    NMM::TInValue<nmm_uint32 > opacity_value(opacity, false);
    NMM::TInValue<nmm_int32 > x_value(x, false);
    NMM::TInValue<nmm_int32 > y_value(y, false);
    NMM::TInValue<string > method_value(method, false);
    NMM::Event event("IMagickManipulationNode::matteFloodfill", &target_r_value, &target_g_value, &target_b_value, &opacity_value, &x_value, &y_value, &method_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::medianFilter(const nmm_double& radius /* = 0.0 */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_double > radius_value(radius, false);
    NMM::Event event("IMagickManipulationNode::medianFilter", &radius_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::minify()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IMagickManipulationNode::minify");
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::modulate(const nmm_double& brightness, const nmm_double& saturation, const nmm_double& hue)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_double > brightness_value(brightness, false);
    NMM::TInValue<nmm_double > saturation_value(saturation, false);
    NMM::TInValue<nmm_double > hue_value(hue, false);
    NMM::Event event("IMagickManipulationNode::modulate", &brightness_value, &saturation_value, &hue_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::negate()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IMagickManipulationNode::negate");
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::normalize()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IMagickManipulationNode::normalize");
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::oilPaint(const nmm_uint32& radius /* = 3 */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > radius_value(radius, false);
    NMM::Event event("IMagickManipulationNode::oilPaint", &radius_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::opacity(const nmm_uint32& opacity)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > opacity_value(opacity, false);
    NMM::Event event("IMagickManipulationNode::opacity", &opacity_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::opaque(const nmm_double& opaque_color_r, const nmm_double& opaque_color_g, const nmm_double& opaque_color_b, const nmm_double& pen_color_r, const nmm_double& pen_color_g, const nmm_double& pen_color_b)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_double > opaque_color_r_value(opaque_color_r, false);
    NMM::TInValue<nmm_double > opaque_color_g_value(opaque_color_g, false);
    NMM::TInValue<nmm_double > opaque_color_b_value(opaque_color_b, false);
    NMM::TInValue<nmm_double > pen_color_r_value(pen_color_r, false);
    NMM::TInValue<nmm_double > pen_color_g_value(pen_color_g, false);
    NMM::TInValue<nmm_double > pen_color_b_value(pen_color_b, false);
    NMM::Event event("IMagickManipulationNode::opaque", &opaque_color_r_value, &opaque_color_g_value, &opaque_color_b_value, &pen_color_r_value, &pen_color_g_value, &pen_color_b_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::quantize(const nmm_bool& measureError /* = false */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > measureError_value(measureError, false);
    NMM::Event event("IMagickManipulationNode::quantize", &measureError_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::raise(const string& geometry, const nmm_bool& raisedFlag /* = false */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > geometry_value(geometry, false);
    NMM::TInValue<nmm_bool > raisedFlag_value(raisedFlag, false);
    NMM::Event event("IMagickManipulationNode::raise", &geometry_value, &raisedFlag_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::reduceNoise(const nmm_uint32& order)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > order_value(order, false);
    NMM::Event event("IMagickManipulationNode::reduceNoise", &order_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::roll(const nmm_int32& columns, const nmm_int32& rows)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > columns_value(columns, false);
    NMM::TInValue<nmm_int32 > rows_value(rows, false);
    NMM::Event event("IMagickManipulationNode::roll", &columns_value, &rows_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::rotate(const nmm_double& degrees)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_double > degrees_value(degrees, false);
    NMM::Event event("IMagickManipulationNode::rotate", &degrees_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::sample(const string& geometry)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > geometry_value(geometry, false);
    NMM::Event event("IMagickManipulationNode::sample", &geometry_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::scale(const string& geometry)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > geometry_value(geometry, false);
    NMM::Event event("IMagickManipulationNode::scale", &geometry_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::segment(const nmm_double& cluster_threshold /* = 1.0 */, const nmm_double& smoothing_threshold /* = 1.5 */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_double > cluster_threshold_value(cluster_threshold, false);
    NMM::TInValue<nmm_double > smoothing_threshold_value(smoothing_threshold, false);
    NMM::Event event("IMagickManipulationNode::segment", &cluster_threshold_value, &smoothing_threshold_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::shade(const nmm_double& azimuth /* = 30 */, const nmm_double& elevation /* = 30 */, const nmm_bool& color_shading /* = false */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_double > azimuth_value(azimuth, false);
    NMM::TInValue<nmm_double > elevation_value(elevation, false);
    NMM::TInValue<nmm_bool > color_shading_value(color_shading, false);
    NMM::Event event("IMagickManipulationNode::shade", &azimuth_value, &elevation_value, &color_shading_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::sharpen(const nmm_double& radius /* = 1 */, const nmm_double& sigma /* = 0.5 */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_double > radius_value(radius, false);
    NMM::TInValue<nmm_double > sigma_value(sigma, false);
    NMM::Event event("IMagickManipulationNode::sharpen", &radius_value, &sigma_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::shave(const string& geometry)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > geometry_value(geometry, false);
    NMM::Event event("IMagickManipulationNode::shave", &geometry_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::shear(const nmm_double& x_shear_angle, const nmm_double& y_shear_angle)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_double > x_shear_angle_value(x_shear_angle, false);
    NMM::TInValue<nmm_double > y_shear_angle_value(y_shear_angle, false);
    NMM::Event event("IMagickManipulationNode::shear", &x_shear_angle_value, &y_shear_angle_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::solarize(const nmm_double& factor /* = 50.0 */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_double > factor_value(factor, false);
    NMM::Event event("IMagickManipulationNode::solarize", &factor_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::spread(const nmm_uint32& amount /* = 3 */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > amount_value(amount, false);
    NMM::Event event("IMagickManipulationNode::spread", &amount_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::swirl(const nmm_double& degrees)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_double > degrees_value(degrees, false);
    NMM::Event event("IMagickManipulationNode::swirl", &degrees_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::threshold(const nmm_double& threshold)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_double > threshold_value(threshold, false);
    NMM::Event event("IMagickManipulationNode::threshold", &threshold_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::transform(const string& image_geometry, const string& crop_geometry)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > image_geometry_value(image_geometry, false);
    NMM::TInValue<string > crop_geometry_value(crop_geometry, false);
    NMM::Event event("IMagickManipulationNode::transform", &image_geometry_value, &crop_geometry_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::transparent(const nmm_double& color_r, const nmm_double& color_g, const nmm_double& color_b)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_double > color_r_value(color_r, false);
    NMM::TInValue<nmm_double > color_g_value(color_g, false);
    NMM::TInValue<nmm_double > color_b_value(color_b, false);
    NMM::Event event("IMagickManipulationNode::transparent", &color_r_value, &color_g_value, &color_b_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::trim()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IMagickManipulationNode::trim");
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::unsharpmask(const nmm_double& radius, const nmm_double& sigma, const nmm_double& amount, const nmm_double& threshold)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_double > radius_value(radius, false);
    NMM::TInValue<nmm_double > sigma_value(sigma, false);
    NMM::TInValue<nmm_double > amount_value(amount, false);
    NMM::TInValue<nmm_double > threshold_value(threshold, false);
    NMM::Event event("IMagickManipulationNode::unsharpmask", &radius_value, &sigma_value, &amount_value, &threshold_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::wave(const nmm_double& amplitude /* = 25.0 */, const nmm_double& wavelength /* = 150.0 */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_double > amplitude_value(amplitude, false);
    NMM::TInValue<nmm_double > wavelength_value(wavelength, false);
    NMM::Event event("IMagickManipulationNode::wave", &amplitude_value, &wavelength_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IMagickManipulationNode::zoom(const string& geometry)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > geometry_value(geometry, false);
    NMM::Event event("IMagickManipulationNode::zoom", &geometry_value);
    if(const_cast<IMagickManipulationNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IMagickManipulationNode::IMagickManipulationNode_type = "IMagickManipulationNode";
  NMM::TConstructor<IMagickManipulationNode > __IMagickManipulationNode_prototype("IMagickManipulationNode","NMM");


  const char* IMagickManipulationNode::setBorderColor_event = "IMagickManipulationNode::setBorderColor";
  const char* IMagickManipulationNode::clearManipulators_event = "IMagickManipulationNode::clearManipulators";
  const char* IMagickManipulationNode::adaptiveThreshold_event = "IMagickManipulationNode::adaptiveThreshold";
  const char* IMagickManipulationNode::addNoise_event = "IMagickManipulationNode::addNoise";
  const char* IMagickManipulationNode::annotate_event = "IMagickManipulationNode::annotate";
  const char* IMagickManipulationNode::blur_event = "IMagickManipulationNode::blur";
  const char* IMagickManipulationNode::border_event = "IMagickManipulationNode::border";
  const char* IMagickManipulationNode::channel_event = "IMagickManipulationNode::channel";
  const char* IMagickManipulationNode::charcoal_event = "IMagickManipulationNode::charcoal";
  const char* IMagickManipulationNode::chop_event = "IMagickManipulationNode::chop";
  const char* IMagickManipulationNode::colorize_event = "IMagickManipulationNode::colorize";
  const char* IMagickManipulationNode::comment_event = "IMagickManipulationNode::comment";
  const char* IMagickManipulationNode::contrast_event = "IMagickManipulationNode::contrast";
  const char* IMagickManipulationNode::crop_event = "IMagickManipulationNode::crop";
  const char* IMagickManipulationNode::cycleColormap_event = "IMagickManipulationNode::cycleColormap";
  const char* IMagickManipulationNode::despeckle_event = "IMagickManipulationNode::despeckle";
  const char* IMagickManipulationNode::edge_event = "IMagickManipulationNode::edge";
  const char* IMagickManipulationNode::emboss_event = "IMagickManipulationNode::emboss";
  const char* IMagickManipulationNode::enhance_event = "IMagickManipulationNode::enhance";
  const char* IMagickManipulationNode::equalize_event = "IMagickManipulationNode::equalize";
  const char* IMagickManipulationNode::erase_event = "IMagickManipulationNode::erase";
  const char* IMagickManipulationNode::flip_event = "IMagickManipulationNode::flip";
  const char* IMagickManipulationNode::floodFillColor_event = "IMagickManipulationNode::floodFillColor";
  const char* IMagickManipulationNode::floodFillOpacity_event = "IMagickManipulationNode::floodFillOpacity";
  const char* IMagickManipulationNode::flop_event = "IMagickManipulationNode::flop";
  const char* IMagickManipulationNode::frame_event = "IMagickManipulationNode::frame";
  const char* IMagickManipulationNode::gamma_event = "IMagickManipulationNode::gamma";
  const char* IMagickManipulationNode::gaussianBlur_event = "IMagickManipulationNode::gaussianBlur";
  const char* IMagickManipulationNode::implode_event = "IMagickManipulationNode::implode";
  const char* IMagickManipulationNode::label_event = "IMagickManipulationNode::label";
  const char* IMagickManipulationNode::magnify_event = "IMagickManipulationNode::magnify";
  const char* IMagickManipulationNode::matteFloodfill_event = "IMagickManipulationNode::matteFloodfill";
  const char* IMagickManipulationNode::medianFilter_event = "IMagickManipulationNode::medianFilter";
  const char* IMagickManipulationNode::minify_event = "IMagickManipulationNode::minify";
  const char* IMagickManipulationNode::modulate_event = "IMagickManipulationNode::modulate";
  const char* IMagickManipulationNode::negate_event = "IMagickManipulationNode::negate";
  const char* IMagickManipulationNode::normalize_event = "IMagickManipulationNode::normalize";
  const char* IMagickManipulationNode::oilPaint_event = "IMagickManipulationNode::oilPaint";
  const char* IMagickManipulationNode::opacity_event = "IMagickManipulationNode::opacity";
  const char* IMagickManipulationNode::opaque_event = "IMagickManipulationNode::opaque";
  const char* IMagickManipulationNode::quantize_event = "IMagickManipulationNode::quantize";
  const char* IMagickManipulationNode::raise_event = "IMagickManipulationNode::raise";
  const char* IMagickManipulationNode::reduceNoise_event = "IMagickManipulationNode::reduceNoise";
  const char* IMagickManipulationNode::roll_event = "IMagickManipulationNode::roll";
  const char* IMagickManipulationNode::rotate_event = "IMagickManipulationNode::rotate";
  const char* IMagickManipulationNode::sample_event = "IMagickManipulationNode::sample";
  const char* IMagickManipulationNode::scale_event = "IMagickManipulationNode::scale";
  const char* IMagickManipulationNode::segment_event = "IMagickManipulationNode::segment";
  const char* IMagickManipulationNode::shade_event = "IMagickManipulationNode::shade";
  const char* IMagickManipulationNode::sharpen_event = "IMagickManipulationNode::sharpen";
  const char* IMagickManipulationNode::shave_event = "IMagickManipulationNode::shave";
  const char* IMagickManipulationNode::shear_event = "IMagickManipulationNode::shear";
  const char* IMagickManipulationNode::solarize_event = "IMagickManipulationNode::solarize";
  const char* IMagickManipulationNode::spread_event = "IMagickManipulationNode::spread";
  const char* IMagickManipulationNode::swirl_event = "IMagickManipulationNode::swirl";
  const char* IMagickManipulationNode::threshold_event = "IMagickManipulationNode::threshold";
  const char* IMagickManipulationNode::transform_event = "IMagickManipulationNode::transform";
  const char* IMagickManipulationNode::transparent_event = "IMagickManipulationNode::transparent";
  const char* IMagickManipulationNode::trim_event = "IMagickManipulationNode::trim";
  const char* IMagickManipulationNode::unsharpmask_event = "IMagickManipulationNode::unsharpmask";
  const char* IMagickManipulationNode::wave_event = "IMagickManipulationNode::wave";
  const char* IMagickManipulationNode::zoom_event = "IMagickManipulationNode::zoom";


}


#include "nmm/nmm_suffix.hpp"

