/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IMagickManipulationNodeImpl.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/messaging/TEDObject.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  
  void IMagickManipulationNodeImpl::registerEvents()
  {
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::setBorderColor_event, new NMM::TEDObject3<IMagickManipulationNodeImpl, nmm_double, nmm_double, nmm_double, void(IMagickManipulationNodeImpl:: * )(const nmm_double&, const nmm_double&, const nmm_double&), void >(this, &IMagickManipulationNodeImpl::setBorderColor));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::clearManipulators_event, new NMM::TEDObject0<IMagickManipulationNodeImpl, void(IMagickManipulationNodeImpl:: * )(), void >(this, &IMagickManipulationNodeImpl::clearManipulators));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::adaptiveThreshold_event, new NMM::TEDObject3<IMagickManipulationNodeImpl, nmm_uint32, nmm_uint32, nmm_uint32, void(IMagickManipulationNodeImpl:: * )(const nmm_uint32&, const nmm_uint32&, const nmm_uint32&), void >(this, &IMagickManipulationNodeImpl::adaptiveThreshold));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::addNoise_event, new NMM::TEDObject1<IMagickManipulationNodeImpl, string, void(IMagickManipulationNodeImpl:: * )(const string&), void >(this, &IMagickManipulationNodeImpl::addNoise));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::annotate_event, new NMM::TEDObject4<IMagickManipulationNodeImpl, string, string, string, nmm_double, void(IMagickManipulationNodeImpl:: * )(const string&, const string&, const string&, const nmm_double&), void >(this, &IMagickManipulationNodeImpl::annotate));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::blur_event, new NMM::TEDObject2<IMagickManipulationNodeImpl, nmm_double, nmm_double, void(IMagickManipulationNodeImpl:: * )(const nmm_double&, const nmm_double&), void >(this, &IMagickManipulationNodeImpl::blur));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::border_event, new NMM::TEDObject1<IMagickManipulationNodeImpl, string, void(IMagickManipulationNodeImpl:: * )(const string&), void >(this, &IMagickManipulationNodeImpl::border));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::channel_event, new NMM::TEDObject1<IMagickManipulationNodeImpl, string, void(IMagickManipulationNodeImpl:: * )(const string&), void >(this, &IMagickManipulationNodeImpl::channel));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::charcoal_event, new NMM::TEDObject2<IMagickManipulationNodeImpl, nmm_double, nmm_double, void(IMagickManipulationNodeImpl:: * )(const nmm_double&, const nmm_double&), void >(this, &IMagickManipulationNodeImpl::charcoal));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::chop_event, new NMM::TEDObject1<IMagickManipulationNodeImpl, string, void(IMagickManipulationNodeImpl:: * )(const string&), void >(this, &IMagickManipulationNodeImpl::chop));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::colorize_event, new NMM::TEDObject6<IMagickManipulationNodeImpl, nmm_uint32, nmm_uint32, nmm_uint32, nmm_double, nmm_double, nmm_double, void(IMagickManipulationNodeImpl:: * )(const nmm_uint32&, const nmm_uint32&, const nmm_uint32&, const nmm_double&, const nmm_double&, const nmm_double&), void >(this, &IMagickManipulationNodeImpl::colorize));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::comment_event, new NMM::TEDObject1<IMagickManipulationNodeImpl, string, void(IMagickManipulationNodeImpl:: * )(const string&), void >(this, &IMagickManipulationNodeImpl::comment));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::contrast_event, new NMM::TEDObject1<IMagickManipulationNodeImpl, nmm_uint32, void(IMagickManipulationNodeImpl:: * )(const nmm_uint32&), void >(this, &IMagickManipulationNodeImpl::contrast));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::crop_event, new NMM::TEDObject1<IMagickManipulationNodeImpl, string, void(IMagickManipulationNodeImpl:: * )(const string&), void >(this, &IMagickManipulationNodeImpl::crop));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::cycleColormap_event, new NMM::TEDObject1<IMagickManipulationNodeImpl, nmm_int32, void(IMagickManipulationNodeImpl:: * )(const nmm_int32&), void >(this, &IMagickManipulationNodeImpl::cycleColormap));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::despeckle_event, new NMM::TEDObject0<IMagickManipulationNodeImpl, void(IMagickManipulationNodeImpl:: * )(), void >(this, &IMagickManipulationNodeImpl::despeckle));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::edge_event, new NMM::TEDObject1<IMagickManipulationNodeImpl, nmm_uint32, void(IMagickManipulationNodeImpl:: * )(const nmm_uint32&), void >(this, &IMagickManipulationNodeImpl::edge));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::emboss_event, new NMM::TEDObject2<IMagickManipulationNodeImpl, nmm_double, nmm_double, void(IMagickManipulationNodeImpl:: * )(const nmm_double&, const nmm_double&), void >(this, &IMagickManipulationNodeImpl::emboss));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::enhance_event, new NMM::TEDObject0<IMagickManipulationNodeImpl, void(IMagickManipulationNodeImpl:: * )(), void >(this, &IMagickManipulationNodeImpl::enhance));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::equalize_event, new NMM::TEDObject0<IMagickManipulationNodeImpl, void(IMagickManipulationNodeImpl:: * )(), void >(this, &IMagickManipulationNodeImpl::equalize));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::erase_event, new NMM::TEDObject0<IMagickManipulationNodeImpl, void(IMagickManipulationNodeImpl:: * )(), void >(this, &IMagickManipulationNodeImpl::erase));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::flip_event, new NMM::TEDObject0<IMagickManipulationNodeImpl, void(IMagickManipulationNodeImpl:: * )(), void >(this, &IMagickManipulationNodeImpl::flip));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::floodFillColor_event, new NMM::TEDObject5<IMagickManipulationNodeImpl, nmm_uint32, nmm_uint32, nmm_double, nmm_double, nmm_double, void(IMagickManipulationNodeImpl:: * )(const nmm_uint32&, const nmm_uint32&, const nmm_double&, const nmm_double&, const nmm_double&), void >(this, &IMagickManipulationNodeImpl::floodFillColor));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::floodFillOpacity_event, new NMM::TEDObject4<IMagickManipulationNodeImpl, nmm_int64, nmm_int64, nmm_uint32, string, void(IMagickManipulationNodeImpl:: * )(const nmm_int64&, const nmm_int64&, const nmm_uint32&, const string&), void >(this, &IMagickManipulationNodeImpl::floodFillOpacity));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::flop_event, new NMM::TEDObject0<IMagickManipulationNodeImpl, void(IMagickManipulationNodeImpl:: * )(), void >(this, &IMagickManipulationNodeImpl::flop));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::frame_event, new NMM::TEDObject6<IMagickManipulationNodeImpl, nmm_uint32, nmm_uint32, nmm_int32, nmm_int32, nmm_int32, nmm_int32, void(IMagickManipulationNodeImpl:: * )(const nmm_uint32&, const nmm_uint32&, const nmm_int32&, const nmm_int32&, const nmm_int32&, const nmm_int32&), void >(this, &IMagickManipulationNodeImpl::frame));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::gamma_event, new NMM::TEDObject3<IMagickManipulationNodeImpl, nmm_double, nmm_double, nmm_double, void(IMagickManipulationNodeImpl:: * )(const nmm_double&, const nmm_double&, const nmm_double&), void >(this, &IMagickManipulationNodeImpl::gamma));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::gaussianBlur_event, new NMM::TEDObject2<IMagickManipulationNodeImpl, nmm_double, nmm_double, void(IMagickManipulationNodeImpl:: * )(const nmm_double&, const nmm_double&), void >(this, &IMagickManipulationNodeImpl::gaussianBlur));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::implode_event, new NMM::TEDObject1<IMagickManipulationNodeImpl, nmm_double, void(IMagickManipulationNodeImpl:: * )(const nmm_double&), void >(this, &IMagickManipulationNodeImpl::implode));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::label_event, new NMM::TEDObject1<IMagickManipulationNodeImpl, string, void(IMagickManipulationNodeImpl:: * )(const string&), void >(this, &IMagickManipulationNodeImpl::label));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::magnify_event, new NMM::TEDObject0<IMagickManipulationNodeImpl, void(IMagickManipulationNodeImpl:: * )(), void >(this, &IMagickManipulationNodeImpl::magnify));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::matteFloodfill_event, new NMM::TEDObject7<IMagickManipulationNodeImpl, nmm_double, nmm_double, nmm_double, nmm_uint32, nmm_int32, nmm_int32, string, void(IMagickManipulationNodeImpl:: * )(const nmm_double&, const nmm_double&, const nmm_double&, const nmm_uint32&, const nmm_int32&, const nmm_int32&, const string&), void >(this, &IMagickManipulationNodeImpl::matteFloodfill));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::medianFilter_event, new NMM::TEDObject1<IMagickManipulationNodeImpl, nmm_double, void(IMagickManipulationNodeImpl:: * )(const nmm_double&), void >(this, &IMagickManipulationNodeImpl::medianFilter));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::minify_event, new NMM::TEDObject0<IMagickManipulationNodeImpl, void(IMagickManipulationNodeImpl:: * )(), void >(this, &IMagickManipulationNodeImpl::minify));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::modulate_event, new NMM::TEDObject3<IMagickManipulationNodeImpl, nmm_double, nmm_double, nmm_double, void(IMagickManipulationNodeImpl:: * )(const nmm_double&, const nmm_double&, const nmm_double&), void >(this, &IMagickManipulationNodeImpl::modulate));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::negate_event, new NMM::TEDObject0<IMagickManipulationNodeImpl, void(IMagickManipulationNodeImpl:: * )(), void >(this, &IMagickManipulationNodeImpl::negate));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::normalize_event, new NMM::TEDObject0<IMagickManipulationNodeImpl, void(IMagickManipulationNodeImpl:: * )(), void >(this, &IMagickManipulationNodeImpl::normalize));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::oilPaint_event, new NMM::TEDObject1<IMagickManipulationNodeImpl, nmm_uint32, void(IMagickManipulationNodeImpl:: * )(const nmm_uint32&), void >(this, &IMagickManipulationNodeImpl::oilPaint));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::opacity_event, new NMM::TEDObject1<IMagickManipulationNodeImpl, nmm_uint32, void(IMagickManipulationNodeImpl:: * )(const nmm_uint32&), void >(this, &IMagickManipulationNodeImpl::opacity));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::opaque_event, new NMM::TEDObject6<IMagickManipulationNodeImpl, nmm_double, nmm_double, nmm_double, nmm_double, nmm_double, nmm_double, void(IMagickManipulationNodeImpl:: * )(const nmm_double&, const nmm_double&, const nmm_double&, const nmm_double&, const nmm_double&, const nmm_double&), void >(this, &IMagickManipulationNodeImpl::opaque));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::quantize_event, new NMM::TEDObject1<IMagickManipulationNodeImpl, nmm_bool, void(IMagickManipulationNodeImpl:: * )(const nmm_bool&), void >(this, &IMagickManipulationNodeImpl::quantize));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::raise_event, new NMM::TEDObject2<IMagickManipulationNodeImpl, string, nmm_bool, void(IMagickManipulationNodeImpl:: * )(const string&, const nmm_bool&), void >(this, &IMagickManipulationNodeImpl::raise));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::reduceNoise_event, new NMM::TEDObject1<IMagickManipulationNodeImpl, nmm_uint32, void(IMagickManipulationNodeImpl:: * )(const nmm_uint32&), void >(this, &IMagickManipulationNodeImpl::reduceNoise));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::roll_event, new NMM::TEDObject2<IMagickManipulationNodeImpl, nmm_int32, nmm_int32, void(IMagickManipulationNodeImpl:: * )(const nmm_int32&, const nmm_int32&), void >(this, &IMagickManipulationNodeImpl::roll));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::rotate_event, new NMM::TEDObject1<IMagickManipulationNodeImpl, nmm_double, void(IMagickManipulationNodeImpl:: * )(const nmm_double&), void >(this, &IMagickManipulationNodeImpl::rotate));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::sample_event, new NMM::TEDObject1<IMagickManipulationNodeImpl, string, void(IMagickManipulationNodeImpl:: * )(const string&), void >(this, &IMagickManipulationNodeImpl::sample));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::scale_event, new NMM::TEDObject1<IMagickManipulationNodeImpl, string, void(IMagickManipulationNodeImpl:: * )(const string&), void >(this, &IMagickManipulationNodeImpl::scale));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::segment_event, new NMM::TEDObject2<IMagickManipulationNodeImpl, nmm_double, nmm_double, void(IMagickManipulationNodeImpl:: * )(const nmm_double&, const nmm_double&), void >(this, &IMagickManipulationNodeImpl::segment));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::shade_event, new NMM::TEDObject3<IMagickManipulationNodeImpl, nmm_double, nmm_double, nmm_bool, void(IMagickManipulationNodeImpl:: * )(const nmm_double&, const nmm_double&, const nmm_bool&), void >(this, &IMagickManipulationNodeImpl::shade));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::sharpen_event, new NMM::TEDObject2<IMagickManipulationNodeImpl, nmm_double, nmm_double, void(IMagickManipulationNodeImpl:: * )(const nmm_double&, const nmm_double&), void >(this, &IMagickManipulationNodeImpl::sharpen));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::shave_event, new NMM::TEDObject1<IMagickManipulationNodeImpl, string, void(IMagickManipulationNodeImpl:: * )(const string&), void >(this, &IMagickManipulationNodeImpl::shave));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::shear_event, new NMM::TEDObject2<IMagickManipulationNodeImpl, nmm_double, nmm_double, void(IMagickManipulationNodeImpl:: * )(const nmm_double&, const nmm_double&), void >(this, &IMagickManipulationNodeImpl::shear));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::solarize_event, new NMM::TEDObject1<IMagickManipulationNodeImpl, nmm_double, void(IMagickManipulationNodeImpl:: * )(const nmm_double&), void >(this, &IMagickManipulationNodeImpl::solarize));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::spread_event, new NMM::TEDObject1<IMagickManipulationNodeImpl, nmm_uint32, void(IMagickManipulationNodeImpl:: * )(const nmm_uint32&), void >(this, &IMagickManipulationNodeImpl::spread));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::swirl_event, new NMM::TEDObject1<IMagickManipulationNodeImpl, nmm_double, void(IMagickManipulationNodeImpl:: * )(const nmm_double&), void >(this, &IMagickManipulationNodeImpl::swirl));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::threshold_event, new NMM::TEDObject1<IMagickManipulationNodeImpl, nmm_double, void(IMagickManipulationNodeImpl:: * )(const nmm_double&), void >(this, &IMagickManipulationNodeImpl::threshold));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::transform_event, new NMM::TEDObject2<IMagickManipulationNodeImpl, string, string, void(IMagickManipulationNodeImpl:: * )(const string&, const string&), void >(this, &IMagickManipulationNodeImpl::transform));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::transparent_event, new NMM::TEDObject3<IMagickManipulationNodeImpl, nmm_double, nmm_double, nmm_double, void(IMagickManipulationNodeImpl:: * )(const nmm_double&, const nmm_double&, const nmm_double&), void >(this, &IMagickManipulationNodeImpl::transparent));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::trim_event, new NMM::TEDObject0<IMagickManipulationNodeImpl, void(IMagickManipulationNodeImpl:: * )(), void >(this, &IMagickManipulationNodeImpl::trim));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::unsharpmask_event, new NMM::TEDObject4<IMagickManipulationNodeImpl, nmm_double, nmm_double, nmm_double, nmm_double, void(IMagickManipulationNodeImpl:: * )(const nmm_double&, const nmm_double&, const nmm_double&, const nmm_double&), void >(this, &IMagickManipulationNodeImpl::unsharpmask));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::wave_event, new NMM::TEDObject2<IMagickManipulationNodeImpl, nmm_double, nmm_double, void(IMagickManipulationNodeImpl:: * )(const nmm_double&, const nmm_double&), void >(this, &IMagickManipulationNodeImpl::wave));
  
    getEventDispatcher().registerEvent(IMagickManipulationNode::zoom_event, new NMM::TEDObject1<IMagickManipulationNodeImpl, string, void(IMagickManipulationNodeImpl:: * )(const string&), void >(this, &IMagickManipulationNodeImpl::zoom));

  }

}


#include "nmm/nmm_suffix.hpp"

