/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IVideoMux.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/serialize/SerializeUtilsTime.hpp"
#include "nmm/utils/time/Time.hpp"

namespace NMM
{
  void IVideoMux::used() {} 
  
  
  Time IVideoMux::getCurrentSyncTime()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Time > return_value;
    NMM::Event event("IVideoMux::getCurrentSyncTime");
    event.setReturnValue(&return_value);
    if(const_cast<IVideoMux*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IVideoMux::setPosition(const string& input_jack_tag, const nmm_int32& x, const nmm_int32& y)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > input_jack_tag_value(input_jack_tag, false);
    NMM::TInValue<nmm_int32 > x_value(x, false);
    NMM::TInValue<nmm_int32 > y_value(y, false);
    NMM::Event event("IVideoMux::setPosition", &input_jack_tag_value, &x_value, &y_value);
    if(const_cast<IVideoMux*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IVideoMux::setXPositionControl(const string& input_jack_tag, const Time& start_time, const Time& end_time, const nmm_int32& start_x, const nmm_int32& end_x)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > input_jack_tag_value(input_jack_tag, false);
    NMM::TInValue<Time > start_time_value(start_time, false);
    NMM::TInValue<Time > end_time_value(end_time, false);
    NMM::TInValue<nmm_int32 > start_x_value(start_x, false);
    NMM::TInValue<nmm_int32 > end_x_value(end_x, false);
    NMM::Event event("IVideoMux::setXPositionControl", &input_jack_tag_value, &start_time_value, &end_time_value, &start_x_value, &end_x_value);
    if(const_cast<IVideoMux*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IVideoMux::setYPositionControl(const string& input_jack_tag, const Time& start_time, const Time& end_time, const nmm_int32& start_y, const nmm_int32& end_y)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > input_jack_tag_value(input_jack_tag, false);
    NMM::TInValue<Time > start_time_value(start_time, false);
    NMM::TInValue<Time > end_time_value(end_time, false);
    NMM::TInValue<nmm_int32 > start_y_value(start_y, false);
    NMM::TInValue<nmm_int32 > end_y_value(end_y, false);
    NMM::Event event("IVideoMux::setYPositionControl", &input_jack_tag_value, &start_time_value, &end_time_value, &start_y_value, &end_y_value);
    if(const_cast<IVideoMux*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IVideoMux::setSize(const string& input_jack_tag, const nmm_int32& size_x, const nmm_int32& size_y)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > input_jack_tag_value(input_jack_tag, false);
    NMM::TInValue<nmm_int32 > size_x_value(size_x, false);
    NMM::TInValue<nmm_int32 > size_y_value(size_y, false);
    NMM::Event event("IVideoMux::setSize", &input_jack_tag_value, &size_x_value, &size_y_value);
    if(const_cast<IVideoMux*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IVideoMux::setWidthControl(const string& input_jack_tag, const Time& start_time, const Time& end_time, const nmm_int32& start_width, const nmm_int32& end_width, const IVideoMux::Alignment& align)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > input_jack_tag_value(input_jack_tag, false);
    NMM::TInValue<Time > start_time_value(start_time, false);
    NMM::TInValue<Time > end_time_value(end_time, false);
    NMM::TInValue<nmm_int32 > start_width_value(start_width, false);
    NMM::TInValue<nmm_int32 > end_width_value(end_width, false);
    NMM::TInValue<IVideoMux::Alignment > align_value(align, false);
    NMM::Event event("IVideoMux::setWidthControl", &input_jack_tag_value, &start_time_value, &end_time_value, &start_width_value, &end_width_value, &align_value);
    if(const_cast<IVideoMux*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IVideoMux::setHeightControl(const string& input_jack_tag, const Time& start_time, const Time& end_time, const nmm_int32& start_height, const nmm_int32& end_height, const IVideoMux::Alignment& align)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > input_jack_tag_value(input_jack_tag, false);
    NMM::TInValue<Time > start_time_value(start_time, false);
    NMM::TInValue<Time > end_time_value(end_time, false);
    NMM::TInValue<nmm_int32 > start_height_value(start_height, false);
    NMM::TInValue<nmm_int32 > end_height_value(end_height, false);
    NMM::TInValue<IVideoMux::Alignment > align_value(align, false);
    NMM::Event event("IVideoMux::setHeightControl", &input_jack_tag_value, &start_time_value, &end_time_value, &start_height_value, &end_height_value, &align_value);
    if(const_cast<IVideoMux*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IVideoMux::setWidthHeightCouplingEnabled(const string& input_jack_tag, const nmm_bool& enable)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > input_jack_tag_value(input_jack_tag, false);
    NMM::TInValue<nmm_bool > enable_value(enable, false);
    NMM::Event event("IVideoMux::setWidthHeightCouplingEnabled", &input_jack_tag_value, &enable_value);
    if(const_cast<IVideoMux*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_int32 IVideoMux::getWidth(const string& input_jack_tag)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::TInValue<string > input_jack_tag_value(input_jack_tag, false);
    NMM::Event event("IVideoMux::getWidth", &input_jack_tag_value);
    event.setReturnValue(&return_value);
    if(const_cast<IVideoMux*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 IVideoMux::getHeight(const string& input_jack_tag)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::TInValue<string > input_jack_tag_value(input_jack_tag, false);
    NMM::Event event("IVideoMux::getHeight", &input_jack_tag_value);
    event.setReturnValue(&return_value);
    if(const_cast<IVideoMux*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 IVideoMux::getXPos(const string& input_jack_tag)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::TInValue<string > input_jack_tag_value(input_jack_tag, false);
    NMM::Event event("IVideoMux::getXPos", &input_jack_tag_value);
    event.setReturnValue(&return_value);
    if(const_cast<IVideoMux*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 IVideoMux::getYPos(const string& input_jack_tag)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::TInValue<string > input_jack_tag_value(input_jack_tag, false);
    NMM::Event event("IVideoMux::getYPos", &input_jack_tag_value);
    event.setReturnValue(&return_value);
    if(const_cast<IVideoMux*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IVideoMux::IVideoMux_type = "IVideoMux";
  NMM::TConstructor<IVideoMux > __IVideoMux_prototype("IVideoMux","NMM");
  TValueConstructor< IVideoMux::Alignment > __IVideoMux_Alignment_prototype(TypeInfo("IVideoMux::Alignment", "NMM"));

  NMM::OValueStream& operator<<(NMM::OValueStream& ov, const IVideoMux::Alignment& t)
  {
    ov.beginType(TypeInfo("IVideoMux::Alignment", "NMM"));
    nmm_int32 temp = static_cast<nmm_int32>(t);
    ov << temp;
    ov.endType();
    return ov;
  }

  NMM::IValueStream& operator>>(NMM::IValueStream& iv, IVideoMux::Alignment& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("IVideoMux::Alignment", "NMM")) {
      throw NMM::InvalidTypeException(TypeInfo("IVideoMux::Alignment", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    nmm_int32 temp;
    iv >> temp;
    t = static_cast<IVideoMux::Alignment>(temp);
    iv.goUp();
    return iv;
  }



  const char* IVideoMux::getCurrentSyncTime_event = "IVideoMux::getCurrentSyncTime";
  const char* IVideoMux::setPosition_event = "IVideoMux::setPosition";
  const char* IVideoMux::setXPositionControl_event = "IVideoMux::setXPositionControl";
  const char* IVideoMux::setYPositionControl_event = "IVideoMux::setYPositionControl";
  const char* IVideoMux::setSize_event = "IVideoMux::setSize";
  const char* IVideoMux::setWidthControl_event = "IVideoMux::setWidthControl";
  const char* IVideoMux::setHeightControl_event = "IVideoMux::setHeightControl";
  const char* IVideoMux::setWidthHeightCouplingEnabled_event = "IVideoMux::setWidthHeightCouplingEnabled";
  const char* IVideoMux::getWidth_event = "IVideoMux::getWidth";
  const char* IVideoMux::getHeight_event = "IVideoMux::getHeight";
  const char* IVideoMux::getXPos_event = "IVideoMux::getXPos";
  const char* IVideoMux::getYPos_event = "IVideoMux::getYPos";


}


#include "nmm/nmm_suffix.hpp"

