/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "BorderDecorator.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  /*****************************************************************************************/
  /* BorderDecorator									 */
  /*****************************************************************************************/
  
  BorderDecorator::BorderDecorator() {
    setBorderColor( 0xFFFFFFFF );
    setBorderWidth( 2 );
  }
  
  void BorderDecorator::setBorderWidth( const nmm_uint32& bw ) {
    borderwidth = bw;
  }
  
  void BorderDecorator::setBorderColor( const nmm_uint32& color ) {
    bordercolor = color;
  }
  
  void BorderDecorator::draw( workspace_t* ws ) {
    workspace_t workspace;
    unsigned int w, h;

    /* first, draw the border */
    drawBorder( ws, borderwidth );

    /* we must adjust the workspace to be sure that the object is within the border */
    /* add borderwidth to x and y start position of the workspace */
    workspace.data   = ws->data+(borderwidth<<2)+((borderwidth*ws->stride)<<2);

    /* strink workspace width to width of borderdecorator - borderwith*2 (left and right) */
    if ( width > 0 ) w = xpos+width;
    else w = ws->width;

    if ( w > ws->width ) w = ws->width;
    workspace.width  = w  - (borderwidth<<1);
 
    /* strink workspace height to height of borderdecorator - borderwith*2 (top and bottom) */
    if ( height > 0 ) h = ypos+height;
    else h = ws->height;

    if ( h > ws->height ) h = ws->height;
    workspace.height = h - (borderwidth<<1);

    workspace.stride = ws->stride;

    /* then draw Widget Object */
    Decorator::draw( &workspace );
  }

  void BorderDecorator::drawBorder( workspace_t* ws, unsigned int bwidth ) {
    unsigned int* frame = (unsigned int*) ws->data; // pointer to workspace (destination)
    unsigned int  x, y;
    bool draw_it;
    unsigned int  xd, yd, xc, yc;

    unsigned int v_width, v_height;
    v_width  = adjustDimension( xpos, width, ws->width );
    v_height = adjustDimension( ypos, height, ws->height );

    xd = xpos+v_width-bwidth-1;
    yd = ypos+v_height-bwidth-1;

    xc = xpos+bwidth;
    yc = ypos+bwidth;

    for(y=ypos; y < (ypos+v_height); y++) {
      for( x=xpos; x < (xpos+v_width); x++ ) {
	draw_it = false;
	if ( (y<yc) || (y>yd) ) draw_it = true;
	else if ( (x<xc) || (x>xd) ) draw_it = true;

	if ( draw_it ) frame[x+(y*ws->stride)] = bordercolor;
      }
    } // y
  }

}

#include "nmm/nmm_suffix.hpp"
