/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "Buttons.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  /*****************************************************************************************/
  /* Buttons							                         */
  /*****************************************************************************************/

  Buttons::Buttons() {
    selected_button = -1;
    setStep( 1 );
  }

  Buttons::~Buttons() {
    deleteAllButtons();
  }
  
  void Buttons::draw( workspace_t* ws ) {
    buttonlist_t::iterator iter, stop;
    workspace_t workspace;

    /* begin of the image list */
    iter = buttonlist.begin();

    /* end of the image list */
    stop = buttonlist.end();

    while ( iter != stop ) {
      button_t button = iter->second;

      workspace.data   = ws->data+(button.x<<2)+(button.y*ws->stride<<2);
      workspace.width  = ws->width - button.x;
      workspace.height = ws->height - button.y;
      workspace.stride = ws->stride;

      overlayBitmap( button.id[button.state], &workspace );
      if ( iter->first == selected_button ) overlayBitmap( button.id[SELECTED], &workspace );
      iter++;
    }

  }
  
  void Buttons::selectRight() {
    selectButton( selected_button+1 );
    Widget::selectRight();
  }
  
  void Buttons::selectLeft() {
    selectButton( selected_button-1 );
    Widget::selectLeft();
  }
  
  void Buttons::selectUp() {
    selectButton( selected_button-step );
    Widget::selectUp();
  }
  
  void Buttons::selectDown() {
    selectButton( selected_button+step );
    Widget::selectDown();
  }
  
  void Buttons::selectPush() {
    /* set button state to "activated" */
    buttonlist[selected_button].state = ACTIVE;
    
    /* start callback functions */
    Widget::selectPush();
  }
  
  void Buttons::addButton(const nmm_int32& buttonid, const string& label, const nmm_int32& x, const nmm_int32& y,
                          const string& inactive_id, const string& active_id, const string& select_id) {
    /* create new button structure */
    button_t button;
    
    /* fill button structure with given data */
    button.label        = label;
    button.x            = x;
    button.y            = y;
    button.id[INACTIVE] = inactive_id;
    button.id[SELECTED] = select_id;
    button.id[ACTIVE]   = active_id;
    button.state        = INACTIVE;
    
    /* add button to button list */
    buttonlist[buttonid] = button;
  }
  
  string Buttons::getButtonLabel(const nmm_int32& button_id) {
    return( buttonlist[button_id].label );
  }
  
  string Buttons::getSelectedButtonLabel() {
    return( getButtonLabel( selected_button ) );
  }
  
  nmm_bool Buttons::getButtonState(const nmm_int32& button_id) {
    if ( buttonlist[button_id].state == ACTIVE ) return( true );
    else return( false );
  }
  
  void Buttons::deleteButton(const nmm_int32& button_id) {
    buttonlist.erase( button_id );
  }
  
  void Buttons::deleteAllButtons() {
    buttonlist.clear();
  }
  
  nmm_int32 Buttons::selectButton(const nmm_int32& buttonid) {
    buttonlist_t::iterator iter, stop;

    /* try to find button with buttonid */
    iter = buttonlist.find( buttonid );

    /* iterator to end of the list */
    stop = buttonlist.end();

    /* if button was found, mark it as selected */
    if ( iter != stop ) selected_button = buttonid;

    return( selected_button );
  }
  
  void Buttons::setStep(const nmm_uint32& value) {
    step = value;
  }
  
  void Buttons::deselectButton() {
    selected_button = -1;
  }
  
  void Buttons::activateButton(const nmm_int32& button_id) {
    /* set button state to "activated" */
    buttonlist[button_id].state = ACTIVE;
  }
  
  void Buttons::deactivateButton(const nmm_int32& button_id) {
    /* set button state to "inactive" */
    buttonlist[button_id].state = INACTIVE;
  }
  
  void Buttons::deactivateAllButtons() {
    buttonlist_t::iterator iter, stop;
    
    /* begin of the image(button) list */
    iter = buttonlist.begin();
    
    /* end of the image(button) list */
    stop = buttonlist.end();
    
    while ( iter != stop ) {
      iter->second.state = INACTIVE;
      iter++;
    }
  }
}

#include "nmm/nmm_suffix.hpp"
