/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef _NMM_CALLBACK_HPP_
#define _NMM_CALLBACK_HPP_ _NMM_CALLBACK_HPP_

#include "nmm/nmm_prefix.hpp"


namespace NMM {

  /**
  * Class to handle callback functions
  */
  class Callback {
    public:
      virtual ~Callback() {};

      /**
      * we need only a function to start the callback
      */
      virtual void run( void* ) {};
  };

  /**
  * a callback function consists of a class and a member function,
  * to handle the class name, we need a template
  */
  template<class cl>
  class TCallback : public Callback {
    /* the callback function gets a void pointer to pass data to it */
    typedef void (cl::*callbackfunc_t) ( void* );

    protected:
      /* pointer to the class itself */
      cl*              classdata;

      /* offset to member function */
      callbackfunc_t   callbackfunc;

    public:
      TCallback( cl* cldata, callbackfunc_t cbfunc ) {
        classdata    = cldata;
	callbackfunc = cbfunc;
      };

      void run( void* w ) {
        ((classdata)->*(callbackfunc))( w );
      };


  };



} // NMM

#include "nmm/nmm_suffix.hpp"

#endif //_NMM_CALLBACK_HPP_
