/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "Decorator.hpp"
#include "OSDManagerNode.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  Decorator::Decorator() : vcomponent (NULL) {
  }
  
  void Decorator::setOwnInternalWidget( Widget* vc ) {
    vcomponent = vc;
    
    /* set position to position of the vc object */
    vc->getPosition( xpos, ypos );
    
    /* set dimension to the dimension of the vc object */
    vc->getDimension( width, height );
  }
  
  Widget* Decorator::getOwnInternalWidget() const {
    return vcomponent;
  }
  
  IWidget* Decorator::getInternalWidget() const {
    if (vcomponent == NULL) { return NULL; }
    else { return vcomponent->getCheckedInterface<IWidget>(); }
  }
  
  void Decorator::setInternalWidget(const IWidget& widget) {
    setOwnInternalWidget(manager->getOwnWidget(widget));
  }
  
  void Decorator::setPosition( const nmm_uint32& x, const nmm_uint32& y ) {
    xpos = x;
    ypos = y;
    
    /* pass through */
    vcomponent->setPosition( x, y );
  }
  
  void Decorator::setDimension( const nmm_uint32& w, const nmm_uint32& h ) {
    width  = w;
    height = h;
    
    /* pass through */
    vcomponent->setDimension( w, h );
  }
  
  void Decorator::draw( workspace_t* ws ) {
    vcomponent->draw( ws );
  }
}

#include "nmm/nmm_suffix.hpp"
