/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "HeaderDecorator.hpp"
#include "OSDManagerNode.hpp"
#include "BitmapFont.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  /*****************************************************************************************/
  /* HeaderDecorator									 */
  /*****************************************************************************************/

  HeaderDecorator::HeaderDecorator() : font(NULL) { 
  }
  
  void HeaderDecorator::setFont( const string& value ) {
    font = manager->getFont(value);
  }
  
  void HeaderDecorator::setText( const string& value ) {
    headerstr = value;
  }
  
  void HeaderDecorator::draw( workspace_t* ws ) {
    workspace_t workspace;
    unsigned int ch, h;

    /* first, draw the Header */
    drawHeader( ws );

    ch = font->getMaxHeight();

    /* we must adjust the workspace because the height is changed */
    workspace.data   = ws->data+((ch*ws->stride)<<2);

    /* strink workspace height */
    if ( height > 0 ) h = ypos+height;
    else h = ws->height;

    if ( h > ws->height ) h = ws->height;
    workspace.height = h - ch;
    workspace.width  = ws->width;

    workspace.stride = ws->stride;

    /* then draw Widget Object */
    Decorator::draw( &workspace );
  }

  void HeaderDecorator::drawHeader( workspace_t* ws ) {
    unsigned int* frame = (unsigned int*) ws->data; // pointer to workspace (destination)

    unsigned int v_width, v_height;
    v_width  = adjustDimension( xpos, width, ws->width );
    v_height = adjustDimension( ypos, height, ws->height );

    textdraw.draw( headerstr.c_str(), font, frame, ws->stride,
		   xpos, ypos,
		   v_width+xpos, v_height+ypos );

  }

}

#include "nmm/nmm_suffix.hpp"
