/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef _NMM_DECOHEADER_HPP_
#define _NMM_DECOHEADER_HPP_ _NMM_DECOHEADER_HPP_

#include "nmm/nmm_prefix.hpp"

#include "Decorator.hpp"
#include "TextDraw.hpp"

#include "interfaces/IHeaderDecoratorImpl.hpp"

namespace NMM {
  /*****************************************************************************************/
  /* HeaderDecorator									   */
  /*****************************************************************************************/
  class HeaderDecorator : virtual public Decorator,
                          virtual public IHeaderDecoratorImpl {
  protected:
    /** font that is used to draw the text */
    BitmapFont*    font;
    
    /** class that contains some functions to draw text */
    TextDraw       textdraw;
    
    /** header text */
    string         headerstr;
    
    
    /**
     * Draws a header above a workspace
     * @param ws pointer to the workspace
     */
    void drawHeader( workspace_t* ws );
    
  public:
    HeaderDecorator();
    
    /** IHeaderDecorator interface. */
    virtual void setFont( const string& value );
    virtual void setText( const string& value );
    
    /** Draws Widget with Header. */
    virtual void draw( workspace_t* ws );
    
  }; // HeaderDecorator
} // NMM

#include "nmm/nmm_suffix.hpp"

#endif //_NMM_DECOHEADER_HPP_
