/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef _NMM_LISTUNIT_HPP_
#define _NMM_LISTUNIT_HPP_ _NMM_LISTUNIT_HPP_

#include "nmm/nmm_prefix.hpp"

#include "OSDElement.hpp"

#include "nmm/plugins/video/osd/manager/interfaces/ITextView.hpp"
#include "nmm/plugins/video/osd/manager/interfaces/IPlaneDecorator.hpp"
#include "nmm/plugins/video/osd/manager/interfaces/IBorderDecorator.hpp"
#include "nmm/plugins/video/osd/manager/interfaces/IScrollDecorator.hpp"
#include "nmm/plugins/video/osd/manager/interfaces/IHeaderDecorator.hpp"
#include "visibility.hpp"

namespace NMM {
  
  class DLLEXPORT ListUnit : public OSDElement {
  public:
    
    ListUnit(IOSDManager* manager, IWidgetFactory* factory, string font);
    ~ListUnit();
    
    ITextView* getTextWidget() { return( text.get() ); };
    
    void setColors( unsigned int win_active_color, unsigned int win_inactive_color,
                    unsigned int sel_active_color, unsigned int sel_active_color2, unsigned int sel_inactive_color );
    
    void selectWindow( unsigned int mode=0 );
    void deSelectWindow();
    void setHeader( string h );

    Result selectUp();
    Result selectDown();

    unsigned int getSelectedLine();
    string getLineURL(const unsigned int i);
    void selectLine(const unsigned int& i);
    string getLineContent(int i);
    void deleteAllLines();
    int addLine(string c, string url);
    int addLine(string c);
    void addMultiLine(string c);
    void deSelectLine();

    void markLine(unsigned int line);
    void unmarkLine(unsigned int line);
    list<unsigned int> getMarkedLines();

  protected:
    unsigned int  win_active_color;
    unsigned int  win_inactive_color;
    unsigned int  sel_active_color;
    unsigned int  sel_active_color2;
    unsigned int  sel_inactive_color;
    
    void deleteWidgets();
    
    string            font;
    
    ITextView_var     text;
    
    IPlaneDecorator_var   plane;
    IBorderDecorator_var  border;
    IScrollDecorator_var  scroll;
    IHeaderDecorator_var  header;
  };

}

#include "nmm/nmm_suffix.hpp"

#endif //_NMM_LISTUNIT_HPP_
