/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "MessageUnit.hpp"
#include <sstream>
#include "nmm/nmm_prefix.hpp"

namespace NMM {
  MessageUnit::MessageUnit( IOSDManager* manager, IWidgetFactory* factory, string _font ) : OSDElement(manager, factory),
                                                                                            font(_font) {
  }
  
  MessageUnit::~MessageUnit() {
    deleteWidgets();
  }
  
  Result MessageUnit::selectLeft() {
    if ( radiobuttons.get() ) {
      radiobuttons->selectLeft();
      win->update();
    }

    return(SUCCESS);
  }
  
  Result MessageUnit::selectRight() {
    if ( radiobuttons.get() ) {
      radiobuttons->selectRight();
      win->update();
    }

    return(SUCCESS);
  }
  
  string MessageUnit::messageID() {
    string ret = "";
    if ( radiobuttons.get() ) ret = radiobuttons->getSelectedButtonLabel();

    return( ret );
  }
  
  void MessageUnit::deleteWidgets() {
    if ( text.get() ) {
      manager->deleteWidget(*text);
      text.reset(NULL);
    }
    
    if ( composite.get() ) {
      manager->deleteWidget(*composite);
      composite.reset(NULL);
    }
    
    if ( plane.get() ) {
      manager->deleteWidget(*plane);
      plane.reset(NULL);
    }
    
    if ( border.get() ) {
      manager->deleteWidget(*border);
      border.reset(NULL);
    }
    
    if ( header.get() ) {
      manager->deleteWidget(*header);
      header.reset(NULL);
    }
    
    if ( radiobuttons.get() ) {
      manager->deleteWidget(*radiobuttons);
      radiobuttons.reset(NULL);
    }
  }
  
  Result MessageUnit::print( string headerstr, string textstr, string label1, string label2, string label3, unsigned int sel ) {
    unsigned int w;
    unsigned int h;
    
    w = 400;
    h = 100;
    
    deleteWidgets();
    
    composite.reset(factory->createWidgetComposite());
    
    /* text widget */
    text.reset(factory->createTextView());
    text->setFont( font );

    stringstream s(textstr);
    unsigned int linelength = 0;
    string tmp;
    string line;
    while(s>>tmp)
      {
        linelength += tmp.length() *20; //length of word in pixel (approx)
        
        if(tmp == "/"){
          text->addLine(line);
          h+=20;
          line="";
          linelength = 0;
        }else if(tmp == ":"){
          line += tmp + " ";
          text->addLine(line);
          h+=20;
          line="";
          linelength = 0; 
        }else if(tmp == "..."){
          line += tmp + " ";
          text->addLine(line);
          text->addLine(string());
          h+=40;
          line="";
          linelength = 0;
        }else{
          if(linelength < static_cast<unsigned int>(w))
            {
              line += tmp + " ";
            }else{  
              line += tmp + " ";
              text->addLine(line);
              h+=20;
              line="";
              linelength = 0;
            }
        }
      }
    if (line!="") {
      text->addLine(line);
      h+=20;
    }

    text->setPosition( 4,4 );
    
    composite->add( *text );
    
    /* buttons */
    IBitmapReader_var bmreader(manager->getBitmapReader());
    if ( bmreader.get() ) {
      unsigned int x = 0;
      unsigned int y = 0;
      unsigned int width1 = 0;
      unsigned int width2 = 0;
      unsigned int width3 = 0;
      unsigned int height1, height2, height3;
      unsigned int g_width = 0;
      unsigned int b_nr    = 1;
      
      radiobuttons.reset(factory->createRadioButtons());
      
      if ( label1 != "" ) {
        bmreader->getImage( "OSD_OK", width1, height1 );

	g_width += width1;
	b_nr++;
      }

      if ( label2 != "" ) {
        bmreader->getImage( "OSD_CANCEL", width2, height2 );

	g_width += width2;
	b_nr++;
      }

      if ( label3 != "" ) {
        bmreader->getImage( "OSD_NO", width3, height3 );

	g_width += width3;
	b_nr++;
      }

      g_width = (w-g_width)/b_nr;
      b_nr = 0;

      if ( label1 != "" ) {
        bmreader->createImage( "OSD_SELECT_OK", width1, height1, 0x80e1e1e1, 0xffb7b7b7, 0xffffffff, IBitmapReader::RGBA );

        x += g_width;
        y  = (h-height1)-30;
        radiobuttons->addButton( b_nr, label1, x, y, "OSD_OK", "", "OSD_SELECT_OK" );
	b_nr++;
      }

      if ( label2 != "" ) {
       bmreader->createImage( "OSD_SELECT_CANCEL", width2, height2, 0xa0e1e1e1, 0xffb7b7b7, 0xffffffff, IBitmapReader::RGBA );

        x  += g_width + width1;
        y   = (h-height2)-30;

        radiobuttons->addButton( b_nr, label2, x, y, "OSD_CANCEL", "", "OSD_SELECT_CANCEL" );
	b_nr++;
      }

      if ( label3 != "" ) {
       bmreader->createImage( "OSD_SELECT_NO", width3, height3, 0xa0e1e1e1, 0xffb7b7b7, 0xffffffff, IBitmapReader::RGBA );

        x  += g_width + width2;
        y   = (h-height3)-30;

        radiobuttons->addButton( b_nr, label3, x, y, "OSD_NO", "", "OSD_SELECT_CANCEL" );
	b_nr++;
      }

      radiobuttons->selectButton( sel );
      composite->add( *radiobuttons );
    }
    
    plane.reset(factory->createPlaneDecorator( *composite ));
    plane->setPlaneColor( 0xffff0000 );
    
    border.reset(factory->createBorderDecorator( *plane ));

    header.reset(factory->createHeaderDecorator( *border ));
    header->setFont( font );
    header->setText( headerstr );
    
    // create new YV12 window
    win->setDimension( w, h );
    win->setPosition( 150, 250-((h-100)/2) );
    win->setContent( *header );
    win->update();
    update();
    show();
    
    return SUCCESS;
  }
}

#include "nmm/nmm_suffix.hpp"
