/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "OSDElement.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  OSDElement::OSDElement(IOSDManager* _manager, IWidgetFactory* _factory) {
    manager.reset(dynamic_cast<IOSDManager*>(_manager->copy()));
    factory.reset(dynamic_cast<IWidgetFactory*>(_factory->copy()));
    
    win.reset( manager->createWindow());
  }
  
  OSDElement::~OSDElement() {
    if ( win.get() ) {
      manager->deleteWindow( *win );
      win.reset(NULL);
    }
  }
  
  IWindow* OSDElement::getWindow() {
    return win.get();
  }
  
  void OSDElement::show() {
    win->setVisible(true);
  }
  
  void OSDElement::hide() {
    win->setVisible(false);
  }
  
  bool OSDElement::isHidden() {
    return( !win->getVisible() );
  }
  
  void OSDElement::setDimension( unsigned int w, unsigned int h ) {
    width = w;
    height = h;
    win->setDimension( w, h );
  }
  
  void OSDElement::setPosition( unsigned int x, unsigned int y ) {
    win->setPosition( x, y );
  }
}

#include "nmm/nmm_suffix.hpp"
