/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "OSDElementComposit.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  OSDElementComposit::~OSDElementComposit() {
  }

  Result OSDElementComposit::selectUp() {
    if (focus>=elements.size())
      return FAILURE;
    return elements[focus]->selectUp();
  }

  Result OSDElementComposit::selectDown() {
    if (focus>=elements.size())
      return FAILURE;
    return elements[focus]->selectDown();
  }    

  int OSDElementComposit::addElement(OSDElementBase* elem) {
    elements.push_back(elem);
    return (elements.size()-1);
  }

  void OSDElementComposit::removeElement(OSDElementBase* elem) {
    for (pos=elements.begin(); pos != elements.end(); ++pos) {
      if ((*pos)==elem) {
        delete elem;
        (*pos)=NULL;
        break;
      }
    }
  }

  void OSDElementComposit::removeElement(int nr) {
    delete elements[nr];
    elements[nr]=NULL;
  }

  void OSDElementComposit::update() {
    for (pos=elements.begin(); pos != elements.end(); ++pos) {
      GLOBAL_DEBUG_STREAM("Updating OSDElement "<<(*pos)<<". (do not remove this line!)"<<endl);
      (*pos)->update();
    }
    manager->update();
  }
  
  void OSDElementComposit::show() {
    for (pos=elements.begin(); pos != elements.end(); ++pos) {
      (*pos)->show();
    }
    hidden=false;
  }
  
  void OSDElementComposit::hide() {
    for (pos=elements.begin(); pos != elements.end(); ++pos) {
    (*pos)->hide();
    }
    manager->hideSymbol();
    hidden=true;
  }
}

#include "nmm/nmm_suffix.hpp"
