/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#ifndef _NMM_OSDELEMENTCOMPOSIT_HPP
#define _NMM_OSDELEMENTCOMPOSIT_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/plugins/video/osd/manager/interfaces/IOSDManager.hpp"
#include "nmm/plugins/video/osd/manager/interfaces/IWidgetFactory.hpp"

#include "OSDElementBase.hpp"
#include "visibility.hpp"
namespace NMM {
  /**
   * OSD Element Composit
   */
  class DLLEXPORT OSDElementComposit:
    public OSDElementBase {

  public:

    virtual ~OSDElementComposit();
    typedef struct {
      unsigned int  xpos;
      unsigned int  ypos;
      unsigned int  width;
      unsigned int  height;
      unsigned int  win_active_color;
      unsigned int  win_inactive_color;
      unsigned int  sel_active_color;
      unsigned int  sel_active_color2;
      unsigned int  sel_inactive_color;
      string        name;
    } win_properties_t;

    virtual Result selectLeft()  = 0;
    virtual Result selectRight() = 0;
    virtual Result selectUp();
    virtual Result selectDown();

    int addElement(OSDElementBase*);
    void removeElement(OSDElementBase*);
    void removeElement(int);

    virtual void setWinProperties(int, win_properties_t* ) = 0;
    virtual void createWindows() = 0;
    
    void update();
    void show();
    void hide();
    inline bool isHidden() {return hidden;};

  protected:
    IOSDManager_var    manager;
    IWidgetFactory_var factory;
    vector<OSDElementBase*> elements;
    vector<OSDElementBase*>::iterator pos;
    bool hidden;
    uint focus;
  };
}

#include "nmm/nmm_suffix.hpp"

#endif
