/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "PlaneDecorator.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  /*****************************************************************************************/
  /* PlaneDecorator									 */
  /*****************************************************************************************/
  PlaneDecorator::PlaneDecorator() {
    setPlaneColor( 0xff000000 );
  }
  
  void PlaneDecorator::setPlaneColor( const nmm_uint32& color ) {
    planecolor = color;
  }
  
  void PlaneDecorator::draw( workspace_t* ws ) {

    drawPlane( ws );
    Decorator::draw( ws );
  }
  
  void PlaneDecorator::drawPlane( workspace_t* ws ) {
    unsigned int* frame = (unsigned int*) ws->data; // pointer to workspace (destination)
    unsigned int  x, y;

    unsigned int v_width, v_height;
    v_width  = adjustDimension( xpos, width, ws->width );
    v_height = adjustDimension( ypos, height, ws->height );

    for( y=ypos; y < (ypos+v_height); y++ ) {
      for( x=xpos; x < (xpos+v_width); x++ ) {
	frame[x+y*ws->stride]  = planecolor;
      }
    }
  }

}

#include "nmm/nmm_suffix.hpp"
