/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#include "ProgressBar.hpp"
#include "nmm/utils/time/Rational.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  /*****************************************************************************************/
  /* ProgressBar									         */
  /*****************************************************************************************/
  ProgressBar::ProgressBar() {
    setMaxProgress( 100 );
    setProgress( 0 );
    setBarColor( 0xff808080 );
    setBarSpaces( 4, 16 );
  }

  ProgressBar::~ProgressBar() {
  }
  
  void ProgressBar::drawBar(workspace_t* ws) {
    // pointer to workspace (destination)
    // we work on the data interpreted as integers, representing the pixel-color
    unsigned int* frame = reinterpret_cast<unsigned int*>(ws->data);
    
    // adjust width and height so that they don't extend the workspace dimension
    unsigned int v_width  = adjustDimension(xpos, width, ws->width);
    unsigned int v_height = adjustDimension(ypos, height, ws->height);
    
    // compute the percentual fill
    Rational rat_width = Rational(v_width) * Rational(progress, max);
    v_width = rat_width.getNumerator() / rat_width.getDenominator();
    // a method Rational::getIntegerPart() would be handy here!
    
    // draw bar
    // for all horizontal lines
    for (unsigned int y = ypos;
         y < (ypos+v_height);
         y++) {
      // sc stores our position in one bar-space-group
      unsigned int sc = 0;
      // for each pixel from 0 to v_width(=percentual fill)
      for (unsigned int x = xpos;
           x < (xpos+v_width);
           x++, sc++) {
	if (sc >= space_step+space) {
          // if we reached the end of our bar-space-group
          // we reset the according position
          sc = 0;
        }
	if (sc < space_step) {
          // if the position in out bar-space-group indicates, that we
          // are still in the "bar", draw the pixel with the bar-color
          frame[x+(y*ws->stride)]  = barcolor;
        }
      }
    }
  }

  void ProgressBar::draw( workspace_t* ws ) {
    drawBar( ws );
  }
  
  void ProgressBar::setMaxProgress(const nmm_uint64& value) {
    max = value;
    if ( max < progress ) setProgress( max );
  }
  
  void ProgressBar::setProgress(const nmm_uint64& value) {
    /* check if progress is valid */
    if ( progress<=max ) progress = value;
    else progress = 0;
  }
  
  void ProgressBar::setBarColor(const nmm_uint32& value) {
    barcolor = value;
  }
  
  void ProgressBar::setBarSpaces(const nmm_uint32& space, const nmm_uint32& space_step) {
    this->space      = space;
    this->space_step = space_step;
  }
}

#include "nmm/nmm_suffix.hpp"
