/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "ScrollDecorator.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  /*****************************************************************************************/
  /* ScrollDecorator									 */
  /*****************************************************************************************/
  ScrollDecorator::ScrollDecorator() {
    setBarWidth( 20 );
    setBackgroundColor( 0xffa0a0a0 );
    setBorderColor( 0xff4e4e4e );
    setBarColor( 0xffe1e1e1 );
    setBarBorderColors( 0xffffffff, 0xffb7b7b7 );
  }
  
  void ScrollDecorator::setBarWidth( const nmm_uint32& bw ) {
    barwidth = bw;
  }
  
  void ScrollDecorator::setBackgroundColor( const nmm_uint32& bcolor ) {
    backgroundcolor = bcolor;
  }
  
  void ScrollDecorator::setBorderColor( const nmm_uint32& bcolor ) {
    bordercolor = bcolor;
  }
  
  void ScrollDecorator::setBarColor( const nmm_uint32& bcolor ) {
    barcolor = bcolor;
  }
  
  void ScrollDecorator::setBarBorderColors( const nmm_uint32& bcolor1, const nmm_uint32& bcolor2 ) {
    barbordercolor1 = bcolor1;
    barbordercolor2 = bcolor2;
  }
  
  void ScrollDecorator::draw( workspace_t* ws ) {
    workspace_t workspace;
    unsigned int w, h;

    /* we must adjust the workspace to be sure that the object dosn't overwrite the scrollbar */
    workspace.data   = ws->data;

    if ( width > 0 ) w = xpos+width;
    else w = ws->width;

    if ( w > ws->width ) w = ws->width;
    workspace.width  = w  - barwidth;

    if ( height > 0 ) h = ypos+height;
    else height = ws->height;

    if ( h > ws->height ) h = ws->height;
    workspace.height = h;

    workspace.stride = ws->stride;

    Decorator::draw( &workspace );

    drawScrollBar( ws );
  }

  void ScrollDecorator::drawFilledBox( unsigned int* frame, unsigned int stride,
				       unsigned int xs, unsigned int ys,
				       unsigned int width, unsigned int height,
				       unsigned int color ) {

    for( unsigned int y=ys; y < (ys+height); y++ ) {
      for( unsigned int x=xs; x < (xs+width); x++ ) {
	frame[x+y*stride]  = color;
      }
    }

  }

  void ScrollDecorator::drawBox( unsigned int* frame, unsigned int stride,
				 unsigned int xs, unsigned int ys,
				 unsigned int width, unsigned int height,
				 unsigned int color1, unsigned int color2 ) {
    unsigned int  x, y;
    unsigned int  xd, yd;

    xd = xs+width-1;
    yd = ys+height-1;

    for(x=xs; x <= xd; x++) {
      frame[x+ys*stride]     = color1;
      frame[x+(ys+1)*stride] = color1;

      frame[x+(yd*stride)]     = color2;
      frame[x+((yd-1)*stride)] = color2;
    }

    for(y=ys+1; y <= yd; y++) {
      frame[xs+(y*stride)]   = color1;
      frame[xs+1+(y*stride)] = color1;

      frame[xd+(y*stride)]   = color2;
      frame[xd-1+(y*stride)] = color2;
    }

  }


  void ScrollDecorator::drawScrollBar( workspace_t* ws ) {

    unsigned int v_width, v_height;
    unsigned int sh, sy;
    v_width  = adjustDimension( xpos, width, ws->width );
    v_height = adjustDimension( ypos, height, ws->height );

    /* get scrollbar height information from widget and adjust it to the scrollbar height */
    sh = (vcomponent->getScrollBarHeight() * v_height)/10000;

    /* get scrollbar position information from widget and adjust it */
    sy = (vcomponent->getScrollBarYPosition() * v_height)/10000 + ypos;
    /* don't set it to 0 */
    if (sy==0) sy=1;

    /* draw the scrollbar background */
    drawFilledBox( (unsigned int*) ws->data, ws->stride, xpos+v_width-barwidth, ypos,
		   barwidth, v_height, backgroundcolor );

    drawBox( (unsigned int*) ws->data, ws->stride, xpos+v_width-barwidth, ypos,
	     barwidth, v_height, bordercolor, bordercolor );

    drawFilledBox( (unsigned int*) ws->data, ws->stride, xpos+v_width-barwidth+2, sy,
		   barwidth-4, sh, barcolor );

    drawBox( (unsigned int*) ws->data, ws->stride, xpos+v_width-barwidth+2, sy,
	     barwidth-4, sh, barbordercolor1, barbordercolor2  );


  }

}

#include "nmm/nmm_suffix.hpp"
