/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "TextDraw.hpp"
#include "overlay.h"
#include "BitmapFont.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  bool TextDraw::draw( const char* text, BitmapFont* font, unsigned int* frame, unsigned int stride,
		       unsigned int xstart, unsigned int ystart,
		       unsigned int xend, unsigned int yend ) {

    char c;	       // actual character
    unsigned int cw, ch; // character width and height
    unsigned int c_stride;
    int linelength;      // number of chars in the actual line
    unsigned char* data; // pointer to character pixel data (source)
    unsigned char* pixel; // pointer to character pixel data (source)
    bool x_clip, y_clip;

    int i;

    i = 0;
    linelength = strlen( text );

    /* get one character out of the string */
    while( i < linelength ) {
      c = text[i];

      /* check width and height of the character */
      font->checkDimension( c, &cw, &ch );
      /* get pointer to character pixel data */
      data = font->getPixelData( c );
      // if no pixel data are available, produce a space of 10 pixels
      if ( !data ) cw = 10;

      y_clip = (ystart+ch) >= yend;
      if ( y_clip ) {
	if ( yend > ystart ) ch = yend-ystart;
	else ch = 0;
      }

      c_stride = cw;
      x_clip = (xstart+cw) >= xend;
      if ( x_clip ) {
	if ( xend > xstart ) cw = xend-xstart;
	else cw = 0;
      }

      pixel = (unsigned char*) (frame + ystart*stride + xstart);

      RGBAoverlay( data, c_stride, cw, ch,
		   pixel, stride );

      /* index of next character of the string */
      i++;

      if ( (x_clip || i>=linelength) && y_clip ) return( true );
      xstart+=cw; // next character start position

    }

    return( false );

  }

}

#include "nmm/nmm_suffix.hpp"
