/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "TimeUnit.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  TimeUnit::TimeUnit( IOSDManager* manager, IWidgetFactory* factory, string _font ) : OSDElement(manager, factory),
                                                                                      font(_font) {
    setColors( 0x80ff0000 );
    
    text.reset(factory->createTimeView());
    text->setFont(font);
    plane.reset(factory->createPlaneDecorator( *text ));
    plane->setPlaneColor( color1 );
    border.reset(factory->createBorderDecorator( *plane ));
    header.reset(factory->createHeaderDecorator( *border ));
    header->setFont(font);
    
    win->setDimension( manager->getMaxWidth(font)*4, manager->getMaxHeight(font)+24 );
    win->setContent( *header );
  }
  
  TimeUnit::~TimeUnit() {
    deleteWidgets();
  }

  void TimeUnit::deleteWidgets() {
    if ( text.get() ) {
      manager->deleteWidget(*text);
      text.reset(NULL);
    }

    if ( plane.get() ) {
      manager->deleteWidget(*plane);
      plane.reset(NULL);
    }
    
    if ( border.get() ) {
      manager->deleteWidget(*border);
      border.reset(NULL);
    }

    if ( header.get() ) {
      manager->deleteWidget(*header);
      header.reset(NULL);
    }
  }
  
  void TimeUnit::setHeader( string h ) {
    header->setText( h );
  }
  
  void TimeUnit::setColors( unsigned int _color1 ) {
    color1 = _color1;
  }
}

#include "nmm/nmm_suffix.hpp"
