/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "TimeView.hpp"
#include "OSDManagerNode.hpp"

#include "overlay.h"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  /*****************************************************************************************/
  /* TimeView                                                                              */
  /*****************************************************************************************/

  TimeView::TimeView( BitmapFont* bfont ) {
    font = bfont;
    
    reset();
    setPrefix( "" );
  }
  
  TimeView::~TimeView() {
  }
  
  void TimeView::setFont( const string& fontname ) {
    font = manager->getFont( fontname );
  }
  
  void TimeView::reset() {
    current_time = 0;
    setTimeOffset( 0 );
  }
  
  void TimeView::setPrefix( const string& p ) {
    prefix = p;
  }

  void TimeView::setTimeOffset( const nmm_int32& to ) {
    timeoffset = to;
  }

  void TimeView::setCurrentTime( const nmm_int32& t ) {
    current_time = t;
  }

  nmm_int32 TimeView::getCurrentTime() {
    return current_time;
  }
  
  
  void TimeView::draw( workspace_t* ws ) {
    unsigned int v_width, v_height;
    unsigned int h, m, s;
    long         sec;
    char         timestr[200];
    
    v_width  = adjustDimension( xpos, width, ws->width );
    v_height = adjustDimension( ypos, height, ws->height );
    
    unsigned int* frame = (unsigned int*) ws->data;
    
    sec = current_time + timeoffset;
    h   = sec/3600;
    
    sec = sec - 3600*h;
    m   = sec/60;

    s   = sec - 60*m;

    sprintf( timestr, "%s%02d:%02d:%02d", prefix.c_str(), h, m, s );
    textdraw.draw( timestr, font, frame, ws->stride,
		   xpos, ypos,
		   v_width+xpos, v_height+ypos );

  }

}

#include "nmm/nmm_suffix.hpp"
