/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef _NMM_WGTIMEVIEW_HPP_
#define _NMM_WGTIMEVIEW_HPP_ _NMM_WGTIMEVIEW_HPP_

#include "nmm/nmm_prefix.hpp"

#include "Widget.hpp"
#include "TextDraw.hpp"

#include "nmm/plugins/video/osd/manager/interfaces/ITimeViewImpl.hpp"
#include "visibility.hpp"

namespace NMM {
  
  /**
   * Time view widget.
   * Displays a time.
   */
  class DLLEXPORT TimeView : virtual public Widget,
                   virtual public ITimeViewImpl {
    
  protected:
    /** font that is used to draw the text */
    BitmapFont*    font;
    
    /** class that contains some functions to draw text */
    TextDraw       textdraw;
    
    /** text that should be placed in front of the time */
    string prefix;
    
    /** offset that is added to the time */
    long   timeoffset;
    
    time_t current_time;
    
  public:
    TimeView( BitmapFont* bfont = NULL );
    virtual ~TimeView();
    
    /** ITimeView interface. */
    void setFont(const string& value);
    void setPrefix(const string& value);
    void setTimeOffset(const nmm_int32& value);
    void setCurrentTime(const nmm_int32& value);
    nmm_int32 getCurrentTime();
    void reset();
    
    void draw( workspace_t* ws );
  }; // TextView

}

#include "nmm/nmm_suffix.hpp"

#endif //_NMM_WGTIMEVIEW_HPP_
