/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include<stdio.h>


double getYComponent( unsigned char R, unsigned char G, unsigned char B ) {
  return( R *  0.299 + G *  0.587 + B *  0.114 );
}

double getUComponent( unsigned char R, unsigned char G, unsigned char B ) {
  return( R * -0.169 + G * -0.332 + B *  0.500 + 128 );
}

double getVComponent( unsigned char R, unsigned char G, unsigned char B ) {
  return( R *  0.500 + G * -0.419 + B * -0.0813 + 128 );
}

void RGBAtoYV12A(unsigned char* yuv_buffer, unsigned char* rgbbuffer, int width, int height)
{

    int plane_size = width * height;

    unsigned char* y_plane = yuv_buffer;
    unsigned char* u_plane = yuv_buffer + plane_size;
    unsigned char* v_plane = u_plane + plane_size/4;

    unsigned char *rgba1, *yuv1;
    unsigned char *rgba2, *yuv2;
    unsigned char *rgba3, *yuv3;
    unsigned char *rgba4, *yuv4;
    unsigned char r1, r2, r3, r4, g1, g2, g3, g4, b1, b2, b3, b4;

    unsigned char *yuv_alpha;
    unsigned char *alpha;
    int row, col;

    for (row = 0; row < height/2; row++) {
        rgba1 = rgbbuffer+(row*width << 3);             // rgbbuffer+(2*row*width*4);
        rgba2 = rgba1 + 4;
        rgba3 = rgbbuffer+((2*row+1)* (width<<2));      // rgbbuffer+((2*row+1)*width*4);
        rgba4 = rgba3 + 4;
        yuv1  = y_plane + 2*row * width;
        yuv2  = yuv1 + 1;
        yuv3  = y_plane + (2*row + 1) * width;
        yuv4  = yuv3 + 1;
        
       

        for (col=0; col < width/2; col++) {
            b1 = rgba1[0];
            g1 = rgba1[1];
            r1 = rgba1[2];
            *yuv1 = (unsigned char) getYComponent( r1, g1, b1 );
            
            b2 = rgba2[0];
            g2 = rgba2[1];
            r2 = rgba2[2];
            *yuv2 = (unsigned char) getYComponent( r2, g2, b2 );
            
            b3 = rgba3[0];
            g3 = rgba3[1];
            r3 = rgba3[2];
            *yuv3 = (unsigned char) getYComponent( r3, g3, b3 );
            
            b4 = rgba4[0];
            g4 = rgba4[1];
            r4 = rgba4[2];
            *yuv4 = (unsigned char) getYComponent( r4, g4, b4 );
            
            r1 = (r1+r2+r3+r4)>>2;
            g1 = (g1+g2+g3+g4)>>2;
            b1 = (b1+b2+b3+b4)>>2;
            *u_plane = (unsigned char) getUComponent( r1, g1, b1 );
            *v_plane = (unsigned char) getVComponent( r1, g1, b1 );
            u_plane++; v_plane++;
            
            rgba1 += 8;
            rgba2 = rgba1 + 4;
            rgba3 += 8;
            rgba4 = rgba3 + 4;
            yuv1  += 2;
            yuv2  = yuv1 + 1;
            yuv3  += 2;
            yuv4  = yuv3 + 1;

           
        }//end for col
    }//end for row

    // copy alpha from rgb image to end of yuv_buffer 
    yuv_alpha = yuv_buffer + plane_size + plane_size/2;    
    for (row=0; row < height; row++) {
      alpha = rgbbuffer+(row*width<<2);
      for (col=0; col < width; col++) {
	*yuv_alpha = alpha[3];
	alpha += 4;
	yuv_alpha++;
      }
    }

}
