/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IBorderDecorator.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IBorderDecorator::used() {} 
  
  
  void IBorderDecorator::setBorderWidth(const nmm_uint32& bw)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > bw_value(bw, false);
    NMM::Event event("IBorderDecorator::setBorderWidth", &bw_value);
    if(const_cast<IBorderDecorator*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IBorderDecorator::setBorderColor(const nmm_uint32& color)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > color_value(color, false);
    NMM::Event event("IBorderDecorator::setBorderColor", &color_value);
    if(const_cast<IBorderDecorator*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IBorderDecorator::IBorderDecorator_type = "IBorderDecorator";
  NMM::TConstructor<IBorderDecorator > __IBorderDecorator_prototype("IBorderDecorator","NMM");


  const char* IBorderDecorator::setBorderWidth_event = "IBorderDecorator::setBorderWidth";
  const char* IBorderDecorator::setBorderColor_event = "IBorderDecorator::setBorderColor";


}


#include "nmm/nmm_suffix.hpp"

