/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IBorderDecorator_HPP
#define NMM_IBorderDecorator_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "IDecorator.hpp"

#include "visibility.hpp"

namespace NMM
{
  /**
   * Border decorator interface.
   */

  class DLLEXPORT IBorderDecorator : virtual public IDecorator { 
  public:
    IBorderDecorator() : NMM::Interface(IBorderDecorator::IBorderDecorator_type, "NMM") { };

    IBorderDecorator(NMM::NMMObject& parent) : NMM::Interface (parent, IBorderDecorator::IBorderDecorator_type, "NMM") { };

    virtual ~IBorderDecorator() {};

    static const char* IBorderDecorator_type;

    virtual  string getInterfaceType() const { return IBorderDecorator_type; }

    NMM::Interface* copy() const { return new IBorderDecorator(*this); }
    static void used(); 
    static const char* setBorderWidth_event;
    static const char* setBorderColor_event;
    /**
     * Set border width.
     * @param bw border width
     */

    virtual void setBorderWidth(const nmm_uint32& bw)
      throw(const NMM::SerializedException&);
    /**
     * Set boder color.
     * @param color border color (rgba format)
     */

    virtual void setBorderColor(const nmm_uint32& color)
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IBorderDecorator> IBorderDecorator_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
