/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IButtons.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IButtons::used() {} 
  
  
  void IButtons::addButton(const nmm_int32& buttonid, const string& label, const nmm_int32& x, const nmm_int32& y, const string& inactive_id, const string& active_id, const string& select_id)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > buttonid_value(buttonid, false);
    NMM::TInValue<string > label_value(label, false);
    NMM::TInValue<nmm_int32 > x_value(x, false);
    NMM::TInValue<nmm_int32 > y_value(y, false);
    NMM::TInValue<string > inactive_id_value(inactive_id, false);
    NMM::TInValue<string > active_id_value(active_id, false);
    NMM::TInValue<string > select_id_value(select_id, false);
    NMM::Event event("IButtons::addButton", &buttonid_value, &label_value, &x_value, &y_value, &inactive_id_value, &active_id_value, &select_id_value);
    if(const_cast<IButtons*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  string IButtons::getButtonLabel(const nmm_int32& button_id)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::TInValue<nmm_int32 > button_id_value(button_id, false);
    NMM::Event event("IButtons::getButtonLabel", &button_id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IButtons*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string IButtons::getSelectedButtonLabel()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("IButtons::getSelectedButtonLabel");
    event.setReturnValue(&return_value);
    if(const_cast<IButtons*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool IButtons::getButtonState(const nmm_int32& button_id)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::TInValue<nmm_int32 > button_id_value(button_id, false);
    NMM::Event event("IButtons::getButtonState", &button_id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IButtons*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IButtons::deleteButton(const nmm_int32& button_id)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > button_id_value(button_id, false);
    NMM::Event event("IButtons::deleteButton", &button_id_value);
    if(const_cast<IButtons*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IButtons::deleteAllButtons()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IButtons::deleteAllButtons");
    if(const_cast<IButtons*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_int32 IButtons::selectButton(const nmm_int32& buttonid)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::TInValue<nmm_int32 > buttonid_value(buttonid, false);
    NMM::Event event("IButtons::selectButton", &buttonid_value);
    event.setReturnValue(&return_value);
    if(const_cast<IButtons*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IButtons::setStep(const nmm_uint32& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > value_value(value, false);
    NMM::Event event("IButtons::setStep", &value_value);
    if(const_cast<IButtons*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IButtons::deselectButton()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IButtons::deselectButton");
    if(const_cast<IButtons*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IButtons::activateButton(const nmm_int32& button_id)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > button_id_value(button_id, false);
    NMM::Event event("IButtons::activateButton", &button_id_value);
    if(const_cast<IButtons*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IButtons::deactivateButton(const nmm_int32& button_id)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > button_id_value(button_id, false);
    NMM::Event event("IButtons::deactivateButton", &button_id_value);
    if(const_cast<IButtons*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IButtons::deactivateAllButtons()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IButtons::deactivateAllButtons");
    if(const_cast<IButtons*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IButtons::IButtons_type = "IButtons";
  NMM::TConstructor<IButtons > __IButtons_prototype("IButtons","NMM");


  const char* IButtons::addButton_event = "IButtons::addButton";
  const char* IButtons::getButtonLabel_event = "IButtons::getButtonLabel";
  const char* IButtons::getSelectedButtonLabel_event = "IButtons::getSelectedButtonLabel";
  const char* IButtons::getButtonState_event = "IButtons::getButtonState";
  const char* IButtons::deleteButton_event = "IButtons::deleteButton";
  const char* IButtons::deleteAllButtons_event = "IButtons::deleteAllButtons";
  const char* IButtons::selectButton_event = "IButtons::selectButton";
  const char* IButtons::setStep_event = "IButtons::setStep";
  const char* IButtons::deselectButton_event = "IButtons::deselectButton";
  const char* IButtons::activateButton_event = "IButtons::activateButton";
  const char* IButtons::deactivateButton_event = "IButtons::deactivateButton";
  const char* IButtons::deactivateAllButtons_event = "IButtons::deactivateAllButtons";


}


#include "nmm/nmm_suffix.hpp"

