/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IDecorator_HPP
#define NMM_IDecorator_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "IWidget.hpp"

#include "visibility.hpp"

namespace NMM
{
  /**
   * Decorator interface.
   */

  class DLLEXPORT IDecorator : virtual public IWidget { 
  public:
    IDecorator() : NMM::Interface(IDecorator::IDecorator_type, "NMM") { };

    IDecorator(NMM::NMMObject& parent) : NMM::Interface (parent, IDecorator::IDecorator_type, "NMM") { };

    virtual ~IDecorator() {};

    static const char* IDecorator_type;

    virtual  string getInterfaceType() const { return IDecorator_type; }

    NMM::Interface* copy() const { return new IDecorator(*this); }
    static void used(); 
    static const char* getInternalWidget_event;
    static const char* setInternalWidget_event;
    /** Get internal widget. */

    virtual IWidget* getInternalWidget() const
      throw(const NMM::SerializedException&);
    /** Set internal widget. */

    virtual void setInternalWidget(const IWidget& widget)
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IDecorator> IDecorator_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
