/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IHeaderDecorator.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IHeaderDecorator::used() {} 
  
  
  void IHeaderDecorator::setFont(const string& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > value_value(value, false);
    NMM::Event event("IHeaderDecorator::setFont", &value_value);
    if(const_cast<IHeaderDecorator*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IHeaderDecorator::setText(const string& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > value_value(value, false);
    NMM::Event event("IHeaderDecorator::setText", &value_value);
    if(const_cast<IHeaderDecorator*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IHeaderDecorator::IHeaderDecorator_type = "IHeaderDecorator";
  NMM::TConstructor<IHeaderDecorator > __IHeaderDecorator_prototype("IHeaderDecorator","NMM");


  const char* IHeaderDecorator::setFont_event = "IHeaderDecorator::setFont";
  const char* IHeaderDecorator::setText_event = "IHeaderDecorator::setText";


}


#include "nmm/nmm_suffix.hpp"

