/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IHeaderDecorator_HPP
#define NMM_IHeaderDecorator_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "IDecorator.hpp"

#include "visibility.hpp"

namespace NMM
{
  /**
   * Header decorator interface.
   */

  class DLLEXPORT IHeaderDecorator : virtual public IDecorator { 
  public:
    IHeaderDecorator() : NMM::Interface(IHeaderDecorator::IHeaderDecorator_type, "NMM") { };

    IHeaderDecorator(NMM::NMMObject& parent) : NMM::Interface (parent, IHeaderDecorator::IHeaderDecorator_type, "NMM") { };

    virtual ~IHeaderDecorator() {};

    static const char* IHeaderDecorator_type;

    virtual  string getInterfaceType() const { return IHeaderDecorator_type; }

    NMM::Interface* copy() const { return new IHeaderDecorator(*this); }
    static void used(); 
    static const char* setFont_event;
    static const char* setText_event;
    /**
     * Set header font.
     * Set the font that is used to draw the text.
     * @param value font name
     */

    virtual void setFont(const string& value)
      throw(const NMM::SerializedException&);
    /**
     * Set header text.
     * Set the text that should be displayed in the header.
     * @param value header text
     */

    virtual void setText(const string& value)
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IHeaderDecorator> IHeaderDecorator_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
