/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IImageView_HPP
#define NMM_IImageView_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "IWidget.hpp"

#include "visibility.hpp"

namespace NMM
{
  /**
   * Image view widget interface.
   */

  class DLLEXPORT IImageView : virtual public IWidget { 
  public:
    IImageView() : NMM::Interface(IImageView::IImageView_type, "NMM") { };

    IImageView(NMM::NMMObject& parent) : NMM::Interface (parent, IImageView::IImageView_type, "NMM") { };

    virtual ~IImageView() {};

    static const char* IImageView_type;

    virtual  string getInterfaceType() const { return IImageView_type; }

    NMM::Interface* copy() const { return new IImageView(*this); }
    static void used(); 
    static const char* setBitmapID_event;
    /**
     * Set bitmap identifier.
     * Set identifier of bitmap that should be drawn.
     * @param id bitmap identifier
     */

    virtual void setBitmapID(const string& id)
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IImageView> IImageView_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
