/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IOSDManager.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/serialize/StdValue.hpp"

namespace NMM
{
  void IOSDManager::used() {} 
  
  
  Result IOSDManager::loadFont(const string& id, const string& xmlfilename, const string& fontname)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > id_value(id, false);
    NMM::TInValue<string > xmlfilename_value(xmlfilename, false);
    NMM::TInValue<string > fontname_value(fontname, false);
    NMM::Event event("IOSDManager::loadFont", &id_value, &xmlfilename_value, &fontname_value);
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<string >  IOSDManager::getFontList() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<list<string >  > return_value;
    NMM::Event event("IOSDManager::getFontList");
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::update()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("IOSDManager::update");
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IWidget* IOSDManager::getWidget(const nmm_int32& id)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<IWidget* > return_value;
    NMM::TInValue<nmm_int32 > id_value(id, false);
    NMM::Event event("IOSDManager::getWidget", &id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IWindow* IOSDManager::getWindow(const nmm_int32& id)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<IWindow* > return_value;
    NMM::TInValue<nmm_int32 > id_value(id, false);
    NMM::Event event("IOSDManager::getWindow", &id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IWindow* IOSDManager::createWindow()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<IWindow* > return_value;
    NMM::Event event("IOSDManager::createWindow");
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IOSDManager::deleteWindow(const IWindow& window)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<IWindow > window_value(window, false);
    NMM::Event event("IOSDManager::deleteWindow", &window_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IOSDManager::deleteWidget(const IWidget& widget)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<IWidget > widget_value(widget, false);
    NMM::Event event("IOSDManager::deleteWidget", &widget_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  Result IOSDManager::displayMessage(const string& header, const string& text, const string& label1, const string& label2 /* = "" */, const string& label3 /* = "" */, const nmm_uint32& sel /* = 0 */)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > header_value(header, false);
    NMM::TInValue<string > text_value(text, false);
    NMM::TInValue<string > label1_value(label1, false);
    NMM::TInValue<string > label2_value(label2, false);
    NMM::TInValue<string > label3_value(label3, false);
    NMM::TInValue<nmm_uint32 > sel_value(sel, false);
    NMM::Event event("IOSDManager::displayMessage", &header_value, &text_value, &label1_value, &label2_value, &label3_value, &sel_value);
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::deleteMessage()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("IOSDManager::deleteMessage");
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::displayTime(const string& header /* = "" */, const nmm_uint32& x /* = 500 */, const nmm_uint32& y /* = 410 */)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > header_value(header, false);
    NMM::TInValue<nmm_uint32 > x_value(x, false);
    NMM::TInValue<nmm_uint32 > y_value(y, false);
    NMM::Event event("IOSDManager::displayTime", &header_value, &x_value, &y_value);
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::createTime(const nmm_bool& ct)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_bool > ct_value(ct, false);
    NMM::Event event("IOSDManager::createTime", &ct_value);
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::resetTime()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("IOSDManager::resetTime");
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::hideTime()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("IOSDManager::hideTime");
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::setTime(const nmm_uint64& time)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_uint64 > time_value(time, false);
    NMM::Event event("IOSDManager::setTime", &time_value);
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint64 IOSDManager::getTime()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint64 > return_value;
    NMM::Event event("IOSDManager::getTime");
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::displaySymbol(const string& symbol, const nmm_uint32& x /* = 600 */, const nmm_uint32& y /* = 50 */)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > symbol_value(symbol, false);
    NMM::TInValue<nmm_uint32 > x_value(x, false);
    NMM::TInValue<nmm_uint32 > y_value(y, false);
    NMM::Event event("IOSDManager::displaySymbol", &symbol_value, &x_value, &y_value);
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::hideSymbol()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("IOSDManager::hideSymbol");
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::displayProgressBar(const string& header /* = "" */, const nmm_uint32& x /* = 50 */, const nmm_uint32& y /* = 470 */, const nmm_uint32& width /* = 620 */)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > header_value(header, false);
    NMM::TInValue<nmm_uint32 > x_value(x, false);
    NMM::TInValue<nmm_uint32 > y_value(y, false);
    NMM::TInValue<nmm_uint32 > width_value(width, false);
    NMM::Event event("IOSDManager::displayProgressBar", &header_value, &x_value, &y_value, &width_value);
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::hideProgressBar()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("IOSDManager::hideProgressBar");
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IOSDManager::setMaxProgress(const nmm_uint64& max)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint64 > max_value(max, false);
    NMM::Event event("IOSDManager::setMaxProgress", &max_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IOSDManager::setProgress(const nmm_uint64& progress)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint64 > progress_value(progress, false);
    NMM::Event event("IOSDManager::setProgress", &progress_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IOSDManager::updateProgress()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IOSDManager::updateProgress");
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  Result IOSDManager::displayColorBar(const nmm_uint32& x /* = 80 */, const nmm_uint32& y /* = 480 */)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_uint32 > x_value(x, false);
    NMM::TInValue<nmm_uint32 > y_value(y, false);
    NMM::Event event("IOSDManager::displayColorBar", &x_value, &y_value);
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::hideColorBar()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("IOSDManager::hideColorBar");
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IOSDManager::setColorBarText(const string& red, const string& green, const string& yellow, const string& blue)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > red_value(red, false);
    NMM::TInValue<string > green_value(green, false);
    NMM::TInValue<string > yellow_value(yellow, false);
    NMM::TInValue<string > blue_value(blue, false);
    NMM::Event event("IOSDManager::setColorBarText", &red_value, &green_value, &yellow_value, &blue_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IOSDManager::setRedColorBarText(const string& text)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > text_value(text, false);
    NMM::Event event("IOSDManager::setRedColorBarText", &text_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IOSDManager::setGreenColorBarText(const string& text)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > text_value(text, false);
    NMM::Event event("IOSDManager::setGreenColorBarText", &text_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IOSDManager::setYellowColorBarText(const string& text)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > text_value(text, false);
    NMM::Event event("IOSDManager::setYellowColorBarText", &text_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IOSDManager::setBlueColorBarText(const string& text)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > text_value(text, false);
    NMM::Event event("IOSDManager::setBlueColorBarText", &text_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  string IOSDManager::getRedColorBarText()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("IOSDManager::getRedColorBarText");
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string IOSDManager::getGreenColorBarText()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("IOSDManager::getGreenColorBarText");
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string IOSDManager::getYellowColorBarText()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("IOSDManager::getYellowColorBarText");
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string IOSDManager::getBlueColorBarText()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("IOSDManager::getBlueColorBarText");
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::messageLeft()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("IOSDManager::messageLeft");
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IOSDManager::messageRight()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("IOSDManager::messageRight");
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string IOSDManager::messageID()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("IOSDManager::messageID");
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IOSDManager::setPause(const nmm_bool& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > value_value(value, false);
    NMM::Event event("IOSDManager::setPause", &value_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IOSDManager::sendPauseEvent()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IOSDManager::sendPauseEvent");
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  IBitmapReader* IOSDManager::getBitmapReader()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<IBitmapReader* > return_value;
    NMM::Event event("IOSDManager::getBitmapReader");
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IOSDManager::checkCharDimension(const string& id, const nmm_uchar& c, nmm_uint32& x, nmm_uint32& y) const
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > id_value(id, false);
    NMM::TInValue<nmm_uchar > c_value(c, false);
    NMM::TOutValue<nmm_uint32 > x_value(x);
    NMM::TOutValue<nmm_uint32 > y_value(y);
    NMM::Event event("IOSDManager::checkCharDimension", &id_value, &c_value, &x_value, &y_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    x = x_value.getRValue();
    y = y_value.getRValue();
  }
  
  nmm_uint32 IOSDManager::getMaxWidth(const string& id) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::TInValue<string > id_value(id, false);
    NMM::Event event("IOSDManager::getMaxWidth", &id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 IOSDManager::getMaxHeight(const string& id) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::TInValue<string > id_value(id, false);
    NMM::Event event("IOSDManager::getMaxHeight", &id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IOSDManager::setLineSpace(const string& id, const nmm_uint32& space)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > id_value(id, false);
    NMM::TInValue<nmm_uint32 > space_value(space, false);
    NMM::Event event("IOSDManager::setLineSpace", &id_value, &space_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_uint32 IOSDManager::getLineSpace(const string& id) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::TInValue<string > id_value(id, false);
    NMM::Event event("IOSDManager::getLineSpace", &id_value);
    event.setReturnValue(&return_value);
    if(const_cast<IOSDManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IOSDManager::IOSDManager_type = "IOSDManager";
  NMM::TConstructor<IOSDManager > __IOSDManager_prototype("IOSDManager","NMM");


  const char* IOSDManager::loadFont_event = "IOSDManager::loadFont";
  const char* IOSDManager::getFontList_event = "IOSDManager::getFontList";
  const char* IOSDManager::update_event = "IOSDManager::update";
  const char* IOSDManager::getWidget_event = "IOSDManager::getWidget";
  const char* IOSDManager::getWindow_event = "IOSDManager::getWindow";
  const char* IOSDManager::createWindow_event = "IOSDManager::createWindow";
  const char* IOSDManager::deleteWindow_event = "IOSDManager::deleteWindow";
  const char* IOSDManager::deleteWidget_event = "IOSDManager::deleteWidget";
  const char* IOSDManager::displayMessage_event = "IOSDManager::displayMessage";
  const char* IOSDManager::deleteMessage_event = "IOSDManager::deleteMessage";
  const char* IOSDManager::displayTime_event = "IOSDManager::displayTime";
  const char* IOSDManager::createTime_event = "IOSDManager::createTime";
  const char* IOSDManager::resetTime_event = "IOSDManager::resetTime";
  const char* IOSDManager::hideTime_event = "IOSDManager::hideTime";
  const char* IOSDManager::setTime_event = "IOSDManager::setTime";
  const char* IOSDManager::getTime_event = "IOSDManager::getTime";
  const char* IOSDManager::displaySymbol_event = "IOSDManager::displaySymbol";
  const char* IOSDManager::hideSymbol_event = "IOSDManager::hideSymbol";
  const char* IOSDManager::displayProgressBar_event = "IOSDManager::displayProgressBar";
  const char* IOSDManager::hideProgressBar_event = "IOSDManager::hideProgressBar";
  const char* IOSDManager::setMaxProgress_event = "IOSDManager::setMaxProgress";
  const char* IOSDManager::setProgress_event = "IOSDManager::setProgress";
  const char* IOSDManager::updateProgress_event = "IOSDManager::updateProgress";
  const char* IOSDManager::displayColorBar_event = "IOSDManager::displayColorBar";
  const char* IOSDManager::hideColorBar_event = "IOSDManager::hideColorBar";
  const char* IOSDManager::setColorBarText_event = "IOSDManager::setColorBarText";
  const char* IOSDManager::setRedColorBarText_event = "IOSDManager::setRedColorBarText";
  const char* IOSDManager::setGreenColorBarText_event = "IOSDManager::setGreenColorBarText";
  const char* IOSDManager::setYellowColorBarText_event = "IOSDManager::setYellowColorBarText";
  const char* IOSDManager::setBlueColorBarText_event = "IOSDManager::setBlueColorBarText";
  const char* IOSDManager::getRedColorBarText_event = "IOSDManager::getRedColorBarText";
  const char* IOSDManager::getGreenColorBarText_event = "IOSDManager::getGreenColorBarText";
  const char* IOSDManager::getYellowColorBarText_event = "IOSDManager::getYellowColorBarText";
  const char* IOSDManager::getBlueColorBarText_event = "IOSDManager::getBlueColorBarText";
  const char* IOSDManager::messageLeft_event = "IOSDManager::messageLeft";
  const char* IOSDManager::messageRight_event = "IOSDManager::messageRight";
  const char* IOSDManager::messageID_event = "IOSDManager::messageID";
  const char* IOSDManager::setPause_event = "IOSDManager::setPause";
  const char* IOSDManager::sendPauseEvent_event = "IOSDManager::sendPauseEvent";
  const char* IOSDManager::getBitmapReader_event = "IOSDManager::getBitmapReader";
  const char* IOSDManager::checkCharDimension_event = "IOSDManager::checkCharDimension";
  const char* IOSDManager::getMaxWidth_event = "IOSDManager::getMaxWidth";
  const char* IOSDManager::getMaxHeight_event = "IOSDManager::getMaxHeight";
  const char* IOSDManager::setLineSpace_event = "IOSDManager::setLineSpace";
  const char* IOSDManager::getLineSpace_event = "IOSDManager::getLineSpace";


}


#include "nmm/nmm_suffix.hpp"

