/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IOSDManagerImpl.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/messaging/TEDObject.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/serialize/StdValue.hpp"

namespace NMM
{
  
  void IOSDManagerImpl::registerEvents()
  {
  
    getEventDispatcher().registerEvent(IOSDManager::loadFont_event, new NMM::TEDObject3<IOSDManagerImpl, string, string, string, Result(IOSDManagerImpl:: * )(const string&, const string&, const string&), Result >(this, &IOSDManagerImpl::loadFont));
  
    getEventDispatcher().registerEvent(IOSDManager::getFontList_event, new NMM::TEDObject0<IOSDManagerImpl, list<string > (IOSDManagerImpl:: * )() const, list<string >  >(this, &IOSDManagerImpl::getFontList));
  
    getEventDispatcher().registerEvent(IOSDManager::update_event, new NMM::TEDObject0<IOSDManagerImpl, Result(IOSDManagerImpl:: * )(), Result >(this, &IOSDManagerImpl::update));
  
    getEventDispatcher().registerEvent(IOSDManager::getWidget_event, new NMM::TEDObject1<IOSDManagerImpl, nmm_int32, IWidget*(IOSDManagerImpl:: * )(const nmm_int32&), IWidget* >(this, &IOSDManagerImpl::getWidget));
  
    getEventDispatcher().registerEvent(IOSDManager::getWindow_event, new NMM::TEDObject1<IOSDManagerImpl, nmm_int32, IWindow*(IOSDManagerImpl:: * )(const nmm_int32&), IWindow* >(this, &IOSDManagerImpl::getWindow));
  
    getEventDispatcher().registerEvent(IOSDManager::createWindow_event, new NMM::TEDObject0<IOSDManagerImpl, IWindow*(IOSDManagerImpl:: * )(), IWindow* >(this, &IOSDManagerImpl::createWindow));
  
    getEventDispatcher().registerEvent(IOSDManager::deleteWindow_event, new NMM::TEDObject1<IOSDManagerImpl, IWindow, void(IOSDManagerImpl:: * )(const IWindow&), void >(this, &IOSDManagerImpl::deleteWindow));
  
    getEventDispatcher().registerEvent(IOSDManager::deleteWidget_event, new NMM::TEDObject1<IOSDManagerImpl, IWidget, void(IOSDManagerImpl:: * )(const IWidget&), void >(this, &IOSDManagerImpl::deleteWidget));
  
    getEventDispatcher().registerEvent(IOSDManager::displayMessage_event, new NMM::TEDObject6<IOSDManagerImpl, string, string, string, string, string, nmm_uint32, Result(IOSDManagerImpl:: * )(const string&, const string&, const string&, const string&, const string&, const nmm_uint32&), Result >(this, &IOSDManagerImpl::displayMessage));
  
    getEventDispatcher().registerEvent(IOSDManager::deleteMessage_event, new NMM::TEDObject0<IOSDManagerImpl, Result(IOSDManagerImpl:: * )(), Result >(this, &IOSDManagerImpl::deleteMessage));
  
    getEventDispatcher().registerEvent(IOSDManager::displayTime_event, new NMM::TEDObject3<IOSDManagerImpl, string, nmm_uint32, nmm_uint32, Result(IOSDManagerImpl:: * )(const string&, const nmm_uint32&, const nmm_uint32&), Result >(this, &IOSDManagerImpl::displayTime));
  
    getEventDispatcher().registerEvent(IOSDManager::createTime_event, new NMM::TEDObject1<IOSDManagerImpl, nmm_bool, Result(IOSDManagerImpl:: * )(const nmm_bool&), Result >(this, &IOSDManagerImpl::createTime));
  
    getEventDispatcher().registerEvent(IOSDManager::resetTime_event, new NMM::TEDObject0<IOSDManagerImpl, Result(IOSDManagerImpl:: * )(), Result >(this, &IOSDManagerImpl::resetTime));
  
    getEventDispatcher().registerEvent(IOSDManager::hideTime_event, new NMM::TEDObject0<IOSDManagerImpl, Result(IOSDManagerImpl:: * )(), Result >(this, &IOSDManagerImpl::hideTime));
  
    getEventDispatcher().registerEvent(IOSDManager::setTime_event, new NMM::TEDObject1<IOSDManagerImpl, nmm_uint64, Result(IOSDManagerImpl:: * )(const nmm_uint64&), Result >(this, &IOSDManagerImpl::setTime));
  
    getEventDispatcher().registerEvent(IOSDManager::getTime_event, new NMM::TEDObject0<IOSDManagerImpl, nmm_uint64(IOSDManagerImpl:: * )(), nmm_uint64 >(this, &IOSDManagerImpl::getTime));
  
    getEventDispatcher().registerEvent(IOSDManager::displaySymbol_event, new NMM::TEDObject3<IOSDManagerImpl, string, nmm_uint32, nmm_uint32, Result(IOSDManagerImpl:: * )(const string&, const nmm_uint32&, const nmm_uint32&), Result >(this, &IOSDManagerImpl::displaySymbol));
  
    getEventDispatcher().registerEvent(IOSDManager::hideSymbol_event, new NMM::TEDObject0<IOSDManagerImpl, Result(IOSDManagerImpl:: * )(), Result >(this, &IOSDManagerImpl::hideSymbol));
  
    getEventDispatcher().registerEvent(IOSDManager::displayProgressBar_event, new NMM::TEDObject4<IOSDManagerImpl, string, nmm_uint32, nmm_uint32, nmm_uint32, Result(IOSDManagerImpl:: * )(const string&, const nmm_uint32&, const nmm_uint32&, const nmm_uint32&), Result >(this, &IOSDManagerImpl::displayProgressBar));
  
    getEventDispatcher().registerEvent(IOSDManager::hideProgressBar_event, new NMM::TEDObject0<IOSDManagerImpl, Result(IOSDManagerImpl:: * )(), Result >(this, &IOSDManagerImpl::hideProgressBar));
  
    getEventDispatcher().registerEvent(IOSDManager::setMaxProgress_event, new NMM::TEDObject1<IOSDManagerImpl, nmm_uint64, void(IOSDManagerImpl:: * )(const nmm_uint64&), void >(this, &IOSDManagerImpl::setMaxProgress));
  
    getEventDispatcher().registerEvent(IOSDManager::setProgress_event, new NMM::TEDObject1<IOSDManagerImpl, nmm_uint64, void(IOSDManagerImpl:: * )(const nmm_uint64&), void >(this, &IOSDManagerImpl::setProgress));
  
    getEventDispatcher().registerEvent(IOSDManager::updateProgress_event, new NMM::TEDObject0<IOSDManagerImpl, void(IOSDManagerImpl:: * )(), void >(this, &IOSDManagerImpl::updateProgress));
  
    getEventDispatcher().registerEvent(IOSDManager::displayColorBar_event, new NMM::TEDObject2<IOSDManagerImpl, nmm_uint32, nmm_uint32, Result(IOSDManagerImpl:: * )(const nmm_uint32&, const nmm_uint32&), Result >(this, &IOSDManagerImpl::displayColorBar));
  
    getEventDispatcher().registerEvent(IOSDManager::hideColorBar_event, new NMM::TEDObject0<IOSDManagerImpl, Result(IOSDManagerImpl:: * )(), Result >(this, &IOSDManagerImpl::hideColorBar));
  
    getEventDispatcher().registerEvent(IOSDManager::setColorBarText_event, new NMM::TEDObject4<IOSDManagerImpl, string, string, string, string, void(IOSDManagerImpl:: * )(const string&, const string&, const string&, const string&), void >(this, &IOSDManagerImpl::setColorBarText));
  
    getEventDispatcher().registerEvent(IOSDManager::setRedColorBarText_event, new NMM::TEDObject1<IOSDManagerImpl, string, void(IOSDManagerImpl:: * )(const string&), void >(this, &IOSDManagerImpl::setRedColorBarText));
  
    getEventDispatcher().registerEvent(IOSDManager::setGreenColorBarText_event, new NMM::TEDObject1<IOSDManagerImpl, string, void(IOSDManagerImpl:: * )(const string&), void >(this, &IOSDManagerImpl::setGreenColorBarText));
  
    getEventDispatcher().registerEvent(IOSDManager::setYellowColorBarText_event, new NMM::TEDObject1<IOSDManagerImpl, string, void(IOSDManagerImpl:: * )(const string&), void >(this, &IOSDManagerImpl::setYellowColorBarText));
  
    getEventDispatcher().registerEvent(IOSDManager::setBlueColorBarText_event, new NMM::TEDObject1<IOSDManagerImpl, string, void(IOSDManagerImpl:: * )(const string&), void >(this, &IOSDManagerImpl::setBlueColorBarText));
  
    getEventDispatcher().registerEvent(IOSDManager::getRedColorBarText_event, new NMM::TEDObject0<IOSDManagerImpl, string(IOSDManagerImpl:: * )(), string >(this, &IOSDManagerImpl::getRedColorBarText));
  
    getEventDispatcher().registerEvent(IOSDManager::getGreenColorBarText_event, new NMM::TEDObject0<IOSDManagerImpl, string(IOSDManagerImpl:: * )(), string >(this, &IOSDManagerImpl::getGreenColorBarText));
  
    getEventDispatcher().registerEvent(IOSDManager::getYellowColorBarText_event, new NMM::TEDObject0<IOSDManagerImpl, string(IOSDManagerImpl:: * )(), string >(this, &IOSDManagerImpl::getYellowColorBarText));
  
    getEventDispatcher().registerEvent(IOSDManager::getBlueColorBarText_event, new NMM::TEDObject0<IOSDManagerImpl, string(IOSDManagerImpl:: * )(), string >(this, &IOSDManagerImpl::getBlueColorBarText));
  
    getEventDispatcher().registerEvent(IOSDManager::messageLeft_event, new NMM::TEDObject0<IOSDManagerImpl, Result(IOSDManagerImpl:: * )(), Result >(this, &IOSDManagerImpl::messageLeft));
  
    getEventDispatcher().registerEvent(IOSDManager::messageRight_event, new NMM::TEDObject0<IOSDManagerImpl, Result(IOSDManagerImpl:: * )(), Result >(this, &IOSDManagerImpl::messageRight));
  
    getEventDispatcher().registerEvent(IOSDManager::messageID_event, new NMM::TEDObject0<IOSDManagerImpl, string(IOSDManagerImpl:: * )(), string >(this, &IOSDManagerImpl::messageID));
  
    getEventDispatcher().registerEvent(IOSDManager::setPause_event, new NMM::TEDObject1<IOSDManagerImpl, nmm_bool, void(IOSDManagerImpl:: * )(const nmm_bool&), void >(this, &IOSDManagerImpl::setPause));
  
    getEventDispatcher().registerEvent(IOSDManager::sendPauseEvent_event, new NMM::TEDObject0<IOSDManagerImpl, void(IOSDManagerImpl:: * )(), void >(this, &IOSDManagerImpl::sendPauseEvent));
  
    getEventDispatcher().registerEvent(IOSDManager::getBitmapReader_event, new NMM::TEDObject0<IOSDManagerImpl, IBitmapReader*(IOSDManagerImpl:: * )(), IBitmapReader* >(this, &IOSDManagerImpl::getBitmapReader));
  
    getEventDispatcher().registerEvent(IOSDManager::checkCharDimension_event, new NMM::TEDObject4<IOSDManagerImpl, string, nmm_uchar, nmm_uint32, nmm_uint32, void(IOSDManagerImpl:: * )(const string&, const nmm_uchar&, nmm_uint32&, nmm_uint32&) const, void >(this, &IOSDManagerImpl::checkCharDimension));
  
    getEventDispatcher().registerEvent(IOSDManager::getMaxWidth_event, new NMM::TEDObject1<IOSDManagerImpl, string, nmm_uint32(IOSDManagerImpl:: * )(const string&) const, nmm_uint32 >(this, &IOSDManagerImpl::getMaxWidth));
  
    getEventDispatcher().registerEvent(IOSDManager::getMaxHeight_event, new NMM::TEDObject1<IOSDManagerImpl, string, nmm_uint32(IOSDManagerImpl:: * )(const string&) const, nmm_uint32 >(this, &IOSDManagerImpl::getMaxHeight));
  
    getEventDispatcher().registerEvent(IOSDManager::setLineSpace_event, new NMM::TEDObject2<IOSDManagerImpl, string, nmm_uint32, void(IOSDManagerImpl:: * )(const string&, const nmm_uint32&), void >(this, &IOSDManagerImpl::setLineSpace));
  
    getEventDispatcher().registerEvent(IOSDManager::getLineSpace_event, new NMM::TEDObject1<IOSDManagerImpl, string, nmm_uint32(IOSDManagerImpl:: * )(const string&) const, nmm_uint32 >(this, &IOSDManagerImpl::getLineSpace));

  }

}


#include "nmm/nmm_suffix.hpp"

