/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IPlaneDecorator_HPP
#define NMM_IPlaneDecorator_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "IDecorator.hpp"

#include "visibility.hpp"

namespace NMM
{
  /**
   * Plane decorator interface.
   */

  class DLLEXPORT IPlaneDecorator : virtual public IDecorator { 
  public:
    IPlaneDecorator() : NMM::Interface(IPlaneDecorator::IPlaneDecorator_type, "NMM") { };

    IPlaneDecorator(NMM::NMMObject& parent) : NMM::Interface (parent, IPlaneDecorator::IPlaneDecorator_type, "NMM") { };

    virtual ~IPlaneDecorator() {};

    static const char* IPlaneDecorator_type;

    virtual  string getInterfaceType() const { return IPlaneDecorator_type; }

    NMM::Interface* copy() const { return new IPlaneDecorator(*this); }
    static void used(); 
    static const char* setPlaneColor_event;
    /**
     * Set plane color.
     * @param color 32 bit border color (rgba format)
     */

    virtual void setPlaneColor(const nmm_uint32& color)
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IPlaneDecorator> IPlaneDecorator_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
