/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IProgressBar.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IProgressBar::used() {} 
  
  
  void IProgressBar::setMaxProgress(const nmm_uint64& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint64 > value_value(value, false);
    NMM::Event event("IProgressBar::setMaxProgress", &value_value);
    if(const_cast<IProgressBar*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IProgressBar::setProgress(const nmm_uint64& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint64 > value_value(value, false);
    NMM::Event event("IProgressBar::setProgress", &value_value);
    if(const_cast<IProgressBar*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IProgressBar::setBarColor(const nmm_uint32& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > value_value(value, false);
    NMM::Event event("IProgressBar::setBarColor", &value_value);
    if(const_cast<IProgressBar*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IProgressBar::setBarSpaces(const nmm_uint32& space, const nmm_uint32& space_step)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > space_value(space, false);
    NMM::TInValue<nmm_uint32 > space_step_value(space_step, false);
    NMM::Event event("IProgressBar::setBarSpaces", &space_value, &space_step_value);
    if(const_cast<IProgressBar*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IProgressBar::IProgressBar_type = "IProgressBar";
  NMM::TConstructor<IProgressBar > __IProgressBar_prototype("IProgressBar","NMM");


  const char* IProgressBar::setMaxProgress_event = "IProgressBar::setMaxProgress";
  const char* IProgressBar::setProgress_event = "IProgressBar::setProgress";
  const char* IProgressBar::setBarColor_event = "IProgressBar::setBarColor";
  const char* IProgressBar::setBarSpaces_event = "IProgressBar::setBarSpaces";


}


#include "nmm/nmm_suffix.hpp"

