/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IProgressBar_HPP
#define NMM_IProgressBar_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "IWidget.hpp"

#include "visibility.hpp"

namespace NMM
{
  /**
   * Progress bar widget interface.
   */

  class DLLEXPORT IProgressBar : virtual public IWidget { 
  public:
    IProgressBar() : NMM::Interface(IProgressBar::IProgressBar_type, "NMM") { };

    IProgressBar(NMM::NMMObject& parent) : NMM::Interface (parent, IProgressBar::IProgressBar_type, "NMM") { };

    virtual ~IProgressBar() {};

    static const char* IProgressBar_type;

    virtual  string getInterfaceType() const { return IProgressBar_type; }

    NMM::Interface* copy() const { return new IProgressBar(*this); }
    static void used(); 
    static const char* setMaxProgress_event;
    static const char* setProgress_event;
    static const char* setBarColor_event;
    static const char* setBarSpaces_event;
    /** Set maximum value of progress bar. */

    virtual void setMaxProgress(const nmm_uint64& value)
      throw(const NMM::SerializedException&);
    /** Set current progress value. */

    virtual void setProgress(const nmm_uint64& value)
      throw(const NMM::SerializedException&);
    /**
     * Set bar color.
     * @param value bar color (rgba format)
     */

    virtual void setBarColor(const nmm_uint32& value)
      throw(const NMM::SerializedException&);
    /**
     * Definition of bar spaces.
     * @param space space between progress bars
     * @param space_step gap between spaces
     */

    virtual void setBarSpaces(const nmm_uint32& space, const nmm_uint32& space_step)
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IProgressBar> IProgressBar_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
