/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IProgressBarImpl_HPP
#define NMM_IProgressBarImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IProgressBar.hpp"

#include "IWidgetImpl.hpp"

#include "visibility.hpp"

namespace NMM
{
  /**
   * Progress bar widget interface.
   */

  class DLLEXPORT IProgressBarImpl  : virtual public IWidgetImpl { 
  public:
    IProgressBarImpl() { addInterface(new IProgressBar(*this)); registerEvents(); };

    virtual ~IProgressBarImpl() {};

  protected:
    /** Set maximum value of progress bar. */

    virtual void setMaxProgress(const nmm_uint64& value) = 0;
    /** Set current progress value. */

    virtual void setProgress(const nmm_uint64& value) = 0;
    /**
     * Set bar color.
     * @param value bar color (rgba format)
     */

    virtual void setBarColor(const nmm_uint32& value) = 0;
    /**
     * Definition of bar spaces.
     * @param space space between progress bars
     * @param space_step gap between spaces
     */

    virtual void setBarSpaces(const nmm_uint32& space, const nmm_uint32& space_step) = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
