/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IScrollDecorator.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IScrollDecorator::used() {} 
  
  
  void IScrollDecorator::setBarWidth(const nmm_uint32& bw)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > bw_value(bw, false);
    NMM::Event event("IScrollDecorator::setBarWidth", &bw_value);
    if(const_cast<IScrollDecorator*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IScrollDecorator::setBackgroundColor(const nmm_uint32& bcolor)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > bcolor_value(bcolor, false);
    NMM::Event event("IScrollDecorator::setBackgroundColor", &bcolor_value);
    if(const_cast<IScrollDecorator*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IScrollDecorator::setBorderColor(const nmm_uint32& bcolor)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > bcolor_value(bcolor, false);
    NMM::Event event("IScrollDecorator::setBorderColor", &bcolor_value);
    if(const_cast<IScrollDecorator*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IScrollDecorator::setBarColor(const nmm_uint32& bcolor)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > bcolor_value(bcolor, false);
    NMM::Event event("IScrollDecorator::setBarColor", &bcolor_value);
    if(const_cast<IScrollDecorator*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IScrollDecorator::setBarBorderColors(const nmm_uint32& bcolor1, const nmm_uint32& bcolor2)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > bcolor1_value(bcolor1, false);
    NMM::TInValue<nmm_uint32 > bcolor2_value(bcolor2, false);
    NMM::Event event("IScrollDecorator::setBarBorderColors", &bcolor1_value, &bcolor2_value);
    if(const_cast<IScrollDecorator*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IScrollDecorator::IScrollDecorator_type = "IScrollDecorator";
  NMM::TConstructor<IScrollDecorator > __IScrollDecorator_prototype("IScrollDecorator","NMM");


  const char* IScrollDecorator::setBarWidth_event = "IScrollDecorator::setBarWidth";
  const char* IScrollDecorator::setBackgroundColor_event = "IScrollDecorator::setBackgroundColor";
  const char* IScrollDecorator::setBorderColor_event = "IScrollDecorator::setBorderColor";
  const char* IScrollDecorator::setBarColor_event = "IScrollDecorator::setBarColor";
  const char* IScrollDecorator::setBarBorderColors_event = "IScrollDecorator::setBarBorderColors";


}


#include "nmm/nmm_suffix.hpp"

