/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IScrollDecorator_HPP
#define NMM_IScrollDecorator_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "IDecorator.hpp"

#include "visibility.hpp"

namespace NMM
{
  /**
   * Scroll decorator interface.
   */

  class DLLEXPORT IScrollDecorator : virtual public IDecorator { 
  public:
    IScrollDecorator() : NMM::Interface(IScrollDecorator::IScrollDecorator_type, "NMM") { };

    IScrollDecorator(NMM::NMMObject& parent) : NMM::Interface (parent, IScrollDecorator::IScrollDecorator_type, "NMM") { };

    virtual ~IScrollDecorator() {};

    static const char* IScrollDecorator_type;

    virtual  string getInterfaceType() const { return IScrollDecorator_type; }

    NMM::Interface* copy() const { return new IScrollDecorator(*this); }
    static void used(); 
    static const char* setBarWidth_event;
    static const char* setBackgroundColor_event;
    static const char* setBorderColor_event;
    static const char* setBarColor_event;
    static const char* setBarBorderColors_event;

    virtual void setBarWidth(const nmm_uint32& bw)
      throw(const NMM::SerializedException&);

    virtual void setBackgroundColor(const nmm_uint32& bcolor)
      throw(const NMM::SerializedException&);

    virtual void setBorderColor(const nmm_uint32& bcolor)
      throw(const NMM::SerializedException&);

    virtual void setBarColor(const nmm_uint32& bcolor)
      throw(const NMM::SerializedException&);

    virtual void setBarBorderColors(const nmm_uint32& bcolor1, const nmm_uint32& bcolor2)
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IScrollDecorator> IScrollDecorator_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
