/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IScrollable_HPP
#define NMM_IScrollable_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{
  /**
   * Scrollable item.
   * If an item (need not be a widget) has support for scrollbars, it should
   * implement this interface.
   */

  class DLLEXPORT IScrollable : virtual public NMM::Interface { 
  public:
    IScrollable() : NMM::Interface(IScrollable::IScrollable_type, "NMM") { };

    IScrollable(NMM::NMMObject& parent) : NMM::Interface (parent, IScrollable::IScrollable_type, "NMM") { };

    virtual ~IScrollable() {};

    static const char* IScrollable_type;

    virtual  string getInterfaceType() const { return IScrollable_type; }

    NMM::Interface* copy() const { return new IScrollable(*this); }
    static void used(); 
    static const char* getScrollBarHeight_event;
    static const char* getScrollBarWidth_event;
    static const char* getScrollBarYPosition_event;
    static const char* getScrollBarXPosition_event;
    /**
     * Get height of vertical scrollbar.
     * Returns the actual scrollbar height (per 10000).
     * That means a value of 10000 indicates a full scrollbar height.
     */

    virtual nmm_int32 getScrollBarHeight() const
      throw(const NMM::SerializedException&);
    /**
     * Get width of horizontal scrollbar.
     * Returns the actual scrollbar width (per 10000).
     * That means a value of 10000 indicates a full scrollbar width.
     */

    virtual nmm_int32 getScrollBarWidth() const
      throw(const NMM::SerializedException&);
    /**
     * Get vertical scrollbar position.
     * Returns the actual scrollbar position (per 10000).
     */

    virtual nmm_int32 getScrollBarYPosition() const
      throw(const NMM::SerializedException&);
    /**
     * Get horizontal scrollbar position.
     * Returns the actual scrollbar position (per 10000).
     */

    virtual nmm_int32 getScrollBarXPosition() const
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IScrollable> IScrollable_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
