/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IScrollableImpl_HPP
#define NMM_IScrollableImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IScrollable.hpp"


#include "visibility.hpp"

namespace NMM
{
  /**
   * Scrollable item.
   * If an item (need not be a widget) has support for scrollbars, it should
   * implement this interface.
   */

  class DLLEXPORT IScrollableImpl  : public virtual NMM::NMMObject { 
  public:
    IScrollableImpl() { addInterface(new IScrollable(*this)); registerEvents(); };

    virtual ~IScrollableImpl() {};

  protected:
    /**
     * Get height of vertical scrollbar.
     * Returns the actual scrollbar height (per 10000).
     * That means a value of 10000 indicates a full scrollbar height.
     */

    virtual nmm_int32 getScrollBarHeight() const = 0;
    /**
     * Get width of horizontal scrollbar.
     * Returns the actual scrollbar width (per 10000).
     * That means a value of 10000 indicates a full scrollbar width.
     */

    virtual nmm_int32 getScrollBarWidth() const = 0;
    /**
     * Get vertical scrollbar position.
     * Returns the actual scrollbar position (per 10000).
     */

    virtual nmm_int32 getScrollBarYPosition() const = 0;
    /**
     * Get horizontal scrollbar position.
     * Returns the actual scrollbar position (per 10000).
     */

    virtual nmm_int32 getScrollBarXPosition() const = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
