/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ISymbolChange.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ISymbolChange::used() {} 
  
  
  Result ISymbolChange::symbolChanged(const string& symbol) /* in-stream */ 
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > symbol_value(symbol, false);
    NMM::Event event("ISymbolChange::symbolChanged", &symbol_value);
    event.setReturnValue(&return_value);
    if(const_cast<ISymbolChange*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ISymbolChange::ISymbolChange_type = "ISymbolChange";
  NMM::TConstructor<ISymbolChange > __ISymbolChange_prototype("ISymbolChange","NMM");


  const char* ISymbolChange::symbolChanged_event = "ISymbolChange::symbolChanged";
  NMM::Event* ISymbolChange::create_symbolChanged(const string& symbol)
  {
    NMM::TInValue<string >* symbol_value = new NMM::TInValue<string >();
    symbol_value->setValue(symbol);

    return new NMM::Event("ISymbolChange::symbolChanged", symbol_value);
  }



}


#include "nmm/nmm_suffix.hpp"

