/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ISymbolChange_HPP
#define NMM_ISymbolChange_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>


#include "visibility.hpp"

namespace NMM
{
  /**
   * Symbol has changed.
   * Instream event, sent by the OSDManager when the symbol has been changed.
   * The event is sent after the first video frame that contains the new symbol.
   */

  class DLLEXPORT ISymbolChange : virtual public NMM::Interface { 
  public:
    ISymbolChange() : NMM::Interface(ISymbolChange::ISymbolChange_type, "NMM") { };

    ISymbolChange(NMM::NMMObject& parent) : NMM::Interface (parent, ISymbolChange::ISymbolChange_type, "NMM") { };

    virtual ~ISymbolChange() {};

    static const char* ISymbolChange_type;

    virtual  string getInterfaceType() const { return ISymbolChange_type; }

    NMM::Interface* copy() const { return new ISymbolChange(*this); }
    static void used(); 
    static const char* symbolChanged_event;
    static NMM::Event* create_symbolChanged(const string& symbol);
  /* instream (protected) and non-instream (public) */    
    virtual Result symbolChanged(const string& symbol) /* in-stream */ 
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ISymbolChange> ISymbolChange_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
