/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ITextView.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/serialize/StdValue.hpp"

namespace NMM
{
  void ITextView::used() {} 
  
  
  void ITextView::setFont(const string& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > value_value(value, false);
    NMM::Event event("ITextView::setFont", &value_value);
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_uint32 ITextView::selectLine(const nmm_uint32& line)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::TInValue<nmm_uint32 > line_value(line, false);
    NMM::Event event("ITextView::selectLine", &line_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 ITextView::selectLine2(const nmm_uint32& line)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::TInValue<nmm_uint32 > line_value(line, false);
    NMM::Event event("ITextView::selectLine2", &line_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITextView::deSelectLine()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("ITextView::deSelectLine");
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITextView::deSelectLine2()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("ITextView::deSelectLine2");
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_uint32 ITextView::getSelectedLine() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("ITextView::getSelectedLine");
    event.setReturnValue(&return_value);
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 ITextView::getSelectedLine2() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("ITextView::getSelectedLine2");
    event.setReturnValue(&return_value);
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string ITextView::getLineContent(const nmm_uint32& line) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::TInValue<nmm_uint32 > line_value(line, false);
    NMM::Event event("ITextView::getLineContent", &line_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITextView::changeLineContent(const nmm_uint32& line, const string& text)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > line_value(line, false);
    NMM::TInValue<string > text_value(text, false);
    NMM::Event event("ITextView::changeLineContent", &line_value, &text_value);
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  string ITextView::getLineURL(const nmm_uint32& line) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::TInValue<nmm_uint32 > line_value(line, false);
    NMM::Event event("ITextView::getLineURL", &line_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IWidget* ITextView::getLineWidget(const nmm_uint32& line) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<IWidget* > return_value;
    NMM::TInValue<nmm_uint32 > line_value(line, false);
    NMM::Event event("ITextView::getLineWidget", &line_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITextView::setAutoFocus(const nmm_bool& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > value_value(value, false);
    NMM::Event event("ITextView::setAutoFocus", &value_value);
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITextView::setSelectionColor(const nmm_uint32& color)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > color_value(color, false);
    NMM::Event event("ITextView::setSelectionColor", &color_value);
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITextView::setSelectionColor2(const nmm_uint32& color)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > color_value(color, false);
    NMM::Event event("ITextView::setSelectionColor2", &color_value);
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITextView::reset()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("ITextView::reset");
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITextView::setMaxLines(const nmm_uint32& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > value_value(value, false);
    NMM::Event event("ITextView::setMaxLines", &value_value);
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_uint32 ITextView::getMaxLines() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("ITextView::getMaxLines");
    event.setReturnValue(&return_value);
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 ITextView::setStartLine(const nmm_uint32& start)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::TInValue<nmm_uint32 > start_value(start, false);
    NMM::Event event("ITextView::setStartLine", &start_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 ITextView::getStartLine() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("ITextView::getStartLine");
    event.setReturnValue(&return_value);
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 ITextView::getNumLines() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("ITextView::getNumLines");
    event.setReturnValue(&return_value);
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITextView::focusSelection()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("ITextView::focusSelection");
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITextView::insertLine(const nmm_int32& pos, const string& text, const string& url /* = "" */)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > pos_value(pos, false);
    NMM::TInValue<string > text_value(text, false);
    NMM::TInValue<string > url_value(url, false);
    NMM::Event event("ITextView::insertLine", &pos_value, &text_value, &url_value);
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_int32 ITextView::addLine(const string& text, const string& url /* = "" */)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::TInValue<string > text_value(text, false);
    NMM::TInValue<string > url_value(url, false);
    NMM::Event event("ITextView::addLine", &text_value, &url_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITextView::addLineWidget(const nmm_uint32& line, const IWidget& widget)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > line_value(line, false);
    NMM::TInValue<IWidget > widget_value(widget, false);
    NMM::Event event("ITextView::addLineWidget", &line_value, &widget_value);
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITextView::deleteLine(const nmm_uint32& line)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > line_value(line, false);
    NMM::Event event("ITextView::deleteLine", &line_value);
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITextView::deleteAllLines()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("ITextView::deleteAllLines");
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITextView::markLine(const nmm_uint32& line)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > line_value(line, false);
    NMM::Event event("ITextView::markLine", &line_value);
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITextView::unmarkLine(const nmm_uint32& line)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > line_value(line, false);
    NMM::Event event("ITextView::unmarkLine", &line_value);
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  list<nmm_uint32 >  ITextView::getMarkedLines()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<list<nmm_uint32 >  > return_value;
    NMM::Event event("ITextView::getMarkedLines");
    event.setReturnValue(&return_value);
    if(const_cast<ITextView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ITextView::ITextView_type = "ITextView";
  NMM::TConstructor<ITextView > __ITextView_prototype("ITextView","NMM");


  const char* ITextView::setFont_event = "ITextView::setFont";
  const char* ITextView::selectLine_event = "ITextView::selectLine";
  const char* ITextView::selectLine2_event = "ITextView::selectLine2";
  const char* ITextView::deSelectLine_event = "ITextView::deSelectLine";
  const char* ITextView::deSelectLine2_event = "ITextView::deSelectLine2";
  const char* ITextView::getSelectedLine_event = "ITextView::getSelectedLine";
  const char* ITextView::getSelectedLine2_event = "ITextView::getSelectedLine2";
  const char* ITextView::getLineContent_event = "ITextView::getLineContent";
  const char* ITextView::changeLineContent_event = "ITextView::changeLineContent";
  const char* ITextView::getLineURL_event = "ITextView::getLineURL";
  const char* ITextView::getLineWidget_event = "ITextView::getLineWidget";
  const char* ITextView::setAutoFocus_event = "ITextView::setAutoFocus";
  const char* ITextView::setSelectionColor_event = "ITextView::setSelectionColor";
  const char* ITextView::setSelectionColor2_event = "ITextView::setSelectionColor2";
  const char* ITextView::reset_event = "ITextView::reset";
  const char* ITextView::setMaxLines_event = "ITextView::setMaxLines";
  const char* ITextView::getMaxLines_event = "ITextView::getMaxLines";
  const char* ITextView::setStartLine_event = "ITextView::setStartLine";
  const char* ITextView::getStartLine_event = "ITextView::getStartLine";
  const char* ITextView::getNumLines_event = "ITextView::getNumLines";
  const char* ITextView::focusSelection_event = "ITextView::focusSelection";
  const char* ITextView::insertLine_event = "ITextView::insertLine";
  const char* ITextView::addLine_event = "ITextView::addLine";
  const char* ITextView::addLineWidget_event = "ITextView::addLineWidget";
  const char* ITextView::deleteLine_event = "ITextView::deleteLine";
  const char* ITextView::deleteAllLines_event = "ITextView::deleteAllLines";
  const char* ITextView::markLine_event = "ITextView::markLine";
  const char* ITextView::unmarkLine_event = "ITextView::unmarkLine";
  const char* ITextView::getMarkedLines_event = "ITextView::getMarkedLines";


}


#include "nmm/nmm_suffix.hpp"

