/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ITextViewImpl.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/messaging/TEDObject.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/serialize/StdValue.hpp"

namespace NMM
{
  
  void ITextViewImpl::registerEvents()
  {
  
    getEventDispatcher().registerEvent(ITextView::setFont_event, new NMM::TEDObject1<ITextViewImpl, string, void(ITextViewImpl:: * )(const string&), void >(this, &ITextViewImpl::setFont));
  
    getEventDispatcher().registerEvent(ITextView::selectLine_event, new NMM::TEDObject1<ITextViewImpl, nmm_uint32, nmm_uint32(ITextViewImpl:: * )(const nmm_uint32&), nmm_uint32 >(this, &ITextViewImpl::selectLine));
  
    getEventDispatcher().registerEvent(ITextView::selectLine2_event, new NMM::TEDObject1<ITextViewImpl, nmm_uint32, nmm_uint32(ITextViewImpl:: * )(const nmm_uint32&), nmm_uint32 >(this, &ITextViewImpl::selectLine2));
  
    getEventDispatcher().registerEvent(ITextView::deSelectLine_event, new NMM::TEDObject0<ITextViewImpl, void(ITextViewImpl:: * )(), void >(this, &ITextViewImpl::deSelectLine));
  
    getEventDispatcher().registerEvent(ITextView::deSelectLine2_event, new NMM::TEDObject0<ITextViewImpl, void(ITextViewImpl:: * )(), void >(this, &ITextViewImpl::deSelectLine2));
  
    getEventDispatcher().registerEvent(ITextView::getSelectedLine_event, new NMM::TEDObject0<ITextViewImpl, nmm_uint32(ITextViewImpl:: * )() const, nmm_uint32 >(this, &ITextViewImpl::getSelectedLine));
  
    getEventDispatcher().registerEvent(ITextView::getSelectedLine2_event, new NMM::TEDObject0<ITextViewImpl, nmm_uint32(ITextViewImpl:: * )() const, nmm_uint32 >(this, &ITextViewImpl::getSelectedLine2));
  
    getEventDispatcher().registerEvent(ITextView::getLineContent_event, new NMM::TEDObject1<ITextViewImpl, nmm_uint32, string(ITextViewImpl:: * )(const nmm_uint32&) const, string >(this, &ITextViewImpl::getLineContent));
  
    getEventDispatcher().registerEvent(ITextView::changeLineContent_event, new NMM::TEDObject2<ITextViewImpl, nmm_uint32, string, void(ITextViewImpl:: * )(const nmm_uint32&, const string&), void >(this, &ITextViewImpl::changeLineContent));
  
    getEventDispatcher().registerEvent(ITextView::getLineURL_event, new NMM::TEDObject1<ITextViewImpl, nmm_uint32, string(ITextViewImpl:: * )(const nmm_uint32&) const, string >(this, &ITextViewImpl::getLineURL));
  
    getEventDispatcher().registerEvent(ITextView::getLineWidget_event, new NMM::TEDObject1<ITextViewImpl, nmm_uint32, IWidget*(ITextViewImpl:: * )(const nmm_uint32&) const, IWidget* >(this, &ITextViewImpl::getLineWidget));
  
    getEventDispatcher().registerEvent(ITextView::setAutoFocus_event, new NMM::TEDObject1<ITextViewImpl, nmm_bool, void(ITextViewImpl:: * )(const nmm_bool&), void >(this, &ITextViewImpl::setAutoFocus));
  
    getEventDispatcher().registerEvent(ITextView::setSelectionColor_event, new NMM::TEDObject1<ITextViewImpl, nmm_uint32, void(ITextViewImpl:: * )(const nmm_uint32&), void >(this, &ITextViewImpl::setSelectionColor));
  
    getEventDispatcher().registerEvent(ITextView::setSelectionColor2_event, new NMM::TEDObject1<ITextViewImpl, nmm_uint32, void(ITextViewImpl:: * )(const nmm_uint32&), void >(this, &ITextViewImpl::setSelectionColor2));
  
    getEventDispatcher().registerEvent(ITextView::reset_event, new NMM::TEDObject0<ITextViewImpl, void(ITextViewImpl:: * )(), void >(this, &ITextViewImpl::reset));
  
    getEventDispatcher().registerEvent(ITextView::setMaxLines_event, new NMM::TEDObject1<ITextViewImpl, nmm_uint32, void(ITextViewImpl:: * )(const nmm_uint32&), void >(this, &ITextViewImpl::setMaxLines));
  
    getEventDispatcher().registerEvent(ITextView::getMaxLines_event, new NMM::TEDObject0<ITextViewImpl, nmm_uint32(ITextViewImpl:: * )() const, nmm_uint32 >(this, &ITextViewImpl::getMaxLines));
  
    getEventDispatcher().registerEvent(ITextView::setStartLine_event, new NMM::TEDObject1<ITextViewImpl, nmm_uint32, nmm_uint32(ITextViewImpl:: * )(const nmm_uint32&), nmm_uint32 >(this, &ITextViewImpl::setStartLine));
  
    getEventDispatcher().registerEvent(ITextView::getStartLine_event, new NMM::TEDObject0<ITextViewImpl, nmm_uint32(ITextViewImpl:: * )() const, nmm_uint32 >(this, &ITextViewImpl::getStartLine));
  
    getEventDispatcher().registerEvent(ITextView::getNumLines_event, new NMM::TEDObject0<ITextViewImpl, nmm_uint32(ITextViewImpl:: * )() const, nmm_uint32 >(this, &ITextViewImpl::getNumLines));
  
    getEventDispatcher().registerEvent(ITextView::focusSelection_event, new NMM::TEDObject0<ITextViewImpl, void(ITextViewImpl:: * )(), void >(this, &ITextViewImpl::focusSelection));
  
    getEventDispatcher().registerEvent(ITextView::insertLine_event, new NMM::TEDObject3<ITextViewImpl, nmm_int32, string, string, void(ITextViewImpl:: * )(const nmm_int32&, const string&, const string&), void >(this, &ITextViewImpl::insertLine));
  
    getEventDispatcher().registerEvent(ITextView::addLine_event, new NMM::TEDObject2<ITextViewImpl, string, string, nmm_int32(ITextViewImpl:: * )(const string&, const string&), nmm_int32 >(this, &ITextViewImpl::addLine));
  
    getEventDispatcher().registerEvent(ITextView::addLineWidget_event, new NMM::TEDObject2<ITextViewImpl, nmm_uint32, IWidget, void(ITextViewImpl:: * )(const nmm_uint32&, const IWidget&), void >(this, &ITextViewImpl::addLineWidget));
  
    getEventDispatcher().registerEvent(ITextView::deleteLine_event, new NMM::TEDObject1<ITextViewImpl, nmm_uint32, void(ITextViewImpl:: * )(const nmm_uint32&), void >(this, &ITextViewImpl::deleteLine));
  
    getEventDispatcher().registerEvent(ITextView::deleteAllLines_event, new NMM::TEDObject0<ITextViewImpl, void(ITextViewImpl:: * )(), void >(this, &ITextViewImpl::deleteAllLines));
  
    getEventDispatcher().registerEvent(ITextView::markLine_event, new NMM::TEDObject1<ITextViewImpl, nmm_uint32, void(ITextViewImpl:: * )(const nmm_uint32&), void >(this, &ITextViewImpl::markLine));
  
    getEventDispatcher().registerEvent(ITextView::unmarkLine_event, new NMM::TEDObject1<ITextViewImpl, nmm_uint32, void(ITextViewImpl:: * )(const nmm_uint32&), void >(this, &ITextViewImpl::unmarkLine));
  
    getEventDispatcher().registerEvent(ITextView::getMarkedLines_event, new NMM::TEDObject0<ITextViewImpl, list<nmm_uint32 > (ITextViewImpl:: * )(), list<nmm_uint32 >  >(this, &ITextViewImpl::getMarkedLines));

  }

}


#include "nmm/nmm_suffix.hpp"

