/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ITimeView.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ITimeView::used() {} 
  
  
  void ITimeView::setFont(const string& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > value_value(value, false);
    NMM::Event event("ITimeView::setFont", &value_value);
    if(const_cast<ITimeView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITimeView::setPrefix(const string& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > value_value(value, false);
    NMM::Event event("ITimeView::setPrefix", &value_value);
    if(const_cast<ITimeView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITimeView::setTimeOffset(const nmm_int32& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > value_value(value, false);
    NMM::Event event("ITimeView::setTimeOffset", &value_value);
    if(const_cast<ITimeView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITimeView::setCurrentTime(const nmm_int32& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > value_value(value, false);
    NMM::Event event("ITimeView::setCurrentTime", &value_value);
    if(const_cast<ITimeView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_int32 ITimeView::getCurrentTime()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("ITimeView::getCurrentTime");
    event.setReturnValue(&return_value);
    if(const_cast<ITimeView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITimeView::reset()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("ITimeView::reset");
    if(const_cast<ITimeView*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* ITimeView::ITimeView_type = "ITimeView";
  NMM::TConstructor<ITimeView > __ITimeView_prototype("ITimeView","NMM");


  const char* ITimeView::setFont_event = "ITimeView::setFont";
  const char* ITimeView::setPrefix_event = "ITimeView::setPrefix";
  const char* ITimeView::setTimeOffset_event = "ITimeView::setTimeOffset";
  const char* ITimeView::setCurrentTime_event = "ITimeView::setCurrentTime";
  const char* ITimeView::getCurrentTime_event = "ITimeView::getCurrentTime";
  const char* ITimeView::reset_event = "ITimeView::reset";


}


#include "nmm/nmm_suffix.hpp"

