/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ITimeView_HPP
#define NMM_ITimeView_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "IWidget.hpp"

#include "visibility.hpp"

namespace NMM
{
  /**
   * Time view widget interface.
   */

  class DLLEXPORT ITimeView : virtual public IWidget { 
  public:
    ITimeView() : NMM::Interface(ITimeView::ITimeView_type, "NMM") { };

    ITimeView(NMM::NMMObject& parent) : NMM::Interface (parent, ITimeView::ITimeView_type, "NMM") { };

    virtual ~ITimeView() {};

    static const char* ITimeView_type;

    virtual  string getInterfaceType() const { return ITimeView_type; }

    NMM::Interface* copy() const { return new ITimeView(*this); }
    static void used(); 
    static const char* setFont_event;
    static const char* setPrefix_event;
    static const char* setTimeOffset_event;
    static const char* setCurrentTime_event;
    static const char* getCurrentTime_event;
    static const char* reset_event;
    /**
     * Set font.
     * Set the font that is used to draw the text.
     * @param value font name
     */

    virtual void setFont(const string& value)
      throw(const NMM::SerializedException&);
    /**
     * Set prefix text.
     * Set prefix text that is placed in front of the time.
     * @param value prefix text
     */

    virtual void setPrefix(const string& value)
      throw(const NMM::SerializedException&);
    /** Sets time offset. */

    virtual void setTimeOffset(const nmm_int32& value)
      throw(const NMM::SerializedException&);
    /** Set current time. */

    virtual void setCurrentTime(const nmm_int32& value)
      throw(const NMM::SerializedException&);
    /** Get current time. */

    virtual nmm_int32 getCurrentTime()
      throw(const NMM::SerializedException&);
    /** Resets timer. */

    virtual void reset()
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ITimeView> ITimeView_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
