/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ITimeViewImpl_HPP
#define NMM_ITimeViewImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "ITimeView.hpp"

#include "IWidgetImpl.hpp"

#include "visibility.hpp"

namespace NMM
{
  /**
   * Time view widget interface.
   */

  class DLLEXPORT ITimeViewImpl  : virtual public IWidgetImpl { 
  public:
    ITimeViewImpl() { addInterface(new ITimeView(*this)); registerEvents(); };

    virtual ~ITimeViewImpl() {};

  protected:
    /**
     * Set font.
     * Set the font that is used to draw the text.
     * @param value font name
     */

    virtual void setFont(const string& value) = 0;
    /**
     * Set prefix text.
     * Set prefix text that is placed in front of the time.
     * @param value prefix text
     */

    virtual void setPrefix(const string& value) = 0;
    /** Sets time offset. */

    virtual void setTimeOffset(const nmm_int32& value) = 0;
    /** Set current time. */

    virtual void setCurrentTime(const nmm_int32& value) = 0;
    /** Get current time. */

    virtual nmm_int32 getCurrentTime() = 0;
    /** Resets timer. */

    virtual void reset() = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
