/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IWidget.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IWidget::used() {} 
  
  
  void IWidget::selectRight()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IWidget::selectRight");
    if(const_cast<IWidget*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IWidget::selectLeft()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IWidget::selectLeft");
    if(const_cast<IWidget*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IWidget::selectUp()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IWidget::selectUp");
    if(const_cast<IWidget*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IWidget::selectDown()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IWidget::selectDown");
    if(const_cast<IWidget*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IWidget::selectPush()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IWidget::selectPush");
    if(const_cast<IWidget*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_int32 IWidget::getWidgetID() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("IWidget::getWidgetID");
    event.setReturnValue(&return_value);
    if(const_cast<IWidget*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IWidget::setPosition(const nmm_uint32& x, const nmm_uint32& y)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > x_value(x, false);
    NMM::TInValue<nmm_uint32 > y_value(y, false);
    NMM::Event event("IWidget::setPosition", &x_value, &y_value);
    if(const_cast<IWidget*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IWidget::setDimension(const nmm_uint32& w, const nmm_uint32& h)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > w_value(w, false);
    NMM::TInValue<nmm_uint32 > h_value(h, false);
    NMM::Event event("IWidget::setDimension", &w_value, &h_value);
    if(const_cast<IWidget*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IWidget::getPosition(nmm_uint32& x, nmm_uint32& y) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > x_value(x);
    NMM::TOutValue<nmm_uint32 > y_value(y);
    NMM::Event event("IWidget::getPosition", &x_value, &y_value);
    if(const_cast<IWidget*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    x = x_value.getRValue();
    y = y_value.getRValue();
  }
  
  void IWidget::getDimension(nmm_uint32& w, nmm_uint32& h) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > w_value(w);
    NMM::TOutValue<nmm_uint32 > h_value(h);
    NMM::Event event("IWidget::getDimension", &w_value, &h_value);
    if(const_cast<IWidget*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    w = w_value.getRValue();
    h = h_value.getRValue();
  }

  const char* IWidget::IWidget_type = "IWidget";
  NMM::TConstructor<IWidget > __IWidget_prototype("IWidget","NMM");


  const char* IWidget::selectRight_event = "IWidget::selectRight";
  const char* IWidget::selectLeft_event = "IWidget::selectLeft";
  const char* IWidget::selectUp_event = "IWidget::selectUp";
  const char* IWidget::selectDown_event = "IWidget::selectDown";
  const char* IWidget::selectPush_event = "IWidget::selectPush";
  const char* IWidget::getWidgetID_event = "IWidget::getWidgetID";
  const char* IWidget::setPosition_event = "IWidget::setPosition";
  const char* IWidget::setDimension_event = "IWidget::setDimension";
  const char* IWidget::getPosition_event = "IWidget::getPosition";
  const char* IWidget::getDimension_event = "IWidget::getDimension";


}


#include "nmm/nmm_suffix.hpp"

