/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IWidgetComposite.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IWidgetComposite::used() {} 
  
  
  void IWidgetComposite::add(const IWidget& wg)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<IWidget > wg_value(wg, false);
    NMM::Event event("IWidgetComposite::add", &wg_value);
    if(const_cast<IWidgetComposite*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IWidgetComposite::remove(const IWidget& wg)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<IWidget > wg_value(wg, false);
    NMM::Event event("IWidgetComposite::remove", &wg_value);
    if(const_cast<IWidgetComposite*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IWidgetComposite::removeAll()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IWidgetComposite::removeAll");
    if(const_cast<IWidgetComposite*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IWidgetComposite::IWidgetComposite_type = "IWidgetComposite";
  NMM::TConstructor<IWidgetComposite > __IWidgetComposite_prototype("IWidgetComposite","NMM");


  const char* IWidgetComposite::add_event = "IWidgetComposite::add";
  const char* IWidgetComposite::remove_event = "IWidgetComposite::remove";
  const char* IWidgetComposite::removeAll_event = "IWidgetComposite::removeAll";


}


#include "nmm/nmm_suffix.hpp"

