/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IWidgetComposite_HPP
#define NMM_IWidgetComposite_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "IWidget.hpp"

#include "visibility.hpp"

namespace NMM
{
  /**
   * Widget composite interfaced.
   */

  class DLLEXPORT IWidgetComposite : virtual public IWidget { 
  public:
    IWidgetComposite() : NMM::Interface(IWidgetComposite::IWidgetComposite_type, "NMM") { };

    IWidgetComposite(NMM::NMMObject& parent) : NMM::Interface (parent, IWidgetComposite::IWidgetComposite_type, "NMM") { };

    virtual ~IWidgetComposite() {};

    static const char* IWidgetComposite_type;

    virtual  string getInterfaceType() const { return IWidgetComposite_type; }

    NMM::Interface* copy() const { return new IWidgetComposite(*this); }
    static void used(); 
    static const char* add_event;
    static const char* remove_event;
    static const char* removeAll_event;
    /** Add widget to composite object. */

    virtual void add(const IWidget& wg)
      throw(const NMM::SerializedException&);
    /** Remove widget from composite object. */

    virtual void remove(const IWidget& wg)
      throw(const NMM::SerializedException&);
    /** Remove all widgets from composite object. */

    virtual void removeAll()
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<IWidgetComposite> IWidgetComposite_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
