/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IWidgetFactory.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IWidgetFactory::used() {} 
  
  
  IWidgetComposite* IWidgetFactory::createWidgetComposite()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<IWidgetComposite* > return_value;
    NMM::Event event("IWidgetFactory::createWidgetComposite");
    event.setReturnValue(&return_value);
    if(const_cast<IWidgetFactory*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IProgressBar* IWidgetFactory::createProgressBar()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<IProgressBar* > return_value;
    NMM::Event event("IWidgetFactory::createProgressBar");
    event.setReturnValue(&return_value);
    if(const_cast<IWidgetFactory*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ITextView* IWidgetFactory::createTextView()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ITextView* > return_value;
    NMM::Event event("IWidgetFactory::createTextView");
    event.setReturnValue(&return_value);
    if(const_cast<IWidgetFactory*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ITimeView* IWidgetFactory::createTimeView()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ITimeView* > return_value;
    NMM::Event event("IWidgetFactory::createTimeView");
    event.setReturnValue(&return_value);
    if(const_cast<IWidgetFactory*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IImageView* IWidgetFactory::createImageView()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<IImageView* > return_value;
    NMM::Event event("IWidgetFactory::createImageView");
    event.setReturnValue(&return_value);
    if(const_cast<IWidgetFactory*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IRadioButtons* IWidgetFactory::createRadioButtons()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<IRadioButtons* > return_value;
    NMM::Event event("IWidgetFactory::createRadioButtons");
    event.setReturnValue(&return_value);
    if(const_cast<IWidgetFactory*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IButtons* IWidgetFactory::createButtons()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<IButtons* > return_value;
    NMM::Event event("IWidgetFactory::createButtons");
    event.setReturnValue(&return_value);
    if(const_cast<IWidgetFactory*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IBorderDecorator* IWidgetFactory::createBorderDecorator(const IWidget& internal)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<IBorderDecorator* > return_value;
    NMM::TInValue<IWidget > internal_value(internal, false);
    NMM::Event event("IWidgetFactory::createBorderDecorator", &internal_value);
    event.setReturnValue(&return_value);
    if(const_cast<IWidgetFactory*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IHeaderDecorator* IWidgetFactory::createHeaderDecorator(const IWidget& internal)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<IHeaderDecorator* > return_value;
    NMM::TInValue<IWidget > internal_value(internal, false);
    NMM::Event event("IWidgetFactory::createHeaderDecorator", &internal_value);
    event.setReturnValue(&return_value);
    if(const_cast<IWidgetFactory*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IPlaneDecorator* IWidgetFactory::createPlaneDecorator(const IWidget& internal)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<IPlaneDecorator* > return_value;
    NMM::TInValue<IWidget > internal_value(internal, false);
    NMM::Event event("IWidgetFactory::createPlaneDecorator", &internal_value);
    event.setReturnValue(&return_value);
    if(const_cast<IWidgetFactory*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IScrollDecorator* IWidgetFactory::createScrollDecorator(const IWidget& internal)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<IScrollDecorator* > return_value;
    NMM::TInValue<IWidget > internal_value(internal, false);
    NMM::Event event("IWidgetFactory::createScrollDecorator", &internal_value);
    event.setReturnValue(&return_value);
    if(const_cast<IWidgetFactory*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IWidgetFactory::IWidgetFactory_type = "IWidgetFactory";
  NMM::TConstructor<IWidgetFactory > __IWidgetFactory_prototype("IWidgetFactory","NMM");


  const char* IWidgetFactory::createWidgetComposite_event = "IWidgetFactory::createWidgetComposite";
  const char* IWidgetFactory::createProgressBar_event = "IWidgetFactory::createProgressBar";
  const char* IWidgetFactory::createTextView_event = "IWidgetFactory::createTextView";
  const char* IWidgetFactory::createTimeView_event = "IWidgetFactory::createTimeView";
  const char* IWidgetFactory::createImageView_event = "IWidgetFactory::createImageView";
  const char* IWidgetFactory::createRadioButtons_event = "IWidgetFactory::createRadioButtons";
  const char* IWidgetFactory::createButtons_event = "IWidgetFactory::createButtons";
  const char* IWidgetFactory::createBorderDecorator_event = "IWidgetFactory::createBorderDecorator";
  const char* IWidgetFactory::createHeaderDecorator_event = "IWidgetFactory::createHeaderDecorator";
  const char* IWidgetFactory::createPlaneDecorator_event = "IWidgetFactory::createPlaneDecorator";
  const char* IWidgetFactory::createScrollDecorator_event = "IWidgetFactory::createScrollDecorator";


}


#include "nmm/nmm_suffix.hpp"

