/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IWindow.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IWindow::used() {} 
  
  
  nmm_int32 IWindow::getWindowID() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("IWindow::getWindowID");
    event.setReturnValue(&return_value);
    if(const_cast<IWindow*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IWindow::setPosition(const nmm_uint32& x, const nmm_uint32& y)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > x_value(x, false);
    NMM::TInValue<nmm_uint32 > y_value(y, false);
    NMM::Event event("IWindow::setPosition", &x_value, &y_value);
    if(const_cast<IWindow*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IWindow::getPosition(nmm_uint32& x, nmm_uint32& y) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > x_value(x);
    NMM::TOutValue<nmm_uint32 > y_value(y);
    NMM::Event event("IWindow::getPosition", &x_value, &y_value);
    if(const_cast<IWindow*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    x = x_value.getRValue();
    y = y_value.getRValue();
  }
  
  void IWindow::setDimension(const nmm_uint32& w, const nmm_uint32& h)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > w_value(w, false);
    NMM::TInValue<nmm_uint32 > h_value(h, false);
    NMM::Event event("IWindow::setDimension", &w_value, &h_value);
    if(const_cast<IWindow*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IWindow::getDimension(nmm_uint32& w, nmm_uint32& h) const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > w_value(w);
    NMM::TOutValue<nmm_uint32 > h_value(h);
    NMM::Event event("IWindow::getDimension", &w_value, &h_value);
    if(const_cast<IWindow*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    w = w_value.getRValue();
    h = h_value.getRValue();
  }
  
  void IWindow::update()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IWindow::update");
    if(const_cast<IWindow*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_bool IWindow::getVisible() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("IWindow::getVisible");
    event.setReturnValue(&return_value);
    if(const_cast<IWindow*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IWindow::setVisible(const nmm_bool& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > value_value(value, false);
    NMM::Event event("IWindow::setVisible", &value_value);
    if(const_cast<IWindow*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IWindow::setScaleFactor(const nmm_double& x, const nmm_double& y)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_double > x_value(x, false);
    NMM::TInValue<nmm_double > y_value(y, false);
    NMM::Event event("IWindow::setScaleFactor", &x_value, &y_value);
    if(const_cast<IWindow*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IWindow::setContent(const IWidget& value)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<IWidget > value_value(value, false);
    NMM::Event event("IWindow::setContent", &value_value);
    if(const_cast<IWindow*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  IWidget* IWindow::getContent() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<IWidget* > return_value;
    NMM::Event event("IWindow::getContent");
    event.setReturnValue(&return_value);
    if(const_cast<IWindow*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IWindow::IWindow_type = "IWindow";
  NMM::TConstructor<IWindow > __IWindow_prototype("IWindow","NMM");


  const char* IWindow::getWindowID_event = "IWindow::getWindowID";
  const char* IWindow::setPosition_event = "IWindow::setPosition";
  const char* IWindow::getPosition_event = "IWindow::getPosition";
  const char* IWindow::setDimension_event = "IWindow::setDimension";
  const char* IWindow::getDimension_event = "IWindow::getDimension";
  const char* IWindow::update_event = "IWindow::update";
  const char* IWindow::getVisible_event = "IWindow::getVisible";
  const char* IWindow::setVisible_event = "IWindow::setVisible";
  const char* IWindow::setScaleFactor_event = "IWindow::setScaleFactor";
  const char* IWindow::setContent_event = "IWindow::setContent";
  const char* IWindow::getContent_event = "IWindow::getContent";


}


#include "nmm/nmm_suffix.hpp"

