/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IWindowImpl.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/messaging/TEDObject.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  
  void IWindowImpl::registerEvents()
  {
  
    getEventDispatcher().registerEvent(IWindow::getWindowID_event, new NMM::TEDObject0<IWindowImpl, nmm_int32(IWindowImpl:: * )() const, nmm_int32 >(this, &IWindowImpl::getWindowID));
  
    getEventDispatcher().registerEvent(IWindow::setPosition_event, new NMM::TEDObject2<IWindowImpl, nmm_uint32, nmm_uint32, void(IWindowImpl:: * )(const nmm_uint32&, const nmm_uint32&), void >(this, &IWindowImpl::setPosition));
  
    getEventDispatcher().registerEvent(IWindow::getPosition_event, new NMM::TEDObject2<IWindowImpl, nmm_uint32, nmm_uint32, void(IWindowImpl:: * )(nmm_uint32&, nmm_uint32&) const, void >(this, &IWindowImpl::getPosition));
  
    getEventDispatcher().registerEvent(IWindow::setDimension_event, new NMM::TEDObject2<IWindowImpl, nmm_uint32, nmm_uint32, void(IWindowImpl:: * )(const nmm_uint32&, const nmm_uint32&), void >(this, &IWindowImpl::setDimension));
  
    getEventDispatcher().registerEvent(IWindow::getDimension_event, new NMM::TEDObject2<IWindowImpl, nmm_uint32, nmm_uint32, void(IWindowImpl:: * )(nmm_uint32&, nmm_uint32&) const, void >(this, &IWindowImpl::getDimension));
  
    getEventDispatcher().registerEvent(IWindow::update_event, new NMM::TEDObject0<IWindowImpl, void(IWindowImpl:: * )(), void >(this, &IWindowImpl::update));
  
    getEventDispatcher().registerEvent(IWindow::getVisible_event, new NMM::TEDObject0<IWindowImpl, nmm_bool(IWindowImpl:: * )() const, nmm_bool >(this, &IWindowImpl::getVisible));
  
    getEventDispatcher().registerEvent(IWindow::setVisible_event, new NMM::TEDObject1<IWindowImpl, nmm_bool, void(IWindowImpl:: * )(const nmm_bool&), void >(this, &IWindowImpl::setVisible));
  
    getEventDispatcher().registerEvent(IWindow::setScaleFactor_event, new NMM::TEDObject2<IWindowImpl, nmm_double, nmm_double, void(IWindowImpl:: * )(const nmm_double&, const nmm_double&), void >(this, &IWindowImpl::setScaleFactor));
  
    getEventDispatcher().registerEvent(IWindow::setContent_event, new NMM::TEDObject1<IWindowImpl, IWidget, void(IWindowImpl:: * )(const IWidget&), void >(this, &IWindowImpl::setContent));
  
    getEventDispatcher().registerEvent(IWindow::getContent_event, new NMM::TEDObject0<IWindowImpl, IWidget*(IWindowImpl:: * )() const, IWidget* >(this, &IWindowImpl::getContent));

  }

}


#include "nmm/nmm_suffix.hpp"

