/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

.text
.align

.global RGBAtoYV12A

@ RGBAtoYV12A(unsigned char* yuv_buffer, unsigend char* rgbbuffer, int width, int height)
RGBAtoYV12A:
        stmdb sp!, {r4-r11, lr}
        @reg0 = yuv_buffer
        @reg1 = rgbbuffer
        @reg2 = width
        @reg3 = height

        sub sp, sp, #48         @reserve space on stack

        @Perpare pointer to YUV-planes
        mov r6, r0              @Backup of yuv_buffer
        mul r4, r2, r3          @r4 = planesize
        str r0, [sp, #0]        @store pointer to u_plane on stack
        str r0, [sp, #20]       @pointer to fist yuv-pixel

        add r0, r0, r4          @r0= v_plane
        str r0, [sp, #8]        @store pointer on stack

        @mov r4, r4, lsr #2      @r4 = planesize/4
        add r0, r0, r4, lsr #2  @r0 = u_plane
        str r0, [sp, #4]        @store pointer on stack

        add r0, r0, r4, lsr #2  @r0 = yuv-alpha-plane
        str r0, [sp, #32]       @store pointer on stack
        
        add r6, r6, r2          @move Pointer to next line
        str r6, [sp, #24]       @and store pointer
        
        @prepare RGBA-pointer
        str r1, [sp, #12]       @store Pointer to first RGBA-Pixel
        str r1, [sp, #28]       @store pointer to RGBA-alpha
        
        mov r5, r2, lsl #2      @r5 = width*4
        add r1, r1, r5          @Pointer to RGBa-pixel in next line
        str r1, [sp, #16]       @store Pointer on stack

        
        str r2, [sp, #36]       @store width
        str r2, [sp, #44]       @store backup-width
        str r3, [sp, #40]       @store height

        @Stack-Description
        @ (sp+ 0) Pointer to y_plane
        @ (sp+ 4) Pointer to u_plane
        @ (sp+ 8) Pointer to v_plane
        @ (sp+12) Pointer to rgba-pixel
        @ (sp+16) Pointer to rgba-pixel in next line
        @ (sp+20) Pointer to 1. yuv-pixel
        @ (sp+24) Pointer to 1. yuv-pixel in next line
        @ (sp+28) Pointer to RGBA-alpha
        @ (sp+32) Pointer to YUV-alpha
        @ (sp+36) width
        @ (sp+40) height
        @ (sp+44) Backup of width
        
        add r12, pc, #(constants-.-8) @pointer to constants array
        
_row_loop:
        @update rgba- and yuv-pointer

_col_loop:
        
        ldr r4, [sp, #12]      @load rgba1
        ldrb r5, [r4, #0]      @load red
        ldrb r6, [r4, #1]      @load green
        ldrb r7, [r4, #2]      @load blue

        @converting Y-component
        @pixel 1
        ldr r11, [r12, #(4*0)] @load constant from constants-array
        mul r8, r5, r11        @r8=red * 0.299 *2^14
        ldr r11, [r12, #(4*1)] @load constant from constants-array      
        mul r9, r6, r11        @r9=green * 0.587 * 2^14
        add r8, r8, r9         @r8=r8+r9
        ldr r11, [r12, #(4*2)] @load constant from constants-array
        mul r9, r7, r11        @r9=blue * 0.114 * 2^14
        add r8, r8, r9         @r8=r8+r9
        mov r8, r8, lsr#14     @r8=r8 / 2^14
        
        ldr r9, [sp, #20]      @load yuv1
        strb r8, [r9, #0]      @store new y-value
        str r9, [sp, #20]      @store yuv-pointer
        
        add r4, r4, #4         @get next pixel

        @pixel 2
        ldrb r2, [r4, #0]      @load red
        ldrb r3, [r4, #1]      @load green
        ldrb r10, [r4, #2]     @load blue

        ldr r11, [r12, #(4*0)] @load constant from constants-array
        mul r8, r2, r11        @r8=red * 0.299 *2^14
        ldr r11, [r12, #(4*1)] @load constant from constants-array
        mul r9, r3, r11        @r9=green * 0.587 * 2^14
        add r8, r8, r9         @r8=r8+r9
        ldr r11, [r12, #(4*2)] @load constant from constants-array
        mul r9, r10, r11       @r9=blue * 0.114 * 2^14
        add r8, r8, r9         @r8=r8+r9
        mov r8, r8, lsr#14     @r8=r8 / 2^14
        
        ldr r9, [sp, #20]      @load yuv
        strb r8, [r9, #1]      @store new y-value
        add r9, r9, #2         @increment yuv-pointer
        str r9, [sp, #20]      @store yuv-pointer

        add r5, r5, r2         @add red-values for u-value-computation
        add r6, r6, r3         @add green-values for u-value-computation
        add r7, r7, r10        @add blue-values for u-value-computation

        add r4, r4, #4         @get next pixel
        
        str r4, [sp, #12]      @store new pixel-pointer
        
        
        @pixel 3 = first pixel in next line
        ldr r4, [sp, #16]      @get next pixel
        ldrb r2, [r4, #0]      @load red
        ldrb r3, [r4, #1]      @load green
        ldrb r10, [r4, #2]     @load blue

        ldr r11, [r12, #(4*0)] @load constant from constants-array
        mul r8, r2, r11        @r8=red * 0.299 *2^14
        ldr r11, [r12, #(4*1)] @load constant from constants-array
        mul r9, r3, r11        @r9=green * 0.587 * 2^14
        add r8, r8, r9         @r8=r8+r9
        ldr r11, [r12, #(4*2)] @load constant from constants-array
        mul r9, r10, r11       @r9=blue * 0.114 * 2^14
        add r8, r8, r9         @r8=r8+r9
        mov r8, r8, lsr#14     @r8=r8 / 2^14
        
        ldr r9, [sp, #24]      @load yuv2
        strb r8, [r9, #0]      @store new y-value
        add r9, r9, #2         @increment yuv-pointer
        str r9, [sp, #24]      @store yuv-pointer

        add r5, r5, r2         @add red-values for u-value-computation
        add r6, r6, r3         @add green-values for u-value-computation
        add r7, r7, r10        @add blue-values for u-value-computation

        add r4, r4, #4         @get next pixel
                
        
        @pixel 4
        ldrb r2, [r4, #0]      @load red
        ldrb r3, [r4, #1]      @load green
        ldrb r10, [r4, #2]     @load blue

        ldr r11, [r12, #(4*0)] @load constant from constants-array
        mul r8, r2, r11        @r8=red * 0.299 *2^14
        ldr r11, [r12, #(4*1)] @load constant from constants-array
        mul r9, r3, r11        @r9=green * 0.587 * 2^14
        add r8, r8, r9         @r8=r8+r9
        ldr r11, [r12, #(4*2)] @load constant from constants-array
        mul r9, r10, r11       @r9=blue * 0.114 * 2^14
        add r8, r8, r9         @r8=r8+r9
        mov r8, r8, lsr#14     @r8=r8 / 2^14
        
        ldr r9, [sp, #24]      @load yuv2
        strb r8, [r9, #-1]      @store new y-value
        str r9, [sp, #24]      @store yuv-pointer

        add r5, r5, r2         @add red-values for u-value-computation
        add r6, r6, r3         @add green-values for u-value-computation
        add r7, r7, r10        @add blue-values for u-value-computation

        add r4, r4, #4         @get next pixel
        str r4, [sp, #16]      @store new pixel-pointer

        
        @calc u-value
        mov r5, r5, lsr #2     @red
        mov r6, r6, lsr #2     @green
        mov r7, r7, lsr #2     @blue

        ldr r11, [r12, #(4*3)] @load constant from constants-array
        mul r8, r5, r11        @r8=red * -0.169 * 2^14
        ldr r11, [r12, #(4*4)] @load constant from constants-array
        mul r9, r6, r11        @r9=green * -0.332 * 2^14
        add r8, r8, r9         @r8=r8+r9
        ldr r11, [r12, #(4*5)] @load constant from constants-array
        mul r9, r7, r11        @r9=blue * 0.5 * 2^14
        add r8, r8, r9         @r8=r8+r9
        add r8, r8, #2097152   @r8=r8+128 * 2^14
        mov r8, r8, lsr#14     @r8=r8 / 2^14
        
        ldr r2, [sp, #4]       @load u_plane-pointer        
        strb r8, [r2, #0]      @store u-value
        add r2, r2, #1         @increment u_plane-pointer
        str r2, [sp, #4]       @store new u_plane-pointer
        

        @calc v-value
        ldr r11, [r12, #(4*5)] @load constant from constants-array
        mul r8, r5, r11        @r8=red * 0.5 * 2^14
        ldr r11, [r12, #(4*6)] @load constant from constants-array
        mul r9, r6, r11        @r9=green * -0.419 * 2^14
        add r8, r8, r9         @r8=r8+r9
        ldr r11, [r12, #(4*7)] @load constant from constants-array
        mul r9, r7, r11        @r9=blue * 0.081 * 2^14
        add r8, r8, r9         @r8=r8+r9
        add r8, r8, #2097152   @r8=r8+128 * 2^14
        mov r8, r8, lsr#14     @r8=r8 / 2^14
        
        ldr r10, [sp, #8]       @load v_plane-pointer        
        strb r8, [r10, #0]      @store v-value
        add r10, r10, #1        @increment v_plane-pointer
        str r10, [sp, #8]       @store new v_plane-pointer

        @copy alpha-values (4 values)
        ldr r2, [sp, #28]       @load RGBA-alpha-pointer
        ldr r3, [sp, #32]       @load YUV-alpha-pointer

        ldrb r4, [r2, #3]       @load Alpha-value
        strb r4, [r3, #0]       @store value in YUV-Image

        add r2, r2, #4          @set RGBA-pointer to next pixel
        add r3, r3, #1          @set YUV-alpha-pointer to next pixel

        ldrb r4, [r2, #3]       @load Alpha-value
        strb r4, [r3, #0]       @store value in YUV-Image
        
        add r2, r2, #4          @set RGBA-pointer to next pixel
        add r3, r3, #1          @set YUV-alpha-pointer to next pixel
        
        ldrb r4, [r2, #3]       @load Alpha-value
        strb r4, [r3, #0]       @store value in YUV-Image
        
        add r2, r2, #4          @set RGBA-pointer to next pixel
        add r3, r3, #1          @set YUV-alpha-pointer to next pixel
        
        ldrb r4, [r2, #3]       @load Alpha-value
        strb r4, [r3, #0]       @store value in YUV-Image
        
        add r2, r2, #4          @set RGBA-pointer to next pixel
        add r3, r3, #1          @set YUV-alpha-pointer to next pixel
        
        str r2, [sp, #28]       @store actual RGBA-alpha-pointer
        str r3, [sp, #32]       @store actual YUV-alpha-pointer
        
        @-----------------------------------------------------
        @end of loop

        ldr r2, [sp, #36]      @load width
        subs r2, r2, #2        @decrement column-counter
        str r2, [sp, #36]      @store width
        bne _col_loop          @if yes, go to next col

        ldr r2, [sp, #44]      @load backup-width for loop
        str r2, [sp, #36]      @reset width for loop
        
        ldr r4, [sp, #12]      @load rgba-pointer for update
        add r4, r4, r2, lsl #2 @move pointer to next valid line
        str r4, [sp, #12]      @store new pointer
        ldr r4, [sp, #16]      @load rgba-nextLine-pointer for update
        add r4, r4, r2, lsl #2 @move pointer to next valid line
        str r4, [sp, #16]      @store new pointer

        ldr r4, [sp, #20]      @load yuv-pointer for update
        add r4, r4, r2         @move pointer to next valid line
        str r4, [sp, #20]      @store new pointer
        ldr r4, [sp, #24]      @load yuv-nextLine-pointer for update
        add r4, r4, r2         @move pointer to next valid line
        str r4, [sp, #24]      @store new pointer

        
        ldr r3, [sp, #40]      @load height
        subs r3, r3, #2        @decrement line-counter
        str r3, [sp, #40]      @store height
        bne _row_loop          @if yes, goto next row
        
_end:
        add sp, sp, #48
        ldmia sp!, {r4-r11, pc}


constants:
        @constants-format:       value * 2^14   (e.g 0.299 * 2^14 = 4898 => 0x00001322 )
        .word 0x00001322, 0x00002591, 0x0000074b, 0xfffff530, 0xffffeac1, 0x00002000, 0xffffe530, 0xfffffacc 

