/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

.text
.align

.global RGBAoverlay			@must be declared for linker (ld)
.global YV12Aoverlay
.global YV12Aoverlay2

@void RGBAoverlay( unsigned char* data, unsigned int stride1, unsigned int w, unsigned int h,
@                  unsigned char* pixel, unsigned int stride2 )
     
RGBAoverlay:
        mov r12, sp 
	stmdb   sp!, { r4 - r11, lr }   @ all callee saved regs
        
      	ldr r4, [r12]              @load pixel
        ldr r5, [r12, #4]          @load stride2       

        mov r7, r1                 @move stride1 to r7
        sub r7, r7, r2             @stride1 - w
        mov r7, r7, lsl #2         @(stride - w) << 2
        
        mov r8, r5                 @move stride2 to r8
        sub r8, r8, r2             @stride2 - w
        mov r8, r8, lsl #2         @(stride - w) << 2

        cmp r3 , #0                @test height against 0
        bls end                    @less or equal than 0 => leave

        sub r2, r2, #1             @decrease width by 1 for loop (otherwise, loop runs 1 to far)
        sub r3, r3, #1             @decrease height by 1 for loop
        mov r6, r2                 @store backup-width
        
        stmdb   sp!, { r0-r8 }

        @ Stack-Description
        @ (sp+0) pointer to data
        @ (sp+4) stride1
        @ (sp+8) w => r2
        @ (sp+12) h => r3
        @ (sp+16) pointer to pixel
        @ (sp+20) stride2
        @ (sp+24) Backup of width
        @ (sp+28) (stride1-w) << 2
        @ (sp+32) (stride2-w) << 2

         
_loop_RGBA_1:
       
        ldr r0, [sp, #0]          @load pointer to data
        ldr r1, [sp, #16]         @load pointer to pixel

        ldrb r2, [r0, #3]         @load alpha1-value of data
        mov r3, #256              @set r3 to 255
        sub r3, r3, r2            @r3=255-alpha1

        @---- red ----
        ldrb r5, [r0, #0]         @load red1-value of data
        mul r7, r5, r2            @r7=alpha1 * red1
        ldrb r6, [r1, #0]         @load red2-value from pixel
        mul r8, r6, r3            @r8=(255-alpha1)*r2
        add r7, r7, r8            @r7=alpha1*red1 + (255 - alpha1) * red2
        mov r7, r7, lsr #8        @r7=r7/255
        strb r7, [r1, #0]         @store new color-value to pixel

        @---- green ----
        ldrb r5, [r0, #1]         @load green1-value of data
        mul r7, r5, r2            @r7=alpha1 * green1
        ldrb r6, [r1, #1]         @load green2-value from pixel
        mul r8, r6, r3            @r8=(255-alpha1)*r2
        add r7, r7, r8            @r7=alpha1*green1 + (255 - alpha1) * green2
        mov r7, r7, lsr #8        @r7=r7/255
        strb r7, [r1, #1]         @store new color-value to pixel

        
        @---- blue ----
        ldrb r5, [r0, #2]         @load blue1-value of data
        mul r7, r5, r2            @r7=alpha1 * blue1
        ldrb r6, [r1, #2]         @load blue2-value from pixel
        mul r8, r6, r3            @r8=(255-alpha1)*r2
        add r7, r7, r8            @r7=alpha1*blue1 + (255 - alpha1) * blue2
        mov r7, r7, lsr #8        @r7=r7/255
        strb r7, [r1, #2]         @store new color-value to pixel

        @---- alpha ----
        ldrb r5, [r1, #3]         @load alpha2-value of data
        mov r6, #255              @set r6 to 255
        mul r7, r5, r6            @r7=alpha2 * 255
        sub r6, r6, r5            @r6=255-alpha2
        mul r8, r6, r2            @r8=(255-alpha2)*alpha1
        add r7, r7, r8            @r7=alpha1*255 + (255 - alpha2) * alpha1
        mov r7, r7, lsr #8        @r7=r7/255
        strb r7, [r1, #3]         @store new color-value to pixel

        

        @--- end of line ---
        add r0, r0, #4            @move pointer to next pixel
        str r0, [sp, #0]          @and store new pointer
        add r1, r1, #4            @move pointer to next pixel
        str r1, [sp, #16]         @and store new pointer
        
        ldr r2, [sp, #8]          @load width into r2
        subs r2, r2, #1           @decrement width-counter
        str r2, [sp, #8]          @store actual width-counter
        bpl _loop_RGBA_1          @loop, if not end-of-line

        @--- end of file ---
        ldr r3, [sp, #12]         @load height into r3
        subs r3, r3, #1           @decrement height by 1
        str r3, [sp, #12]         @store height
        ldr r3, [sp, #24]         @load backup-width
        str r3, [sp, #8]          @restore width to start at beginning of line
        ldr r3, [sp, #0]          @load pointer to data
        ldr r4, [sp, #28]         @load stride1
        add r3, r3, r4            @move pointer by stride1

        str r3, [sp, #0]          @store actual pointer to data
        ldr r3, [sp, #16]         @load pointer to pixel
        ldr r4, [sp, #32]         @load stride2
        add r3, r3, r4            @move pointer by stride2

        str r3, [sp, #16]         @store actual pointer to pixel
        bpl _loop_RGBA_1          @loop if not end-of-picture
        
        add sp, sp, #(9*4)              @releae allocated stack-memory 
       	ldmia   sp!, { r4 - r11, pc }   @ restore callee saved regs and return

        b end                           @return 
@----------------------------------------------------------------------------------------------------
@void YV12Aoverlay(unsigned char* source 0, unsigned int xpos 1, unsigned int ypos 2,
@                  unsigned int w 3, unsigned int h 4,
@	           unsigned char* destination 5, unsigned int image_width 6, unsigned image_height 7)

YV12Aoverlay:
        mov r12, sp 
	stmdb   sp!, { r4 - r11, lr }   @ all callee saved regs

      	ldr r4, [r12, #0]           @load h
      	ldr r5, [r12, #4]           @load destination
        ldr r6, [r12, #8]           @load image_width
        ldr r7, [r12, #12]          @load image_height
        
        sub sp, sp, #28             @make some room on the stack for data
        
        mul r8, r3, r4              @planesize=w * h

        str r0, [sp, #0]            @save source_y to stack
        
        add r0, r0, r8              @compute source_u
        str r0, [sp, #4]            @save source_u to stack
        
        add r0, r0, r8, lsr #2      @compute source_v
        str r0, [sp, #8]            @save source_v to stack
        
        add r0, r0, r8, lsr #2      @compute source _a
        str r0, [sp, #12]           @save source_a to stack

        mul r8, r6, r2              @r8=image_width * ypos
        add r0, r5, r8              @r0=destinaion + r8
        add r0, r0, r1              @r0=r0+xpos
        str r0, [sp, #16]           @save dst_y to stack
        
        mul r0, r6, r7              @r0=image_width * image_height
        add r0, r0, r5              @r0=r0 + destination
        mov r8, r6, lsr #1          @r8=image_width/2
        mov r10, r2, lsr #1         @r10=ypos/2
        mul r9, r8, r10             @r9=image_width/2 * ypos/2
        add r0, r0, r9              @r0=r0 + r9
        add r0, r0, r1, lsr #1      @r0=r0 + xpos/2
        str r0, [sp, #20]           @store dst_u to stack
        
        mul r9, r6, r7              @r9=image_width * image_heigth 
        mov r8, r9, lsr #2          @r8=r9/4
        add r0, r0, r8              @r0=r0 + r8
        str r0, [sp, #24]           @store dst_v to stack

        str r3, [sp, #28]           @store w to stack
        str r4, [sp, #32]           @store h to stack
        str r6, [sp, #36]           @store image_width to stack
        str r7, [sp, #40]           @store image_height to stack
        mov r8, r3                  @save backup of w
        str r8, [sp, #44]           @store backup_w to stack
        
        @ Stack-Description
        @ (sp+0)  pointer to source_y
        @ (sp+4)  pointer to source_u
        @ (sp+8)  pointer to source_v
        @ (sp+12) pointer to source_a
        @ (sp+16) pointer to destination_y
        @ (sp+20) pointer to destination_u
        @ (sp+24) pointer to destination_v
        @ (sp+28) w   =>r3
        @ (sp+32) h   =>r4
        @ (sp+36) image_width   =>r6
        @ (sp+40) image_height  =>r7
        @ (sp+44) backup-width  =>r8

        
        
_loop_YV12A_1:
        
        @--- y-value ---
        @--- 1. y-value
        ldr r0, [sp, #16]          @get pointer to destination_y-value
        ldr r1, [sp, #12]          @get pointer to source_a
        ldr r2, [sp, #0]           @get pointer to source_y-value

        sub r7, r3, #2             @subtract 2 for correct position
       
        ldrb r9, [r0, r7]          @get destination_y-value
        ldrb r10, [r1, r7]         @get source_a-value
        ldrb r11, [r2, r7]         @get source_y-value
        
        sub r11, r11, r9           @r11=source_y - destination_y
        mul r5, r10, r11           @r5=r11 * source_alpha
        add r9, r9, r5, lsr #8     @r9= destination_y + r5/255
        strb r9, [r0, r7]          @store new y-value in destination

        @---2. y-value
        add r7, r7, #1             @get next pixel
        ldrb r9, [r0, r7]          @get destination_y-value
        ldrb r10, [r1, r7]         @get source_a-value
        ldrb r11, [r2, r7]         @get source_y-value
        
        sub r11, r11, r9           @r11=source_y - destination_y
        mul r5, r10, r11           @r5=r11 * source_alpha
        add r9, r9, r5, lsr #8     @r9= destination_y + r5/255
        strb r9, [r0, r7]          @store new y-value in destination

        @--- 3. y-value (1. in next-line)
        add r0, r0, r6             @move dst_y-pointer to next line
        add r1, r1, r8             @move src_alpha-pointer to next line
        add r2, r2, r8             @move source_y-pointer to next line
        sub r7, r7, #1             @get 1. pixel
        
        ldrb r9, [r0, r7]          @get destination_y-value
        ldrb r10, [r1, r7]         @get source_a-value
        ldrb r11, [r2, r7]         @get source_y-value
        
        sub r11, r11, r9           @r11=source_y - destination_y
        mul r5, r10, r11           @r5=r11 * source_alpha
        add r9, r9, r5, lsr #8     @r9= destination_y + r5/255
        strb r9, [r0, r7]          @store new y-value in destination

        @--- 4. pixel
        add r7, r7, #1             @get next pixel
        ldrb r9, [r0, r7]          @get destination_y-value
        ldrb r10, [r1, r7]         @get source_a-value
        ldrb r11, [r2, r7]         @get source_y-value
        
        sub r11, r11, r9           @r11=source_y - destination_y
        mul r5, r10, r11           @r5=r11 * source_alpha
        add r9, r9, r5, lsr #8     @r9= destination_y + r5/255
        strb r9, [r0, r7]          @store new y-value in destination

        
        sub r7, r3, #2
        mov r7, r7, lsr #1         @set r7 to correct position for u/v-calculation
        @--- u-value ---
        ldr r0, [sp, #20]          @get pointer to destination_u-pointer
        ldr r2, [sp, #4]           @get pointer to source_u-pointer
        
        ldrb r9, [r0, r7]          @get destination_u-value
        ldrb r11, [r2, r7]         @get source_u-value
        
        sub r11, r11, r9           @r11=source_u - destination_u
        mul r5, r10, r11           @r5=r11 * source_alpha
        add r9, r9, r5, lsr #8     @r9= destination_u + r5/255
        strb r9, [r0, r7]          @store new u-value in destination

        
        @--- v-value ---
        ldr r0, [sp, #24]          @get pointer to destination_v-pointer
        ldr r2, [sp, #8]           @get pointer to source_v-pointer
        
        ldrb r9, [r0, r7]          @get destrination_v-value
        ldrb r11, [r2, r7]         @get source_v-value
        
        sub r11, r11, r9           @r11=source_v - destination_v
        mul r5, r10, r11           @r5=r11 * source_alpha
        add r9, r9, r5, lsr #8     @r9= destination_v + r5/255
        strb r9, [r0, r7]          @store new v-value in destination

        
_loop_YV12A_EOL:                                      

        @--- test for eol ---
        subs r3, r3, #2            @decrement width-counter
        bne _loop_YV12A_1          @loop, if not end-of-line

        ldr r0, [sp, #16]          @load destination_y-poniter
        add r0, r0, r6, lsl #1     @increment destination_y-pointer
        str r0, [sp, #16]          @and write back to stack        
        ldr r2, [sp, #0]           @load source_y-pointer           
        add r2, r2, r8, lsl #1     @increment source_y-pointer
        str r2, [sp, #0]           @store new source_y-pointer

        ldr r1, [sp, #12]          @get source_a-pointer
        add r1, r1, r8, lsl #1     @increment src_a-pointer
        str r1, [sp, #12]          @and write back to stack

        ldr r0, [sp, #20]          @get pointer to destination_u-pointer
        add r0, r0, r6, lsr #1     @move destination_u-pointer by w/2
        str r0, [sp, #20]          @store new pointer
        ldr r2, [sp, #4]           @get pointer to source_u-pointer
        add r2, r2, r8, lsr #1     @move source_u-pointer by w/2
        str r2, [sp, #4]           @store new pointer        

        ldr r0, [sp, #24]          @get pointer to destination_v-pointer
        add r0, r0, r6, lsr #1     @move destination_v-pointer by w/2
        str r0, [sp, #24]          @store new pointer
        ldr r2, [sp, #8]           @get pointer to source_v-pointer
        add r2, r2, r8, lsr #1     @move source_v-pointer by w/2
        str r2, [sp, #8]           @store new pointer
        
_loop_YV12A_EOF:                                              
        @--- go to next line ---
        subs r4, r4, #2            @decrement height by 1
        mov r3, r8                 @restore width for beginning at begin-of-line
        bne _loop_YV12A_1          @loop if not end-of-picture

        add sp, sp, #28
       	ldmia   sp!, { r4 - r11, pc }   @ restore callee saved regs and return

        b end
@-----------------------------------------------------------------------------------------------------
@void YV12Aoverlay2(unsigned char* source r0, unsigned int xpos r1, unsigned int ypos r2,
@                  unsigned int xstart r3, unsigned int ystart #0, unsigned int seg_width #4, unsigned int seg_height #8,
@                  unsigned int w #12, unsigned int h #16,
@	           unsigned char* destination #20, unsigned int image_width #24, unsigned image_height #28)

YV12Aoverlay2:
        mov r12, sp 
	stmdb   sp!, { r4 - r11, lr }   @ all callee saved regs

      	ldr r4, [r12, #0]           @load y_start
      	ldr r5, [r12, #20]          @load destination
        ldr r6, [r12, #24]          @load image_width
        ldr r7, [r12, #28]          @load image_height
        
        sub sp, sp, #52             @make some room on the stack for data
        
        ldr r10, [r12, #12]         @load w
        add r0, r0, r3              @r0=source+xstart
        mul r9, r4, r10             @ystart*w
        add r0, r0, r9              @r0=source+xstart+ystart*w 
        str r0, [sp, #0]            @save source_y to stack

        ldr r11, [r12, #16]         @load h
        mul r8, r10, r11            @planesize=w * h
                
        add r0, r0, r8              @r0=source_u+planesize
        add r0, r0, r3, lsr #1      @r0=source_u+planesize+xstart/2
        mul r11, r4, r10            @r11=ystart*w
        add r0, r0, r11, lsr #2     @r0=source_u+planesize+xstart/2+(ystart/2)*(w/2)
        str r0, [sp, #4]            @save source_u to stack
        
        add r0, r0, r8, lsr #2      @r0=source_v+planesize/4
        add r0, r0, r3, lsr #1      @r0=r0+xstart/2
        add r0, r0, r11, lsr #2     @r0=r0+ystart/2*w/2
        str r0, [sp, #8]            @save source_v to stack
        
        add r0, r0, r8, lsr #2      @compute source _a
        str r0, [sp, #12]           @save source_a to stack

        mul r8, r6, r2              @r8=image_width * ypos
        add r0, r5, r8              @r0=destinaion + r8
        add r0, r0, r1              @r0=r0+xpos
        str r0, [sp, #16]           @save dst_y to stack
        
        mul r0, r6, r7              @r0=image_width * image_height
        add r0, r0, r5              @r0=r0 + destination
        mov r8, r6, lsr #1          @r8=image_width/2
        mov r10, r2, lsr #1         @r10=ypos/2
        mul r9, r8, r10             @r9=image_width/2 * ypos/2
        add r0, r0, r9              @r0=r0 + r9
        add r0, r0, r1, lsr #1      @r0=r0 + xpos/2
        str r0, [sp, #20]           @store dst_u to stack
        
        mul r9, r6, r7              @r9=image_width * image_heigth 
        mov r8, r9, lsr #2          @r8=r9/4
        add r0, r0, r8              @r0=r0 + r8
        str r0, [sp, #24]           @store dst_v to stack

        ldr r3, [r12, #12]          @load w
        str r3, [sp, #48]           @store w 
        ldr r3, [r12, #16]          @load h
        str r3, [sp, #52]           @store h
        
        ldr r3, [r12, #4]           @load seg_width
        ldr r4, [r12, #8]           @load seg_height
        mov r8, r3                  @save backup of w

        str r3, [sp, #28]           @store seq_width to stack
        str r4, [sp, #32]           @store seq_height to stack
        str r6, [sp, #36]           @store image_width to stack
        str r7, [sp, #40]           @store image_height to stack
        str r8, [sp, #44]           @@store backup-seq_width to stack
        
        @ Stack-Description
        @ (sp+0)  pointer to source_y
        @ (sp+4)  pointer to source_u
        @ (sp+8)  pointer to source_v
        @ (sp+12) pointer to source_a
        @ (sp+16) pointer to destination_y
        @ (sp+20) pointer to destination_u
        @ (sp+24) pointer to destination_v
        @ (sp+28) seg_width   =>r3
        @ (sp+32) seg_height   =>r4
        @ (sp+36) image_width   =>r6
        @ (sp+40) image_height  =>r7
        @ (sp+44) backup-seg_width  =>r8
        @ (sp+48) w
        @ (sp+52) h
        
_loop_YV12A2_1:
        
        @--- y-value ---
        @--- 1. y-value
        ldr r0, [sp, #16]          @get pointer to destination_y-value
        ldr r1, [sp, #12]          @get pointer to source_a
        ldr r2, [sp, #0]           @get pointer to source_y-value

        sub r7, r3, #2             @subtract 2 for correct position
       
        ldrb r9, [r0, r7]          @get destination_y-value
        ldrb r10, [r1, r7]         @get source_a-value
        ldrb r11, [r2, r7]         @get source_y-value
        
        sub r11, r11, r9           @r11=source_y - destination_y
        mul r5, r10, r11           @r5=r11 * source_alpha
        add r9, r9, r5, lsr #8     @r9= destination_y + r5/255
        strb r9, [r0, r7]          @store new y-value in destination

        @---2. y-value
        add r7, r7, #1             @get next pixel
        ldrb r9, [r0, r7]          @get destination_y-value
        ldrb r10, [r1, r7]         @get source_a-value
        ldrb r11, [r2, r7]         @get source_y-value
        
        sub r11, r11, r9           @r11=source_y - destination_y
        mul r5, r10, r11           @r5=r11 * source_alpha
        add r9, r9, r5, lsr #8     @r9= destination_y + r5/255
        strb r9, [r0, r7]          @store new y-value in destination

        @--- 3. y-value (1. in next-line)
        ldr r10, [sp, #44]         @load w
        add r0, r0, r6             @move dst_y-pointer to next line
        add r1, r1, r10             @move src_alpha-pointer to next line
        add r2, r2, r10             @move source_y-pointer to next line
        sub r7, r7, #1             @get 1. pixel
        
        ldrb r9, [r0, r7]          @get destination_y-value
        ldrb r10, [r1, r7]         @get source_a-value
        ldrb r11, [r2, r7]         @get source_y-value
        
        sub r11, r11, r9           @r11=source_y - destination_y
        mul r5, r10, r11           @r5=r11 * source_alpha
        add r9, r9, r5, lsr #8     @r9= destination_y + r5/255
        strb r9, [r0, r7]          @store new y-value in destination

        @--- 4. pixel
        add r7, r7, #1             @get next pixel
        ldrb r9, [r0, r7]          @get destination_y-value
        ldrb r10, [r1, r7]         @get source_a-value
        ldrb r11, [r2, r7]         @get source_y-value
        
        sub r11, r11, r9           @r11=source_y - destination_y
        mul r5, r10, r11           @r5=r11 * source_alpha
        add r9, r9, r5, lsr #8     @r9= destination_y + r5/255
        strb r9, [r0, r7]          @store new y-value in destination

        
        sub r7, r3, #2
        mov r7, r7, lsr #1         @set r7 to correct position for u/v-calculation
        @--- u-value ---
        ldr r0, [sp, #20]          @get pointer to destination_u-pointer
        ldr r2, [sp, #4]           @get pointer to source_u-pointer
        
        ldrb r9, [r0, r7]          @get destination_u-value
        ldrb r11, [r2, r7]         @get source_u-value
        
        sub r11, r11, r9           @r11=source_u - destination_u
        mul r5, r10, r11           @r5=r11 * source_alpha
        add r9, r9, r5, lsr #8     @r9= destination_u + r5/255
        strb r9, [r0, r7]          @store new u-value in destination

        
        @--- v-value ---
        ldr r0, [sp, #24]          @get pointer to destination_v-pointer
        ldr r2, [sp, #8]           @get pointer to source_v-pointer
        
        ldrb r9, [r0, r7]          @get destrination_v-value
        ldrb r11, [r2, r7]         @get source_v-value
        
        sub r11, r11, r9           @r11=source_v - destination_v
        mul r5, r10, r11           @r5=r11 * source_alpha
        add r9, r9, r5, lsr #8     @r9= destination_v + r5/255
        strb r9, [r0, r7]          @store new v-value in destination

        
_loop_YV12A2_EOL:                                      

        @--- test for eol ---
        subs r3, r3, #2            @decrement width-counter
        bne _loop_YV12A2_1          @loop, if not end-of-line

        ldr r10, [sp, #48]         @load w
        
        ldr r0, [sp, #16]          @load destination_y-poniter
        add r0, r0, r6, lsl #1     @increment destination_y-pointer
        str r0, [sp, #16]          @and write back to stack        
        ldr r2, [sp, #0]           @load source_y-pointer           
        add r2, r2, r10, lsl #1    @increment source_y-pointer
        str r2, [sp, #0]           @store new source_y-pointer

        ldr r1, [sp, #12]          @get source_a-pointer
        add r1, r1, r10, lsl #1    @increment src_a-pointer
        str r1, [sp, #12]          @and write back to stack

        ldr r0, [sp, #20]          @get pointer to destination_u-pointer
        add r0, r0, r6, lsr #1     @move destination_u-pointer by w/2
        str r0, [sp, #20]          @store new pointer
        ldr r2, [sp, #4]           @get pointer to source_u-pointer
        add r2, r2, r10, lsr #1    @move source_u-pointer by w/2
        str r2, [sp, #4]           @store new pointer        

        ldr r0, [sp, #24]          @get pointer to destination_v-pointer
        add r0, r0, r6, lsr #1     @move destination_v-pointer by w/2
        str r0, [sp, #24]          @store new pointer
        ldr r2, [sp, #8]           @get pointer to source_v-pointer
        add r2, r2, r10, lsr #1    @move source_v-pointer by w/2
        str r2, [sp, #8]           @store new pointer
        
_loop_YV12A2_EOF:                                              
        @--- go to next line ---
        subs r4, r4, #2            @decrement height by 2
        mov r3, r8                 @restore width for beginning at begin-of-line
        bne _loop_YV12A2_1          @loop if not end-of-picture

        add sp, sp, #52
       	ldmia   sp!, { r4 - r11, pc }   @ restore callee saved regs and return

        b end

end:    
