; Network-Integrated Multimedia Middleware (NMM)
; http://www.motama.com/nmm.html
;  
; Copyright (C) 2006-2007
; Motama GmbH, Saarbruecken, Germany
; http://www.motama.com
;  
; This file is part of NMM. 
; 
; NMM is free software; you can redistribute it and/or modify it    
; under the terms of the GNU General Public License as published by 
; the Free Software Foundation; only version 2 of the License.      
; 
; NMM is distributed in the hope that it will be useful, but WITHOUT 
; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
; or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
; License for more details.  
; 
; You should have received a copy of the GNU General Public License 
; along with NMM; if not, write to the Free Software Foundation,    
; Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
;
;

; there we can find information about our data offset
; because we can not simply access data if the file is linked dynamically
extern _GLOBAL_OFFSET_TABLE_

; this macro returns an offset in EBX that must be added to pointers
; to get access to the data section
%macro get_GOT 0
          call %%getgot
%%getgot: pop ebx
          add ebx,_GLOBAL_OFFSET_TABLE_+$$-%%getgot wrt ..gotpc
%endmacro

global RGBAoverlay:function			;must be declared for linker (ld)
global YV12Aoverlay:function
global YV12Aoverlay2:function

;void RGBAoverlay( unsigned char* data, unsigned int stride1, unsigned int w, unsigned int h,
;                  unsigned char* pixel, unsigned int stride2 )

section	.text

RGBAoverlay:		       ; tell linker entry point
	push      ebp
 	mov       ebp,esp
	push      ebx
	push      esi
	push      edi

	mov       eax, [ebp+16]; eax = width
        or        eax, eax
	jz        NEAR _end				; if width==0 then exit

	mov       ecx, [ebp+20]; ecx = height
 	or        ecx, ecx
	jz        NEAR _end				; if height==0 then exit

	get_GOT                                         ; get global offset table
	pxor      mm5, mm5                              ; mm5 = 0
	movq      mm6, [ebx + mask_1 wrt ..gotoff]	; mm6 = 01 00 00 00 00 00 00 00
	movq      mm7, [ebx + mask_2 wrt ..gotoff]      ; mm7 = 00 00 ff ff ff ff ff ff

	mov       esi, [ebp+8]                          ; esi = pointer to source plane      (foreground)
	mov       ebx, [ebp+12]                         ; ebx = stride1

	mov       edi, [ebp+24]                         ; edi = pointer to destination plane (background)
	mov       edx, [ebp+28]                         ; edx = stride2

	sub       ebx, eax
	shl       ebx, 2	                        ; ebx = (stride1-width)*4
	sub       edx, eax
	shl       edx, 2	                        ; edx = (stride2-width)*4

_loop1:
	mov       eax, [ebp+16] ; eax = width

_loop2:
	movd      mm0, [esi]   ; mm0 = 00 00 00 00 A1 B1 G1 R1
	movd      mm1, [edi]   ; mm1 = 00 00 00 00 A2 B2 G2 R2


	punpcklbw mm0, mm5     ; mm0 = 00 A1 00 B1 00 G1 00 R1
	punpcklbw mm1, mm5     ; mm1 = 00 A2 00 B2 00 G2 00 R2

	movq      mm2, mm0
	punpckhwd mm2, mm2     ; mm2 = 00 A1 00 A1 xx xx xx xx
	punpckhdq mm2, mm2     ; mm2 = 00 A1 00 A1 00 A1 00 A1

	pand      mm0, mm7
	por       mm0, mm6     ; mm0 = 10 00 00 B1 00 G1 00 B1

	psubw     mm0, mm1     ; mm0 = mm0-mm1	= (0x100-A2) (B1-B2) (G1-G2) (R1-R2)
	pmullw    mm0, mm2     ; mm0 = A1*(0x100-A2) | A1*(B1-B2) | A1*(G1-G2) | A1*(R1-R2)

        psllw     mm1, 8
	paddw     mm0, mm1     ; mm0 = A1*(FF-A2)+A2 | A1*(B1-B2)+B2 | A1*(G1-G2)+G2 | A1*(R1-R2)+R2

	psrlw     mm0, 8

	packuswb  mm0, mm0     ; mm0 = A2 B2 G2 R2 A3 B3 G3 R3

	movd     [edi], mm0

	add      esi, 4
	add      edi, 4
	dec	 eax
	jnz      _loop2

	add      esi, ebx
	add      edi, edx
	dec	 ecx
	jnz 	 _loop1

	emms
_end:
	pop      edi
	pop      esi

	mov      ebx,[ebp-4]
        leave                  ; mov esp,ebp / pop ebp
        ret

;void YV12Aoverlay(unsigned char* source 8, unsigned int xpos 12, unsigned int ypos 16,
;                  unsigned int w 20, unsigned int h 24,
;	           unsigned char* destination 28, unsigned int image_width 32, unsigned image_height 36)

YV12Aoverlay:		       ; tell linker entry point
	push      ebp
 	mov       ebp,esp
	push      ebx
	push      esi
	push      edi

	mov       ecx, [ebp+20]; ecx = width (source)
        or        ecx, ecx
	jz        NEAR _yend				; if width==0 then exit

	mov       edx, [ebp+24]; edx = height (source)
 	or        edx, edx
	jz        NEAR _yend				; if height==0 then exit


	mov       esi, [ebp+8]     ; esi = pointer to Y plane (source)
	mov       edi, esi	   ; edi = esi

	mov       eax, ecx
	mul       edx		   ; eax     = width*height (source)
	add       edi, eax	   ; edi     = pointer to U-plane (source)
	mov       [ebp+8], edi	   ; [ebp+8] = pointer to U-plane (source) <<<<<<
	shr       eax, 2	   ; eax     = (width*height)/4
	add       edi, eax	   ; edi     = pointer to V-plane (source)

	mov       ebx, [ebp+12]    ; ebx = xpos

	mov	  [ebp+12], edi    ; [ebp+12]= pointer to V-plane (source) <<<<<
	add       edi, eax         ; edi     = pointer to A-plane (source)

	mov       eax, [ebp+16]    ; eax = ypos
	mov	  [ebp+16], edi    ; [ebp+16]= pointer to A-plane (source) <<<<<

	mov       edi, [ebp+28]    ; edi = pointer to Y plane (destination)

	mov	  ecx, eax         ; ecx      = ypos
	mul	  dword [ebp+32]   ; eax      = ypos*image_width
	add       eax, ebx         ; eax      = ypos*image_width+xpos
	add       [ebp+28], eax    ; [ebp+28] = pointer to Y-plane (destination) (at position xpos, ypos) <<<<

	mov       eax, [ebp+32]    ; eax = image_width (destination)
	mul       ecx              ; eax = imgage_width*ypos
	shr       eax, 2           ; eax = image_width/2 * ypos/2
	shr       ebx, 1           ; ebx = xpos/2
	add       ebx, eax         ; ebx = image_width/2 * ypos/2 + xpos/2

	mov       eax, [ebp+32]    ; eax = image_width (destination)
	mov       ecx, eax	   ; ecx = image_width (destination)
	mul       dword [ebp+36]   ; eax = image_width*image_height

	add       edi, eax
	add       edi, ebx	   ; edi = destination+image_width*image_height+(image_width/2)*(ypos/2)+(xpos/2)
	mov       [ebp+32], edi    ; [ebp+32] = pointer to U-plane (destination) (at position xpos, ypos) <<<<<

        shr       eax, 2           ; eax = (image_width*image_height)/4
	add       edi, eax
	mov       [ebp+36], edi    ; [ebp+36] = pointer to V-plane (destination) (at position xpos, ypos) <<<<<

	mov       eax,  [ebp+20]    ; eax = width (source)
	mov       [ebp+20], ecx     ; [ebp+20] = width (destination)
	shr       dword [ebp+24], 1 ; [ebp+24] = height/2 (source)

	; esi    = Y-Plane (Source)
	; ebp+ 8 = U-Plane (Source)
	; ebp+12 = V-Plane (Source)
	; ebp+16 = A-Plane (Source)
	; eax    = width   (Source)
	; ebp+24 = height/2(Source)

	; ebp+20 = width   (Destination)
	; ebp+28 = Y-Plane (Destination)
	; ebp+32 = U-Plane (Destination)
	; ebp+36 = V-Plane (Destination)


	pxor      mm5, mm5	   ; mm5 = 0
_yloop1:
	mov       ecx, eax
	shr       ecx, 1	   ; ecx = source_width/2

	mov       edx, [ebp+20]	   ; edx = destination_width

_yloop2:
        ; get 4 Alpha Pixels from the source         (->mm2)
	mov       edi, [ebp+16]
	mov       bx,  [edi]
	shl       ebx, 16
	mov       bx,  [edi+eax]   ; ebx = A2 A1 A4 A3
	or        bx, bx
	jz        NEAR _next_pixels
	movd      mm2, ebx         ; mm2 = 00 00 00 00 A2  A1  A4  A3

        ; get 4 Y Pixels from the source             (->mm0)
	mov	  bx,  [esi]
	shl       ebx, 16          ; ebx = SY2 SY1 00 00
	mov       bx,  [esi+eax]   ; ebx = SY2 SY1 SY4 SY3
	movd      mm0, ebx         ; mm0 = 00 00 00 00 SY2 SY1 SY4 SY3

	; get 1 U and 1 V Pixel from the source      (->mm1)
	xor	  ebx, ebx
	mov       edi, [ebp+8]
	mov	  bh,  [edi]	   ; bx = SU1 00
	mov       edi, [ebp+12]
	mov       bl,  [edi]       ; bx = SU1 SV1
	movd      mm1, ebx	   ; mm1 = 00 00 00 00 00  00  SU1 SV1

	; get 1 U and 1 V Pixel from the destination (->mm4)
	mov       edi, [ebp+32]
	mov	  bh,  [edi]	   ; bx = DU1 00
	mov       edi, [ebp+36]
	mov       bl,  [edi]       ; bx = DU1 DV1
	movd      mm4, ebx	   ; mm1 = 00 00 00 00 00  00  DU1 DV1

	; get 4 Y Pixels from the destination        (->mm3)
	mov       edi, [ebp+28]
	mov       bx,  [edi]
	shl       ebx, 16
	mov       bx,  [edi+edx]   ; ebx = DY2 DY1 DY4 DY3
	movd      mm3, ebx         ; mm3 = 00 00 00 00 DY2 DY1 DY4 DY3

	; ---------------------------------
	; BEGIN: mmx overlay of a 2x2 block
	; ---------------------------------
	punpcklbw mm0, mm5     ; mm0 = 00 SY2 00 SY1 00 SY4 00 SY3
	punpcklbw mm1, mm5     ; mm1 = 00 00  00 00  00 SU1 00 SV1
	punpcklbw mm2, mm5     ; mm2 = 00 A2  00 A1  00 A4  00 A3
	punpcklbw mm3, mm5     ; mm3 = 00 DY2 00 DY1 00 DY4 00 DY3
	punpcklbw mm4, mm5     ; mm4 = 00 00  00 00  00 DU1 00 DV1

	; calculate Y value
	psubw     mm0, mm3     ; mm0 = mm0-mm3	= (SY2-DY2) (SY1-DY1) (SY4-DY4) (SY3-DY3)
	pmullw    mm0, mm2     ; mm0 = A2*(SY2-DY2) | A1*(SY1-DY1) | A4*(SY4-DY4) | A3*(SY3-DY3)

        psllw     mm3, 8       ; mm3 = mm3*256
	paddw     mm0, mm3     ; mm0 = A2*(SY2-DY2) + DY2 | A1*(SY1-DY1) + DY1 | A4*(SY4-DY4) + DY4 | A3*(SY3-DY3) + DY3

	psrlw     mm0, 8       ; mm0 = mm0/256
	packuswb  mm0, mm0     ; mm0 = xx xx xx xx Y2 Y1 Y4 Y3

	; calculate U and V value
	psubw     mm1, mm4     ; mm1 = mm1-mm4	= 00 00 00 00 (SU1-DU1) (SV1-DV1)
	pmullw    mm1, mm2     ; mm1 = 00 00 00 00 A4*(SU1-DU1) | A3*(SV1-DV1)

        psllw     mm4, 8       ; mm4 = mm4*256
	paddw     mm1, mm4     ; mm1 = 00 00 00 00 A4*(SU1-DU1) + DU1 | A3*(SV1-DV1) + DV1

	psrlw     mm1, 8       ; mm1 = mm1/256
	packuswb  mm1, mm1     ; mm1 = xx xx xx xx 00 00 U V
	; ---------------------------------
	; END: mmx overlay of a 2x2 block
	; ---------------------------------

	; write V and U Pixel to destination
	movd      ebx,   mm1
	mov	  edi,   [ebp+36]
	mov       [edi], bl       ; V -> [[ebp+36]]
        shr       bx,    8
	mov       edi,   [ebp+32]
	mov       [edi], bl       ; U -> [[ebp+32]]

	; write 4 Y Pixels to destination
	movd      ebx,   mm0
	mov       edi,   [ebp+28]
	mov       [edi+edx], bx
	shr       ebx,   16
	mov       [edi], bx	  ; Y -> [[ebp+28]]

_next_pixels:
	; Calculate new source plane pointers
	add      esi, 2		   ; go to next 4 Y Pixels (2 in a row and 2 in a column)
	inc      dword [ebp+8]	   ; go to next U Pixel
	inc      dword [ebp+12]    ; go to next V Pixel
	add      dword [ebp+16], 2 ; go to next 4 Alpha Pixels (2 in a row and 2 in a column)

	; Calculate new destination plane pointers
	add      dword [ebp+28], 2 ; go to next 4 Y Pixels
	inc      dword [ebp+32]    ; go to next U Pixel
	inc      dword [ebp+36]    ; go to next V Pixel

	dec	 ecx		   ; (width/2)--
	jnz      NEAR _yloop2	   ; jump, if height > 0

	; process new line...
	add      esi, eax	   ; esi = esi+source width (skip 1 line, because we processed a 2x2 block)
	add      [ebp+16], eax     ; the same for the alpha plane

	mov      ebx, edx	   ; ebx = destination width
	shr      ebx, 1		   ; for u and v we need only half the pixels
	mov      ecx, eax	   ; ecx = source width
	shr      ecx, 1
	sub      ebx, ecx	   ; ebx = stride to next line

	add      [ebp+32], ebx	   ; pointer to next line of the u-plane (dest)
	add      [ebp+36], ebx     ; pointer to next line of the v-plane (dest)

	shl      edx, 1		   ; edx = 2*width of destination (because we skip one line)
	sub      edx, eax
	add      [ebp+28], edx     ; pointer to (over) next line of the y-plane (dest)

	dec      dword [ebp+24]    ; (height/2)--
	jnz 	 NEAR _yloop1

	emms
_yend:
	pop      edi
	pop      esi
	mov      ebx,[ebp-4]
        leave                  ; mov esp,ebp / pop ebp
        ret

;void YV12Aoverlay2(unsigned char* source 8, unsigned int xpos 12, unsigned int ypos 16,
;                  unsigned int xstart 20, unsigned int ystart 24, unsigned int seg_width 28, unsigned int seg_height 32,
;                  unsigned int w 36, unsigned int h 40,
;	           unsigned char* destination 44, unsigned int image_width 48, unsigned image_height 52)

YV12Aoverlay2:		       ; tell linker entry point
	push      ebp
 	mov       ebp,esp
	push      ebx
	push      esi
	push      edi

	mov       ecx, [ebp+36]; ecx = width (source)
        or        ecx, ecx
	jz        NEAR _yend2				; if width==0 then exit

	mov       edx, [ebp+40]; edx = height (source)
 	or        edx, edx
	jz        NEAR _yend2				; if height==0 then exit


	mov       esi, [ebp+8]     ; esi = pointer to Y plane (source)
	mov       edi, esi	   ; edi = esi

	mov       eax, ecx
	mul       edx		   ; eax     = width*height (source)
	add       edi, eax         ; edi     = pointer to U-plane (source)
	mov       [ebp+8], edi	   ; [ebp+8] = pointer to U-plane (source)
	shr       eax, 2	   ; eax     = (width*height)/4
	add       edi, eax	   ; edi     = pointer to V-plane (source)

	mov       ebx, [ebp+12]    ; ebx = xpos

	mov	  [ebp+12], edi    ; [ebp+12]= pointer to V-plane (source)
	add       edi, eax         ; edi     = pointer to A-plane (source)

	mov       ecx, [ebp+16]    ; ecx = ypos
	mov	  [ebp+16], edi    ; [ebp+16]= pointer to A-plane (source)

	mov	  eax, [ebp+24]    ; eax = ystart
	mul	  dword [ebp+36]   ; eax = ystart*w
	mov       edx, eax         ; edx = ystart*w
	add       eax, [ebp+20]    ; eax = ystart*w+xstart
	add       esi, eax         ; esi = pointer to Y-plane (source) (at position xstart, ystart)
	add       [ebp+16], eax    ; [ebp+16]= pointer to A-plane (source) (at position xstart, ystart)
	shr       edx, 2           ; edx = ystart/2 * w/2
	mov       eax, [ebp+20]    ; eax = xstart
	shr       eax, 1	   ; eax = xtart/2
	add       eax, edx	   ; eax = ystart/2 * w/2 + xstart/2
	add       [ebp+8], eax     ; [ebp+8] = pointer to U-plane (source) (at position xstart, ystart)
	add       [ebp+12], eax    ; [ebp+12]= pointer to V-plane (source) (at position xstart, ystart)


	mov       edi, [ebp+44]    ; [ebp+44] = pointer to Y plane (destination)

	mov	  eax, ecx         ; eax      = ypos
	mul	  dword [ebp+48]   ; eax      = ypos*image_width
	add       eax, ebx         ; eax      = ypos*image_width+xpos
	add       [ebp+44], eax    ; [ebp+44] = pointer to Y-plane (destination) (at position xpos, ypos) <<<<

	mov       eax, [ebp+48]    ; eax = image_width (destination)
	mul       ecx              ; eax = image_width*ypos
	shr       eax, 2           ; eax = image_width/2 * ypos/2
	shr       ebx, 1           ; ebx = xpos/2
	add       ebx, eax         ; ebx = image_width/2 * ypos/2 + xpos/2

	mov       eax, [ebp+48]    ; eax = image_width (destination)
	mov       ecx, eax	   ; ecx = image_width (destination)
	mul       dword [ebp+52]   ; eax = image_width*image_height

	add       edi, eax
	add       edi, ebx	   ; edi = destination+image_width*image_height+(image_width/2)*(ypos/2)+(xpos/2)
	mov       [ebp+48], edi    ; [ebp+48] = pointer to U-plane (destination) (at position xpos, ypos) <<<<<

        shr       eax, 2           ; eax = (image_width*image_height)/4
	add       edi, eax
	mov       [ebp+52], edi    ; [ebp+52] = pointer to V-plane (destination) (at position xpos, ypos) <<<<<

	mov        eax,  [ebp+36]
	mov        edx,  [ebp+28]    ; edx = segment width (source)
	sub       [ebp+36], edx      ; [ebp+36] = source width - segment width = stride

	mov       [ebp+20], ecx     ; [ebp+20] = width (destination)
	shr       dword [ebp+32], 1 ; [ebp+32] = height/2 (source)

	; esi    = Y-Plane       (Source)
	; ebp+ 8 = U-Plane       (Source)
	; ebp+12 = V-Plane       (Source)
	; ebp+16 = A-Plane       (Source)
	; eax    = width         (Source)
	; ebp+32 = height/2      (Source)
	; ebp+28 = segment width (Source)
	; ebp+36 = stride        (Source)

	; ebp+20 = width   (Destination)
	; ebp+44 = Y-Plane (Destination)
	; ebp+48 = U-Plane (Destination)
	; ebp+52 = V-Plane (Destination)


	pxor      mm5, mm5	   ; mm5 = 0
_yloop12:
	mov       ecx, [ebp+28]
	shr       ecx, 1	   ; ecx = source_width/2

	mov       edx, [ebp+20]	   ; edx = destination_width

_yloop22:
        ; get 4 Alpha Pixels from the source         (->mm2)
	mov       edi, [ebp+16]
	mov       bx,  [edi]
	shl       ebx, 16
	mov       bx,  [edi+eax]   ; ebx = A2 A1 A4 A3
	or        bx, bx
	jz        NEAR _next_pixels2
	movd      mm2, ebx         ; mm2 = 00 00 00 00 A2  A1  A4  A3

        ; get 4 Y Pixels from the source             (->mm0)
	mov	  bx,  [esi]
	shl       ebx, 16          ; ebx = SY2 SY1 00 00
	mov       bx,  [esi+eax]   ; ebx = SY2 SY1 SY4 SY3
	movd      mm0, ebx         ; mm0 = 00 00 00 00 SY2 SY1 SY4 SY3

	; get 1 U and 1 V Pixel from the source      (->mm1)
	xor	  ebx, ebx
	mov       edi, [ebp+8]
	mov	  bh,  [edi]	   ; bx = SU1 00
	mov       edi, [ebp+12]
	mov       bl,  [edi]       ; bx = SU1 SV1
	movd      mm1, ebx	   ; mm1 = 00 00 00 00 00  00  SU1 SV1

	; get 1 U and 1 V Pixel from the destination (->mm4)
	mov       edi, [ebp+48]
	mov	  bh,  [edi]	   ; bx = DU1 00
	mov       edi, [ebp+52]
	mov       bl,  [edi]       ; bx = DU1 DV1
	movd      mm4, ebx	   ; mm1 = 00 00 00 00 00  00  DU1 DV1

	; get 4 Y Pixels from the destination        (->mm3)
	mov       edi, [ebp+44]
	mov       bx,  [edi]
	shl       ebx, 16
	mov       bx,  [edi+edx]   ; ebx = DY2 DY1 DY4 DY3
	movd      mm3, ebx         ; mm3 = 00 00 00 00 DY2 DY1 DY4 DY3

	; ---------------------------------
	; BEGIN: mmx overlay of a 2x2 block
	; ---------------------------------
	punpcklbw mm0, mm5     ; mm0 = 00 SY2 00 SY1 00 SY4 00 SY3
	punpcklbw mm1, mm5     ; mm1 = 00 00  00 00  00 SU1 00 SV1
	punpcklbw mm2, mm5     ; mm2 = 00 A2  00 A1  00 A4  00 A3
	punpcklbw mm3, mm5     ; mm3 = 00 DY2 00 DY1 00 DY4 00 DY3
	punpcklbw mm4, mm5     ; mm4 = 00 00  00 00  00 DU1 00 DV1

	; calculate Y value
	psubw     mm0, mm3     ; mm0 = mm0-mm3	= (SY2-DY2) (SY1-DY1) (SY4-DY4) (SY3-DY3)
	pmullw    mm0, mm2     ; mm0 = A2*(SY2-DY2) | A1*(SY1-DY1) | A4*(SY4-DY4) | A3*(SY3-DY3)

        psllw     mm3, 8       ; mm3 = mm3*256
	paddw     mm0, mm3     ; mm0 = A2*(SY2-DY2) + DY2 | A1*(SY1-DY1) + DY1 | A4*(SY4-DY4) + DY4 | A3*(SY3-DY3) + DY3

	psrlw     mm0, 8       ; mm0 = mm0/256
	packuswb  mm0, mm0     ; mm0 = xx xx xx xx Y2 Y1 Y4 Y3

	; calculate U and V value
	psubw     mm1, mm4     ; mm1 = mm1-mm4	= 00 00 00 00 (SU1-DU1) (SV1-DV1)
	pmullw    mm1, mm2     ; mm1 = 00 00 00 00 A4*(SU1-DU1) | A3*(SV1-DV1)

        psllw     mm4, 8       ; mm4 = mm4*256
	paddw     mm1, mm4     ; mm1 = 00 00 00 00 A4*(SU1-DU1) + DU1 | A3*(SV1-DV1) + DV1

	psrlw     mm1, 8       ; mm1 = mm1/256
	packuswb  mm1, mm1     ; mm1 = xx xx xx xx 00 00 U V
	; ---------------------------------
	; END: mmx overlay of a 2x2 block
	; ---------------------------------

	; write V and U Pixel to destination
	movd      ebx,   mm1
	mov	  edi,   [ebp+52]
	mov       [edi], bl       ; V -> [[ebp+52]]
        shr       bx,    8
	mov       edi,   [ebp+48]
	mov       [edi], bl       ; U -> [[ebp+48]]

	; write 4 Y Pixels to destination
	movd      ebx,   mm0
	mov       edi,   [ebp+44]
	mov       [edi+edx], bx
	shr       ebx,   16
	mov       [edi], bx	  ; Y -> [[ebp+44]]

_next_pixels2:
	; Calculate new source plane pointers
	add      esi, 2		   ; go to next 4 Y Pixels (2 in a row and 2 in a column)
	inc      dword [ebp+8]	   ; go to next U Pixel
	inc      dword [ebp+12]    ; go to next V Pixel
	add      dword [ebp+16], 2 ; go to next 4 Alpha Pixels (2 in a row and 2 in a column)

	; Calculate new destination plane pointers
	add      dword [ebp+44], 2 ; go to next 4 Y Pixels
	inc      dword [ebp+48]    ; go to next U Pixel
	inc      dword [ebp+52]    ; go to next V Pixel

	dec	 ecx		   ; (width/2)--
	jnz      NEAR _yloop22	   ; jump, if height > 0

	; process new line...
	mov      ebx, [ebp+36]
        add      esi, ebx
	add      [ebp+16], ebx
	shr      ebx, 1
	add      [ebp+8], ebx
	add      [ebp+12], ebx

	add      esi, eax	   ; esi = esi+source width (skip 1 line, because we processed a 2x2 block)
	add      [ebp+16], eax     ; the same for the alpha plane

	mov      ebx, edx	   ; ebx = destination width
	shr      ebx, 1		   ; for u and v we need only half the pixels
	mov      ecx, [ebp+28]	   ; ecx = source width
	shr      ecx, 1
	sub      ebx, ecx	   ; ebx = stride to next line

	add      [ebp+48], ebx	   ; pointer to next line of the u-plane (dest)
	add      [ebp+52], ebx     ; pointer to next line of the v-plane (dest)

	shl      edx, 1		   ; edx = 2*width of destination (because we skip one line)
	sub      edx, [ebp+28]
	add      [ebp+44], edx     ; pointer to (over) next line of the y-plane (dest)

	dec      dword [ebp+32]    ; (height/2)--
	jnz 	 NEAR _yloop12

	emms
_yend2:
	pop      edi
	pop      esi
	mov      ebx,[ebp-4]
        leave                  ; mov esp,ebp / pop ebp
        ret

section .data

mask_1  dd 0x00000000, 0x01000000	; value: 0x0100000000000000
mask_2  dd 0xffffffff, 0x0000ffff	; value: 0x0000ffffffffffff
