/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "DVBReadNodeTVGuideManager.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM{
  DVBReadNodeTVGuideManager::DVBReadNodeTVGuideManager()
    :mutex()
  {
  }

  DVBReadNodeTVGuideManager::~DVBReadNodeTVGuideManager(){
    //delete all registered TVGuides
    for(list<pair<ITVGuide*, int> >::iterator i = iTVGuide_list.begin(); i != iTVGuide_list.end(); i++){
      delete i->first;
    } 
    iTVGuide_list.clear();
  }
    
  void DVBReadNodeTVGuideManager::addTVGuide(const ITVGuide *tvguide){
    MutexGuard mg(mutex);

    if(tvguide == NULL){
    }else{
      for(list<pair<ITVGuide*, int> >::iterator i = iTVGuide_list.begin(); i != iTVGuide_list.end(); i++){
        if(i->first->getParentObject() == tvguide->getParentObject()){
          //If this TVGuide is already registered, return
          return;
        }
      }

      ITVGuide* iTVGuide(dynamic_cast<ITVGuide*>(tvguide->copy()));
      int pluginPriority = 0;
      //Determine the Plugin-Priority of DVBEPG-Plugin
      ITVGuidePlugin *plugin = iTVGuide->getPluginInterface("IDVBEPGPlugin");
      if(plugin){
        pluginPriority = plugin->getPriority();
      }      
      iTVGuide_list.push_back(pair<ITVGuide*, int>(iTVGuide, pluginPriority));
    }
  }

  void DVBReadNodeTVGuideManager::removeTVGuide(const ITVGuide *tvguide){
    for(list<pair<ITVGuide*, int> >::iterator i = iTVGuide_list.begin(); i != iTVGuide_list.end(); i++){
      //determine the TVGuide to be removed
      if(i->first->getParentObject() == tvguide->getParentObject()){
        delete i->first;
        iTVGuide_list.erase(i);
        return;
      }
    }
  }

  Result DVBReadNodeTVGuideManager::insertEvent(const ProgramDescription &pd){
    MutexGuard mg(mutex);
    if(iTVGuide_list.empty()) return FAILURE;
    ProgramDescription p(pd);
    for(list<pair<ITVGuide*, int> >::iterator i = iTVGuide_list.begin(); i != iTVGuide_list.end(); i++){
      //add the corresponding Source-Priority
      p.setSourcePriority(i->second);
      //and send the ProgramDescription to the corresponding EPG
      i->first->insertEvent(p);
    }
    return SUCCESS;
  }

  bool DVBReadNodeTVGuideManager::hasTVGuides(){
    return (!iTVGuide_list.empty());
  }
}
#ifdef __CYGWIN__ /* explicitly instantiate static variables of Singleton template */
  NMM::SDVBReadNodeTVGuideManager::InstanceType* NMM::SDVBReadNodeTVGuideManager::_instance = 0;
  bool NMM::SDVBReadNodeTVGuideManager::destroyed = false;
#endif

#include "nmm/nmm_suffix.hpp"
