/*
 * remux.h: A streaming MPEG2 remultiplexer
 *
 * Copyright (C) 2000 Klaus Schmidinger
 * Copyright of modifications (C) 2006-2007 Motama GmbH
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 *
 * The author can be reached at kls@cadsoft.de
 *
 * The project's page is at http://www.cadsoft.de/people/kls/vdr
 *
 */

#ifndef __REMUX_H
#define __REMUX_H

#include <time.h> //XXX FIXME: DVB/ost/include/ost/dmx.h should include <time.h> itself!!!
#include <ost/dmx.h>

namespace VDR {

// Picture types:
#define NO_PICTURE 0
#define I_FRAME    1
#define P_FRAME    2
#define B_FRAME    3

//XXX -> remux.c???
// Start codes:
#define SC_PICTURE 0x00  // "picture header"
#define SC_SEQU    0xB3  // "sequence header"
#define SC_PHEAD   0xBA  // "pack header"
#define SC_SHEAD   0xBB  // "system header"
#define SC_AUDIO   0xC0
#define SC_VIDEO   0xE0

// The minimum amount of video data necessary to identify frames:
#define MINVIDEODATA (16*1024) // just a safe guess (max. size of any frame block, plus some safety)

typedef unsigned char uchar;
class cTS2PES;

class cRemux {
private:
  bool exitOnFailure;
  bool synced;
  int skipped;
  int vPid, aPid;
  cTS2PES *vTS2PES, *aTS2PES;
  uchar resultBuffer[MINVIDEODATA * 4];//XXX
  int resultCount;
  int resultDelivered;
  int GetPid(const uchar *Data);
  int GetPacketLength(const uchar *Data, int Count, int Offset);
  int ScanVideoPacket(const uchar *Data, int Count, int Offset, uchar &PictureType);
public:
  cRemux(int VPid, int APid, bool ExitOnFailure = false);
  ~cRemux();
  const uchar *Process(const uchar *Data, int &Count, int &Result, uchar *PictureType = NULL);
  };

}

#endif // __REMUX_H
