/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IChannelInfo.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IChannelInfo::used() {} 
  
  
  Result IChannelInfo::hasChannel(nmm_int32& c1, const string& c2)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TOutValue<nmm_int32 > c1_value(c1);
    NMM::TInValue<string > c2_value(c2, false);
    NMM::Event event("IChannelInfo::hasChannel", &c1_value, &c2_value);
    event.setReturnValue(&return_value);
    if(const_cast<IChannelInfo*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    c1 = c1_value.getRValue();
    return return_value.getRValue();
  }
  
  TVChannelList IChannelInfo::getChannelList()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<TVChannelList > return_value;
    NMM::Event event("IChannelInfo::getChannelList");
    event.setReturnValue(&return_value);
    if(const_cast<IChannelInfo*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IChannelInfo::setChannelConfigFile(const string& filename) /* limited(INode::CONSTRUCTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<string > filename_value(filename, false);
    NMM::Event event("IChannelInfo::setChannelConfigFile", &filename_value);
    if(const_cast<IChannelInfo*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IChannelInfo::IChannelInfo_type = "IChannelInfo";
  NMM::TConstructor<IChannelInfo > __IChannelInfo_prototype("IChannelInfo","NMM");


  const char* IChannelInfo::hasChannel_event = "IChannelInfo::hasChannel";
  const char* IChannelInfo::getChannelList_event = "IChannelInfo::getChannelList";
  const char* IChannelInfo::setChannelConfigFile_event = "IChannelInfo::setChannelConfigFile";


}


#include "nmm/nmm_suffix.hpp"

