/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IChannelInfo_HPP
#define NMM_IChannelInfo_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/plugins/video/tv/interfaces/TVChannelList.hpp"
#include "nmm/comm/serialize/StdValue.hpp"
#include "nmm/multimedia/INode.hpp"

#include "visibility.hpp"

namespace NMM
{
  /**
   *Interface for requesting channel-information from the Node. It is used to build the global channel-list.
   */

  class DLLEXPORT IChannelInfo : virtual public NMM::Interface { 
  public:
    IChannelInfo() : NMM::Interface(IChannelInfo::IChannelInfo_type, "NMM") { };

    IChannelInfo(NMM::NMMObject& parent) : NMM::Interface (parent, IChannelInfo::IChannelInfo_type, "NMM") { };

    virtual ~IChannelInfo() {};

    static const char* IChannelInfo_type;

    virtual  string getInterfaceType() const { return IChannelInfo_type; }

    NMM::Interface* copy() const { return new IChannelInfo(*this); }
    static void used(); 
    static const char* hasChannel_event;
    static const char* getChannelList_event;
    static const char* setChannelConfigFile_event;
    /**
     *Tests, if the TVCard has the channel with the given name. It writes the corrensponding channel-number to c1
     */

    virtual Result hasChannel(nmm_int32& c1, const string& c2)
      throw(const NMM::SerializedException&);
    /**
     *Returns a complete List of all available channels
     *@return TVCannelList the channel-list
     */

    virtual TVChannelList getChannelList()
      throw(const NMM::SerializedException&);
    /**
     *Tell the node to read the channel-configuration from the given file. 
     *@param filename complete path to the channel-configuration-file
     */

    virtual void setChannelConfigFile(const string& filename) /* limited(INode::CONSTRUCTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);


  };

  typedef auto_ptr<IChannelInfo> IChannelInfo_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
