/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IChannelInfoImpl_HPP
#define NMM_IChannelInfoImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IChannelInfo.hpp"

#include "nmm/multimedia/INodeImpl.hpp"

#include "visibility.hpp"

namespace NMM
{
  /**
   *Interface for requesting channel-information from the Node. It is used to build the global channel-list.
   */

  class DLLEXPORT IChannelInfoImpl  : public virtual NMM::NMMObject { 
  public:
    IChannelInfoImpl() { addInterface(new IChannelInfo(*this)); registerEvents(); };

    virtual ~IChannelInfoImpl() {};

  protected:
    /**
     *Tests, if the TVCard has the channel with the given name. It writes the corrensponding channel-number to c1
     */

    virtual Result hasChannel(nmm_int32& c1, const string& c2) = 0;
    /**
     *Returns a complete List of all available channels
     *@return TVCannelList the channel-list
     */

    virtual TVChannelList getChannelList() = 0;
    /**
     *Tell the node to read the channel-configuration from the given file. 
     *@param filename complete path to the channel-configuration-file
     */

    virtual void setChannelConfigFile(const string& filename) /* limited(INode::CONSTRUCTED) */ = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
