/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IEPGImpl_HPP
#define NMM_IEPGImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IEPG.hpp"

#include "nmm/multimedia/INodeImpl.hpp"

#include "visibility.hpp"

namespace NMM
{
  /**
   * Interface for an Electronic Programming Guide
   * to provide additional information to the program shown
   */

  class DLLEXPORT IEPGImpl  : public virtual NMM::NMMObject { 
  public:
    IEPGImpl() { addInterface(new IEPG(*this)); registerEvents(); };

    virtual ~IEPGImpl() {};

  protected:
    /**
     * returns a string wich contains the starttime, 
     * endtime and title of the current show
     * Format: hh:mm - hh:mm : title
     */

    virtual string getCurrentShow() /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /**
     * returns a string wich contains the starttime, 
     * endtime and title of the following show
     * Format: hh:mm - hh:mm : title
     */

    virtual string getFollowingShow() /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /**
     * returns a string containing a detailed description
     * of the curretn show
     */

    virtual string getDetailedDescription() /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /**
     * sets the Directory, where the XML-file with EPG-information is stored
     */

    virtual void setEPGDirectory(const string& dir) = 0;
    /**
     * restores the channel before starting the scanning-function
     */

    virtual void getActivity() /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /**
     * starts the scanning-function of VDR to collect EPG-data
     */

    virtual void scan() /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
