/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ITV.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/serialize/StdValue.hpp"

namespace NMM
{
  void ITV::used() {} 
  
  
  nmm_uint32 ITV::getAudioCount() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("ITV::getAudioCount");
    event.setReturnValue(&return_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  TVAudio ITV::getAudio(const nmm_uint32& index) const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<TVAudio > return_value;
    NMM::TInValue<nmm_uint32 > index_value(index, false);
    NMM::Event event("ITV::getAudio", &index_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<TVAudio >  ITV::getAudioList() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<list<TVAudio >  > return_value;
    NMM::Event event("ITV::getAudioList");
    event.setReturnValue(&return_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 ITV::getCurrentAudio() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("ITV::getCurrentAudio");
    event.setReturnValue(&return_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITV::setCurrentAudio(const nmm_uint32& index) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_uint32 > index_value(index, false);
    NMM::Event event("ITV::setCurrentAudio", &index_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_uint32 ITV::getInputCount() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("ITV::getInputCount");
    event.setReturnValue(&return_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  TVInput ITV::getInput(const nmm_uint32& num) const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<TVInput > return_value;
    NMM::TInValue<nmm_uint32 > num_value(num, false);
    NMM::Event event("ITV::getInput", &num_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<TVInput >  ITV::getInputList() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<list<TVInput >  > return_value;
    NMM::Event event("ITV::getInputList");
    event.setReturnValue(&return_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 ITV::getCurrentInput() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("ITV::getCurrentInput");
    event.setReturnValue(&return_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITV::setCurrentInput(const nmm_uint32& num) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_uint32 > num_value(num, false);
    NMM::Event event("ITV::setCurrentInput", &num_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_uint32 ITV::getStandardByName(const string& name) const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::TInValue<string > name_value(name, false);
    NMM::Event event("ITV::getStandardByName", &name_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  TVStandard ITV::getStandard(const nmm_uint32& num) const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<TVStandard > return_value;
    NMM::TInValue<nmm_uint32 > num_value(num, false);
    NMM::Event event("ITV::getStandard", &num_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<TVStandard >  ITV::getStandardList() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<list<TVStandard >  > return_value;
    NMM::Event event("ITV::getStandardList");
    event.setReturnValue(&return_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 ITV::getCurrentStandard() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("ITV::getCurrentStandard");
    event.setReturnValue(&return_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITV::setCurrentStandard(const nmm_uint32& standard) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_uint32 > standard_value(standard, false);
    NMM::Event event("ITV::setCurrentStandard", &standard_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITV::setStandardByName(const string& name) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<string > name_value(name, false);
    NMM::Event event("ITV::setStandardByName", &name_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_uint32 ITV::getTunerCount() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("ITV::getTunerCount");
    event.setReturnValue(&return_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  TVTuner ITV::getTuner(const nmm_uint32& index) const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<TVTuner > return_value;
    NMM::TInValue<nmm_uint32 > index_value(index, false);
    NMM::Event event("ITV::getTuner", &index_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<TVTuner >  ITV::getTunerList() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<list<TVTuner >  > return_value;
    NMM::Event event("ITV::getTunerList");
    event.setReturnValue(&return_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_float ITV::getFrequency(const nmm_uint32& tuner) const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<nmm_float > return_value;
    NMM::TInValue<nmm_uint32 > tuner_value(tuner, false);
    NMM::Event event("ITV::getFrequency", &tuner_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITV::setFrequency(const nmm_uint32& tuner, const nmm_float& frequency) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_uint32 > tuner_value(tuner, false);
    NMM::TInValue<nmm_float > frequency_value(frequency, false);
    NMM::Event event("ITV::setFrequency", &tuner_value, &frequency_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_uint32 ITV::getBrightness() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("ITV::getBrightness");
    event.setReturnValue(&return_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 ITV::getContrast() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("ITV::getContrast");
    event.setReturnValue(&return_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 ITV::getSaturation() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("ITV::getSaturation");
    event.setReturnValue(&return_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_int32 ITV::getHue() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<nmm_int32 > return_value;
    NMM::Event event("ITV::getHue");
    event.setReturnValue(&return_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 ITV::getVolume() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("ITV::getVolume");
    event.setReturnValue(&return_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool ITV::getMute() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ITV::getMute");
    event.setReturnValue(&return_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITV::setBrightness(const nmm_uint32& brightness) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_uint32 > brightness_value(brightness, false);
    NMM::Event event("ITV::setBrightness", &brightness_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITV::setContrast(const nmm_uint32& contrast) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_uint32 > contrast_value(contrast, false);
    NMM::Event event("ITV::setContrast", &contrast_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITV::setSaturation(const nmm_uint32& saturation) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_uint32 > saturation_value(saturation, false);
    NMM::Event event("ITV::setSaturation", &saturation_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITV::setHue(const nmm_int32& hue) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_int32 > hue_value(hue, false);
    NMM::Event event("ITV::setHue", &hue_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITV::setVolume(const nmm_uint32& volume) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_uint32 > volume_value(volume, false);
    NMM::Event event("ITV::setVolume", &volume_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITV::setMute(const nmm_bool& mute) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_bool > mute_value(mute, false);
    NMM::Event event("ITV::setMute", &mute_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  list<string >  ITV::getFrequencyTableList() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<list<string >  > return_value;
    NMM::Event event("ITV::getFrequencyTableList");
    event.setReturnValue(&return_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string ITV::getFrequencyTable() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("ITV::getFrequencyTable");
    event.setReturnValue(&return_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITV::setFrequencyTable(const string& table) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<string > table_value(table, false);
    NMM::Event event("ITV::setFrequencyTable", &table_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITV::setInputOutput(const nmm_int32& input, const nmm_int32& output) /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_int32 > input_value(input, false);
    NMM::TInValue<nmm_int32 > output_value(output, false);
    NMM::Event event("ITV::setInputOutput", &input_value, &output_value);
    if(const_cast<ITV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* ITV::ITV_type = "ITV";
  NMM::TConstructor<ITV > __ITV_prototype("ITV","NMM");


  const char* ITV::getAudioCount_event = "ITV::getAudioCount";
  const char* ITV::getAudio_event = "ITV::getAudio";
  const char* ITV::getAudioList_event = "ITV::getAudioList";
  const char* ITV::getCurrentAudio_event = "ITV::getCurrentAudio";
  const char* ITV::setCurrentAudio_event = "ITV::setCurrentAudio";
  const char* ITV::getInputCount_event = "ITV::getInputCount";
  const char* ITV::getInput_event = "ITV::getInput";
  const char* ITV::getInputList_event = "ITV::getInputList";
  const char* ITV::getCurrentInput_event = "ITV::getCurrentInput";
  const char* ITV::setCurrentInput_event = "ITV::setCurrentInput";
  const char* ITV::getStandardByName_event = "ITV::getStandardByName";
  const char* ITV::getStandard_event = "ITV::getStandard";
  const char* ITV::getStandardList_event = "ITV::getStandardList";
  const char* ITV::getCurrentStandard_event = "ITV::getCurrentStandard";
  const char* ITV::setCurrentStandard_event = "ITV::setCurrentStandard";
  const char* ITV::setStandardByName_event = "ITV::setStandardByName";
  const char* ITV::getTunerCount_event = "ITV::getTunerCount";
  const char* ITV::getTuner_event = "ITV::getTuner";
  const char* ITV::getTunerList_event = "ITV::getTunerList";
  const char* ITV::getFrequency_event = "ITV::getFrequency";
  const char* ITV::setFrequency_event = "ITV::setFrequency";
  const char* ITV::getBrightness_event = "ITV::getBrightness";
  const char* ITV::getContrast_event = "ITV::getContrast";
  const char* ITV::getSaturation_event = "ITV::getSaturation";
  const char* ITV::getHue_event = "ITV::getHue";
  const char* ITV::getVolume_event = "ITV::getVolume";
  const char* ITV::getMute_event = "ITV::getMute";
  const char* ITV::setBrightness_event = "ITV::setBrightness";
  const char* ITV::setContrast_event = "ITV::setContrast";
  const char* ITV::setSaturation_event = "ITV::setSaturation";
  const char* ITV::setHue_event = "ITV::setHue";
  const char* ITV::setVolume_event = "ITV::setVolume";
  const char* ITV::setMute_event = "ITV::setMute";
  const char* ITV::getFrequencyTableList_event = "ITV::getFrequencyTableList";
  const char* ITV::getFrequencyTable_event = "ITV::getFrequencyTable";
  const char* ITV::setFrequencyTable_event = "ITV::setFrequencyTable";
  const char* ITV::setInputOutput_event = "ITV::setInputOutput";


}


#include "nmm/nmm_suffix.hpp"

