/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ITVCard.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ITVCard::used() {} 
  
  
  void ITVCard::setResolution(const nmm_int32& x, const nmm_int32& y) /* limited(INode::CONSTRUCTED) */ /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_int32 > x_value(x, false);
    NMM::TInValue<nmm_int32 > y_value(y, false);
    NMM::Event event("ITVCard::setResolution", &x_value, &y_value);
    if(const_cast<ITVCard*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITVCard::getResolution(nmm_int32& x, nmm_int32& y)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_int32 > x_value(x);
    NMM::TOutValue<nmm_int32 > y_value(y);
    NMM::Event event("ITVCard::getResolution", &x_value, &y_value);
    if(const_cast<ITVCard*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    x = x_value.getRValue();
    y = y_value.getRValue();
  }
  
  Result ITVCard::setChannel(const nmm_int32& channel) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_int32 > channel_value(channel, false);
    NMM::Event event("ITVCard::setChannel", &channel_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITVCard*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVCard::getChannel(nmm_int32& c1, string& c2)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TOutValue<nmm_int32 > c1_value(c1);
    NMM::TOutValue<string > c2_value(c2);
    NMM::Event event("ITVCard::getChannel", &c1_value, &c2_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITVCard*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    c1 = c1_value.getRValue();
    c2 = c2_value.getRValue();
    return return_value.getRValue();
  }

  const char* ITVCard::ITVCard_type = "ITVCard";
  NMM::TConstructor<ITVCard > __ITVCard_prototype("ITVCard","NMM");


  const char* ITVCard::setResolution_event = "ITVCard::setResolution";
  const char* ITVCard::getResolution_event = "ITVCard::getResolution";
  const char* ITVCard::setChannel_event = "ITVCard::setChannel";
  const char* ITVCard::getChannel_event = "ITVCard::getChannel";


}


#include "nmm/nmm_suffix.hpp"

