/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ITVCard_HPP
#define NMM_ITVCard_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "nmm/multimedia/INode.hpp"
#include "nmm/multimedia/IDevice.hpp"

#include "visibility.hpp"

namespace NMM
{

  class DLLEXPORT ITVCard : virtual public IDevice { 
  public:
    ITVCard() : NMM::Interface(ITVCard::ITVCard_type, "NMM") { };

    ITVCard(NMM::NMMObject& parent) : NMM::Interface (parent, ITVCard::ITVCard_type, "NMM") { };

    virtual ~ITVCard() {};

    static const char* ITVCard_type;

    virtual  string getInterfaceType() const { return ITVCard_type; }

    NMM::Interface* copy() const { return new ITVCard(*this); }
    static void used(); 
    static const char* setResolution_event;
    static const char* getResolution_event;
    static const char* setChannel_event;
    static const char* getChannel_event;

    virtual void setResolution(const nmm_int32& x, const nmm_int32& y) /* limited(INode::CONSTRUCTED) */ /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);

    virtual void getResolution(nmm_int32& x, nmm_int32& y)
      throw(const NMM::SerializedException&);

    virtual Result setChannel(const nmm_int32& channel) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);

    virtual Result getChannel(nmm_int32& c1, string& c2)
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ITVCard> ITVCard_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
