/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ITVImpl.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/messaging/TEDObject.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/comm/serialize/StdValue.hpp"

namespace NMM
{
  
  void ITVImpl::registerEvents()
  {
  
    getEventDispatcher().registerEvent(ITV::getAudioCount_event, new NMM::TEDObject0<ITVImpl, nmm_uint32(ITVImpl:: * )() const, nmm_uint32 >(this, &ITVImpl::getAudioCount));
    getEventDispatcher().addLimited(ITV::getAudioCount_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::getAudioCount_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::getAudioCount_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::getAudioCount_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::getAudio_event, new NMM::TEDObject1<ITVImpl, nmm_uint32, TVAudio(ITVImpl:: * )(const nmm_uint32&) const, TVAudio >(this, &ITVImpl::getAudio));
    getEventDispatcher().addLimited(ITV::getAudio_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::getAudio_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::getAudio_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::getAudio_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::getAudioList_event, new NMM::TEDObject0<ITVImpl, list<TVAudio > (ITVImpl:: * )() const, list<TVAudio >  >(this, &ITVImpl::getAudioList));
    getEventDispatcher().addLimited(ITV::getAudioList_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::getAudioList_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::getAudioList_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::getAudioList_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::getCurrentAudio_event, new NMM::TEDObject0<ITVImpl, nmm_uint32(ITVImpl:: * )() const, nmm_uint32 >(this, &ITVImpl::getCurrentAudio));
    getEventDispatcher().addLimited(ITV::getCurrentAudio_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::getCurrentAudio_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::getCurrentAudio_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::getCurrentAudio_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::setCurrentAudio_event, new NMM::TEDObject1<ITVImpl, nmm_uint32, void(ITVImpl:: * )(const nmm_uint32&), void >(this, &ITVImpl::setCurrentAudio));
    getEventDispatcher().addLimited(ITV::setCurrentAudio_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::setCurrentAudio_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::setCurrentAudio_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::setCurrentAudio_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::getInputCount_event, new NMM::TEDObject0<ITVImpl, nmm_uint32(ITVImpl:: * )() const, nmm_uint32 >(this, &ITVImpl::getInputCount));
    getEventDispatcher().addLimited(ITV::getInputCount_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::getInputCount_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::getInputCount_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::getInputCount_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::getInput_event, new NMM::TEDObject1<ITVImpl, nmm_uint32, TVInput(ITVImpl:: * )(const nmm_uint32&) const, TVInput >(this, &ITVImpl::getInput));
    getEventDispatcher().addLimited(ITV::getInput_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::getInput_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::getInput_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::getInput_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::getInputList_event, new NMM::TEDObject0<ITVImpl, list<TVInput > (ITVImpl:: * )() const, list<TVInput >  >(this, &ITVImpl::getInputList));
    getEventDispatcher().addLimited(ITV::getInputList_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::getInputList_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::getInputList_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::getInputList_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::getCurrentInput_event, new NMM::TEDObject0<ITVImpl, nmm_uint32(ITVImpl:: * )() const, nmm_uint32 >(this, &ITVImpl::getCurrentInput));
    getEventDispatcher().addLimited(ITV::getCurrentInput_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::getCurrentInput_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::getCurrentInput_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::getCurrentInput_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::setCurrentInput_event, new NMM::TEDObject1<ITVImpl, nmm_uint32, void(ITVImpl:: * )(const nmm_uint32&), void >(this, &ITVImpl::setCurrentInput));
    getEventDispatcher().addLimited(ITV::setCurrentInput_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::setCurrentInput_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::setCurrentInput_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::setCurrentInput_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::getStandardByName_event, new NMM::TEDObject1<ITVImpl, string, nmm_uint32(ITVImpl:: * )(const string&) const, nmm_uint32 >(this, &ITVImpl::getStandardByName));
    getEventDispatcher().addLimited(ITV::getStandardByName_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::getStandardByName_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::getStandardByName_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::getStandardByName_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::getStandard_event, new NMM::TEDObject1<ITVImpl, nmm_uint32, TVStandard(ITVImpl:: * )(const nmm_uint32&) const, TVStandard >(this, &ITVImpl::getStandard));
    getEventDispatcher().addLimited(ITV::getStandard_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::getStandard_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::getStandard_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::getStandard_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::getStandardList_event, new NMM::TEDObject0<ITVImpl, list<TVStandard > (ITVImpl:: * )() const, list<TVStandard >  >(this, &ITVImpl::getStandardList));
    getEventDispatcher().addLimited(ITV::getStandardList_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::getStandardList_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::getStandardList_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::getStandardList_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::getCurrentStandard_event, new NMM::TEDObject0<ITVImpl, nmm_uint32(ITVImpl:: * )() const, nmm_uint32 >(this, &ITVImpl::getCurrentStandard));
    getEventDispatcher().addLimited(ITV::getCurrentStandard_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::getCurrentStandard_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::getCurrentStandard_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::getCurrentStandard_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::setCurrentStandard_event, new NMM::TEDObject1<ITVImpl, nmm_uint32, void(ITVImpl:: * )(const nmm_uint32&), void >(this, &ITVImpl::setCurrentStandard));
    getEventDispatcher().addLimited(ITV::setCurrentStandard_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::setCurrentStandard_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::setCurrentStandard_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::setCurrentStandard_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::setStandardByName_event, new NMM::TEDObject1<ITVImpl, string, void(ITVImpl:: * )(const string&), void >(this, &ITVImpl::setStandardByName));
    getEventDispatcher().addLimited(ITV::setStandardByName_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::setStandardByName_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::setStandardByName_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::setStandardByName_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::getTunerCount_event, new NMM::TEDObject0<ITVImpl, nmm_uint32(ITVImpl:: * )() const, nmm_uint32 >(this, &ITVImpl::getTunerCount));
    getEventDispatcher().addLimited(ITV::getTunerCount_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::getTunerCount_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::getTunerCount_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::getTunerCount_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::getTuner_event, new NMM::TEDObject1<ITVImpl, nmm_uint32, TVTuner(ITVImpl:: * )(const nmm_uint32&) const, TVTuner >(this, &ITVImpl::getTuner));
    getEventDispatcher().addLimited(ITV::getTuner_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::getTuner_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::getTuner_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::getTuner_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::getTunerList_event, new NMM::TEDObject0<ITVImpl, list<TVTuner > (ITVImpl:: * )() const, list<TVTuner >  >(this, &ITVImpl::getTunerList));
    getEventDispatcher().addLimited(ITV::getTunerList_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::getTunerList_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::getTunerList_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::getTunerList_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::getFrequency_event, new NMM::TEDObject1<ITVImpl, nmm_uint32, nmm_float(ITVImpl:: * )(const nmm_uint32&) const, nmm_float >(this, &ITVImpl::getFrequency));
    getEventDispatcher().addLimited(ITV::getFrequency_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::getFrequency_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::getFrequency_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::getFrequency_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::setFrequency_event, new NMM::TEDObject2<ITVImpl, nmm_uint32, nmm_float, void(ITVImpl:: * )(const nmm_uint32&, const nmm_float&), void >(this, &ITVImpl::setFrequency));
    getEventDispatcher().addLimited(ITV::setFrequency_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::setFrequency_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::setFrequency_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::setFrequency_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::getBrightness_event, new NMM::TEDObject0<ITVImpl, nmm_uint32(ITVImpl:: * )() const, nmm_uint32 >(this, &ITVImpl::getBrightness));
    getEventDispatcher().addLimited(ITV::getBrightness_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::getBrightness_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::getBrightness_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::getBrightness_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::getContrast_event, new NMM::TEDObject0<ITVImpl, nmm_uint32(ITVImpl:: * )() const, nmm_uint32 >(this, &ITVImpl::getContrast));
    getEventDispatcher().addLimited(ITV::getContrast_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::getContrast_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::getContrast_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::getContrast_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::getSaturation_event, new NMM::TEDObject0<ITVImpl, nmm_uint32(ITVImpl:: * )() const, nmm_uint32 >(this, &ITVImpl::getSaturation));
    getEventDispatcher().addLimited(ITV::getSaturation_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::getSaturation_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::getSaturation_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::getSaturation_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::getHue_event, new NMM::TEDObject0<ITVImpl, nmm_int32(ITVImpl:: * )() const, nmm_int32 >(this, &ITVImpl::getHue));
    getEventDispatcher().addLimited(ITV::getHue_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::getHue_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::getHue_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::getHue_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::getVolume_event, new NMM::TEDObject0<ITVImpl, nmm_uint32(ITVImpl:: * )() const, nmm_uint32 >(this, &ITVImpl::getVolume));
    getEventDispatcher().addLimited(ITV::getVolume_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::getVolume_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::getVolume_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::getVolume_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::getMute_event, new NMM::TEDObject0<ITVImpl, nmm_bool(ITVImpl:: * )() const, nmm_bool >(this, &ITVImpl::getMute));
    getEventDispatcher().addLimited(ITV::getMute_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::getMute_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::getMute_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::getMute_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::setBrightness_event, new NMM::TEDObject1<ITVImpl, nmm_uint32, void(ITVImpl:: * )(const nmm_uint32&), void >(this, &ITVImpl::setBrightness));
    getEventDispatcher().addLimited(ITV::setBrightness_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::setBrightness_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::setBrightness_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::setBrightness_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::setContrast_event, new NMM::TEDObject1<ITVImpl, nmm_uint32, void(ITVImpl:: * )(const nmm_uint32&), void >(this, &ITVImpl::setContrast));
    getEventDispatcher().addLimited(ITV::setContrast_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::setContrast_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::setContrast_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::setContrast_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::setSaturation_event, new NMM::TEDObject1<ITVImpl, nmm_uint32, void(ITVImpl:: * )(const nmm_uint32&), void >(this, &ITVImpl::setSaturation));
    getEventDispatcher().addLimited(ITV::setSaturation_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::setSaturation_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::setSaturation_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::setSaturation_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::setHue_event, new NMM::TEDObject1<ITVImpl, nmm_int32, void(ITVImpl:: * )(const nmm_int32&), void >(this, &ITVImpl::setHue));
    getEventDispatcher().addLimited(ITV::setHue_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::setHue_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::setHue_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::setHue_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::setVolume_event, new NMM::TEDObject1<ITVImpl, nmm_uint32, void(ITVImpl:: * )(const nmm_uint32&), void >(this, &ITVImpl::setVolume));
    getEventDispatcher().addLimited(ITV::setVolume_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::setVolume_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::setVolume_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::setVolume_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::setMute_event, new NMM::TEDObject1<ITVImpl, nmm_bool, void(ITVImpl:: * )(const nmm_bool&), void >(this, &ITVImpl::setMute));
    getEventDispatcher().addLimited(ITV::setMute_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::setMute_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::setMute_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::setMute_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::getFrequencyTableList_event, new NMM::TEDObject0<ITVImpl, list<string > (ITVImpl:: * )() const, list<string >  >(this, &ITVImpl::getFrequencyTableList));
    getEventDispatcher().addLimited(ITV::getFrequencyTableList_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::getFrequencyTableList_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::getFrequencyTableList_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::getFrequencyTableList_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::getFrequencyTable_event, new NMM::TEDObject0<ITVImpl, string(ITVImpl:: * )() const, string >(this, &ITVImpl::getFrequencyTable));
    getEventDispatcher().addLimited(ITV::getFrequencyTable_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::getFrequencyTable_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::getFrequencyTable_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::getFrequencyTable_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::setFrequencyTable_event, new NMM::TEDObject1<ITVImpl, string, void(ITVImpl:: * )(const string&), void >(this, &ITVImpl::setFrequencyTable));
    getEventDispatcher().addLimited(ITV::setFrequencyTable_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(ITV::setFrequencyTable_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::setFrequencyTable_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::setFrequencyTable_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(ITV::setInputOutput_event, new NMM::TEDObject2<ITVImpl, nmm_int32, nmm_int32, void(ITVImpl:: * )(const nmm_int32&, const nmm_int32&), void >(this, &ITVImpl::setInputOutput));
    getEventDispatcher().addLimited(ITV::setInputOutput_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(ITV::setInputOutput_event, INode::STARTED);
    getEventDispatcher().addLimited(ITV::setInputOutput_event, INode::OUTPUT_INITIALIZED);

  }

}


#include "nmm/nmm_suffix.hpp"

