/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "TVChannel.hpp"

#include "nmm/comm/serialize/StdValue.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"
#include "nmm/utils/xml/XMLNode.hpp"
#include "nmm/utils/xml/XMLTree.hpp"
#include "nmm/utils/directory_utils.hpp"

#include <sstream>

#include "nmm/nmm_prefix.hpp"

namespace NMM{

  /* ****************************************
   *           TVChannel         *
   **************************************** */  
  TVChannel::TVChannel()
    :NMM::SerializedValue("TVChannel","NMM"),
          channelName(""),
          channelNumber(0)
  {
    registerAttributes();
  }

  TVChannel::TVChannel(string name)
    :NMM::SerializedValue("TVChannel","NMM"),
          channelName(name),
          channelNumber(0)
  {
    registerAttributes();
  }

  TVChannel::TVChannel(const TVChannel& t)
    :SerializedValue("TVChannelDecription","NMM"),
          channelName(""),
          channelNumber(0)
  {
    registerAttributes();
    *this=t;
  }

  TVChannel::~TVChannel(){}

  void TVChannel::registerAttributes(){
    registerAttribute(&channelName);
    registerAttribute(&channelNumber);
    registerAttribute(&nameAliases);
    registerAttribute(&cards);
  }

  bool TVChannel::operator<(const TVChannel &cd) const{
    if(channelNumber.getRValue() < cd.getChannelNumber()){
      return true;
    }else{
      return false;
    }
  }

  void TVChannel::addCard(string URL, unsigned int channelNumber){
    cards.getRValue().push_back(pair<string,int>(URL, channelNumber));
  }

  TVChannel::cardList_t TVChannel::getCards(){ 
    return cards.getRValue();
  }
  
  list<string> TVChannel::getAliases(){
    return nameAliases.getRValue();
  }

  void TVChannel::addAlias(string a){
    nameAliases.getRValue().push_back(a);
    nameAliases.getRValue().sort(); 
    nameAliases.getRValue().unique();
  }

  OValueStream& operator<<(OValueStream& os, const list<TVChannel>& tvcd){
    return writeList<TVChannel>(os, tvcd, "TVChannel");
  }
  IValueStream& operator>>(IValueStream& is, list<TVChannel>& tvcd){
    return readList<TVChannel>(is, tvcd, "TVChannel");
  }

  XMLNode TVChannel::toXMLNode(){

    XMLNode channel("tvchannel");

    channel.add_property("name", channelName.getRValue());
    for(list<pair<string, int> >::iterator i = cards.getRValue().begin(); i != cards.getRValue().end(); i++){
      XMLNode *n = channel.add_child("card");
      
      ostringstream s;
      s << i->second;

      n->add_property("number", s.str());
      n->add_content(i->first);
    }

    for(list<string>::iterator i = nameAliases.getRValue().begin(); i != nameAliases.getRValue().end(); i++){
      XMLNode *n = channel.add_child("alias");
      n->add_content(*i);
    }
  
    return channel;
  }


  void TVChannel::readAliases(string filename){
    //Open Channel-Alias-file
    if(filename.empty()){
      filename = getNMMResourceDir() + "/tvguide/channelAliases.xml";
    }
    XMLTree *tree = new XMLTree(filename);

    //BEGIN> read Aliases
    if(tree){
      XMLNode *root = tree->root();
      if(root){
        XMLNodeList nodeList = root->children("channel");
        for(XMLNodeIterator i = nodeList.begin(); i != nodeList.end(); i++){
          list<string> aliases;
          XMLNodeList childList = (*i)->children();
          bool matchFound = false;
          for(XMLNodeIterator j = childList.begin(); j != childList.end(); j++){
            //cerr << "TVChannelchecking: " << (*j)->getData() << " " << channelName.getRValue() << endl;
            if((*j)->getData() == channelName.getRValue()){
              //cerr << "TVChannel matchFound" << endl;
              matchFound = true;
            }else{
              aliases.push_back((*j)->getData());
            }
          }// end for(childList)
          if(matchFound){
            setChannelAliases(aliases);
            break;
          }
          aliases.clear();
        }//end for(nodeList)
      }//end if(root)
      delete tree;
      tree = NULL;
    }//end if(tree)          
    //<END
  }

  TConstructor<TVChannel> nmm_tvchanneldescription("TVChannel", "NMM");
  TValueConstructor<list<TVChannel> > nmm_tvchanneldescription_list("list_TVChannel","NMM");

}

#include "nmm/nmm_suffix.hpp"
