/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#ifndef TVCHANNEL_HPP
#define TVCHANNEL_HPP TVCHANNEL_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/serialize/SerializedValue.hpp"
#include "nmm/comm/serialize/TValue.hpp"

namespace NMM{

  class XMLNode;

  /**
   * Class for storing informations about a TV-Channel provided by a TV-Card
   */

  class TVChannel : public SerializedValue{

  public :
    //typedef map<string, int> cardList_t;
    typedef list<pair<string, int> >cardList_t;
  protected:
    NMM::TValue<string> channelName;
    NMM::TValue<nmm_uint32> channelNumber;
    NMM::TValue<list<string> > nameAliases;
    NMM::TValue<cardList_t> cards;

  public:
    TVChannel();
    TVChannel(string name);
    TVChannel(const TVChannel& t);

    ~TVChannel();

    void setChannelName(string name){channelName = name;}
    string getChannelName() const {return channelName.getRValue();}

    void setChannelNumber(unsigned int number){channelNumber = number;}
    unsigned int getChannelNumber() const {return channelNumber.getRValue();}

    void setChannelAliases(list<string> aliases){nameAliases = aliases;}
    list<string> getChannelAliases()const {return nameAliases.getRValue();}

    bool operator<(const TVChannel &cd) const;

    void addCard(string URL, unsigned int channelNumber);
    cardList_t getCards();
    
    list<string> getAliases();
    void addAlias(string a);

    XMLNode toXMLNode();
    void readAliases(string filename = "");

  private:
    void registerAttributes();
  };

  OValueStream& operator<<(OValueStream& os, const list<TVChannel>& tvcd);
  IValueStream& operator>>(IValueStream& is, list<TVChannel>& tvcd);
}
#include "nmm/nmm_suffix.hpp"

#endif
