/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "TVStructs.hpp"
#include "nmm/comm/serialize/StdValue.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  /*============================== TVAudio ==============================*/
  TVAudio::TVAudio()
    : SerializedValue("TVAudio", "NMM"),
      __index(0),
      __name(),
      __caps(0),
      __mode(0)
  {
    registerAttributes();
  }

  TVAudio::TVAudio(const TVAudio& other)
    : SerializedValue("TVAudio", "NMM"),
      __index(),
      __name(),
      __caps(0),
      __mode(0)
  {
    assign(other);
    registerAttributes();
  }
  
  TVAudio::TVAudio(unsigned int index, const string& name, unsigned int caps, unsigned int mode)
    : SerializedValue("TVAudio", "NMM"),
      __index(index),
      __name(name),
      __caps(caps),
      __mode(mode)
  {
    registerAttributes();
  }
  
  const TVAudio& TVAudio::operator=(const TVAudio& other)  {
    SerializedValue::operator=(other);
    assign(other);
    return *this;
  }

  void TVAudio::assign(const TVAudio& other) {
    /* Begin> assign the attributes */
    __index     = other.getIndex();
    __name      = other.getName();
    __caps      = other.getCapabilities();
    __mode      = other.getMode();
    /* End < */
  }
  
  void TVAudio::registerAttributes() {
    /* Begin> register the attributes by the baseclass */
    insertArgument(&__index);
    insertArgument(&__name);
    insertArgument(&__caps);
    insertArgument(&__mode);
    /* End< */
  }
  
  TConstructor<TVAudio> __nmm__tv_audio("TVAudio", "NMM");
  /*============================== TVAudio ==============================*/
  
  
  /*============================== TVInput ==============================*/
  TVInput::TVInput()
    : SerializedValue("TVInput", "NMM"),
      __num(0),
      __name(),
      __type(TYPE_UNKNOWN),
      __audio_set(0),
      __tuner(0),
      __standard(0),
      __status(0)
  {
    registerAttributes();
  }
  
  TVInput::TVInput(const TVInput& other)
    : SerializedValue(other)
  {
    assign(other);
    registerAttributes();
  }
  
  TVInput::TVInput(unsigned int num, const string& name, unsigned int type, unsigned int audio_set,
                   unsigned int tuner, unsigned int standard, unsigned int status)
    : SerializedValue("TVInput", "NMM"),
      __num(num),
      __name(name),
      __type(type),
      __audio_set(audio_set),
      __tuner(tuner),
      __standard(standard),
      __status(status)
  {
    registerAttributes();
  }
  
  const TVInput& TVInput::operator=(const TVInput& other)  {
    SerializedValue::operator=(other);
    assign(other);
    return *this;
  }
  
  void TVInput::assign(const TVInput& other) {
    /* Begin> assign the attributes */
    __num       = other.getNum();
    __name      = other.getName();
    __type      = other.getType();
    __audio_set = other.getAudioSet();
    __tuner     = other.getTuner();
    __standard  = other.getStandard();
    __status    = other.getStatus();
    /* End < */
  }
  
  void TVInput::registerAttributes() {
    /* Begin> register the attributes by the baseclass */
    insertArgument(&__num);
    insertArgument(&__name);
    insertArgument(&__type);
    insertArgument(&__audio_set);
    insertArgument(&__tuner);
    insertArgument(&__standard);
    insertArgument(&__status);
    /* End< */
  }
  
  TConstructor<TVInput> __nmm__tv_input("TVInput", "NMM");
  /*============================== TVInput ==============================*/
  
  
  /*============================== TVStandard ==============================*/
  TVStandard::TVStandard()
    : SerializedValue("TVStandard", "NMM"),
      __id(0),
      __name(),
      __fp_numerator(0),
      __fp_denominator(0),
      __lines(0)
  {
    registerAttributes();
  }
  
  TVStandard::TVStandard(const TVStandard& other)
    : SerializedValue("TVStandard", "NMM"),
      __id(0),
      __name(),
      __fp_numerator(0),
      __fp_denominator(0),
      __lines(0)
  {
    assign(other);
    registerAttributes();
  }
  
  TVStandard::TVStandard(unsigned int id, const string& name, unsigned int fp_numerator, unsigned int fp_denominator,
                             unsigned int lines)
    : SerializedValue("TVStandard", "NMM"),
      __id(id),
      __name(name),
      __fp_numerator(fp_numerator),
      __fp_denominator(fp_denominator),
      __lines(lines)
  {
    registerAttributes();
  }
  
  const TVStandard& TVStandard::operator=(const TVStandard& other)  {
    SerializedValue::operator=(other);
    assign(other);
    return *this;
  }
  
  void TVStandard::assign(const TVStandard& other) {
    /* Begin> assign the attributes */
    __id             = other.getID();
    __name           = other.getName();
    __fp_numerator   = other.getFramePeriodNumerator();
    __fp_denominator = other.getFramePeriodDenominator();
    __lines          = other.getFrameLines();
    /* End < */
  }
  
  void TVStandard::registerAttributes() {
    /* Begin> register the attributes by the baseclass */
    insertArgument(&__id);
    insertArgument(&__name);
    insertArgument(&__fp_numerator);
    insertArgument(&__fp_denominator);
    insertArgument(&__lines);
    /* End< */
  }
  
  TConstructor<TVStandard> __nmm__tv_standard("TVStandard", "NMM");
  /*============================== TVStandard ==============================*/


  /*============================== TVTuner ==============================*/
  TVTuner::TVTuner()
    : SerializedValue("TVTuner", "NMM"),
      __index(0),
      __name(),
      __type(TYPE_UNKNOWN),
      __range_min(0),
      __range_max(0),
      __granularity(0)
  {
    registerAttributes();
  }
  
  TVTuner::TVTuner(const TVTuner& other)
    : SerializedValue("TVTuner", "NMM"),
      __index(0),
      __name(),
      __type(TYPE_UNKNOWN),
      __range_min(0),
      __range_max(0),
      __granularity(0)
  {
    assign(other);
    registerAttributes();
  }
  
  TVTuner::TVTuner(unsigned int index, const string& name, unsigned int type,
                   float range_min, float range_max, float granularity)
    : SerializedValue("TVTuner", "NMM"),
      __index(index),
      __name(name),
      __type(type),
      __range_min(range_min),
      __range_max(range_max),
      __granularity(granularity)
  {
    registerAttributes();
  }
  
  const TVTuner& TVTuner::operator=(const TVTuner& other)  {
    SerializedValue::operator=(other);
    assign(other);
    return *this;
  }
  
  void TVTuner::assign(const TVTuner& other) {
    /* Begin> assign the attributes */
    __index          = other.getIndex();
    __name           = other.getName();
    __type           = other.getType();
    __range_min      = other.getMinFrequency();
    __range_max      = other.getMaxFrequency();
    __granularity    = other.getGranularity();
    /* End < */
  }
  
  void TVTuner::registerAttributes() {
    /* Begin> register the attributes by the baseclass */
    insertArgument(&__index);
    insertArgument(&__name);
    insertArgument(&__type);
    insertArgument(&__range_min);
    insertArgument(&__range_max);
    insertArgument(&__granularity);
    /* End< */
  }
  
  TConstructor<TVTuner> __nmm__tv_tuner("TVTuner", "NMM");
  /*============================== TVTuner ==============================*/
  
  
  /*============================== List serializers ==============================*/
  IValueStream& operator>>(IValueStream& iv, list<TVAudio>& tv_audio_list){
    return readList<TVAudio>(iv, tv_audio_list, "TVAudio");
  }
  IValueStream& operator>>(IValueStream& iv, list<TVInput>& tv_input_list){
    return readList<TVInput>(iv, tv_input_list, "TVInput");
  }
  IValueStream& operator>>(IValueStream& iv, list<TVStandard>& tv_standard_list){
    return readList<TVStandard>(iv, tv_standard_list, "TVStandard");
  }
  IValueStream& operator>>(IValueStream& iv, list<TVTuner>& tv_tuner_list){
    return readList<TVTuner>(iv, tv_tuner_list, "TVTuner");
  }
  
  OValueStream& operator<<(OValueStream& ov, const list<TVAudio>& tv_audio_list) {
    return writeList<TVAudio>(ov, tv_audio_list, "TVAudio");
  }
  OValueStream& operator<<(OValueStream& ov, const list<TVInput>& tv_input_list) {
    return writeList<TVInput>(ov, tv_input_list, "TVInput");
  }
  OValueStream& operator<<(OValueStream& ov, const list<TVStandard>& tv_standard_list) {
    return writeList<TVStandard>(ov, tv_standard_list, "TVStandard");
  }
  OValueStream& operator<<(OValueStream& ov, const list<TVTuner>& tv_tuner_list) {
    return writeList<TVTuner>(ov, tv_tuner_list, "TVTuner");
  }
  
  TValueConstructor< list<TVAudio> >    tv_audio_value   ("list_TVAudio",    "std");
  TValueConstructor< list<TVInput> >    tv_input_value   ("list_TVInput",    "std");
  TValueConstructor< list<TVStandard> > tv_standard_value("list_TVStandard", "std");
  TValueConstructor< list<TVTuner> >    tv_tuner_value   ("list_TVTuner",    "std");
  /*============================== List serializers ==============================*/
  
#include "nmm/nmm_prefix.hpp"

} // namespace NMM

#include "nmm/nmm_suffix.hpp"
