/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ChannelDescription.hpp"
#include "nmm/comm/serialize/StdValue.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM{
  ChannelDescription::ChannelDescription()
    :NMM::SerializedValue("ChannelDescription", "NMM"),
          id("")
  {
    registerValues();
  }

  ChannelDescription::ChannelDescription(const ChannelDescription& cd)
    :NMM::SerializedValue("ChannelDescription", "NMM")
  {
    registerValues();
    *this = cd;
  }

  ChannelDescription::~ChannelDescription(){}

  void ChannelDescription::setID(string channelid)
  {
    id = channelid;
  }

  string ChannelDescription::getID() const  {
    return id.getRValue();
  }
  string ChannelDescription::getID(){
    return id.getRValue();
  }

  void ChannelDescription::addChannelName(string name)
  {
    (names.getRValue()).push_back(name);
    (names.getRValue()).sort();
    (names.getRValue()).unique();                       
  }
  void ChannelDescription::setChannelNames(list<string> namelist)
  {
    names = namelist;
  }

  list<string> ChannelDescription::getChannelNames() const {
    return names.getRValue();
  }
  list<string> ChannelDescription::getChannelNames() {
    return names.getRValue();
  }

  ChannelDescription* ChannelDescription::copy(){
    return new ChannelDescription(*this);
  }

  void ChannelDescription::registerValues(){
    insertArgument(&id);
    insertArgument(&names);
  }


  bool ChannelDescription::operator==(const ChannelDescription &cd) const{
    if(id.getRValue() == cd.getID() && names.getRValue() == cd.getChannelNames()){
      return true;
    }
    return false;
  }

  bool ChannelDescription::operator<(const ChannelDescription &cd) const{
    if(id.getRValue() <  cd.getID()){
      return true;
    }
    return false;
  }

  OValueStream& operator<<(OValueStream& os, const list<ChannelDescription>& cd){
    return writeList<ChannelDescription>(os, cd, "ChannelDescription");
  }
  IValueStream& operator>>(IValueStream& is, list<ChannelDescription>& cd){
    return readList<ChannelDescription>(is, cd, "ChannelDescription");
  }

  TConstructor<ChannelDescription> nmm_cd("ChannelDescription", "NMM");
  TValueConstructor<list<ChannelDescription> > nmm_cd_list("list_ChannelDescription","std");


  OValueStream& operator<<(OValueStream& os, const list<ChannelDescriptionList>& cl){
    return writeList<ChannelDescriptionList>(os, cl, "ChannelDescriptionList");
  }
  IValueStream& operator>>(IValueStream& is, list<ChannelDescriptionList>& cl){
    return readList<ChannelDescriptionList>(is, cl, "CHannelDescriptionList");
  }
  
  TConstructor<ChannelDescriptionList> nmm_cl("ChannelDescriptionList", "NMM");
  TValueConstructor<list<ChannelDescriptionList> > nmm_cl_list("list_ChannelDescriptionList","NMM");

}

#include "nmm/nmm_suffix.hpp"
