/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "INxtvepgPlugin.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void INxtvepgPlugin::used() {} 
  
  
  void INxtvepgPlugin::setLocation(const string& loc)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > loc_value(loc, false);
    NMM::Event event("INxtvepgPlugin::setLocation", &loc_value);
    if(const_cast<INxtvepgPlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void INxtvepgPlugin::setPort(const nmm_int32& p)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > p_value(p, false);
    NMM::Event event("INxtvepgPlugin::setPort", &p_value);
    if(const_cast<INxtvepgPlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void INxtvepgPlugin::addProvider(const string& p)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > p_value(p, false);
    NMM::Event event("INxtvepgPlugin::addProvider", &p_value);
    if(const_cast<INxtvepgPlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  list<string >  INxtvepgPlugin::getProvider()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<list<string >  > return_value;
    NMM::Event event("INxtvepgPlugin::getProvider");
    event.setReturnValue(&return_value);
    if(const_cast<INxtvepgPlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void INxtvepgPlugin::setConfigFile(const string& file)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > file_value(file, false);
    NMM::Event event("INxtvepgPlugin::setConfigFile", &file_value);
    if(const_cast<INxtvepgPlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  string INxtvepgPlugin::getConfigFile()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("INxtvepgPlugin::getConfigFile");
    event.setReturnValue(&return_value);
    if(const_cast<INxtvepgPlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void INxtvepgPlugin::setDBDir(const string& db)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > db_value(db, false);
    NMM::Event event("INxtvepgPlugin::setDBDir", &db_value);
    if(const_cast<INxtvepgPlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  string INxtvepgPlugin::getDBDir()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("INxtvepgPlugin::getDBDir");
    event.setReturnValue(&return_value);
    if(const_cast<INxtvepgPlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void INxtvepgPlugin::setLocale(const string& l)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > l_value(l, false);
    NMM::Event event("INxtvepgPlugin::setLocale", &l_value);
    if(const_cast<INxtvepgPlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  string INxtvepgPlugin::getLocale()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("INxtvepgPlugin::getLocale");
    event.setReturnValue(&return_value);
    if(const_cast<INxtvepgPlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void INxtvepgPlugin::setPeriod(const string& p)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > p_value(p, false);
    NMM::Event event("INxtvepgPlugin::setPeriod", &p_value);
    if(const_cast<INxtvepgPlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  string INxtvepgPlugin::getPeriod()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("INxtvepgPlugin::getPeriod");
    event.setReturnValue(&return_value);
    if(const_cast<INxtvepgPlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void INxtvepgPlugin::setUseTv_split(const nmm_bool& b)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > b_value(b, false);
    NMM::Event event("INxtvepgPlugin::setUseTv_split", &b_value);
    if(const_cast<INxtvepgPlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_bool INxtvepgPlugin::getUseTv_split()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("INxtvepgPlugin::getUseTv_split");
    event.setReturnValue(&return_value);
    if(const_cast<INxtvepgPlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* INxtvepgPlugin::INxtvepgPlugin_type = "INxtvepgPlugin";
  NMM::TConstructor<INxtvepgPlugin > __INxtvepgPlugin_prototype("INxtvepgPlugin","NMM");


  const char* INxtvepgPlugin::setLocation_event = "INxtvepgPlugin::setLocation";
  const char* INxtvepgPlugin::setPort_event = "INxtvepgPlugin::setPort";
  const char* INxtvepgPlugin::addProvider_event = "INxtvepgPlugin::addProvider";
  const char* INxtvepgPlugin::getProvider_event = "INxtvepgPlugin::getProvider";
  const char* INxtvepgPlugin::setConfigFile_event = "INxtvepgPlugin::setConfigFile";
  const char* INxtvepgPlugin::getConfigFile_event = "INxtvepgPlugin::getConfigFile";
  const char* INxtvepgPlugin::setDBDir_event = "INxtvepgPlugin::setDBDir";
  const char* INxtvepgPlugin::getDBDir_event = "INxtvepgPlugin::getDBDir";
  const char* INxtvepgPlugin::setLocale_event = "INxtvepgPlugin::setLocale";
  const char* INxtvepgPlugin::getLocale_event = "INxtvepgPlugin::getLocale";
  const char* INxtvepgPlugin::setPeriod_event = "INxtvepgPlugin::setPeriod";
  const char* INxtvepgPlugin::getPeriod_event = "INxtvepgPlugin::getPeriod";
  const char* INxtvepgPlugin::setUseTv_split_event = "INxtvepgPlugin::setUseTv_split";
  const char* INxtvepgPlugin::getUseTv_split_event = "INxtvepgPlugin::getUseTv_split";


}


#include "nmm/nmm_suffix.hpp"

