/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ITVGuide.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ITVGuide::used() {} 
  
  
  Result ITVGuide::insertChannel(const string& id /* = "" */, const string& name /* = "" */)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > id_value(id, false);
    NMM::TInValue<string > name_value(name, false);
    NMM::Event event("ITVGuide::insertChannel", &id_value, &name_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuide*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVGuide::insertChannelDescription(const ChannelDescription& cd)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<ChannelDescription > cd_value(cd, false);
    NMM::Event event("ITVGuide::insertChannelDescription", &cd_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuide*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVGuide::insertEvent(const ProgramDescription& pd)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<ProgramDescription > pd_value(pd, false);
    NMM::Event event("ITVGuide::insertEvent", &pd_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuide*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string ITVGuide::getDirectory()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("ITVGuide::getDirectory");
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuide*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<ChannelDescription >  ITVGuide::getChannelsList()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<list<ChannelDescription >  > return_value;
    NMM::Event event("ITVGuide::getChannelsList");
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuide*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string ITVGuide::getChannelById(const string& id)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::TInValue<string > id_value(id, false);
    NMM::Event event("ITVGuide::getChannelById", &id_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuide*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string ITVGuide::getChannelByName(const string& name)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::TInValue<string > name_value(name, false);
    NMM::Event event("ITVGuide::getChannelByName", &name_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuide*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ProgramDescription ITVGuide::getEvent(const LocalTime& start, const LocalTime& end, const ChannelDescription& channel)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ProgramDescription > return_value;
    NMM::TInValue<LocalTime > start_value(start, false);
    NMM::TInValue<LocalTime > end_value(end, false);
    NMM::TInValue<ChannelDescription > channel_value(channel, false);
    NMM::Event event("ITVGuide::getEvent", &start_value, &end_value, &channel_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuide*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<ProgramDescription >  ITVGuide::getEventsOnDayOnChannel(const LocalTime& day, const ChannelDescription& channel)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<list<ProgramDescription >  > return_value;
    NMM::TInValue<LocalTime > day_value(day, false);
    NMM::TInValue<ChannelDescription > channel_value(channel, false);
    NMM::Event event("ITVGuide::getEventsOnDayOnChannel", &day_value, &channel_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuide*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<ProgramDescription >  ITVGuide::search(const ProgramFilter& pf, const ITVGuidePeerObject* peer /* = NULL */)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<list<ProgramDescription >  > return_value;
    NMM::TInValue<ProgramFilter > pf_value(pf, false);
    NMM::TInValue<ITVGuidePeerObject* > peer_value(const_cast<ITVGuidePeerObject* >(peer), false);
    NMM::Event event("ITVGuide::search", &pf_value, &peer_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuide*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ProgramDescription ITVGuide::getCurrentEventOnChannel(const ChannelDescription& channel)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ProgramDescription > return_value;
    NMM::TInValue<ChannelDescription > channel_value(channel, false);
    NMM::Event event("ITVGuide::getCurrentEventOnChannel", &channel_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuide*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ProgramDescription ITVGuide::getFollowingEventOnChannel(const ChannelDescription& channel)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ProgramDescription > return_value;
    NMM::TInValue<ChannelDescription > channel_value(channel, false);
    NMM::Event event("ITVGuide::getFollowingEventOnChannel", &channel_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuide*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<string >  ITVGuide::getCategories()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<list<string >  > return_value;
    NMM::Event event("ITVGuide::getCategories");
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuide*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ITVGuidePlugin* ITVGuide::getPluginInterface(const string& type)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ITVGuidePlugin* > return_value;
    NMM::TInValue<string > type_value(type, false);
    NMM::Event event("ITVGuide::getPluginInterface", &type_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuide*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITVGuide::cleanDatabase(const nmm_int32& days)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_int32 > days_value(days, false);
    NMM::Event event("ITVGuide::cleanDatabase", &days_value);
    if(const_cast<ITVGuide*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* ITVGuide::ITVGuide_type = "ITVGuide";
  NMM::TConstructor<ITVGuide > __ITVGuide_prototype("ITVGuide","NMM");


  const char* ITVGuide::insertChannel_event = "ITVGuide::insertChannel";
  const char* ITVGuide::insertChannelDescription_event = "ITVGuide::insertChannelDescription";
  const char* ITVGuide::insertEvent_event = "ITVGuide::insertEvent";
  const char* ITVGuide::getDirectory_event = "ITVGuide::getDirectory";
  const char* ITVGuide::getChannelsList_event = "ITVGuide::getChannelsList";
  const char* ITVGuide::getChannelById_event = "ITVGuide::getChannelById";
  const char* ITVGuide::getChannelByName_event = "ITVGuide::getChannelByName";
  const char* ITVGuide::getEvent_event = "ITVGuide::getEvent";
  const char* ITVGuide::getEventsOnDayOnChannel_event = "ITVGuide::getEventsOnDayOnChannel";
  const char* ITVGuide::search_event = "ITVGuide::search";
  const char* ITVGuide::getCurrentEventOnChannel_event = "ITVGuide::getCurrentEventOnChannel";
  const char* ITVGuide::getFollowingEventOnChannel_event = "ITVGuide::getFollowingEventOnChannel";
  const char* ITVGuide::getCategories_event = "ITVGuide::getCategories";
  const char* ITVGuide::getPluginInterface_event = "ITVGuide::getPluginInterface";
  const char* ITVGuide::cleanDatabase_event = "ITVGuide::cleanDatabase";


}


#include "nmm/nmm_suffix.hpp"

