/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ITVGuideEvents.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ITVGuideEvents::used() {} 
  
  
  Result ITVGuideEvents::partiallyFinished()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("ITVGuideEvents::partiallyFinished");
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuideEvents*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVGuideEvents::finished()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("ITVGuideEvents::finished");
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuideEvents*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVGuideEvents::newChannel()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("ITVGuideEvents::newChannel");
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuideEvents*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVGuideEvents::newShow(const string& channelID)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > channelID_value(channelID, false);
    NMM::Event event("ITVGuideEvents::newShow", &channelID_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuideEvents*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVGuideEvents::informationUpdate()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("ITVGuideEvents::informationUpdate");
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuideEvents*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ITVGuideEvents::ITVGuideEvents_type = "ITVGuideEvents";
  NMM::TConstructor<ITVGuideEvents > __ITVGuideEvents_prototype("ITVGuideEvents","NMM");


  const char* ITVGuideEvents::partiallyFinished_event = "ITVGuideEvents::partiallyFinished";
  const char* ITVGuideEvents::finished_event = "ITVGuideEvents::finished";
  const char* ITVGuideEvents::newChannel_event = "ITVGuideEvents::newChannel";
  const char* ITVGuideEvents::newShow_event = "ITVGuideEvents::newShow";
  const char* ITVGuideEvents::informationUpdate_event = "ITVGuideEvents::informationUpdate";


}


#include "nmm/nmm_suffix.hpp"

