/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ITVGuideEventsImpl_HPP
#define NMM_ITVGuideEventsImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "ITVGuideEvents.hpp"


#include "visibility.hpp"

namespace NMM
{
  /**
   *This interface represents the events sent by a TVGuide-Plugin or the TVGuide itself
   */

  class DLLEXPORT ITVGuideEventsImpl  : public virtual NMM::NMMObject { 
  public:
    ITVGuideEventsImpl() { addInterface(new ITVGuideEvents(*this)); registerEvents(); };

    virtual ~ITVGuideEventsImpl() {};

  protected:
    /**
     * Event sent when a certain state in grabbing data is reached
     */

    virtual Result partiallyFinished() = 0;
    /*
     * This event is sent when all the data available is grabbed. Can only be sent by plugins wich are starteted periodically
     */

    virtual Result finished() = 0;
    /**
     * Indicates, that a new channel is available in the database
     */

    virtual Result newChannel() = 0;
    /**
     * indicates, that a new Event-Information is available for the given channel
     */

    virtual Result newShow(const string& channelID) = 0;
    /**
     * Indicates, that the information for an already insertet event has been updated
     */

    virtual Result informationUpdate() = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
