/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ITVGuideImpl.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/messaging/TEDObject.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  
  void ITVGuideImpl::registerEvents()
  {
  
    getEventDispatcher().registerEvent(ITVGuide::insertChannel_event, new NMM::TEDObject2<ITVGuideImpl, string, string, Result(ITVGuideImpl:: * )(const string&, const string&), Result >(this, &ITVGuideImpl::insertChannel));
  
    getEventDispatcher().registerEvent(ITVGuide::insertChannelDescription_event, new NMM::TEDObject1<ITVGuideImpl, ChannelDescription, Result(ITVGuideImpl:: * )(const ChannelDescription&), Result >(this, &ITVGuideImpl::insertChannelDescription));
  
    getEventDispatcher().registerEvent(ITVGuide::insertEvent_event, new NMM::TEDObject1<ITVGuideImpl, ProgramDescription, Result(ITVGuideImpl:: * )(const ProgramDescription&), Result >(this, &ITVGuideImpl::insertEvent));
  
    getEventDispatcher().registerEvent(ITVGuide::getDirectory_event, new NMM::TEDObject0<ITVGuideImpl, string(ITVGuideImpl:: * )(), string >(this, &ITVGuideImpl::getDirectory));
  
    getEventDispatcher().registerEvent(ITVGuide::getChannelsList_event, new NMM::TEDObject0<ITVGuideImpl, list<ChannelDescription > (ITVGuideImpl:: * )(), list<ChannelDescription >  >(this, &ITVGuideImpl::getChannelsList));
  
    getEventDispatcher().registerEvent(ITVGuide::getChannelById_event, new NMM::TEDObject1<ITVGuideImpl, string, string(ITVGuideImpl:: * )(const string&), string >(this, &ITVGuideImpl::getChannelById));
  
    getEventDispatcher().registerEvent(ITVGuide::getChannelByName_event, new NMM::TEDObject1<ITVGuideImpl, string, string(ITVGuideImpl:: * )(const string&), string >(this, &ITVGuideImpl::getChannelByName));
  
    getEventDispatcher().registerEvent(ITVGuide::getEvent_event, new NMM::TEDObject3<ITVGuideImpl, LocalTime, LocalTime, ChannelDescription, ProgramDescription(ITVGuideImpl:: * )(const LocalTime&, const LocalTime&, const ChannelDescription&), ProgramDescription >(this, &ITVGuideImpl::getEvent));
  
    getEventDispatcher().registerEvent(ITVGuide::getEventsOnDayOnChannel_event, new NMM::TEDObject2<ITVGuideImpl, LocalTime, ChannelDescription, list<ProgramDescription > (ITVGuideImpl:: * )(const LocalTime&, const ChannelDescription&), list<ProgramDescription >  >(this, &ITVGuideImpl::getEventsOnDayOnChannel));
  
    getEventDispatcher().registerEvent(ITVGuide::search_event, new NMM::TEDObject2<ITVGuideImpl, ProgramFilter, ITVGuidePeerObject*, list<ProgramDescription > (ITVGuideImpl:: * )(const ProgramFilter&, const ITVGuidePeerObject*), list<ProgramDescription >  >(this, &ITVGuideImpl::search));
  
    getEventDispatcher().registerEvent(ITVGuide::getCurrentEventOnChannel_event, new NMM::TEDObject1<ITVGuideImpl, ChannelDescription, ProgramDescription(ITVGuideImpl:: * )(const ChannelDescription&), ProgramDescription >(this, &ITVGuideImpl::getCurrentEventOnChannel));
  
    getEventDispatcher().registerEvent(ITVGuide::getFollowingEventOnChannel_event, new NMM::TEDObject1<ITVGuideImpl, ChannelDescription, ProgramDescription(ITVGuideImpl:: * )(const ChannelDescription&), ProgramDescription >(this, &ITVGuideImpl::getFollowingEventOnChannel));
  
    getEventDispatcher().registerEvent(ITVGuide::getCategories_event, new NMM::TEDObject0<ITVGuideImpl, list<string > (ITVGuideImpl:: * )(), list<string >  >(this, &ITVGuideImpl::getCategories));
  
    getEventDispatcher().registerEvent(ITVGuide::getPluginInterface_event, new NMM::TEDObject1<ITVGuideImpl, string, ITVGuidePlugin*(ITVGuideImpl:: * )(const string&), ITVGuidePlugin* >(this, &ITVGuideImpl::getPluginInterface));
  
    getEventDispatcher().registerEvent(ITVGuide::cleanDatabase_event, new NMM::TEDObject1<ITVGuideImpl, nmm_int32, void(ITVGuideImpl:: * )(const nmm_int32&), void >(this, &ITVGuideImpl::cleanDatabase));

  }

}


#include "nmm/nmm_suffix.hpp"

