/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ITVGuidePeerObject.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ITVGuidePeerObject::used() {} 
  
  
  Result ITVGuidePeerObject::peerResult(const ProgramDescriptionList& pdlist, const ProgramFilter& pf, const list<Interface* > & history)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<ProgramDescriptionList > pdlist_value(pdlist, false);
    NMM::TInValue<ProgramFilter > pf_value(pf, false);
    NMM::TInValue<list<Interface* >  > history_value(history, false);
    NMM::Event event("ITVGuidePeerObject::peerResult", &pdlist_value, &pf_value, &history_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuidePeerObject*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVGuidePeerObject::peerEmptyResult(const ProgramFilter& pf, const list<Interface* > & history)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<ProgramFilter > pf_value(pf, false);
    NMM::TInValue<list<Interface* >  > history_value(history, false);
    NMM::Event event("ITVGuidePeerObject::peerEmptyResult", &pf_value, &history_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuidePeerObject*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVGuidePeerObject::peerChannelResult(const ChannelDescriptionList& cdlist, const list<Interface* > & history)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<ChannelDescriptionList > cdlist_value(cdlist, false);
    NMM::TInValue<list<Interface* >  > history_value(history, false);
    NMM::Event event("ITVGuidePeerObject::peerChannelResult", &cdlist_value, &history_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuidePeerObject*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVGuidePeerObject::peerEmptyChannelResult(const list<Interface* > & history)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<list<Interface* >  > history_value(history, false);
    NMM::Event event("ITVGuidePeerObject::peerEmptyChannelResult", &history_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuidePeerObject*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ITVGuidePeerObject::ITVGuidePeerObject_type = "ITVGuidePeerObject";
  NMM::TConstructor<ITVGuidePeerObject > __ITVGuidePeerObject_prototype("ITVGuidePeerObject","NMM");


  const char* ITVGuidePeerObject::peerResult_event = "ITVGuidePeerObject::peerResult";
  const char* ITVGuidePeerObject::peerEmptyResult_event = "ITVGuidePeerObject::peerEmptyResult";
  const char* ITVGuidePeerObject::peerChannelResult_event = "ITVGuidePeerObject::peerChannelResult";
  const char* ITVGuidePeerObject::peerEmptyChannelResult_event = "ITVGuidePeerObject::peerEmptyChannelResult";
  void ITVGuidePeerSearch::used() {} 
  
  
  list<ProgramDescription >  ITVGuidePeerSearch::peerSearch(const ProgramFilter& filter, const list<Interface* > & history)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<list<ProgramDescription >  > return_value;
    NMM::TInValue<ProgramFilter > filter_value(filter, false);
    NMM::TInValue<list<Interface* >  > history_value(history, false);
    NMM::Event event("ITVGuidePeerSearch::peerSearch", &filter_value, &history_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuidePeerSearch*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<ChannelDescription >  ITVGuidePeerSearch::peerChannelSearch(const list<Interface* > & history)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<list<ChannelDescription >  > return_value;
    NMM::TInValue<list<Interface* >  > history_value(history, false);
    NMM::Event event("ITVGuidePeerSearch::peerChannelSearch", &history_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuidePeerSearch*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ITVGuidePeerSearch::ITVGuidePeerSearch_type = "ITVGuidePeerSearch";
  NMM::TConstructor<ITVGuidePeerSearch > __ITVGuidePeerSearch_prototype("ITVGuidePeerSearch","NMM");


  const char* ITVGuidePeerSearch::peerSearch_event = "ITVGuidePeerSearch::peerSearch";
  const char* ITVGuidePeerSearch::peerChannelSearch_event = "ITVGuidePeerSearch::peerChannelSearch";


}


#include "nmm/nmm_suffix.hpp"

