/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ITVGuidePeerObject_HPP
#define NMM_ITVGuidePeerObject_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/plugins/video/tv/interfaces/tvguide/ProgramDescription.hpp"
#include "nmm/plugins/video/tv/interfaces/tvguide/ProgramFilter.hpp"

#include "visibility.hpp"

namespace NMM
{
  /**
   *Interface to get notification from remote TVGuides of available information
   */

  class DLLEXPORT ITVGuidePeerObject : virtual public NMM::Interface { 
  public:
    ITVGuidePeerObject() : NMM::Interface(ITVGuidePeerObject::ITVGuidePeerObject_type, "NMM") { };

    ITVGuidePeerObject(NMM::NMMObject& parent) : NMM::Interface (parent, ITVGuidePeerObject::ITVGuidePeerObject_type, "NMM") { };

    virtual ~ITVGuidePeerObject() {};

    static const char* ITVGuidePeerObject_type;

    virtual  string getInterfaceType() const { return ITVGuidePeerObject_type; }

    NMM::Interface* copy() const { return new ITVGuidePeerObject(*this); }
    static void used(); 
    static const char* peerResult_event;
    static const char* peerEmptyResult_event;
    static const char* peerChannelResult_event;
    static const char* peerEmptyChannelResult_event;
    /**
     *This method is calles from the remote TVGuide to notify the local one of available information. 
     *@param pdlist list of ProgramDescriptions the local TVGuide has rewusted
     *@param id if the search. This is for the application wich started the request to match the resault to a request
     *@param history This list stores the ITVGuidePeerObect-interfaces of TVGuide wich should be notified about the result
     */

    virtual Result peerResult(const ProgramDescriptionList& pdlist, const ProgramFilter& pf, const list<Interface* > & history)
      throw(const NMM::SerializedException&);

    virtual Result peerEmptyResult(const ProgramFilter& pf, const list<Interface* > & history)
      throw(const NMM::SerializedException&);

    virtual Result peerChannelResult(const ChannelDescriptionList& cdlist, const list<Interface* > & history)
      throw(const NMM::SerializedException&);

    virtual Result peerEmptyChannelResult(const list<Interface* > & history)
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ITVGuidePeerObject> ITVGuidePeerObject_var;


  /**
   *Interface to search remote TVGuides for information 
   */

  class DLLEXPORT ITVGuidePeerSearch : virtual public NMM::Interface { 
  public:
    ITVGuidePeerSearch() : NMM::Interface(ITVGuidePeerSearch::ITVGuidePeerSearch_type, "NMM") { };

    ITVGuidePeerSearch(NMM::NMMObject& parent) : NMM::Interface (parent, ITVGuidePeerSearch::ITVGuidePeerSearch_type, "NMM") { };

    virtual ~ITVGuidePeerSearch() {};

    static const char* ITVGuidePeerSearch_type;

    virtual  string getInterfaceType() const { return ITVGuidePeerSearch_type; }

    NMM::Interface* copy() const { return new ITVGuidePeerSearch(*this); }
    static void used(); 
    static const char* peerSearch_event;
    static const char* peerChannelSearch_event;
    /**
     *method to search a remote TVGuide for information. 
     *@param filter A ProgramFilter containing the request
     *@param history This list stores the ITVGuidePeerObect-interfaces of TVGuide wich should be notified about the result
     */

    virtual list<ProgramDescription >  peerSearch(const ProgramFilter& filter, const list<Interface* > & history)
      throw(const NMM::SerializedException&);

    virtual list<ChannelDescription >  peerChannelSearch(const list<Interface* > & history)
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ITVGuidePeerSearch> ITVGuidePeerSearch_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
