/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ITVGuidePeerObjectImpl_HPP
#define NMM_ITVGuidePeerObjectImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "ITVGuidePeerObject.hpp"


#include "visibility.hpp"

namespace NMM
{
  /**
   *Interface to get notification from remote TVGuides of available information
   */

  class DLLEXPORT ITVGuidePeerObjectImpl  : public virtual NMM::NMMObject { 
  public:
    ITVGuidePeerObjectImpl() { addInterface(new ITVGuidePeerObject(*this)); registerEvents(); };

    virtual ~ITVGuidePeerObjectImpl() {};

  protected:
    /**
     *This method is calles from the remote TVGuide to notify the local one of available information. 
     *@param pdlist list of ProgramDescriptions the local TVGuide has rewusted
     *@param id if the search. This is for the application wich started the request to match the resault to a request
     *@param history This list stores the ITVGuidePeerObect-interfaces of TVGuide wich should be notified about the result
     */

    virtual Result peerResult(const ProgramDescriptionList& pdlist, const ProgramFilter& pf, const list<Interface* > & history) = 0;

    virtual Result peerEmptyResult(const ProgramFilter& pf, const list<Interface* > & history) = 0;

    virtual Result peerChannelResult(const ChannelDescriptionList& cdlist, const list<Interface* > & history) = 0;

    virtual Result peerEmptyChannelResult(const list<Interface* > & history) = 0;


  private:
    void registerEvents();
  };
  /**
   *Interface to search remote TVGuides for information 
   */

  class DLLEXPORT ITVGuidePeerSearchImpl  : public virtual NMM::NMMObject { 
  public:
    ITVGuidePeerSearchImpl() { addInterface(new ITVGuidePeerSearch(*this)); registerEvents(); };

    virtual ~ITVGuidePeerSearchImpl() {};

  protected:
    /**
     *method to search a remote TVGuide for information. 
     *@param filter A ProgramFilter containing the request
     *@param history This list stores the ITVGuidePeerObect-interfaces of TVGuide wich should be notified about the result
     */

    virtual list<ProgramDescription >  peerSearch(const ProgramFilter& filter, const list<Interface* > & history) = 0;

    virtual list<ChannelDescription >  peerChannelSearch(const list<Interface* > & history) = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
