/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ITVGuidePlugin.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ITVGuidePlugin::used() {} 
  
  
  string ITVGuidePlugin::getName()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("ITVGuidePlugin::getName");
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuidePlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVGuidePlugin::start()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("ITVGuidePlugin::start");
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuidePlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVGuidePlugin::stop()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("ITVGuidePlugin::stop");
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuidePlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVGuidePlugin::init()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("ITVGuidePlugin::init");
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuidePlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVGuidePlugin::deinit()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("ITVGuidePlugin::deinit");
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuidePlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ITVGuidePlugin::State ITVGuidePlugin::getState()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ITVGuidePlugin::State > return_value;
    NMM::Event event("ITVGuidePlugin::getState");
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuidePlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITVGuidePlugin::setPriority(const nmm_uint32& priority)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > priority_value(priority, false);
    NMM::Event event("ITVGuidePlugin::setPriority", &priority_value);
    if(const_cast<ITVGuidePlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_uint32 ITVGuidePlugin::getPriority()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("ITVGuidePlugin::getPriority");
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuidePlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVGuidePlugin::enable()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("ITVGuidePlugin::enable");
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuidePlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVGuidePlugin::disable()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("ITVGuidePlugin::disable");
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuidePlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_bool ITVGuidePlugin::getEnabled()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("ITVGuidePlugin::getEnabled");
    event.setReturnValue(&return_value);
    if(const_cast<ITVGuidePlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ITVGuidePlugin::ITVGuidePlugin_type = "ITVGuidePlugin";
  NMM::TConstructor<ITVGuidePlugin > __ITVGuidePlugin_prototype("ITVGuidePlugin","NMM");
  TValueConstructor< ITVGuidePlugin::State > __ITVGuidePlugin_State_prototype(TypeInfo("ITVGuidePlugin::State", "NMM"));

  NMM::OValueStream& operator<<(NMM::OValueStream& ov, const ITVGuidePlugin::State& t)
  {
    ov.beginType(TypeInfo("ITVGuidePlugin::State", "NMM"));
    nmm_int32 temp = static_cast<nmm_int32>(t);
    ov << temp;
    ov.endType();
    return ov;
  }

  NMM::IValueStream& operator>>(NMM::IValueStream& iv, ITVGuidePlugin::State& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("ITVGuidePlugin::State", "NMM")) {
      throw NMM::InvalidTypeException(TypeInfo("ITVGuidePlugin::State", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    nmm_int32 temp;
    iv >> temp;
    t = static_cast<ITVGuidePlugin::State>(temp);
    iv.goUp();
    return iv;
  }



  const char* ITVGuidePlugin::getName_event = "ITVGuidePlugin::getName";
  const char* ITVGuidePlugin::start_event = "ITVGuidePlugin::start";
  const char* ITVGuidePlugin::stop_event = "ITVGuidePlugin::stop";
  const char* ITVGuidePlugin::init_event = "ITVGuidePlugin::init";
  const char* ITVGuidePlugin::deinit_event = "ITVGuidePlugin::deinit";
  const char* ITVGuidePlugin::getState_event = "ITVGuidePlugin::getState";
  const char* ITVGuidePlugin::setPriority_event = "ITVGuidePlugin::setPriority";
  const char* ITVGuidePlugin::getPriority_event = "ITVGuidePlugin::getPriority";
  const char* ITVGuidePlugin::enable_event = "ITVGuidePlugin::enable";
  const char* ITVGuidePlugin::disable_event = "ITVGuidePlugin::disable";
  const char* ITVGuidePlugin::getEnabled_event = "ITVGuidePlugin::getEnabled";


}


#include "nmm/nmm_suffix.hpp"

