/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ITVGuidePluginImpl_HPP
#define NMM_ITVGuidePluginImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "ITVGuidePlugin.hpp"


#include "visibility.hpp"

namespace NMM
{
  /**
   * Interface for controlling EPG-Plugins
   */

  class DLLEXPORT ITVGuidePluginImpl  : public virtual NMM::NMMObject { 
  public:
    ITVGuidePluginImpl() { addInterface(new ITVGuidePlugin(*this)); registerEvents(); };

    virtual ~ITVGuidePluginImpl() {};

  protected:
    /**
     * returns the name of the plugin
     */

    virtual string getName() = 0;
    /**
     * Starts an update
     */

    virtual Result start() = 0;
    /**
     * stops the update
     */

    virtual Result stop() = 0;
    /**
     * initializes the plugin
     */

    virtual Result init() = 0;
    /**
     * deinitializes the plugin
     */

    virtual Result deinit() = 0;
    /**
     * get the current State of the plugin
     *@return plugin-state
     */

    virtual ITVGuidePlugin::State getState() = 0;
    /**
     * set the priority of an TVGuide-plugin
     */

    virtual void setPriority(const nmm_uint32& priority) = 0;
    /**
     * get the priority of an TVGuide-plugin
     */

    virtual nmm_uint32 getPriority() = 0;
    /**
     * enables a TVGuide-Plugin. Indicates, that it should be startet by the TVGuide
     */

    virtual Result enable() = 0;
    /**
     * disables a TVGuide-plugin. This plugin will not be startetd by TVGuide
     */

    virtual Result disable() = 0;
    /**
     * returns, if the TVGuide-plugin is enabled
     */

    virtual nmm_bool getEnabled() = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
