/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ITVGuideProducer.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ITVGuideProducer::used() {} 
  
  
  void ITVGuideProducer::registerTVGuide(const ITVGuide* itvg)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<ITVGuide* > itvg_value (const_cast<ITVGuide* >(itvg), false);
    NMM::Event event("ITVGuideProducer::registerTVGuide", &itvg_value);
    if(const_cast<ITVGuideProducer*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITVGuideProducer::unregisterTVGuide(const ITVGuide* itvg)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<ITVGuide* > itvg_value (const_cast<ITVGuide* >(itvg), false);
    NMM::Event event("ITVGuideProducer::unregisterTVGuide", &itvg_value);
    if(const_cast<ITVGuideProducer*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* ITVGuideProducer::ITVGuideProducer_type = "ITVGuideProducer";
  NMM::TConstructor<ITVGuideProducer > __ITVGuideProducer_prototype("ITVGuideProducer","NMM");


  const char* ITVGuideProducer::registerTVGuide_event = "ITVGuideProducer::registerTVGuide";
  const char* ITVGuideProducer::unregisterTVGuide_event = "ITVGuideProducer::unregisterTVGuide";


}


#include "nmm/nmm_suffix.hpp"

