/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ITVGuideProducer_HPP
#define NMM_ITVGuideProducer_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "nmm/plugins/video/tv/interfaces/tvguide/ITVGuide.hpp"

#include "visibility.hpp"

namespace NMM
{
  /**
   * This Interface must be implemented by EPG-Sources to register the ITVGuide-Interface
   */

  class DLLEXPORT ITVGuideProducer : virtual public NMM::Interface { 
  public:
    ITVGuideProducer() : NMM::Interface(ITVGuideProducer::ITVGuideProducer_type, "NMM") { };

    ITVGuideProducer(NMM::NMMObject& parent) : NMM::Interface (parent, ITVGuideProducer::ITVGuideProducer_type, "NMM") { };

    virtual ~ITVGuideProducer() {};

    static const char* ITVGuideProducer_type;

    virtual  string getInterfaceType() const { return ITVGuideProducer_type; }

    NMM::Interface* copy() const { return new ITVGuideProducer(*this); }
    static void used(); 
    static const char* registerTVGuide_event;
    static const char* unregisterTVGuide_event;
    /**
     * Registers the ITVGuide-Interface to add Program-Information to the EPG
     *\param itvg ITVGuide-Interface
     */

    virtual void registerTVGuide(const ITVGuide* itvg)
      throw(const NMM::SerializedException&);
    /**
     * unegisters the ITVGuide-Interface
     *\param itvg ITVGuide-Interface
     */

    virtual void unregisterTVGuide(const ITVGuide* itvg)
      throw(const NMM::SerializedException&);


  };

  typedef auto_ptr<ITVGuideProducer> ITVGuideProducer_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
