/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ITVGuideProducerImpl_HPP
#define NMM_ITVGuideProducerImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "ITVGuideProducer.hpp"

#include "nmm/plugins/video/tv/interfaces/tvguide/ITVGuideImpl.hpp"

#include "visibility.hpp"

namespace NMM
{
  /**
   * This Interface must be implemented by EPG-Sources to register the ITVGuide-Interface
   */

  class DLLEXPORT ITVGuideProducerImpl  : public virtual NMM::NMMObject { 
  public:
    ITVGuideProducerImpl() { addInterface(new ITVGuideProducer(*this)); registerEvents(); };

    virtual ~ITVGuideProducerImpl() {};

  protected:
    /**
     * Registers the ITVGuide-Interface to add Program-Information to the EPG
     *\param itvg ITVGuide-Interface
     */

    virtual void registerTVGuide(const ITVGuide* itvg) = 0;
    /**
     * unegisters the ITVGuide-Interface
     *\param itvg ITVGuide-Interface
     */

    virtual void unregisterTVGuide(const ITVGuide* itvg) = 0;


  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
