/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IXMLTVPlugin.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IXMLTVPlugin::used() {} 
  
  
  void IXMLTVPlugin::setFilename(const string& file)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > file_value(file, false);
    NMM::Event event("IXMLTVPlugin::setFilename", &file_value);
    if(const_cast<IXMLTVPlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  string IXMLTVPlugin::getFilename()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("IXMLTVPlugin::getFilename");
    event.setReturnValue(&return_value);
    if(const_cast<IXMLTVPlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IXMLTVPlugin::setConfigFile(const string& file)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > file_value(file, false);
    NMM::Event event("IXMLTVPlugin::setConfigFile", &file_value);
    if(const_cast<IXMLTVPlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  string IXMLTVPlugin::getConfigFile()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("IXMLTVPlugin::getConfigFile");
    event.setReturnValue(&return_value);
    if(const_cast<IXMLTVPlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IXMLTVPlugin::setLocale(const string& loc)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > loc_value(loc, false);
    NMM::Event event("IXMLTVPlugin::setLocale", &loc_value);
    event.setReturnValue(&return_value);
    if(const_cast<IXMLTVPlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string IXMLTVPlugin::getLocale()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("IXMLTVPlugin::getLocale");
    event.setReturnValue(&return_value);
    if(const_cast<IXMLTVPlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IXMLTVPlugin::setPeriod(const string& p)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<string > p_value(p, false);
    NMM::Event event("IXMLTVPlugin::setPeriod", &p_value);
    if(const_cast<IXMLTVPlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  string IXMLTVPlugin::getPeriod()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<string > return_value;
    NMM::Event event("IXMLTVPlugin::getPeriod");
    event.setReturnValue(&return_value);
    if(const_cast<IXMLTVPlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IXMLTVPlugin::setFillHolesInDB(const nmm_bool& b)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_bool > b_value(b, false);
    NMM::Event event("IXMLTVPlugin::setFillHolesInDB", &b_value);
    if(const_cast<IXMLTVPlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_bool IXMLTVPlugin::getFillHolesInDB()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_bool > return_value;
    NMM::Event event("IXMLTVPlugin::getFillHolesInDB");
    event.setReturnValue(&return_value);
    if(const_cast<IXMLTVPlugin*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IXMLTVPlugin::IXMLTVPlugin_type = "IXMLTVPlugin";
  NMM::TConstructor<IXMLTVPlugin > __IXMLTVPlugin_prototype("IXMLTVPlugin","NMM");


  const char* IXMLTVPlugin::setFilename_event = "IXMLTVPlugin::setFilename";
  const char* IXMLTVPlugin::getFilename_event = "IXMLTVPlugin::getFilename";
  const char* IXMLTVPlugin::setConfigFile_event = "IXMLTVPlugin::setConfigFile";
  const char* IXMLTVPlugin::getConfigFile_event = "IXMLTVPlugin::getConfigFile";
  const char* IXMLTVPlugin::setLocale_event = "IXMLTVPlugin::setLocale";
  const char* IXMLTVPlugin::getLocale_event = "IXMLTVPlugin::getLocale";
  const char* IXMLTVPlugin::setPeriod_event = "IXMLTVPlugin::setPeriod";
  const char* IXMLTVPlugin::getPeriod_event = "IXMLTVPlugin::getPeriod";
  const char* IXMLTVPlugin::setFillHolesInDB_event = "IXMLTVPlugin::setFillHolesInDB";
  const char* IXMLTVPlugin::getFillHolesInDB_event = "IXMLTVPlugin::getFillHolesInDB";


}


#include "nmm/nmm_suffix.hpp"

