/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ITVManager.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ITVManager::used() {} 
  
  
  Result ITVManager::setChannel(const nmm_uint32& channel)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<nmm_uint32 > channel_value(channel, false);
    NMM::Event event("ITVManager::setChannel", &channel_value);
    event.setReturnValue(&return_value);
    if(const_cast<ITVManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVManager::channelUp()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("ITVManager::channelUp");
    event.setReturnValue(&return_value);
    if(const_cast<ITVManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVManager::channelDown()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("ITVManager::channelDown");
    event.setReturnValue(&return_value);
    if(const_cast<ITVManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  INode* ITVManager::getTVCard()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<INode* > return_value;
    NMM::Event event("ITVManager::getTVCard");
    event.setReturnValue(&return_value);
    if(const_cast<ITVManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVManager::startGraph()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("ITVManager::startGraph");
    event.setReturnValue(&return_value);
    if(const_cast<ITVManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITVManager::setUsageNotify(const IUsageNotify* u)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<IUsageNotify* > u_value (const_cast<IUsageNotify* >(u), false);
    NMM::Event event("ITVManager::setUsageNotify", &u_value);
    if(const_cast<ITVManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ITVManager::releaseGraph()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("ITVManager::releaseGraph");
    if(const_cast<ITVManager*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* ITVManager::ITVManager_type = "ITVManager";
  NMM::TConstructor<ITVManager > __ITVManager_prototype("ITVManager","NMM");


  const char* ITVManager::setChannel_event = "ITVManager::setChannel";
  const char* ITVManager::channelUp_event = "ITVManager::channelUp";
  const char* ITVManager::channelDown_event = "ITVManager::channelDown";
  const char* ITVManager::getTVCard_event = "ITVManager::getTVCard";
  const char* ITVManager::startGraph_event = "ITVManager::startGraph";
  const char* ITVManager::setUsageNotify_event = "ITVManager::setUsageNotify";
  const char* ITVManager::releaseGraph_event = "ITVManager::releaseGraph";
  void ITVManagerEvents::used() {} 
  
  
  Result ITVManagerEvents::switchingSource()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("ITVManagerEvents::switchingSource");
    event.setReturnValue(&return_value);
    if(const_cast<ITVManagerEvents*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVManagerEvents::sharedSource()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("ITVManagerEvents::sharedSource");
    event.setReturnValue(&return_value);
    if(const_cast<ITVManagerEvents*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVManagerEvents::skippingChannel()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::Event event("ITVManagerEvents::skippingChannel");
    event.setReturnValue(&return_value);
    if(const_cast<ITVManagerEvents*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ITVManagerEvents::ITVManagerEvents_type = "ITVManagerEvents";
  NMM::TConstructor<ITVManagerEvents > __ITVManagerEvents_prototype("ITVManagerEvents","NMM");


  const char* ITVManagerEvents::switchingSource_event = "ITVManagerEvents::switchingSource";
  const char* ITVManagerEvents::sharedSource_event = "ITVManagerEvents::sharedSource";
  const char* ITVManagerEvents::skippingChannel_event = "ITVManagerEvents::skippingChannel";


}


#include "nmm/nmm_suffix.hpp"

