/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ExtIVTVStructs.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"


#include "nmm/nmm_prefix.hpp"

namespace NMM {  
  
  /*============================== IVTVCodec ==============================*/
  IVTVCodec::IVTVCodec()
    : SerializedValue("IVTVCodec", "NMM"),
      __audio(0),
      __bframes(0),
      __bitrate_mode(0),
      __bitrate(0),
      __bitrate_peak(0),
      __dnr_mode(0),
      __dnr_spatial(0),
      __dnr_temporal(0),
      __dnr_type(0),
      __framerate(0),
      __frames_per_gop(0),
      __gop_closure(0),
      __pulldown(0),
      __stream_type(0)
  {
    registerAttributes();
  }
  
  IVTVCodec::IVTVCodec(unsigned int audio, unsigned int bframes,
                       unsigned int bitrate_mode, unsigned int bitrate, unsigned int bitrate_peak,
                       unsigned int dnr_mode, unsigned int dnr_spatial, unsigned int dnr_temporal, unsigned int dnr_type,
                       unsigned int framerate, unsigned int frames_per_gop, unsigned int gop_closure,
                       unsigned int pulldown, unsigned int stream_type)
    : SerializedValue("IVTVCodec", "NMM"),
      __audio(audio),
      __bframes(bframes),
      __bitrate_mode(bitrate_mode),
      __bitrate(bitrate),
      __bitrate_peak(bitrate_peak),
      __dnr_mode(dnr_mode),
      __dnr_spatial(dnr_spatial),
      __dnr_temporal(dnr_temporal),
      __dnr_type(dnr_type),
      __framerate(framerate),
      __frames_per_gop(frames_per_gop),
      __gop_closure(gop_closure),
      __pulldown(pulldown),
      __stream_type(stream_type)
  {
    registerAttributes();
  }
  
  void IVTVCodec::registerAttributes() {
    /* Begin> register the attributes by the baseclass */
    insertArgument(&__audio);
    insertArgument(&__bframes);
    insertArgument(&__bitrate_mode);
    insertArgument(&__bitrate);
    insertArgument(&__bitrate_peak);
    insertArgument(&__dnr_mode);
    insertArgument(&__dnr_spatial);
    insertArgument(&__dnr_temporal);
    insertArgument(&__dnr_type);
    insertArgument(&__framerate);
    insertArgument(&__frames_per_gop);
    insertArgument(&__gop_closure);
    insertArgument(&__pulldown);
    insertArgument(&__stream_type);
    /* End< */
  }
  
  TConstructor<IVTVCodec> __nmm__ivtv_codec("IVTVCodec", "NMM");
  /*============================== IVTVCodec ==============================*/
  
#include "nmm/nmm_prefix.hpp"

} // namespace NMM

#include "nmm/nmm_suffix.hpp"
