/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IIVTV.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void IIVTV::used() {} 
  
  
  IVTVCodec IIVTV::getCodecParams() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<IVTVCodec > return_value;
    NMM::Event event("IIVTV::getCodecParams");
    event.setReturnValue(&return_value);
    if(const_cast<IIVTV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IIVTV::setCodecParams(const IVTVCodec& codec) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<IVTVCodec > codec_value(codec, false);
    NMM::Event event("IIVTV::setCodecParams", &codec_value);
    if(const_cast<IIVTV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IIVTV::setGOPSize(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_uint32 > value_value(value, false);
    NMM::Event event("IIVTV::setGOPSize", &value_value);
    if(const_cast<IIVTV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IIVTV::setBFrames(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_uint32 > value_value(value, false);
    NMM::Event event("IIVTV::setBFrames", &value_value);
    if(const_cast<IIVTV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IIVTV::setGOPClosure(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_uint32 > value_value(value, false);
    NMM::Event event("IIVTV::setGOPClosure", &value_value);
    if(const_cast<IIVTV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IIVTV::setPulldown(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_uint32 > value_value(value, false);
    NMM::Event event("IIVTV::setPulldown", &value_value);
    if(const_cast<IIVTV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IIVTV::setBitrate(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_uint32 > value_value(value, false);
    NMM::Event event("IIVTV::setBitrate", &value_value);
    if(const_cast<IIVTV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IIVTV::setPeakBitrate(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_uint32 > value_value(value, false);
    NMM::Event event("IIVTV::setPeakBitrate", &value_value);
    if(const_cast<IIVTV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IIVTV::setStreamType(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_uint32 > value_value(value, false);
    NMM::Event event("IIVTV::setStreamType", &value_value);
    if(const_cast<IIVTV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IIVTV::setBitrateMode(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_uint32 > value_value(value, false);
    NMM::Event event("IIVTV::setBitrateMode", &value_value);
    if(const_cast<IIVTV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IIVTV::setDNRMode(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_uint32 > value_value(value, false);
    NMM::Event event("IIVTV::setDNRMode", &value_value);
    if(const_cast<IIVTV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IIVTV::setDNRType(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_uint32 > value_value(value, false);
    NMM::Event event("IIVTV::setDNRType", &value_value);
    if(const_cast<IIVTV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IIVTV::setDNRSpatial(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_uint32 > value_value(value, false);
    NMM::Event event("IIVTV::setDNRSpatial", &value_value);
    if(const_cast<IIVTV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IIVTV::setDNRTemporal(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_uint32 > value_value(value, false);
    NMM::Event event("IIVTV::setDNRTemporal", &value_value);
    if(const_cast<IIVTV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IIVTV::setAspect(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TInValue<nmm_uint32 > value_value(value, false);
    NMM::Event event("IIVTV::setAspect", &value_value);
    if(const_cast<IIVTV*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IIVTV::IIVTV_type = "IIVTV";
  NMM::TConstructor<IIVTV > __IIVTV_prototype("IIVTV","NMM");


  const char* IIVTV::getCodecParams_event = "IIVTV::getCodecParams";
  const char* IIVTV::setCodecParams_event = "IIVTV::setCodecParams";
  const char* IIVTV::setGOPSize_event = "IIVTV::setGOPSize";
  const char* IIVTV::setBFrames_event = "IIVTV::setBFrames";
  const char* IIVTV::setGOPClosure_event = "IIVTV::setGOPClosure";
  const char* IIVTV::setPulldown_event = "IIVTV::setPulldown";
  const char* IIVTV::setBitrate_event = "IIVTV::setBitrate";
  const char* IIVTV::setPeakBitrate_event = "IIVTV::setPeakBitrate";
  const char* IIVTV::setStreamType_event = "IIVTV::setStreamType";
  const char* IIVTV::setBitrateMode_event = "IIVTV::setBitrateMode";
  const char* IIVTV::setDNRMode_event = "IIVTV::setDNRMode";
  const char* IIVTV::setDNRType_event = "IIVTV::setDNRType";
  const char* IIVTV::setDNRSpatial_event = "IIVTV::setDNRSpatial";
  const char* IIVTV::setDNRTemporal_event = "IIVTV::setDNRTemporal";
  const char* IIVTV::setAspect_event = "IIVTV::setAspect";


}


#include "nmm/nmm_suffix.hpp"

