/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IIVTV_HPP
#define NMM_IIVTV_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/plugins/video/tv/ivtv/ExtIVTVStructs.hpp"

#include "nmm/plugins/video/tv/interfaces/ITV.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Interface for IVTV driver.
   */

  class DLLEXPORT IIVTV : virtual public ITV { 
  public:
    IIVTV() : NMM::Interface(IIVTV::IIVTV_type, "NMM") { };

    IIVTV(NMM::NMMObject& parent) : NMM::Interface (parent, IIVTV::IIVTV_type, "NMM") { };

    virtual ~IIVTV() {};

    static const char* IIVTV_type;

    virtual  string getInterfaceType() const { return IIVTV_type; }

    NMM::Interface* copy() const { return new IIVTV(*this); }
    static void used(); 
    static const char* getCodecParams_event;
    static const char* setCodecParams_event;
    static const char* setGOPSize_event;
    static const char* setBFrames_event;
    static const char* setGOPClosure_event;
    static const char* setPulldown_event;
    static const char* setBitrate_event;
    static const char* setPeakBitrate_event;
    static const char* setStreamType_event;
    static const char* setBitrateMode_event;
    static const char* setDNRMode_event;
    static const char* setDNRType_event;
    static const char* setDNRSpatial_event;
    static const char* setDNRTemporal_event;
    static const char* setAspect_event;
    /** @name Codec.
     */
//@{
    /** This method returns the codec parameters.
     */

    virtual IVTVCodec getCodecParams() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /** This method sets the codec parameters.
     */

    virtual void setCodecParams(const IVTVCodec& codec) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /** This method sets the GOP size
     */

    virtual void setGOPSize(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /** This method sets the number of B-Frames
     */

    virtual void setBFrames(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /** This method sets the GOP closure mode of the codec (1=close GOP, 0=open GOP)
     */

    virtual void setGOPClosure(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /** This method sets inverse telecine (1=ON, 0=OFF)
     */

    virtual void setPulldown(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /** This method sets bitrate
     */

    virtual void setBitrate(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /** This method sets peak bitrate (must be greater than bitrate)
     */

    virtual void setPeakBitrate(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /** This method sets the Streamtype
     *   IVTV_STREAM_PS     = 0
     *   IVTV_STREAM_TS     = 1
     *   IVTV_STREAM_MPEG1  = 2
     *   IVTV_STREAM_PES_AV = 3
     *   IVTV_STREAM_PES_V  = 5
     *   IVTV_STREAM_PES_A  = 7
     *   IVTV_STREAM_DVD    = 10
    */

    virtual void setStreamType(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /** This method sets bitrate mode
     */

    virtual void setBitrateMode(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /** This method sets DNRMode
     */

    virtual void setDNRMode(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /** This method sets DNRType
     */

    virtual void setDNRType(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /** This method sets DNRSpatial
     */

    virtual void setDNRSpatial(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /** This method sets DNRTemporal
     */

    virtual void setDNRTemporal(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);
    /** This method sets aspect
     */

    virtual void setAspect(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);

//@}

  };

  typedef auto_ptr<IIVTV> IIVTV_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
