/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IIVTVImpl.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/messaging/TEDObject.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  
  void IIVTVImpl::registerEvents()
  {
  
    getEventDispatcher().registerEvent(IIVTV::getCodecParams_event, new NMM::TEDObject0<IIVTVImpl, IVTVCodec(IIVTVImpl:: * )() const, IVTVCodec >(this, &IIVTVImpl::getCodecParams));
    getEventDispatcher().addLimited(IIVTV::getCodecParams_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(IIVTV::getCodecParams_event, INode::ACTIVATED);
    getEventDispatcher().addLimited(IIVTV::getCodecParams_event, INode::STARTED);
    getEventDispatcher().addLimited(IIVTV::getCodecParams_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(IIVTV::setCodecParams_event, new NMM::TEDObject1<IIVTVImpl, IVTVCodec, void(IIVTVImpl:: * )(const IVTVCodec&), void >(this, &IIVTVImpl::setCodecParams));
    getEventDispatcher().addLimited(IIVTV::setCodecParams_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(IIVTV::setCodecParams_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(IIVTV::setGOPSize_event, new NMM::TEDObject1<IIVTVImpl, nmm_uint32, void(IIVTVImpl:: * )(const nmm_uint32&), void >(this, &IIVTVImpl::setGOPSize));
    getEventDispatcher().addLimited(IIVTV::setGOPSize_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(IIVTV::setGOPSize_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(IIVTV::setBFrames_event, new NMM::TEDObject1<IIVTVImpl, nmm_uint32, void(IIVTVImpl:: * )(const nmm_uint32&), void >(this, &IIVTVImpl::setBFrames));
    getEventDispatcher().addLimited(IIVTV::setBFrames_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(IIVTV::setBFrames_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(IIVTV::setGOPClosure_event, new NMM::TEDObject1<IIVTVImpl, nmm_uint32, void(IIVTVImpl:: * )(const nmm_uint32&), void >(this, &IIVTVImpl::setGOPClosure));
    getEventDispatcher().addLimited(IIVTV::setGOPClosure_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(IIVTV::setGOPClosure_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(IIVTV::setPulldown_event, new NMM::TEDObject1<IIVTVImpl, nmm_uint32, void(IIVTVImpl:: * )(const nmm_uint32&), void >(this, &IIVTVImpl::setPulldown));
    getEventDispatcher().addLimited(IIVTV::setPulldown_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(IIVTV::setPulldown_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(IIVTV::setBitrate_event, new NMM::TEDObject1<IIVTVImpl, nmm_uint32, void(IIVTVImpl:: * )(const nmm_uint32&), void >(this, &IIVTVImpl::setBitrate));
    getEventDispatcher().addLimited(IIVTV::setBitrate_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(IIVTV::setBitrate_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(IIVTV::setPeakBitrate_event, new NMM::TEDObject1<IIVTVImpl, nmm_uint32, void(IIVTVImpl:: * )(const nmm_uint32&), void >(this, &IIVTVImpl::setPeakBitrate));
    getEventDispatcher().addLimited(IIVTV::setPeakBitrate_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(IIVTV::setPeakBitrate_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(IIVTV::setStreamType_event, new NMM::TEDObject1<IIVTVImpl, nmm_uint32, void(IIVTVImpl:: * )(const nmm_uint32&), void >(this, &IIVTVImpl::setStreamType));
    getEventDispatcher().addLimited(IIVTV::setStreamType_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(IIVTV::setStreamType_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(IIVTV::setBitrateMode_event, new NMM::TEDObject1<IIVTVImpl, nmm_uint32, void(IIVTVImpl:: * )(const nmm_uint32&), void >(this, &IIVTVImpl::setBitrateMode));
    getEventDispatcher().addLimited(IIVTV::setBitrateMode_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(IIVTV::setBitrateMode_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(IIVTV::setDNRMode_event, new NMM::TEDObject1<IIVTVImpl, nmm_uint32, void(IIVTVImpl:: * )(const nmm_uint32&), void >(this, &IIVTVImpl::setDNRMode));
    getEventDispatcher().addLimited(IIVTV::setDNRMode_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(IIVTV::setDNRMode_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(IIVTV::setDNRType_event, new NMM::TEDObject1<IIVTVImpl, nmm_uint32, void(IIVTVImpl:: * )(const nmm_uint32&), void >(this, &IIVTVImpl::setDNRType));
    getEventDispatcher().addLimited(IIVTV::setDNRType_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(IIVTV::setDNRType_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(IIVTV::setDNRSpatial_event, new NMM::TEDObject1<IIVTVImpl, nmm_uint32, void(IIVTVImpl:: * )(const nmm_uint32&), void >(this, &IIVTVImpl::setDNRSpatial));
    getEventDispatcher().addLimited(IIVTV::setDNRSpatial_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(IIVTV::setDNRSpatial_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(IIVTV::setDNRTemporal_event, new NMM::TEDObject1<IIVTVImpl, nmm_uint32, void(IIVTVImpl:: * )(const nmm_uint32&), void >(this, &IIVTVImpl::setDNRTemporal));
    getEventDispatcher().addLimited(IIVTV::setDNRTemporal_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(IIVTV::setDNRTemporal_event, INode::OUTPUT_INITIALIZED);
  
    getEventDispatcher().registerEvent(IIVTV::setAspect_event, new NMM::TEDObject1<IIVTVImpl, nmm_uint32, void(IIVTVImpl:: * )(const nmm_uint32&), void >(this, &IIVTVImpl::setAspect));
    getEventDispatcher().addLimited(IIVTV::setAspect_event, INode::INITIALIZED);
    getEventDispatcher().addLimited(IIVTV::setAspect_event, INode::OUTPUT_INITIALIZED);

  }

}


#include "nmm/nmm_suffix.hpp"

