/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_IIVTVImpl_HPP
#define NMM_IIVTVImpl_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/NMMObject.hpp"
#include "IIVTV.hpp"

#include "nmm/plugins/video/tv/interfaces/ITVImpl.hpp"

#include "visibility.hpp"

namespace NMM
{
  /** Interface for IVTV driver.
   */

  class DLLEXPORT IIVTVImpl  : virtual public ITVImpl { 
  public:
    IIVTVImpl() { addInterface(new IIVTV(*this)); registerEvents(); };

    virtual ~IIVTVImpl() {};

  protected:
    /** @name Codec.
     */
//@{
    /** This method returns the codec parameters.
     */

    virtual IVTVCodec getCodecParams() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets the codec parameters.
     */

    virtual void setCodecParams(const IVTVCodec& codec) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets the GOP size
     */

    virtual void setGOPSize(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets the number of B-Frames
     */

    virtual void setBFrames(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets the GOP closure mode of the codec (1=close GOP, 0=open GOP)
     */

    virtual void setGOPClosure(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets inverse telecine (1=ON, 0=OFF)
     */

    virtual void setPulldown(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets bitrate
     */

    virtual void setBitrate(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets peak bitrate (must be greater than bitrate)
     */

    virtual void setPeakBitrate(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets the Streamtype
     *   IVTV_STREAM_PS     = 0
     *   IVTV_STREAM_TS     = 1
     *   IVTV_STREAM_MPEG1  = 2
     *   IVTV_STREAM_PES_AV = 3
     *   IVTV_STREAM_PES_V  = 5
     *   IVTV_STREAM_PES_A  = 7
     *   IVTV_STREAM_DVD    = 10
    */

    virtual void setStreamType(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets bitrate mode
     */

    virtual void setBitrateMode(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets DNRMode
     */

    virtual void setDNRMode(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets DNRType
     */

    virtual void setDNRType(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets DNRSpatial
     */

    virtual void setDNRSpatial(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets DNRTemporal
     */

    virtual void setDNRTemporal(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets aspect
     */

    virtual void setAspect(const nmm_uint32& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;

//@}

  private:
    void registerEvents();
  };

}

#include "nmm/nmm_suffix.hpp"

#endif
