/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "FrequencyTable.hpp"
//#include "nmm/utils/utils.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  
  FrequencyTable::FrequencyTable()
  {}
  
  FrequencyTable::FrequencyTable(const string& name)
    : __name(name)
  {}
  
  unsigned int FrequencyTable::getChannelFrequency(unsigned int channel) const
    throw (const Exception&) 
  {
    if (channel >= __channels.size()) {
      throw Exception(string("Invalid channel number ") + static_cast<int>(channel));
    }
    
    return __channels[channel].channel_frequency;
  }

  const string& FrequencyTable::getChannelName(unsigned int channel) const
    throw (const Exception&) 
  {
    if (channel >= __channels.size()) {
      throw Exception(string("Invalid channel number ") + static_cast<int>(channel));
    }
    
    return __channels[channel].channel_name;
  }
  
  int FrequencyTable::findChannel(const string& name) const {
    map<string, unsigned int>::const_iterator found(__map.find(name));
    if (found == __map.end()) {
      return -1;
    }
    
    return found->second;
  }
  
  void FrequencyTable::addChannel(const string& name, unsigned int frequency)
    throw (const Exception&)
  {
    map<string, unsigned int>::const_iterator found(__map.find(name));
    if (found != __map.end()) {
      throw Exception("channel " + name + " already exists");
    }
    
    unsigned int channel = __channels.size();
    __channels.push_back(Channel(name, frequency));
    __map[name] = channel;
  }
  
#include "nmm/nmm_prefix.hpp"

} // namespace NMM

#include "nmm/nmm_suffix.hpp"
