/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

/**
 * This NMM node was developed in the context of 
 * the multimedia course in WS2001/02 at 
 * University of Saarbruecken/Germany
 */


#ifndef _NMM_TVCARDREADNODE_HPP_
#define _NMM_TVCARDREADNODE_HPP_ _NMM_TVCARDREADNODE_HPP_


#include "nmm/multimedia/sync/GenericSyncSourceNode.hpp"
#include "nmm/multimedia/sync/StreamTimer.hpp"
#include "nmm/plugins/interfaces/IBufferSizeImpl.hpp"
#include "nmm/plugins/video/tv/interfaces/ITVCardImpl.hpp"
//#include "nmm/multimedia/IDeviceImpl.hpp"

//#include "nmm/comm/format/Property.hpp"



#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <linux/videodev.h>

extern "C"
{
#include <linux/videodev.h>
}

namespace NMM {

  class TVCardReadNode
    : public GenericSyncSourceNode,
      public IBufferSizeImpl,
      public ITVCardImpl
  {
  private:
    StreamTimer* timer;

    
  public:
  /** @name Constructor&Destructor
   */  
  
   /** Constructor
   */
    TVCardReadNode(const char *name = "TVCardReadNode",
		   StreamQueue::Mode = StreamQueue::MODE_SUSPEND,
		   int = StreamQueue::DEFAULT_SIZE);
  /** 
   * Virtual Destructor
   */
       
    virtual ~TVCardReadNode();
    
  
  protected:
  
    virtual Result doInit();
    virtual Result doDeinit();
    virtual Result doInitOutput();
    virtual Result doDeinitOutput();
    virtual Result doActivate();
    virtual Result doDeactivate();
    virtual Message* produceBuffer(Buffer *);
    
   /** @name IBufferSize interface
    */
   //@{
   /** Set buffer size.
    */
   virtual void setBufferSize(const int& value);

   /** Get buffer size.
    */
   virtual int getBufferSize();
   //@}

   /** @name ITVCard Interface
    */
   
   //TVCardImpl functions
    
   /** Set device.
    */
   virtual void setDevice(const string& value) throw(const DeviceNotFoundException&);

   /** Get device.
    */
   virtual string getDevice() const;

   /** Set size.
    */
   virtual void setSize(const int& x, const int& y);

   /** Get size.
    */
   virtual void getSize(int& x, int& y);

   /** Get resolution
    */
   virtual void getResolution(int& x, int& y);

   /** Set resolution
    */
   virtual void setResolution(const int& x, const int& y);

   /** Set channel.
    */
   virtual Result setChannel(const int& channel);

   /** Get channel.
    */
   virtual Result getChannel(int& channel, string& name);
   //@}

   virtual Result hasChannel(int&, const string&){return FAILURE;}
    
   virtual void setInputOutput(const int& input, const int& output);

   Time getLocalTime(u_int64_t);
   
 protected:
   int _framesize;
   int _int_frame_rate;
   const int _frame_rate_base;

   
    string		__device;//device;
    int			__fd;//device_fd;

   float                       __frame_rate;        ///< Frame rate of current standard
   /* End< */

   /* Begin> Output */
   ssize_t                     __buffer_size;       ///< Read buffer size
   bool                        __do_flush;          ///< Need to flush the stream
   StreamTimer                 __stream_timer;      ///< Stream timer
   /* End< */

   /* Begin> Channels */
   unsigned int                __cur_channel;       ///< Current channel

  /* Begin> Buffers of device */
  struct buffer {
    void *start;
    size_t length;
  };
  struct buffer *buffers;
  unsigned int n_buffers;
        
    
    int width, height;
    int framesize;
    float frame_rate;
    int int_frame_rate;
    

    bool sendEvent; //status flag whether to send an event
    string filename;
    int fd;
    size_t buffer_size;
    long int seek;
    long int filesize;
    int _size;
    int _frame;
    int picture_size, offset1, offset2, uv_size;
    int grab_format;
    
    unsigned char* _mmap_buffer;
    video_capability _capability;
    video_buffer _vid_buffer;
    video_mbuf _vid_mbuf;
    video_mmap _vid_mmap;
    video_picture _vid_picture;
    video_window _vid_win;
    video_channel _vid_channel;
    video_tuner _vid_tuner;

    ThreadMutex mutex_v4l;

    Result setOutputFormat(int&);

    //    Result setFrquency();
    //    Result getFrequency();
    //    Result searchFrequency();


  };

}

#endif
