/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#ifndef NMM_WHITE_NOISE_NODE_HPP
#define NMM_WHITE_NOISE_NODE_HPP

#include "nmm/multimedia/GenericSourceNode.hpp"
#include "nmm/multimedia/sync/StreamTimer.hpp"

#include "nmm/utils/thread/ThreadCondition.hpp"
#include "nmm/utils/thread/ThreadMutex.hpp"
#include "nmm/utils/time/TimedElement.hpp"

namespace NMM {

  class WhiteNoiseNode : public GenericSourceNode, public TimedElement {
  private:
    StreamTimer* stimer;
    
    Buffer* produceRGBBuffer();
    Buffer* produceYUVBuffer();
    Buffer* produceYV12Buffer();
    
    int image_width;
    int image_height;
    int image_bpp;
    double image_fps;
    
    void addSupportedFormats(Property* property);

    Time            __last_timer;     ///< stores the latest timespamp
    ThreadMutex     __cond_mutex;     ///< ThreadMutex, required for the ThreadCondition
    ThreadCondition __condition;      ///< used to wait until next buffer must be send
  protected:
    virtual Message* processBuffer(Buffer*);
    
    virtual Result doInit();
    virtual Result doInitOutput();
    virtual Result doActivate();
    virtual Result doStop();
    void waitToSendBuffer();
    
  public:
    WhiteNoiseNode(const char* name = "WhiteNoiseNode",
                   StreamQueue::Mode = StreamQueue::MODE_SUSPEND,
                   int = StreamQueue::DEFAULT_SIZE);
    
    virtual ~WhiteNoiseNode();
  };
}

#endif
