/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ICompositeNode.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  void ICompositeNode::used() {} 
  
  
  nmm_uint32 ICompositeNode::addNode(const INode& node) /* limited(INode::CONSTRUCTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::TInValue<INode > node_value(node, false);
    NMM::Event event("ICompositeNode::addNode", &node_value);
    event.setReturnValue(&return_value);
    if(const_cast<ICompositeNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 ICompositeNode::addNodeWithInterfaces(const INode& node, const list<Interface* > & interfaces) /* limited(INode::CONSTRUCTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::TInValue<INode > node_value(node, false);
    NMM::TInValue<list<Interface* >  > interfaces_value(interfaces, false);
    NMM::Event event("ICompositeNode::addNodeWithInterfaces", &node_value, &interfaces_value);
    event.setReturnValue(&return_value);
    if(const_cast<ICompositeNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ICompositeNode::renameInputJackTag(const string& old_cn_jack_tag, const string& new_cn_jack_tag) /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > old_cn_jack_tag_value(old_cn_jack_tag, false);
    NMM::TInValue<string > new_cn_jack_tag_value(new_cn_jack_tag, false);
    NMM::Event event("ICompositeNode::renameInputJackTag", &old_cn_jack_tag_value, &new_cn_jack_tag_value);
    event.setReturnValue(&return_value);
    if(const_cast<ICompositeNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ICompositeNode::renameOutputJackTag(const string& old_cn_jack_tag, const string& new_cn_jack_tag) /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&)
  {
    NMM::TOutValue<Result > return_value;
    NMM::TInValue<string > old_cn_jack_tag_value(old_cn_jack_tag, false);
    NMM::TInValue<string > new_cn_jack_tag_value(new_cn_jack_tag, false);
    NMM::Event event("ICompositeNode::renameOutputJackTag", &old_cn_jack_tag_value, &new_cn_jack_tag_value);
    event.setReturnValue(&return_value);
    if(const_cast<ICompositeNode*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(NMM::IllegalStateException* ex = dynamic_cast<NMM::IllegalStateException*>(event.getException())) {
          throw NMM::IllegalStateException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ICompositeNode::ICompositeNode_type = "ICompositeNode";
  NMM::TConstructor<ICompositeNode > __ICompositeNode_prototype("ICompositeNode","NMM");


  const char* ICompositeNode::addNode_event = "ICompositeNode::addNode";
  const char* ICompositeNode::addNodeWithInterfaces_event = "ICompositeNode::addNodeWithInterfaces";
  const char* ICompositeNode::renameInputJackTag_event = "ICompositeNode::renameInputJackTag";
  const char* ICompositeNode::renameOutputJackTag_event = "ICompositeNode::renameOutputJackTag";


}


#include "nmm/nmm_suffix.hpp"

