/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_ICompositeNode_HPP
#define NMM_ICompositeNode_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/Interface.hpp"
#include "nmm/comm/messaging/Event.hpp"
#include "nmm/comm/messaging/IllegalStateException.hpp"
#include <memory>

#include "nmm/multimedia/INode.hpp"

#include "visibility.hpp"

namespace NMM
{

  class DLLEXPORT ICompositeNode : virtual public NMM::Interface { 
  public:
    ICompositeNode() : NMM::Interface(ICompositeNode::ICompositeNode_type, "NMM") { };

    ICompositeNode(NMM::NMMObject& parent) : NMM::Interface (parent, ICompositeNode::ICompositeNode_type, "NMM") { };

    virtual ~ICompositeNode() {};

    static const char* ICompositeNode_type;

    virtual  string getInterfaceType() const { return ICompositeNode_type; }

    NMM::Interface* copy() const { return new ICompositeNode(*this); }
    static void used(); 
    static const char* addNode_event;
    static const char* addNodeWithInterfaces_event;
    static const char* renameInputJackTag_event;
    static const char* renameOutputJackTag_event;

    virtual nmm_uint32 addNode(const INode& node) /* limited(INode::CONSTRUCTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);

    virtual nmm_uint32 addNodeWithInterfaces(const INode& node, const list<Interface* > & interfaces) /* limited(INode::CONSTRUCTED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);

    virtual Result renameInputJackTag(const string& old_cn_jack_tag, const string& new_cn_jack_tag) /* limited(INode::INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);

    virtual Result renameOutputJackTag(const string& old_cn_jack_tag, const string& new_cn_jack_tag) /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const NMM::SerializedException&, const NMM::IllegalStateException&);


  };

  typedef auto_ptr<ICompositeNode> ICompositeNode_var;




}

#include "nmm/nmm_suffix.hpp"

#endif
