/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_BINDING_DESCRIPTION_HPP
#define NMM_BINDING_DESCRIPTION_HPP NMM_BINDING_DESCRIPTION_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/serialize/SerializedValue.hpp"

#include "visibility.hpp"

namespace NMM {  

class BindingDescription;
  
  /** This abstract class describes a binding between to objects.
   * A binding is described by a vector of sub-bindings and the configuration
   * of the top-level transport strategy.
   *
   * This is an abstract base class of InstreamBindingDescription and OutOfBandBindingDescription.
   */
  class DLLEXPORT BindingDescription
    : public SerializedValue
  {
  public:
    /** @name Constructor & Destructor
     */
    //@{
    /** Default constructor.
     * @param ti include type information of this object. 
     */
    BindingDescription(const TypeInfo& ti);
    
    /** Copy constructor.
     * @param bd reference to BindingDescription that should be copied. 
     */
    BindingDescription(const BindingDescription& bd);
    
    /** Virtual copy constructor
     * @return pointer to a copy of this BindingDescription
     */
    virtual BindingDescription* copy() const = 0;
    //@}
  };  
}

#include "nmm/nmm_suffix.hpp"

#endif
