/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "DynamicLibrary_Linux.hpp"
#include "nmm/utils/NamedObject.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  DynamicLibrary::DynamicLibrary() : handle(0) {
    int retval = lt_dlinit();
    if (retval != 0) {
      GLOBAL_ERROR_STREAM(" error occured" << endl);
      __is_valid = false;
    } else {
      __is_valid = true;
    }
  }

  DynamicLibrary::~DynamicLibrary(){
    try {
      close();
    } catch (const Exception& e) {
      GLOBAL_ERROR_STREAM(e << endl);
    }
    lt_dlexit();
  }

  void DynamicLibrary::open(const string& n, int )
    throw (const Exception&)
  {
    if (handle) {
      try {
        close();
      } catch (const Exception& e) {
        GLOBAL_ERROR_STREAM(e << endl);
      }
    }

    name = n;
    handle = lt_dlopen(name.c_str());

    if (!handle) {
      throw Exception("Error while loading shared library '" + name +"'. Error message :" + lt_dlerror());
    }
  }

  void* DynamicLibrary::resolve(const string& symbol) {
    if (handle != 0)
      return lt_dlsym(handle, const_cast<char*> (symbol.c_str()));
    return 0;
  }

  void DynamicLibrary::close()
    throw (const Exception&)
  {
    if (!handle)
      return;

    if (lt_dlclose(handle) != 0) {
      handle = 0;
      throw Exception("Error while closing shared library '" + name +"'. Error message :" + lt_dlerror());
    }

    handle = 0;
  }

}

#include "nmm/nmm_suffix.hpp"
