/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_FACTORY1394_HPP
#define NMM_FACTORY1394_HPP

#include "nmm/nmm_prefix.hpp"


#include "nmm/services/registry/NodeFactory.hpp"
#include "nmm/utils/thread/Thread.hpp"
#include "nmm/utils/thread/ThreadMutex.hpp"

namespace L1394 {
  class Session;
}

namespace NMM{
class Camera1394;
class GenericFireWireNode;

class Factory1394 : public NodeFactory {
public:

/** @name Constructor & Destructor
  */
//@{
/** Constructor
  */
  Factory1394(NodeFactory* parent);

/** Destructor
  */
  virtual ~Factory1394();
//@}

/** @name factory methods
  * These methods can be used to create and destroy Nodes
  */
//@{
/** This method creates a Node from a given FactoryTicket. If the node can not be created for some reason
  * an exception is thrown. The FactoryTicket is creates by a Registry, if the node is reserved. After
  * this method the FactoryTicket is invalid and can not be used again.
  * \param ticket The ticket is created by a registry and allows the maintainer to create the node.
  * \return A refernce to the new Node
  */
  virtual  Node& createNode (const FactoryTicket& ticket)
    throw (const FactoryException& );

/** This method releases a creates node. The NodeFactory can delete this node or store it for a later use.
  * \param node The node that should be released
  */
  virtual  void  releaseNode (const Node& node)
    throw (const FactoryException& );
//@}

  virtual void registerPlugin(NMM::Plugin*) { } // Nothing to do

  virtual void unregisterPlugin(const NMM::Plugin*) { } // Nothing to do

protected:
  Camera1394& createCamera(const u_int64_t& )
    throw (const FactoryException&);

private:
  ThreadMutex mutex;                                      ///< Internal mutex
  list<GenericFireWireNode*> device_list;                 ///< Stores all devices that are available
  list<GenericFireWireNode*> used_list;                   ///< Stores the used devices
  L1394::Session* session;                                ///< Pointer to the static Session object
  bool is_init;                                           ///< Is true if it is initialized
};



}

#include "nmm/nmm_suffix.hpp"

#endif

