/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_FACTORYEXCEPTION_HPP
#define NMM_FACTORYEXCEPTION_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/serialize/SerializedException.hpp"

namespace NMM {
/** This type of Exception is thrown if a Node can not be instantiated. 
  */
class FactoryException : public SerializedException {
public:
  /** Constructor
   */
  FactoryException()
    : SerializedException("FactoryException") { setTypeName("FactoryException"); setComment("FactoryException");};
  
  /** Constructor.
   *
   * @param s The parameter can be used to set an error message
   */
  FactoryException(const string& s) : SerializedException(s) {};
  
  /** Constructor.
   * @param n Object that should be used for the output
   * @param s The parameter can be used to set an error message
   */
  FactoryException (const NamedObject* n, const string& s = "")
    : SerializedException(n, ((s == "") ? "FactoryException" : "FactoryException : " + s)) {setTypeName("FactoryException");};

  /** Virtual copy constructor.
   * @return A copy of this object. 
   */
  virtual FactoryException* copy() const {return new FactoryException(*this); }


};

}
#include "nmm/nmm_suffix.hpp"

#endif
