/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "FactoryTicket.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"
 
#include "nmm/nmm_prefix.hpp"

namespace NMM {

FactoryTicket::FactoryTicket()
  : SerializedValue(TypeInfo("FactoryTicket", "NMM")), __is_valid(false)
{
  registerAttributes();
}

FactoryTicket::FactoryTicket(const FactoryTicket& ft)
  : SerializedValue(ft)
{
  registerAttributes();
  *this = ft;
}

FactoryTicket& FactoryTicket::operator=(const FactoryTicket& ft){
  SerializedValue::operator=(ft);

  /* Begin> assign the values in this struct */
  __config_event = ft.__config_event;
  __factory_id   = ft.__factory_id;
  __node_id      = ft.__node_id;
  __is_valid     = ft.__is_valid;
  /* End< */

  /* If you copy a ticket the old one is invalid  */
  ft.__is_valid = false;
  return *this;
}

void FactoryTicket::registerAttributes() {
  /* Begin> register the attributes */
  insertArgument(&__factory_id);
  insertArgument(&__node_id);
  insertArgument(&__is_valid);
  insertArgument(&__config_event);
  /* End< */
}

/* TConstructor to register both classes in the ValueFactory */
TConstructor<FactoryTicket>    __nmm__factory_ticket_id("FactoryTicket", "NMM");

}//end namespace

#include "nmm/nmm_suffix.hpp"
