/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IPluginManagerImpl.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/messaging/TEDObject.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  
  void IPluginManagerImpl::registerEvents()
  {
  
    getEventDispatcher().registerEvent(IPluginManager::addPluginPath_event, new NMM::TEDObject1<IPluginManagerImpl, string, void(IPluginManagerImpl:: * )(const string&), void >(this, &IPluginManagerImpl::addPluginPath));
  
    getEventDispatcher().registerEvent(IPluginManager::loadLibrary_event, new NMM::TEDObject1<IPluginManagerImpl, string, void(IPluginManagerImpl:: * )(const string&), void >(this, &IPluginManagerImpl::loadLibrary));
  
    getEventDispatcher().registerEvent(IPluginManager::loadPlugin_event, new NMM::TEDObject1<IPluginManagerImpl, string, void(IPluginManagerImpl:: * )(const string&), void >(this, &IPluginManagerImpl::loadPlugin));
  
    getEventDispatcher().registerEvent(IPluginManager::loadExternalPlugins_event, new NMM::TEDObject1<IPluginManagerImpl, string, void(IPluginManagerImpl:: * )(const string&), void >(this, &IPluginManagerImpl::loadExternalPlugins));
  
    getEventDispatcher().registerEvent(IPluginManager::removeExternalPlugins_event, new NMM::TEDObject1<IPluginManagerImpl, string, void(IPluginManagerImpl:: * )(const string&), void >(this, &IPluginManagerImpl::removeExternalPlugins));
  
    getEventDispatcher().registerEvent(IPluginManager::activateRegistryPart_event, new NMM::TEDObject1<IPluginManagerImpl, string, void(IPluginManagerImpl:: * )(const string&), void >(this, &IPluginManagerImpl::activateRegistryPart));
  
    getEventDispatcher().registerEvent(IPluginManager::activateFactoryPart_event, new NMM::TEDObject1<IPluginManagerImpl, string, void(IPluginManagerImpl:: * )(const string&), void >(this, &IPluginManagerImpl::activateFactoryPart));
  
    getEventDispatcher().registerEvent(IPluginManager::deactivateRegistryPart_event, new NMM::TEDObject1<IPluginManagerImpl, string, void(IPluginManagerImpl:: * )(const string&), void >(this, &IPluginManagerImpl::deactivateRegistryPart));
  
    getEventDispatcher().registerEvent(IPluginManager::deactivateFactoryPart_event, new NMM::TEDObject1<IPluginManagerImpl, string, void(IPluginManagerImpl:: * )(const string&), void >(this, &IPluginManagerImpl::deactivateFactoryPart));
  
    getEventDispatcher().registerEvent(IPluginManager::removeLibrary_event, new NMM::TEDObject1<IPluginManagerImpl, string, void(IPluginManagerImpl:: * )(const string&), void >(this, &IPluginManagerImpl::removeLibrary));
  
    getEventDispatcher().registerEvent(IPluginManager::getPluginID_event, new NMM::TEDObject2<IPluginManagerImpl, string, nmm_uint32, list<ObjectID > (IPluginManagerImpl:: * )(const string&, const nmm_uint32&), list<ObjectID >  >(this, &IPluginManagerImpl::getPluginID));
  
    getEventDispatcher().registerEvent(IPluginManager::getPlugins_event, new NMM::TEDObject0<IPluginManagerImpl, list<string > (IPluginManagerImpl:: * )() const, list<string >  >(this, &IPluginManagerImpl::getPlugins));
  
    getEventDispatcher().registerEvent(IPluginManager::getUsedPlugins_event, new NMM::TEDObject0<IPluginManagerImpl, list<string > (IPluginManagerImpl:: * )() const, list<string >  >(this, &IPluginManagerImpl::getUsedPlugins));
  
    getEventDispatcher().registerEvent(IPluginManager::getPluginPath_event, new NMM::TEDObject0<IPluginManagerImpl, list<string > (IPluginManagerImpl:: * )() const, list<string >  >(this, &IPluginManagerImpl::getPluginPath));

  }

}


#include "nmm/nmm_suffix.hpp"

