/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IRegistryImpl.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/messaging/TEDObject.hpp"

#include "nmm/nmm_prefix.hpp"


namespace NMM
{
  
  void IRegistryImpl::registerEvents()
  {
  
    getEventDispatcher().registerEvent(IRegistry::requestNode_event, new NMM::TEDObject4<IRegistryImpl, RegistryID, ObjectID, UserInfo, NodeDescription, INode*(IRegistryImpl:: * )(const RegistryID&, const ObjectID&, const UserInfo&, NodeDescription&), INode* >(this, &IRegistryImpl::requestNode));
  
    getEventDispatcher().registerEvent(IRegistry::requestObject_event, new NMM::TEDObject3<IRegistryImpl, RegistryID, ObjectID, UserInfo, Interface*(IRegistryImpl:: * )(const RegistryID&, const ObjectID&, const UserInfo&), Interface* >(this, &IRegistryImpl::requestObject));
  
    getEventDispatcher().registerEvent(IRegistry::releaseNode_event, new NMM::TEDObject4<IRegistryImpl, RegistryID, NMMReference, ObjectID, UserInfo, void(IRegistryImpl:: * )(const RegistryID&, const NMMReference&, const ObjectID&, const UserInfo&), void >(this, &IRegistryImpl::releaseNode));
  
    getEventDispatcher().registerEvent(IRegistry::releaseObject_event, new NMM::TEDObject4<IRegistryImpl, RegistryID, NMMReference, ObjectID, UserInfo, void(IRegistryImpl:: * )(const RegistryID&, const NMMReference&, const ObjectID&, const UserInfo&), void >(this, &IRegistryImpl::releaseObject));
  
    getEventDispatcher().registerEvent(IRegistry::initRequest_event, new NMM::TEDObject1<IRegistryImpl, NodeDescription, list<Response > (IRegistryImpl:: * )(const NodeDescription&) const, list<Response >  >(this, &IRegistryImpl::initRequest));
  
    getEventDispatcher().registerEvent(IRegistry::initObjectRequest_event, new NMM::TEDObject1<IRegistryImpl, ObjectDescription, list<Response > (IRegistryImpl:: * )(const ObjectDescription&) const, list<Response >  >(this, &IRegistryImpl::initObjectRequest));
  
    getEventDispatcher().registerEvent(IRegistry::searchNode_event, new NMM::TEDObject2<IRegistryImpl, RegistryID, NodeDescription, list<ObjectID > (IRegistryImpl:: * )(const RegistryID&, const NodeDescription&) const, list<ObjectID >  >(this, &IRegistryImpl::searchNode));
  
    getEventDispatcher().registerEvent(IRegistry::reserveNode_event, new NMM::TEDObject3<IRegistryImpl, RegistryID, ObjectID, UserInfo, FactoryTicket(IRegistryImpl:: * )(const RegistryID&, const ObjectID&, const UserInfo&), FactoryTicket >(this, &IRegistryImpl::reserveNode));
  
    getEventDispatcher().registerEvent(IRegistry::instantiateNode_event, new NMM::TEDObject3<IRegistryImpl, RegistryID, FactoryTicket, UserInfo, INode*(IRegistryImpl:: * )(const RegistryID&, const FactoryTicket&, const UserInfo&), INode* >(this, &IRegistryImpl::instantiateNode));
  
    getEventDispatcher().registerEvent(IRegistry::releaseReservation_event, new NMM::TEDObject3<IRegistryImpl, RegistryID, ObjectID, UserInfo, void(IRegistryImpl:: * )(const RegistryID&, const ObjectID&, const UserInfo&), void >(this, &IRegistryImpl::releaseReservation));
  
    getEventDispatcher().registerEvent(IRegistry::destroyNode_event, new NMM::TEDObject3<IRegistryImpl, RegistryID, NMMReference, UserInfo, void(IRegistryImpl:: * )(const RegistryID&, const NMMReference&, const UserInfo&), void >(this, &IRegistryImpl::destroyNode));
  
    getEventDispatcher().registerEvent(IRegistry::getMaxInstanceCount_event, new NMM::TEDObject2<IRegistryImpl, RegistryID, ObjectID, nmm_int32(IRegistryImpl:: * )(const RegistryID&, const ObjectID&) const, nmm_int32 >(this, &IRegistryImpl::getMaxInstanceCount));
  
    getEventDispatcher().registerEvent(IRegistry::getInstanceCount_event, new NMM::TEDObject2<IRegistryImpl, RegistryID, ObjectID, nmm_int32(IRegistryImpl:: * )(const RegistryID&, const ObjectID&) const, nmm_int32 >(this, &IRegistryImpl::getInstanceCount));
  
    getEventDispatcher().registerEvent(IRegistry::getPluginInfo_event, new NMM::TEDObject2<IRegistryImpl, RegistryID, ObjectID, string(IRegistryImpl:: * )(const RegistryID&, const ObjectID&) const, string >(this, &IRegistryImpl::getPluginInfo));
  
    getEventDispatcher().registerEvent(IRegistry::getNodeList_event, new NMM::TEDObject1<IRegistryImpl, RegistryID, list<ObjectID > (IRegistryImpl:: * )(const RegistryID&) const, list<ObjectID >  >(this, &IRegistryImpl::getNodeList));
  
    getEventDispatcher().registerEvent(IRegistry::getReservedNodeList_event, new NMM::TEDObject1<IRegistryImpl, RegistryID, list<ObjectID > (IRegistryImpl:: * )(const RegistryID&) const, list<ObjectID >  >(this, &IRegistryImpl::getReservedNodeList));
  
    getEventDispatcher().registerEvent(IRegistry::getNodeInfoList_event, new NMM::TEDObject1<IRegistryImpl, RegistryID, list<NodeInfo > (IRegistryImpl:: * )(const RegistryID&) const, list<NodeInfo >  >(this, &IRegistryImpl::getNodeInfoList));
  
    getEventDispatcher().registerEvent(IRegistry::getNodeDescription_event, new NMM::TEDObject2<IRegistryImpl, RegistryID, ObjectID, NodeDescription(IRegistryImpl:: * )(const RegistryID&, const ObjectID&) const, NodeDescription >(this, &IRegistryImpl::getNodeDescription));
  
    getEventDispatcher().registerEvent(IRegistry::getNodeDescriptionList_event, new NMM::TEDObject1<IRegistryImpl, RegistryID, list<NodeDescription > (IRegistryImpl:: * )(const RegistryID&), list<NodeDescription >  >(this, &IRegistryImpl::getNodeDescriptionList));
  
    getEventDispatcher().registerEvent(IRegistry::getNodeFactoryID_event, new NMM::TEDObject2<IRegistryImpl, RegistryID, ObjectID, NodeFactoryID(IRegistryImpl:: * )(const RegistryID&, const ObjectID&) const, NodeFactoryID >(this, &IRegistryImpl::getNodeFactoryID));
  
    getEventDispatcher().registerEvent(IRegistry::resetInternalReservation_event, new NMM::TEDObject0<IRegistryImpl, void(IRegistryImpl:: * )(), void >(this, &IRegistryImpl::resetInternalReservation));
  
    getEventDispatcher().registerEvent(IRegistry::resetSpecificInternalReservation_event, new NMM::TEDObject1<IRegistryImpl, UserInfo, void(IRegistryImpl:: * )(const UserInfo&), void >(this, &IRegistryImpl::resetSpecificInternalReservation));
  
    getEventDispatcher().registerEvent(IRegistry::initTransaction_event, new NMM::TEDObject1<IRegistryImpl, UserInfo, void(IRegistryImpl:: * )(const UserInfo&), void >(this, &IRegistryImpl::initTransaction));
  
    getEventDispatcher().registerEvent(IRegistry::closeTransaction_event, new NMM::TEDObject1<IRegistryImpl, UserInfo, void(IRegistryImpl:: * )(const UserInfo&), void >(this, &IRegistryImpl::closeTransaction));
  
    getEventDispatcher().registerEvent(IRegistry::getRegistryID_event, new NMM::TEDObject0<IRegistryImpl, RegistryID(IRegistryImpl:: * )() const, RegistryID >(this, &IRegistryImpl::getRegistryID));
  
    getEventDispatcher().registerEvent(IRegistry::getChildList_event, new NMM::TEDObject1<IRegistryImpl, RegistryID, list<RegistryID > (IRegistryImpl:: * )(const RegistryID&) const, list<RegistryID >  >(this, &IRegistryImpl::getChildList));
  
    getEventDispatcher().registerEvent(IRegistry::isSubRegistry_event, new NMM::TEDObject1<IRegistryImpl, RegistryID, nmm_bool(IRegistryImpl:: * )(const RegistryID&) const, nmm_bool >(this, &IRegistryImpl::isSubRegistry));
  
    getEventDispatcher().registerEvent(IRegistry::getMemento_event, new NMM::TEDObject0<IRegistryImpl, RegistryMemento(IRegistryImpl:: * )(), RegistryMemento >(this, &IRegistryImpl::getMemento));
  
    getEventDispatcher().registerEvent(IRegistry::setMemento_event, new NMM::TEDObject1<IRegistryImpl, RegistryMemento, void(IRegistryImpl:: * )(const RegistryMemento&), void >(this, &IRegistryImpl::setMemento));

  }

}


#include "nmm/nmm_suffix.hpp"

