/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IServerRegistry.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/utils/time/Time.hpp"
#include "nmm/comm/serialize/SerializeUtilsTime.hpp"

namespace NMM
{
  void IServerRegistry::used() {} 
  
  
  Time IServerRegistry::getSystemTime()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Time > return_value;
    NMM::Event event("IServerRegistry::getSystemTime");
    event.setReturnValue(&return_value);
    if(const_cast<IServerRegistry*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 IServerRegistry::getTransactionTimeout()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("IServerRegistry::getTransactionTimeout");
    event.setReturnValue(&return_value);
    if(const_cast<IServerRegistry*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IServerRegistry::setTransactionTimeout(const nmm_uint32& time_out)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > time_out_value(time_out, false);
    NMM::Event event("IServerRegistry::setTransactionTimeout", &time_out_value);
    if(const_cast<IServerRegistry*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_uint32 IServerRegistry::getNodeLeasingTime()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::Event event("IServerRegistry::getNodeLeasingTime");
    event.setReturnValue(&return_value);
    if(const_cast<IServerRegistry*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IServerRegistry::setNodeLeasingTime(const nmm_uint32& time_out)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<nmm_uint32 > time_out_value(time_out, false);
    NMM::Event event("IServerRegistry::setNodeLeasingTime", &time_out_value);
    if(const_cast<IServerRegistry*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  nmm_uint32 IServerRegistry::refreshNodeLeasing(const RegistryID& reg_id, const ObjectID& node_id, const UserInfo& user_info)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::TInValue<RegistryID > reg_id_value(reg_id, false);
    NMM::TInValue<ObjectID > node_id_value(node_id, false);
    NMM::TInValue<UserInfo > user_info_value(user_info, false);
    NMM::Event event("IServerRegistry::refreshNodeLeasing", &reg_id_value, &node_id_value, &user_info_value);
    event.setReturnValue(&return_value);
    if(const_cast<IServerRegistry*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  nmm_uint32 IServerRegistry::refreshTransaction(const UserInfo& user_info)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<nmm_uint32 > return_value;
    NMM::TInValue<UserInfo > user_info_value(user_info, false);
    NMM::Event event("IServerRegistry::refreshTransaction", &user_info_value);
    event.setReturnValue(&return_value);
    if(const_cast<IServerRegistry*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Interface* IServerRegistry::requestNMMObject(const ObjectDescription& od)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<Interface* > return_value;
    NMM::TInValue<ObjectDescription > od_value(od, false);
    NMM::Event event("IServerRegistry::requestNMMObject", &od_value);
    event.setReturnValue(&return_value);
    if(const_cast<IServerRegistry*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IServerRegistry::releaseNMMObject(const Interface& i_face)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<Interface > i_face_value(i_face, false);
    NMM::Event event("IServerRegistry::releaseNMMObject", &i_face_value);
    if(const_cast<IServerRegistry*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  ISession* IServerRegistry::requestSharedGraph(GraphDescription& gd, const UserInfo& user_info) 
      throw(const NMM::SerializedException&, const RegistryException&)
  {
    NMM::TOutValue<ISession* > return_value;
    NMM::TInOutValue<GraphDescription > gd_value;
    gd_value.setValue(gd);
    NMM::TInValue<UserInfo > user_info_value(user_info, false);
    NMM::Event event("IServerRegistry::requestSharedGraph", &gd_value, &user_info_value);
    event.setReturnValue(&return_value);
    if(const_cast<IServerRegistry*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        if(RegistryException* ex = dynamic_cast<RegistryException*>(event.getException())) {
          throw RegistryException(*ex);
        }
        throw NMM::SerializedException(*event.getException());
      }
    }
    gd = gd_value.getRValue();
    return return_value.getRValue();
  }
  
  void IServerRegistry::releaseSession(const ISession& session, const UserInfo& user_info)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<ISession > session_value(session, false);
    NMM::TInValue<UserInfo > user_info_value(user_info, false);
    NMM::Event event("IServerRegistry::releaseSession", &session_value, &user_info_value);
    if(const_cast<IServerRegistry*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void IServerRegistry::releaseSharedNode(const INode& node)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<INode > node_value(node, false);
    NMM::Event event("IServerRegistry::releaseSharedNode", &node_value);
    if(const_cast<IServerRegistry*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  ISession* IServerRegistry::initSession(GraphDescription& gd, const UserInfo& u_info, const ISession* given_session)
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ISession* > return_value;
    NMM::TInOutValue<GraphDescription > gd_value;
    gd_value.setValue(gd);
    NMM::TInValue<UserInfo > u_info_value(u_info, false);
    NMM::TInValue<ISession* > given_session_value (const_cast<ISession* >(given_session), false);
    NMM::Event event("IServerRegistry::initSession", &gd_value, &u_info_value, &given_session_value);
    event.setReturnValue(&return_value);
    if(const_cast<IServerRegistry*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    gd = gd_value.getRValue();
    return return_value.getRValue();
  }
  
  void IServerRegistry::requestGraph(GraphDescription& gd, const UserInfo& user_info)
      throw(const NMM::SerializedException&)
  {
    NMM::TInOutValue<GraphDescription > gd_value;
    gd_value.setValue(gd);
    NMM::TInValue<UserInfo > user_info_value(user_info, false);
    NMM::Event event("IServerRegistry::requestGraph", &gd_value, &user_info_value);
    if(const_cast<IServerRegistry*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    gd = gd_value.getRValue();
  }
  
  void IServerRegistry::sendIServerRegistry()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("IServerRegistry::sendIServerRegistry");
    if(const_cast<IServerRegistry*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* IServerRegistry::IServerRegistry_type = "IServerRegistry";
  NMM::TConstructor<IServerRegistry > __IServerRegistry_prototype("IServerRegistry","NMM");


  const char* IServerRegistry::getSystemTime_event = "IServerRegistry::getSystemTime";
  const char* IServerRegistry::getTransactionTimeout_event = "IServerRegistry::getTransactionTimeout";
  const char* IServerRegistry::setTransactionTimeout_event = "IServerRegistry::setTransactionTimeout";
  const char* IServerRegistry::getNodeLeasingTime_event = "IServerRegistry::getNodeLeasingTime";
  const char* IServerRegistry::setNodeLeasingTime_event = "IServerRegistry::setNodeLeasingTime";
  const char* IServerRegistry::refreshNodeLeasing_event = "IServerRegistry::refreshNodeLeasing";
  const char* IServerRegistry::refreshTransaction_event = "IServerRegistry::refreshTransaction";
  const char* IServerRegistry::requestNMMObject_event = "IServerRegistry::requestNMMObject";
  const char* IServerRegistry::releaseNMMObject_event = "IServerRegistry::releaseNMMObject";
  const char* IServerRegistry::requestSharedGraph_event = "IServerRegistry::requestSharedGraph";
  const char* IServerRegistry::releaseSession_event = "IServerRegistry::releaseSession";
  const char* IServerRegistry::releaseSharedNode_event = "IServerRegistry::releaseSharedNode";
  const char* IServerRegistry::initSession_event = "IServerRegistry::initSession";
  const char* IServerRegistry::requestGraph_event = "IServerRegistry::requestGraph";
  const char* IServerRegistry::sendIServerRegistry_event = "IServerRegistry::sendIServerRegistry";


}


#include "nmm/nmm_suffix.hpp"

