/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "IServerRegistryImpl.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/messaging/TEDObject.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "nmm/utils/time/Time.hpp"
#include "nmm/comm/serialize/SerializeUtilsTime.hpp"

namespace NMM
{
  
  void IServerRegistryImpl::registerEvents()
  {
  
    getEventDispatcher().registerEvent(IServerRegistry::getSystemTime_event, new NMM::TEDObject0<IServerRegistryImpl, Time(IServerRegistryImpl:: * )(), Time >(this, &IServerRegistryImpl::getSystemTime));
  
    getEventDispatcher().registerEvent(IServerRegistry::getTransactionTimeout_event, new NMM::TEDObject0<IServerRegistryImpl, nmm_uint32(IServerRegistryImpl:: * )(), nmm_uint32 >(this, &IServerRegistryImpl::getTransactionTimeout));
  
    getEventDispatcher().registerEvent(IServerRegistry::setTransactionTimeout_event, new NMM::TEDObject1<IServerRegistryImpl, nmm_uint32, void(IServerRegistryImpl:: * )(const nmm_uint32&), void >(this, &IServerRegistryImpl::setTransactionTimeout));
  
    getEventDispatcher().registerEvent(IServerRegistry::getNodeLeasingTime_event, new NMM::TEDObject0<IServerRegistryImpl, nmm_uint32(IServerRegistryImpl:: * )(), nmm_uint32 >(this, &IServerRegistryImpl::getNodeLeasingTime));
  
    getEventDispatcher().registerEvent(IServerRegistry::setNodeLeasingTime_event, new NMM::TEDObject1<IServerRegistryImpl, nmm_uint32, void(IServerRegistryImpl:: * )(const nmm_uint32&), void >(this, &IServerRegistryImpl::setNodeLeasingTime));
  
    getEventDispatcher().registerEvent(IServerRegistry::refreshNodeLeasing_event, new NMM::TEDObject3<IServerRegistryImpl, RegistryID, ObjectID, UserInfo, nmm_uint32(IServerRegistryImpl:: * )(const RegistryID&, const ObjectID&, const UserInfo&), nmm_uint32 >(this, &IServerRegistryImpl::refreshNodeLeasing));
  
    getEventDispatcher().registerEvent(IServerRegistry::refreshTransaction_event, new NMM::TEDObject1<IServerRegistryImpl, UserInfo, nmm_uint32(IServerRegistryImpl:: * )(const UserInfo&), nmm_uint32 >(this, &IServerRegistryImpl::refreshTransaction));
  
    getEventDispatcher().registerEvent(IServerRegistry::requestNMMObject_event, new NMM::TEDObject1<IServerRegistryImpl, ObjectDescription, Interface*(IServerRegistryImpl:: * )(const ObjectDescription&), Interface* >(this, &IServerRegistryImpl::requestNMMObject));
  
    getEventDispatcher().registerEvent(IServerRegistry::releaseNMMObject_event, new NMM::TEDObject1<IServerRegistryImpl, Interface, void(IServerRegistryImpl:: * )(const Interface&), void >(this, &IServerRegistryImpl::releaseNMMObject));
  
    getEventDispatcher().registerEvent(IServerRegistry::requestSharedGraph_event, new NMM::TEDObject2<IServerRegistryImpl, GraphDescription, UserInfo, ISession*(IServerRegistryImpl:: * )(GraphDescription&, const UserInfo&), ISession* >(this, &IServerRegistryImpl::requestSharedGraph));
  
    getEventDispatcher().registerEvent(IServerRegistry::releaseSession_event, new NMM::TEDObject2<IServerRegistryImpl, ISession, UserInfo, void(IServerRegistryImpl:: * )(const ISession&, const UserInfo&), void >(this, &IServerRegistryImpl::releaseSession));
  
    getEventDispatcher().registerEvent(IServerRegistry::releaseSharedNode_event, new NMM::TEDObject1<IServerRegistryImpl, INode, void(IServerRegistryImpl:: * )(const INode&), void >(this, &IServerRegistryImpl::releaseSharedNode));
  
    getEventDispatcher().registerEvent(IServerRegistry::initSession_event, new NMM::TEDObject3<IServerRegistryImpl, GraphDescription, UserInfo, ISession*, ISession*(IServerRegistryImpl:: * )(GraphDescription&, const UserInfo&, const ISession*), ISession* >(this, &IServerRegistryImpl::initSession));
  
    getEventDispatcher().registerEvent(IServerRegistry::requestGraph_event, new NMM::TEDObject2<IServerRegistryImpl, GraphDescription, UserInfo, void(IServerRegistryImpl:: * )(GraphDescription&, const UserInfo&), void >(this, &IServerRegistryImpl::requestGraph));
  
    getEventDispatcher().registerEvent(IServerRegistry::sendIServerRegistry_event, new NMM::TEDObject0<IServerRegistryImpl, void(IServerRegistryImpl:: * )(), void >(this, &IServerRegistryImpl::sendIServerRegistry));

  }

}


#include "nmm/nmm_suffix.hpp"

