/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ISession.hpp"
#include "nmm/comm/serialize/SerializeResult.hpp"
#include "nmm/comm/serialize/InvalidTypeException.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *
 * See file LICENCE for terms and conditions of usage
 *
 */

#include "GraphDescription.hpp"

namespace NMM
{
  void ISession::used() {} 
  
  
  GraphDescription* ISession::getGraphDescription() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<GraphDescription* > return_value;
    NMM::Event event("ISession::getGraphDescription");
    event.setReturnValue(&return_value);
    if(const_cast<ISession*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ISourceSynchronizer* ISession::getSourceSynchronizer() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ISourceSynchronizer* > return_value;
    NMM::Event event("ISession::getSourceSynchronizer");
    event.setReturnValue(&return_value);
    if(const_cast<ISession*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ISinkSynchronizer* ISession::getSinkSynchronizer() const
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<ISinkSynchronizer* > return_value;
    NMM::Event event("ISession::getSinkSynchronizer");
    event.setReturnValue(&return_value);
    if(const_cast<ISession*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ISession::setSinkSynchronizer(const ISinkSynchronizer& synchronizer)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<ISinkSynchronizer > synchronizer_value(synchronizer, false);
    NMM::Event event("ISession::setSinkSynchronizer", &synchronizer_value);
    if(const_cast<ISession*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ISession::setSourceSynchronizer(const ISourceSynchronizer& synchronizer)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<ISourceSynchronizer > synchronizer_value(synchronizer, false);
    NMM::Event event("ISession::setSourceSynchronizer", &synchronizer_value);
    if(const_cast<ISession*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ISession::setGraphDescription(const GraphDescription& gd)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<GraphDescription > gd_value(gd, false);
    NMM::Event event("ISession::setGraphDescription", &gd_value);
    if(const_cast<ISession*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  GraphDescription* ISession::getMasterGraph()
      throw(const NMM::SerializedException&)
  {
    NMM::TOutValue<GraphDescription* > return_value;
    NMM::Event event("ISession::getMasterGraph");
    event.setReturnValue(&return_value);
    if(const_cast<ISession*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ISession::setMasterGraph(const GraphDescription& master_graph)
      throw(const NMM::SerializedException&)
  {
    NMM::TInValue<GraphDescription > master_graph_value(master_graph, false);
    NMM::Event event("ISession::setMasterGraph", &master_graph_value);
    if(const_cast<ISession*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ISession::realizeGraph()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("ISession::realizeGraph");
    if(const_cast<ISession*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ISession::activateGraph()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("ISession::activateGraph");
    if(const_cast<ISession*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ISession::startGraph()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("ISession::startGraph");
    if(const_cast<ISession*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }
  
  void ISession::stopGraph()
      throw(const NMM::SerializedException&)
  {
    NMM::Event event("ISession::stopGraph");
    if(const_cast<ISession*>(this)->sendEvent(event) == NMM::FAILURE) {
      if (event.getException()) {
        throw NMM::SerializedException(*event.getException());
      }
    }
  }

  const char* ISession::ISession_type = "ISession";
  NMM::TConstructor<ISession > __ISession_prototype("ISession","NMM");


  const char* ISession::getGraphDescription_event = "ISession::getGraphDescription";
  const char* ISession::getSourceSynchronizer_event = "ISession::getSourceSynchronizer";
  const char* ISession::getSinkSynchronizer_event = "ISession::getSinkSynchronizer";
  const char* ISession::setSinkSynchronizer_event = "ISession::setSinkSynchronizer";
  const char* ISession::setSourceSynchronizer_event = "ISession::setSourceSynchronizer";
  const char* ISession::setGraphDescription_event = "ISession::setGraphDescription";
  const char* ISession::getMasterGraph_event = "ISession::getMasterGraph";
  const char* ISession::setMasterGraph_event = "ISession::setMasterGraph";
  const char* ISession::realizeGraph_event = "ISession::realizeGraph";
  const char* ISession::activateGraph_event = "ISession::activateGraph";
  const char* ISession::startGraph_event = "ISession::startGraph";
  const char* ISession::stopGraph_event = "ISession::stopGraph";


}


#include "nmm/nmm_suffix.hpp"

