/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "InstreamBindingDescription.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"
#include "nmm/comm/IInputJack.hpp"
#include "nmm/comm/IOutputJack.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
 
  InstreamBindingDescription::InstreamBindingDescription()
    : BindingDescription(TypeInfo("InstreamBindingDescription", "NMM")),
      __strategies(),
      __upstream_top_level_config(),
      __downstream_top_level_config()
  {
    /* don't use TValue wrappers */
    __strategies.useTValue(false);
    
    registerAttributes();
  }
  
  InstreamBindingDescription::InstreamBindingDescription(const InstreamBindingDescription& other)
    : BindingDescription(other),
      __strategies(other.__strategies),
      __upstream_top_level_config(other.__upstream_top_level_config),
      __downstream_top_level_config(other.__downstream_top_level_config)
  {
    registerAttributes();
  }
  
  void InstreamBindingDescription::registerAttributes() {
    insertArgument(&__strategies);
    insertArgument(&__upstream_top_level_config);
    insertArgument(&__downstream_top_level_config);
  }
  
  unsigned int InstreamBindingDescription::addTransportStrategy(const string& strategy_type, unsigned int roles) {
    InstreamStrategyDescription* desc = new InstreamStrategyDescription();
    desc->setStrategyType(strategy_type);
    desc->setRoles(roles);
    desc->setExclusive(false);
    
    __strategies.push_back(desc);
    return __strategies.size() - 1;
  }
  
  unsigned int InstreamBindingDescription::addExclusiveTransportStrategy(const string& strategy_type, unsigned int roles) {
    InstreamStrategyDescription* desc = new InstreamStrategyDescription();
    desc->setStrategyType(strategy_type);
    desc->setRoles(roles);
    desc->setExclusive(true);
    
    __strategies.push_back(desc);
    return __strategies.size() - 1;
  }
  
  void InstreamBindingDescription::addUpstreamTopLevelConfigEvent(const Event& event) {
    __upstream_top_level_config.insertEvent(event);
  }
  
  void InstreamBindingDescription::addDownstreamTopLevelConfigEvent(const Event& event) {
    __downstream_top_level_config.insertEvent(event);
  }
  
  void InstreamBindingDescription::addUpstreamConfigEvent(const Event& event, InstreamChannel::CommunicationRole role) {
    for(unsigned int i=0; i<__strategies.size(); ++i) {
      if(__strategies[i]->getRoles() & role) {
        __strategies[i]->addUpstreamConfigEvent(event);
        return;
      }
    }
    
    GLOBAL_WARNING_STREAM("InstreamBindingDescription: No strategy description with the given roles exists, can't add config event" << endl);
  }
  
  void InstreamBindingDescription::addDownstreamConfigEvent(const Event& event, InstreamChannel::CommunicationRole role) {
    for(unsigned int i=0; i<__strategies.size(); ++i) {
      if(__strategies[i]->getRoles() & role) {
        __strategies[i]->addDownstreamConfigEvent(event);
        return;
      }
    }
    
    GLOBAL_WARNING_STREAM("InstreamBindingDescription: No strategy description with the given roles exists, can't add config event" << endl);
  }
  
  bool InstreamBindingDescription::isImplicit() {
    return __strategies.empty() &&
      __upstream_top_level_config.size() == 0 &&
      __downstream_top_level_config.size() == 0;
  }
  
  static TConstructor<InstreamBindingDescription> __instream_binding_description(TypeInfo("InstreamBindingDescription", "NMM"));
}

#include "nmm/nmm_suffix.hpp"
