/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "InstreamStrategyDescription.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
   
  InstreamStrategyDescription::InstreamStrategyDescription()
    : StrategyDescription(TypeInfo("InstreamStrategyDescription", "NMM")),
      __upstream_config(),
      __downstream_config()
  {
    registerAttributes();
  }
  
  InstreamStrategyDescription::InstreamStrategyDescription(const InstreamStrategyDescription& other)
    : StrategyDescription(other),
      __upstream_config(other.__upstream_config),
      __downstream_config(other.__downstream_config)
  {
    registerAttributes();
  }
  
  void InstreamStrategyDescription::addUpstreamConfigEvent(const Event& event) {
    __upstream_config.insertEvent(event);
  }
  
  void InstreamStrategyDescription::addDownstreamConfigEvent(const Event& event) {
    __downstream_config.insertEvent(event);
  }
  
  void InstreamStrategyDescription::registerAttributes() {
    insertArgument(&__upstream_config);
    insertArgument(&__downstream_config);
  }
  
  static TConstructor<InstreamStrategyDescription> __instream_strategy_description(TypeInfo("InstreamStrategyDescription", "NMM"));
  
}

#include "nmm/nmm_suffix.hpp"
