/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "NodeFactoryID.hpp"
#include "nmm/comm/ProcessUserInfo.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"
 
#include "nmm/nmm_prefix.hpp"

namespace NMM {

NodeFactoryID::NodeFactoryID(const string& fac_name)
  : SerializedValue(TypeInfo("NodeFactoryID", "NMM"))
{
  __hostname = SProcessUserInfo::getInstance().getHostname();
  __fac_name = fac_name;
  registerAttributes();
}

NodeFactoryID::NodeFactoryID(const NodeFactoryID& f_id)
  : SerializedValue(f_id)
{
  registerAttributes();
  *this = f_id;
}

void NodeFactoryID::registerAttributes() {
  insertArgument(&__hostname);
  insertArgument(&__fac_name);
}

NodeFactoryID& NodeFactoryID::operator=(const NodeFactoryID& id) {
  SerializedValue::operator=(id);

  __hostname = id.__hostname;
  __fac_name = id.__fac_name;
  return *this;
}


bool NodeFactoryID::operator==(const NodeFactoryID& id) const {
  if (getFactoryName() != id.getFactoryName())
    return false;

  return true;
}
TConstructor<NodeFactoryID>    __nmm__nodefactory_id("NodeFactoryID", "NMM");
}

#include "nmm/nmm_suffix.hpp"
