/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "NodeInfo.hpp"
#include "nmm/comm/serialize/StdValue.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"
#include "nmm/comm/serialize/TValueConstructor.hpp"

 
#include "nmm/nmm_prefix.hpp"

namespace NMM {

NodeInfo::NodeInfo()
  :  SerializedValue(TypeInfo("NodeInfo", "NMM"))
{
  registerAttributes();
}

NodeInfo::NodeInfo(const ObjectID& node_id, const string& lib_name, const string& node_name, int inst_count, int max_count, int nt)
  : SerializedValue(TypeInfo("NodeInfo", "NMM")), __node_id(node_id), __library_name(lib_name),
    __node_name(node_name), __instance_count(inst_count), __max_instance_count(max_count), __node_type(nt)
{
  registerAttributes();
}

NodeInfo::NodeInfo(const NodeInfo& ni)
  : SerializedValue(ni)
{
  registerAttributes();
  *this = ni;
}

void NodeInfo::registerAttributes() {
  /* Begin> register the attributes by the baseclass */
  insertArgument(&__node_id);
  insertArgument(&__library_name);
  insertArgument(&__node_name);
  insertArgument(&__instance_count);
  insertArgument(&__max_instance_count);
  insertArgument(&__node_type);
  /* End< */
}


OValueStream& operator<<(OValueStream& ov, const list<NodeInfo>& string_list) {
  return writeList<NodeInfo>(ov, string_list, "NodeInfo");
}

IValueStream& operator>>(IValueStream& ov,list<NodeInfo>& string_list) {
  return readList<NodeInfo>(ov, string_list, "NodeInfo");
}

TValueConstructor<list<NodeInfo> >     list_NodeInfo_value("list_NodeInfo", "std");

TConstructor<NodeInfo>    __nmm__node_info("NodeInfo", "NMM");

}

#include "nmm/nmm_suffix.hpp"
