/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#include "ObjectFactory.hpp"
#include "ObjectFactoryObject.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  void ObjectFactory::registerFactoryObject(const ObjectFactoryObject& object)
  {
    // Add object.getClassName() --> object to factory map
    __factory_map.insert(std::make_pair(object.getClassName(), &object));
  }

  void ObjectFactory::unregisterFactoryObject(const ObjectFactoryObject& object)
  {
    // Remove key object.getClassName() from factory map
    __factory_map.erase(object.getClassName());
  }

  NMMObject* ObjectFactory::createObject(const std::string& class_name) const
  {
    FactoryMap::const_iterator found = __factory_map.find(class_name);
    if(found != __factory_map.end()) {
      // Factory object found for class name => invoke factory method and return object pointer
      return found->second->createObject();
    }
    else {
      // No factory object found for class name => return null pointer
      return 0;
    }
  }

}

DEFINE_SINGLETON(NMM::ObjectFactory);

#include "nmm/nmm_suffix.hpp"
