/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
  
#include "OutOfBandBindingDescription.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  OutOfBandBindingDescription::OutOfBandBindingDescription()
    : BindingDescription(TypeInfo("OutOfBandBindingDescription", "NMM")),
      __strategies(),
      __request_top_level_config(),
      __reply_top_level_config()
  {
    /* don't use TValue wrappers */
    __strategies.useTValue(false);
    
    registerAttributes();
  }
  
  OutOfBandBindingDescription::OutOfBandBindingDescription(const OutOfBandBindingDescription& other)
    : BindingDescription(other),
      __strategies(other.__strategies),
      __request_top_level_config(other.__request_top_level_config),
      __reply_top_level_config(other.__reply_top_level_config)
  {
    registerAttributes();
  }
  
  void OutOfBandBindingDescription::registerAttributes() {
    insertArgument(&__strategies);
    insertArgument(&__request_top_level_config);
    insertArgument(&__reply_top_level_config);  }
  
  unsigned int  OutOfBandBindingDescription::addTransportStrategy(const string& strategy_type, unsigned int roles) {
    OutOfBandStrategyDescription* desc = new OutOfBandStrategyDescription();
    desc->setStrategyType(strategy_type);
    desc->setRoles(roles);
    desc->setExclusive(false);
    
    __strategies.push_back(desc);
    return __strategies.size() - 1;
  }
  
  unsigned int  OutOfBandBindingDescription::addExclusiveTransportStrategy(const string& strategy_type, unsigned int roles) {
    OutOfBandStrategyDescription* desc = new OutOfBandStrategyDescription();
    desc->setStrategyType(strategy_type);
    desc->setRoles(roles);
    desc->setExclusive(true);
    
    __strategies.push_back(desc);
    return __strategies.size() - 1;
  }

  void OutOfBandBindingDescription::addRequestTopLevelConfigEvent(const Event& event) {
    __request_top_level_config.insertEvent(event);
  }
  
  void OutOfBandBindingDescription::addReplyTopLevelConfigEvent(const Event& event) {
    __reply_top_level_config.insertEvent(event);
  }
  
  void OutOfBandBindingDescription::addRequestConfigEvent(const Event& event, OutOfBandChannel::CommunicationRole role) {
    for(unsigned int i=0; i<__strategies.size(); ++i) {
      if(__strategies[i]->getRoles() & role) {
        __strategies[i]->addRequestConfigEvent(event);
        return;
      }
    }
    
    GLOBAL_WARNING_STREAM("OutOfBandBindingDescription: No strategy description with the given roles exists, can't add config event" << endl);
  }
  
  void OutOfBandBindingDescription::addReplyConfigEvent(const Event& event, OutOfBandChannel::CommunicationRole role) {
    for(unsigned int i=0; i<__strategies.size(); ++i) {
      if(__strategies[i]->getRoles() & role) {
        __strategies[i]->addReplyConfigEvent(event);
        return;
      }
    }
    
    GLOBAL_WARNING_STREAM("OutOfBandBindingDescription: No strategy description with the given roles exists, can't add config event" << endl);
  }
  
  bool OutOfBandBindingDescription::isImplicit() {
    return __strategies.empty() &&
      __request_top_level_config.size() == 0 &&
      __reply_top_level_config.size() == 0;
  }
  
  static TConstructor<OutOfBandBindingDescription> __out_of_band_binding_description(TypeInfo("OutOfBandBindingDescription", "NMM"));
}

#include "nmm/nmm_suffix.hpp"
