/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "OutOfBandStrategyDescription.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {
    
  OutOfBandStrategyDescription::OutOfBandStrategyDescription()
    : StrategyDescription(TypeInfo("OutOfBandStrategyDescription", "NMM")),
      __request_config(),
      __reply_config()
  {
    registerAttributes();
  }
  
  OutOfBandStrategyDescription::OutOfBandStrategyDescription(const OutOfBandStrategyDescription& other)
    : StrategyDescription(other),
      __request_config(other.__request_config),
      __reply_config(other.__reply_config)
  {
    registerAttributes();
  }
  
  void OutOfBandStrategyDescription::addRequestConfigEvent(const Event& event) {
    __request_config.insertEvent(event);
  }
  
  void OutOfBandStrategyDescription::addReplyConfigEvent(const Event& event) {
    __reply_config.insertEvent(event);
  }
  
  void OutOfBandStrategyDescription::registerAttributes() {
    insertArgument(&__request_config);
    insertArgument(&__reply_config);
  }
  
  static TConstructor<OutOfBandStrategyDescription> __out_of_band_strategy_description(TypeInfo("OutOfBandStrategyDescription", "NMM"));

}

#include "nmm/nmm_suffix.hpp"
