/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "Plugin1394.hpp"
#include "nmm/services/registry/LocalNodeFactory.hpp"


#include "nmm/nmm_prefix.hpp"

namespace NMM {

Plugin1394::~Plugin1394() {
  setPluginManager(0);
}

void Plugin1394::nmmInit()
   throw (const NMM::Exception&)
  {
    if (!L1394::SSession::getInstance()) {
      throw NMM::Exception("Could not get the L1394 Session object. Check if ieee1394 driver are loaded and try to load the plugin again.");
    }
  }

  void Plugin1394::nmmFini() {

  }

  void Plugin1394::nmmRegistryFini(){
    delete __registry1394;
    __registry1394 = 0;
  }

  void Plugin1394::nmmRegistryInit()
    throw (const NMM::Exception&)
  {
    if (!__registry1394)
      __registry1394 = new NMM::Registry1394(&SLocalRegistry::getInstance());
  }


  void Plugin1394::nmmFactoryInit()
    throw (const NMM::Exception&)
  {
    if (!__factory1394)
      __factory1394 = new NMM::Factory1394(&SLocalNodeFactory::getInstance());
  }

  void Plugin1394::nmmFactoryFini() {
    delete __factory1394;
    __factory1394 = 0;
  }


};
extern "C" {

// How to handle static versions of this library?

static NMM::Plugin1394 plugin1394;

NMM::Plugin* nmm_plugins[2] = {&plugin1394, 0};

}

#include "nmm/nmm_suffix.hpp"
