/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "ProxyApplication.hpp"
#include "nmm/comm/netstrategy/TCPAddress.hpp"
// #include "nmm/utils/net/NetReactor.hpp"
// #include "nmm/utils/utils.hpp"
#include "nmm/services/registry/PluginManager.hpp"
#include "nmm/services/registry/Registry.hpp"
#include "nmm/comm/IObjectAcceptor.hpp"
// #include "nmm/base/proxy/tcp/TCPAcceptor.hpp"
#include "nmm/comm/ProxyObject.hpp"
#include "nmm/comm/ProcessUserInfo.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  ProxyApplication::ProxyApplication(int argc, char** argv, string dist_hostname, int port, bool enable_acceptor)
  : NMMApplication (argc, argv, false, enable_acceptor), __default_host(dist_hostname), __default_port(port)
{
  __client_registry.reset(new ClientRegistry());
  string hostname = SProcessUserInfo::getInstance().getHostname();

  TCPAddress address;
  address.setPort(__default_port);

  if (__default_host.empty())
    address.setHostname(hostname);
  else
    address.setHostname(__default_host);

  /* Try to connect the ServerRegistry with the given address */
  __client_registry->addRegistry(address, true);

  __i_plugin_manager.reset(__client_registry->getRegistryList().find(RegistryName(hostname, __default_port))->second.plugin_manager->getParentObject()->getInterface<IPluginManager>());
}

ProxyApplication::~ProxyApplication() {

}

ClientRegistry& ProxyApplication::getRegistry() {
  return *__client_registry;
}
const ClientRegistry& ProxyApplication::getRegistry() const {
  return *__client_registry;
}

IPluginManager& ProxyApplication::getPluginManager(){
  return *__i_plugin_manager.get();
}

const IPluginManager& ProxyApplication::getPluginManager() const {
  return *__i_plugin_manager.get();
}

NMMApplication* ProxyApplication::getApplication(int argc, char** argv, string host , int port, bool enable_acceptor) {
  try {
    return new ProxyApplication(argc, argv, host, port, enable_acceptor);
  } catch (const BindingException& e) {
    GLOBAL_ERROR_STREAM("Connection to locally running serverregistry failed, will use internal registry instead" << endl) ;
    return new NMMApplication(argc, argv, true, port, enable_acceptor);
  }
}

}//end namespace


#include "nmm/nmm_suffix.hpp"
