/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#ifndef NMM_REGISTRYEXCEPTION_HPP
#define NMM_REGISTRYEXCEPTION_HPP

#include "nmm/nmm_prefix.hpp"

#include "nmm/comm/serialize/SerializedException.hpp"

#include "visibility.hpp"

namespace NMM {

  /** This Exception is thrown whenever an error related to a registry occurs.
   */

  class DLLEXPORT RegistryException : public SerializedException {
  public:
    /** @name Constructor & Destructor
     */
    //@{
    /** Constructor.
     */
    RegistryException()
      : SerializedException("RegistryException") { setTypeName("RegistryException"); setComment("RegistryException");};

    /** Constructor with error message.
     *
     * @param s The parameter can be used to set an error message
     */
    RegistryException (const string& s)
      : SerializedException("RegistryException : " + s) {setTypeName("RegistryException"); setComment("RegistryException : " + s);};

    /** Constructor with NamedObject and error message.
     * @param n NamedObject from which the exception is thrown
     * @param s The parameter can be used to set an error message
     */
    RegistryException (const NamedObject* n, const string& s = "")
      : SerializedException(n, ((s == "") ? "RegistryException" : "RegistryException : " + s)) {setTypeName("RegistryException");};

    /** Virtual copy constructor.
     *
     * @return pointer to copy of RegistryException
     */
    virtual RegistryException* copy() const {return new RegistryException(*this); }
    //@}
  };

}//end namespace
#include "nmm/nmm_suffix.hpp"

#endif

