/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "RegistryID.hpp"
#include "nmm/comm/serialize/StdValue.hpp"
#include "nmm/comm/serialize/TConstructor.hpp"
#include "nmm/comm/serialize/TValueConstructor.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

RegistryID::RegistryID()
    :  SerializedValue("RegistryID", "NMM"), __reference(0)
{
  allocate();
}

RegistryID::RegistryID(const string& reg_name, const NMMReference& reference)
    :  SerializedValue("RegistryID", "NMM"), __reg_name(reg_name), __reference(reference)
{
  allocate();
}


RegistryID::RegistryID(const RegistryID& ri)
  : SerializedValue(ri)
{
  allocate();
  *this = ri;
}

void RegistryID::allocate() {
  /* Begin> register the attributes by the baseclass */
  insertArgument(&__reg_name);
  insertArgument(&__reference);
  insertArgument(&__user_info);
  /* End< */
}

bool RegistryID::operator==(const RegistryID& id) const {
  /* Begin> Compare all attributes */

  /* compare the NMMReference */
  if (getReference() != id.getReference())
    return false;

  /* compare the UserInfo */
  if (getUserInfo() != id.getUserInfo())
    return false;
  /* End< */

  return true;
}

ostream& operator<<(ostream& o, const RegistryID& reg_id) {
  o << reg_id.getRegistryName () << endl;
  o << reg_id.getReference() << endl;
  o << reg_id.getUserInfo() << endl;
  return o;
}

OValueStream& operator<<(OValueStream& ov, const list<RegistryID>& string_list)
{
  return writeList<RegistryID>(ov, string_list, "RegistryID");
}

IValueStream& operator>>(IValueStream& ov, list<RegistryID>& string_list){
  return readList<RegistryID>(ov, string_list, "RegistryID");
}

TValueConstructor<list<RegistryID> >                  list_RegistryID_value("list_RegistryID", "std");
TConstructor<RegistryID>                              __nmm__registry_id("RegistryID", "NMM");

}

#include "nmm/nmm_suffix.hpp"
