/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef NMM_REGISTRYLOCK_HPP
#define NMM_REGISTRYLOCK_HPP

#include "nmm/nmm_prefix.hpp"

#include "ClientRegistry.hpp"
 
#include "visibility.hpp"
namespace NMM {

/**
 * Simple class for locking the registry.
 * Using this class we assure that the registry is unlocked even if something goes wrong.
 */
class DLLEXPORT RegistryLock {
private:
  class ClientRegistry* reg; ///< ClientRegistry to lock
  
public:

  /** Constructor.
   * 
   * @param registry ClientRegistry to lock
   */
  RegistryLock(class ClientRegistry& registry) : reg(&registry) {
    reg->initTransaction();
  }
  
  /** Unlocks ClientRegistry.
   */
  void unlock() {
    if (reg) {
      reg->releaseTransaction();
      reg = NULL;
    }
  }
  
  /** Destructor.
   *
   * Unlocks ClientRegistry when called.
   */
  ~RegistryLock() {
    unlock();
  }
};

}

#include "nmm/nmm_suffix.hpp"

#endif
