/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#include "NMMConfig.hpp"

#include "nmm/utils/NamedObject.hpp"

#include <fstream>

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  NMMConfig::Setting::Setting(const std::string& d /*= std::string()*/, const std::string& v /*= std::string()*/)
    : desc(d)
  {
    /* extract space seperated values from std::string and put them into a list */
    bool finish = false;
    unsigned int pos;
    std::string valuestr = v;
    do {
      pos = valuestr.find_first_of(' ');
      std::string value;
      if ( pos != std::string::npos ) {
	value = valuestr.substr( 0, pos );
	valuestr = valuestr.substr( pos+1, valuestr.length() );
      } else {
	value = valuestr;
	finish = true;
      }
    
      values.push_back( value );
    } while( !finish );
  }

  NMMConfig::NMMConfig() : rc_file() {
  } // NMMConfig()

  NMMConfig::NMMConfig(const std::string filename)  : rc_file() {
    parseFile(filename);
  } // NMMConfig(std::string)

  NMMConfig::~NMMConfig() {
  } // ~NMMConfig


  bool NMMConfig::getValue(const std::string var, std::string& value) {
    if (config_table.find(var) == config_table.end()) {
      return false;
    }
    value = config_table[var];
    return true;
  } // getValue (for std::string)

  bool NMMConfig::getValue(const std::string var, int& value) {
    if (config_table.find(var) == config_table.end()) {
      return false;
    }
    value = atoi(config_table[var].c_str());
    return true;
  } // getValue (for std::string)

  bool NMMConfig::getValue(const std::string var, float& value) {
    if (config_table.find(var) == config_table.end()) {
      return false;
    }
    value = atof(config_table[var].c_str());
    return true;
  } // getValue (for std::string)

  bool NMMConfig::getValue(const std::string var, bool& value) {
    if (config_table.find(var) == config_table.end()) {
      return false;
    }
    value = (("true" == config_table[var]) ||
	     ("yes"  == config_table[var]) ||
	     ("1" == config_table[var]));
    return true;
  } // getValue (for std::string)

  void NMMConfig::setValue( std::string var, std::string value ) {
    config_table[var] = value;
  }

  void NMMConfig::saveFile()
  {
    // first, create backup
    std::string backup_mmboxrc = rc_file + ".backup";
    std::string create_backup = "mv " + rc_file + " " + backup_mmboxrc;
    GLOBAL_DEBUG_STREAM("Creating backup: " << create_backup << endl);
    if(system(create_backup.c_str()) != 0) {
      GLOBAL_ERROR_STREAM("error creating backup of mmboxrc : " << create_backup << endl);
      return;
    }

    ifstream file;
    file.open(backup_mmboxrc.c_str());
    ofstream file2;
    file2.open(rc_file.c_str());

    if (!file || !file2) {
      GLOBAL_ERROR_STREAM("error opening file " << backup_mmboxrc << " or " << rc_file << endl);
      return;
    }
    while(!file.eof()) {
      char c;
      while (file.peek() == '#') {
	do {
	  file.get(c);
	  file2.put(c);
	  if (file.eof()) {
	    break;
	  }
	} while ('\n' != c);
      }

      std::string var = "";
      std::string value = "";
      std::string new_value = "";
      bool found_equal=false;
      while (!file.eof()) {
	file.get(c);
	if ('\n' == c) {
	  file2.put(c);
	  break;
	} else if ('=' == c) {
	  found_equal = true;
	  getValue(var, new_value); 
	  file2.put(c);
	  continue;
	}
	if (!found_equal) {
	  var += c;
	  file2.put(c);
	} else {
	  if(new_value.size()) {
	    if((var == "configfile") && (new_value.find(".xml") == std::string::npos)) {
	      new_value += ".xml";
	    }
	    file2 << new_value;
	    while (!file.eof()) {
	      file.get(c);
	      if ('\n' == c) {
		file2.put(c);
		break;
	      }
	    }
	    break;
	  }
	  else {
	    value += c;
	    file2.put(c);
	  }
	}
      }

    }
    file2.close();
  }

  void NMMConfig::parseFile(const std::string filename) {
    // set filename for later storing new values to
    rc_file = filename;

    ifstream file(filename.c_str());
    if (!file) {
      return;
    }
    while(!file.eof()) {
      char c;
      while (file.peek() == '#') {
	do {
	  file.get(c);
	  if (file.eof()) {
	    break;
	  }
	} while ('\n' != c);
      }

      std::string var = "";
      std::string value = "";
      bool found_equal=false;
      while (!file.eof()) {
	file.get(c);
	if ('\n' == c) {
	  break;
	} else if (!found_equal && '=' == c) {
	  found_equal = true;
	  continue;
	}
	if (!found_equal) {
	  var += c;
	} else {
	  value += c;
	}
      }

      setValue(var, value); 
    }
  } // parseFile

  bool NMMConfig::getSetting(const std::string& id, const NMMConfig::Setting*& setting) const {
    map<std::string, Setting>::const_iterator found = settings.find(id);
    if (found == settings.end()) {
      return false;
    } else {
      setting = &found->second;
      return true;
    }
  }

  void NMMConfig::addSetting(const std::string& d /*= std::string()*/, const std::string& s /*= std::string()*/, const std::string& v /*= std::string()*/) {
    Setting setting(d,v);
    settings[s] = setting;
  
    /* Add default value */
    if (config_table.find(s) == config_table.end()) {
      if (!setting.getValues().empty()) {
	config_table[s] = setting.getValues().front();
      }
    }
  }

}

#include "nmm/nmm_suffix.hpp"
