/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "Fork.hpp"
#include <iostream>
#include <sys/wait.h>
#include <signal.h>
#include <errno.h>

#define BUFFERSIZE 300

#include "nmm/nmm_prefix.hpp"

namespace NMM{

  Fork::Fork() 
    : ThreadedObject(),
      executemutex(),
      childPid(-1)
  {
    
    socketpair(AF_UNIX, SOCK_STREAM, 0, sv);
    activateThread();
    
    //cerr << "$$$$$$$$$$$$$$$$$$$$$$$$$" << endl;
  }
  
    
  Fork::~Fork(){
    
    write(sv[0], "exit",8 );
    close(sv[0]);
    close(sv[1]);
    deactivateThread();
    
  }
  
  int Fork::getReadSocket(){
    return sv[0];
  }
  
  int Fork::getWriteSocket(){
    return sv[1];
  }


  int Fork::killall(){

    if(childPid > 0){
      kill(childPid, SIGKILL);
    }
    return 0;
  }

  
  /**
   * execute the given command
   */
  int Fork::executeCommand(string command){
    //lock the mutex
    MutexGuard mg(executemutex);

    //cerr << "EXECUTE COMMAND: waiting for " << command << endl;    

    //We send the Command-line to the Main-Child-Process to start the process
    char tmp[BUFFERSIZE];
    memset(&tmp, 0, BUFFERSIZE);
    sprintf(tmp, "%s", command.c_str());
    write(sv[0], tmp , sizeof(tmp));

    char recvBuffer[BUFFERSIZE]; 
    //wait until a buffer is received
    int len = read(sv[0], recvBuffer, sizeof(recvBuffer));
    recvBuffer[len] = '\0'; //cap buffer

    int finishedID = atoi(recvBuffer);
    return finishedID;
  }
  
  void Fork::start(){
    string gotBuffer;


    //Start Child-Process
    switch(childPid = ::fork()){    
    case -1:
      break;
    case 0: 
      do{
        char buffer[BUFFERSIZE];
        //cerr << "  MAINCHILD: started with pid " << getpid() << " from parent " << getppid() << endl;
        //Child waits for incoming messages on the socket
        int len = read(sv[1], buffer,sizeof(buffer));
        buffer[len] = '\0'; //cap the received buffer
        //cerr << "  MAINCHILD got: " << buffer << endl << endl;
        
        //Store buffer
        gotBuffer = string(buffer);
        if(gotBuffer == "exit") break;

        int res = system(gotBuffer.c_str());
        
        //cerr << "res: " << res << endl;
        //Send our return-code back to the main-program
        char t[20];
        memset(&t, 0, 20);
        
        sprintf(t, "%i", res);
        //If Crtl-c is pressed, exit Child-process
        if(string(t) == "2") break;

        write(sv[1], t, sizeof(t));
        //now, we evaluate our Child-Buffer, if we can go on
      }while(gotBuffer != "exit");
     
      //cerr << "  MAINCHILD: exiting " << getpid() << endl;;
      break;
    default:
      break;
    }

    if(childPid != 0){
      //waitpid(childPid, 0, WNOHANG);
      waitpid(-1, 0, WNOHANG);
    }
  } 

  void Fork::run(){
    start();
  }

  int Fork::setChildPriority(int prio){
    #ifndef __CYGWIN__
    #ifndef BUILD_PPC
    return setpriority(PRIO_PROCESS, childPid, prio);
    #else 
    return 0;
    #endif
    #endif
  }

#ifdef __CYGWIN__ /* explicitly instantiate static variables of Singleton template */
  NMM::SFork::InstanceType* NMM::SFork::_instance = 0;
  bool NMM::SFork::destroyed = false;
#endif

}

#include "nmm/nmm_suffix.hpp"
