/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#ifndef _NMM_IMAGECONVERTER_UTILS_HPP_
#define _NMM_IMAGECONVERTER_UTILS_HPP_

#include "nmm/nmm_prefix.hpp"

#include "visibility.hpp"

namespace NMM {

  /**
   * Helper method for convertYV12ToRGB
   */
  DLLEXPORT unsigned int yuv2rgb(int Y, int Cb, int Cr);

  /** 
   * Converts an image in YV12 or YV12A format into an RGB image with colorspace rgb24,
   * 24 bits per pixel (no padding bits!)
   */
  DLLEXPORT void convertYV12ToRGB24(unsigned char* rgb_image, unsigned char* yv12_image, unsigned int width, unsigned int height, bool use_alpha);

  /** 
   * Converts an image in YV12 or YV12A format into an RGB image with colorspace rgb32,
   * 32 bits per pixel (no padding bits!)
   */
  DLLEXPORT void convertYV12ToRGB32(unsigned char* rgb_image, unsigned char* yv12_image, unsigned int width, unsigned int height, bool use_alpha);


  /** 
   * Converts an image in RGBX32 format (rgb24 + 8bit padding) into an RGB image in RGB24 format,
   * 24 bits per pixel (no padding bits!)
   * The conversion is done in place, no new memory is allocated
   */
  DLLEXPORT void convertRGBX32toRGB24InPlace(unsigned char*, unsigned int, unsigned int);

}

#include "nmm/nmm_suffix.hpp"

#endif //_NMM_IMAGECONVERTER_UTILS_HPP_
