/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "Interpolator.hpp"
#include "nmm/utils/time/Interval.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  float Interpolator::getCurrentValue(const Time& current_time) 
  {
    float delta_value = end_value - start_value;
    float delta_time  = (float) Interval(end_time - start_time);

    if(delta_time == 0.0f) {
      return 0.0f;
    }

    float m = delta_value / delta_time;
    float c = start_value - ((float) Interval(start_time - Time())) * m;

    float current_value = 0.0f;
    // clamp to start/end, or compute current value according to well-known equation
    if(current_time >= end_time) {
      current_value = end_value;
    }
    else if(current_time <= start_time) {
      current_value = start_value;
    }
    else {
      current_value = m * ((float) Interval(current_time - Time())) + c;
    }

    return current_value;
  }
}

#include "nmm/nmm_suffix.hpp"
