/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "DispatchedNetHandler.hpp"
#include "NetReactor.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM{

  NetHandler::NetResult DispatchedNetHandler::receiveEvent(SocketHandle socket, short events, short revents) {
    return handleEvents(socket, events&revents);
  }

  NetHandler::NetResult DispatchedNetHandler::handleEvents(SocketHandle socket, short events) {
    short bit = 1;
    NetResult final_result = VOID;

    while ( (bit <= NetReactor::NR_POLLMSG) && (bit > 0) ) /*bit > 0 check for an overflow*/
    {
      NetResult result = VOID;

      switch (events & bit) {
        case 0:
          result = VOID;
          break;

        /* Begin> Read cases */
        case NetReactor::NR_POLLIN:
        case NetReactor::NR_POLLRDNORM:
        case NetReactor::NR_POLLRDBAND:
        case NetReactor::NR_POLLPRI:
          result = receiveData(socket);
          break;
        /* End< */

        /* Begin> write cases */
        case NetReactor::NR_POLLOUT:
        case NetReactor::NR_POLLWRNORM:
          result = writeData(socket);
          break;
        /* End< */

        /* Begin> error cases */
        case NetReactor::NR_POLLHUP:
        case NetReactor::NR_POLLERR:
        case NetReactor::NR_POLLNVAL:
          result = errorOccured(socket);
          break;
        /* End< */

        default:
          result = VOID;
          break;
      }

      /* Accumulate final result.
         REMOVE overrides RECHECK, RECHECK overrides VOID. */
      if(result == REMOVE) {
        final_result = REMOVE;
      } else if(result == RECHECK && final_result != REMOVE) {
        final_result = RECHECK;
      }

      bit = bit << 1;
    }

    return final_result;
  }

}

#include "nmm/nmm_suffix.hpp"
