/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "SockDGram.hpp"
#include "nmm/utils/NamedObject.hpp"

#include <cerrno>

#include "nmm_netutils.hpp"

#include <sys/socket.h>

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  /****************** class SockDGram ******************/
  SockDGram::SockDGram(bool owner /* = true */)
    : __socket(-1),
      __own_socket(owner)
  {
    struct protoent* protocol = getprotobyname("udp");
    __socket = ::socket(PF_INET, SOCK_DGRAM, protocol->p_proto);
  }
  
  SockDGram::SockDGram(SocketHandle socket, bool owner) 
    : __socket(socket),
      __own_socket(owner)
  {
  }

  SockDGram::~SockDGram() {
    if (__own_socket && __socket >= 0) {
      ::close(__socket);
      __socket = -1;
    }
  }
  
  void SockDGram::setPeerAddress(const INetAddr& addr) {
    __peer_addr = addr;
  }
  
  void SockDGram::setSocket(SocketHandle socket, bool owner /* = false */) {
    if (__own_socket && __socket >= 0) {
      ::close(__socket);
    }
    
    __socket = socket;
    __own_socket = owner;
  }
  
  INetAddr SockDGram::getLocalAddress() const {
    struct sockaddr_in addr;
    socklen_t len_addr = sizeof(addr);
    
    if (getsockname(__socket, reinterpret_cast<struct sockaddr*>(&addr), &len_addr) == 0) {
      INetAddr address;
      address.setAddress(&addr);
      return address;
    } else {
      GLOBAL_ERROR_STREAM("Failed to get datagram socket address" << endl);
      return INetAddr();
    }
  }
  
  size_t SockDGram::recv (void* buffer, size_t len)
    throw (const TransmissionException&) 
  {
    int retval = 0;
    struct sockaddr_in peer_addr;
    socklen_t len_peer_addr = sizeof(peer_addr);
    
    retval = ::recvfrom(__socket, static_cast<char*>(buffer), len, 0,
                        reinterpret_cast<sockaddr*>(&peer_addr),
                        &len_peer_addr);
    if (retval < 0) {
      createReadTransmissionException(errno);
    }
    
    // XXX: fw: check if peer_addr == __peer_addr !
    
    return retval;
  }
  
  size_t SockDGram::send (const void* buffer, size_t len) 
    throw (const TransmissionException&) 
  {
    ssize_t retval = ::sendto(__socket, static_cast<const char*>(buffer),
                              len, 0, __peer_addr.getAddress(),
                              __peer_addr.getSize());
    if (retval < 0) {
      createWriteTransmissionException(errno);
    }
    
    return retval;
  }
  
  void SockDGram::close() {}
}

#include "nmm/nmm_suffix.hpp"
