/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include<stdio.h>
#include "overlay.h"

void RGBAoverlay( unsigned char* data, unsigned int stride1, unsigned int w, unsigned int h,
                  unsigned char* pixel, unsigned int stride2 ) {
  unsigned char r1, g1, b1, r2, g2, b2, a1, a2, r3, g3, b3, a3;
  unsigned int lx, ly;
  unsigned int step1, step2;
  step1 = (stride1-w)<<2;
  step2 = (stride2-w)<<2;

  for( ly=0; ly < h; ly++ ) {
    for( lx=0; lx < w; lx++ ) {
      r1 = data[0];
      g1 = data[1];
      b1 = data[2];
      a1 = data[3];

      r2 = pixel[0];
      g2 = pixel[1];
      b2 = pixel[2];
      a2 = pixel[3];

      r3 = (a1*r1 + (255-a1) * r2)/255;
      g3 = (a1*g1 + (255-a1) * g2)/255;
      b3 = (a1*b1 + (255-a1) * b2)/255;
      a3 = (a2*255 + (255-a2) * a1)/255;

      pixel[0] = r3;
      pixel[1] = g3;
      pixel[2] = b3;
      pixel[3] = a3;

      data+=4;
      pixel+=4;
    }
    data  += step1;
    pixel += step2;

  }
}

void YV12Aoverlay(unsigned char* source, unsigned int xpos, unsigned int ypos, unsigned int w, unsigned int h,
	          unsigned char* destination, unsigned int image_width, unsigned image_height ) {
    int plane_size = w * h;
    unsigned char *src_y = source;
    unsigned char *src_u = source + plane_size;
    unsigned char *src_v = src_u + plane_size/4;
    unsigned char *src_a = src_v + plane_size/4;
    int src_u_stride = w/2;
    int src_v_stride = src_u_stride;
    int src_y_stride = w;
    int src_a_stride = w;

    int NO_BLENDING      = 0;
    int REPLACE_BLENDING = 255;
    int x, y;

    int dst_y_stride = image_width;
    int dst_u_stride = image_width/2;
    int dst_v_stride = dst_u_stride;
    unsigned char *dst_y = (unsigned char *) destination + image_width*ypos + xpos;
    unsigned char *dst_u = (unsigned char *) destination + image_width*image_height + (image_width/2)*(ypos/2) + (xpos/2);
    unsigned char *dst_v =  dst_u + (image_width*image_height)/4;
    int xdiv2 = 0;

    for(y = 0; y < h; y++) {
      for(x = 0; x < w; x++) {
	/* if alpha of source is 0 then skip this pixel because it is unvisible */
	if(src_a[x] != NO_BLENDING) {
	  /* if alpha of source is 255 then copy this pixel to destination because it is completly visible */
	  if(src_a[x] == REPLACE_BLENDING) {
	    dst_y[x] = src_y[x];
	    if((x % 2) == 0) {
	      xdiv2 = x/2;
	      dst_u[xdiv2] = src_u[xdiv2];
	      dst_v[xdiv2] = src_v[xdiv2];
	    }
	  }
	  else {

	    // alpha of foreground
	    // a = (int) src_a[x];
	    // take the foreground with its alpha and the background with (256 - foreground alpha) which is src_a[x]
	    dst_y[x] = (unsigned char) (( (int) src_a[x] * ( (int) src_y[x] - (int) dst_y[x] ) ) / 255 + (int) dst_y[x]);

	    if( ((x % 2) == 0) && ((y % 2) == 0 )) {
	      xdiv2 = x/2;
	      dst_u[xdiv2] =
	        (unsigned char) (( (int) src_a[x] * ( (int) src_u[xdiv2] - (int) dst_u[xdiv2] ) ) / 255 + (int) dst_u[xdiv2]);
	      dst_v[xdiv2] =
	        (unsigned char) (( (int) src_a[x] * ( (int) src_v[xdiv2] - (int) dst_v[xdiv2] ) ) / 255 + (int) dst_v[xdiv2]);
	    }


	  }
	}
      }
      src_y += src_y_stride;
      src_a += src_a_stride;
      dst_y += dst_y_stride;
      if((y % 2) == 1) {
	dst_u += dst_u_stride;
	dst_v += dst_v_stride;
	src_u += src_u_stride;
	src_v += src_v_stride;
      }
    }
}

void YV12Aoverlay2( unsigned char* source, unsigned int xpos, unsigned int ypos,
                    unsigned int xstart, unsigned int ystart, unsigned int seg_width, unsigned int seg_height,
                    unsigned int w, unsigned int h,
	            unsigned char* destination, unsigned int image_width, unsigned image_height) {


    int plane_size = w * h;
    unsigned char *src_y = source + xstart + ystart*w;
    unsigned char *src_u = source + plane_size + xstart/2 + (ystart/2)*(w/2);
    unsigned char *src_v = source + plane_size + plane_size/4 + xstart/2 + (ystart/2)*(w/2);
    unsigned char *src_a = source + plane_size + plane_size/2 + xstart + ystart*w;

    int src_u_stride = w/2;
    int src_v_stride = src_u_stride;
    int src_y_stride = w;
    int src_a_stride = w;

    int NO_BLENDING      = 0;
    int REPLACE_BLENDING = 255;
    int x, y;

    int dst_y_stride = image_width;
    int dst_u_stride = image_width/2;
    int dst_v_stride = dst_u_stride;
    unsigned char *dst_y = (unsigned char *) destination + image_width*ypos + xpos;
    unsigned char *dst_u = (unsigned char *) destination + image_width*image_height + (image_width/2)*(ypos/2) + (xpos/2);
    unsigned char *dst_v =  dst_u + (image_width*image_height)/4;
    int xdiv2 = 0;

    for(y = 0; y < seg_height; y++) {
      for(x = 0; x < seg_width; x++) {
	/* if alpha of source is 0 then skip this pixel because it is unvisible */
	if(src_a[x] != NO_BLENDING) {
	  /* if alpha of source is 255 then copy this pixel to destination because it is completly visible */
	  if(src_a[x] == REPLACE_BLENDING) {
	    dst_y[x] = src_y[x];
	    if((x % 2) == 0) {
	      xdiv2 = x/2;
	      dst_u[xdiv2] = src_u[xdiv2];
	      dst_v[xdiv2] = src_v[xdiv2];
	    }
	  }
	  else {

	    // alpha of foreground
	    // a = (int) src_a[x];
	    // take the foreground with its alpha and the background with (256 - foreground alpha) which is src_a[x]
	    dst_y[x] = (unsigned char) (( (int) src_a[x] * ( (int) src_y[x] - (int) dst_y[x] ) ) / 255 + (int) dst_y[x]);

	    if( ((x % 2) == 0) && ((y % 2) == 0 )) {
	      xdiv2 = x/2;
	      dst_u[xdiv2] =
	        (unsigned char) (( (int) src_a[x] * ( (int) src_u[xdiv2] - (int) dst_u[xdiv2] ) ) / 255 + (int) dst_u[xdiv2]);
	      dst_v[xdiv2] =
	        (unsigned char) (( (int) src_a[x] * ( (int) src_v[xdiv2] - (int) dst_v[xdiv2] ) ) / 255 + (int) dst_v[xdiv2]);
	    }


	  }
	}
      }
      src_y += src_y_stride;
      src_a += src_a_stride;
      dst_y += dst_y_stride;
      if((y % 2) == 1) {
	dst_u += dst_u_stride;
	dst_v += dst_v_stride;
	src_u += src_u_stride;
	src_v += src_v_stride;
      }
    }
}

