; Network-Integrated Multimedia Middleware (NMM)
; http://www.motama.com/nmm.html
;
; Copyright (C) 2006-2007
; Motama GmbH, Saarbruecken, Germany
; http://www.motama.com
;
; This file is part of NMM. 
; 
; NMM is free software; you can redistribute it and/or modify it    
; under the terms of the GNU General Public License as published by 
; the Free Software Foundation; only version 2 of the License.      
; 
; NMM is distributed in the hope that it will be useful, but WITHOUT 
; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
; or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
; License for more details.  
; 
; You should have received a copy of the GNU General Public License 
; along with NMM; if not, write to the Free Software Foundation,    
; Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
;
;

; void scaleImagePlane(char* out_data, int out_width, int out_height, int out_pitch,
;		       const char* in_data, int in_pitch,
;		       int in_delta_x, int in_delta_y);
;global scaleImagePlane:function

; void scaleImageRow(char* out_data, int out_width, const char* in_data, int in_pitch, int in_delta_x, int iny_frac);
global scaleImageRow:function
global scaleImageRowFiltering:function

section .text

;scaleImagePlane:
;	
;	push ebp            	; safe on stack
;        mov ebp, esp        	; stack pointer in ebp
;	push ebx            	; save ebx
;	push esi            	; save esi
;	push edi            	; save edi
;	
;	; TODO: no need to do this in assembler
;	
;	emms                	; clean up mmx/fpu registers
;	
;	pop ebx
;	pop edi
;	pop esi
;	
;	leave
;	ret


scaleImageRow:

	push ebp            	; save parent frame pointer
        mov ebp, esp        	; init stack frame
	push ebx            	; save ebx
	push esi            	; save esi
	push edi            	; save edi
		
	mov ecx, [ebp + 12]	; get number of pixels to draw (out_width)
	mov edi, [ebp + 8]	; get out_data
	mov esi, [ebp + 16]	; get in_data
	mov ebx, [ebp + 24]	; get in_delta_x
	
	test ecx, ecx		; test whether width is zero
	jz NEAR .f1		; ...and if so, skip initialization and the until-loop below
	
	push ebp		; save frame pointer
	
	xor edx, edx		; source x
	xor ebp, ebp		; source x integer part
	
.l1:	add edx, ebx		; increment source x
	mov al, [esi + ebp]	; read pixel from source
	
	mov ebp, edx		; copy source x
	mov [edi], al		; store pixel in dest
	
	shr ebp, 16		; get integer part of source x
       	inc edi			; increment dest pointer
	
	dec ecx			; decrement line counter
	jne .l1			; loop
	
	pop ebp			; restore frame pointer
	
.f1:	pop edi			; restore edi
	pop esi			; restore esi
	pop ebx			; restore ebx
	
	pop ebp			; restore parent frame pointer
	ret


scaleImageRowFiltering:

	push ebp            	; save parent frame pointer
        mov ebp, esp        	; init stack frame
	push ebx            	; save ebx
	push esi            	; save esi
	push edi            	; save edi
	
	mov ecx, [ebp + 12]	; get number of pixels to draw (out_width)
	mov edi, [ebp + 8]	; get out_data
	mov esi, [ebp + 16]	; get in_data
	
	test ecx, ecx		; test whether width is zero
	jz NEAR .f1		; ...and if so, skip initialization and the until-loop below
	
	mov eax, [ebp + 28]	; build up y interpolation coefficients (get frac delta y)
	shl eax, 1
	mov ebx, 0x7FFF
	sub ebx, eax
	movd mm3, eax
	movd mm4, ebx
	movq mm0, mm3
	movq mm1, mm4
	psllq mm0, 16
	psllq mm1, 16
	por mm3, mm0
	por mm4, mm1
	psllq mm4, 32
	por mm3, mm4		; mm3: words: HIGH:[ 1-fracsy 1-fracsy fracsy fracsy ]:LOW
	
       	xor edx, edx		; source x
	
.l1:	add edx, [ebp + 24]	; increment source x
	
	mov ebx, edx		; get it part of source x
	shr ebx, 16
	
	xor eax, eax		; build up cxx register (UNOPTIMISED!!!)
	mov al, [esi + ebx]	; get c00
	shl eax, 16
	mov al, [esi + ebx + 1]	; get c10
	movd mm0, eax
	add ebx,[ebp + 20]	; add in_pitch (go to next line)
	mov al, [esi + ebx]	; get c01
	shl eax, 16
	mov al, [esi + ebx + 1]	; get c11
	psllq mm0, 32
	movd mm1, eax
	por mm0, mm1		; mm0: words: HIGH:[ c00 c10 c01 c11 ]:LOW
	
	mov eax, edx		; get frac source x
	mov ebx, 0x7FFF		; get 1 - frac source x
	and eax, 0xFFFF
	shl eax, 1
	sub ebx, eax
	shl ebx, 16
	or eax, ebx
	movd mm1, eax
	movd mm2, eax
	psllq mm2, 32
	por mm1, mm2		; mm1: words: HIGH:[ 1-fracsx fracsx 1-fracsx fracsx ]:LOW
	
; doesn't work: pmaddwd interprets input as signed values
;	pmaddwd mm0, mm1	; mm0: words: HIGH:[ 0000 c1 0000 c0 ]:LOW
;	packssdw mm0, mm0	; mm0: words: HIGH:[ c1 c0 c1 c0 ]:LOW
;	pmaddwd mm0, mm3	; mm0: result in low dword
;	movd eax, mm0
	
	psllw mm0, 1
	pmulhw mm0, mm1		; mm0: words: HIGH:[ c00*(1-fsx) c10*fsx c01*(1-fsx) c11*fsx ]:LOW
	pmulhw mm0, mm3		; mm0: words: HIGH:[ c00*(1-fsx)*(1-fsy) c10*fsx*(1-fsy) c01*(1-fsx)*fsy c11*fsx*fsy ]:LOW
	movq mm1, mm0
	movq mm2, mm0
	movq mm4, mm0
	psrlq mm1, 16
	psrlq mm2, 32
	psrlq mm4, 48
	paddw mm0, mm1
	paddw mm0, mm2
	paddw mm0, mm4
	movd eax, mm0
	
	mov [edi], al
	inc edi
	
	dec ecx			; decrement pixel counter
	jne NEAR .l1		; loop
	
.f1:	emms			; clean up FPU/MMX registers
	
	pop edi			; restore edi
	pop esi			; restore esi
	pop ebx			; restore ebx
	
	pop ebp			; restore parent frame pointer
	ret


; filtering TODO:
; - read nearest 4 pixels around the current sampling position
; - convert them to 16 bit unsigned integers and store them in a single mmx register (aaaaargh!)
; - convert the fractional parts of the current sampling position to 16 bit unsigned integers and copy them in a single mmx reg
; - multiply and add using PMADDWD
; - store pixel in destination
;
; (almost) independent tasks (for insn scheduling):
; - build fraction register
; - build color register
; - calculate offset in source image
; - load/store pixels
;
; helpers:
; - look at PACKSSDW, PACKSSWB, PACKUSWB
