/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#ifndef NMM_TTASK_HPP
#define NMM_TTASK_HPP

#include "nmm/nmm_prefix.hpp"

#include "Task.hpp"

namespace NMM {
  /** A task for a WorkerThread.
   * Encapsulates pointer to object methods as a Task object which allows to execute them within a WorkerThread. 
   * These object methods must return void and expect no arguments.
   *
   * Template parameters:
   *
   * - @c T Object type 
   */
  template<class T>
  class TTask : public Task {
  typedef void (T::*Method)();
  public:
    /** Constructor
     *
     * @param object Pointer to object    
     * @param method Pointer to object method 
     */
    TTask(T* object, Method method) : __object(object), __method(method) {}

    /** Executes the method pointer. 
     */
    virtual void run() { (__object->*__method)(); }

  private:
    T*     __object;
    Method __method;
  };
}


#include "nmm/nmm_suffix.hpp"

#endif // NMM_WORKERTHREAD_HPP
