/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */



#include "ThreadCondition.hpp"
#include "ThreadMutex.hpp"
#include "nmm/utils/NamedObject.hpp"

#include <sys/time.h>
#ifdef PROFILING
#include "gprof_pthreads.h"
#endif

#include "nmm/utils/time/timeutils.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

ThreadCondition::ThreadCondition(const ThreadMutex& mutex) : mutex(mutex) {
  pthread_cond_init(&cond, 0);
}


ThreadCondition::~ThreadCondition() {
  pthread_cond_destroy(&cond);
}

Result ThreadCondition::timedWait(long sec, long nsec) const {
  struct timespec timeout;
  timeout.tv_sec  = sec;
  timeout.tv_nsec = nsec;
  int result = pthread_cond_timedwait(&cond, &(mutex.mutex), &timeout);
  if(result != 0) {
    return TIME_OUT;
  } else {
    return SUCCESS;
  }
}

Result ThreadCondition::relativeTimedWait(long sec, long nsec) const {
  struct timeval current_time;
  ::gettimeofday(&current_time, 0);

  struct timespec timeout;
  timeout.tv_sec  = current_time.tv_sec + sec;
  timeout.tv_nsec = current_time.tv_usec * 1000 + nsec;

  int result = pthread_cond_timedwait(&cond, &(mutex.mutex), &timeout);

  if(result != 0) {
    return TIME_OUT;
  } else {
    return SUCCESS;
  }
}

void ThreadCondition::wait() const {
  pthread_cond_wait(&cond, &(mutex.mutex));
}

}

#include "nmm/nmm_suffix.hpp"
