/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */



#include "ThreadMutex.hpp"

#ifdef PROFILING
#include "gprof_pthreads.h"
#endif

#include "nmm/nmm_prefix.hpp"

namespace NMM {

ThreadMutex::ThreadMutex(MutexKind mk) {
  // Initialize mutex attributes
  pthread_mutexattr_t attr;
  pthread_mutexattr_init(&attr);
  if(mk == RECURSIVE) {
    // Recursive mutex (if locked again by the same thread => increment lock counter)
    pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE);
  }
  else {
    // Normal mutex (if locked again by the same thread => deadlock)
    pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_NORMAL);

    // Error-checking mutex (if locked again by the same thread => error)
    //pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK);
  }

  // Initialize mutex
  pthread_mutex_init(&mutex, &attr);
}

ThreadMutex::~ThreadMutex() {
  pthread_mutex_destroy(&mutex);
}

Result ThreadMutex::tryLock() const {
  if(pthread_mutex_trylock(&mutex) == 0) {
    return SUCCESS;
  }
  else {
    return FAILURE;
  }
}

void ThreadMutex::lock() const {
  pthread_mutex_lock(&mutex);
}

void ThreadMutex::unlock() const {
  pthread_mutex_unlock(&mutex);
}

}

#include "nmm/nmm_suffix.hpp"
