/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
#include "ThreadPool.hpp"
#include "MutexGuard.hpp"
#include "nmm/utils/NamedObject.hpp"
#include <iostream>

#include "nmm/nmm_prefix.hpp"

namespace NMM {
  ThreadPool::ThreadPool() :
  __thread_condition(__thread_mutex), __allocated_threads(3), __notify(false)
  {
    for (unsigned int i = 0; i< __allocated_threads; ++i)
      __thread_pool.push_back(new WorkerThread());
  }

  ThreadPool::~ThreadPool() {
    __thread_mutex.lock();
    __notify = true;

    while ( __allocated_threads != __thread_pool.size() ) {
      GLOBAL_DEBUG_STREAM("Wait until all threads have been releeased" << std::endl);
      __thread_condition.wait();
    }
    GLOBAL_DEBUG_STREAM("All threads have been released" << std::endl);

    /* Delete all allocated threads */
    for (list<WorkerThread*>::iterator it = __thread_pool.begin(); it != __thread_pool.end(); ++it)
      delete *it;
  }

  WorkerThread* ThreadPool::requestThread() {
    MutexGuard mg(__thread_mutex);
    WorkerThread* free_thread = 0;

    if (__thread_pool.empty()) {
      /* increment the internal counter */
      ++__allocated_threads;
      GLOBAL_DEBUG_STREAM("Must spawn a new Thread" << std::endl);
      free_thread = new WorkerThread();
    } else {
      free_thread = __thread_pool.front();
      __thread_pool.erase(__thread_pool.begin());
    }
    /* enable automatic release */
    free_thread->setReleaseFlag(true);
    return free_thread;
  }

  void ThreadPool::releaseThread( WorkerThread* returning_thread) {
    MutexGuard mg(__thread_mutex);
    __thread_pool.push_back(returning_thread);
    if (__notify) {
      __thread_condition.notify();
    }
  }
}

DEFINE_SINGLETON(NMM::ThreadPool);

#include "nmm/nmm_suffix.hpp"
