/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "Benchmark.hpp"
#include "Interval.hpp"

#include "nmm/utils/NamedObject.hpp"

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  Benchmark::Benchmark(NamedObject *obj , const char* msg) {
    setNamedObject(obj);
    setMessage(msg);
    enabled = false;
    frame_count = total_bytes = fps_looptimer_diff = bpf = bps = 0;
    fps = 0.0;
  }
  
  Benchmark::~Benchmark() {}
  
  void Benchmark::setNamedObject(NamedObject *obj) {
    master = obj;
  } 
  
  void Benchmark::setMessage(const char *msg) {
    if(msg != 0) {
      message = msg;
    } else {
      message = "Benchmark>";
    }
  }
  
  void Benchmark::setEnabled(bool enabled) {
    this->enabled = enabled; 
    if(enabled) {
      frame_count = total_bytes = fps_looptimer_diff = bpf = bps = 0;
      fps = 0.0;
      fps_looptimer1 = fps_clock.getTime();
    }
  }
  
  void Benchmark::benchmark(const long used_bytes) {
    if(enabled) {
      ++frame_count;
      total_bytes += used_bytes;
      if((frame_count % 10) == 0) {
	fps_looptimer2 = fps_clock.getTime();
        Interval duration = fps_looptimer2 - fps_looptimer1;
        fps_looptimer_diff = 
          static_cast<long>(static_cast<double>(duration.sec) * 1000 +
                            static_cast<double>(duration.nsec) / 1000000);
	fps = (10000.0f / ((float)fps_looptimer_diff));
	bps = (long) ((1000.0 * (float) total_bytes) / (float) fps_looptimer_diff);
	bpf = (total_bytes / 10);
	if(master != 0) {
	  NODE_DEBUG_STREAM(master, " " << std::dec << message 
                            << " last 10 frames with " << bpf
                            << " bytes/frame, " << fps
                            << " fps, " << bps << " bytes/s " << std::endl);
	}
	else {
	  GLOBAL_MESSAGE_STREAM( " " << std::dec << message 
                                 << " last 10 frames with " << bpf 
                                 << " bytes/frame, " << fps
                                 << " fps, " << bps << " bytes/s " << std::endl);
	}
	fps_looptimer1 = fps_clock.getTime();
	total_bytes = 0;
      }
    }
  }
  
}



#include "nmm/nmm_suffix.hpp"
