/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#ifndef NMM_TIMEDELEMENT_HPP
#define NMM_TIMEDELEMENT_HPP NMM_TIMEDELEMENT_HPP

#include "nmm/nmm_prefix.hpp"

#include "Clock.hpp"

#include "visibility.hpp"

namespace NMM {

  /**
   * This class is used as a superclass for all objects that need 
   * a clock (They are called "TimedElements".) 
   * Examples are source nodes with a clock for timestamping the 
   * buffers they produce or sink nodes that should present a buffer
   * exactly at a specified time.
   */
  class DLLEXPORT TimedElement {
  private:
    static Clock clock;         //! The TimedElement's common clock.

  public:

    //! The main constructor.
    /*! 
     * Returns a TimedElement with access to the common clock.
     */
    TimedElement();

    //! The main destructor.
    virtual ~TimedElement();


    //! Returns the time value at the moment of calling.
    static inline Time getTime() { return clock.getTime(); }
  };
}

#include "nmm/nmm_suffix.hpp"

#endif // _NMM_TIMEDELEMENT_HPP_
