/* Network-Integrated Multimedia Middleware (NMM)
 * http://www.motama.com/nmm.html
 *  
 * Copyright (C) 2006-2007
 * Motama GmbH, Saarbruecken, Germany
 * http://www.motama.com
 *  
 * This file is part of NMM. 
 * 
 * NMM is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; only version 2 of the License.      
 * 
 * NMM is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
 * License for more details.  
 * 
 * You should have received a copy of the GNU General Public License 
 * along with NMM; if not, write to the Free Software Foundation,    
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "UserTime.hpp"
#include "Time.hpp"
#include "Interval.hpp"

#include <iostream>

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  UserTime::UserTime(const Time& time) {
    hour = static_cast<nmm_int32>(time.sec / 3600);
    min  = static_cast<nmm_int32>((time.sec - hour * 3600) / 60);
    sec  = static_cast<nmm_int32>(time.sec - hour * 3600 - min * 60);
    msec = static_cast<nmm_int32>(time.nsec / 1000000L);
  };
    
  UserTime::UserTime(const Interval& interval) {
    hour = static_cast<nmm_int32>(interval.sec / 3600);
    min  = static_cast<nmm_int32>((interval.sec - hour * 3600) / 60);
    sec  = static_cast<nmm_int32>(interval.sec - hour * 3600 - min * 60);
    msec = static_cast<nmm_int32>(interval.nsec / 1000000L);
  };
  
  void UserTime::output(std::ostream& stream, bool with_seconds, bool with_microseconds) const {
    stream.width(2);
    stream.fill('0');
    stream << hour << ":";
    stream.width(2);
    stream.fill('0');
    stream << min;
    if(with_seconds) {
      stream << ":";
      stream.width(2);
      stream.fill('0');
      stream << sec;
    }
    if(with_microseconds) {
      stream << "+";
      stream.width(3);
      stream.fill('0');
      stream << msec;
      stream << "msec";
    }
  }
  
  std::ostream& operator<<(std::ostream& stream, const UserTime& time) {
    time.output(stream, true, true);
    return stream;
  }
  
}

#include "nmm/nmm_suffix.hpp"
