/* libxml++
 * Copyright (C) 2000  Ari Johnson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

/* Modified by Motama GmbH, Copyright (C) 2006-2007 Motama GmbH */


#include "XMLEncoder.hpp"

#include <libxml/encoding.h>

#include "nmm/nmm_prefix.hpp"

namespace NMM {

  std::string XMLEncoder::getUTF8(const std::string &s) {

    xmlChar * out;
    int ret;
    int size;
    int out_size;
    int temp;
    xmlCharEncodingHandlerPtr handler;
  
    if (s == "") return std::string("");
  
    handler = xmlFindCharEncodingHandler(_encoding.c_str());
  
    if (!handler) {
      //cerr << "ConvertInput: no encoding handler found for " << _encoding << endl;
      return std::string("");
    }

    size = (int) s.size() + 1;
    out_size = size * 2 - 1;
    out = (unsigned char *) xmlMalloc((size_t) out_size);

    if (out != 0) {
      temp = size - 1;
      ret = handler->input(out, &out_size, (const xmlChar *)(s.c_str()), &temp);
      if (ret || temp - size + 1) {
        /*
          if (ret) {
          cerr << "ConvertInput: conversion wasn't successful." << endl;
          } else {
          cerr << "ConvertInput: conversion wasn't successful. converted: " << temp << "octets." << endl;
          }
        */
        xmlFree(out);
        out = 0;
      } else {
        out = (unsigned char *) xmlRealloc(out, out_size + 1);
        out[out_size] = 0;  /*null terminating out */
      }
      return std::string((char *)out);
    } else {
      //std::cerr << "ConvertInput: no mem" << endl;
      return ("");
    }
  }

  std::string XMLEncoder::getEncodedString(const std::string &s) {
    xmlChar * out;
    int ret;
    int size;
    int out_size;
    int temp;
    xmlCharEncodingHandlerPtr handler;
  
    if (s == "") return std::string("");
    handler = xmlFindCharEncodingHandler(_encoding.c_str());
  
    if (!handler) {
      //cerr << "ConvertInput: no encoding handler found for " << _encoding << endl;
      return std::string("");
    }

    size = (int) s.size() + 1;
    out_size = size * 2 - 1;
    out = (unsigned char *) xmlMalloc((size_t) out_size);

    if (out != 0) {
      temp = size - 1;
      ret = handler->output(out, &out_size, (const xmlChar *)(s.c_str()), &temp);
      if (ret || temp - size + 1) {
        /*
          if (ret) {
          cerr << "ConvertInput: conversion wasn't successful." << endl;
          } else {
          cerr << "ConvertInput: conversion wasn't successful. converted: " << temp << "octets." << endl;
          }
        */
        xmlFree(out);
        out = 0;
      } else {
        out = (unsigned char *) xmlRealloc(out, out_size + 1);
        out[out_size] = 0;  /*null terminating out */
      }
      return std::string((char *)out);
    } else {
      //std::cerr << "ConvertInput: no mem" << endl;
      return std::string("");
    }
  }

}

#include "nmm/nmm_suffix.hpp"
